@@ -1,5 +1,5 @@
 /*
- * $Id: SND_OpenALDevice.cpp,v 1.16 2004/12/05 00:50:55 kester Exp $
+ * $Id: SND_OpenALDevice.cpp,v 1.20 2005/06/08 18:54:16 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,10 +31,6 @@
  * SND_OpenALDevice derived from SND_IAudioDevice
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef WIN32
 #pragma warning (disable:4786) // get rid of stupid stl-visual compiler debug warning
 #endif //WIN32
@@ -393,14 +389,16 @@ SND_WaveSlot* SND_OpenALDevice::LoadSample(const STR_String& name,
 			{
 				int buffer = waveslot->GetBuffer();
 				void* data = NULL;
+#ifndef __APPLE__
 				char loop = 'a';
+#endif
 				int sampleformat, bitrate, numberofchannels;
 				ALenum al_error = alGetError();
 				
 #ifdef OUDE_OPENAL
 				unsigned int samplerate, numberofsamples;		// openal_2.12
 #else
-				int samplerate, numberofsamples, frequency;  // openal_2.14+
+				int samplerate, numberofsamples;  // openal_2.14+
 #endif
 				
 				/* Give them some safe defaults just incase */
@@ -432,13 +430,9 @@ SND_WaveSlot* SND_OpenALDevice::LoadSample(const STR_String& name,
 				else
 				{
 #ifdef __APPLE__
-					alutLoadWAVFile((signed char*)samplename.Ptr(), &sampleformat, &data, &numberofsamples, &samplerate);
-#elif defined(WIN32)
-					alutLoadWAVFile((signed char*)samplename.Ptr(), &sampleformat, &data, &numberofsamples, &samplerate, &loop);
+					alutLoadWAVFile((ALbyte *)samplename.Ptr(), &sampleformat, &data, &numberofsamples, &samplerate);
 #else
-					alutLoadWAV((char*)samplename.Ptr(), &data,
-						&sampleformat, &numberofsamples,
-						&samplerate, &frequency);
+					alutLoadWAVFile((ALbyte *)samplename.Ptr(), &sampleformat, &data, &numberofsamples, &samplerate, &loop);
 #endif
 					/* put it in the buffer */
 					alBufferData(m_buffers[buffer], sampleformat, data, numberofsamples, samplerate);
