@@ -1,5 +1,5 @@
 /**
- * $Id: GHOST_SystemCarbon.cpp,v 1.18 2004/11/06 14:52:25 ton Exp $
+ * $Id: GHOST_SystemCarbon.cpp,v 1.19 2005/03/09 19:45:53 lukep Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
 
 /**
 
- * $Id: GHOST_SystemCarbon.cpp,v 1.18 2004/11/06 14:52:25 ton Exp $
+ * $Id: GHOST_SystemCarbon.cpp,v 1.19 2005/03/09 19:45:53 lukep Exp $
  * Copyright (C) 2001 NaN Technologies B.V.
  * @author	Maarten Gribnau
  * @date	May 7, 2001
@@ -902,7 +902,7 @@ bool GHOST_SystemCarbon::handleMouseDown(EventRef event)
 							::ZoomWindow(window, part, true);
 							ghostWindow->setMac_windowState(1);
 					} else { // need to force size again
-						GHOST_TUns32 scr_x,scr_y;
+					//	GHOST_TUns32 scr_x,scr_y; /*unused*/
 						Rect outAvailableRect;
 						
 						ghostWindow->setMac_windowState(2);
