@@ -5,7 +5,7 @@
  * any data in Blender. This block is not serialized, but built anew
  * for every fresh Blender run.
  *
- * $Id: BKE_global.h,v 1.11 2004/03/28 08:46:33 zuster Exp $ 
+ * $Id: BKE_global.h,v 1.25 2005/06/11 16:30:36 jesterking Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -63,16 +63,6 @@ struct bSoundListener;
 struct BMF_Font;
 struct EditMesh;
 
-#define UNDO_MAX	64
-#define UNDO_EDIT_MAX 	64
-
-typedef struct UndoBufferEdit {
-
-	void *datablock;
-	char name[64];
-
-} UndoBufferEdit;
-
 
 typedef struct Global {
 
@@ -97,12 +87,14 @@ typedef struct Global {
     /* strings: lastsaved */
     char ima[160], sce[160], lib[160];
     
-    /* totalen */
+    /* totals */
     short totobj, totlamp, totobjsel, totcurve, totmesh, totmat;
-    int totvert, totface, totvertsel, totfacesel;
+    int totvert, totedge, totface, totvertsel, totedgesel, totfacesel;
     
     short machine, afbreek, moving, colact, zbuf;
     short qual, background, imagewin, animspeed;
+	short winpos, displaymode;	/* used to be in Render */
+	
 	/**
 	 * The current version of Blender.
 	 */
@@ -153,14 +145,6 @@ typedef struct Global {
     short special1, special2;
 
     int flags;
-
-    /* editmode undo - written by intrr, ported by Det. Thorn  */
-    struct UndoBufferEdit undo_edit[UNDO_EDIT_MAX+1];/* need one more for undoing first undo */
-    int undo_edit_level;			/* index of last undo buffer created */
-    int undo_edit_highest;			/* index of highest undo buffer in use */
-    void *undo_last_data;			/* pointer to last datablock edited */
-    void (*undo_clear)(void);		/* pointer to function to free the undo data */
-
         
 } Global;
 
@@ -180,12 +164,22 @@ typedef struct Global {
 #define G_VERTEXPAINT	1024
 #define G_ALLEDGES		2048
 #define G_DEBUG			4096
-#define G_SCENESCRIPT	8192
-#define G_PROPORTIONAL	16384
-#define G_WEIGHTPAINT	32768	/* __NLA */
+#define G_DOSCRIPTLINKS (1 << 13)
+/* #define G_PROPORTIONAL	16384	removed! so can be used later for other stuff */
+#define G_WEIGHTPAINT	32768	
 #define G_TEXTUREPAINT	65536
-#define G_NOFROZEN	(1 << 17) // frozen modules inactive
+/* #define G_NOFROZEN	(1 << 17) also removed */
 #define G_DRAWEDGES	(1 << 18)
+#define G_DRAWCREASES	(1 << 19)
+#define G_DRAWSEAMS     (1 << 20)
+#define G_HIDDENEDGES   (1 << 21)
+
+/* Measurement info Drawing */
+#define G_DRAW_EDGELEN   (1 << 22) 
+#define G_DRAW_FACEAREA (1 << 23)
+#define G_DRAW_EDGEANG  (1 << 24)
+
+#define G_RECORDKEYS  (1 << 25)
 
 /* G.fileflags */
 
@@ -199,6 +193,8 @@ typedef struct Global {
 #define G_FILE_LOCK_BIT                 7
 #define G_FILE_SIGN_BIT                 8
 #define G_FILE_PUBLISH_BIT				9
+#define G_FILE_NO_UI_BIT				10
+
 
 #define G_AUTOPACK               (1 << G_AUTOPACK_BIT)
 #define G_FILE_COMPRESS          (1 << G_FILE_COMPRESS_BIT)
@@ -210,6 +206,7 @@ typedef struct Global {
 #define G_FILE_LOCK              (1 << G_FILE_LOCK_BIT)
 #define G_FILE_SIGN              (1 << G_FILE_SIGN_BIT)
 #define G_FILE_PUBLISH			 (1 << G_FILE_PUBLISH_BIT)
+#define G_FILE_NO_UI			 (1 << G_FILE_NO_UI_BIT)
 
 /* G.windowstate */
 #define G_WINDOWSTATE_USERDEF		0
@@ -227,21 +224,27 @@ typedef struct Global {
 #define G_FLAGS_AUTOPLAY		(1 << G_FLAGS_AUTOPLAY_BIT)
 
 /* G.qual */
-#define R_SHIFTKEY	1
-#define L_SHIFTKEY	2
-#define LR_SHIFTKEY 3
-#define R_ALTKEY	4
-#define L_ALTKEY	8
-#define LR_ALTKEY	12
-#define R_CTRLKEY	16
-#define L_CTRLKEY	32
-#define LR_CTRLKEY  48
+#define R_SHIFTKEY		1
+#define L_SHIFTKEY		2
+#define LR_SHIFTKEY 	3
+#define R_ALTKEY		4
+#define L_ALTKEY		8
+#define LR_ALTKEY		12
+#define R_CTRLKEY		16
+#define L_CTRLKEY		32
+#define LR_CTRLKEY  	48
+#define LR_COMMANDKEY 	64
 
 /* G.order: indicates what endianness the platform where the file was
  * written had. */
 #define L_ENDIAN	1
 #define B_ENDIAN	0
 
+/* G.moving, signals drawing in (3d) window to denote transform */
+#define G_TRANSFORM_OBJ		1
+#define G_TRANSFORM_EDIT	2
+#define G_TRANSFORM_MANIP	4
+
 /* G.special1 */
 
 /* Memory is allocated where? blender.c */
