@@ -1,4 +1,4 @@
-/* $Id: BKE_subsurf.h,v 1.10 2004/03/28 08:46:33 zuster Exp $ 
+/* $Id: BKE_subsurf.h,v 1.18 2005/04/04 03:38:21 zuster Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -33,12 +33,11 @@
 
 struct Mesh;
 struct Object;
-struct Displist;
-struct DispListMesh;
+struct DerivedMesh;
 struct EditMesh;
 
-struct DispListMesh *subsurf_make_dispListMesh_from_editmesh(struct EditMesh *em, int subdivLevels, int flags, short type);
-struct DispListMesh *subsurf_make_dispListMesh_from_mesh(struct Mesh *me, float *extverts, int subdivLevels, int flags);
+struct DerivedMesh *subsurf_make_derived_from_editmesh(struct EditMesh *em, int subdivLevels, short type, struct DerivedMesh *oldDerived);
+struct DerivedMesh *subsurf_make_derived_from_mesh(struct Mesh *me, int subdivLevels);
 
 void subsurf_calculate_limit_positions(Mesh *me, float (*positions_r)[3]);
 
