@@ -2,7 +2,7 @@
 /*  curve.c 
  * 
  *  
- * $Id: curve.c,v 1.6 2004/03/14 16:59:48 zuster Exp $
+ * $Id: curve.c,v 1.12 2005/04/17 18:00:32 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,9 +43,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
 #include "MEM_guardedalloc.h"
 #include "BLI_blenlib.h"  
 #include "BLI_arithb.h"  
@@ -1217,6 +1214,8 @@ void makebevelcurve(Object *ob, ListBase *disp)
 						dlnew->verts= MEM_mallocN(3*sizeof(float)*dl->parts*dl->nr, "makebevelcurve1");
 						memcpy(dlnew->verts, dl->verts, 3*sizeof(float)*dl->parts*dl->nr);
 						
+						if(dlnew->type==DL_SEGM) dlnew->flag |= (DL_FRONT_CURVE|DL_BACK_CURVE);
+						
 						BLI_addtail(disp, dlnew);
 						fp= dlnew->verts;
 						nr= dlnew->parts*dlnew->nr;
@@ -1238,7 +1237,9 @@ void makebevelcurve(Object *ob, ListBase *disp)
 		BLI_addtail(disp, dl);
 		dl->type= DL_SEGM;
 		dl->parts= 1;
+		dl->flag= DL_FRONT_CURVE|DL_BACK_CURVE;
 		dl->nr= 2;
+		
 		fp= dl->verts;
 		fp[0]= fp[1]= 0.0;
 		fp[2]= -cu->ext1;
@@ -1246,39 +1247,89 @@ void makebevelcurve(Object *ob, ListBase *disp)
 		fp[5]= cu->ext1;
 	}
 	else {
-		nr= 4+2*cu->bevresol;
-
-		dl= MEM_callocN(sizeof(DispList), "makebevelcurve3");
-		dl->verts= MEM_mallocN(nr*3*sizeof(float), "makebevelcurve3");
+		short dnr;
+		
+		/* bevel now in three parts, for proper vertex normals */
+		/* part 1 */
+		dnr= nr= 2+ cu->bevresol;
+		if( (cu->flag & (CU_FRONT|CU_BACK))==0)	// we make a full round bevel in that case
+			nr= 3+ 2*cu->bevresol;
+		   
+		dl= MEM_callocN(sizeof(DispList), "makebevelcurve p1");
+		dl->verts= MEM_mallocN(nr*3*sizeof(float), "makebevelcurve p1");
 		BLI_addtail(disp, dl);
 		dl->type= DL_SEGM;
 		dl->parts= 1;
+		dl->flag= DL_BACK_CURVE;
 		dl->nr= nr;
 
-		/* first make circle */
+		/* half a circle */
 		fp= dl->verts;
-		hoek= -0.5*M_PI;
-		dhoek= (float)(M_PI/(nr-2));
+		dhoek= (0.5*M_PI/(dnr-1));
+		hoek= -(nr-1)*dhoek;
+		
 		for(a=0; a<nr; a++) {
 			fp[0]= 0.0;
 			fp[1]= (float)(cos(hoek)*(cu->ext2));
-			fp[2]= (float)(sin(hoek)*(cu->ext2));
+			fp[2]= (float)(sin(hoek)*(cu->ext2)) - cu->ext1;
 			hoek+= dhoek;
 			fp+= 3;
-			if(cu->ext1!=0.0 && a==((nr/2)-1) ) {
-				VECCOPY(fp, fp-3);
-				fp+=3;
-				a++;
+		}
+		
+		/* part 2, sidefaces */
+		if(cu->ext1!=0.0) {
+			nr= 2;
+			
+			dl= MEM_callocN(sizeof(DispList), "makebevelcurve p2");
+			dl->verts= MEM_callocN(nr*3*sizeof(float), "makebevelcurve p2");
+			BLI_addtail(disp, dl);
+			dl->type= DL_SEGM;
+			dl->parts= 1;
+			dl->nr= nr;
+			
+			fp= dl->verts;
+			fp[1]= cu->ext2;
+			fp[2]= -cu->ext1;
+			fp[4]= cu->ext2;
+			fp[5]= cu->ext1;
+			
+			if( (cu->flag & (CU_FRONT|CU_BACK))==0) {
+				dl= MEM_dupallocN(dl);
+				dl->verts= MEM_dupallocN(dl->verts);
+				BLI_addtail(disp, dl);
+				
+				fp= dl->verts;
+				fp[1]= -fp[1];
+				fp[2]= -fp[2];
+				fp[4]= -fp[4];
+				fp[5]= -fp[5];
 			}
 		}
-		if(cu->ext1==0.0) dl->nr--;
-		else {
-			fp= dl->verts;
-			for(a=0; a<nr; a++) {
-				if(a<=(nr/2-1)) fp[2]-= (cu->ext1);
-				else fp[2]+= (cu->ext1);
-				fp+= 3;
-			}
+		
+		/* part 3 */
+		dnr= nr= 2+ cu->bevresol;
+		if( (cu->flag & (CU_FRONT|CU_BACK))==0)
+			nr= 3+ 2*cu->bevresol;
+		
+		dl= MEM_callocN(sizeof(DispList), "makebevelcurve p3");
+		dl->verts= MEM_mallocN(nr*3*sizeof(float), "makebevelcurve p3");
+		BLI_addtail(disp, dl);
+		dl->type= DL_SEGM;
+		dl->flag= DL_FRONT_CURVE;
+		dl->parts= 1;
+		dl->nr= nr;
+		
+		/* half a circle */
+		fp= dl->verts;
+		hoek= 0.0;
+		dhoek= (0.5*M_PI/(dnr-1));
+		
+		for(a=0; a<nr; a++) {
+			fp[0]= 0.0;
+			fp[1]= (float)(cos(hoek)*(cu->ext2));
+			fp[2]= (float)(sin(hoek)*(cu->ext2)) + cu->ext1;
+			hoek+= dhoek;
+			fp+= 3;
 		}
 	}
 
@@ -1936,7 +1987,7 @@ void calchandleNurb(BezTriple *bezt,BezTriple *prev, BezTriple *next, int mode)
 		vx= dx1/len2 + dx/len1;
 		vy= dy1/len2 + dy/len1;
 		vz= dz1/len2 + dz/len1;
-		len= 2.71f*(float)sqrt(vx*vx + vy*vy + vz*vz);
+		len= 2.5614f*(float)sqrt(vx*vx + vy*vy + vz*vz);
 		if(len!=0.0f) {
 		
 			if(len1>5.0f*len2) len1= 5.0f*len2;	
@@ -2245,11 +2296,11 @@ void swapdata(void *adr1, void *adr2, int len)
 	else {
 		char *adr;
 
-		adr= (char *)malloc(len);
+		adr= (char *)MEM_mallocN(len, "curve swap");
 		memcpy(adr, adr1, len);
 		memcpy(adr1, adr2, len);
 		memcpy(adr2, adr, len);
-		free(adr);
+		MEM_freeN(adr);
 	}
 }
 
