@@ -1,7 +1,7 @@
 /*  effect.c
  * 
  * 
- * $Id: effect.c,v 1.28 2004/12/14 18:12:30 ton Exp $
+ * $Id: effect.c,v 1.45 2005/06/06 18:52:07 bjornmose Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -39,6 +39,7 @@
 #include "DNA_listBase.h"
 #include "DNA_effect_types.h"
 #include "DNA_object_types.h"
+#include "DNA_object_force.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_material_types.h"
@@ -59,6 +60,7 @@
 #include "BKE_constraint.h"
 #include "BKE_deform.h"
 #include "BKE_displist.h"
+#include "BKE_DerivedMesh.h"
 #include "BKE_effect.h"
 #include "BKE_global.h"
 #include "BKE_ipo.h"
@@ -71,6 +73,9 @@
 #include "BKE_screen.h"
 #include "BKE_utildefines.h"
 
+#include "render.h"  // externtex, bad level call (ton)
+
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -311,39 +316,38 @@ void where_is_particle(PartEff *paf, Particle *pa, float ctime, float *vec)
 
 void particle_tex(MTex *mtex, PartEff *paf, float *co, float *no)
 {				
-	extern float Tin, Tr, Tg, Tb;
-	extern void externtex(struct MTex *mtex, float *vec);
+	float tin, tr, tg, tb, ta;
 	float old;
 	
-	externtex(mtex, co);
+	externtex(mtex, co, &tin, &tr, &tg, &tb, &ta);
 
 	if(paf->texmap==PAF_TEXINT) {
-		Tin*= paf->texfac;
-		no[0]+= Tin*paf->defvec[0];
-		no[1]+= Tin*paf->defvec[1];
-		no[2]+= Tin*paf->defvec[2];
+		tin*= paf->texfac;
+		no[0]+= tin*paf->defvec[0];
+		no[1]+= tin*paf->defvec[1];
+		no[2]+= tin*paf->defvec[2];
 	}
 	else if(paf->texmap==PAF_TEXRGB) {
-		no[0]+= (Tr-0.5f)*paf->texfac;
-		no[1]+= (Tg-0.5f)*paf->texfac;
-		no[2]+= (Tb-0.5f)*paf->texfac;
+		no[0]+= (tr-0.5f)*paf->texfac;
+		no[1]+= (tg-0.5f)*paf->texfac;
+		no[2]+= (tb-0.5f)*paf->texfac;
 	}
 	else {	/* PAF_TEXGRAD */
 		
-		old= Tin;
+		old= tin;
 		co[0]+= paf->nabla;
-		externtex(mtex, co);
-		no[0]+= (old-Tin)*paf->texfac;
+		externtex(mtex, co, &tin, &tr, &tg, &tb, &ta);
+		no[0]+= (old-tin)*paf->texfac;
 		
 		co[0]-= paf->nabla;
 		co[1]+= paf->nabla;
-		externtex(mtex, co);
-		no[1]+= (old-Tin)*paf->texfac;
+		externtex(mtex, co, &tin, &tr, &tg, &tb, &ta);
+		no[1]+= (old-tin)*paf->texfac;
 		
 		co[1]-= paf->nabla;
 		co[2]+= paf->nabla;
-		externtex(mtex, co);
-		no[2]+= (old-Tin)*paf->texfac;
+		externtex(mtex, co, &tin, &tr, &tg, &tb, &ta);
+		no[2]+= (old-tin)*paf->texfac;
 		
 	}
 }
@@ -377,11 +381,18 @@ static int linetriangle(float p1[3], float p2[3], float v0[3], float v1[3], floa
 	return 1;
 }
 
-static void get_effector(float opco[], float force[], float speed[], float cur_time, unsigned int par_layer)
+/*  -------- pdDoEffector() --------
+    generic force/speed system, now used for particles and softbodies
+	opco		= global coord, as input
+    force		= force accumulator
+    speed		= speed accumulator
+    cur_time	= in frames
+    par_layer	= layer the caller is in
+
+*/
+void pdDoEffector(float *opco, float *force, float *speed, float cur_time, unsigned int par_layer,unsigned int flags)
 {
 /*
-	Particle effector field code
-
 	Modifies the force on a particle according to its
 	relation with the effector object
 	Different kind of effectors include:
@@ -394,46 +405,80 @@ static void get_effector(float opco[], float force[], float speed[], float cur_t
 */
 	Object *ob;
 	Base *base;
+	PartDeflect *pd;
 	float vect_to_vert[3];
 	float force_vec[3];
 	float f_force, distance;
-	float obloc[3];
+	float *obloc;
 	float force_val, ffall_val;
 	short cur_frame;
 
 	/* Cycle through objects, get total of (1/(gravity_strength * dist^gravity_power)) */
-	/* Check for min distance here? */
-	base = G.scene->base.first;
-	while (base) {
-		if(base->lay & par_layer) {
+	/* Check for min distance here? (yes would be cool to add that, ton) */
+	
+	for(base = G.scene->base.first; base; base= base->next) {
+		if( (base->lay & par_layer) && base->object->pd) {
 			ob= base->object;
-			if(ob->pd && ob->pd->forcefield == PFIELD_FORCE) {
-
-				/* Need to set r.cfra for paths (investigate, ton) */
+			pd= ob->pd;
+			
+			/* checking if to continue or not */
+			if(pd->forcefield==0) continue;
+			
+			/* Get IPO force strength and fall off values here */
+			if (has_ipo_code(ob->ipo, OB_PD_FSTR))
+				force_val = IPO_GetFloatValue(ob->ipo, OB_PD_FSTR, cur_time);
+			else 
+				force_val = pd->f_strength;
+			
+			if (has_ipo_code(ob->ipo, OB_PD_FFALL)) 
+				ffall_val = IPO_GetFloatValue(ob->ipo, OB_PD_FFALL, cur_time);
+			else 
+				ffall_val = pd->f_power;
+			
+			
+			/* Need to set r.cfra for paths (investigate, ton) (uses ob->ctime now, ton) */
+			if(ob->ctime!=cur_time) {
 				cur_frame = G.scene->r.cfra;
 				G.scene->r.cfra = (short)cur_time;
 				where_is_object_time(ob, cur_time);
 				G.scene->r.cfra = cur_frame;
+			}
+			
+			/* use center of object for distance calculus */
+			obloc= ob->obmat[3];
+			VECSUB(vect_to_vert, obloc, opco);
+			distance = VecLength(vect_to_vert);
+			
+			if((pd->flag & PFIELD_USEMAX) && distance>pd->maxdist)
+				;	/* don't do anything */
+			else if(pd->forcefield == PFIELD_WIND) {
+				VECCOPY(force_vec, ob->obmat[2]);
+				
+				/* wind works harder perpendicular to normal, would be nice for softbody later (ton) */
+				
+				/* Limit minimum distance to vertex so that */
+				/* the force is not too big */
+				if (distance < 0.001) distance = 0.001f;
+				f_force = (force_val)*(1/(1000 * (float)pow((double)distance, (double)ffall_val)));
+				if(flags &&PE_WIND_AS_SPEED){
+				speed[0] -= (force_vec[0] * f_force );
+				speed[1] -= (force_vec[1] * f_force );
+				speed[2] -= (force_vec[2] * f_force );
+				}
+				else{
+				force[0] += force_vec[0]*f_force;
+				force[1] += force_vec[1]*f_force;
+				force[2] += force_vec[2]*f_force;
+				}
+			}
+			else if(pd->forcefield == PFIELD_FORCE) {
 				
 				/* only use center of object */
-				obloc[0] = ob->obmat[3][0];
-				obloc[1] = ob->obmat[3][1];
-				obloc[2] = ob->obmat[3][2];
-				
-				/* Get IPO force strength and fall off values here */
-				if (has_ipo_code(ob->ipo, OB_PD_FSTR))
-					force_val = IPO_GetFloatValue(ob->ipo, OB_PD_FSTR, cur_time);
-				else 
-					force_val = ob->pd->f_strength;
-
-				if (has_ipo_code(ob->ipo, OB_PD_FFALL)) 
-					ffall_val = IPO_GetFloatValue(ob->ipo, OB_PD_FFALL, cur_time);
-				else 
-					ffall_val = ob->pd->f_power;
+				obloc= ob->obmat[3];
 
 				/* Now calculate the gravitational force */
 				VECSUB(vect_to_vert, obloc, opco);
-				distance = Normalise(vect_to_vert);
+				distance = VecLength(vect_to_vert);
 
 				/* Limit minimum distance to vertex so that */
 				/* the force is not too big */
@@ -444,33 +489,14 @@ static void get_effector(float opco[], float force[], float speed[], float cur_t
 				force[2] += (vect_to_vert[2] * f_force );
 
 			}
-			else if(ob->pd && ob->pd->forcefield == PFIELD_VORTEX) {
-				/* Need to set r.cfra for paths (investigate, ton) */
-				cur_frame = G.scene->r.cfra;
-				G.scene->r.cfra = (short)cur_time;
-				where_is_object_time(ob, cur_time);
-				G.scene->r.cfra = cur_frame;
-				
-				/* only use center of object */
-				obloc[0] = ob->obmat[3][0];
-				obloc[1] = ob->obmat[3][1];
-				obloc[2] = ob->obmat[3][2];
-				
-				/* Get IPO force strength and fall off values here */
-				if (has_ipo_code(ob->ipo, OB_PD_FSTR))
-					force_val = IPO_GetFloatValue(ob->ipo, OB_PD_FSTR, cur_time);
-				else 
-					force_val = ob->pd->f_strength;
+			else if(pd->forcefield == PFIELD_VORTEX) {
 
-				if (has_ipo_code(ob->ipo, OB_PD_FFALL)) 
-					ffall_val = IPO_GetFloatValue(ob->ipo, OB_PD_FFALL, cur_time);
-				else 
-					ffall_val = ob->pd->f_power;
+				/* only use center of object */
+				obloc= ob->obmat[3];
 
 				/* Now calculate the vortex force */
 				VECSUB(vect_to_vert, obloc, opco);
-
-				distance = Normalise(vect_to_vert);
+				distance = VecLength(vect_to_vert);
 
 				Crossf(force_vec, ob->obmat[2], vect_to_vert);
 				Normalise(force_vec);
@@ -485,7 +511,6 @@ static void get_effector(float opco[], float force[], float speed[], float cur_t
 
 			}
 		}
-		base = base->next;
 	}
 }
 
@@ -510,7 +535,7 @@ static void cache_object_vertices(Object *ob)
 	}
 }
 
-static int get_deflection(float opco[3], float npco[3], float opno[3],
+int pdDoDeflection(float opco[3], float npco[3], float opno[3],
         float npno[3], float life, float force[3], int def_depth,
         float cur_time, unsigned int par_layer, int *last_object,
 		int *last_face, int *same_face)
@@ -539,7 +564,7 @@ static int get_deflection(float opco[3], float npco[3], float opno[3],
 	float damping, perm_thresh;
 	float perm_val, rdamp_val;
 	int a, deflected=0, deflected_now=0;
-	float t, min_t;
+	float t,t2, min_t;
 	float mat[3][3], obloc[3];
 	short cur_frame;
 	float time_before, time_after;
@@ -583,6 +608,8 @@ static int get_deflection(float opco[3], float npco[3], float opno[3],
 					obloc[0] = ob->obmat[3][0];
 					obloc[1] = ob->obmat[3][1];
 					obloc[2] = ob->obmat[3][2];
+					vcache= NULL;
+
 				}
 				
 				while (a--) {
@@ -622,36 +649,44 @@ static int get_deflection(float opco[3], float npco[3], float opno[3],
 					}
 					
 					deflected_now = 0;
-					
-					t= 0.5;	// this is labda of line, can use it optimize quad intersection
-					
+
+						
+						
+//					t= 0.5;	// this is labda of line, can use it optimize quad intersection
+// sorry but no .. see below (BM)					
 					if( linetriangle(opco, npco, nv1, nv2, nv3, &t) ) {
 						if (t < min_t) {
 							deflected = 1;
 							deflected_now = 1;
 						}
 					}
-					else if (mface->v4 && (t>=0.0 && t<=1.0)) {
-						if( linetriangle(opco, npco, nv1, nv3, nv4, &t) ) {
-							if (t < min_t) {
+//					else if (mface->v4 && (t>=0.0 && t<=1.0)) {
+// no, you can't skip testing the other triangle
+// it might give a smaller t on (close to) the edge .. this is numerics not esoteric maths :)
+// note: the 2 triangles don't need to share a plane ! (BM)
+					if (mface->v4) {
+						if( linetriangle(opco, npco, nv1, nv3, nv4, &t2) ) {
+							if (t2 < min_t) {
 								deflected = 1;
 								deflected_now = 2;
 							}
 	  					}
 					}
 					
-					if ((deflected_now > 0) && (t < min_t)) {
+					if ((deflected_now > 0) && ((t < min_t) ||(t2 < min_t))) {
                     	min_t = t;
                     	ds_object = d_object;
 						ds_face = d_face;
 						deflection_object = ob;
 						deflection_face = mface;
 						if (deflected_now==1) {
+                    	min_t = t;
 							VECCOPY(dv1, nv1);
 							VECCOPY(dv2, nv2);
 							VECCOPY(dv3, nv3);
 						}
 						else {
+                    	min_t = t2;
 							VECCOPY(dv1, nv1);
 							VECCOPY(dv2, nv3);
 							VECCOPY(dv3, nv4);
@@ -664,6 +699,7 @@ static int get_deflection(float opco[3], float npco[3], float opno[3],
 		base = base->next;
 	}
 
+
 	/* Here's the point to do the permeability calculation */
 	/* Set deflected to 0 if a random number is below the value */
 	/* Get the permeability IPO here*/
@@ -872,7 +908,7 @@ void make_particle_keys(int depth, int nr, PartEff *paf, Particle *part, float *
 
 		/* Check force field */
 		cur_time = pa->time;
-		get_effector(opco, new_force, new_speed, cur_time, par_layer);
+		pdDoEffector(opco, new_force, new_speed, cur_time, par_layer,0);
 
 		/* new location */
 		pa->co[0]= opa->co[0] + deltalife * (opa->no[0] + new_speed[0] + 0.5f*new_force[0]);
@@ -907,7 +943,7 @@ void make_particle_keys(int depth, int nr, PartEff *paf, Particle *part, float *
 		/* Bail out if we've done the calculation 10 times - this seems ok     */
         /* for most scenes I've tested */
 		while (finish_defs) {
-			deflected =  get_deflection(opco, npco, opno, npno, life, new_force,
+			deflected =  pdDoDeflection(opco, npco, opno, npno, life, new_force,
 							def_count, cur_time, par_layer,
 							&last_ob, &last_fc, &same_fc);
 			if (deflected) {
@@ -1020,7 +1056,7 @@ void init_mv_jit(float *jit, int num,int seed2)
 }
 
 
-void give_mesh_mvert(Mesh *me, int nr, float *co, short *no, int seed2)
+static void give_mesh_mvert(Mesh *me, DispListMesh *dlm, int nr, float *co, short *no, int seed2)
 {
 	static float *jit=0;
 	static int jitlevel=1;
@@ -1037,16 +1073,11 @@ void give_mesh_mvert(Mesh *me, int nr, float *co, short *no, int seed2)
 		return;
 	}
 	
-	/* get it from displist? */
-	if(me->disp.first) {
-		DispList *dl= me->disp.first;
-		if(dl->type==DL_MESH) {
-			DispListMesh *dlm= dl->mesh;
-			mvertbase= dlm->mvert;
-			mfacebase= dlm->mface;
-			totface= dlm->totface;
-			totvert= dlm->totvert;
-		}
+	if(dlm) {
+		mvertbase= dlm->mvert;
+		mfacebase= dlm->mface;
+		totface= dlm->totface;
+		totvert= dlm->totvert;
 	}
 	
 	if(totvert==0) {
@@ -1131,6 +1162,7 @@ void build_particle_system(Object *ob)
 	MVert *mvert;
 	MTex *mtexmove=0;
 	Material *ma;
+	DispListMesh *dlm;
 	float framelenont, ftime, dtime, force[3], imat[3][3], vec[3];
 	float fac, prevobmat[4][4], sfraont, co[3];
 	int deform=0, a, cur, cfraont, cfralast, totpart;
@@ -1231,7 +1263,15 @@ void build_particle_system(Object *ob)
 	}
 	
 	/* init */
-	give_mesh_mvert(me, totpart, co, no, paf->seed);
+	if (mesh_uses_displist(me)) {
+		DerivedMesh *dm = mesh_get_derived(ob);
+
+		dlm = dm->convertToDispListMesh(dm);
+	} else {
+		dlm = NULL;
+	}
+
+	give_mesh_mvert(me, dlm, totpart, co, no, paf->seed);
 	
 	if(G.f & G_DEBUG) {
 		printf("\n");
@@ -1272,6 +1312,10 @@ void build_particle_system(Object *ob)
 					/* do_ob_ipo(par); */
 					par->ctime= -1234567.0;
 					do_ob_key(par);
+					if(par->type==OB_ARMATURE) {
+						do_all_actions(par);	// only does this object actions
+						clear_object_constraint_status(par);	// mysterious call, otherwise do_actions doesnt work???
+					}
 					par= par->parent;
 				}
 				if(ma) do_mat_ipo(ma);
@@ -1282,7 +1326,7 @@ void build_particle_system(Object *ob)
 			}
 		}
 		/* get coordinates */
-		if(paf->flag & PAF_FACE) give_mesh_mvert(me, a, co, no,paf->seed);
+		if(paf->flag & PAF_FACE) give_mesh_mvert(me, dlm, a, co, no, paf->seed);
 		else {
 			mvert= me->mvert + (a % me->totvert);
 			VECCOPY(co, mvert->co);
@@ -1339,7 +1383,7 @@ void build_particle_system(Object *ob)
 	/* restore */
 	G.scene->r.cfra= cfraont;
 	G.scene->r.framelen= framelenont;
-	give_mesh_mvert(0, 0, 0, 0,paf->seed);
+	give_mesh_mvert(0, 0, 0, 0, 0,paf->seed);
 
 	/* put hierarchy back */
 	par= ob;
@@ -1347,6 +1391,11 @@ void build_particle_system(Object *ob)
 		popfirst(par);
 		/* do not do ob->ipo: keep insertkey */
 		do_ob_key(par);
+		
+		if(par->type==OB_ARMATURE) {
+			do_all_actions(par);	// only does this object actions
+			clear_object_constraint_status(par);	// mysterious call, otherwise do_actions doesnt work???
+		}
 		par= par->parent;
 	}
 
@@ -1368,6 +1417,9 @@ void build_particle_system(Object *ob)
 
 	waitcursor(0);
 
+	if (dlm) {
+		displistmesh_free(dlm);
+	}
 }
 
 /* ************* WAVE **************** */
@@ -1465,3 +1517,246 @@ int object_wave(Object *ob)
 	}
 	return 1;
 }
+
+int SoftBodyDetectCollision(float opco[3], float npco[3], float colco[3],
+							float facenormal[3], float *damp, float force[3], int mode,
+							float cur_time, unsigned int par_layer,struct Object *vertexowner)
+{
+	Base *base;
+	Object *ob, *deflection_object = NULL;
+	Mesh *def_mesh;
+	MFace *mface, *deflection_face = NULL;
+	float *v1, *v2, *v3, *v4, *vcache=NULL;
+	float mat[3][3];
+	float nv1[3], nv2[3], nv3[3], nv4[3], edge1[3], edge2[3],d_nvect[3], obloc[3];
+	float dv1[3], dv2[3], dv3[3];
+	float facedist,n_mag,t,t2, min_t,force_mag_norm;
+	int a, deflected=0, deflected_now=0;
+	short cur_frame;
+	int d_object=0, d_face=0, ds_object=0, ds_face=0;
+	
+	// i'm going to rearrange it to declaration rules when WIP is finished (BM)
+	float innerfacethickness = -0.5f;
+	float outerfacethickness = 0.2f;
+	float ee = 5.0f;
+	float ff = 0.1f;
+	float fa;
+	
+	min_t = 200000;
+	
+	/* The first part of the code, finding the first intersected face*/
+	base= G.scene->base.first;
+	while (base) {
+		/*Only proceed for mesh object in same layer */
+		if(base->object->type==OB_MESH && (base->lay & par_layer)) {
+			ob= base->object;
+			if((vertexowner) && (ob == vertexowner)){ 
+			/* if vertexowner is given 
+				* we don't want to check collision with owner object */ 
+				base = base->next;
+				continue;
+			}
+			/* only with deflecting set */
+			if(ob->pd && ob->pd->deflect) {
+				def_mesh= ob->data;
+				
+				d_object = d_object + 1;
+				
+				d_face = d_face + 1;
+				mface= def_mesh->mface;
+				a = def_mesh->totface;
+				/* need to have user control for that since it depends on model scale */
+				
+				innerfacethickness =-ob->pd->pdef_sbift;
+				outerfacethickness =ob->pd->pdef_sboft;
+				fa = (ff*outerfacethickness-outerfacethickness);
+				fa *= fa;
+				fa = 1.0f/fa;
+				
+				if(ob->parent==NULL && ob->ipo==NULL) {	// static
+					if(ob->sumohandle==NULL) cache_object_vertices(ob);
+					vcache= ob->sumohandle;
+				}
+				else {
+					/*Find out where the object is at this time*/
+					cur_frame = G.scene->r.cfra;
+					G.scene->r.cfra = (short)cur_time;
+					where_is_object_time(ob, cur_time);
+					G.scene->r.cfra = cur_frame;
+					
+					/*Pass the values from ob->obmat to mat*/
+					/*and the location values to obloc           */
+					Mat3CpyMat4(mat,ob->obmat);
+					obloc[0] = ob->obmat[3][0];
+					obloc[1] = ob->obmat[3][1];
+					obloc[2] = ob->obmat[3][2];
+					/* not cachable */
+					vcache= NULL;
+					
+				}
+				
+				while (a--) {
+					
+					if(vcache) {
+						v1= vcache+ 3*(mface->v1);
+						VECCOPY(nv1, v1);
+						v1= vcache+ 3*(mface->v2);
+						VECCOPY(nv2, v1);
+						v1= vcache+ 3*(mface->v3);
+						VECCOPY(nv3, v1);
+						v1= vcache+ 3*(mface->v4);
+						VECCOPY(nv4, v1);
+					}
+					else {
+						/* Calculate the global co-ordinates of the vertices*/
+						v1= (def_mesh->mvert+(mface->v1))->co;
+						v2= (def_mesh->mvert+(mface->v2))->co;
+						v3= (def_mesh->mvert+(mface->v3))->co;
+						v4= (def_mesh->mvert+(mface->v4))->co;
+						
+						VECCOPY(nv1, v1);
+						VECCOPY(nv2, v2);
+						VECCOPY(nv3, v3);
+						VECCOPY(nv4, v4);
+						
+						/*Apply the objects deformation matrix*/
+						Mat3MulVecfl(mat, nv1);
+						Mat3MulVecfl(mat, nv2);
+						Mat3MulVecfl(mat, nv3);
+						Mat3MulVecfl(mat, nv4);
+						
+						VECADD(nv1, nv1, obloc);
+						VECADD(nv2, nv2, obloc);
+						VECADD(nv3, nv3, obloc);
+						VECADD(nv4, nv4, obloc);
+					}
+					
+					deflected_now = 0;
+					
+					if (mode == 1){ // face intrusion test
+						// switch origin to be nv2
+						VECSUB(edge1, nv1, nv2);
+						VECSUB(edge2, nv3, nv2);
+						VECSUB(dv1,opco,nv2); // abuse dv1 to have vertex in question at *origin* of triangle
+						
+						Crossf(d_nvect, edge2, edge1);
+						n_mag = Normalise(d_nvect);
+						facedist = Inpf(dv1,d_nvect);
+						
+						if ((facedist > innerfacethickness) && (facedist < outerfacethickness)){		
+							dv2[0] = opco[0] - 2.0f*facedist*d_nvect[0];
+							dv2[1] = opco[1] - 2.0f*facedist*d_nvect[1];
+							dv2[2] = opco[2] - 2.0f*facedist*d_nvect[2];
+							if ( linetriangle( opco, dv2, nv1, nv2, nv3, &t)){
+								force_mag_norm =(float)exp(-ee*facedist);
+								if (facedist > outerfacethickness*ff)
+									force_mag_norm =(float)force_mag_norm*fa*(facedist - outerfacethickness)*(facedist - outerfacethickness);
+								
+								force[0] += force_mag_norm*d_nvect[0] ;
+								force[1] += force_mag_norm*d_nvect[1] ;
+								force[2] += force_mag_norm*d_nvect[2] ;
+								*damp=ob->pd->pdef_sbdamp;
+								deflected = 2;
+							}
+						}		
+						if (mface->v4){ // quad
+							// switch origin to be nv4
+							VECSUB(edge1, nv3, nv4);
+							VECSUB(edge2, nv1, nv4);
+							VECSUB(dv1,opco,nv4); // abuse dv1 to have vertex in question at *origin* of triangle
+							
+							Crossf(d_nvect, edge2, edge1);
+							n_mag = Normalise(d_nvect);
+							facedist = Inpf(dv1,d_nvect);
+							
+							if ((facedist > innerfacethickness) && (facedist < outerfacethickness)){
+								dv2[0] = opco[0] - 2.0f*facedist*d_nvect[0];
+								dv2[1] = opco[1] - 2.0f*facedist*d_nvect[1];
+								dv2[2] = opco[2] - 2.0f*facedist*d_nvect[2];
+								if ( linetriangle( opco, dv2, nv1, nv3, nv4, &t)){
+									force_mag_norm =(float)exp(-ee*facedist);
+									if (facedist > outerfacethickness*ff)
+										force_mag_norm =(float)force_mag_norm*fa*(facedist - outerfacethickness)*(facedist - outerfacethickness);
+									
+									force[0] += force_mag_norm*d_nvect[0] ;
+									force[1] += force_mag_norm*d_nvect[1] ;
+									force[2] += force_mag_norm*d_nvect[2] ;
+									*damp=ob->pd->pdef_sbdamp;
+									deflected = 2;
+								}
+							}
+							
+						}
+					}
+					if (mode == 2){ // edge intrusion test
+						
+						
+						//t= 0.5;	// this is labda of line, can use it optimize quad intersection
+						// sorry but no .. see below (BM)					
+						if( linetriangle(opco, npco, nv1, nv2, nv3, &t) ) {
+							if (t < min_t) {
+								deflected = 1;
+								deflected_now = 1;
+							}
+						}
+						//					else if (mface->v4 && (t>=0.0 && t<=1.0)) {
+						// no, you can't skip testing the other triangle
+						// it might give a smaller t on (close to) the edge .. this is numerics not esoteric maths :)
+						// note: the 2 triangles don't need to share a plane ! (BM)
+						if (mface->v4) {
+							if( linetriangle(opco, npco, nv1, nv3, nv4, &t2) ) {
+								if (t2 < min_t) {
+									deflected = 1;
+									deflected_now = 2;
+								}
+							}
+						}
+						
+						if ((deflected_now > 0) && ((t < min_t) ||(t2 < min_t))) {
+							min_t = t;
+							ds_object = d_object;
+							ds_face = d_face;
+							deflection_object = ob;
+							deflection_face = mface;
+							if (deflected_now==1) {
+								min_t = t;
+								VECCOPY(dv1, nv1);
+								VECCOPY(dv2, nv2);
+								VECCOPY(dv3, nv3);
+							}
+							else {
+								min_t = t2;
+								VECCOPY(dv1, nv1);
+								VECCOPY(dv2, nv3);
+								VECCOPY(dv3, nv4);
+							}
+						}
+					}  
+					mface++;
+				}
+			}
+		}
+		base = base->next;
+	} // while (base)
+	
+	if (mode == 1){ // face 
+		return deflected;
+	}
+	
+	if (mode == 2){ // edge intrusion test
+		if (deflected) {
+			VECSUB(edge1, dv1, dv2);
+			VECSUB(edge2, dv3, dv2);
+			Crossf(d_nvect, edge2, edge1);
+			n_mag = Normalise(d_nvect);
+			// return point of intersection
+			colco[0] = opco[0] + (min_t * (npco[0] - opco[0]));
+			colco[1] = opco[1] + (min_t * (npco[1] - opco[1]));
+			colco[2] = opco[2] + (min_t * (npco[2] - opco[2]));
+			
+			VECCOPY(facenormal,d_nvect);					
+		}
+	}
+	return deflected;
+}
+
