@@ -1,6 +1,6 @@
-/* ipo.c 
+/* ipo.c
  * 
- * $Id: ipo.c,v 1.4 2003/04/26 11:56:44 ton Exp $
+ * $Id: ipo.c,v 1.18 2005/05/08 20:10:58 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -39,9 +39,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
 #include "MEM_guardedalloc.h"
 
 #include "DNA_ika_types.h"
@@ -55,9 +52,12 @@
 #include "DNA_texture_types.h"
 #include "DNA_material_types.h"
 #include "DNA_object_types.h"
+#include "DNA_object_force.h"
+#include "DNA_mesh_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_ipo_types.h"
 #include "DNA_action_types.h"
+
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 
@@ -72,6 +72,7 @@
 #include "BKE_blender.h"
 #include "BKE_ipo.h"
 #include "BKE_constraint.h"
+#include "BKE_mesh.h"
 
 #define SMALL -1.0e-10
 
@@ -88,7 +89,8 @@ int ob_ar[OB_TOTIPO]= {
 	OB_LOC_X, OB_LOC_Y, OB_LOC_Z, OB_DLOC_X, OB_DLOC_Y, OB_DLOC_Z, 
 	OB_ROT_X, OB_ROT_Y, OB_ROT_Z, OB_DROT_X, OB_DROT_Y, OB_DROT_Z, 
 	OB_SIZE_X, OB_SIZE_Y, OB_SIZE_Z, OB_DSIZE_X, OB_DSIZE_Y, OB_DSIZE_Z, 
-	OB_LAY, OB_TIME, OB_EFF_X, OB_EFF_Y, OB_EFF_Z, OB_COL_A
+	OB_LAY, OB_TIME, OB_EFF_X, OB_EFF_Y, OB_EFF_Z, OB_COL_A,
+	OB_PD_FSTR, OB_PD_FFALL, OB_PD_SDAMP, OB_PD_RDAMP, OB_PD_PERM
 };
 
 int ac_ar[AC_TOTIPO]= {
@@ -100,15 +102,30 @@ int ac_ar[AC_TOTIPO]= {
 int ma_ar[MA_TOTIPO]= {
 	MA_COL_R, MA_COL_G, MA_COL_B, 
 	MA_SPEC_R, MA_SPEC_G, MA_SPEC_B, 
-	MA_MIR_R, MA_MIR_G, MA_MIR_B, 
+	MA_MIR_R, MA_MIR_G, MA_MIR_B,
 	MA_REF, MA_ALPHA, MA_EMIT, MA_AMB, 
-	MA_SPEC, MA_HARD, MA_SPTR, MA_ANG, 
-	MA_MODE, MA_HASIZE, 
+	MA_SPEC, MA_HARD, MA_SPTR, MA_IOR, 
+	MA_MODE, MA_HASIZE, MA_TRANSLU, MA_RAYM,
+	MA_FRESMIR, MA_FRESMIRI, MA_FRESTRA, MA_FRESTRAI, MA_ADD,
 	
 	MA_MAP1+MAP_OFS_X, MA_MAP1+MAP_OFS_Y, MA_MAP1+MAP_OFS_Z, 
 	MA_MAP1+MAP_SIZE_X, MA_MAP1+MAP_SIZE_Y, MA_MAP1+MAP_SIZE_Z, 
 	MA_MAP1+MAP_R, MA_MAP1+MAP_G, MA_MAP1+MAP_B,
-	MA_MAP1+MAP_DVAR, MA_MAP1+MAP_COLF, MA_MAP1+MAP_NORF, MA_MAP1+MAP_VARF
+	MA_MAP1+MAP_DVAR, MA_MAP1+MAP_COLF, MA_MAP1+MAP_NORF, MA_MAP1+MAP_VARF, MA_MAP1+MAP_DISP
+};
+
+int te_ar[TE_TOTIPO] ={
+	
+	TE_NSIZE, TE_NDEPTH, TE_NTYPE, TE_TURB,
+	
+	TE_VNW1, TE_VNW2, TE_VNW3, TE_VNW4,
+	TE_VNMEXP, TE_VN_COLT, TE_VN_DISTM,
+	
+	TE_ISCA, TE_DISTA,
+	
+	TE_MG_TYP, TE_MGH, TE_MG_LAC, TE_MG_OCT, TE_MG_OFF, TE_MG_GAIN,
+	
+	TE_N_BAS1, TE_N_BAS2
 };
 
 int seq_ar[SEQ_TOTIPO]= {
@@ -122,7 +139,10 @@ int cu_ar[CU_TOTIPO]= {
 int key_ar[KEY_TOTIPO]= {
 	KEY_SPEED, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 
 	11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 
-	21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31
+	21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
+	32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42,
+	43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53,
+	54, 55, 56, 57, 58, 59, 60, 61, 62, 63
 };
 
 int wo_ar[WO_TOTIPO]= {
@@ -144,11 +164,12 @@ int la_ar[LA_TOTIPO]= {
 	MA_MAP1+MAP_OFS_X, MA_MAP1+MAP_OFS_Y, MA_MAP1+MAP_OFS_Z, 
 	MA_MAP1+MAP_SIZE_X, MA_MAP1+MAP_SIZE_Y, MA_MAP1+MAP_SIZE_Z, 
 	MA_MAP1+MAP_R, MA_MAP1+MAP_G, MA_MAP1+MAP_B,
-	MA_MAP1+MAP_DVAR, MA_MAP1+MAP_COLF, MA_MAP1+MAP_NORF, MA_MAP1+MAP_VARF
+	MA_MAP1+MAP_DVAR, MA_MAP1+MAP_COLF
 };
 
+/* yafray: aperture & focal distance curves added */
 int cam_ar[CAM_TOTIPO]= {
-	CAM_LENS, CAM_STA, CAM_END
+	CAM_LENS, CAM_STA, CAM_END, CAM_YF_APERT, CAM_YF_FDIST
 };
 
 int snd_ar[SND_TOTIPO]= {
@@ -205,7 +226,7 @@ Ipo *copy_ipo(Ipo *ipo)
 	ipon= copy_libblock(ipo);
 	
 	duplicatelist(&(ipon->curve), &(ipo->curve));
-	
+
 	icu= ipon->curve.first;
 	while(icu) {
 		icu->bezt= MEM_dupallocN(icu->bezt);
@@ -701,7 +722,7 @@ float eval_icu(IpoCurve *icu, float ipotime)
 				prevbezt+= a;
 				dx= ipotime-prevbezt->vec[1][0];
 				fac= prevbezt->vec[2][0]-prevbezt->vec[1][0];
-				
+
 				if(fac!=0) {
 					fac= (prevbezt->vec[2][1]-prevbezt->vec[1][1])/fac;
 					cvalue= prevbezt->vec[1][1]+fac*dx;
@@ -856,6 +877,58 @@ float read_ipo_poin(void *poin, int type)
 	return val;
 }
 
+static void *give_tex_poin(Tex *tex, int adrcode, int *type )
+{
+	void *poin=0;
+
+	switch(adrcode) {
+	case TE_NSIZE:
+		poin= &(tex->noisesize); break;
+	case TE_TURB:
+		poin= &(tex->turbul); break;
+	case TE_NDEPTH:
+		poin= &(tex->noisedepth); *type= IPO_SHORT; break;
+	case TE_NTYPE:
+		poin= &(tex->noisetype); *type= IPO_SHORT; break;
+	case TE_VNW1:
+		poin= &(tex->vn_w1); break;
+	case TE_VNW2:
+		poin= &(tex->vn_w2); break;
+	case TE_VNW3:
+		poin= &(tex->vn_w3); break;
+	case TE_VNW4:
+		poin= &(tex->vn_w4); break;
+	case TE_VNMEXP:
+		poin= &(tex->vn_mexp); break;
+	case TE_ISCA:
+		poin= &(tex->ns_outscale); break;
+	case TE_DISTA:
+		poin= &(tex->dist_amount); break;
+	case TE_VN_COLT:
+		poin= &(tex->vn_coltype); *type= IPO_SHORT; break;
+	case TE_VN_DISTM:
+		poin= &(tex->vn_distm); *type= IPO_SHORT; break;
+	case TE_MG_TYP:
+		poin= &(tex->stype); *type= IPO_SHORT; break;
+	case TE_MGH:
+		poin= &(tex->mg_H); break;
+	case TE_MG_LAC:
+		poin= &(tex->mg_lacunarity); break;
+	case TE_MG_OCT:
+		poin= &(tex->mg_octaves); break;
+	case TE_MG_OFF:
+		poin= &(tex->mg_offset); break;
+	case TE_MG_GAIN:
+		poin= &(tex->mg_gain); break;
+	case TE_N_BAS1:
+		poin= &(tex->noisebasis); *type= IPO_SHORT; break;
+	case TE_N_BAS2:
+		poin= &(tex->noisebasis2); *type= IPO_SHORT; break;
+	}
+	
+	return poin;
+}
+
 void *give_mtex_poin(MTex *mtex, int adrcode )
 {
 	void *poin=0;
@@ -887,6 +960,8 @@ void *give_mtex_poin(MTex *mtex, int adrcode )
 		poin= &(mtex->norfac); break;
 	case MAP_VARF:
 		poin= &(mtex->varfac); break;
+	case MAP_DISP:
+		poin= &(mtex->dispfac); break;
 	}
 	
 	return poin;
@@ -907,11 +982,11 @@ void *give_mtex_poin(MTex *mtex, int adrcode )
 
 void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 {
-	void *poin= 0;
+	void *poin= NULL;
 	Object *ob;
 	Material *ma;
 	MTex *mtex;
-	Ika *ika= 0;
+	Tex *tex;
 	Lamp *la;
 	Sequence *seq;
 	World *wo;
@@ -921,13 +996,10 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 
 	*type= IPO_FLOAT;
 
-
 	if( GS(id->name)==ID_OB) {
 	
 		ob= (Object *)id;
 
-		if(ob->type==OB_IKA) ika= ob->data;
-		
 		switch(icu->adrcode) {
 		case OB_LOC_X:
 			poin= &(ob->loc[0]); break;
@@ -967,25 +1039,37 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 			poin= &(ob->dsize[1]); break;
 		case OB_DSIZE_Z:
 			poin= &(ob->dsize[2]); break;
-		
+
 		case OB_LAY:
 			poin= &(ob->lay); *type= IPO_INT_BIT; break;
 			
-		case OB_EFF_X:	/* OB_COL_R */
-			if(ika) poin= &(ika->effg[0]);
-			else poin= &(ob->col[0]);
+		case OB_COL_R:	
+			poin= &(ob->col[0]);
 			break;
-		case OB_EFF_Y:	/* OB_COL_G */
-			if(ika) poin= &(ika->effg[1]);
-			else poin= &(ob->col[1]);
+		case OB_COL_G:
+			poin= &(ob->col[1]);
 			break;
-		case OB_EFF_Z:	/* OB_COL_B */
-			if(ika) poin= &(ika->effg[2]);
-			else poin= &(ob->col[2]);
+		case OB_COL_B:
+			poin= &(ob->col[2]);
 			break;
 		case OB_COL_A:
 			poin= &(ob->col[3]);
 			break;
+		case OB_PD_FSTR:
+			if(ob->pd) poin= &(ob->pd->f_strength);
+			break;
+		case OB_PD_FFALL:
+			if(ob->pd) poin= &(ob->pd->f_power);
+			break;
+		case OB_PD_SDAMP:
+			if(ob->pd) poin= &(ob->pd->pdef_damp);
+			break;
+		case OB_PD_RDAMP:
+			if(ob->pd) poin= &(ob->pd->pdef_rdamp);
+			break;
+		case OB_PD_PERM:
+			if(ob->pd) poin= &(ob->pd->pdef_perm);
+			break;
 		}
 	}
 	else if (GS(id->name)==ID_AC){
@@ -1055,12 +1139,26 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 			poin= &(ma->har); *type= IPO_SHORT; break;
 		case MA_SPTR:
 			poin= &(ma->spectra); break;
-		case MA_ANG:
+		case MA_IOR:
 			poin= &(ma->ang); break;
 		case MA_MODE:
 			poin= &(ma->mode); *type= IPO_INT_BIT; break;
 		case MA_HASIZE:
 			poin= &(ma->hasize); break;
+		case MA_TRANSLU:
+			poin= &(ma->translucency); break;
+		case MA_RAYM:
+			poin= &(ma->ray_mirror); break;
+		case MA_FRESMIR:
+			poin= &(ma->fresnel_mir); break;
+		case MA_FRESMIRI:
+			poin= &(ma->fresnel_mir_i); break;
+		case MA_FRESTRA:
+			poin= &(ma->fresnel_tra); break;
+		case MA_FRESTRAI:
+			poin= &(ma->fresnel_tra_i); break;
+		case MA_ADD:
+			poin= &(ma->add); break;
 		}
 		
 		if(poin==0) {
@@ -1073,12 +1171,19 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 			else if(icu->adrcode & MA_MAP6) mtex= ma->mtex[5];
 			else if(icu->adrcode & MA_MAP7) mtex= ma->mtex[6];
 			else if(icu->adrcode & MA_MAP8) mtex= ma->mtex[7];
+			else if(icu->adrcode & MA_MAP9) mtex= ma->mtex[8];
+			else if(icu->adrcode & MA_MAP10) mtex= ma->mtex[9];
 			
 			if(mtex) {
 				poin= give_mtex_poin(mtex, icu->adrcode & (MA_MAP1-1) );
 			}
 		}
 	}
+	else if( GS(id->name)==ID_TE) {
+		tex= (Tex *)id;
+		
+		if(tex) poin= give_tex_poin(tex, icu->adrcode, type);
+	}
 	else if( GS(id->name)==ID_SEQ) {
 		seq= (Sequence *)id;
 		
@@ -1150,6 +1255,8 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 			else if(icu->adrcode & MA_MAP6) mtex= wo->mtex[5];
 			else if(icu->adrcode & MA_MAP7) mtex= wo->mtex[6];
 			else if(icu->adrcode & MA_MAP8) mtex= wo->mtex[7];
+			else if(icu->adrcode & MA_MAP9) mtex= wo->mtex[8];
+			else if(icu->adrcode & MA_MAP10) mtex= wo->mtex[9];
 			
 			if(mtex) {
 				poin= give_mtex_poin(mtex, icu->adrcode & (MA_MAP1-1) );
@@ -1193,6 +1300,8 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 			else if(icu->adrcode & MA_MAP6) mtex= la->mtex[5];
 			else if(icu->adrcode & MA_MAP7) mtex= la->mtex[6];
 			else if(icu->adrcode & MA_MAP8) mtex= la->mtex[7];
+			else if(icu->adrcode & MA_MAP9) mtex= la->mtex[8];
+			else if(icu->adrcode & MA_MAP10) mtex= la->mtex[9];
 			
 			if(mtex) {
 				poin= give_mtex_poin(mtex, icu->adrcode & (MA_MAP1-1) );
@@ -1202,6 +1311,7 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 	else if(GS(id->name)==ID_CA) {
 		Camera *ca= (Camera *)id;
 		
+		/* yafray: aperture & focal distance params */
 		switch(icu->adrcode) {
 		case CAM_LENS:
 			poin= &(ca->lens); break;
@@ -1209,6 +1319,10 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 			poin= &(ca->clipsta); break;
 		case CAM_END:
 			poin= &(ca->clipend); break;
+		case CAM_YF_APERT:
+			poin= &(ca->YF_aperture); break;
+		case CAM_YF_FDIST:
+			poin= &(ca->YF_dofdist); break;
 		}
 	}
 	else if(GS(id->name)==ID_SO) {
@@ -1255,12 +1369,28 @@ void set_icu_vars(IpoCurve *icu)
 				icu->ymax= 2.0; break;
 			case MA_MODE:
 				icu->ipo= IPO_CONST;
-				icu->vartype= IPO_BITS;
-				break;
-				
+				icu->vartype= IPO_BITS; break;
+			case MA_RAYM:
+				icu->ymax= 1.0; break;
+			case MA_TRANSLU:
+				icu->ymax= 1.0; break;
+			case MA_IOR:
+				icu->ymin= 1.0;
+				icu->ymax= 3.0; break;
+			case MA_FRESMIR:
+				icu->ymax= 5.0; break;
+			case MA_FRESMIRI:
+				icu->ymin= 1.0;
+				icu->ymax= 5.0; break;
+			case MA_FRESTRA:
+				icu->ymax= 5.0; break;
+			case MA_FRESTRAI:
+				icu->ymin= 1.0;
+				icu->ymax= 5.0; break;
+			case MA_ADD:
+				icu->ymax= 1.0; break;
 			default:
-				icu->ymax= 1.0;
-				break;
+				icu->ymax= 1.0; break;
 			}
 		}
 		else {
@@ -1281,14 +1411,72 @@ void set_icu_vars(IpoCurve *icu)
 			case MAP_DVAR:
 			case MAP_COLF:
 			case MAP_VARF:
+			case MAP_DISP:
 				icu->ymax= 1.0;
 				break;
 			case MAP_NORF:
-				icu->ymax= 5.0;
+				icu->ymax= 25.0;
 				break;
 			}
 		}
 	}
+	else if(icu->blocktype==ID_TE) {
+		switch(icu->adrcode & (MA_MAP1-1)) {
+			case TE_NSIZE:
+				icu->ymin= 0.0001;
+				icu->ymax= 2.0; break;
+			case TE_NDEPTH:
+				icu->vartype= IPO_SHORT;
+				icu->ipo= IPO_CONST;
+				icu->ymax= 6.0; break;
+			case TE_NTYPE:
+				icu->vartype= IPO_SHORT;
+				icu->ipo= IPO_CONST;
+				icu->ymax= 1.0; break;
+			case TE_TURB:
+				icu->ymax= 200.0; break;
+			case TE_VNW1:
+			case TE_VNW2:
+			case TE_VNW3:
+			case TE_VNW4:
+				icu->ymax= 2.0;
+				icu->ymin= -2.0; break;
+			case TE_VNMEXP:
+				icu->ymax= 10.0;
+				icu->ymin= 0.01; break;
+			case TE_VN_DISTM:
+				icu->vartype= IPO_SHORT;
+				icu->ipo= IPO_CONST;
+				icu->ymax= 6.0; break;
+			case TE_VN_COLT:
+				icu->vartype= IPO_SHORT;
+				icu->ipo= IPO_CONST;
+				icu->ymax= 3.0; break;
+			case TE_ISCA:
+				icu->ymax= 10.0;
+				icu->ymin= 0.01; break;
+			case TE_DISTA:
+				icu->ymax= 10.0; break;
+			case TE_MG_TYP:
+				icu->vartype= IPO_SHORT;
+				icu->ipo= IPO_CONST;
+				icu->ymax= 4.0; break;
+			case TE_MGH:
+				icu->ymin= 0.0001;
+				icu->ymax= 2.0; break;
+			case TE_MG_LAC:
+			case TE_MG_OFF:
+			case TE_MG_GAIN:
+				icu->ymax= 6.0; break;
+			case TE_MG_OCT:
+				icu->ymax= 8.0; break;
+			case TE_N_BAS1:
+			case TE_N_BAS2:
+				icu->vartype= IPO_SHORT;
+				icu->ipo= IPO_CONST;
+				icu->ymax= 8.0; break;
+		}
+	}
 	else if(icu->blocktype==ID_SEQ) {
 	
 		icu->ymax= 1.0;
@@ -1336,6 +1524,7 @@ void set_icu_vars(IpoCurve *icu)
 			case MAP_COLF:
 			case MAP_NORF:
 			case MAP_VARF:
+			case MAP_DISP:
 				icu->ymax= 1.0;
 			}
 		}
@@ -1378,21 +1567,32 @@ void set_icu_vars(IpoCurve *icu)
 			case MAP_COLF:
 			case MAP_NORF:
 			case MAP_VARF:
+			case MAP_DISP:
 				icu->ymax= 1.0;
 			}
 		}
 	}	
 	else if(icu->blocktype==ID_CA) {
 
+		/* yafray: aperture & focal distance params */
 		switch(icu->adrcode) {
 		case CAM_LENS:
 			icu->ymin= 5.0;
-			icu->ymax= 1000.0; break;
+			icu->ymax= 1000.0;
+			break;
 		case CAM_STA:
 			icu->ymin= 0.001f;
 			break;
 		case CAM_END:
 			icu->ymin= 0.1f;
+			break;
+		case CAM_YF_APERT:
+			icu->ymin = 0.0;
+			icu->ymax = 2.0;
+			break;
+		case CAM_YF_FDIST:
+			icu->ymin = 0.0;
+			icu->ymax = 5000.0;
 		}
 	}
 	else if(icu->blocktype==ID_SO) {
@@ -1442,6 +1642,7 @@ void do_ipo_nocalc(Ipo *ipo)
 {
 	Object *ob;
 	Material *ma;
+	Tex *tex;
 	World *wo;
 	Lamp *la;
 	Camera *ca;
@@ -1467,6 +1668,13 @@ void do_ipo_nocalc(Ipo *ipo)
 			ma= ma->id.next;
 		}
 		break;
+	case ID_TE:
+		tex= G.main->tex.first;
+		while(tex) {
+			if(tex->ipo==ipo) execute_ipo((ID *)tex, ipo);
+			tex=tex->id.next;
+		}
+		break;
 	case ID_WO:
 		wo= G.main->world.first;
 		while(wo) {
@@ -1596,11 +1804,14 @@ void do_all_ipos()
 {
 	Base *base;
 	Material *ma;
+	Tex *tex;
 	World *wo;
 	Ipo *ipo;
 	Lamp *la;
 	Camera *ca;
 	bSound *snd;
+	Sequence *seq;
+	Editing *ed;
 	float ctime;
 	int set;
 
@@ -1635,6 +1846,12 @@ void do_all_ipos()
 		}
 	}
 
+	tex= G.main->tex.first;
+	while(tex) {
+		if(tex->ipo) execute_ipo((ID *)tex, tex->ipo);
+		tex= tex->id.next;
+	}
+
 	ma= G.main->mat.first;
 	while(ma) {
 		if(ma->ipo) execute_ipo((ID *)ma, ma->ipo);
@@ -1690,6 +1907,18 @@ void do_all_ipos()
 			base= base->next;
 		}
 	}
+
+	/* intrr: process FAC Ipos used as volume envelopes */
+	ed= G.scene->ed;
+	if (ed) {
+		seq= ed->seqbasep->first;
+		while(seq) {
+			if ((seq->type == SEQ_SOUND) && (seq->ipo)
+			  &&(seq->startdisp<=G.scene->r.cfra+2) && (seq->enddisp>G.scene->r.cfra)) do_seq_ipo(seq);
+			seq= seq->next;
+		}
+	}
+
 }
 
 
@@ -1794,6 +2023,11 @@ void make_cfra_list(Ipo *ipo, ListBase *elems)
 				case OB_SIZE_X:
 				case OB_SIZE_Y:
 				case OB_SIZE_Z:
+				case OB_PD_FSTR:
+				case OB_PD_FFALL:
+				case OB_PD_SDAMP:
+				case OB_PD_RDAMP:
+				case OB_PD_PERM:
 					bezt= icu->bezt;
 					if(bezt) {
 						a= icu->totvert;
@@ -1808,6 +2042,50 @@ void make_cfra_list(Ipo *ipo, ListBase *elems)
 			icu= icu->next;
 		}
 	}
+	else if(ipo->blocktype==ID_AC) {
+		icu= ipo->curve.first;
+		while(icu) {
+			if(icu->flag & IPO_VISIBLE) {
+				switch(icu->adrcode) {
+				case AC_LOC_X:
+				case AC_LOC_Y:
+				case AC_LOC_Z:
+				case AC_SIZE_X:
+				case AC_SIZE_Y:
+				case AC_SIZE_Z:
+				case AC_QUAT_W:
+				case AC_QUAT_X:
+				case AC_QUAT_Y:
+				case AC_QUAT_Z:
+					bezt= icu->bezt;
+					if(bezt) {
+						a= icu->totvert;
+						while(a--) {
+							add_to_cfra_elem(elems, bezt);
+							bezt++;
+						}
+					}
+					break;
+				}
+			}
+			icu= icu->next;
+		}
+	}
+	else {
+		for(icu= ipo->curve.first; icu; icu= icu->next) {
+			if(icu->flag & IPO_VISIBLE) {
+				bezt= icu->bezt;
+				if(bezt) {
+					a= icu->totvert;
+					while(a--) {
+						add_to_cfra_elem(elems, bezt);
+						bezt++;
+					}
+				}
+			}
+		}
+	}
+	
 	if(ipo->showkey==0) {
 		/* deselect all keys */
 		ce= elems->first;
