@@ -4,7 +4,7 @@
  *  Contains management of ID's and libraries
  *  allocate and free of all library data
  * 
- * $Id: library.c,v 1.5 2003/10/25 00:08:10 ton Exp $
+ * $Id: library.c,v 1.12 2005/04/02 15:36:56 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,10 +43,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include "MEM_guardedalloc.h"
 
 /* all types are needed here, in order to do memory operations */
@@ -86,11 +82,12 @@
 #include "BKE_sound.h"
 #include "BKE_object.h"
 #include "BKE_screen.h"
+#include "BKE_script.h"
 #include "BKE_mesh.h"
 #include "BKE_material.h"
 #include "BKE_curve.h"
 #include "BKE_mball.h"
- #include "BKE_text.h"
+#include "BKE_text.h"
 #include "BKE_texture.h"
 #include "BKE_scene.h"
 #include "BKE_image.h"
@@ -103,9 +100,10 @@
 #include "BKE_lattice.h"
 #include "BKE_armature.h"
 #include "BKE_action.h"
+#include "BPI_script.h"
 
 #define MAX_IDPUP		30	/* was 24 */
-#define MAX_LIBARRAY	100 /* was 30 */
+#define MAX_LIBARRAY	100 /* was 30, warning: used it readfile.c too */
 
 /* ************* general ************************ */
 
@@ -173,6 +171,8 @@ ListBase *wich_libbase(Main *mainlib, short type)
 			return &(mainlib->vfont);
 		case ID_TXT:
 			return &(mainlib->text);
+		case ID_SCRIPT:
+			return &(mainlib->script);
 		case ID_SO:
 			return &(mainlib->sound);
 		case ID_SAMPLE:
@@ -225,9 +225,10 @@ int set_listbasepointers(Main *main, ListBase **lb)
 	lb[23]= &(main->group);
 
 	lb[24]= samples;
-	lb[25]= 0;
-	
-	return 25;
+	lb[25]= &(main->script);
+	lb[26]=0;
+
+	return 26;
 }
 
 /* *********** ALLOC AND FREE *****************
@@ -305,6 +306,9 @@ static ID *alloc_libblock_notest(short type)
 		case ID_TXT:
 			id= MEM_callocN(sizeof(Text), "text");
 			break;
+		case ID_SCRIPT:
+			id= MEM_callocN(sizeof(Script), "script");
+			break;
 		case ID_SO:
 			id= MEM_callocN(sizeof(bSound), "sound");
 			break;
@@ -450,6 +454,9 @@ void free_libblock(ListBase *lb, void *idv)
 		case ID_TXT:
 			free_text((Text *)id);
 			break;
+		case ID_SCRIPT:
+			free_script((Script *)id);
+			break;
 		case ID_SO:
 			sound_free_sound((bSound *)id);
 			break;
@@ -503,7 +510,7 @@ void free_main(Main *mainvar)
 		ListBase *lb= lbarray[a];
 		ID *id;
 		
-		while (id= lb->first) {
+		while ( (id= lb->first) ) {
 			free_libblock(lb, id);
 		}
 	}
@@ -551,9 +558,9 @@ static void IDnames_to_dyn_pupstring(DynStr *pupds, ListBase *lb, ID *link, shor
 {
 	int i, nids= BLI_countlist(lb);
 		
-	*nr= -1;
+	if (nr) *nr= -1;
 	
-	if (nids>MAX_IDPUP) {
+	if (nr && nids>MAX_IDPUP) {
 		BLI_dynstr_append(pupds, "DataBrowse %x-2");
 	} else {
 		ID *id;
@@ -561,8 +568,7 @@ static void IDnames_to_dyn_pupstring(DynStr *pupds, ListBase *lb, ID *link, shor
 		for (i=0, id= lb->first; id; id= id->next, i++) {
 			char buf[32];
 			
-			if (id==link)
-				*nr= i+1;
+			if (nr && id==link) *nr= i+1;
 			
 			get_flags_for_id(id, buf);
 				
@@ -622,6 +628,7 @@ static void IPOnames_to_dyn_pupstring(DynStr *pupds, ListBase *lb, ID *link, sho
 }
 
 // used by headerbuttons.c buttons.c editobject.c editseq.c
+// if nr==NULL no MAX_IDPUP, this for non-header browsing
 void IDnames_to_pupstring(char **str, char *title, char *extraops, ListBase *lb, ID *link, short *nr)
 {
 	DynStr *pupds= BLI_dynstr_new();
@@ -699,7 +706,7 @@ static void sort_alpha_id(ListBase *lb, ID *id)
 		
 		idtest= lb->first;
 		while(idtest) {
-			if(strcasecmp(idtest->name, id->name)>0 || idtest->lib) {
+			if(BLI_strcasecmp(idtest->name, id->name)>0 || idtest->lib) {
 				BLI_insertlinkbefore(lb, idtest, id);
 				break;
 			}
@@ -799,7 +806,7 @@ int new_id(ListBase *lb, ID *id, char *tname)
 			return (new_id(lb, id, left));
 		}
 		/* this format specifier is from hell... */
-		sprintf(id->name+2, "%s.%0.3d", left, nr);
+		sprintf(id->name+2, "%s.%.3d", left, nr);
 	}
 	
 	sort_alpha_id(lb, id);	
@@ -825,7 +832,8 @@ void clear_id_newpoins()
 	}
 }
 
-void all_local(void)
+/* if lib!=NULL, only all from lib local */
+void all_local(Library *lib)
 {
 	ListBase *lbarray[MAX_LIBARRAY], tempbase={0, 0};
 	ID *id, *idn;
@@ -836,35 +844,20 @@ void all_local(void)
 		id= lbarray[a]->first;
 		
 		while(id) {
-			id->newid= 0;
-			id->flag &= ~(LIB_EXTERN|LIB_INDIRECT|LIB_NEW);
-			
+			id->newid= NULL;
 			idn= id->next;		/* id is possibly being inserted again */
-			if(id->lib) {
-				id->lib= 0;
-				new_id(lbarray[a], id, 0);	/* new_id only does it with double names */
-				sort_alpha_id(lbarray[a], id);
-			}
-			else {
-				/* patch: check for alphabetic ordering */
-			        /* has been removed... why!? (ton) */
-/*
-				if(idn) {
-					if(strcasecmp(id->name, idn->name)>0) {
-						remlink(lbarray[a], id);
-						addtail(&tempbase, id);
-					}
-					else if(id->prev) {
-						idp= id->prev;
-						if(strcasecmp(idp->name, id->name)>0) {
-							remlink(lbarray[a], id);
-							addtail(&tempbase, id);
-						}
+			
+			if(id->flag & (LIB_EXTERN|LIB_INDIRECT|LIB_NEW)) {
+				if(lib==NULL || id->lib==lib) {
+					id->flag &= ~(LIB_EXTERN|LIB_INDIRECT|LIB_NEW);
+
+					if(id->lib) {
+						id->lib= NULL;
+						new_id(lbarray[a], id, 0);	/* new_id only does it with double names */
+						sort_alpha_id(lbarray[a], id);
 					}
 				}
-*/				
 			}
-			
 			id= idn;
 		}
 		
