@@ -2,7 +2,7 @@
 /*  material.c
  *
  * 
- * $Id: material.c,v 1.18 2004/03/20 23:59:57 zuster Exp $
+ * $Id: material.c,v 1.26 2005/04/17 17:43:06 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -59,10 +59,6 @@
 
 #include "BPY_extern.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 void free_material(Material *ma)
 {
 	int a;
@@ -70,14 +66,14 @@ void free_material(Material *ma)
 
 	BPY_free_scriptlink(&ma->scriptlink);
 	
-	if(ma->ren) MEM_freeN(ma->ren);
-	ma->ren= 0;
-	
-	for(a=0; a<8; a++) {
+	for(a=0; a<MAX_MTEX; a++) {
 		mtex= ma->mtex[a];
 		if(mtex && mtex->tex) mtex->tex->id.us--;
 		if(mtex) MEM_freeN(mtex);
 	}
+	
+	if(ma->ramp_col) MEM_freeN(ma->ramp_col);
+	if(ma->ramp_spec) MEM_freeN(ma->ramp_spec);
 }
 
 void init_material(Material *ma)
@@ -104,6 +100,8 @@ void init_material(Material *ma)
 	ma->param[1]= 0.1;
 	ma->param[2]= 0.5;
 	ma->param[3]= 0.1;
+	ma->rms=0.1;
+	ma->darkness=1.0;	
 	
 	ma->ang= 1.0;
 	ma->ray_depth= 2;
@@ -113,6 +111,10 @@ void init_material(Material *ma)
 	ma->fresnel_tra_i= 1.25;
 	ma->fresnel_mir_i= 1.25;
 	
+	ma->rampfac_col= 1.0;
+	ma->rampfac_spec= 1.0;
+	ma->pr_lamp= 3; // two lamps, is bits
+	
 	ma->mode= MA_TRACEBLE+MA_SHADOW+MA_RADIO;	
 }
 
@@ -136,7 +138,7 @@ Material *copy_material(Material *ma)
 	
 	id_us_plus((ID *)man->ipo);
 	
-	for(a=0; a<8; a++) {
+	for(a=0; a<MAX_MTEX; a++) {
 		if(ma->mtex[a]) {
 			man->mtex[a]= MEM_mallocN(sizeof(MTex), "copymaterial");
 			memcpy(man->mtex[a], ma->mtex[a], sizeof(MTex));
@@ -145,6 +147,8 @@ Material *copy_material(Material *ma)
 	}
 	
 	BPY_copy_scriptlink(&ma->scriptlink);
+	if(ma->ramp_col) man->ramp_col= MEM_dupallocN(ma->ramp_col);
+	if(ma->ramp_spec) man->ramp_spec= MEM_dupallocN(ma->ramp_spec);
 	
 	return man;
 }
@@ -168,7 +172,7 @@ void make_local_material(Material *ma)
 		ma->id.lib= 0;
 		ma->id.flag= LIB_LOCAL;
 		new_id(0, (ID *)ma, 0);
-		for(a=0; a<8; a++) {
+		for(a=0; a<MAX_MTEX; a++) {
 			if(ma->mtex[a]) id_lib_extern((ID *)ma->mtex[a]->tex);
 		}
 		
@@ -232,7 +236,7 @@ void make_local_material(Material *ma)
 		ma->id.lib= 0;
 		ma->id.flag= LIB_LOCAL;
 		
-		for(a=0; a<8; a++) {
+		for(a=0; a<MAX_MTEX; a++) {
 			if(ma->mtex[a]) id_lib_extern((ID *)ma->mtex[a]->tex);
 		}
 		
@@ -534,18 +538,12 @@ void init_render_material(Material *ma)
 	MTex *mtex;
 	int a, needuv=0;
 	
-	if(ma->ren) return;
-
 	if(ma->flarec==0) ma->flarec= 1;
 
-	ma->ren= MEM_mallocN(sizeof(Material), "initrendermaterial");
-	memcpy(ma->ren, ma, sizeof(Material));
-	
 	/* add all texcoflags from mtex */
-	ma= ma->ren;
 	ma->texco= 0;
 	ma->mapto= 0;
-	for(a=0; a<8; a++) {
+	for(a=0; a<MAX_MTEX; a++) {
 		mtex= ma->mtex[a];
 		if(mtex && mtex->tex) {
 			
@@ -563,7 +561,7 @@ void init_render_material(Material *ma)
 	}
 	if(ma->mode & MA_ZTRA) {
 		/* if(ma->alpha==0.0 || ma->alpha==1.0) */
-		R.flag |= R_ZTRA;
+		if(R.flag & R_RENDERING) R.flag |= R_ZTRA;
 	}
 	if(ma->mode & MA_VERTEXCOLP) ma->mode |= MA_VERTEXCOL; 
 	
@@ -574,9 +572,9 @@ void init_render_material(Material *ma)
 		if(R.osa) ma->texco |= TEXCO_OSA;		/* for texfaces */
 	}
 	if(needuv) ma->texco |= NEED_UV;
-
+	
 	// since the raytracer doesnt recalc O structs for each ray, we have to preset them all
-	if(ma->mode & (MA_RAYMIRROR|MA_RAYTRANSP)) { 
+	if(ma->mode & (MA_RAYMIRROR|MA_RAYTRANSP|MA_SHADOW_TRA)) { 
 		ma->texco |= NEED_UV|TEXCO_ORCO|TEXCO_REFL|TEXCO_NORM;
 		if(R.osa) ma->texco |= TEXCO_OSA;
 	}
@@ -601,10 +599,7 @@ void init_render_materials()
 
 void end_render_material(Material *ma)
 {
-	
-	if(ma->ren) MEM_freeN(ma->ren);
-	ma->ren= 0;
-
+	/* XXXX obsolete? check! */
 	if(ma->mode & (MA_VERTEXCOLP|MA_FACETEXTURE)) {
 		if( !(ma->mode & MA_HALO) ) {
 			ma->r= ma->g= ma->b= 1.0;
@@ -736,8 +731,6 @@ void delete_material_index()
 		}
 		obt= obt->id.next;
 	}
-	allqueue(REDRAWBUTSMAT, 0);
-	
 
 	/* check indices from mesh */
 
