@@ -1,7 +1,7 @@
 /* object.c
  *
  * 
- * $Id: object.c,v 1.16 2004/04/04 17:23:51 theeth Exp $
+ * $Id: object.c,v 1.43 2005/05/20 20:44:33 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,9 +40,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
 #include "MEM_guardedalloc.h"
 
 #include "DNA_action_types.h"
@@ -59,6 +56,8 @@
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
+#include "DNA_object_force.h"
+#include "DNA_oops_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_space_types.h"
@@ -100,6 +99,7 @@
 #include "BKE_lattice.h"
 #include "BKE_constraint.h"
 #include "BKE_scene.h"
+#include "BKE_softbody.h"
 
 #include "BPY_extern.h"
 
@@ -147,6 +147,29 @@ void update_base_layer(Object *ob)
 	}
 }
 
+static void free_hooks(ListBase *lb)
+{
+	while(lb->first) {
+		ObHook *hook= lb->first;
+		if(hook->indexar) MEM_freeN(hook->indexar);
+		BLI_remlink(lb, hook);
+		MEM_freeN(hook);
+	}
+}
+
+static void copy_hooks(ListBase *new, ListBase *old)
+{
+	ObHook *hook, *hookn;
+	new->first= new->last= NULL;
+	
+	for(hook= old->first; hook; hook= hook->next) {
+		hookn= MEM_dupallocN(hook);
+		hookn->indexar= MEM_dupallocN(hookn->indexar);
+		BLI_addtail(new, hookn);
+	}
+
+}
+
 /* do not free object itself */
 void free_object(Object *ob)
 {
@@ -192,10 +215,15 @@ void free_object(Object *ob)
 	free_constraints(&ob->constraints);
 	free_constraint_channels(&ob->constraintChannels);
 	free_nlastrips(&ob->nlastrips);
-
+	
+	free_hooks(&ob->hooks);
+	
 	freedisplist(&ob->disp);
 	
 	BPY_free_scriptlink(&ob->scriptlink);
+	
+	if(ob->pd) MEM_freeN(ob->pd);
+	if(ob->soft) sbFree(ob->soft);
 }
 
 void unlink_object(Object *ob)
@@ -207,6 +235,7 @@ void unlink_object(Object *ob)
 	Scene *sce;
 	Curve *cu;
 	Tex *tex;
+	ObHook *hook;
 	Group *group;
 	int a;
 
@@ -216,28 +245,32 @@ void unlink_object(Object *ob)
 	/* check all objects: parents en bevels */
 	obt= G.main->object.first;
 	while(obt) {
-		if(obt->id.lib==0) {
+		if(obt->id.lib==NULL) {
 			if(obt->parent==ob) {
-				obt->parent= 0;
+				obt->parent= NULL;
 				if(ob->type==OB_LATTICE) freedisplist(&obt->disp);
 			}
-			if(obt->track==ob) obt->track= 0;
+			if(obt->track==ob) obt->track= NULL;
+			for(hook=obt->hooks.first; hook; hook= hook->next) {
+				if(hook->parent==ob) hook->parent= NULL;
+			}
 			if ELEM(obt->type, OB_CURVE, OB_FONT) {
 				cu= obt->data;
-				if(cu->bevobj==ob) cu->bevobj= 0;
-				if(cu->textoncurve==ob) cu->textoncurve= 0;
+				if(cu->bevobj==ob) cu->bevobj= NULL;
+				if(cu->taperobj==ob) cu->taperobj= NULL;
+				if(cu->textoncurve==ob) cu->textoncurve= NULL;
 			}
 			if(obt->type==OB_IKA) {
 				Ika *ika= obt->data;
 				Deform *def= ika->def;
 				
-				if(ika->parent==ob) ika->parent= 0;
+				if(ika->parent==ob) ika->parent= NULL;
 				a= ika->totdef;
 				while(a--) {
 					if(def->ob==ob) {
 						ika->totdef= 0;
 						MEM_freeN(ika->def);
-						ika->def= 0;
+						ika->def= NULL;
 						break;
 					}
 					def++;
@@ -252,7 +285,7 @@ void unlink_object(Object *ob)
 	mat= G.main->mat.first;
 	while(mat) {
 	
-		for(a=0; a<8; a++) {
+		for(a=0; a<MAX_MTEX; a++) {
 			if(mat->mtex[a] && ob==mat->mtex[a]->object) {
 				/* actually, test for lib here... to do */
 				mat->mtex[a]->object= 0;
@@ -281,7 +314,7 @@ void unlink_object(Object *ob)
 	wrld= G.main->world.first;
 	while(wrld) {
 		if(wrld->id.lib==0) {
-			for(a=0; a<6; a++) {
+			for(a=0; a<MAX_MTEX; a++) {
 				if(wrld->mtex[a] && ob==wrld->mtex[a]->object)
 					wrld->mtex[a]->object =0;
 			}
@@ -320,6 +353,28 @@ void unlink_object(Object *ob)
 						if(v3d->localvd->persp>1) v3d->localvd->persp= 1;
 					}
 				}
+				else if(sl->spacetype==SPACE_IPO) {
+					SpaceIpo *sipo= (SpaceIpo *)sl;
+					if(sipo->from == (ID *)ob) sipo->from= NULL;
+				}
+				else if(sl->spacetype==SPACE_OOPS) {
+					SpaceOops *so= (SpaceOops *)sl;
+					Oops *oops;
+
+					oops= so->oops.first;
+					while(oops) {
+						if(oops->id==(ID *)ob) oops->id= NULL;
+						oops= oops->next;
+					}
+					if(so->treestore) {
+						TreeStoreElem *tselem= so->treestore->data;
+						int a;
+						for(a=0; a<so->treestore->usedelem; a++, tselem++) {
+							if(tselem->id==(ID *)ob) tselem->id= NULL;
+						}
+					}
+					so->lockpoin= NULL;
+				}
 			}
 
 			sa= sa->next;
@@ -357,6 +412,7 @@ void *add_camera()
 	cam->clipsta= 0.1f;
 	cam->clipend= 100.0f;
 	cam->drawsize= 0.5f;
+	cam->ortho_scale= 6.0;
 	
 	return cam;
 }
@@ -462,7 +518,7 @@ Lamp *copy_lamp(Lamp *la)
 	
 	lan= copy_libblock(la);
 
-	for(a=0; a<8; a++) {
+	for(a=0; a<MAX_MTEX; a++) {
 		if(lan->mtex[a]) {
 			lan->mtex[a]= MEM_mallocN(sizeof(MTex), "copylamptex");
 			memcpy(lan->mtex[a], la->mtex[a], sizeof(MTex));
@@ -543,7 +599,7 @@ void free_lamp(Lamp *la)
 		
 	BPY_free_scriptlink(&la->scriptlink);
 	
-	for(a=0; a<8; a++) {
+	for(a=0; a<MAX_MTEX; a++) {
 		mtex= la->mtex[a];
 		if(mtex && mtex->tex) mtex->tex->id.us--;
 		if(mtex) MEM_freeN(mtex);
@@ -645,7 +701,7 @@ Object *add_object(int type)
 		ob->upflag= OB_POSZ;
 	}
 	ob->ipoflag = OB_OFFS_OB+OB_OFFS_PARENT;
-	
+	ob->ipowin= ID_OB;	/* the ipowin shown */
 	ob->dupon= 1; ob->dupoff= 0;
 	ob->dupsta= 1; ob->dupend= 100;
 
@@ -690,6 +746,24 @@ void base_init_from_view3d(Base *base, View3D *v3d)
 	v3d->viewquat[0]= -v3d->viewquat[0];
 }
 
+SoftBody *copy_softbody(SoftBody *sb)
+{
+	SoftBody *sbn;
+	
+	if (sb==NULL) return(NULL);
+	
+	sbn= MEM_dupallocN(sb);
+	sbn->totspring= sbn->totpoint= 0;
+	sbn->bpoint= NULL;
+	sbn->bspring= NULL;
+	sbn->ctime= 0.0f;
+	
+	sbn->keys= NULL;
+	sbn->totkey= sbn->totpointkey= 0;
+	
+	return sbn;
+}
+
 Object *copy_object(Object *ob)
 {
 	Object *obn;
@@ -722,18 +796,24 @@ Object *copy_object(Object *ob)
 	copy_nlastrips(&obn->nlastrips, &ob->nlastrips);
 	copy_constraints (&obn->constraints, &ob->constraints);
 
+	copy_hooks( &obn->hooks, &ob->hooks);
+	
 	actcon = clone_constraint_channels (&obn->constraintChannels, &ob->constraintChannels, ob->activecon);
 	/* If the active constraint channel was in this list, update it */
 	if (actcon)
 		obn->activecon = actcon;
 
+
 	/* increase user numbers */
 	id_us_plus((ID *)obn->data);
 	id_us_plus((ID *)obn->ipo);
 	id_us_plus((ID *)obn->action);
 	for(a=0; a<obn->totcol; a++) id_us_plus((ID *)obn->mat[a]);
 	
-	obn->disp.first= obn->disp.last= 0;
+	obn->disp.first= obn->disp.last= NULL;
+	
+	if(ob->pd) obn->pd= MEM_dupallocN(ob->pd);
+	obn->soft= copy_softbody(ob->soft);
 	
 	return obn;
 }
@@ -833,22 +913,27 @@ void disable_speed_curve(int val)
 	no_speed_curve= val;
 }
 
+/* ob can be NULL */
 float bsystem_time(Object *ob, Object *par, float cfra, float ofs)
 {
 	/* returns float ( see frame_to_float in ipo.c) */
 
-	if(no_speed_curve==0) if(ob && ob->ipo) cfra= calc_ipo_time(ob->ipo, cfra);
-	
 	/* 2nd field */
 	if(R.flag & R_SEC_FIELD) {
 		if(R.r.mode & R_FIELDSTILL); else cfra+= .5;
 	}
 
-	/* motion blur */
-	cfra+= bluroffs;
 
-	/* global time */
-	cfra*= G.scene->r.framelen;	
+	if(ob && (ob->flag & OB_FROMDUPLI));
+	else {
+			/* motion blur */
+		cfra+= bluroffs;
+	
+		/* global time */
+		cfra*= G.scene->r.framelen;	
+	}
+	
+	if(no_speed_curve==0) if(ob && ob->ipo) cfra= calc_ipo_time(ob->ipo, cfra);
 	
 	/* ofset frames */
 	if(ob && (ob->ipoflag & OB_OFFS_PARENT)) {
@@ -923,12 +1008,18 @@ void ob_parcurve(Object *ob, Object *par, float mat[][4])
 {
 	Curve *cu;
 	float q[4], vec[4], dir[3], *quat, x1, ctime;
+	float timeoffs= 0.0;
 	
 	Mat4One(mat);
 	
 	cu= par->data;
-	if(cu->path==0 || cu->path->data==0) calc_curvepath(par);
-	if(cu->path==0) return;
+	if(cu->path==NULL || cu->path->data==NULL) calc_curvepath(par);
+	if(cu->path==NULL) return;
+	
+	/* exception, timeoffset is regarded as distance offset */
+	if(cu->flag & CU_OFFS_PATHDIST) {
+		SWAP(float, timeoffs, ob->sf);
+	}
 	
 	/* catch exceptions: curve paths used as a duplicator */
 	if(enable_cu_speed) {
@@ -945,12 +1036,23 @@ void ob_parcurve(Object *ob, Object *par, float mat[][4])
 		
 		CLAMP(ctime, 0.0, 1.0);
 	}
+	
+	/* time calculus is correct, now apply distance offset */
+	if(cu->flag & CU_OFFS_PATHDIST) {
+		ctime += timeoffs/cu->path->totdist;
 
+		/* restore */
+		SWAP(float, timeoffs, ob->sf);
+	}
+	
+	
+	/* vec: 4 items! */
  	if( where_on_path(par, ctime, vec, dir) ) {
 
 		if(cu->flag & CU_FOLLOW) {
 			quat= vectoquat(dir, ob->trackflag, ob->upflag);
-
+			
+			/* the tilt */
 			Normalise(dir);
 			q[0]= (float)cos(0.5*vec[3]);
 			x1= (float)sin(0.5*vec[3]);
@@ -1167,24 +1269,34 @@ void set_dtime(int dtime)
 static int during_script_flag=0;
 void disable_where_script(short on)
 {
-	during_script_flag= on;	
+	during_script_flag= on;
 }
 
 int during_script(void) {
 	return during_script_flag;
 }
 
+static int during_scriptlink_flag=0;
+void disable_where_scriptlink(short on)
+{
+	during_scriptlink_flag= on;
+}
+
+int during_scriptlink(void) {
+	return during_scriptlink_flag;
+}
+
 void where_is_object_time(Object *ob, float ctime)
 {
 	Object *par;
 	float *fp1, *fp2, slowmat[4][4] = MAT4_UNITY;
-	float stime, fac1, fac2;
+	float stime, fac1, fac2, vec[3];
 	int a;
 	int pop; 
 	
 	/* new version: correct parent+vertexparent and track+parent */
 	/* this one only calculates direct attached parent and track */
-	/* hij is sneller, maar moet wel de timeoffs in de gaten houden */
+	/* is faster, but should keep track of timeoffs */
 	
 	if(ob==0) return;
 
@@ -1214,7 +1326,7 @@ void where_is_object_time(Object *ob, float ctime)
 		pop= 0;
 		if(no_parent_ipo==0 && ctime != par->ctime) {
 		
-			// alleen voor ipo systemen? 
+			// only for ipo systems? 
 			pushdata(par, sizeof(Object));
 			pop= 1;
 			
@@ -1228,7 +1340,7 @@ void where_is_object_time(Object *ob, float ctime)
 		}
 		
 		if(ob->partype & PARSLOW) {
-			// framerate meetellen 
+			// include framerate
 
 			fac1= (float)(timefac/(1.0+ fabs(ob->sf)));
 			if(fac1>=1.0) return;
@@ -1256,8 +1368,13 @@ void where_is_object_time(Object *ob, float ctime)
 	solve_constraints (ob, TARGET_OBJECT, NULL, ctime);
 
 	if(ob->scriptlink.totscript && !during_script()) {
-		BPY_do_pyscript((ID *)ob, SCRIPT_REDRAW);
+		if (G.f & G_DOSCRIPTLINKS) BPY_do_pyscript((ID *)ob, SCRIPT_REDRAW);
 	}
+	
+	/* set negative scale flag in object */
+	Crossf(vec, ob->obmat[0], ob->obmat[1]);
+	if( Inpf(vec, ob->obmat[2]) < 0.0 ) ob->transflag |= OB_NEG_SCALE;
+	else ob->transflag &= ~OB_NEG_SCALE;
 }
 
 static void solve_parenting (Object *ob, Object *par, float slowmat[][4], int simul)
@@ -1328,7 +1445,7 @@ static void solve_parenting (Object *ob, Object *par, float slowmat[][4], int si
 #endif
 	}
 	
-	// totaal 
+	// total 
 	Mat4MulSerie(tmat, totmat, ob->parentinv,         
 		NULL, NULL, NULL, NULL, NULL, NULL);
 	Mat4MulSerie(ob->obmat, tmat, obmat,         
@@ -1338,11 +1455,10 @@ static void solve_parenting (Object *ob, Object *par, float slowmat[][4], int si
 
 	}
 	else{
-		// >>>>>>>>>>>>>>>>>>
-		// dit is een extern bruikbare originmat 
+		// external usable originmat 
 		Mat3CpyMat4(originmat, tmat);
 		
-		// origin, voor hulplijntje 
+		// origin, voor help line
 		if( (ob->partype & 15)==PARSKEL ) {
 			VECCOPY(ob->orig, par->obmat[3]);
 		}
@@ -1472,7 +1588,7 @@ void solve_constraints (Object *ob, short obtype, void *obdata, float ctime)
 				enf = con->enforce;
 
 			/* Get the targetmat */
-			get_constraint_target(con, obtype, obdata, tmat, size, ctime);
+			get_constraint_target_matrix(con, obtype, obdata, tmat, size, ctime);
 			
 			Mat4CpyMat4(focusmat, tmat);
 			
