@@ -1,7 +1,7 @@
 /* text.c
  *
  *
- * $Id: text.c,v 1.10 2004/10/14 23:37:04 stiv Exp $
+ * $Id: text.c,v 1.16 2005/05/19 03:15:49 stiv Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -73,7 +73,8 @@ A text should relate to a file as follows -
 	TXT_ISEXT - should always be set if the Text is not to be written into
 					the .blend
 	TXT_ISSCRIPT - should be set if the user has designated the text
-					as a script.
+					as a script. (NEW: this was unused, but now it is needed by
+					space handler script links (see header_view3d.c, for example)
 
 ->>> see also: /makesdna/DNA_text_types.h
 
@@ -139,9 +140,12 @@ int txt_get_undostate(void)
 void free_text(Text *text)
 {
 	TextLine *tmp;
-	
-	for (tmp= text->lines.first; tmp; tmp= tmp->next)
+
+	for (tmp= text->lines.first; tmp; tmp= tmp->next) {
 		MEM_freeN(tmp->line);
+		if (tmp->format)
+		  MEM_freeN(tmp->format);
+	}
 	
 	BLI_freelistN(&text->lines);
 
@@ -171,7 +175,8 @@ Text *add_empty_text(void)
 
 	tmp= (TextLine*) MEM_mallocN(sizeof(TextLine), "textline");
 	tmp->line= (char*) MEM_mallocN(1, "textline_string");
-
+	tmp->format= (char*) MEM_mallocN(2, "Syntax_format");
+	
 	tmp->line[0]=0;
 	tmp->len= 0;
 				
@@ -222,9 +227,11 @@ int reopen_text(Text *text)
 
 	/* free memory: */
 
-	for (tmp= text->lines.first; tmp; tmp= tmp->next)
+	for (tmp= text->lines.first; tmp; tmp= tmp->next) {
 		MEM_freeN(tmp->line);
-
+		if (tmp->format) MEM_freeN(tmp->format);
+	}
+	
 	BLI_freelistN(&text->lines);
 
 	text->lines.first= text->lines.last= NULL;
@@ -258,7 +265,8 @@ int reopen_text(Text *text)
 		if (buffer[i]=='\n') {
 			tmp= (TextLine*) MEM_mallocN(sizeof(TextLine), "textline");
 			tmp->line= (char*) MEM_mallocN(llen+1, "textline_string");
-				
+			tmp->format= (char*) MEM_mallocN(llen+2, "Syntax_format");
+			
 			if(llen) memcpy(tmp->line, &buffer[i-llen], llen);
 			tmp->line[llen]=0;
 			tmp->len= llen;
@@ -277,6 +285,7 @@ int reopen_text(Text *text)
 	if (llen!=0 || text->nlines==0) {
 		tmp= (TextLine*) MEM_mallocN(sizeof(TextLine), "textline");
 		tmp->line= (char*) MEM_mallocN(llen+1, "textline_string");
+		tmp->format= (char*) MEM_mallocN(llen+2, "Syntax_format");
 		
 		if(llen) memcpy(tmp->line, &buffer[i-llen], llen);
 
@@ -345,7 +354,8 @@ Text *add_text(char *file)
 		if (buffer[i]=='\n') {
 			tmp= (TextLine*) MEM_mallocN(sizeof(TextLine), "textline");
 			tmp->line= (char*) MEM_mallocN(llen+1, "textline_string");
-				
+			tmp->format= (char*) MEM_mallocN(llen+2, "Syntax_format");
+			
 			if(llen) memcpy(tmp->line, &buffer[i-llen], llen);
 			tmp->line[llen]=0;
 			tmp->len= llen;
@@ -364,6 +374,7 @@ Text *add_text(char *file)
 	if (llen!=0 || ta->nlines==0) {
 		tmp= (TextLine*) MEM_mallocN(sizeof(TextLine), "textline");
 		tmp->line= (char*) MEM_mallocN(llen+1, "textline_string");
+		tmp->format= (char*) MEM_mallocN(llen+2, "Syntax_format");
 		
 		if(llen) memcpy(tmp->line, &buffer[i-llen], llen);
 
@@ -406,8 +417,10 @@ Text *copy_text(Text *ta)
 	while (line) {
 		tmp= (TextLine*) MEM_mallocN(sizeof(TextLine), "textline");
 		tmp->line= MEM_mallocN(line->len+1, "textline_string");
-		strcpy(tmp->line, line->line);
+		tmp->format= MEM_mallocN(line->len+2, "Syntax_format");
 		
+		strcpy(tmp->line, line->line);
+
 		tmp->len= line->len;
 		
 		BLI_addtail(&tan->lines, tmp);
@@ -425,12 +438,14 @@ Text *copy_text(Text *ta)
 /* Editing utility functions */
 /*****************************/
 
-static void make_new_line (TextLine *line, char *newline) 
+static void make_new_line (TextLine *line, char *newline, char *newformat) 
 {
-	MEM_freeN(line->line);
+	if (line->line) MEM_freeN(line->line);
+	if (line->format) MEM_freeN(line->format);
 	
 	line->line= newline;
-	line->len= strlen(newline);	
+	line->len= strlen(newline);
+	line->format= newformat;
 }
 
 static TextLine *txt_new_line(char *str)
@@ -441,6 +456,8 @@ static TextLine *txt_new_line(char *str)
 	
 	tmp= (TextLine *) MEM_mallocN(sizeof(TextLine), "textline");
 	tmp->line= MEM_mallocN(strlen(str)+1, "textline_string");
+	tmp->format= MEM_mallocN(strlen(str)+2, "Syntax_format");
+	
 	strcpy(tmp->line, str);
 	
 	tmp->len= strlen(str);
@@ -457,6 +474,8 @@ static TextLine *txt_new_linen(char *str, int n)
 	
 	tmp= (TextLine *) MEM_mallocN(sizeof(TextLine), "textline");
 	tmp->line= MEM_mallocN(n+1, "textline_string");
+	tmp->format= MEM_mallocN(n+2, "Syntax_format");
+	
 	BLI_strncpy(tmp->line, str, n+1);
 	
 	tmp->len= strlen(tmp->line);
@@ -851,7 +870,7 @@ int txt_has_sel(Text *text)
 static void txt_delete_sel (Text *text)
 {
 	TextLine *tmpl;
-	char *buf;
+	char *buf, *format;
 	
 	if (!text) return;
 	if (!text->curl) return;
@@ -868,12 +887,13 @@ static void txt_delete_sel (Text *text)
 	}
 
 	buf= MEM_mallocN(text->curc+(text->sell->len - text->selc)+1, "textline_string");
+	format= MEM_mallocN(text->curc+(text->sell->len - text->selc)+2, "Syntax_format");
 	
 	strncpy(buf, text->curl->line, text->curc);
 	strcpy(buf+text->curc, text->sell->line + text->selc);
 	buf[text->curc+(text->sell->len - text->selc)]=0;
-	
-	make_new_line(text->curl, buf);
+
+	make_new_line(text->curl, buf, format);
 	
 	tmpl= text->sell;
 	while (tmpl != text->curl) {
@@ -1806,7 +1826,7 @@ void txt_do_redo(Text *text)
 
 void txt_split_curline (Text *text) {
 	TextLine *ins;
-	char *left, *right;
+	char *left, *right, *fleft, *fright;
 	
 	if (!text) return;
 	if (!text->curl) return;
@@ -1816,22 +1836,27 @@ void txt_split_curline (Text *text) {
 	/* Make the two half strings */
 
 	left= MEM_mallocN(text->curc+1, "textline_string");
+	fleft= MEM_mallocN(text->curc+2, "Syntax_format");
 	if (text->curc) memcpy(left, text->curl->line, text->curc);
 	left[text->curc]=0;
 	
 	right= MEM_mallocN(text->curl->len - text->curc+1, "textline_string");
+	fright= MEM_mallocN(text->curl->len - text->curc+2, "Syntax_format");
 	if (text->curl->len - text->curc) memcpy(right, text->curl->line+text->curc, text->curl->len-text->curc);
 	right[text->curl->len - text->curc]=0;
 
 	MEM_freeN(text->curl->line);
+	if (text->curl->format) MEM_freeN(text->curl->format);
 
 	/* Make the new TextLine */
 	
 	ins= MEM_mallocN(sizeof(TextLine), "textline");
 	ins->line= left;
+	ins->format= fleft;
 	ins->len= text->curc;
 	
 	text->curl->line= right;
+	text->curl->format= fright;
 	text->curl->len= text->curl->len - text->curc;
 	
 	BLI_insertlinkbefore(&text->lines, text->curl, ins);	
@@ -1853,6 +1878,8 @@ static void txt_delete_line (Text *text, TextLine *line)
 	BLI_remlink (&text->lines, line);
 	
 	if (line->line) MEM_freeN(line->line);
+	if (line->format) MEM_freeN(line->format);
+
 	MEM_freeN(line);
 
 	txt_make_dirty(text);
@@ -1861,18 +1888,19 @@ static void txt_delete_line (Text *text, TextLine *line)
 
 static void txt_combine_lines (Text *text, TextLine *linea, TextLine *lineb)
 {
-	char *tmp;
+	char *tmp, *format;
 	
 	if (!text) return;
 	
 	if(!linea || !lineb) return;
 	
 	tmp= MEM_mallocN(linea->len+lineb->len+1, "textline_string");
+	format= MEM_mallocN(linea->len+lineb->len+1, "Syntax_format");
 	
 	strcpy(tmp, linea->line);
 	strcat(tmp, lineb->line);
-	
-	make_new_line(linea, tmp);
+
+	make_new_line(linea, tmp, format);
 	
 	txt_delete_line(text, lineb); 
 	
@@ -1954,7 +1982,7 @@ void txt_backspace_char (Text *text) {
 
 int txt_add_char (Text *text, char add) {
 	int len;
-	char *tmp;
+	char *tmp, *format;
 	
 	if (!text) return 0;
 	if (!text->curl) return 0;
@@ -1969,6 +1997,7 @@ int txt_add_char (Text *text, char add) {
 	txt_delete_sel(text);
 	
 	tmp= MEM_mallocN(text->curl->len+2, "textline_string");
+	format= MEM_mallocN(text->curl->len+4, "Syntax_format");
 	
 	if(text->curc) memcpy(tmp, text->curl->line, text->curc);
 	tmp[text->curc]= add;
@@ -1976,8 +2005,7 @@ int txt_add_char (Text *text, char add) {
 	len= text->curl->len - text->curc;
 	if(len>0) memcpy(tmp+text->curc+1, text->curl->line+text->curc, len);
 	tmp[text->curl->len+1]=0;
-	
-	make_new_line(text->curl, tmp);
+	make_new_line(text->curl, tmp, format);
 		
 	text->curc++;
 
@@ -1990,157 +2018,228 @@ int txt_add_char (Text *text, char add) {
 	return 1;
 }
 
-//Antihc3(rick) used the paste function below 
-//used txt_cut_sel, txt_insert_buf modified
-
-void indent_paste(Text *text)
+void indent(Text *text)
 {
-
-	indent(text, txt_cut_buffer);
-}
-
-void indent(Text *text, char *in_buffer)
-{
-	int i=0, len;
-
+	int len, num;
+	char *tmp, *format;
+	char add = '\t';
+	
 	if (!text) return;
 	if (!text->curl) return;
 	if (!text->sell) return;
-	if (!in_buffer) return;
+	num = 0;
+	while (TRUE)
+	{
+		tmp= MEM_mallocN(text->curl->len+2, "textline_string");
+		format= MEM_mallocN(text->curl->len+3, "Syntax_format");
+		
+		text->curc = 0; 
+		if(text->curc) memcpy(tmp, text->curl->line, text->curc);
+		tmp[text->curc]= add;
+		
+		len= text->curl->len - text->curc;
+		if(len>0) memcpy(tmp+text->curc+1, text->curl->line+text->curc, len);
+		tmp[text->curl->len+1]=0;
 
-	txt_delete_sel(text); //need to change this to remove the undo 	
-
-	/* Read the first line (or as close as possible */
-	len= strlen(in_buffer);
-	while ( i < len ) {
-		txt_add_char(text, '\t');
-		while (in_buffer[i] && in_buffer[i]!='\n') {
-			txt_add_char(text, in_buffer[i]);
-			i++;
-		}
-	
-		if (in_buffer[i]=='\n') {
-			txt_add_char(text, '\n');
+		make_new_line(text->curl, tmp, format);
 			
+		text->curc++;
+		
+		txt_make_dirty(text);
+		txt_clean_text(text);
+		
+		if(text->curl == text->sell) 
+		{
+			text->selc = text->sell->len;
+			break;
+		} else {
+			text->curl = text->curl->next;
+			num++;
 		}
-	i++;
+	}
+	text->curc = 0;
+	while( num > 0 )
+	{
+		text->curl = text->curl->prev;
+		num--;
 	}
 }
 
 void unindent(Text *text)
 {
-	unindent_lines(text, txt_cut_buffer);
-}
-
-void unindent_lines(Text *text, char *in_buffer)
-{
-	int i=0, len;
-
+	int num = 0;
+	
 	if (!text) return;
 	if (!text->curl) return;
 	if (!text->sell) return;
-	if (!in_buffer) return;
-
-	txt_delete_sel(text);	
-
-	/* Read the first line (or as close as possible */
-	len = strlen(in_buffer);
-	while ( i < len ) {
-		if (in_buffer[i] != '\t') {
-			while (in_buffer[i] && in_buffer[i]!='\n') {
-				txt_add_char(text, in_buffer[i]);
+	
+	while(TRUE)
+	{
+		int i = 0;
+		
+		if (text->curl->line[i] == '\t')
+		{
+			while(i< text->curl->len) {
+				text->curl->line[i]= text->curl->line[i+1];
 				i++;
 			}
-			
-			if (in_buffer[i]=='\n') {
-				txt_add_char(text, '\n');
-			
-			}
-		i++;
+			text->curl->len--;
 		}
-		else {
-			i++;
-			while (in_buffer[i] && in_buffer[i]!='\n') {
-				txt_add_char(text, in_buffer[i]);
-				i++;
-			}
-			
-			if (in_buffer[i]=='\n') {
-				txt_add_char(text, '\n');
-			
-			}
-		i++;
+			 
+	
+		txt_make_dirty(text);
+		txt_clean_text(text);
+		
+		if(text->curl == text->sell) 
+		{
+			text->selc = text->sell->len;
+			break;
+		} else {
+			text->curl = text->curl->next;
+			num++;
 		}
+		
+	}
+	
+	while( num > 0 )
+	{
+		text->curl = text->curl->prev;
+		num--;
 	}
 }
 
 void comment(Text *text)
 {
-	comment_paste(text, txt_cut_buffer);
-}
-
-void comment_paste(Text *text, char *in_buffer)
-{
-	int i=0, len;
-
+	int len, num;
+	char *tmp, *format;
+	char add = '#';
+	
 	if (!text) return;
 	if (!text->curl) return;
 	if (!text->sell) return;
-	if (!in_buffer) return;
-
-	txt_delete_sel(text); 	
-
-	/* Read the first line (or as close as possible */
-	
-	len= strlen(in_buffer);
-		while ( i < len ) {
-			txt_add_char(text, '#');
-			while (in_buffer[i] && in_buffer[i]!='\n') {
-				txt_add_char(text, in_buffer[i]);
-				i++;
-			}
+	num = 0;
+	while (TRUE)
+	{
+		tmp= MEM_mallocN(text->curl->len+2, "textline_string");
+		format = MEM_mallocN(text->curl->len+3, "Syntax_format");
 		
-			if (in_buffer[i]=='\n') {
-				txt_add_char(text, '\n');
-				
-			}
-		i++;
-		}
-}
+		text->curc = 0; 
+		if(text->curc) memcpy(tmp, text->curl->line, text->curc);
+		tmp[text->curc]= add;
+		
+		len= text->curl->len - text->curc;
+		if(len>0) memcpy(tmp+text->curc+1, text->curl->line+text->curc, len);
+		tmp[text->curl->len+1]=0;
 
+		make_new_line(text->curl, tmp, format);
+			
+		text->curc++;
+		
+		txt_make_dirty(text);
+		txt_clean_text(text);
+		
+		if(text->curl == text->sell) 
+		{
+			text->selc = text->sell->len;
+			break;
+		} else {
+			text->curl = text->curl->next;
+			num++;
+		}
+	}
+	
+	while( num > 0 )
+	{
+		text->curl = text->curl->prev;
+		num--;
+	}
+}
 
 void uncomment(Text *text)
 {
-	uncomment_paste(text, txt_cut_buffer);
-}
-
-void uncomment_paste(Text *text, char *in_buffer)
-{
+	int num = 0;
 	
-	int i=0, len;
-
 	if (!text) return;
 	if (!text->curl) return;
 	if (!text->sell) return;
-
-	if (!in_buffer) return;
-
-	txt_delete_sel(text);
-
-	/* Read the first line (or as close as possible */
-	len = strlen(in_buffer);
-	while ( i < len ) {
-		if (in_buffer[i] != '#') {
-			while (in_buffer[i] && in_buffer[i]!='\n') {
-				txt_add_char(text, in_buffer[i]);
+	
+	while(TRUE)
+	{
+		int i = 0;
+		
+		if (text->curl->line[i] == '#')
+		{
+			while(i< text->curl->len) {
+				text->curl->line[i]= text->curl->line[i+1];
 				i++;
 			}
-			
-			if (in_buffer[i]=='\n') {
-				txt_add_char(text, '\n');
-			
-			}		
+			text->curl->len--;
 		}
-	i++;
+			 
+	
+		txt_make_dirty(text);
+		txt_clean_text(text);
+		
+		if(text->curl == text->sell) 
+		{
+			text->selc = text->sell->len;
+			break;
+		} else {
+			text->curl = text->curl->next;
+			num++;
+		}
+		
+	}
+	
+	while( num > 0 )
+	{
+		text->curl = text->curl->prev;
+		num--;
 	}
 }
+
+int setcurr_tab (Text *text)
+{
+	int i = 0;
+	int test = 0;
+	char *word = ":";
+	char *comm = "#";
+	char back_words[3][7] = {"return", "break", "pass"};
+	if (!text) return 0;
+	if (!text->curl) return 0;
+	
+	while (text->curl->line[i] == '\t')
+	{
+		//we only count thos tabs that are before any text or before the curs;
+		if (i == text->curc)
+		{
+			return i;
+		} else {
+			i++;
+		}
+	}
+	if(strstr(text->curl->line, word))
+	{
+		//if we find a : then add a tab but not if it is in a comment
+		if(strcspn(text->curl->line, word) < strcspn(text->curl->line, comm))
+		{
+			i++;
+
+		}
+	}
+
+	while(test < 3)
+	{
+		//if there are these 3 key words then remove a tab because we are done with the block
+		if(strstr(text->curl->line, back_words[test]) && i > 0)
+		{
+			if(strcspn(text->curl->line, back_words[test]) < strcspn(text->curl->line, comm))
+			{
+				i--;
+			}
+		}
+		test++;
+	}
+	return i;
+}
+
