@@ -3,7 +3,7 @@
 /**
  * blenlib/BLI_arithb.h    mar 2001 Nzc
  *
- * $Id: BLI_arithb.h,v 1.9 2004/04/24 21:09:05 theeth Exp $ 
+ * $Id: BLI_arithb.h,v 1.18 2005/04/22 20:16:01 blendix Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -44,6 +44,19 @@
 extern "C" {
 #endif
 
+#ifndef M_PI
+#define M_PI		3.14159265358979323846
+#endif
+#ifndef M_PI_2
+#define M_PI_2		1.57079632679489661923
+#endif
+#ifndef M_SQRT2
+#define M_SQRT2		1.41421356237309504880
+#endif
+#ifndef M_SQRT1_2
+#define M_SQRT1_2	0.70710678118654752440
+#endif
+
 #define MAT4_UNITY {{ 1.0, 0.0, 0.0, 0.0},\
 					{ 0.0, 1.0, 0.0, 0.0},\
 					{ 0.0, 0.0, 1.0, 0.0},\
@@ -178,6 +191,11 @@ QuatSub(
 	float *q2
 );
 
+
+void QuatInterpol(float *result, float *quat1, float *quat2, float t);
+void QuatAdd(float *result, float *quat1, float *quat2, float t);
+
+
 /**
  * @section matrix multiplication can copying routines
  */
@@ -434,6 +452,10 @@ CalcNormFloat4(
 VecLenf(
 	float *v1, 
 	float *v2
+);
+    float 
+VecLength(
+	float *v
 );
 	void 
 VecMulf(
@@ -473,6 +495,11 @@ sasqrt(
 printvecf(
 	char *str,
 	float v[3]
+);
+	void
+printvec4f(
+	char *str,
+	float v[4]
 );
 	float 
 Inpf(
@@ -540,6 +567,18 @@ VecRotToMat3(
 	float phi, 
 	float mat[][3]
 );
+
+/* intersect Line-Line
+	return:
+	-1: colliniar
+	 0: no intersection of segments
+	 1: exact intersection of segments
+	 2: cross-intersection of segments
+*/
+extern short IsectLL2Df(float *v1, float *v2, float *v3, float *v4);
+extern short IsectLL2Ds(short *v1, short *v2, short *v3, short *v4);
+
+
 	float *
 vectoquat(
 	float *vec, 
@@ -547,6 +586,13 @@ vectoquat(
 	short upflag
 );
 
+	float
+VecAngle3(
+	float *v1, 
+	float *v2,
+	float *v3
+);
+
 	void 
 i_lookat(
 	float vx, float vy, 
@@ -749,8 +795,47 @@ triatoquat(
 MinMaxRGB(
 	short c[]
 );
+	float
+Vec2Lenf(
+	float *v1,
+	float *v2
+);
+	void 
+Vec2Mulf(
+	float *v1, 
+	float f
+);
+	void 
+Vec2Addf(
+	float *v,
+	float *v1, 
+	float *v2
+);
+	void 
+Vec2Subf(
+	float *v,
+	float *v1, 
+	float *v2
+);
+	void
+Vec2Copyf(
+	float *v1,
+	float *v2
+);
+	float 
+Inp2f(
+	float *v1, 
+	float *v2
+);
+	float 
+Normalise2(
+	float *n
+);
 
-
+void tubemap(float x, float y, float z, float *u, float *v);
+void spheremap(float x, float y, float z, float *u, float *v);
+			  
+			  
 #ifdef __cplusplus
 }
 #endif
