@@ -4,7 +4,7 @@
  * Some editing types needed in the lib (unfortunately) for
  * scanfill.c
  *
- * $Id: BLI_editVert.h,v 1.7 2003/11/19 22:00:12 ton Exp $
+ * $Id: BLI_editVert.h,v 1.18 2005/04/04 04:50:41 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -38,35 +38,70 @@
 #ifndef BLI_EDITVERT_H
 #define BLI_EDITVERT_H
 
+struct DerivedMesh;
+
+/* note; changing this also might affect the undo copy in editmesh.c */
 typedef struct EditVert
 {
 	struct EditVert *next, *prev, *vn;
 	float no[3];
 	float co[3];
 	short xs, ys;
-	unsigned char f, h, f1, hash;
-	int	totweight;				/* __NLA */
+	unsigned char f, h, f1, f2;
+	short fast;	/* only 0 or 1, for editmesh_fastmalloc */
+	short	totweight;				/* __NLA */
+	int hash;
 	struct MDeformWeight *dw;	/* __NLA */
+	int keyindex; /* original index #, for restoring  key information */
 } EditVert;
 
+struct EditEdge;
+
+typedef struct HashEdge {
+	struct EditEdge *eed;
+	struct HashEdge *next;
+} HashEdge;
+
+/* note; changing this also might affect the undo copy in editmesh.c */
 typedef struct EditEdge
 {
 	struct EditEdge *next, *prev;
 	struct EditVert *v1, *v2, *vn;
-	short f,h;
-	short f1, dir;
+	short f1, f2;	/* short, f1 is (ab)used in subdiv */
+	unsigned char f, h, dir, seam;
+	float crease;
+	short fast; 		/* only 0 or 1, for editmesh_fastmalloc */
+	short fgoni;		/* index for fgon, for search */
+	HashEdge hash;
 } EditEdge;
 
-typedef struct EditVlak
+/* note; changing this also might affect the undo copy in editmesh.c */
+typedef struct EditFace
 {
-	struct EditVlak *next, *prev;
+	struct EditFace *next, *prev;
 	struct EditVert *v1, *v2, *v3, *v4;
 	struct EditEdge *e1, *e2, *e3, *e4;
-	float n[3];
+	float n[3], cent[3];
+	short xs, ys;		/* selection */
 	struct TFace tf;	/* a copy of original tface. */
 	unsigned char mat_nr, flag;
-	unsigned char f, f1;
-} EditVlak;
+	unsigned char f, f1, h, puno;
+	unsigned char fast;			/* only 0 or 1, for editmesh_fastmalloc */
+	unsigned char fgonf;		/* flag for fgon options */
+} EditFace;
+
+typedef struct EditMesh
+{
+	ListBase verts, edges, faces;
+	HashEdge *hashedgetab;
+	
+	/* this is for the editmesh_fastmalloc */
+	EditVert *allverts, *curvert;
+	EditEdge *alledges, *curedge;
+	EditFace *allfaces, *curface;
+	
+	struct DerivedMesh *derived;
+} EditMesh;
 
 #endif
 
