@@ -7,7 +7,7 @@
  * Functions here get counterparts with MTC prefixes. Basically, we phase
  * out the calls here in favour of fully prototyped versions.
  *
- * $Id: arithb.c,v 1.9 2004/04/24 21:09:05 theeth Exp $
+ * $Id: arithb.c,v 1.27 2005/04/22 20:16:02 blendix Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -93,8 +93,8 @@ float Normalise(float *n)
 	float d;
 	
 	d= n[0]*n[0]+n[1]*n[1]+n[2]*n[2];
-	/* FLT_EPSILON is too large! A larger value causes normalise errors in a scaled down utah teapot */
-	if(d>0.0000000000001) {
+	/* A larger value causes normalise errors in a scaled down models with camera xtreme close */
+	if(d>1.0e-35F) {
 		d= (float)sqrt(d);
 
 		n[0]/=d; 
@@ -957,13 +957,21 @@ void printvecf( char *str,  float v[3])
 
 }
 
+void printvec4f( char *str,  float v[4])
+{
+	printf("%s\n", str);
+	printf("%f %f %f %f\n",v[0],v[1],v[2], v[3]);
+	printf("\n");
+
+}
+
 void printmatrix4( char *str,  float m[][4])
 {
 	printf("%s\n", str);
-	printf("%f %f %f %f\n",m[0][0],m[0][1],m[0][2],m[0][3]);
-	printf("%f %f %f %f\n",m[1][0],m[1][1],m[1][2],m[1][3]);
-	printf("%f %f %f %f\n",m[2][0],m[2][1],m[2][2],m[2][3]);
-	printf("%f %f %f %f\n",m[3][0],m[3][1],m[3][2],m[3][3]);
+	printf("%f %f %f %f\n",m[0][0],m[1][0],m[2][0],m[3][0]);
+	printf("%f %f %f %f\n",m[0][1],m[1][1],m[2][1],m[3][1]);
+	printf("%f %f %f %f\n",m[0][2],m[1][2],m[2][2],m[3][2]);
+	printf("%f %f %f %f\n",m[0][3],m[1][3],m[2][3],m[3][3]);
 	printf("\n");
 
 }
@@ -971,9 +979,9 @@ void printmatrix4( char *str,  float m[][4])
 void printmatrix3( char *str,  float m[][3])
 {
 	printf("%s\n", str);
-	printf("%f %f %f\n",m[0][0],m[0][1],m[0][2]);
-	printf("%f %f %f\n",m[1][0],m[1][1],m[1][2]);
-	printf("%f %f %f\n",m[2][0],m[2][1],m[2][2]);
+	printf("%f %f %f\n",m[0][0],m[1][0],m[2][0]);
+	printf("%f %f %f\n",m[0][1],m[1][1],m[2][1]);
+	printf("%f %f %f\n",m[0][2],m[1][2],m[2][2]);
 	printf("\n");
 
 }
@@ -1386,6 +1394,93 @@ void VecUpMat3(float *vec, float mat[][3], short axis)
 	
 }
 
+/* A & M Watt, Advanced animation and rendering techniques, 1992 ACM press */
+void QuatInterpolW(float *, float *, float *, float );
+
+void QuatInterpolW(float *result, float *quat1, float *quat2, float t)
+{
+	float omega, cosom, sinom, sc1, sc2;
+
+	cosom = quat1[0]*quat2[0] + quat1[1]*quat2[1] + quat1[2]*quat2[2] + quat1[3]*quat2[3] ;
+	
+	/* rotate around shortest angle */
+	if ((1.0 + cosom) > 0.0001) {
+		
+		if ((1.0 - cosom) > 0.0001) {
+			omega = acos(cosom);
+			sinom = sin(omega);
+			sc1 = sin((1.0 - t) * omega) / sinom;
+			sc2 = sin(t * omega) / sinom;
+		} 
+		else {
+			sc1 = 1.0 - t;
+			sc2 = t;
+		}
+		result[0] = sc1*quat1[0] + sc2*quat2[0];
+		result[1] = sc1*quat1[1] + sc2*quat2[1];
+		result[2] = sc1*quat1[2] + sc2*quat2[2];
+		result[3] = sc1*quat1[3] + sc2*quat2[3];
+	} 
+	else {
+		result[0] = quat2[3];
+		result[1] = -quat2[2];
+		result[2] = quat2[1];
+		result[3] = -quat2[0];
+		
+		sc1 = sin((1.0 - t)*M_PI_2);
+		sc2 = sin(t*M_PI_2);
+
+		result[0] = sc1*quat1[0] + sc2*result[0];
+		result[1] = sc1*quat1[1] + sc2*result[1];
+		result[2] = sc1*quat1[2] + sc2*result[2];
+		result[3] = sc1*quat1[3] + sc2*result[3];
+	}
+}
+
+void QuatInterpol(float *result, float *quat1, float *quat2, float t)
+{
+	float quat[4], omega, cosom, sinom, sc1, sc2;
+
+	cosom = quat1[0]*quat2[0] + quat1[1]*quat2[1] + quat1[2]*quat2[2] + quat1[3]*quat2[3] ;
+	
+	/* rotate around shortest angle */
+	if (cosom < 0.0) {
+		cosom = -cosom;
+		quat[0]= -quat1[0];
+		quat[1]= -quat1[1];
+		quat[2]= -quat1[2];
+		quat[3]= -quat1[3];
+	} 
+	else {
+		quat[0]= quat1[0];
+		quat[1]= quat1[1];
+		quat[2]= quat1[2];
+		quat[3]= quat1[3];
+	}
+	
+	if ((1.0 - cosom) > 0.0001) {
+		omega = acos(cosom);
+		sinom = sin(omega);
+		sc1 = sin((1 - t) * omega) / sinom;
+		sc2 = sin(t * omega) / sinom;
+	} else {
+		sc1= 1.0 - t;
+		sc2= t;
+	}
+	
+	result[0] = sc1 * quat[0] + sc2 * quat2[0];
+	result[1] = sc1 * quat[1] + sc2 * quat2[1];
+	result[2] = sc1 * quat[2] + sc2 * quat2[2];
+	result[3] = sc1 * quat[3] + sc2 * quat2[3];
+}
+
+void QuatAdd(float *result, float *quat1, float *quat2, float t)
+{
+	result[0]= quat1[0] + t*quat2[0];
+	result[1]= quat1[1] + t*quat2[1];
+	result[2]= quat1[2] + t*quat2[2];
+	result[3]= quat1[3] + t*quat2[3];
+}
 
 /* **************** VIEW / PROJECTION ********************************  */
 
@@ -1622,6 +1717,11 @@ float VecLenf( float *v1, float *v2)
 	return (float)sqrt(x*x+y*y+z*z);
 }
 
+float VecLength(float *v)
+{
+	return (float) sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);
+}
+
 void VecAddf(float *v, float *v1, float *v2)
 {
 	v[0]= v1[0]+ v2[0];
@@ -1874,6 +1974,56 @@ float AreaPoly3Dfl(int nr, float *verts, float *normal)
 	return (float)fabs(0.5*area/max);
 }
 
+/* intersect Line-Line, shorts */
+short IsectLL2Ds(short *v1, short *v2, short *v3, short *v4)
+{
+	/* return:
+	-1: colliniar
+	 0: no intersection of segments
+	 1: exact intersection of segments
+	 2: cross-intersection of segments
+	*/
+	float div, labda, mu;
+	
+	div= (v2[0]-v1[0])*(v4[1]-v3[1])-(v2[1]-v1[1])*(v4[0]-v3[0]);
+	if(div==0.0) return -1;
+	
+	labda= ((float)(v1[1]-v3[1])*(v4[0]-v3[0])-(v1[0]-v3[0])*(v4[1]-v3[1]))/div;
+	
+	mu= ((float)(v1[1]-v3[1])*(v2[0]-v1[0])-(v1[0]-v3[0])*(v2[1]-v1[1]))/div;
+	
+	if(labda>=0.0 && labda<=1.0 && mu>=0.0 && mu<=1.0) {
+		if(labda==0.0 || labda==1.0 || mu==0.0 || mu==1.0) return 1;
+		return 2;
+	}
+	return 0;
+}
+
+/* intersect Line-Line, floats */
+short IsectLL2Df(float *v1, float *v2, float *v3, float *v4)
+{
+	/* return:
+	-1: colliniar
+0: no intersection of segments
+1: exact intersection of segments
+2: cross-intersection of segments
+	*/
+	float div, labda, mu;
+	
+	div= (v2[0]-v1[0])*(v4[1]-v3[1])-(v2[1]-v1[1])*(v4[0]-v3[0]);
+	if(div==0.0) return -1;
+	
+	labda= ((float)(v1[1]-v3[1])*(v4[0]-v3[0])-(v1[0]-v3[0])*(v4[1]-v3[1]))/div;
+	
+	mu= ((float)(v1[1]-v3[1])*(v2[0]-v1[0])-(v1[0]-v3[0])*(v2[1]-v1[1]))/div;
+	
+	if(labda>=0.0 && labda<=1.0 && mu>=0.0 && mu<=1.0) {
+		if(labda==0.0 || labda==1.0 || mu==0.0 || mu==1.0) return 1;
+		return 2;
+	}
+	return 0;
+}
+
 void MinMax3(float *min, float *max, float *vec)
 {
 	if(min[0]>vec[0]) min[0]= vec[0];
@@ -2065,6 +2215,20 @@ void VecRotToQuat( float *vec, float phi, float *quat)
 	}
 }
 
+/* Return the angle in degrees between vecs 1-2 and 2-3 */
+float VecAngle3( float *v1, float *v2, float *v3)
+{
+	float vec1[3], vec2[3];
+
+	VecSubf(vec1, v2, v1);
+	VecSubf(vec2, v2, v3);
+	Normalise(vec1);
+	Normalise(vec2);
+	
+	return saacos(vec1[0]*vec2[0] + vec1[1]*vec2[1] + vec1[2]*vec2[2]) * 180.0/M_PI;
+}
+
+
 void euler_rot(float *beul, float ang, char axis)
 {
 	float eul[3], mat1[3][3], mat2[3][3], totmat[3][3];
@@ -2102,7 +2266,6 @@ void Mat3ToSize( float mat[][3], float *size)
 {
 	float vec[3];
 
-
 	VecCopyf(vec, mat[0]);
 	size[0]= Normalise(vec);
 	VecCopyf(vec, mat[1]);
@@ -2181,6 +2344,62 @@ void MinMaxRGB(short c[])
 	else if(c[2]<0) c[2]=0;
 }
 
+float Vec2Lenf(float *v1, float *v2)
+{
+	float x, y;
+
+	x = v1[0]-v2[0];
+	y = v1[1]-v2[1];
+	return (float)sqrt(x*x+y*y);
+}
+
+void Vec2Mulf(float *v1, float f)
+{
+	v1[0]*= f;
+	v1[1]*= f;
+}
+
+void Vec2Addf(float *v, float *v1, float *v2)
+{
+	v[0]= v1[0]+ v2[0];
+	v[1]= v1[1]+ v2[1];
+}
+
+void Vec2Subf(float *v, float *v1, float *v2)
+{
+	v[0]= v1[0]- v2[0];
+	v[1]= v1[1]- v2[1];
+}
+
+void Vec2Copyf(float *v1, float *v2)
+{
+	v1[0]= v2[0];
+	v1[1]= v2[1];
+}
+
+float Inp2f(float *v1, float *v2)
+{
+	return v1[0]*v2[0]+v1[1]*v2[1];
+}
+
+float Normalise2(float *n)
+{
+	float d;
+	
+	d= n[0]*n[0]+n[1]*n[1];
+
+	if(d>1.0e-35F) {
+		d= (float)sqrt(d);
+
+		n[0]/=d; 
+		n[1]/=d; 
+	} else {
+		n[0]=n[1]= 0.0;
+		d= 0.0;
+	}
+	return d;
+}
+
 void hsv_to_rgb(float h, float s, float v, float *r, float *g, float *b)
 {
 	int i;
@@ -2188,7 +2407,7 @@ void hsv_to_rgb(float h, float s, float v, float *r, float *g, float *b)
 
 	h *= 360.0f;
 	
-	if(s==0 && 0) {
+	if(s==0.0) {
 		*r = v;
 		*g = v;
 		*b = v;
@@ -2331,3 +2550,38 @@ void cpack_to_rgb(unsigned int col, float *r, float *g, float *b)
 	*b= (float)(((col)>>16)&0xFF);
 	*b /= 255.0f;
 }
+
+
+/* *************** PROJECTIONS ******************* */
+
+void tubemap(float x, float y, float z, float *u, float *v)
+{
+	float len;
+	
+	*v = (z + 1.0) / 2.0;
+	
+	len= sqrt(x*x+y*y);
+	if(len>0) {
+		*u = (1.0 - (atan2(x/len,y/len) / M_PI)) / 2.0;
+	}
+}
+
+/* ------------------------------------------------------------------------- */
+
+void spheremap(float x, float y, float z, float *u, float *v)
+{
+	float len;
+	
+	len= sqrt(x*x+y*y+z*z);
+	if(len>0.0) {
+		
+		if(x==0.0 && y==0.0) *u= 0.0;	/* othwise domain error */
+		else *u = (1.0 - atan2(x,y)/M_PI )/2.0;
+		
+		z/=len;
+		*v = 1.0- saacos(z)/M_PI;
+	}
+}
+
+/* ------------------------------------------------------------------------- */
+
