@@ -1,5 +1,5 @@
 /**
- * $Id: psfont.c,v 1.5 2003/04/26 16:02:22 ton Exp $
+ * $Id: psfont.c,v 1.7 2005/03/19 21:08:10 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -207,11 +207,6 @@ static int started;
 #define SETCURRENTPOINT	(256+33)
 #define WHAT0		(0)
 
-/* some dirt for windows */
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 static char oneline[LINELEN];
 static objfnt *fnt;
 
@@ -871,7 +866,7 @@ static int decodetype1(PackedFile * pf, char *outname)
 		datbytes = pf->size - pf->seek;
 		memcpy(bindat, ((char *) pf->data) + pf->seek, datbytes);
 
-		if ((bindat[2] << 8 + bindat[3]) == 0x800){
+		if ((bindat[2] << (8 + bindat[3])) == 0x800){
 			/* order data (remove 6 bytes headers) */
 			i = datbytes;
 			hptr = bptr = bindat + 4;
