@@ -3,7 +3,7 @@
  * various string, file, list operations.
  *
  *
- * $Id: util.c,v 1.17 2004/12/12 13:29:54 lukep Exp $
+ * $Id: util.c,v 1.26 2005/06/10 13:12:59 jesterking Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -74,6 +74,16 @@
 #include <CoreFoundation/CoreFoundation.h>
 #endif
 
+
+// copies from BKE_utildefines
+#ifndef FILE_MAXDIR
+#define FILE_MAXDIR  160
+#endif
+
+#ifndef FILE_MAXFILE
+#define FILE_MAXFILE 80
+#endif
+
 /* local */
 
 static int add_win32_extension(char *name);
@@ -107,23 +117,23 @@ int BLI_stringdec(char *string, char *kop, char *staart, unsigned short *numlen)
 	len2 = len =  strlen( string);
 	
 	if (len > 6) {
-		if (strncasecmp(string + len - 6, ".blend", 6) == 0) len -= 6;
-		else if (strncasecmp(string + len - 6, ".trace", 6) == 0) len -= 6;
+		if (BLI_strncasecmp(string + len - 6, ".blend", 6) == 0) len -= 6;
+		else if (BLI_strncasecmp(string + len - 6, ".trace", 6) == 0) len -= 6;
 	}
 	
 	if (len == len2) {
 		if (len > 4) {
 			/* handle .jf0 en .jf1 for jstreams */
-			if (strncasecmp(string + len - 4, ".jf", 3) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".tga", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".jpg", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".png", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".txt", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".cyc", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".enh", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".rgb", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".psx", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".ble", 4) == 0) len -= 4;
+			if (BLI_strncasecmp(string + len - 4, ".jf", 3) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".tga", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".jpg", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".png", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".txt", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".cyc", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".enh", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".rgb", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".psx", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".ble", 4) == 0) len -= 4;
 		}
 	}
 	
@@ -413,7 +423,7 @@ int BLI_streq(char *a, char *b) {
 	return (strcmp(a, b)==0);
 }
 int BLI_strcaseeq(char *a, char *b) {
-	return (strcasecmp(a, b)==0);
+	return (BLI_strcasecmp(a, b)==0);
 }
 
 void BLI_makestringcode(char *fromfile, char *str)
@@ -421,10 +431,12 @@ void BLI_makestringcode(char *fromfile, char *str)
 	char *slash, len, temp[512];
 
 	strcpy(temp, fromfile);
+	
+	BLI_char_switch(temp, '\\', '/');
+	BLI_char_switch(str, '\\', '/');
 
 	/* Find the last slash */
-	slash = (strrchr(temp, '/')>strrchr(temp, '\\'))
-	    ? strrchr(temp, '/') : strrchr(temp, '\\');
+	slash = strrchr(temp, '/');
 	if(slash) {
 		*(slash+1)= 0;
 		len= strlen(temp);
@@ -433,6 +445,9 @@ void BLI_makestringcode(char *fromfile, char *str)
 				temp[0]= '/';
 				temp[1]= '/';
 				strcpy(temp+2, str+len);
+#ifdef			WIN32
+				BLI_char_switch(temp+2, '/', '\\');
+#endif
 				strcpy(str, temp);
 			}
 		}
@@ -441,28 +456,55 @@ void BLI_makestringcode(char *fromfile, char *str)
 
 int BLI_convertstringcode(char *path, char *basepath, int framenum)
 {
-	int len, wasrelative= (strncmp(path, "//", 2)==0);
+	int len, wasrelative;
+	char tmp[FILE_MAXDIR+FILE_MAXFILE];
+	char base[FILE_MAXDIR];
+	
+	strcpy(tmp, path);
+	strcpy(base, basepath);
+	
+	/* push slashes into unix mode - strings entering this part are
+	   potentially messed up: having both back- and forward slashes.
+	   Here we push into one conform direction, and at the end we
+	   push them into the system specific dir. This ensures uniformity
+	   of paths and solving some problems (and prevent potential future
+	   ones) -jesterKing. */
+	BLI_char_switch(tmp, '\\', '/');
+	BLI_char_switch(base, '\\', '/');
+	
+	wasrelative= (strncmp(tmp, "//", 2)==0);
 
-	if (path[0] == '/' && path[1] == '/') {
-		char *filepart= BLI_strdup(path+2); /* skip code */
-		char *lslash= BLI_last_slash(basepath);
+	if (tmp[0] == '/' && tmp[1] == '/') {
+		char *filepart= BLI_strdup(tmp+2); /* skip code */
+		char *lslash= BLI_last_slash(base);
 
 		if (lslash) {
-			int baselen= (int) (lslash-basepath) + 1;
+			int baselen= (int) (lslash-base) + 1;
 
-			memcpy(path, basepath, baselen);
-			strcpy(path+baselen, filepart);
+			memcpy(tmp, base, baselen);
+			strcpy(tmp+baselen, filepart);
 		} else {
-			strcpy(path, filepart);
+			strcpy(tmp, filepart);
 		}
 		
 		MEM_freeN(filepart);
 	}
 
-	len= strlen(path);
-	if(len && path[len-1]=='#') {
-		sprintf(path+len-1, "%04d", framenum);
+	len= strlen(tmp);
+	if(len && tmp[len-1]=='#') {
+		sprintf(tmp+len-1, "%04d", framenum);
 	}
+	
+	strcpy(path, tmp);
+#ifdef WIN32
+	/* skip first two chars, which in case of
+	   absolute path will be drive:/blabla and
+	   in case of relpath //blabla/. So relpath
+	   // will be retained, rest will be nice and
+	   shiny win32 backward slashes :) -jesterKing
+	*/
+	BLI_char_switch(path+2, '/', '\\');
+#endif
 
 	return wasrelative;
 }
@@ -490,13 +532,12 @@ char *BLI_gethome(void) {
 	#else /* Windows */
 		char * ret;
 		static char dir[512];
-		char tmpdir[512];
 
 		/* Check for %HOME% env var */
 
 		ret = getenv("HOME");
 		if(ret) {
-			sprintf(dir, "%s\.blender", ret);
+			sprintf(dir, "%s\\.blender", ret);
 			if (BLI_exists(dir)) return dir;
 		}
 
@@ -506,7 +547,7 @@ char *BLI_gethome(void) {
 
 		if (BLI_exists(dir))
 		{
-			strcat(dir,"/.blender");
+			strcat(dir,"\\.blender");
 			if (BLI_exists(dir)) return(dir);
 		}
 
@@ -515,10 +556,10 @@ char *BLI_gethome(void) {
 		ret = getenv("USERPROFILE");
 		if (ret) {
 			if (BLI_exists(ret)) { /* from fop, also below... */
-				sprintf(dir, "%s/Application Data/Blender Foundation/Blender", ret);
+				sprintf(dir, "%s\\Application Data\\Blender Foundation\\Blender", ret);
 				BLI_recurdir_fileops(dir);
 				if (BLI_exists(dir)) {
-					strcat(dir,"/.blender");
+					strcat(dir,"\\.blender");
 					if(BLI_exists(dir)) return(dir);
 				}
 			}
@@ -543,22 +584,34 @@ char *BLI_gethome(void) {
 	#endif
 }
 
-static void char_switch(char *string, char from, char to) 
+void BLI_clean(char *path)
 {
+	if(path==0) return;
+#ifdef WIN32
+	if(path && strlen(path)>2) {
+		BLI_char_switch(path+2, '/', '\\');
+	}
+#else
+	BLI_char_switch(path, '\\', '/');
+#endif
+}
+
+void BLI_char_switch(char *string, char from, char to) 
+{
+	if(string==0) return;
 	while (*string != 0) {
 		if (*string == from) *string = to;
 		string++;
 	}
 }
 
-void BLI_make_exist(char *dir)
-{
+void BLI_make_exist(char *dir) {
 	int a;
 
 	#ifdef WIN32
-		char_switch(dir, '/', '\\');
+		BLI_char_switch(dir, '/', '\\');
 	#else
-		char_switch(dir, '\\', '/');
+		BLI_char_switch(dir, '\\', '/');
 	#endif	
 	
 	a = strlen(dir);
@@ -599,6 +652,12 @@ void BLI_make_file_string(char *relabase, char *string,  char *dir,  char *file)
 	
 	string[0]= 0; /* ton */
 
+	/* we first push all slashes into unix mode, just to make sure we don't get
+	   any mess with slashes later on. -jesterKing */
+	BLI_char_switch(relabase, '\\', '/');
+	BLI_char_switch(dir, '\\', '/');
+	BLI_char_switch(file, '\\', '/');
+
 	/* Resolve relative references */	
 	if (relabase && dir[0] == '/' && dir[1] == '/') {
 		char *lslash;
@@ -625,11 +684,7 @@ void BLI_make_file_string(char *relabase, char *string,  char *dir,  char *file)
 	strcat (string, file);
 	
 	/* Push all slashes to the system preferred direction */
-	#ifdef WIN32
-		char_switch(string, '/', '\\');
-	#else
-		char_switch(string, '\\', '/');
-	#endif	
+	BLI_clean(string);
 }
 
 int BLI_testextensie(char *str, char *ext)
@@ -642,7 +697,7 @@ int BLI_testextensie(char *str, char *ext)
 
 	if(a==0 || b==0 || b>=a) {
 		retval = 0;
-	} else if (strcasecmp(ext, str + a - b)) {
+	} else if (BLI_strcasecmp(ext, str + a - b)) {
 		retval = 0;	
 	} else {
 		retval = 1;
@@ -651,6 +706,8 @@ int BLI_testextensie(char *str, char *ext)
 	return (retval);
 }
 
+
+
 void BLI_split_dirfile(char *string, char *dir, char *file)
 {
 	int a;
@@ -659,16 +716,19 @@ void BLI_split_dirfile(char *string, char *dir, char *file)
 	file[0]= 0;
 
 #ifdef WIN32
+	BLI_char_switch(string, '/', '\\'); /* make sure we have a valid path format */
+
 	if (strlen(string)) {
+		int len;
 		if (string[0] == '/' || string[0] == '\\') { 
-			strcpy(dir, string);
+			BLI_strncpy(dir, string, FILE_MAXDIR);
 		} else if (string[1] == ':' && string[2] == '\\') {
-            strcpy(dir, string);
+            BLI_strncpy(dir, string, FILE_MAXDIR);
         } else {
             BLI_getwdN(dir);
             strcat(dir,"/");
             strcat(dir,string);
-            strcpy(string,dir);
+            BLI_strncpy(string,dir,FILE_MAXDIR+FILE_MAXFILE);
         }
 
         BLI_make_exist(dir);
@@ -680,9 +740,20 @@ void BLI_split_dirfile(char *string, char *dir, char *file)
         if(a>=4 && dir[a-1]=='\\') dir[a-1] = 0;
 
         if (S_ISDIR(BLI_exist(dir))) {
-            strcpy(file,string + strlen(dir));
 
-            if (strrchr(file,'\\')) strcpy(file,strrchr(file,'\\')+1);
+	    /* copy from end of string into file, to ensure filename itself isn't truncated 
+	       if string is too long. (aphex) */
+
+	    len = FILE_MAXFILE - strlen(string);
+
+	    if (len < 0)
+		BLI_strncpy(file,string + abs(len),FILE_MAXFILE);
+	    else
+		BLI_strncpy(file,string,FILE_MAXFILE);
+	    
+            if (strrchr(string,'\\')){
+		BLI_strncpy(file,strrchr(string,'\\')+1,FILE_MAXFILE);
+	    }
 
             if (a = strlen(dir)) {
                 if (dir[a-1] != '\\') strcat(dir,"\\");
@@ -692,7 +763,7 @@ void BLI_split_dirfile(char *string, char *dir, char *file)
 			a = strlen(dir) - 1;
 			while(a>0 && dir[a] != '\\') a--;
 			dir[a + 1] = 0;
-			strcpy(file, string + strlen(dir));
+			BLI_strncpy(file, string + strlen(dir),FILE_MAXFILE);
 		}
 		
 	}
@@ -739,15 +810,6 @@ void BLI_split_dirfile(char *string, char *dir, char *file)
 	}
 #endif
 }
-
-// copies from BKE_utildefines
-#ifndef FILE_MAXDIR
-#define FILE_MAXDIR  160
-#endif
-
-#ifndef FILE_MAXFILE
-#define FILE_MAXFILE 80
-#endif
  
 static int add_win32_extension(char *name)
 {
@@ -881,3 +943,40 @@ char* BLI_getbundle(void) {
 }
 #endif
 
+int BLI_strcasecmp(const char *s1, const char *s2) {
+	int i;
+
+	for (i=0; ; i++) {
+		char c1 = tolower(s1[i]);
+		char c2 = tolower(s2[i]);
+
+		if (c1<c2) {
+			return -1;
+		} else if (c1>c2) {
+			return 1;
+		} else if (c1==0) {
+			break;
+		}
+	}
+
+	return 0;
+}
+
+int BLI_strncasecmp(const char *s1, const char *s2, int n) {
+	int i;
+
+	for (i=0; i<n; i++) {
+		char c1 = tolower(s1[i]);
+		char c2 = tolower(s2[i]);
+
+		if (c1<c2) {
+			return -1;
+		} else if (c1>c2) {
+			return 1;
+		} else if (c1==0) {
+			break;
+		}
+	}
+
+	return 0;
+}
