@@ -2,7 +2,7 @@
  *
  * .blend file writing
  *
- * $Id: writefile.c,v 1.36 2004/12/16 14:40:25 ton Exp $
+ * $Id: writefile.c,v 1.44 2005/05/08 21:20:32 ianwill Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -100,59 +100,62 @@ Important to know is that 'streaming' has been added to files, for Blender Publi
 
 #include "nla.h" //  __NLA is defined
 
-#include "DNA_packedFile_types.h"
-#include "DNA_sdna_types.h"
-#include "DNA_property_types.h"
-#include "DNA_sensor_types.h"
-#include "DNA_controller_types.h"
+#include "DNA_armature_types.h"
+#include "DNA_action_types.h"
 #include "DNA_actuator_types.h"
-#include "DNA_effect_types.h"
-#include "DNA_object_types.h"
-#include "DNA_userdef_types.h"
-#include "DNA_vfont_types.h"
-#include "DNA_ipo_types.h"
+#include "DNA_controller_types.h"
 #include "DNA_curve_types.h"
+#include "DNA_constraint_types.h"
 #include "DNA_camera_types.h"
+#include "DNA_effect_types.h"
+#include "DNA_group_types.h"
+#include "DNA_image_types.h"
+#include "DNA_ika_types.h"
+#include "DNA_ipo_types.h"
+#include "DNA_fileglobal_types.h"
+#include "DNA_key_types.h"
+#include "DNA_lattice_types.h"
+#include "DNA_listBase.h" /* for Listbase, the type of samples, ...*/
+#include "DNA_lamp_types.h"
 #include "DNA_meta_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_material_types.h"
-#include "DNA_lattice_types.h"
-#include "DNA_armature_types.h"
-#include "DNA_sequence_types.h"
-#include "DNA_ika_types.h"
-#include "DNA_group_types.h"
+#include "DNA_nla_types.h"
+#include "DNA_object_types.h"
+#include "DNA_object_force.h"
 #include "DNA_oops_types.h"
+#include "DNA_packedFile_types.h"
+#include "DNA_property_types.h"
+#include "DNA_scene_types.h"
+#include "DNA_sdna_types.h"
+#include "DNA_sequence_types.h"
+#include "DNA_sensor_types.h"
 #include "DNA_space_types.h"
 #include "DNA_screen_types.h"
-#include "DNA_view3d_types.h"
-#include "DNA_lamp_types.h"
-#include "DNA_fileglobal_types.h"
 #include "DNA_sound_types.h"
 #include "DNA_texture_types.h"
 #include "DNA_text_types.h"
-#include "DNA_image_types.h"
-#include "DNA_key_types.h"
-#include "DNA_scene_types.h"
-#include "DNA_constraint_types.h"
-#include "DNA_listBase.h" /* for Listbase, the type of samples, ...*/
-#include "DNA_action_types.h"
-#include "DNA_nla_types.h"
+#include "DNA_view3d_types.h"
+#include "DNA_vfont_types.h"
+#include "DNA_userdef_types.h"
 
 #include "MEM_guardedalloc.h" // MEM_freeN
 #include "BLI_blenlib.h"
 #include "BLI_linklist.h"
 
 #include "BKE_action.h"
-#include "BKE_utildefines.h" // for KNOTSU KNOTSV WHILE_SEQ END_SEQ defines
 #include "BKE_bad_level_calls.h" // build_seqar (from WHILE_SEQ) free_oops error
+#include "BKE_curve.h"
 #include "BKE_constraint.h"
-#include "BKE_main.h" // G.main
 #include "BKE_global.h" // for G
-#include "BKE_screen.h" // for waitcursor
-#include "BKE_packedFile.h" // for packAll
 #include "BKE_library.h" // for  set_listbasepointers
+#include "BKE_main.h" // G.main
+#include "BKE_packedFile.h" // for packAll
+#include "BKE_screen.h" // for waitcursor
+#include "BKE_scene.h" // for do_seq
 #include "BKE_sound.h" /* ... and for samples */
+#include "BKE_utildefines.h" // for defines
 
 #include "GEN_messaging.h"
 
@@ -472,6 +475,9 @@ static void write_sensors(WriteData *wd, ListBase *lb)
 		case SENS_MESSAGE:
 			writestruct(wd, DATA, "bMessageSensor", 1, sens->data);
 			break;
+		case SENS_JOYSTICK:
+			writestruct(wd, DATA, "bJoystickSensor", 1, sens->data);
+			break;
 		default:
 			; /* error: don't know how to write this file */
 		}
@@ -656,6 +662,7 @@ static void write_objects(WriteData *wd, ListBase *idbase)
 {
 	Object *ob;
 	ObHook *hook;
+	int a;
 	
 	ob= idbase->first;
 	while(ob) {
@@ -678,6 +685,16 @@ static void write_objects(WriteData *wd, ListBase *idbase)
 			write_nlastrips(wd, &ob->nlastrips);
 			
 			writestruct(wd, DATA, "PartDeflect", 1, ob->pd);
+			writestruct(wd, DATA, "SoftBody", 1, ob->soft);
+			if(ob->soft) {
+				SoftBody *sb= ob->soft;
+				if(sb->keys) {
+					writedata(wd, DATA, sizeof(void *)*sb->totkey, sb->keys);
+					for(a=0; a<sb->totkey; a++) {
+						writestruct(wd, DATA, "SBVertex", sb->totpoint, sb->keys[a]);
+					}
+				}
+			}
 			
 			for(hook= ob->hooks.first; hook; hook= hook->next) {
 				writestruct(wd, DATA, "ObHook", 1, hook);
@@ -1065,6 +1082,7 @@ static void write_scenes(WriteData *wd, ListBase *scebase)
 	Sequence *seq;
 	MetaStack *ms;
 	Strip *strip;
+	TimeMarker *marker;
 
 	sce= scebase->first;
 	while(sce) {
@@ -1139,6 +1157,13 @@ static void write_scenes(WriteData *wd, ListBase *scebase)
 			if (sce->r.qtcodecdata->cdParms) writedata(wd, DATA, sce->r.qtcodecdata->cdSize, sce->r.qtcodecdata->cdParms);
 		}
 
+		/* writing dynamic list of TimeMarkers to the blend file */
+		marker= sce->markers.first;
+		while(marker){
+			writestruct(wd, DATA, "TimeMarker", 1, marker);
+			marker= marker->next;
+		}
+		
 		sce= sce->id.next;
 	}
 	/* flush helps the compression for undo-save */
@@ -1183,6 +1208,9 @@ static void write_screens(WriteData *wd, ListBase *scrbase)
 				pa= pa->next;
 			}
 
+			/* space handler scriptlinks */
+			write_scriptlink(wd, &sa->scriptlink);
+
 			sl= sa->spacedata.first;
 			while(sl) {
 				if(sl->spacetype==SPACE_VIEW3D) {
@@ -1254,6 +1282,9 @@ static void write_screens(WriteData *wd, ListBase *scrbase)
 				else if(sl->spacetype==SPACE_NLA){
 					writestruct(wd, DATA, "SpaceNla", 1, sl);
 				}
+				else if(sl->spacetype==SPACE_TIME){
+					writestruct(wd, DATA, "SpaceTime", 1, sl);
+				}
 				sl= sl->next;
 			}
 
@@ -1494,8 +1525,8 @@ static void write_global(WriteData *wd)
 
 	fg.curscreen= G.curscreen;
 	fg.curscene= G.scene;
-	fg.displaymode= R.displaymode;
-	fg.winpos= R.winpos;
+	fg.displaymode= G.displaymode;
+	fg.winpos= G.winpos;
 	fg.fileflags= (G.fileflags & ~G_FILE_NO_UI);	// prevent to save this, is not good convention, and feature with concerns...
 	fg.globalf= G.f;
 
@@ -1509,7 +1540,7 @@ static int write_file_handle(int handle, MemFile *compare, MemFile *current, int
 	ListBase mainlist;
 	char buf[13];
 	WriteData *wd;
-	int data;
+/*	int data; */ /*unused*/
 
 	mainlist.first= mainlist.last= G.main;
 	G.main->next= NULL;
@@ -1573,9 +1604,6 @@ int BLO_write_file(char *dir, int write_flags, char **error_r)
 	char userfilename[FILE_MAXDIR+FILE_MAXFILE];
 	char tempname[FILE_MAXDIR+FILE_MAXFILE];
 	int file, fout, write_user_block;
-#ifdef WIN32
-	char tmpdir[FILE_MAXDIR+FILE_MAXFILE];
-#endif
 
 	sprintf(tempname, "%s@", dir);
 
