@@ -1,5 +1,5 @@
 /**
- * $Id: BDR_editobject.h,v 1.10 2004/04/24 21:09:05 theeth Exp $
+ * $Id: BDR_editobject.h,v 1.17 2005/05/04 12:53:35 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -33,14 +33,13 @@
 #ifndef BDR_EDITOBJECT_H
 #define BDR_EDITOBJECT_H
 
-struct TransVert;
 struct Object;
-struct TransOb;
 struct Tex;
 struct Material;
 struct Base;
 
 void add_object_draw(int type);
+void add_objectLamp(short type);
 void free_and_unlink_base(struct Base *base);
 void delete_obj(int ok);
 void make_track(void);
@@ -55,9 +54,12 @@ int test_parent_loop(struct Object *par, struct Object *ob);
 void make_parent(void);
 
 void make_displists_by_parent(struct Object *ob);
+
 void exit_editmode(int freedata);
 void check_editmode(int type);
-void docentre(void);
+void enter_editmode(void);
+
+void docentre(int centremode);
 void docentre_new(void);
 void docentre_cursor(void);
 void movetolayer(void);
@@ -71,28 +73,20 @@ void make_links_menu(void);
 void make_links(short event);
 void make_duplilist_real(void);
 void apply_object(void);
-void ob_to_transob(struct Object *ob, struct TransOb *tob);
-void ob_to_tex_transob(struct Object *ob, struct TransOb *tob);
-void make_trans_objects(void);
-void enter_editmode(void);
+
 void copymenu_logicbricks(struct Object *ob);
-void clearbaseflags_for_editing(void);
-void make_trans_verts(float *min, float *max, int mode);
-void draw_prop_circle(void);
-void set_proportional_weight(struct TransVert *tv, float *min, float *max);
-void special_trans_update(int keyflags);
-void special_aftertrans_update(char mode, int flip, short canceled, int keyflags);
-void calc_trans_verts(void);
+
+
+/* old transform */
 void apply_keyb_grid(float *val, float fac1, float fac2, float fac3, int invert);
 void compatible_eul(float *eul, float *oldrot);
 void headerprint(char *str);
-void add_ipo_tob_poin(float *poin, float *old, float delta);
-void restore_tob(struct TransOb *tob);
+/* used for old game engine collision optimize */
 int cylinder_intersect_test(void);
 int sphere_intersect_test(void);
-int my_clock(void);
-void transform(int mode);
-void std_rmouse_transform(void (*xf_func)(int));
+
+
+void std_rmouse_transform(void (*xf_func)(int, int));
 void rightmouse_transform(void);
 void single_object_users(int flag);
 void new_id_matar(struct Material **matar, int totcol);
@@ -116,8 +110,9 @@ void first_base(void);
 void make_displists_by_obdata(void *obdata);
 void flip_subdivison(struct Object *ob, int);
 void mirrormenu(void);
-void mirror_edit(short mode);
-void mirror_object(short mode);
+
+
+void add_hook(void);
 
 #endif /*  BDR_EDITOBJECT_H */
 
