@@ -1,5 +1,5 @@
 /**
- * $Id: BDR_unwrapper.h,v 1.2 2004/09/06 14:30:28 blendix Exp $
+ * $Id: BDR_unwrapper.h,v 1.4 2005/04/23 01:36:07 blendix Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -33,9 +33,9 @@
 #ifndef BDR_UNWRAPPER_H
 #define BDR_UNWRAPPER_H
 
-void set_seamtface(); /* set TF_SEAM flags in tfaces */
-void unwrap_lscm(); /* unwrap selected tfaces */
-void select_linked_tfaces_with_seams();
+void set_seamtface(void); /* set TF_SEAM flags in tfaces */
+void unwrap_lscm(void); /* unwrap selected tfaces */
+void select_linked_tfaces_with_seams(int mode, Mesh *me, unsigned int index);
 
 #endif /* BDR_UNWRAPPER_H */
 
