@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_editdeform.h,v 1.6 2004/04/20 22:00:32 ton Exp $
+ * $Id: BIF_editdeform.h,v 1.8 2005/01/12 23:57:32 blendix Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,7 +43,6 @@ struct MDeformVert;
 struct MDeformWeight;
 struct bDeformGroup;
 
-struct bDeformGroup *get_named_vertexgroup (Object *ob, char *name);
 void unique_vertexgroup_name (struct bDeformGroup *dg, struct Object *ob);
 struct bDeformGroup *add_defgroup_name (struct Object *ob, char *name);
 void add_defgroup (struct Object *ob);
@@ -53,7 +52,6 @@ void remove_verts_defgroup (int allverts);
 void sel_verts_defgroup (int select);
 struct MDeformWeight *verify_defweight (struct MDeformVert *dv, int defgroup);
 void verify_defgroups (struct Object *ob);
-int  get_defgroup_num (struct Object *ob, struct bDeformGroup   *dg);
 void add_vert_to_defgroup (struct Object *ob, struct bDeformGroup *dg, 
 						   int vertnum, float weight, 
 						   int assignmode);
@@ -61,5 +59,7 @@ void remove_vert_defgroup (struct Object *ob, struct bDeformGroup  *dg,
 						   int vertnum);
 void create_dverts(Mesh *me);
 
+extern void object_apply_deform(struct Object *ob);
+
 #endif
 
