@@ -1,5 +1,5 @@
 /**
- * $Id: BSE_headerbuttons.h,v 1.15 2004/05/12 16:53:10 broken Exp $
+ * $Id: BSE_headerbuttons.h,v 1.21 2005/05/19 19:58:52 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,6 +34,7 @@
 #define BSE_HEADERBUTTONS_H
 
 struct uiBlock;
+struct ScrArea;
 struct ID;
 
 /* these used to be in blender/src/headerbuttons.c: */
@@ -55,6 +56,7 @@ void load_sound_buttons(char *str);
 /* end of declarations moved from old headerbuttons.c */
 
 void update_for_newframe_muted(void);
+void update_for_newframe_nodraw(int nosound);
 void free_matcopybuf(void);
 void clear_matcopybuf(void);
 void write_videoscape_fs(void);
@@ -66,8 +68,8 @@ struct Scene *copy_scene(struct Scene *sce, int level);
 
 void buttons_active_id(struct ID **id, struct ID **idfrom);
 
-int start_progress_bar();
-void end_progress_bar();
+int start_progress_bar(void);
+void end_progress_bar(void);
 int progress_bar(float done, char *busy_info);
 
 void update_for_newframe(void);
@@ -85,6 +87,7 @@ void seq_buttons(void);
 void sound_buttons(void);
 void text_buttons(void);
 void script_buttons(void);
+void time_buttons(struct ScrArea *sa);
 void view3d_buttons(void);
 
 void do_global_buttons(unsigned short event);
@@ -103,16 +106,21 @@ void do_oops_buttons(short event);
 void do_seq_buttons(short event);
 void do_sound_buttons(unsigned short event);
 void do_text_buttons(unsigned short event);
+void do_time_buttons(struct ScrArea *sa, unsigned short event);
 void do_script_buttons(unsigned short event);
 void do_view3d_buttons(short event); 
 
 void do_headerbuttons(short event);
 
+/* header_text.c */
+void do_text_editmenu_to3dmenu(void *arg, int event);
+
 /* header_info.c */
 void do_info_add_meshmenu(void *arg, int event);
 void do_info_add_curvemenu(void *arg, int event);
 void do_info_add_surfacemenu(void *arg, int event);
 void do_info_add_metamenu(void *arg, int event);
+void do_info_add_lampmenu(void *arg, int event);
 void do_info_addmenu(void *arg, int event);
 
 /* header_view3d.c */
