@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_ipo_types.h (mar-2001 nzc)
  *
- * $Id: DNA_ipo_types.h,v 1.6 2003/04/27 11:55:09 ton Exp $ 
+ * $Id: DNA_ipo_types.h,v 1.14 2005/03/01 13:14:37 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -57,6 +57,9 @@ typedef short IPO_Channel;
 
 #define IPOBUTY	17
 
+#define TOB_IPO	1
+#define TOB_IPODROT	2
+#define TOB_IKA	4
 
 /* disptype */
 #define IPO_DISPDEGR	1
@@ -65,7 +68,8 @@ typedef short IPO_Channel;
 
 /* ******************** */
 
-#define OB_TOTIPO	24
+#define OB_TOTIPO	29
+#define OB_TOTNAM	29
 
 #define OB_LOC_X	1
 #define OB_LOC_Y	2
@@ -101,11 +105,18 @@ typedef short IPO_Channel;
 #define OB_COL_B	23
 #define OB_COL_A	24
 
+#define OB_PD_FSTR	25
+#define OB_PD_FFALL	26
+#define OB_PD_SDAMP	27
+#define OB_PD_RDAMP	28
+#define OB_PD_PERM	29
+
 
 
 /* ******************** */
 
-#define MA_TOTIPO	32
+#define MA_TOTIPO	40
+#define MA_TOTNAM	26
 
 #define MA_COL_R	1
 #define MA_COL_G	2
@@ -123,9 +134,16 @@ typedef short IPO_Channel;
 #define MA_SPEC		14
 #define MA_HARD		15
 #define MA_SPTR		16
-#define MA_ANG		17
+#define MA_IOR		17
 #define MA_MODE		18
 #define MA_HASIZE	19
+#define MA_TRANSLU	20
+#define MA_RAYM		21
+#define MA_FRESMIR	22
+#define MA_FRESMIRI	23
+#define MA_FRESTRA	24
+#define MA_FRESTRAI	25
+#define MA_ADD		26
 
 #define MA_MAP1		0x20
 #define MA_MAP2		0x40
@@ -135,6 +153,10 @@ typedef short IPO_Channel;
 #define MA_MAP6		0x400
 #define MA_MAP7		0x800
 #define MA_MAP8		0x1000
+#define MA_MAP9		0x2000
+#define MA_MAP10	0x4000
+
+#define TEX_TOTNAM	14
 
 #define MAP_OFS_X	1
 #define MAP_OFS_Y	2
@@ -150,22 +172,57 @@ typedef short IPO_Channel;
 #define MAP_COLF	11
 #define MAP_NORF	12
 #define MAP_VARF	13
+#define MAP_DISP	14
+
+/* ******************** */
+
+#define TE_TOTIPO	21
+#define TE_TOTNAM	21
+
+#define TE_NSIZE	1
+#define TE_NDEPTH	2
+#define TE_NTYPE	3
+#define TE_TURB		4
+
+#define TE_VNW1		5
+#define TE_VNW2		6
+#define TE_VNW3		7
+#define TE_VNW4		8
+#define TE_VNMEXP	9
+#define TE_VN_DISTM	10
+#define TE_VN_COLT	11
+
+#define TE_ISCA		12
+#define TE_DISTA	13
+
+#define TE_MG_TYP	14
+#define TE_MGH		15
+#define TE_MG_LAC	16
+#define TE_MG_OCT	17
+#define TE_MG_OFF	18
+#define TE_MG_GAIN	19
+
+#define TE_N_BAS1	20
+#define TE_N_BAS2	21
 
 /* ******************** */
 
 #define SEQ_TOTIPO	1
+#define SEQ_TOTNAM	1
 
 #define SEQ_FAC1	1
 
 /* ******************** */
 
 #define CU_TOTIPO	1
+#define CU_TOTNAM	1
 
 #define CU_SPEED	1
 
 /* ******************** */
 
-#define KEY_TOTIPO	32
+#define KEY_TOTIPO	64
+#define KEY_TOTNAM	64
 
 #define KEY_SPEED	0
 #define KEY_NR		1
@@ -173,6 +230,7 @@ typedef short IPO_Channel;
 /* ******************** */
 
 #define WO_TOTIPO	29
+#define WO_TOTNAM	16
 
 #define WO_HOR_R	1
 #define WO_HOR_G	2
@@ -196,7 +254,8 @@ typedef short IPO_Channel;
 
 /* ******************** */
 
-#define LA_TOTIPO	23
+#define LA_TOTIPO	21
+#define LA_TOTNAM	10
 
 #define LA_ENERGY	1
 #define LA_COL_R	2
@@ -211,16 +270,21 @@ typedef short IPO_Channel;
 
 /* ******************** */
 
-#define CAM_TOTIPO	3
+/* yafray: totipo & totnam +2 because of added curves */
+#define CAM_TOTIPO	5
+#define CAM_TOTNAM	5
 
 #define CAM_LENS	1
 #define CAM_STA		2
 #define CAM_END		3
-
+/* yafray aperture & focal distance curves */
+#define CAM_YF_APERT	4
+#define CAM_YF_FDIST	5
 
 /* ******************** */
 
 #define SND_TOTIPO	4
+#define SND_TOTNAM	4
 
 #define SND_VOLUME	1
 #define SND_PITCH	2
@@ -230,6 +294,7 @@ typedef short IPO_Channel;
 /* ******************** */
 
 #define AC_TOTIPO	10	/* __NLA */
+#define AC_TOTNAM	10
 
 #define AC_LOC_X	1
 #define AC_LOC_Y	2
@@ -246,6 +311,7 @@ typedef short IPO_Channel;
 
 /* ******************** */
 #define CO_TOTIPO	1	/* Constraint Ipos */
+#define CO_TOTNAM	1
 
 #define CO_ENFORCE	1
 /*
