@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_material_types.h (mar-2001 nzc)
  *
- * $Id: DNA_material_types.h,v 1.15 2004/04/26 16:49:05 ton Exp $ 
+ * $Id: DNA_material_types.h,v 1.28 2005/05/27 17:52:52 eeshlo Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -39,9 +39,14 @@
 #include "DNA_ID.h"
 #include "DNA_scriptlink_types.h"
 
+#ifndef MAX_MTEX
+#define MAX_MTEX	10
+#endif
+
 struct MTex;
 struct Ipo;
 struct Material;
+struct ColorBand;
 
 /* WATCH IT: change type? also make changes in ipo.h  */
 
@@ -59,6 +64,7 @@ typedef struct Material {
 	float translucency;
 	float fresnel_mir, fresnel_mir_i;
 	float fresnel_tra, fresnel_tra_i;
+	float filter, pad0;		/* filter added, for raytrace transparency */
 	short ray_depth, ray_depth_tra;
 	short har;
 	char seed1, seed2;
@@ -69,24 +75,38 @@ typedef struct Material {
 	float hasize, flaresize, subsize, flareboost;
 	
 	/* for buttons and render*/
-	char rgbsel, texact, pr_type, septex;
-	short pr_back, pr_lamp;
-
+	char rgbsel, texact, pr_type, pad;
+	short pr_back, pr_lamp, septex, pad4;
+	int pad5;
+	
 	/* shaders */
 	short diff_shader, spec_shader;
 	float roughness, refrac;
 	float param[4];		/* size, smooth, size, smooth, for toonshader */
+	float rms;
+	float darkness;
 	short texco, mapto;
 	
-	struct MTex *mtex[8];
+	/* ramp colors */
+	struct ColorBand *ramp_col;
+	struct ColorBand *ramp_spec;
+	char rampin_col, rampin_spec;
+	char rampblend_col, rampblend_spec;
+	short ramp_show, pad3;
+	float rampfac_col, rampfac_spec;
+
+	struct MTex *mtex[10];
 	struct Ipo *ipo;
-	struct Material *ren;
 	
 	/* dynamic properties */
 	float friction, fh, reflect;
 	float fhdist, xyfrict;
 	short dynamode, pad2;
-	
+
+	/* yafray: absorption color, dispersion parameters and material preset menu */
+	float YF_ar, YF_ag, YF_ab, YF_dscale, YF_dpwr;
+	int YF_dsmp, YF_preset, YF_djit;
+
 	ScriptLink scriptlink;
 } Material;
 
@@ -129,22 +149,44 @@ typedef struct Material {
 #define MA_RAYTRANSP	0x20000
 #define MA_RAYMIRROR	0x40000
 #define MA_SHADOW_TRA	0x80000
+#define MA_RAMP_COL		0x100000
+#define MA_RAMP_SPEC	0x200000
+#define MA_RAYBIAS		0x400000
+#define MA_FULL_OSA		0x800000
 
 /* diff_shader */
 #define MA_DIFF_LAMBERT		0
 #define MA_DIFF_ORENNAYAR	1
 #define MA_DIFF_TOON		2
+#define MA_DIFF_MINNAERT        3
 
 /* spec_shader */
 #define MA_SPEC_COOKTORR	0
 #define MA_SPEC_PHONG		1
 #define MA_SPEC_BLINN		2
 #define MA_SPEC_TOON		3
+#define MA_SPEC_WARDISO		4
 
 /* dynamode */
 #define MA_DRAW_DYNABUTS    1
 #define MA_FH_NOR	        2
 
+/* ramps */
+#define MA_RAMP_IN_SHADER	0
+#define MA_RAMP_IN_ENERGY	1
+#define MA_RAMP_IN_NOR		2
+#define MA_RAMP_IN_RESULT	3
+
+#define MA_RAMP_BLEND		0
+#define MA_RAMP_ADD			1
+#define MA_RAMP_MULT		2
+#define MA_RAMP_SUB			3
+#define MA_RAMP_SCREEN		4
+#define MA_RAMP_DIV			5
+#define MA_RAMP_DIFF		6
+#define MA_RAMP_DARK		7
+#define MA_RAMP_LIGHT		8
+
 /* texco */
 #define TEXCO_ORCO		1
 #define TEXCO_REFL		2
@@ -158,6 +200,8 @@ typedef struct Material {
 #define TEXCO_OSA		512
 #define TEXCO_WINDOW	1024
 #define NEED_UV			2048
+/* optim = use simpler AA */
+#define TEXCO_OPTIM		4096
 
 /* mapto */
 #define MAP_COL			1
@@ -174,6 +218,7 @@ typedef struct Material {
 #define MAP_TRANSLU		1024
 #define MAP_AMB			2048
 #define MAP_DISPLACE	4096
+#define MAP_WARP		8192
 
 /* pr_type */
 #define MA_FLAT			0
