@@ -1,5 +1,5 @@
 /**
- * $Id: DNA_meshdata_types.h,v 1.1 2004/03/20 22:55:38 zuster Exp $ 
+ * $Id: DNA_meshdata_types.h,v 1.7 2004/09/24 12:58:21 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,6 +40,12 @@ typedef struct MFace {
 	char edcode, flag;
 } MFace;
 
+typedef struct MEdge {
+	unsigned int v1, v2;
+	char crease, pad;
+	short flag;
+} MEdge;
+
 typedef struct MDeformWeight {
 	int				def_nr;
 	float			weight;
@@ -66,4 +72,37 @@ typedef struct MSticky {
 	float co[2];
 } MSticky;
 
+/* mvert->flag (1=SELECT) */
+#define ME_SPHERETEST	2
+#define ME_SPHERETEMP	4
+#define ME_HIDE			16
+
+/* medge->flag (1=SELECT)*/
+#define ME_EDGEDRAW		2
+#define ME_SEAM         4
+#define ME_FGON			8
+						// reserve 16 for ME_HIDE
+
+/* puno = vertexnormal (mface) */
+#define ME_FLIPV1		1
+#define ME_FLIPV2		2
+#define ME_FLIPV3		4
+#define ME_FLIPV4		8
+#define ME_PROJXY		16
+#define ME_PROJXZ		32
+#define ME_PROJYZ		64
+
+/* edcode (mface) */
+#define ME_V1V2			1
+#define ME_V2V3			2
+#define ME_V3V1			4
+#define ME_V3V4			4
+#define ME_V4V1			8
+
+/* flag (mface) */
+#define ME_SMOOTH		1
+#define ME_FACE_SEL		2
+						/* flag ME_HIDE is used here too */ 
+
+
 #endif
