@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_space_types.h (mar-2001 nzc)
  *	
- * $Id: DNA_space_types.h,v 1.19 2004/04/19 22:05:36 ton Exp $ 
+ * $Id: DNA_space_types.h,v 1.30 2005/05/19 19:58:53 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -47,6 +47,8 @@ struct ImBuf;
 struct Image;
 struct SpaceIpo;
 struct BlendHandle;
+struct TreeStore;
+
 
 	/**
 	 * The base structure all the other spaces
@@ -194,6 +196,11 @@ typedef struct SpaceOops {
 	short pin, visiflag, flag, rt;
 	void *lockpoin;
 	
+	ListBase tree;
+	struct TreeStore *treestore;
+	short type, outlinevis, storeflag;
+	short deps_flags;
+	
 } SpaceOops;
 
 typedef struct SpaceImage {
@@ -208,10 +215,9 @@ typedef struct SpaceImage {
 	
 	struct Image *image;
 	float zoom;
-	float pad2; /* padding is with 8 bytes aligned  */
 	short mode, pin;
 	short imanr, curtile;
-	short xof, yof;
+	float xof, yof;
 	short flag, lock;
 	
 } SpaceImage;
@@ -248,6 +254,11 @@ typedef struct SpaceText {
 	int left;
 	int showlinenrs;
 	
+	int tabnumber;
+	int currtab_set; 
+	int showsyntax;
+	int unused_padd;
+	
 	float pix_per_line;
 
 	struct rcti txtscroll, txtbar;
@@ -266,6 +277,20 @@ typedef struct SpaceScript {
 
 } SpaceScript;
 
+typedef struct SpaceTime {
+	SpaceLink *next, *prev;
+	int spacetype;
+	float blockscale;
+	struct ScrArea *area;
+	
+	View2D v2d;
+	
+	int flag, redraws;
+	
+} SpaceTime;
+
+
+
 #
 #
 typedef struct OneSelectableIma {
@@ -395,6 +420,9 @@ typedef struct SpaceImaSel {
 #define FILE_STRINGCODE		2
 #define FILE_LINK			4
 #define FILE_HIDE_DOT		8
+#define FILE_AUTOSELECT		16
+#define FILE_ACTIVELAY		32
+#define FILE_ATCURSOR		64
 
 /* sfile->sort */
 #define FILE_SORTALPHA		0
@@ -435,6 +463,10 @@ typedef struct SpaceImaSel {
 #define ST_SCROLL_SELECT        0x0001 // scrollable
 #define ST_CLEAR_NAMESPACE      0x0010 // clear namespace after script
                                        // execution (see BPY_main.c)
+/* SpaceOops->type */
+#define SO_OOPS			0
+#define SO_OUTLINER		1
+#define SO_DEPSGRAPH    2
 
 /* SpaceOops->flag */
 #define SO_TESTBLOCKS	1
@@ -455,6 +487,17 @@ typedef struct SpaceImaSel {
 #define OOPS_LI		2048
 #define OOPS_IM		4096
 
+/* SpaceOops->outlinevis */
+#define SO_ALL_SCENES	0
+#define SO_CUR_SCENE	1
+#define SO_VISIBLE		2
+#define SO_SELECTED		3
+#define SO_ACTIVE		4
+#define SO_SAME_TYPE	5
+
+/* SpaceOops->storeflag */
+#define SO_TREESTORE_CLEANUP	1
+
 /* headerbuttons: 450-499 */
 
 #define B_IMASELHOME		451
@@ -491,5 +534,16 @@ typedef struct SpaceImaSel {
 #define IMS_INFILE			3
 #define IMS_INFILESLI		4
 
-#endif
+/* time->flag */
+#define TIME_DRAWFRAMES		1
+#define TIME_CFRA_NUM		2
 
+/* time->redraws */
+#define TIME_LEFTMOST_3D_WIN	1
+#define TIME_ALL_3D_WIN			2
+#define TIME_ALL_ANIM_WIN		4
+#define TIME_ALL_BUTS_WIN		8
+#define TIME_WITH_SEQ_AUDIO		16
+
+
+#endif
