@@ -1,5 +1,5 @@
 /**
- * $Id: makesdna.c,v 1.8 2004/03/20 22:55:38 zuster Exp $
+ * $Id: makesdna.c,v 1.14 2005/05/02 13:28:12 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -49,7 +49,7 @@
  * numbers give more output.
  * */
 
-#define DNA_VERSION_DATE "$Id: makesdna.c,v 1.8 2004/03/20 22:55:38 zuster Exp $"
+#define DNA_VERSION_DATE "$Id: makesdna.c,v 1.14 2005/05/02 13:28:12 ton Exp $"
 
 #include <string.h>
 #include <stdlib.h>
@@ -95,6 +95,7 @@ char *includefiles[] = {
 	"DNA_meshdata_types.h",
 	"DNA_lattice_types.h",	
 	"DNA_object_types.h",
+	"DNA_object_force.h",
 	"DNA_world_types.h",
 	"DNA_radio_types.h",
 	"DNA_scene_types.h",
@@ -464,7 +465,7 @@ int preprocess_include(char *maindata, int len)
 	return newlen;
 }
 
-void *read_file_data(char *filename, int *len_r)
+static void *read_file_data(char *filename, int *len_r)
 {
 #ifdef WIN32
 	FILE *fp= fopen(filename, "rb");
@@ -718,10 +719,12 @@ int calculate_structlens(void)
 						
 						/* 2-4 aligned/ */
 						if(typelens[type]>3 && (len % 4) ) {
-							printf("Align 4 error in struct: %s %s\n", types[structtype], cp);
+							printf("Align 4 error in struct: %s %s (add %d padding bytes)\n", types[structtype], cp, len%4);
+							dna_error = 1;
 						}
 						else if(typelens[type]==2 && (len % 2) ) {
-							printf("Align 2 error in struct: %s %s\n", types[structtype], cp);
+							printf("Align 2 error in struct: %s %s (add %d padding bytes)\n", types[structtype], cp, len%2);
+							dna_error = 1;
 						}
 
 						len += mul*typelens[type];
@@ -743,10 +746,16 @@ int calculate_structlens(void)
 					// has_pointer is set or alphalen != len
 					if (has_pointer || alphalen != len) {
 						if (alphalen % 8) {
-							printf("Sizeerror in struct: %s\n", types[structtype]);
+							printf("Sizeerror in struct: %s (add %d bytes)\n", types[structtype], alphalen%8);
 							dna_error = 1;
 						}
 					}
+					
+					if(len % 4) {
+						printf("Sizeerror in struct: %s (add %d bytes)\n", types[structtype], len%4);
+						dna_error = 1;
+					}
+					
 				}
 			}
 		}
@@ -1039,7 +1048,7 @@ int make_structDNA(FILE *file)
 
 /* ************************* END MAKE DNA ********************** */
 
-void make_bad_file(char *file)
+static void make_bad_file(char *file)
 {
 	FILE *fp= fopen(file, "w");
 	fprintf(fp, "ERROR! Cannot make correct DNA.c file, STUPID!\n");
@@ -1101,6 +1110,7 @@ int main(int argc, char ** argv)
 #include "DNA_meshdata_types.h"
 #include "DNA_lattice_types.h"	
 #include "DNA_object_types.h"
+#include "DNA_object_force.h"
 #include "DNA_world_types.h"
 #include "DNA_radio_types.h"
 #include "DNA_scene_types.h"
