@@ -1,5 +1,5 @@
 /* 
- * $Id: BPY_interface.c,v 1.59 2004/11/27 18:32:25 ianwill Exp $
+ * $Id: BPY_interface.c,v 1.71 2005/06/11 05:30:14 ianwill Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,6 +43,7 @@
 #include <MEM_guardedalloc.h>
 #include <BLI_blenlib.h>	/* for BLI_last_slash() */
 
+#include "BIF_gl.h" /* glPushAttrib, glPopAttrib for DRAW space handlers */
 #include <BIF_interface.h>	/* for pupmenu() */
 #include <BIF_space.h>
 #include <BIF_screen.h>
@@ -73,6 +74,7 @@
 #include "api2_2x/EXPP_interface.h"
 #include "api2_2x/constant.h"
 
+#include "api2_2x/gen_utils.h"
 #include "api2_2x/modules.h"
 
 /* bpy_registryDict is declared in api2_2x/Registry.h and defined
@@ -109,6 +111,13 @@ PyObject *importText( char *name );
 void init_ourImport( void );
 PyObject *blender_import( PyObject * self, PyObject * args );
 
+int BPY_txt_do_python_Text( struct Text *text );
+void BPY_Err_Handle( char *script_name );
+PyObject *traceback_getFilename( PyObject * tb );
+
+void BPY_free_screen_spacehandlers(struct bScreen *sc);
+
+
 /****************************************************************************
 * Description: This function will initialise Python and all the implemented 
 *	          api variations.
@@ -142,8 +151,8 @@ void BPY_start_python( int argc, char **argv )
 	 * rest of our init msgs.
 	 */
 
-	if( first_time ) { /* so it only prints msg on first_time */
-	}
+	/* Py_GetVersion() returns a ptr to astatic string */
+	printf( "Using Python version %.3s\n", Py_GetVersion() );
 
 	Py_Initialize(  );
 	PySys_SetArgv( argc_copy, argv_copy );
@@ -206,7 +215,7 @@ void init_syspath( int first_time )
 	PyObject *mod, *d;
 	PyObject *p;
 	char *c, *progname;
-	char execdir[FILE_MAXDIR + FILE_MAXFILE];	/*defines from DNA_space_types.h */
+	char execdir[FILE_MAXDIR];	/*defines from DNA_space_types.h */
 
 	int n;
 
@@ -304,26 +313,43 @@ that dir info is available.
 ****************************************************************************/
 void BPY_post_start_python( void )
 {
-	PyObject *result, *dict;
+	char dirpath[FILE_MAXDIR];
+	char *sdir = NULL;
 
-	if( U.pythondir && U.pythondir[0] != '\0' )
-		syspath_append( U.pythondir );	/* append to module search path */
+	if(U.pythondir[0] != '\0' ) {
+		char modpath[FILE_MAXDIR];
+		int upyslen = strlen(U.pythondir);
+
+		/* check if user pydir ends with a slash and, if so, remove the slash
+		 * (for eventual implementations of c library's stat function that might
+		 * not like it) */
+		if (upyslen > 2) { /* avoids doing anything if dir == '//' */
+			char ending = U.pythondir[upyslen - 1];
+
+			if (ending == '/' || ending == '\\')
+				U.pythondir[upyslen - 1] = '\0';
+		}
+
+		BLI_strncpy(dirpath, U.pythondir, FILE_MAXDIR);
+		BLI_convertstringcode(dirpath, G.sce, 0);
+		syspath_append(dirpath);	/* append to module search path */
+
+		BLI_make_file_string("/", modpath, dirpath, "bpymodules");
+		if (BLI_exists(modpath)) syspath_append(modpath);
+	}
+
+	sdir = bpy_gethome(1);
+	if (sdir) {
+
+		syspath_append(sdir);
+
+		BLI_make_file_string("/", dirpath, sdir, "bpymodules");
+		if (BLI_exists(dirpath)) syspath_append(dirpath);
+	}
 
 	BPyMenu_Init( 0 );	/* get dynamic menus (registered scripts) data */
 
-	dict = PyDict_New(  );
-
-	/* here we check if the user has (some of) the expected modules */
-	if( dict ) {
-		char *s = "import chunk, gzip, math, os, struct, string";
-		result = PyRun_String( s, Py_eval_input, dict, dict );
-		if( !result ) {
-			PyErr_Clear(  );
-			/*XXX print msg about this, point to readme.html */
-		} else
-			Py_DECREF( result );
-		Py_DECREF( dict );
-	}
+	return;
 }
 
 /****************************************************************************
@@ -348,17 +374,22 @@ const char *BPY_Err_getFilename( void )
 /*****************************************************************************/
 PyObject *traceback_getFilename( PyObject * tb )
 {
-	PyObject *v;
+	PyObject *v = NULL;
 
 /* co_filename is in f_code, which is in tb_frame, which is in tb */
 
 	v = PyObject_GetAttrString( tb, "tb_frame" );
-	Py_XDECREF( v );
-	v = PyObject_GetAttrString( v, "f_code" );
-	Py_XDECREF( v );
-	v = PyObject_GetAttrString( v, "co_filename" );
+	if (v) {
+		Py_DECREF( v );
+		v = PyObject_GetAttrString( v, "f_code" );
+		if (v) {
+			Py_DECREF( v );
+			v = PyObject_GetAttrString( v, "co_filename" );
+		}
+	}
 
-	return v;
+	if (v) return v;
+	else return PyString_FromString("unknown");
 }
 
 /****************************************************************************
@@ -421,11 +452,8 @@ void BPY_Err_Handle( char *script_name )
 		while( 1 ) {
 			v = PyObject_GetAttrString( tb, "tb_next" );
 
-			if( v == Py_None
-			    ||
-			    strcmp( PyString_AsString
-				    ( traceback_getFilename( v ) ),
-				    script_name ) ) {
+			if( !v || v == Py_None ||
+				strcmp(PyString_AsString(traceback_getFilename(v)), script_name)) {
 				break;
 			}
 
@@ -434,12 +462,16 @@ void BPY_Err_Handle( char *script_name )
 		}
 
 		v = PyObject_GetAttrString( tb, "tb_lineno" );
-		g_script_error.lineno = PyInt_AsLong( v );
-		Py_XDECREF( v );
+		if (v) {
+			g_script_error.lineno = PyInt_AsLong(v);
+			Py_DECREF(v);
+		}
 		v = traceback_getFilename( tb );
-		strncpy( g_script_error.filename, PyString_AsString( v ),
-			 FILENAME_LENGTH );
-		Py_XDECREF( v );
+		if (v) {
+			strncpy( g_script_error.filename, PyString_AsString( v ),
+				FILENAME_LENGTH );
+			Py_DECREF(v);
+		}
 		Py_DECREF( tb );
 	}
 
@@ -495,6 +527,7 @@ int BPY_txt_do_python_Text( struct Text *text )
 	script->py_draw = NULL;
 	script->py_event = NULL;
 	script->py_button = NULL;
+	script->py_browsercallback = NULL;
 
 	py_dict = CreateGlobalDictionary(  );
 
@@ -534,17 +567,6 @@ int BPY_txt_do_python_Text( struct Text *text )
 	return 1;		/* normal return */
 }
 
-/****************************************************************************
-* Description: The original function of this name has been refactored	   
-* into BPY_txt_do_python_Text.  That version is needed for the command	
-* line support for Python.  This is here to keep the interface the 
-* same and reduce code changes elsewhere. 
-****************************************************************************/
-int BPY_txt_do_python( struct SpaceText *st )
-{
-	return BPY_txt_do_python_Text( st->text );
-}
-
 /****************************************************************************
 * Description: Called from command line to run a Python script
 * automatically. 
@@ -552,39 +574,42 @@ int BPY_txt_do_python( struct SpaceText *st )
 void BPY_run_python_script( char *fn )
 {
 	Text *text = NULL;
-	int is_blenText = 0;
+	int is_blender_text = 0;
 
-	if( !BLI_exists( fn ) ) {	/* if there's no such filename ... */
+	if (!BLI_exists(fn)) {	/* if there's no such filename ... */
 		text = G.main->text.first;	/* try an already existing Blender Text */
-		while( text ) {
-			if( !strcmp( fn, text->id.name + 2 ) )
-				break;
+
+		while (text) {
+			if (!strcmp(fn, text->id.name + 2)) break;
 			text = text->id.next;
 		}
 
-		if( !text ) {
-			printf( "\nError: no such file or Blender text -- %s.\n", fn );
+		if (text == NULL) {
+			printf("\nError: no such file or Blender text -- %s.\n", fn);
 			return;
-		} else
-			is_blenText = 1;	/* fn is already a Blender Text */
+		}
+		else is_blender_text = 1;	/* fn is already a Blender Text */
 	}
 
-	if( !is_blenText )
-		text = add_text( fn );
-	if( text == NULL ) {
-		printf( "Error in BPY_run_python_script: couldn't create Blender text " "from %s\n", fn );
-		// Chris: On Windows if I continue I just get a segmentation
-		// violation.  To get a baseline file I exit here.
-		exit( 2 );
+	else {
+		text = add_text(fn);
+
+		if (text == NULL) {
+			printf("\nError in BPY_run_python_script:\n"
+				"couldn't create Blender text from %s\n", fn);
+		/* Chris: On Windows if I continue I just get a segmentation
+		 * violation.  To get a baseline file I exit here. */
+		exit(2);
+		/* return; */
+		}
 	}
 
-	if( BPY_txt_do_python_Text( text ) != 1 ) {
-		printf( "\nError executing Python script from command-line:\n"
-			"%s (at line %d).\n", fn, BPY_Err_getLinenumber(  ) );
+	if (BPY_txt_do_python_Text(text) != 1) {
+		printf("\nError executing Python script from command-line:\n"
+			"%s (at line %d).\n", fn, BPY_Err_getLinenumber());
 	}
 
-	if( !is_blenText )
-		free_libblock( &G.main->text, text );
+	if (!is_blender_text) free_libblock(&G.main->text, text);
 }
 
 /****************************************************************************
@@ -603,7 +628,6 @@ int BPY_menu_do_python( short menutype, int event )
 	FILE *fp = NULL;
 	char *buffer, *s;
 	char filestr[FILE_MAXDIR + FILE_MAXFILE];
-	char dirname[FILE_MAXDIR];
 	char scriptname[21];
 	Script *script = NULL;
 	int len;
@@ -640,18 +664,28 @@ int BPY_menu_do_python( short menutype, int event )
 		}
 	}
 
-	if( !pyarg ) {		/* no submenus */
+	if( !pyarg ) { /* no submenus */
 		Py_INCREF( Py_None );
 		pyarg = Py_None;
 	}
 
-	if( pym->dir )		/* script is in U.pythondir */
-		BLI_make_file_string( "/", filestr, U.pythondir,
-				      pym->filename );
-	else {			/* script is in ~/.blender/scripts/ */
-		BLI_make_file_string( "/", dirname, bpy_gethome(  ),
-				      "scripts" );
-		BLI_make_file_string( "/", filestr, dirname, pym->filename );
+	if( pym->dir ) { /* script is in U.pythondir */
+		char upythondir[FILE_MAXDIR];
+
+		/* dirs in Blender can be "//", which has a special meaning */
+		BLI_strncpy(upythondir, U.pythondir, FILE_MAXDIR);
+		BLI_convertstringcode(upythondir, G.sce, 0); /* if so, this expands it */
+		BLI_make_file_string( "/", filestr, upythondir, pym->filename );
+	}
+	else { /* script is in default scripts dir */
+		char *scriptsdir = bpy_gethome(1);
+
+		if (!scriptsdir) {
+			printf("Error loading script: can't find default scripts dir!");
+			return 0;
+		}
+
+		BLI_make_file_string( "/", filestr, scriptsdir, pym->filename );
 	}
 
 	fp = fopen( filestr, "rb" );
@@ -684,9 +718,10 @@ int BPY_menu_do_python( short menutype, int event )
 	 * each group will be put to code this properly) */
 	switch ( menutype ) {
 
-	case PYMENU_IMPORT:	/* first 3 were handled in header_info.c */
+	case PYMENU_IMPORT:	/* first 4 were handled in header_info.c */
 	case PYMENU_EXPORT:
 	case PYMENU_HELP:
+	case PYMENU_RENDER:
 	case PYMENU_WIZARDS:
 		break;
 
@@ -722,6 +757,7 @@ int BPY_menu_do_python( short menutype, int event )
 	script->py_draw = NULL;
 	script->py_event = NULL;
 	script->py_button = NULL;
+	script->py_browsercallback = NULL;
 
 	py_dict = CreateGlobalDictionary(  );
 
@@ -783,8 +819,8 @@ int BPY_menu_do_python( short menutype, int event )
 	if( !py_res ) {		/* Failed execution of the script */
 
 		BPY_Err_Handle( script->id.name + 2 );
-		PyErr_Print(  );
 		ReleaseGlobalDictionary( py_dict );
+		script->py_globaldict = NULL;
 		if( G.main->script.first )
 			free_libblock( &G.main->script, script );
 		error( "Python script error: check console" );
@@ -844,7 +880,7 @@ void BPY_free_finished_script( Script * script )
 	return;
 }
 
-void unlink_script( Script * script )
+static void unlink_script( Script * script )
 {	/* copied from unlink_text in drawtext.c */
 	bScreen *scr;
 	ScrArea *area;
@@ -881,6 +917,7 @@ void BPY_clear_script( Script * script )
 	Py_XDECREF( ( PyObject * ) script->py_draw );
 	Py_XDECREF( ( PyObject * ) script->py_event );
 	Py_XDECREF( ( PyObject * ) script->py_button );
+	Py_XDECREF( ( PyObject * ) script->py_browsercallback );
 
 	dict = script->py_globaldict;
 
@@ -1025,14 +1062,15 @@ void BPY_do_pyscript( ID * id, short event )
 		disable_where_scriptlink( during_slink );
 
 		/* set globals in Blender module to identify scriptlink */
-		Py_INCREF( Py_True );
-		PyDict_SetItemString( g_blenderdict, "bylink", Py_True );
+		PyDict_SetItemString( g_blenderdict, "bylink", EXPP_incr_ret_True() );
 		PyDict_SetItemString( g_blenderdict, "link",
 				      ID_asPyObject( id ) );
 		PyDict_SetItemString( g_blenderdict, "event",
 				      PyString_FromString( event_to_name
 							   ( event ) ) );
 
+		if (event == SCRIPT_POSTRENDER) event = SCRIPT_RENDER;
+
 		for( index = 0; index < scriptlink->totscript; index++ ) {
 			if( ( scriptlink->flag[index] == event ) &&
 			    ( scriptlink->scripts[index] != NULL ) ) {
@@ -1066,8 +1104,7 @@ void BPY_do_pyscript( ID * id, short event )
 		/* cleanup bylink flag and clear link so PyObject
 		 * can be released 
 		 */
-		Py_INCREF( Py_False );
-		PyDict_SetItemString( g_blenderdict, "bylink", Py_False );
+		PyDict_SetItemString( g_blenderdict, "bylink", EXPP_incr_ret_False() );
 		Py_INCREF( Py_None );
 		PyDict_SetItemString( g_blenderdict, "link", Py_None );
 		PyDict_SetItemString( g_blenderdict, "event",
@@ -1075,6 +1112,249 @@ void BPY_do_pyscript( ID * id, short event )
 	}
 }
 
+/* SPACE HANDLERS */
+
+/* These are special script links that can be assigned to ScrArea's to
+ * (EVENT type) receive events sent to a given space (and use or ignore them) or
+ * (DRAW type) be called after the space is drawn, to draw anything on top of
+ * the space area. */
+
+/* How to add space handlers to other spaces:
+ * - add the space event defines to DNA_scriptlink_types.h, as done for
+ *   3d view: SPACEHANDLER_VIEW3D_EVENT, for example;
+ * - add the new defines to Blender.SpaceHandler dictionary in Blender.c;
+ * - check space.c for how to call the event handlers;
+ * - check drawview.c for how to call the draw handlers;
+ * - check header_view3d.c for how to add the "Space Handler Scripts" menu.
+ * Note: DRAW handlers should be called with 'event = 0', chech drawview.c */
+
+int BPY_has_spacehandler(Text *text, ScrArea *sa)
+{
+	ScriptLink *slink;
+	int index;
+
+	if (!sa || !text) return 0;
+
+	slink = &sa->scriptlink;
+
+	for (index = 0; index < slink->totscript; index++) {
+		if (slink->scripts[index] && (slink->scripts[index] == (ID *)text))
+			return 1;
+	}
+
+	return 0;	
+}
+
+int BPY_is_spacehandler(Text *text, char spacetype)
+{
+	TextLine *tline = text->lines.first;
+	unsigned short type = 0;
+
+	if (tline && (tline->len > 10)) {
+		char *line = tline->line;
+
+		/* Expected format: # SPACEHANDLER.SPACE.TYPE
+		 * Ex: # SPACEHANDLER.VIEW3D.DRAW
+		 * The actual checks are forgiving, so slight variations also work. */
+		if (line && line[0] == '#' && strstr(line, "HANDLER")) {
+			line++; /* skip '#' */
+
+			/* only done for 3D View right now, trivial to add for others: */
+			switch (spacetype) {
+				case SPACE_VIEW3D:
+					if (strstr(line, "3D")) { /* VIEW3D, 3DVIEW */
+						if (strstr(line, "DRAW")) type = SPACEHANDLER_VIEW3D_DRAW;
+						else if (strstr(line, "EVENT")) type = SPACEHANDLER_VIEW3D_EVENT;
+					}
+					break;
+			}
+		}
+	}
+	return type; /* 0 if not a space handler */
+}
+
+int BPY_del_spacehandler(Text *text, ScrArea *sa)
+{
+	ScriptLink *slink;
+	int i, j;
+
+	if (!sa || !text) return -1;
+
+	slink = &sa->scriptlink;
+	if (slink->totscript < 1) return -1;
+
+	for (i = 0; i < slink->totscript; i++) {
+		if (text == (Text *)slink->scripts[i]) {
+
+			for (j = i; j < slink->totscript - 1; j++) {
+				slink->flag[j] = slink->flag[j+1];
+				slink->scripts[j] = slink->scripts[j+1];
+			}
+			slink->totscript--;
+			/* like done in buttons_script.c we just free memory
+			 * if all slinks have been removed -- less fragmentation,
+			 * these should be quite small arrays */
+			if (slink->totscript == 0) {
+				if (slink->scripts) MEM_freeN(slink->scripts);
+				if (slink->flag) MEM_freeN(slink->flag);
+				break;
+			}
+		}
+	}
+	return 0;
+}
+
+int BPY_add_spacehandler(Text *text, ScrArea *sa, char spacetype)
+{
+	unsigned short handlertype;
+
+	if (!sa || !text) return -1;
+
+	handlertype = BPY_is_spacehandler(text, spacetype);
+
+	if (handlertype) {
+		ScriptLink *slink = &sa->scriptlink;
+		void *stmp, *ftmp;
+		unsigned short space_event = SPACEHANDLER_VIEW3D_EVENT;
+
+		/* extend slink */
+
+		stmp= slink->scripts;		
+		slink->scripts= MEM_mallocN(sizeof(ID*)*(slink->totscript+1),
+			"spacehandlerscripts");
+	
+		ftmp= slink->flag;		
+		slink->flag= MEM_mallocN(sizeof(short*)*(slink->totscript+1),
+			"spacehandlerflags");
+	
+		if (slink->totscript) {
+			memcpy(slink->scripts, stmp, sizeof(ID*)*(slink->totscript));
+			MEM_freeN(stmp);
+
+			memcpy(slink->flag, ftmp, sizeof(short)*(slink->totscript));
+			MEM_freeN(ftmp);
+		}
+
+		switch (spacetype) {
+			case SPACE_VIEW3D:
+				if (handlertype == 1) space_event = SPACEHANDLER_VIEW3D_EVENT;
+				else space_event = SPACEHANDLER_VIEW3D_DRAW;
+				break;
+			default:
+				break;
+		}
+
+		slink->scripts[slink->totscript] = (ID *)text;
+		slink->flag[slink->totscript]= space_event;
+
+		slink->totscript++;
+		slink->actscript = slink->totscript;
+
+	}
+	return 0;
+}
+
+int BPY_do_spacehandlers( ScrArea *sa, unsigned short event,
+	unsigned short space_event )
+{
+	ScriptLink *scriptlink;
+	int retval = 0;
+
+	if (!sa) return 0;
+
+	scriptlink = &sa->scriptlink;
+
+	if (scriptlink->totscript > 0) {
+		PyObject *dict;
+		PyObject *ret;
+		int index, during_slink = during_scriptlink();
+
+		/* invalid scriptlinks (new .blend was just loaded), return */
+		if (during_slink < 0) return 0;
+
+		/* tell we're running a scriptlink.  The sum also tells if this script
+		 * is running nested inside another.  Blender.Load needs this info to
+		 * avoid trouble with invalid slink pointers. */
+		during_slink++;
+		disable_where_scriptlink( during_slink );
+
+		/* set globals in Blender module to identify space handler scriptlink */
+		PyDict_SetItemString(g_blenderdict, "bylink", EXPP_incr_ret_True());
+		/* unlike normal scriptlinks, here Blender.link is int (space event type) */
+		PyDict_SetItemString(g_blenderdict, "link", PyInt_FromLong(space_event));
+		/* note: DRAW space_events set event to 0 */
+		PyDict_SetItemString(g_blenderdict, "event", PyInt_FromLong(event));
+
+		/* now run all assigned space handlers for this space and space_event */
+		for( index = 0; index < scriptlink->totscript; index++ ) {
+
+			/* for DRAW handlers: */
+			if (event == 0) {
+				glPushAttrib(GL_ALL_ATTRIB_BITS);
+				glMatrixMode(GL_PROJECTION);
+				glPushMatrix();
+				glMatrixMode(GL_MODELVIEW);
+				glPushMatrix();
+			}
+
+			if( ( scriptlink->flag[index] == space_event ) &&
+			    ( scriptlink->scripts[index] != NULL ) ) {
+				dict = CreateGlobalDictionary();
+				ret = RunPython( ( Text * ) scriptlink->scripts[index], dict );
+				ReleaseGlobalDictionary( dict );
+
+				if (!ret) { /* Failed execution of the script */
+					BPY_Err_Handle( scriptlink->scripts[index]->name+2 );
+				} else {
+					Py_DECREF(ret);
+
+					/* an EVENT type (event != 0) script can either accept an event or
+					 * ignore it:
+					 * if the script sets Blender.event to None it accepted it;
+					 * otherwise the space's event handling callback that called us
+					 * can go on processing the event */
+					if (event && (PyDict_GetItemString(g_blenderdict,"event") == Py_None))
+						retval = 1; /* event was swallowed */
+				}
+
+				/* If a scriptlink has just loaded a new .blend file, the
+				 * scriptlink pointer became invalid (see api2_2x/Blender.c),
+				 * so we stop here. */
+				if( during_scriptlink(  ) == -1 ) {
+					during_slink = 1;
+					if (event == 0) glPopAttrib();
+					break;
+				}
+			}
+
+			/* for DRAW handlers: */
+			if (event == 0) {
+				glMatrixMode(GL_PROJECTION);
+				glPopMatrix();
+				glMatrixMode(GL_MODELVIEW);
+				glPopMatrix();
+				glPopAttrib();
+			}
+
+		}
+
+		disable_where_scriptlink( during_slink - 1 );
+
+		PyDict_SetItemString(g_blenderdict, "bylink", EXPP_incr_ret_False());
+		PyDict_SetItemString(g_blenderdict, "link", EXPP_incr_ret(Py_None));
+		PyDict_SetItemString(g_blenderdict, "event", PyString_FromString(""));
+	}
+	
+	/* retval:
+	 * space_event is of type EVENT:
+	 * 0 - event was returned,
+	 * 1 - event was processed;
+	 * space_event is of type DRAW:
+	 * 0 always */
+
+	return retval;
+}
+
 /*****************************************************************************
 * Description:	
 * Notes:
@@ -1091,7 +1371,32 @@ void BPY_free_scriptlink( struct ScriptLink *slink )
 	return;
 }
 
-int CheckAllScriptsFromList( ListBase * list, Text * text )
+void BPY_free_screen_spacehandlers(struct bScreen *sc)
+{
+	ScrArea *sa;
+
+	for (sa = sc->areabase.first; sa; sa = sa->next)
+		BPY_free_scriptlink(&sa->scriptlink);
+}
+
+static int CheckAllSpaceHandlers(Text *text)
+{
+	bScreen *screen;
+	ScrArea *sa;
+	ScriptLink *slink;
+	int fixed = 0;
+
+	for (screen = G.main->screen.first; screen; screen = screen->id.next) {
+		for (sa = screen->areabase.first; sa; sa = sa->next) {
+			slink = &sa->scriptlink;
+			if (!slink->totscript) continue;
+			if (BPY_del_spacehandler(text, sa) == 0) fixed++;
+		}
+	}
+	return fixed;
+}
+
+static int CheckAllScriptsFromList( ListBase * list, Text * text )
 {
 	ID *id;
 	ScriptLink *scriptlink;
@@ -1126,11 +1431,11 @@ int BPY_check_all_scriptlinks( Text * text )
 	fixed += CheckAllScriptsFromList( &( G.main->mat ), text );
 	fixed += CheckAllScriptsFromList( &( G.main->world ), text );
 	fixed += CheckAllScriptsFromList( &( G.main->scene ), text );
+	fixed += CheckAllSpaceHandlers(text);
 
 	return fixed;
 }
 
-
 /*****************************************************************************
 * Description: 
 * Notes:
@@ -1347,40 +1652,3 @@ void init_ourImport( void )
 	d = PyModule_GetDict( m );
 	PyDict_SetItemString( d, "__import__", import );
 }
-
-/* this makes sure BLI_gethome() returns a path with '.blender' appended
- * Besides, this function now either returns userhome/.blender (if it exists)
- * or blenderInstallDir/.blender/ otherwise 
-*/
-char *bpy_gethome(  )
-{
-	static char homedir[FILE_MAXDIR];
-	char bprogdir[FILE_MAXDIR];
-	char *s;
-	int i;
-
-	if( homedir[0] != '\0' )
-		return homedir;	/* no need to search twice */
-
-	s = BLI_gethome(  );
-
-	if( strstr( s, ".blender" ) )
-		PyOS_snprintf( homedir, FILE_MAXDIR, s );
-	else
-		BLI_make_file_string( "/", homedir, s, ".blender/" );
-
-	/* if userhome/.blender/ exists, return it */
-	if( BLI_exists( homedir ) )
-		return homedir;
-
-	/* otherwise, use argv[0] (bprogname) to get .blender/ in
-	 * Blender's installation dir */
-	s = BLI_last_slash( bprogname );
-
-	i = s - bprogname + 1;
-
-	PyOS_snprintf( bprogdir, i, bprogname );
-	BLI_make_file_string( "/", homedir, bprogdir, ".blender/" );
-
-	return homedir;
-}
