@@ -1,4 +1,5 @@
 /* 
+ * $Id: Ipocurve.h,v 1.9 2005/04/24 19:03:38 stiv Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -38,12 +39,18 @@
 /*****************************************************************************/
 /* Python C_IpoCurve structure definition:                                   */
 /*****************************************************************************/
-typedef struct
-{
-  PyObject_HEAD			/* required macro */
-  IpoCurve * ipocurve;
-}
-C_IpoCurve;
+typedef struct {
+	PyObject_HEAD		/* required macro */
+	IpoCurve * ipocurve;
+} C_IpoCurve;
 
 
-#endif /* EXPP_IPOCURVE_H */
+
+PyObject *IpoCurve_Init( void );
+PyObject *IpoCurve_CreatePyObject( IpoCurve * ipo );
+int IpoCurve_CheckPyObject( PyObject * pyobj );
+IpoCurve *IpoCurve_FromPyObject( PyObject * pyobj );
+char *getIpoCurveName( IpoCurve * icu );
+
+
+#endif				/* EXPP_IPOCURVE_H */
