@@ -1,5 +1,5 @@
 /* 
- * $Id: Lattice.c,v 1.6 2004/10/07 19:25:39 stiv Exp $
+ * $Id: Lattice.c,v 1.10 2005/05/22 07:22:33 ianwill Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -96,7 +96,7 @@ static PyObject *Lattice_setMode( BPy_Lattice * self, PyObject * args );
 static PyObject *Lattice_getMode( BPy_Lattice * self, PyObject * args );
 static PyObject *Lattice_setPoint( BPy_Lattice * self, PyObject * args );
 static PyObject *Lattice_getPoint( BPy_Lattice * self, PyObject * args );
-static PyObject *Lattice_applyDeform( BPy_Lattice * self );
+static PyObject *Lattice_applyDeform( BPy_Lattice * self, PyObject *args );
 static PyObject *Lattice_insertKey( BPy_Lattice * self, PyObject * args );
 
 /*****************************************************************************/
@@ -132,7 +132,12 @@ static char Lattice_getPoint_doc[] =
 	"(str) - Get the coordinates of a point on the lattice";
 
 static char Lattice_applyDeform_doc[] =
-	"(str) - Apply the new lattice deformation to children";
+	"(force = False) - Apply the new lattice deformation to children\n\n\
+(force = False) - if given and True, children of mesh type are not ignored.\n\
+Meshes are treated differently in Blender, deformation is stored directly in\n\
+their vertices when first redrawn (ex: with Blender.Redraw) after getting a\n\
+Lattice parent, without needing this method (except for command line bg\n\
+mode). If forced, the deformation will be applied over any previous one(s).";
 
 static char Lattice_insertKey_doc[] =
 	"(str) - Set a new key for the lattice at specified frame";
@@ -162,7 +167,7 @@ static PyMethodDef BPy_Lattice_methods[] = {
 	 Lattice_setPoint_doc},
 	{"getPoint", ( PyCFunction ) Lattice_getPoint, METH_VARARGS,
 	 Lattice_getPoint_doc},
-	{"applyDeform", ( PyCFunction ) Lattice_applyDeform, METH_NOARGS,
+	{"applyDeform", ( PyCFunction ) Lattice_applyDeform, METH_VARARGS,
 	 Lattice_applyDeform_doc},
 	{"insertKey", ( PyCFunction ) Lattice_insertKey, METH_VARARGS,
 	 Lattice_insertKey_doc},
@@ -202,6 +207,7 @@ PyTypeObject Lattice_Type = {
 	0, 0, 0, 0, 0, 0,
 	BPy_Lattice_methods,	/* tp_methods */
 	0,			/* tp_members */
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 };
 
 static int Lattice_InLatList( BPy_Lattice * self );
@@ -674,22 +680,35 @@ static PyObject *Lattice_getPoint( BPy_Lattice * self, PyObject * args )
 }
 
 //This function will not do anything if there are no children
-static PyObject *Lattice_applyDeform( BPy_Lattice * self )
+static PyObject *Lattice_applyDeform( BPy_Lattice * self, PyObject *args )
 {
 	//Object* ob; unused
 	Base *base;
 	Object *par;
+	int forced = 0;
 
 	if( !Lattice_IsLinkedToObject( self ) )
 		return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
 						"Lattice must be linked to an object to apply it's deformation!" ) );
 
-	//deform children
+	if( !PyArg_ParseTuple( args, "|i", &forced ) )
+		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
+						"expected nothing or True or False argument" ) );
+
+	/* deform children */
 	base = FIRSTBASE;
 	while( base ) {
-		if( ( par = base->object->parent ) ) {
-			if( par->type == OB_LATTICE ) {
-				object_deform( base->object );
+		if( ( par = base->object->parent ) ) { /* check/assign if ob has parent */
+			/* meshes have their mverts deformed, others ob types use displist,
+			 * so we're not doing meshes here (unless forced), or else they get
+			 * deformed twice, since parenting a Lattice to an object and redrawing
+			 * already applies lattice deformation. 'forced' is useful for
+			 * command line background mode, when no redraws occur and so this
+			 * method is needed.  Or for users who actually want to apply the
+			 * deformation n times. */
+			if((self->Lattice == par->data)) {
+				if ((base->object->type != OB_MESH) || forced)
+					object_deform( base->object );
 			}
 		}
 		base = base->next;
@@ -712,7 +731,7 @@ static PyObject *Lattice_insertKey( BPy_Lattice * self, PyObject * args )
 
 	//set the current frame
 	if( frame > 0 ) {
-		frame = EXPP_ClampInt( frame, 1, 18000 );
+		frame = EXPP_ClampInt( frame, 1, MAXFRAME );
 		oldfra = G.scene->r.cfra;
 		G.scene->r.cfra = frame;
 	}
@@ -804,12 +823,15 @@ static PyObject *Lattice_getAttr( BPy_Lattice * self, char *name )
 		attr = Py_BuildValue( "i", self->Lattice->pntsu *
 				      self->Lattice->pntsv *
 				      self->Lattice->pntsw );
+	} else if( strcmp( name, "users" ) == 0 ) {
+		attr = PyInt_FromLong( self->Lattice->id.us );
+	
 	} else if( strcmp( name, "__members__" ) == 0 )
 		attr = Py_BuildValue( "[s,s,s,s,s,s,s,s,s]", "name", "width",
 				      "height", "depth", "widthType",
 				      "heightType", "depthType", "mode",
-				      "latSize" );
-
+				      "latSize", "users" );
+	
 	if( !attr )
 		return ( EXPP_ReturnPyObjError( PyExc_MemoryError,
 						"couldn't create PyObject" ) );
