@@ -1,5 +1,5 @@
 /* 
- * $Id: Material.c,v 1.27 2004/12/04 21:49:01 ton Exp $
+ * $Id: Material.c,v 1.33 2005/05/27 05:39:40 ianwill Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -26,7 +26,7 @@
  * This is a new part of Blender.
  *
  * Contributor(s): Willian P. Germano, Michel Selten, Alex Mole,
- * Alexander Szakaly
+ * Alexander Szakaly, Campbell Barton, Ken Hughes
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
@@ -37,6 +37,9 @@
 #include <MEM_guardedalloc.h>
 #include <DNA_ID.h>
 #include <BLI_blenlib.h>
+#include <BSE_editipo.h>
+#include <BIF_space.h>
+#include <mydevice.h>
 
 #include "constant.h"
 #include "gen_utils.h"
@@ -47,6 +50,11 @@
 #include "Material.h"
 #include "Ipo.h"
 
+
+/* only used for .oopsLoc at the moment */
+#include "DNA_oops_types.h"
+#include "DNA_space_types.h"
+
 /*****************************************************************************/
 /* Python BPy_Material defaults: */
 /*****************************************************************************/
@@ -145,7 +153,15 @@
 #define EXPP_MAT_MIRRTRANSADD_MAX			1.0
 
 
-
+#define IPOKEY_RGB          0
+#define IPOKEY_ALPHA        1 
+#define IPOKEY_HALOSIZE     2 
+#define IPOKEY_MODE         3
+#define IPOKEY_ALLCOLOR     10
+#define IPOKEY_ALLMIRROR    14
+#define IPOKEY_OFS          12
+#define IPOKEY_SIZE         13
+#define IPOKEY_ALLMAPPING   11
 
 
 
@@ -157,6 +173,10 @@ static PyObject *M_Material_New( PyObject * self, PyObject * args,
 				 PyObject * keywords );
 static PyObject *M_Material_Get( PyObject * self, PyObject * args );
 
+/* Not exposed nor used */
+Material *GetMaterialByName( char *name );
+
+
 /*****************************************************************************/
 /* The following string definitions are used for documentation strings.	 */
 /* In Python these will be written to the console when doing a		 */
@@ -360,6 +380,17 @@ PyObject *Material_Init( void )
 
 	if( Modes )
 		PyModule_AddObject( submodule, "Modes", Modes );
+
+	PyModule_AddIntConstant( submodule, "RGB", IPOKEY_RGB );
+	PyModule_AddIntConstant( submodule, "ALPHA", IPOKEY_ALPHA );
+	PyModule_AddIntConstant( submodule, "HALOSIZE", IPOKEY_HALOSIZE );
+	PyModule_AddIntConstant( submodule, "MODE", IPOKEY_MODE );
+	PyModule_AddIntConstant( submodule, "ALLCOLOR", IPOKEY_ALLCOLOR );
+	PyModule_AddIntConstant( submodule, "ALLMIRROR", IPOKEY_ALLMIRROR );
+	PyModule_AddIntConstant( submodule, "OFS", IPOKEY_OFS );
+	PyModule_AddIntConstant( submodule, "SIZE", IPOKEY_SIZE );
+	PyModule_AddIntConstant( submodule, "ALLMAPPING", IPOKEY_ALLMAPPING );
+
 	return ( submodule );
 }
 
@@ -456,11 +487,12 @@ static PyObject *Material_clearTexture( BPy_Material * self, PyObject * args );
 static PyObject *Material_setColorComponent( BPy_Material * self, char *key,
 					     PyObject * args );
 
-static PyObject *Material_getScriptLinks( BPy_Material * self,
-					  PyObject * args );
-static PyObject *Material_addScriptLink( BPy_Material * self,
-					 PyObject * args );
-static PyObject *Material_clearScriptLinks( BPy_Material * self );
+static PyObject *Material_getScriptLinks(BPy_Material *self, PyObject * args );
+static PyObject *Material_addScriptLink(BPy_Material * self, PyObject * args );
+static PyObject *Material_clearScriptLinks(BPy_Material *self, PyObject *args);
+
+static PyObject *Material_insertIpoKey( BPy_Material * self, PyObject * args );
+
 
 /*****************************************************************************/
 /* Python BPy_Material methods table: */
@@ -548,6 +580,8 @@ static PyMethodDef BPy_Material_methods[] = {
 	 "(Blender Ipo) - Change Material's Ipo"},
 	{"clearIpo", ( PyCFunction ) Material_clearIpo, METH_NOARGS,
 	 "(Blender Ipo) - Unlink Ipo from this Material"},
+	{"insertIpoKey", ( PyCFunction ) Material_insertIpoKey, METH_VARARGS,
+	 "(Material Ipo Constant) - Insert IPO Key at current frame"},	 
 	{"setMode", ( PyCFunction ) Material_setMode, METH_VARARGS,
 	 "([s[,s]]) - Set Material's mode flag(s)"},
 	{"setRGBCol", ( PyCFunction ) Material_setRGBCol, METH_VARARGS,
@@ -626,14 +660,15 @@ static PyMethodDef BPy_Material_methods[] = {
 	 METH_VARARGS,
 	 "(eventname) - Get a list of this material's scriptlinks (Text names) "
 	 "of the given type\n"
-	 "(eventname) - string: FrameChanged or Redraw."},
+	 "(eventname) - string: FrameChanged, Redraw or Render."},
 	{"addScriptLink", ( PyCFunction ) Material_addScriptLink, METH_VARARGS,
 	 "(text, evt) - Add a new material scriptlink.\n"
 	 "(text) - string: an existing Blender Text name;\n"
-	 "(evt) string: FrameChanged or Redraw."},
+	 "(evt) string: FrameChanged, Redraw or Render."},
 	{"clearScriptLinks", ( PyCFunction ) Material_clearScriptLinks,
-	 METH_NOARGS,
-	 "() - Delete all scriptlinks from this material."},
+	 METH_VARARGS,
+	 "() - Delete all scriptlinks from this material.\n"
+	 "([s1<,s2,s3...>]) - Delete specified scriptlinks from this material."},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -670,6 +705,7 @@ PyTypeObject Material_Type = {
 	0, 0, 0, 0, 0, 0,
 	BPy_Material_methods,	/* tp_methods */
 	0,			/* tp_members */
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, /* up to tp_del to avoid a warning */
 };
 
 /*****************************************************************************/
@@ -1217,14 +1253,93 @@ static PyObject *Material_clearIpo( BPy_Material * self )
 			id->us--;
 		mat->ipo = NULL;
 
-		Py_INCREF( Py_True );
-		return Py_True;
+		return EXPP_incr_ret_True();
 	}
 
-	Py_INCREF( Py_False );	/* no ipo found */
-	return Py_False;
+	return EXPP_incr_ret_False(); /* no ipo found */
 }
 
+
+/* 
+ *  Material_insertIpoKey( key )
+ *   inserts Material IPO key at current frame
+ */
+
+static PyObject *Material_insertIpoKey( BPy_Material * self, PyObject * args )
+{
+    int key = 0, map;
+    
+	if( !PyArg_ParseTuple( args, "i", &( key ) ) )
+		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+						"expected int argument" ) ); 
+    				
+	map = texchannel_to_adrcode(self->material->texact);
+	
+	if(key==IPOKEY_RGB || key==IPOKEY_ALLCOLOR) {
+		insertkey((ID *)self->material, MA_COL_R);
+		insertkey((ID *)self->material, MA_COL_G);
+		insertkey((ID *)self->material, MA_COL_B);
+	}
+	if(key==IPOKEY_ALPHA || key==IPOKEY_ALLCOLOR) {
+		insertkey((ID *)self->material, MA_ALPHA);
+	}
+	if(key==IPOKEY_HALOSIZE || key==IPOKEY_ALLCOLOR) {
+		insertkey((ID *)self->material, MA_HASIZE);
+	}
+	if(key==IPOKEY_MODE || key==IPOKEY_ALLCOLOR) {
+		insertkey((ID *)self->material, MA_MODE);
+	}
+	if(key==IPOKEY_ALLCOLOR) {
+		insertkey((ID *)self->material, MA_SPEC_R);
+		insertkey((ID *)self->material, MA_SPEC_G);
+		insertkey((ID *)self->material, MA_SPEC_B);
+		insertkey((ID *)self->material, MA_REF);
+		insertkey((ID *)self->material, MA_EMIT);
+		insertkey((ID *)self->material, MA_AMB);
+		insertkey((ID *)self->material, MA_SPEC);
+		insertkey((ID *)self->material, MA_HARD);
+		insertkey((ID *)self->material, MA_MODE);
+		insertkey((ID *)self->material, MA_TRANSLU);
+		insertkey((ID *)self->material, MA_ADD);
+	}
+	if(key==IPOKEY_ALLMIRROR) {
+		insertkey((ID *)self->material, MA_RAYM);
+		insertkey((ID *)self->material, MA_FRESMIR);
+		insertkey((ID *)self->material, MA_FRESMIRI);
+		insertkey((ID *)self->material, MA_FRESTRA);
+		insertkey((ID *)self->material, MA_FRESTRAI);
+	}
+	if(key==IPOKEY_OFS || key==IPOKEY_ALLMAPPING) {
+		insertkey((ID *)self->material, map+MAP_OFS_X);
+		insertkey((ID *)self->material, map+MAP_OFS_Y);
+		insertkey((ID *)self->material, map+MAP_OFS_Z);
+	}
+	if(key==IPOKEY_SIZE || key==IPOKEY_ALLMAPPING) {
+		insertkey((ID *)self->material, map+MAP_SIZE_X);
+		insertkey((ID *)self->material, map+MAP_SIZE_Y);
+		insertkey((ID *)self->material, map+MAP_SIZE_Z);
+	}
+	if(key==IPOKEY_ALLMAPPING) {
+		insertkey((ID *)self->material, map+MAP_R);
+		insertkey((ID *)self->material, map+MAP_G);
+		insertkey((ID *)self->material, map+MAP_B);
+		insertkey((ID *)self->material, map+MAP_DVAR);
+		insertkey((ID *)self->material, map+MAP_COLF);
+		insertkey((ID *)self->material, map+MAP_NORF);
+		insertkey((ID *)self->material, map+MAP_VARF);
+		insertkey((ID *)self->material, map+MAP_DISP);
+	}
+
+	allspace(REMAKEIPO, 0);
+	EXPP_allqueue(REDRAWIPO, 0);
+	EXPP_allqueue(REDRAWVIEW3D, 0);
+	EXPP_allqueue(REDRAWACTION, 0);
+	EXPP_allqueue(REDRAWNLA, 0);
+
+	return  EXPP_incr_ret( Py_None );		
+}
+
+
 static PyObject *Material_setName( BPy_Material * self, PyObject * args )
 {
 	char *name;
@@ -1248,89 +1363,109 @@ static PyObject *Material_setName( BPy_Material * self, PyObject * args )
  * haloShaded, haloFlare */
 static PyObject *Material_setMode( BPy_Material * self, PyObject * args )
 {
-	int i, flag = 0;
+	unsigned int i, flag = 0, ok = 0;
+
 	char *m[28] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 		NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 		NULL, NULL, NULL, NULL, NULL, NULL
 	};
 
-	if( !PyArg_ParseTuple( args, "|ssssssssssssssssssssssssssss",
+	/* 
+	 * check for a single integer argument; do a quick check for now
+	 * that the value is not larger than double the highest flag bit
+	 */
+
+	if ( (PySequence_Size( args ) == 1)
+		    && PyInt_Check ( PySequence_Fast_GET_ITEM ( args , 0 ) )
+		    && PyArg_ParseTuple( args, "i", &flag ) 
+		    && flag < (EXPP_MAT_MODE_RAYMIRROR >> 1) ) {
+			ok = 1;
+
+	/*
+	 * check for either an empty argument list, or up to 22 strings
+	 */
+
+	} else if( PyArg_ParseTuple( args, "|ssssssssssssssssssssssssssss",
 			       &m[0], &m[1], &m[2], &m[3], &m[4], &m[5], &m[6],
 			       &m[7], &m[8], &m[9], &m[10], &m[11], &m[12],
 			       &m[13], &m[14], &m[15], &m[16], &m[17], &m[18],
 			       &m[19], &m[20], &m[21], &m[22], &m[23], &m[24],
 			       &m[25], &m[26], &m[27] ) ) {
-		return ( EXPP_ReturnPyObjError
-			 ( PyExc_AttributeError,
-			   "expected from none to 22 string argument(s)" ) );
+		for( i = 0; i < 28; i++ ) {
+			if( m[i] == NULL )
+				break;
+			if( strcmp( m[i], "Traceable" ) == 0 )
+				flag |= EXPP_MAT_MODE_TRACEABLE;
+			else if( strcmp( m[i], "Shadow" ) == 0 )
+				flag |= EXPP_MAT_MODE_SHADOW;
+			else if( strcmp( m[i], "Shadeless" ) == 0 )
+				flag |= EXPP_MAT_MODE_SHADELESS;
+			else if( strcmp( m[i], "Wire" ) == 0 )
+				flag |= EXPP_MAT_MODE_WIRE;
+			else if( strcmp( m[i], "VColLight" ) == 0 )
+				flag |= EXPP_MAT_MODE_VCOL_LIGHT;
+			else if( strcmp( m[i], "VColPaint" ) == 0 )
+				flag |= EXPP_MAT_MODE_VCOL_PAINT;
+			else if( strcmp( m[i], "Halo" ) == 0 )
+				flag |= EXPP_MAT_MODE_HALO;
+			else if( strcmp( m[i], "ZTransp" ) == 0 )
+				flag |= EXPP_MAT_MODE_ZTRANSP;
+			else if( strcmp( m[i], "ZInvert" ) == 0 )
+				flag |= EXPP_MAT_MODE_ZINVERT;
+			else if( strcmp( m[i], "HaloRings" ) == 0 )
+				flag |= EXPP_MAT_MODE_HALORINGS;
+			else if( strcmp( m[i], "HaloLines" ) == 0 )
+				flag |= EXPP_MAT_MODE_HALOLINES;
+			else if( strcmp( m[i], "OnlyShadow" ) == 0 )
+				flag |= EXPP_MAT_MODE_ONLYSHADOW;
+			else if( strcmp( m[i], "HaloXAlpha" ) == 0 )
+				flag |= EXPP_MAT_MODE_HALOXALPHA;
+			else if( strcmp( m[i], "HaloStar" ) == 0 )
+				flag |= EXPP_MAT_MODE_HALOSTAR;
+			else if( strcmp( m[i], "TexFace" ) == 0 )
+				flag |= EXPP_MAT_MODE_TEXFACE;
+			else if( strcmp( m[i], "HaloTex" ) == 0 )
+				flag |= EXPP_MAT_MODE_HALOTEX;
+			else if( strcmp( m[i], "HaloPuno" ) == 0 )
+				flag |= EXPP_MAT_MODE_HALOPUNO;
+			else if( strcmp( m[i], "NoMist" ) == 0 )
+				flag |= EXPP_MAT_MODE_NOMIST;
+			else if( strcmp( m[i], "HaloShaded" ) == 0 )
+				flag |= EXPP_MAT_MODE_HALOSHADE;
+			else if( strcmp( m[i], "HaloFlare" ) == 0 )
+				flag |= EXPP_MAT_MODE_HALOFLARE;
+			else if( strcmp( m[i], "Radio" ) == 0 )
+				flag |= EXPP_MAT_MODE_RADIO;
+			/* ** Mirror ** */
+			else if( strcmp( m[i], "RayMirr" ) == 0 )
+				flag |= EXPP_MAT_MODE_RAYMIRROR;
+			else if( strcmp( m[i], "ZTransp" ) == 0 )
+				flag |= EXPP_MAT_MODE_ZTRA;
+			else if( strcmp( m[i], "RayTransp" ) == 0 )
+				flag |= EXPP_MAT_MODE_RAYTRANSP;
+			else if( strcmp( m[i], "OnlyShadow" ) == 0 )
+				flag |= EXPP_MAT_MODE_ONLYSHADOW;
+			else if( strcmp( m[i], "NoMist" ) == 0 )
+				flag |= EXPP_MAT_MODE_NOMIST;
+			else if( strcmp( m[i], "Env" ) == 0 )
+				flag |= EXPP_MAT_MODE_ENV;
+			/* ** */
+			else
+				return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+								"unknown Material mode argument" ) );
+		}
+	    	ok = 1;
 	}
 
-	for( i = 0; i < 28; i++ ) {
-		if( m[i] == NULL )
-			break;
-		if( strcmp( m[i], "Traceable" ) == 0 )
-			flag |= EXPP_MAT_MODE_TRACEABLE;
-		else if( strcmp( m[i], "Shadow" ) == 0 )
-			flag |= EXPP_MAT_MODE_SHADOW;
-		else if( strcmp( m[i], "Shadeless" ) == 0 )
-			flag |= EXPP_MAT_MODE_SHADELESS;
-		else if( strcmp( m[i], "Wire" ) == 0 )
-			flag |= EXPP_MAT_MODE_WIRE;
-		else if( strcmp( m[i], "VColLight" ) == 0 )
-			flag |= EXPP_MAT_MODE_VCOL_LIGHT;
-		else if( strcmp( m[i], "VColPaint" ) == 0 )
-			flag |= EXPP_MAT_MODE_VCOL_PAINT;
-		else if( strcmp( m[i], "Halo" ) == 0 )
-			flag |= EXPP_MAT_MODE_HALO;
-		else if( strcmp( m[i], "ZTransp" ) == 0 )
-			flag |= EXPP_MAT_MODE_ZTRANSP;
-		else if( strcmp( m[i], "ZInvert" ) == 0 )
-			flag |= EXPP_MAT_MODE_ZINVERT;
-		else if( strcmp( m[i], "HaloRings" ) == 0 )
-			flag |= EXPP_MAT_MODE_HALORINGS;
-		else if( strcmp( m[i], "Env" ) == 0 )
-			flag |= EXPP_MAT_MODE_ENV;
-		else if( strcmp( m[i], "HaloLines" ) == 0 )
-			flag |= EXPP_MAT_MODE_HALOLINES;
-		else if( strcmp( m[i], "OnlyShadow" ) == 0 )
-			flag |= EXPP_MAT_MODE_ONLYSHADOW;
-		else if( strcmp( m[i], "HaloXAlpha" ) == 0 )
-			flag |= EXPP_MAT_MODE_HALOXALPHA;
-		else if( strcmp( m[i], "HaloStar" ) == 0 )
-			flag |= EXPP_MAT_MODE_HALOSTAR;
-		else if( strcmp( m[i], "TexFace" ) == 0 )
-			flag |= EXPP_MAT_MODE_TEXFACE;
-		else if( strcmp( m[i], "HaloTex" ) == 0 )
-			flag |= EXPP_MAT_MODE_HALOTEX;
-		else if( strcmp( m[i], "HaloPuno" ) == 0 )
-			flag |= EXPP_MAT_MODE_HALOPUNO;
-		else if( strcmp( m[i], "NoMist" ) == 0 )
-			flag |= EXPP_MAT_MODE_NOMIST;
-		else if( strcmp( m[i], "HaloShaded" ) == 0 )
-			flag |= EXPP_MAT_MODE_HALOSHADE;
-		else if( strcmp( m[i], "HaloFlare" ) == 0 )
-			flag |= EXPP_MAT_MODE_HALOFLARE;
-		else if( strcmp( m[i], "Radio" ) == 0 )
-			flag |= EXPP_MAT_MODE_RADIO;
-		/* ** Mirror ** */
-		else if( strcmp( m[i], "RayMirr" ) == 0 )
-			flag |= EXPP_MAT_MODE_RAYMIRROR;
-		else if( strcmp( m[i], "ZTransp" ) == 0 )
-			flag |= EXPP_MAT_MODE_ZTRA;
-		else if( strcmp( m[i], "RayTransp" ) == 0 )
-			flag |= EXPP_MAT_MODE_RAYTRANSP;
-		else if( strcmp( m[i], "OnlyShadow" ) == 0 )
-			flag |= EXPP_MAT_MODE_ONLYSHADOW;
-		else if( strcmp( m[i], "NoMist" ) == 0 )
-			flag |= EXPP_MAT_MODE_NOMIST;
-		else if( strcmp( m[i], "Env" ) == 0 )
-			flag |= EXPP_MAT_MODE_ENV;
-		/* ** */
-		else
-			return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
-							"unknown Material mode argument" ) );
-	}
+	/* if neither input method worked, then throw an exception */
+
+	if ( ok == 0 )
+		return ( EXPP_ReturnPyObjError
+			 ( PyExc_AttributeError,
+			   "expected nothing, an integer or up to 22 string argument(s)" ) );
+
+	/* update the mode flag, return None */
 
 	self->material->mode = flag;
 
@@ -1861,22 +1996,18 @@ static PyObject *Material_addScriptLink( BPy_Material * self, PyObject * args )
 
 	slink = &( mat )->scriptlink;
 
-	if( !EXPP_addScriptLink( slink, args, 0 ) )
-		return EXPP_incr_ret( Py_None );
-	else
-		return NULL;
+	return EXPP_addScriptLink( slink, args, 0 );
 }
 
 /* mat.clearScriptLinks */
-static PyObject *Material_clearScriptLinks( BPy_Material * self )
+static PyObject *Material_clearScriptLinks(BPy_Material *self, PyObject *args )
 {
 	Material *mat = self->material;
 	ScriptLink *slink = NULL;
 
 	slink = &( mat )->scriptlink;
 
-	return EXPP_incr_ret( Py_BuildValue
-			      ( "i", EXPP_clearScriptLinks( slink ) ) );
+	return EXPP_clearScriptLinks( slink, args );
 }
 
 /* mat.getScriptLinks */
@@ -1995,11 +2126,47 @@ static PyObject *Material_getAttr( BPy_Material * self, char *name )
 	else if( strcmp( name, "fresnelTransFac" ) == 0 )
 		attr = PyFloat_FromDouble( ( double ) self->material->
 					   fresnel_tra_i );
-
+	else if( strcmp( name, "users" ) == 0 )
+		attr = PyInt_FromLong( ( double ) self->material->
+					   id.us );
+  else if (strcmp(name, "oopsLoc") == 0) {
+    if (G.soops) { 
+      Oops *oops= G.soops->oops.first;
+      while(oops) {
+        if(oops->type==ID_MA) {
+          if ((Material *)oops->id == self->material) {
+            return (Py_BuildValue ("ff", oops->x, oops->y));
+          }
+        }
+        oops= oops->next;
+      }
+    }
+    Py_INCREF (Py_None);
+    return (Py_None);
+  }
+  /* Select in the oops view only since its a mesh */
+  else if (strcmp(name, "oopsSel") == 0) {
+    if (G.soops) {
+      Oops *oops= G.soops->oops.first;
+      while(oops) {
+        if(oops->type==ID_MA) {
+          if ((Material *)oops->id == self->material) {
+            if (oops->flag & SELECT)
+							return EXPP_incr_ret_True();
+            else
+							return EXPP_incr_ret_False();
+          }
+        }
+        oops= oops->next;
+      }
+    }
+    Py_INCREF (Py_None);
+		return (Py_None);    
+  }    
 	else if( strcmp( name, "__members__" ) == 0 ) {
-		attr =		/* 27 items */
+		attr =		/* 30 items */
 			Py_BuildValue
-			( "[s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s]",
+			( "[s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s]",
 			  "name", "mode", "rgbCol", "specCol", "mirCol", "R",
 			  "G", "B", "alpha", "amb", "emit", "ref", "spec",
 			  "specTransp", "add", "zOffset", "haloSize",
@@ -2007,7 +2174,7 @@ static PyObject *Material_getAttr( BPy_Material * self, char *name )
 			  "subSize", "hard", "nFlares", "nStars", "nLines",
 			  "nRings", "rayMirr", "rayMirrDepth", "fresnelDepth",
 			  "fresnelDepthFac", "IOR", "transDepth",
-			  "fresnelTrans", "fresnelTransFac" );
+			  "fresnelTrans", "fresnelTransFac", "users", "oopsLoc", "oopsSel" );
 	}
 
 	if( !attr )
@@ -2127,7 +2294,46 @@ static int Material_setAttr( BPy_Material * self, char *name,
 		error = Material_setFresnelTrans( self, valtuple );
 	else if( strcmp( name, "fresnelTransFac" ) == 0 )
 		error = Material_setFresnelTransFac( self, valtuple );
-
+  else if (strcmp (name, "oopsLoc") == 0) {
+    if (G.soops) {
+      Oops *oops= G.soops->oops.first;
+      while(oops) {
+        if(oops->type==ID_MA) {
+          if ((Material *)oops->id == self->material) {
+            if (!PyArg_ParseTuple  (value, "ff", &(oops->x),&(oops->y)))
+							PyErr_SetString(PyExc_AttributeError,
+								"expected two floats as arguments");
+            break;
+          }
+        }
+        oops= oops->next;
+      }
+			if (!oops)
+				PyErr_SetString(PyExc_RuntimeError,
+					"couldn't find oopsLoc data for this material!");
+			else error = EXPP_incr_ret (Py_None);
+    }
+  }
+  /* Select in the oops view only since its a mesh */
+  else if (strcmp (name, "oopsSel") == 0) {
+    int sel;
+    if (!PyArg_Parse (value, "i", &sel))
+      PyErr_SetString (PyExc_TypeError, "expected an integer, 0 or 1");
+		else if (G.soops) {
+      Oops *oops= G.soops->oops.first;
+      while(oops) {
+        if(oops->type==ID_MA) {
+          if ((Material *)oops->id == self->material) {
+            if(sel == 0) oops->flag &= ~SELECT;
+            else oops->flag |= SELECT;
+            break;
+          }
+        }
+        oops= oops->next;
+      }
+    	error = EXPP_incr_ret (Py_None);
+    }
+  }	
 	else {			/* Error */
 		Py_DECREF( valtuple );
 		return ( EXPP_ReturnIntError( PyExc_AttributeError, name ) );
