@@ -1,5 +1,5 @@
 /* 
- * $Id: Mathutils.c,v 1.5 2004/10/12 23:58:03 stiv Exp $
+ * $Id: Mathutils.c,v 1.11 2005/05/25 04:52:52 ianwill Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -25,7 +25,7 @@
  *
  * This is a new part of Blender.
  *
- * Contributor(s): Joseph Gilbert
+ * Contributor(s): Joseph Gilbert, Campbell Barton
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
@@ -251,47 +251,50 @@ static PyObject *M_Mathutils_Rand( PyObject * self, PyObject * args )
 static PyObject *M_Mathutils_Vector( PyObject * self, PyObject * args )
 {
 	PyObject *listObject = NULL;
-	PyObject *checkOb = NULL;
-	PyObject *retval = NULL;
-	int x;
-	float *vec;
+	int size, i;
+	float vec[4];
 
-	if( !PyArg_ParseTuple( args, "|O!", &PyList_Type, &listObject ) )
-		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
-						"0 or 1 list expected" ) );
-
-	if( !listObject )
-		return ( PyObject * ) newVectorObject( NULL, 3 );
-
-	//2D 3D 4D supported
-	if( PyList_Size( listObject ) != 2 && PyList_Size( listObject ) != 3
-	    && PyList_Size( listObject ) != 4 )
-		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
-						"2D, 3D and 4D vectors supported\n" ) );
-
-	for( x = 0; x < PyList_Size( listObject ); x++ ) {
-		checkOb = PyList_GetItem( listObject, x );  /* borrowed refernce */
-		if( !PyInt_Check( checkOb ) && !PyFloat_Check( checkOb ) )
-			return ( EXPP_ReturnPyObjError( PyExc_TypeError,
-							"expected list of numbers\n" ) );
-	}
-
-	//allocate memory
-	vec = PyMem_Malloc( PyList_Size( listObject ) * sizeof( float ) );
-
-	//parse it all as floats
-	for( x = 0; x < PyList_Size( listObject ); x++ ) {
-		if( !PyArg_Parse
-		    ( PyList_GetItem( listObject, x ), "f", &vec[x] ) ) {
-			return EXPP_ReturnPyObjError( PyExc_TypeError,
-						      "python list not parseable\n" );
+	size = PySequence_Length(args);
+	if ( size == 1 ) {
+		listObject = PySequence_GetItem(args, 0);
+		if ( PySequence_Check(listObject) ) {
+			size = PySequence_Length(listObject);
+		} else {
+			goto bad_args;	// Single argument was not a sequence
 		}
+	} else if ( size == 0 ) {
+		return ( PyObject * ) newVectorObject( NULL, 3 );
+	} else {
+		Py_INCREF(args);
+		listObject = args;
 	}
+	if (size<2 || size>4) {
+		goto bad_args;		// Invalid vector size
+	}
+	for (i=0; i<size; i++) {
+		PyObject *v, *f;
 
-	retval =  ( PyObject * ) newVectorObject( vec,
-						  PyList_Size( listObject ) );
-	PyMem_Free( vec );
-	return retval;
+		v=PySequence_GetItem(listObject, i);
+		if (v==NULL) {
+			Py_DECREF(listObject);
+			return NULL;	// Failed to read sequence
+		}
+		f=PyNumber_Float(v);
+		if(f==NULL) {
+			Py_DECREF(v);
+			goto bad_args;
+		}
+		vec[i]=PyFloat_AS_DOUBLE(f);
+		Py_DECREF(f);
+		Py_DECREF(v);
+	}
+	Py_DECREF(listObject);
+	return ( PyObject * ) newVectorObject( vec, size );
+
+bad_args:
+	Py_XDECREF(listObject);
+	PyErr_SetString( PyExc_TypeError, "2-4 floats expected (optionally in a sequence)");
+	return NULL;
 }
 
 //***************************************************************************
@@ -366,12 +369,24 @@ static PyObject *M_Mathutils_DotVecs( PyObject * self, PyObject * args )
 static PyObject *M_Mathutils_AngleBetweenVecs( PyObject * self,
 					       PyObject * args )
 {
+	// original vectors, makea copy of these
 	VectorObject *vec1;
 	VectorObject *vec2;
-	float dot, angleRads, norm;
+	
+	/* copy of the 2 input vectors, these can be normalized
+	without input vectors being normalized. bugfix 
+	no need to use vector objects, just use floats
+	No Chance of 4D vectors getting in.
+	
+	Use doubles, since floats will return nan when input vecs are large.*/
+	double vec1copy[3];
+	double vec2copy[3];
+	
+	float norm;
+	double dot, angleRads;
 	int x;
 
-	dot = 0;
+	dot = 0.0f;
 	if( !PyArg_ParseTuple
 	    ( args, "O!O!", &vector_Type, &vec1, &vector_Type, &vec2 ) )
 		return ( EXPP_ReturnPyObjError
@@ -382,37 +397,49 @@ static PyObject *M_Mathutils_AngleBetweenVecs( PyObject * self,
 	if( vec1->size > 3 || vec2->size > 3 )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"only 2D,3D vectors are supported\n" ) );
-
-	//normalize vec1
+	
+	/* Check for 2 vectors being the same */
+	if (vec1->size == 3 &&
+		vec1->vec[0] == vec2->vec[0] &&
+		vec1->vec[1] == vec2->vec[1] &&
+		vec1->vec[2] == vec2->vec[2])
+			return PyFloat_FromDouble( dot ); /* 2 points are the same, return zero */
+	else if (vec1->size == 2 &&
+		vec1->vec[0] == vec2->vec[0] &&
+		vec1->vec[1] == vec2->vec[1])
+			return PyFloat_FromDouble( dot ); /* 2 points are the same, return zero */
+	
+	//normalize vec1copy
 	norm = 0.0f;
 	for( x = 0; x < vec1->size; x++ ) {
-		norm += vec1->vec[x] * vec1->vec[x];
+		vec1copy[x] = vec1->vec[x]; /* Assign new vector in the loop */
+		norm += vec1copy[x] * vec1copy[x];
 	}
 	norm = ( float ) sqrt( norm );
 	for( x = 0; x < vec1->size; x++ ) {
-		vec1->vec[x] /= norm;
+		vec1copy[x] /= norm;
 	}
 
-	//normalize vec2
+	//normalize vec2copy
 	norm = 0.0f;
 	for( x = 0; x < vec2->size; x++ ) {
-		norm += vec2->vec[x] * vec2->vec[x];
+		vec2copy[x] = vec2->vec[x]; /* Assign new vector in the loop */
+		norm += vec2copy[x] * vec2copy[x];
 	}
 	norm = ( float ) sqrt( norm );
 	for( x = 0; x < vec2->size; x++ ) {
-		vec2->vec[x] /= norm;
+		vec2copy[x] /= norm;
 	}
 
 	//dot product
 	for( x = 0; x < vec1->size; x++ ) {
-		dot += vec1->vec[x] * vec2->vec[x];
+		dot += vec1copy[x] * vec2copy[x];
 	}
 
 	//I believe saacos checks to see if the vectors are normalized
-	angleRads = saacos( dot );
+	angleRads = (double)acos( dot );
 
-	return PyFloat_FromDouble( ( double )
-				   ( angleRads * ( 180 / Py_PI ) ) );
+	return PyFloat_FromDouble( angleRads * ( 180 / Py_PI ) );
 }
 
 static PyObject *M_Mathutils_MidpointVecs( PyObject * self, PyObject * args )
