@@ -1,5 +1,5 @@
 /* 
- * $Id: Metaball.c,v 1.18 2004/10/07 19:25:39 stiv Exp $
+ * $Id: Metaball.c,v 1.20 2005/04/11 19:31:05 jiri Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -410,7 +410,7 @@ static PyObject *Metaball_addMetaelem( BPy_Metaball * self, PyObject * args )
 {
 	MetaElem *ml;
 	PyObject *listargs = 0;
-	int type, lay;
+	int type;
 	float x, y, z, rad, s, expx, expy, expz;
 	if( !PyArg_ParseTuple( args, "O", &listargs ) )
 		return ( EXPP_ReturnPyObjError
@@ -425,7 +425,6 @@ static PyObject *Metaball_addMetaelem( BPy_Metaball * self, PyObject * args )
 	y = PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
 	z = PyFloat_AsDouble( PyList_GetItem( listargs, 3 ) );
 	rad = PyFloat_AsDouble( PyList_GetItem( listargs, 4 ) );
-	lay = PyInt_AsLong( PyList_GetItem( listargs, 5 ) );
 	s = PyFloat_AsDouble( PyList_GetItem( listargs, 6 ) );
 	expx = PyFloat_AsDouble( PyList_GetItem( listargs, 7 ) );
 	expy = PyFloat_AsDouble( PyList_GetItem( listargs, 8 ) );
@@ -438,7 +437,6 @@ static PyObject *Metaball_addMetaelem( BPy_Metaball * self, PyObject * args )
 	ml->y = y;
 	ml->z = z;
 	ml->rad = rad;
-	ml->lay = lay;
 	ml->s = s;
 	ml->flag = SELECT;
 	ml->type = type;
@@ -1092,6 +1090,8 @@ static PyObject *MetaballGetAttr( BPy_Metaball * self, char *name )
 		return Metaball_getloc( self );
 	if( strcmp( name, "size" ) == 0 )
 		return Metaball_getsize( self );
+	if( strcmp( name, "users" ) == 0 )
+		return PyInt_FromLong( self->metaball->id.us );	
 	return Py_FindMethod( BPy_Metaball_methods, ( PyObject * ) self,
 			      name );
 }
