@@ -1,5 +1,5 @@
 /**
- * $Id: Noise.c,v 1.3 2004/09/25 20:30:40 stiv Exp $
+ * $Id: Noise.c,v 1.4 2005/03/09 19:45:55 lukep Exp $
  *
  * Blender.Noise BPython module implementation.
  * This submodule has functions to generate noise of various types.
@@ -59,6 +59,8 @@ static int left = 1;
 static int initf = 0;
 static unsigned long *next;
 
+PyObject *Noise_Init(void);
+
 /* initializes state[N] with a seed */
 static void init_genrand( unsigned long s )
 {
@@ -603,7 +605,7 @@ static PyMethodDef NoiseMethods[] = {
 
 /*----------------------------------------------------------------------*/
 
-PyObject *Noise_Init(  )
+PyObject *Noise_Init(void)
 {
 	PyObject *NoiseTypes, *DistanceMetrics,
 		*md =
