@@ -1,5 +1,5 @@
 /*
- * $Id: Particle.c,v 1.7 2004/09/25 20:30:40 stiv Exp $
+ * $Id: Particle.c,v 1.8 2005/03/09 19:45:55 lukep Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -124,6 +124,9 @@ static PyMethodDef BPy_Particle_methods[] = {
 	{NULL, NULL, 0, NULL}
 };
 
+/**************** prototypes ********************/
+PyObject *Particle_Init( void );
+
 
 /*****************************************************************************/
 /* Python Particle_Type structure definition:                                */
