@@ -1,5 +1,5 @@
 /* 
- * $Id: Sound.c,v 1.3 2004/10/20 05:51:23 ianwill Exp $
+ * $Id: Sound.c,v 1.5 2005/03/19 06:24:54 ianwill Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -36,7 +36,7 @@
 #include <BKE_sound.h>
 #include <BLI_blenlib.h>
 #include <BIF_editsound.h>
-#include <BIF_space.h>		/* allqueue() */
+#include <BIF_space.h>		/* EXPP_allqueue() */
 
 #include "mydevice.h"		/* redraw defines */
 
@@ -153,9 +153,12 @@ static PyMethodDef BPy_Sound_methods[] = {
 	 "() - play this sound"},
 	{"setCurrent", ( PyCFunction ) Sound_setCurrent, METH_NOARGS,
 	 "() - make this the active sound in the sound buttons win (also redraws)"},
-	//{"reload", ( PyCFunction ) Sound_setCurrent, METH_NOARGS,
-	 //"() - reload this Sound object's sample.\n\
-//This is only useful if the original sound file has changed."},
+
+/*
+	{"reload", ( PyCFunction ) Sound_setCurrent, METH_NOARGS,
+	 "() - reload this Sound object's sample.\n\
+    This is only useful if the original sound file has changed."},
+*/
 	SOUND_FLOAT_METHOD_FUNCS( Volume )
 	SOUND_FLOAT_METHOD_FUNCS( Attenuation )
 	SOUND_FLOAT_METHOD_FUNCS( Pitch )
@@ -424,8 +427,8 @@ static PyObject *Sound_setCurrent( BPy_Sound * self )
 		}
 	}
 
-	allqueue( REDRAWSOUND, 0 );
-	allqueue( REDRAWBUTSLOGIC, 0 );
+	EXPP_allqueue( REDRAWSOUND, 0 );
+	EXPP_allqueue( REDRAWBUTSLOGIC, 0 );
 
 	Py_INCREF( Py_None );
 	return Py_None;
