@@ -3,7 +3,7 @@
 """
 The Blender.Material submodule.
 
-B{New}: scriptLink methods: L{Material.getScriptLinks}, ...
+B{New}: L{Material.clearScriptLinks} accepts a parameter now.
 
 Material 
 ========
@@ -14,7 +14,7 @@ Example::
   import Blender
   from Blender import Material
   mat = Material.New('newMat')          # create a new Material called 'newMat'
-  print mat.rgbCol                      # print its rgb color triplet
+  print mat.rgbCol                      # print its rgb color triplet sequence
   mat.rgbCol = [0.8, 0.2, 0.2]          # change its color
   mat.setAlpha(0.2)                     # mat.alpha = 0.2 -- almost transparent
   mat.emit = 0.7                        # equivalent to mat.setEmit(0.8)
@@ -83,47 +83,47 @@ class Material:
   The Material object
   ===================
    This object gives access to Materials in Blender.
-  @cvar name: Material's name.
+  @ivar name: Material's name.
   @type mode: int
-  @cvar mode: Mode flags as an or'ed int value.  See the Modes dictionary keys
+  @ivar mode: Mode flags as an or'ed int value.  See the Modes dictionary keys
       and descriptions in L{Modes}.
-  @cvar rgbCol: Material's RGB color triplet.
-  @cvar specCol: Specular color rgb triplet.
-  @cvar mirCol: Mirror color rgb triplet.
-  @cvar R: Red component of L{rgbCol} - [0.0, 1.0].
-  @cvar G: Green component of L{rgbCol} - [0.0, 1.0].
-  @cvar B: Blue component of L{rgbCol} - [0.0, 1.0].
-  @cvar alpha: Alpha (translucency) component of the Material - [0.0, 1.0].
-  @cvar amb: Ambient factor - [0.0, 1.0].
-  @cvar emit: Emitting light intensity - [0.0, 1.0].
-  @cvar ref:  Reflectivity - [0.0, 1.0].
-  @cvar spec: Specularity - [0.0, 2.0].
-  @cvar specTransp: Specular transparency - [0.0, 1.0].
-  @cvar add: Glow factor - [0.0, 1.0].
-  @cvar zOffset: Artificial Z offset for faces - [0.0, 10.0].
-  @cvar haloSize: Dimension of the halo - [0.0, 100.0].
-  @cvar flareSize: Factor the flare is larger than the halo - [0.1, 25.0].
-  @cvar flareBoost: Flare's extra strength - [0.1, 10.0].
-  @cvar haloSeed: To use random values for ring dimension and line location -
+  @ivar rgbCol: Material's RGB color triplet sequence.
+  @ivar specCol: Specular color rgb triplet sequence.
+  @ivar mirCol: Mirror color rgb triplet sequence.
+  @ivar R: Red component of L{rgbCol} - [0.0, 1.0].
+  @ivar G: Green component of L{rgbCol} - [0.0, 1.0].
+  @ivar B: Blue component of L{rgbCol} - [0.0, 1.0].
+  @ivar alpha: Alpha (translucency) component of the Material - [0.0, 1.0].
+  @ivar amb: Ambient factor - [0.0, 1.0].
+  @ivar emit: Emitting light intensity - [0.0, 1.0].
+  @ivar ref:  Reflectivity - [0.0, 1.0].
+  @ivar spec: Specularity - [0.0, 2.0].
+  @ivar specTransp: Specular transparency - [0.0, 1.0].
+  @ivar add: Glow factor - [0.0, 1.0].
+  @ivar zOffset: Artificial Z offset for faces - [0.0, 10.0].
+  @ivar haloSize: Dimension of the halo - [0.0, 100.0].
+  @ivar flareSize: Factor the flare is larger than the halo - [0.1, 25.0].
+  @ivar flareBoost: Flare's extra strength - [0.1, 10.0].
+  @ivar haloSeed: To use random values for ring dimension and line location -
      [0, 255].
-  @cvar flareSeed: Offset in the seed table - [0, 255].
-  @cvar subSize:  Dimension of subflares, dots and circles - [0.1, 25.0].
-  @cvar hard: Hardness of the specularity - [1, 255].
-  @cvar nFlares: Number of halo subflares - [1, 32].
-  @cvar nStars: Number of points on the halo stars - [3, 50].
-  @cvar nLines: Number of star shaped lines on each halo - [0, 250].
-  @cvar nRings: Number of halo rings - [0, 24].
+  @ivar flareSeed: Offset in the seed table - [0, 255].
+  @ivar subSize:  Dimension of subflares, dots and circles - [0.1, 25.0].
+  @ivar hard: Hardness of the specularity - [1, 255].
+  @ivar nFlares: Number of halo subflares - [1, 32].
+  @ivar nStars: Number of points on the halo stars - [3, 50].
+  @ivar nLines: Number of star shaped lines on each halo - [0, 250].
+  @ivar nRings: Number of halo rings - [0, 24].
   @type ipo: Blender Ipo
-  @cvar ipo: This Material's ipo.
-  @cvar rayMirr: Amount mirror reflection for raytrace.
-  @cvar rayMirrDepth: Amount of inter-reflections calculated maximal.
-  @cvar fresnelDepth: Power of Fresnel for mirror reflection. 
-  @cvar fresnelDepthFac: Blending factor for Fresnel. 
-  @cvar IOR: Sets the angular index of refraction for raytrace. 
-  @cvar transDepth: Amount of refractions calculated maximal. 
-  @cvar fresnelTrans: Power of Fresnel for transparency.
-  @cvar fresnelTransFac: Blending factor for Fresnel.
-  @cvar specTrans: Makes specular areas opaque on transparent materials.
+  @ivar ipo: This Material's ipo.
+  @ivar rayMirr: Amount mirror reflection for raytrace.
+  @ivar rayMirrDepth: Amount of inter-reflections calculated maximal.
+  @ivar fresnelDepth: Power of Fresnel for mirror reflection. 
+  @ivar fresnelDepthFac: Blending factor for Fresnel. 
+  @ivar IOR: Sets the angular index of refraction for raytrace. 
+  @ivar transDepth: Amount of refractions calculated maximal. 
+  @ivar fresnelTrans: Power of Fresnel for transparency.
+  @ivar fresnelTransFac: Blending factor for Fresnel.
+  @ivar specTrans: Makes specular areas opaque on transparent materials.
   @warning: Most member variables assume values in some [Min, Max] interval.
    When trying to set them, the given parameter will be clamped to lie in
    that range: if val < Min, then val = Min, if val > Max, then val = Max.
@@ -162,6 +162,23 @@ class Material:
     @return: True if there was an ipo linked or False otherwise.
     """
 
+  def insertIpoKey(keytype):
+    """
+    Inserts keytype values in material ipo at curframe. Uses module constants.
+    @type keytype: Integer
+    @param keytype:
+           -RGB
+           -ALPHA
+           -HALOSIZE
+           -MODE
+           -ALLCOLOR
+           -ALLMIRROR
+           -OFS
+           -SIZE
+           -ALLMAPPING
+    @return: py_none
+    """
+
   def getMode():
     """
     Get this Material's mode flags.
@@ -179,26 +196,39 @@ class Material:
             print "Not a halo"
     """
 
-  def setMode(m = None, m2 = None, m3 = None, and_so_on = None,
-              up_to_21 = None):
+  def setMode(param, stringN=None):
     """
-    Set this Material's mode flags. Mode strings given are turned 'on'.
-    Those not provided are turned 'off', so mat.setMode() -- without 
-    arguments -- turns off all mode flags for Material mat.
-    @type m: string
-    @param m: A mode flag. From 1 to 21 can be set at the same time.
+    Set this Material's mode flags. Up to 22 mode strings can be given
+    and specify the modes which are turned 'on'.  Those not provided are 
+    turned 'off', so mat.setMode() -- without arguments -- turns off all 
+    mode flags for Material mat.  Valid mode strings are "Traceable", 
+    "Shadow", "Shadeless", "Wire", "VColLight", "VColPaint", "Halo",
+    "ZTransp", "ZInvert", "HaloRings", "HaloLines", "OnlyShadow",
+    "HaloXAlpha", "HaloStar", "TexFace", "HaloTex", "HaloPuno", "NoMist",
+    "HaloShaded", "HaloFlare", "Radio", "RayMirr", "ZTransp", "RayTransp",
+    "OnlyShadow", "NoMist", "Env"
+
+    An integer can also be given, which directly sets the mode flag.  The
+    Modes dictionary keys can (and should) be added or ORed to specify
+    which modes to turn 'on'.  The value returned from getMode() can
+    also be modified and input to this method.
+
+    @type param: string, None or int
+    @param param: A mode value (int) or flag (string).  Can also be None.
+    @type stringN: string
+    @param stringN: A mode flag. Up to 22 flags can be set at the same time.
     """
 
   def getRGBCol():
     """
-    Get the rgb color triplet.
+    Get the rgb color triplet sequence.
     @rtype: list of 3 floats
     @return: [r, g, b]
     """
 
   def setRGBCol(rgb = None):
     """
-    Set the rgb color triplet.  If B{rgb} is None, set the color to black.
+    Set the rgb color triplet sequence.  If B{rgb} is None, set the color to black.
     @type rgb: three floats or a list of three floats
     @param rgb: The rgb color values in [0.0, 1.0] as:
         - a list of three floats: setRGBCol ([r, g, b]) B{or}
@@ -207,14 +237,14 @@ class Material:
  
   def getSpecCol():
     """
-    Get the specular color triplet.
+    Get the specular color triplet sequence.
     @rtype: list of 3 floats
     @return: [specR, specG, specB]
     """
 
   def setSpecCol(rgb = None):
     """
-    Set the specular color triplet.  If B{rgb} is None, set the color to black.
+    Set the specular color triplet sequence.  If B{rgb} is None, set the color to black.
     @type rgb: three floats or a list of three floats
     @param rgb: The rgb color values in [0.0, 1.0] as:
         - a list of three floats: setSpecCol ([r, g, b]) B{or}
@@ -223,14 +253,14 @@ class Material:
 
   def getMirCol():
     """
-    Get the mirror color triplet.
+    Get the mirror color triplet sequence.
     @rtype: list of 3 floats
     @return: [mirR, mirG, mirb]
     """
 
   def setMirCol(rgb = None):
     """
-    Set the mirror color triplet.  If B{rgb} is None, set the color to black.
+    Set the mirror color triplet sequence.  If B{rgb} is None, set the color to black.
     @type rgb: three floats or a list of three floats
     @param rgb: The rgb color values in [0.0, 1.0] as:
         - a list of three floats: setMirCol ([r, g, b]) B{or}
@@ -429,7 +459,7 @@ class Material:
     """
     Set the hardness of the specularity.
     @type hardness: int
-    @param hardness: The new value in [1, 255].
+    @param hardness: The new value in [1, 511].
     """
 
   def getNFlares():
@@ -592,7 +622,7 @@ class Material:
     """
     Assign a Blender Texture object to slot number 'number'.
     @type index: int
-    @param index: material's texture index in [0, 7].
+    @param index: material's texture index in [0, 9].
     @type texture: Blender Texture
     @param texture: a Blender Texture object.
     @type texco: int
@@ -605,7 +635,7 @@ class Material:
     """
     Clear the ith (given by 'index') texture channel of this material.
     @type index: int
-    @param index: material's texture channel index in [0, 7].
+    @param index: material's texture channel index in [0, 9].
     """
 
   def getTextures ():
@@ -626,11 +656,12 @@ class Material:
         'event' type) or None if there are no script links at all.
     """
 
-  def clearScriptLinks ():
+  def clearScriptLinks (links = None):
     """
-    Delete all this Material's script links.
-    @rtype: bool
-    @return: 0 if some internal problem occurred or 1 if successful.
+    Delete script links from this Material.  If no list is specified, all
+    script links are deleted.
+    @type links: list of strings
+    @param links: None (default) or a list of Blender L{Text} names.
     """
 
   def addScriptLink (text, event):
