@@ -3,7 +3,12 @@
 """
 The Blender.NMesh submodule.
 
-B{New}: L{NMesh.getMaterials}, L{NMesh.setMaterials}.
+B{New}:
+ - edges class (L{NMEdge}) and nmesh methods (L{NMesh.addEdge},
+L{NMesh.addEdgesData}, etc.);
+ - new optional arguments to L{NMesh.update};
+ - L{NMesh.transform};
+ - L{GetNames}.
 
 Mesh Data
 =========
@@ -13,8 +18,11 @@ This module provides access to B{Mesh Data} objects in Blender.
 Example::
 
   import Blender
-  from Blender import NMesh, Material
-  #
+  from Blender import NMesh, Material, Window
+
+  editmode = Window.EditMode()    # are we in edit mode?  If so ...
+  if editmode: Window.EditMode(0) # leave edit mode before getting the mesh
+
   me = NMesh.GetRaw("Plane")       # get the mesh data called "Plane"
 
   if not me.materials:             # if there are no materials ...
@@ -30,6 +38,8 @@ Example::
     v.co[2] *= 2.5
   me.update()                      # update the real mesh in Blender
 
+  if editmode: Window.EditMode(1)  # optional, just being nice
+
 @type Modes: readonly dictionary
 @type FaceFlags: readonly dictionary
 @type FaceModes: readonly dictionary
@@ -40,23 +50,25 @@ Example::
     - AUTOSMOOTH - turn auto smoothing of faces "on".
     - SUBSURF - turn Catmull-Clark subdivision of surfaces "on".
     - OPTIMAL - optimal drawing of edges when "SubSurf" is "on".
-@var FaceFlags: The available face selection flags.
+@var FaceFlags: The available *texture face* (uv face select mode) selection
+  flags.  Note: these refer to TexFace faces, available if nmesh.hasFaceUV()
+  returns true.
     - SELECT - selected.
     - HIDE - hidden.
     - ACTIVE - the active face.
-@var FaceModes: The available face modes. Note: these are only meaninful if
-  nmesh.hasFaceUV() returns true, since in Blender this info is stored at the
-  TexFace (TexFace button in Edit Mesh buttons) structure.
+@var FaceModes: The available *texture face* modes. Note: these are only
+  meaningful if nmesh.hasFaceUV() returns true, since in Blender this info is
+  stored at the TexFace (TexFace button in Edit Mesh buttons) structure.
     - ALL - set all modes at once.
     - BILLBOARD - always orient after camera.
     - HALO - halo face, always point to camera.
-    - DINAMYC - respond to collisions.
+    - DYNAMIC - respond to collisions.
     - INVISIBLE - invisible face.
-    - LIGHT - dinamyc lighting.
-    - OBCOL - use object colour instead of vertex colours.
+    - LIGHT - dynamic lighting.
+    - OBCOL - use object color instead of vertex colors.
     - SHADOW - shadow type.
     - SHAREDVERT - apparently unused in Blender.
-    - SHAREDCOL - shared vertex colours (per vertex).
+    - SHAREDCOL - shared vertex colors (per vertex).
     - TEX - has texture image.
     - TILES - uses tiled image.
     - TWOSIDE - two-sided face.
@@ -66,13 +78,18 @@ Example::
     - ADD - add to background (halo).
     - ALPHA - draw with transparency.
     - SUB - subtract from background.
+@var EdgeFlags: The available edge flags.
+    - SELECT - selected.
+    - EDGEDRAW - edge is drawn out of edition mode.
+    - SEAM - edge is a seam for LSCM UV unwrapping
+    - FGON - edge is part of a F-Gon.
 """
 
 def Col(col = [255, 255, 255, 255]):
   """
   Get a new mesh rgba color.
   @type col: list
-  @param col: A list [red, green, blue, alpha] of int values in [0, 255].
+  @param col: A list [red, green, blue, alpha] of integer values in [0, 255].
   @rtype: NMCol
   @return:  A new NMCol (mesh rgba color) object.
   """
@@ -121,9 +138,19 @@ def GetRaw(name = None):
       - () - A new (empty) NMesh object.
   """
 
+def GetNames():
+  """
+  Get a list with the names of all available meshes in Blender.
+  @rtype: list of strings
+  @return: a list of mesh names.
+  @note: to get actual mesh data, pass a mesh name to L{GetRaw}.
+  """
+
 def GetRawFromObject(name):
   """
-  Get the raw mesh data object from the Object in Blender called I{name}.
+  Get the raw mesh data object from the Object in Blender called I{name}.\n
+  Note: The mesh coordinates are in local space, not the world space of its Object.\n
+  For world space vertex coordinates, each vertex location must be multiplied by the object's 4x4 matrix.
   @type name: string
   @param name: The name of an Object of type "Mesh".
   @rtype: NMesh
@@ -139,16 +166,23 @@ def GetRawFromObject(name):
       be created.
   """
 
-def PutRaw(nmesh, name = None, recalculate_normals = 1):
+def PutRaw(nmesh, name = None, recalc_normals = 1, store_edges = 0):
   """
-  Put an NMesh object back in Blender.
+  Put a BPython NMesh object as a mesh data object in Blender.
+  @note: if there is already a mesh with the given 'name', its contents are
+  freed and the new data is put in it.  Also, if this mesh is not linked to any
+  object, a new object for it is created.  Reminder: in Blender an object is
+  composed of the base object and linked object data (mesh, metaball, camera,
+  etc. etc).
   @type nmesh: NMesh
   @type name: string
-  @type recalculate_normals: int
+  @type recalc_normals: int
+  @type store_edges: int
   @param name: The name of the mesh data object in Blender which will receive
      this nmesh data.  It can be an existing mesh data object or a new one.
-  @param recalculate_normals: If non-zero, the vertex normals for the mesh will
+  @param recalc_normals: If non-zero, the vertex normals for the mesh will
      be recalculated.
+  @param store_edges: if non-zero, the edges data are stored
   @rtype: None or Object
   @return: It depends on the 'name' parameter:
       - I{name} refers to an existing mesh data obj already linked to an
@@ -162,11 +196,11 @@ class NMCol:
   The NMCol object
   ================
     This object is a list of ints: [r, g, b, a] representing an
-    rgba colour.
-  @cvar r: The Red component in [0, 255].
-  @cvar g: The Green component in [0, 255].
-  @cvar b: The Blue component in [0, 255].
-  @cvar a: The Alpha (transparency) component in [0, 255].
+    rgba color.
+  @ivar r: The Red component in [0, 255].
+  @ivar g: The Green component in [0, 255].
+  @ivar b: The Blue component in [0, 255].
+  @ivar a: The Alpha (transparency) component in [0, 255].
   """
 
 class NMVert:
@@ -174,16 +208,17 @@ class NMVert:
   The NMVert object
   =================
     This object holds mesh vertex data.
-  @type co: list of three floats
-  @cvar co: The vertex coordinates (x, y, z).
-  @type no: list of three floats
-  @cvar no: The vertex normal vector (x, y, z).
-  @type uvco: list of two floats
-  @cvar uvco: The vertex texture "sticky" coordinates.
+  @type co: 3D Vector object.
+  @ivar co: The vertex coordinates (x, y, z).
+  @type no: 3D Vector object. (unit length)
+  @ivar no: The vertex normal vector (x, y, z).
+  @type uvco: 3D Vector object.
+  @ivar uvco: The vertex texture "sticky" coordinates. The Z value of the Vector is ignored.
   @type index: int
-  @cvar index: The vertex index, if owned by a mesh.
+  @ivar index: The vertex index, if owned by a mesh.
   @type sel: int
-  @cvar sel: The selection state (selected:1, unselected:0) of this vertex.
+  @ivar sel: The selection state (selected:1, unselected:0) of this vertex.\n
+  Note: An NMesh will return the selection state of the mesh when EditMod was last exited. A python script operating in EditMode must exit edit mode, before getting the current selection state of the mesh.
   @warn:  There are two kinds of uv texture coordinates in Blender: per vertex
      ("sticky") and per face vertex (uv in L{NMFace}).  In the first, there's
      only one uv pair of coordinates for each vertex in the mesh.  In the
@@ -193,6 +228,21 @@ class NMVert:
      each face can be independently mapped to any part of its texture.
   """
 
+class NMEdge:
+  """
+  The NMEdge object
+  =================
+    This object holds mesh edge data.
+  @type v1: NMVert
+  @ivar v1: The first vertex of the edge.
+  @type v2: NMVert
+  @ivar v2: The second vertex of the edge.
+  @type crease: int
+  @ivar crease: The crease value of the edge. It is in the range [0,255].
+  @type flag: int
+  @ivar flag: The bitmask describing edge properties. See L{NMesh.EdgeFlags<EdgeFlags>}.
+  """
+
 class NMFace:
   """
   The NMFace object
@@ -201,38 +251,71 @@ class NMFace:
 
   Example::
    import Blender
-   from Blender import NMesh
+   from Blender import NMesh, Window
+
+   in_emode = Window.EditMode()
+   if in_emode: Window.EditMode(0)
 
    me = NMesh.GetRaw("Mesh")
    faces = me.faces
+
+   ## Example for editmode faces selection:
+   selected_faces = []
+   for f in faces:
+     if f.sel:
+       selected_faces.append(f)
+   # ... unselect selected and select all the others:
+   for f in faces:
+     f.sel = 1 - f.sel # 1 becomes 0, 0 becomes 1
+
+   ## Example for uv textured faces selection:
    selected_faces = []
    SEL = NMesh.FaceFlags['SELECT']
    # get selected faces:
    for f in faces:
      if f.flag & SEL:
-     selected_faces.append(f)
+       selected_faces.append(f)
    # ... unselect selected and select all the others:
    for f in faces:
      if f.flag & SEL:
        f.flag &=~SEL # unselect these
      else: f.flag |= SEL # and select these
 
+   me.update()
+   if in_emode: Window.EditMode(1)
+   Blender.Redraw()
+
   @type v: list
-  @cvar v: The list of face vertices (B{up to 4}).
-  @cvar col: The list of vertex colours.
-  @cvar mat: Same as I{materialIndex} below.
-  @cvar materialIndex: The index of this face's material in its NMesh materials
+  @ivar v: The list of face vertices (B{up to 4}).
+  @type sel: bool
+  @ivar sel: The selection state (1: selected, 0: unselected) of this NMesh's
+      faces *in edit mode*.  This is not the same as the selection state of
+      the textured faces (see L{NMesh.NMFace.flag}).
+  @type hide: bool
+  @ivar hide: The visibility state (1: hidden, 0: visible) of this NMesh's
+      faces *in edit mode*.  This is not the same as the visibility state of
+      the textured faces (see L{NMesh.NMFace.flag}).
+  @ivar col: The list of vertex colours.
+  @ivar mat: Same as I{materialIndex} below.
+  @ivar materialIndex: The index of this face's material in its NMesh materials
       list.
-  @cvar smooth: If non-zero, the vertex normals are averaged to make this
+  @ivar smooth: If non-zero, the vertex normals are averaged to make this
      face look smooth.
-  @cvar image: The Image used as a texture for this face.
-  @cvar mode: The display mode (see L{Mesh.FaceModes<FaceModes>})
-  @cvar flag: Bit vector specifying selection flags
-     (see L{NMesh.FaceFlags<FaceFlags>}).
-  @cvar transp: Transparency mode bit vector
+  @ivar image: The Image used as a texture for this face.
+  @ivar mode: The display mode (see L{Mesh.FaceModes<FaceModes>})
+  @ivar flag: Bit vector specifying selection / visibility flags for uv
+     textured faces (visible in Face Select mode, see
+     L{NMesh.FaceFlags<FaceFlags>}).
+  @ivar transp: Transparency mode bit vector
      (see L{NMesh.FaceTranspModes<FaceTranspModes>}).
-  @cvar uv: List of per-face UV coordinates: [(u0, v0), (u1, v1), ...].
-  @cvar normal: (or just B{no}) The normal vector for this face: [x,y,z].
+  @ivar uv: List of per-face UV coordinates: [(u0, v0), (u1, v1), ...].
+  @ivar normal: (or just B{no}) The normal vector for this face: [x,y,z].
+  @note: there are normal faces and textured faces in Blender, both currently
+    with their own selection and visibility states, due to a mix of old and new
+    code.  To (un)select or (un)hide normal faces (visible in editmode), use
+    L{NMFace.sel} and L{NMFace.hide} vars.  For textured faces (Face Select
+    mode in Blender) use the old L{NMFace.flag} bitflag.  Also check the
+    example above and note L{Window.EditMode}.
   @note: Assigning uv textures to mesh faces in Blender works like this:
     1. Select your mesh.
     2. Enter face select mode (press f) and select at least some face(s).
@@ -257,18 +340,79 @@ class NMesh:
   =====================
     This object gives access to mesh data in Blender.  We refer to mesh as the
     object in Blender and NMesh as its Python counterpart.
-  @cvar name: The NMesh name.  It's common to use this field to store extra
+  @ivar name: The NMesh name.  It's common to use this field to store extra
      data about the mesh (to be exported to another program, for example).
-  @cvar materials: The list of materials used by this NMesh.  See
+  @ivar materials: The list of materials used by this NMesh.  See
      L{NMesh.getMaterials} for important details.
-  @cvar verts: The list of NMesh vertices (NMVerts).
-  @cvar users: The number of Objects using (linked to) this mesh.
-  @cvar faces: The list of NMesh faces (NMFaces).
-  @cvar mode:  The mode flags for this mesh.  See L{setMode}.
-  @cvar subDivLevels: The [display, rendering] subdivision levels in [1, 6].
-  @cvar maxSmoothAngle: The max angle for auto smoothing.  See L{setMode}.
+  @ivar verts: The list of NMesh vertices (NMVerts).
+  @ivar users: The number of Objects using (linked to) this mesh.
+  @ivar faces: The list of NMesh faces (NMFaces).
+  @ivar edges: None if mesh has no edge data, else a list of L{NMEdge} edges. Use L{addEdgesData} to create edge data if it do not exist.
+  @ivar mode:  The mode flags for this mesh.  See L{setMode}.
+  @ivar subDivLevels: The [display, rendering] subdivision levels in [1, 6].
+  @ivar maxSmoothAngle: The max angle for auto smoothing.  See L{setMode}.
   """
 
+  def addEdge(v1, v2):
+    """
+    Create an edge between two vertices.
+    If an edge already exists between those vertices, it is returned.
+    Created edge is automatically added to edges list.
+    You can only call this method if mesh has edge data.
+    @note: In Blender only zero or one edge can link two vertices.
+    @type v1: NMVert
+    @param v1: the first vertex of the edge.
+    @type v2: NMVert
+    @param v2: the second vertex of the edge.
+    @rtype: NMEdge
+    @return: The created or already existing edge.
+    """
+
+  def findEdge(v1, v2):
+    """
+    Try to find an edge between two vertices.
+    If no edge exists between v1 and v2, None is returned.
+    You can only call this method if mesh has edge data.
+    @type v1: NMVert
+    @param v1: the first vertex of the edge.
+    @type v2: NMVert
+    @param v2: the second vertex of the edge.
+    @rtype: NMEdge
+    @return: The found edge. None if no edge was found.
+    """
+
+  def removeEdge(v1, v2):
+    """
+    Remove an edge between two vertices.
+    All faces using this edge are removed from faces list.
+    You can only call this method if mesh has edge data.
+    @type v1: NMVert
+    @param v1: the first vertex of the edge.
+    @type v2: NMVert
+    @param v2: the second vertex of the edge.
+    """
+
+  def addFace(face):
+    """
+    Add a face to face list and add to edge list (if edge data exists) necessary edges.
+    @type face: NMFace
+    @param face: the face to add to the mesh.
+    @rtype: list of NMEdge
+    @return: If mesh has edge data, return the list of face edges.
+    """
+
+  def removeFace(face):
+    """
+    Remove a face for face list and remove edges no more used by any other face (if edge data exists).
+    @type face: NMFace
+    @param face: the face to add to the mesh.
+    """
+
+  def addEdgesData():
+    """
+    If edge data does not exist for the mesh (ie L{edges}==None), then create them.
+    """
+    
   def addMaterial(material):
     """
     Add a new material to this NMesh's list of materials.  This method is the
@@ -367,6 +511,9 @@ class NMesh:
     @return:  It depends on the I{flag} parameter:
         - if None or zero: List of NMFace objects.
         - if non-zero: List of indices to NMFace objects.
+    @warn: this method exists to speed up retrieving of selected faces from
+       the actual mesh in Blender.  So, if you make changes to the nmesh, you
+       need to L{update} it before using this method.
     """
 
   def getVertexInfluences(index):
@@ -412,19 +559,77 @@ class NMesh:
        add them.
     """
 
-  def update(recalc_normals = 0):
+  def update(recalc_normals = 0, store_edges = 0, vertex_shade = 0):
     """
     Update the mesh in Blender.  The changes made are put back to the mesh in
-    Blender, if available, or put in a newly created mesh object if this NMesh
-    wasn't already linked to one.
-    @type recalc_normals: int
-    @param recalc_normals: If given and equal to 1, the vertex normals are
-        recalculated.
+    Blender, if available, or put in a newly created mesh if this NMesh wasn't
+    already linked to one.
+    @type recalc_normals: int (bool)
+    @param recalc_normals: if nonzero the vertex normals are recalculated.
+    @type store_edges: int (bool)
+    @param store_edges: if nonzero, then edge data is stored.
+    @type vertex_shade: int (bool)
+    @param vertex_shade: if nonzero vertices are colored based on the
+        current lighting setup, like when there are no vertex colors and no
+        textured faces and a user enters Vertex Paint Mode in Blender (only
+        lamps in visible layers account).  To use this functionality, be out of
+        edit mode or else an error will be returned.
+    @warn: edit mesh and normal mesh are two different structures in Blender,
+        synchronized upon leaving or entering edit mode.  Always remember to
+        leave edit mode (L{Window.EditMode}) before calling this update
+        method, or your changes will be lost.  Even better: for the same reason
+        programmers should leave EditMode B{before} getting a mesh, or changes
+        made to the editmesh in Blender may not be visible to your script
+        (check the example at the top of NMesh module doc).
+    @warn: unlike the L{PutRaw} function, this method doesn't check validity of
+        vertex, face and material lists, because it is meant to be as fast as
+        possible (and already performs many tasks).  So programmers should make
+        sure they only feed proper data to the nmesh -- a good general
+        recommendation, of course.  It's also trivial to write code to check
+        all data before updating, for example by comparing each item's type
+        with the actual L{Types}, if you need to.
+    @note: this method also redraws the 3d view and -- if 'vertex_shade' is
+        nonzero -- the edit buttons window.
     @note: if your mesh disappears after it's updated, try
         L{Object.Object.makeDisplayList}.  'Subsurf' meshes (see L{getMode},
         L{setMode}) need their display lists updated, too.
     """
 
+  def transform(matrix, recalc_normals = False):
+    """
+    Transforms the mesh by the specified 4x4 matrix, as returned by
+    L{Object.Object.getMatrix}, though this will work with any invertible 4x4
+    matrix type.  Ideal usage for this is exporting to an external file where
+    global vertex locations are required for each object.
+    Sometimes external renderers or file formats do not use vertex normals.
+    In this case, you can skip transforming the vertex normals by leaving
+    the optional parameter recalc_normals as False or 0 ( the default value ).
+    
+    Example::
+     # This script outputs deformed meshes worldspace vertex locations
+     # for a selected object
+     import Blender
+     from Blender import NMesh, Object
+     
+     ob = Object.GetSelected()[0] # Get the first selected object
+     me = NMesh.GetRawFromObject(ob.name) # Get the objects deformed mesh data
+     me.transform(ob.matrix)
+   
+     for v in me.verts:
+       print 'worldspace vert', v.co
+    
+    @type matrix: Py_Matrix
+    @param matrix: 4x4 Matrix which can contain location, scale and rotation. 
+    @type recalc_normals: int (bool)
+    @param recalc_normals: if True or 1, transform normals as well as vertex coordinates.
+    @warn: if you call this method and later L{update} the mesh, the new
+        vertex positions will be passed back to Blender, but the object
+        matrix of each object linked to this mesh won't be automatically
+        updated.  You need to set the object transformations (rotation,
+        translation and scaling) to identities, then, or the mesh data will
+        be changed ("transformed twice").
+    """
+
   def getMode():
     """
     Get this mesh's mode flags.
@@ -437,8 +642,9 @@ class NMesh:
     Set the mode flags for this mesh.  Given mode strings turn the mode "on".
     Modes not passed in are turned "off", so setMode() (without arguments)
     unsets all mode flags.
-    @type m: string
-    @param m: mode string.  From none to 5 can be given:
+    @type m: string or int (bitflag)
+    @param m: mode string or int.  An int (see L{Modes}) or from none to 5
+      strings can be given:
        - "NoVNormalsFlip"
        - "TwoSided"
        - "AutoSmooth"
@@ -572,6 +778,8 @@ class NMesh:
   def getMaxSmoothAngle():
     """
     Get the max angle for auto smoothing.
+    Note: This will only affect smoothing generated at render time.
+    Smoothing can also be set per face which is visible in Blenders 3D View.
     @return: The value in degrees.
     """
 
