@@ -3,20 +3,20 @@
 """
 The Blender.Scene.Render submodule.
 
-Scene
-=====
+Scene.Render
+============
 
 This module provides access to B{Scene Rendering Contexts} in Blender.
 
 Example::
   import Blender
-  from Blender import Scene
-  from Blender import Scene.Render
+  from Blender import *
+  from Blender.Scene import Render
   
   scn = Scene.GetCurrent()
   context = scn.getRenderingContext()
   
-  Render..enableDispWin()
+  Render.EnableDispWin()
   context.enableExtensions(1)
   context.setRenderPath("C:/myRenderdir/")
   context.sizePreset(Scene.PC)
@@ -29,7 +29,7 @@ Example::
   context.framesPerSec(15)
   context.startFrame(15)
   context.endFrame(22)
- context.renderAnim()
+  context.renderAnim()
 
   Render.CloseRenderWindow()
   print context.framesPerSec()
@@ -87,38 +87,7 @@ class RenderData:
   =====================
     This object gives access to Scene rendering contexts in Blender.
   """
-  
-  def getRenderdir():
-    """
-    @warn: B{Depricated}: use RenderData.getRenderPath()
-    """
-
-  def getBackbufdir():
-    """
-    @warn: B{Depricated}: use RenderData.getBackbufPath()
-    """
-
-  def getChildren():
-    """
-    Get all objects linked to this Scene.
-    @rtype: list of Blender Objects
-    @return: A list with all Blender Objects linked to this Scene.
-    """
-
-  def getCurrentCamera():
-    """
-    Get the currently active Camera for this Scene.
-    @rtype: Blender Camera
-    @return: The currently active Camera.
-    """
-
-  def setCurrentCamera(camera):
-    """
-    Set the currently active Camera in this Scene.
-    @type camera: Blender Camera
-    @param camera: The new active Camera.
-    """
-
+ 
   def render():
     """
     Render the scene.
@@ -134,6 +103,13 @@ class RenderData:
     play animation of rendered images/avi (searches Pics: field).
     """
 
+  def getTimeCode():
+    """
+    Get the current frame as a string in HH:MM:SS:FF format
+    @rtype: string 
+    @return: current frame as a string in HH:MM:SS:FF format
+    """
+
   def setRenderPath(path):
     """
     Set the path to where the renderer will write to.
@@ -389,6 +365,22 @@ class RenderData:
     @param toggle: pass 1 for on / 0 for off
     """
 
+  def setBorder(left,bottom,right,top):
+    """
+    Set a border for rendering from cameras in the scene.
+    The left,bottom coordinates and right,top coordinates
+    define the size of the border. (0,0,1,1) will set the border
+    to the whole camera. (0,0) lower left and (1,1) upper right.
+    @type left: float
+    @param left: float between 0 and 1
+    @type right: float
+    @param right: float between 0 and 1
+    @type bottom: float
+    @param bottom: float between 0 and 1
+    @type top: float
+    @param top: float between 0 and 1
+    """
+
   def enableGammaCorrection(toggle):
     """
     Enable/disable gamma correction.
@@ -851,4 +843,4 @@ class RenderData:
     @param value: must be between 1 - 900
     @rtype: int (if prototype is empty)
     @return: Current new map value for the scene.
-    """
\ No newline at end of file
+    """
