@@ -1,5 +1,5 @@
 /*
- * $Id: vector.c,v 1.16 2004/10/14 17:35:15 stiv Exp $
+ * $Id: vector.c,v 1.21 2005/05/25 04:52:52 ianwill Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -23,7 +23,7 @@
  * All rights reserved.
  *
  * 
- * Contributor(s): Willian P. Germano & Joseph Gilbert
+ * Contributor(s): Willian P. Germano, Joseph Gilbert, Ken Hughes
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
@@ -55,6 +55,14 @@ struct PyMethodDef Vector_methods[] = {
 	{NULL, NULL, 0, NULL}
 };
 
+/******prototypes*************/
+PyObject *Vector_add( PyObject * v1, PyObject * v2 );
+PyObject *Vector_sub( PyObject * v1, PyObject * v2 );
+PyObject *Vector_mul( PyObject * v1, PyObject * v2 );
+PyObject *Vector_div( PyObject * v1, PyObject * v2 );
+int Vector_coerce( PyObject ** v1, PyObject ** v2 );
+
+
 /*****************************/
 //    Vector Python Object   
 /*****************************/
@@ -542,6 +550,10 @@ int Vector_coerce( PyObject ** v1, PyObject ** v2 )
 				printf( "vector/matrix numeric protocols unsupported...\n" );
 				Py_INCREF( *v1 );
 				return 0;	//operation will type check
+			} else if( *v2 == Py_None ) {
+				Py_INCREF(*v1);
+				Py_INCREF(Py_None);
+				return 0;
 			} else if( PyNumber_Check( *v2 ) ) {
 				if( PyInt_Check( *v2 ) ) {	//cast scalar to vector
 					tempI = PyMem_Malloc( 1 *
@@ -588,6 +600,7 @@ int Vector_coerce( PyObject ** v1, PyObject ** v2 )
 			//unknown type or numeric cast failure
 			printf( "attempting vector operation with unsupported type...\n" );
 			Py_INCREF( *v1 );
+			Py_INCREF( *v2 );
 			return 0;	//operation will type check
 		}
 	} else {
@@ -707,7 +720,6 @@ PyObject *newVectorObject( float *vec, int size )
 PyObject *newVectorProxy( float *vec, int size)
 {
 	VectorObject *proxy;
-	int x;
 
 	proxy = PyObject_NEW( VectorObject, &vector_Type );
 
