@@ -1,7 +1,7 @@
 /*
  * texture_ext.h
  *
- * $Id: texture.h,v 1.6 2003/12/21 21:52:48 ton Exp $
+ * $Id: texture.h,v 1.8 2005/04/25 20:43:28 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -35,12 +35,33 @@
 #ifndef TEXTURE_EXT_H
 #define TEXTURE_EXT_H
 
+#define BRICONT		texres->tin= (texres->tin-0.5)*tex->contrast+tex->bright-0.5; \
+if(texres->tin<0.0) texres->tin= 0.0; else if(texres->tin>1.0) texres->tin= 1.0;
+
+#define BRICONTRGB	texres->tr= tex->rfac*((texres->tr-0.5)*tex->contrast+tex->bright-0.5); \
+if(texres->tr<0.0) texres->tr= 0.0; \
+texres->tg= tex->gfac*((texres->tg-0.5)*tex->contrast+tex->bright-0.5); \
+if(texres->tg<0.0) texres->tg= 0.0; \
+texres->tb= tex->bfac*((texres->tb-0.5)*tex->contrast+tex->bright-0.5); \
+if(texres->tb<0.0) texres->tb= 0.0; 
+
+
 struct HaloRen;
 struct ShadeInput;
+struct TexResult;
+struct Tex;
+struct Image;
+
+/* texture.h */
 
 void do_halo_tex(struct HaloRen *har, float xn, float yn, float *colf);
+void do_sky_tex(float *lo, float *dxyview, float *hor, float *zen, float *blend);
 void render_realtime_texture(struct ShadeInput *shi);           
 
+/* imagetexture.h */
+
+int imagewraposa(struct Tex *tex, struct Image *ima, float *texvec, float *dxt, float *dyt, struct TexResult *texres);
+int imagewrap(struct Tex *tex, struct Image *ima, float *texvec, struct TexResult *texres);
 
 #endif /* TEXTURE_EXT_H */
 
