@@ -1,5 +1,5 @@
 /**
- * $Id: initrender.c,v 1.20 2004/04/24 12:02:39 ton Exp $
+ * $Id: initrender.c,v 1.60 2005/06/09 11:02:06 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,10 +42,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include "blendef.h"
 #include "MEM_guardedalloc.h"
 
@@ -82,21 +78,28 @@
 #include "BSE_drawview.h"
 #include "BSE_sequence.h"
 
+#include "IMB_imbuf_types.h"
+#include "IMB_imbuf.h"
+
 #ifdef WITH_QUICKTIME
 #include "quicktime_export.h"
 #endif
 
+#include "SDL_thread.h"
+
 /* this module */
 #include "render.h"
-#include "render_intern.h"
 
 #include "RE_callbacks.h"
 #include "zbuf.h"
 #include "rendercore.h" /* part handler for the old renderer, shading functions */
+#include "pixelshading.h"
 #include "renderPreAndPost.h"
-#include "outerRenderLoop.h"
+#include "vanillaRenderPipe.h"
 #include "renderHelp.h"
 #include "jitter.h"
+#include "gammaCorrectionTables.h"
+#include "zblur.h"
 
 /* Own includes */
 #include "initrender.h"
@@ -104,72 +107,20 @@
 /* yafray: include for yafray export/render */
 #include "YafRay_Api.h"
 
-/* Some crud :/ */
-#define ELEM3(a, b, c, d)       ( ELEM(a, b, c) || (a)==(d) )
 
-
-/* from render.c */
-extern float fmask[256], centLut[16];
-extern unsigned short *mask1[9], *mask2[9], /*  *igamtab1, */ *igamtab2/*,  *gamtab */;
-extern char cmask[256], *centmask;
+float centLut[16], *fmask1[9], *fmask2[9];
+unsigned short *gamtab, *igamtab2, *igamtab1;
+char cmask[256], *centmask;
 
 Material defmaterial;
-short pa; /* pa is global part, for print */
-short allparts[65][4];
-int qscount;
 
-/* ********************* *********************** */
+/* ------- prototypes ----------- */ 
+void init_filt_mask(void);
 
 
-void init_def_material(void)
-{
-	Material *ma;
-
-	ma= &defmaterial;
-	
-	init_material(&defmaterial);
-
-	init_render_material(ma);
-}
-
-void RE_init_render_data(void)
-{
-	memset(&R, 0, sizeof(RE_Render));
-	memset(&O, 0, sizeof(Osa));
-	O.dxwin[0]= 1.0;
-	O.dywin[1]= 1.0;
-	
-	R.blove= (VertRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Blove");
-	R.blovl= (VlakRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Blovl");
-	R.bloha= (HaloRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Bloha");
-	R.la= (LampRen **)MEM_mallocN(LAMPINITSIZE*sizeof(void *),"renderlamparray");
-
-	init_def_material();
-}
-
-void RE_free_render_data()
-{
-	MEM_freeN(R.blove);
-	R.blove= 0;
-	MEM_freeN(R.blovl);
-	R.blovl= 0;
-	MEM_freeN(R.bloha);
-	R.bloha= 0;
-	MEM_freeN(R.la);
-	R.la= 0;
-	if(R.rectot) MEM_freeN(R.rectot);
-	if(R.rectz) MEM_freeN(R.rectz);
-	if(R.rectspare) MEM_freeN(R.rectspare);
-	R.rectot= 0;
-	R.rectz= 0;
-	R.rectspare= 0;
-
-	end_render_material(&defmaterial);
-}
-
 /* ****************** GAMMA, MASKS and LUTS **************** */
 
-float  calc_weight(float *weight, int i, int j)
+static float calc_weight(float *weight, int i, int j)
 {
 	float x, y, dist, totw= 0.0, fac;
 	int a;
@@ -178,13 +129,13 @@ float  calc_weight(float *weight, int i, int j)
 	fac*= fac;
 
 	for(a=0; a<R.osa; a++) {
-		x= jit[a][0]-0.5+ i;
-		y= jit[a][1]-0.5+ j;
+		x= jit[a][0] + i;
+		y= jit[a][1] + j;
 		dist= sqrt(x*x+y*y);
 
 		weight[a]= 0.0;
 
-		/* gaussian weighting has been cancelled */
+		/* gaussian weighting */
 		if(R.r.mode & R_GAUSS) {
 			if(dist<R.r.gauss) {
 				x = dist*R.r.gauss;
@@ -201,19 +152,23 @@ float  calc_weight(float *weight, int i, int j)
 	return totw;
 }
 
-void RE_init_filt_mask(void)
+// extern called in render_envmap, to disable gauss...
+void init_filt_mask(void)
 {
 	static int firsttime=1;
+	static int lastosa=0;
+	static int lastgauss=0;
 	static float lastgamma= 0.0;
-	float gamma, igamma;
-	float weight[32], totw, val, *fpx1, *fpx2, *fpy1, *fpy2;
+	float gamma, igamma, flweight[32], fmask[256];
+	float weight[32], totw, val, *fpx1, *fpx2, *fpy1, *fpy2, *m3, *m4;
 	int i, j, a;
-	unsigned short *m1, *m2, shweight[32];
 
 	if(firsttime) {
+		firsttime= 0;
+		
 		for(a=0; a<9;a++) {
-			mask1[a]= MEM_mallocN(256*sizeof(short), "initfilt");
-			mask2[a]= MEM_mallocN(256*sizeof(short), "initfilt");
+			fmask1[a]= MEM_mallocN(256*sizeof(float), "initfilt");
+			fmask2[a]= MEM_mallocN(256*sizeof(float), "initfilt");
 		}
 		for(a=0; a<256; a++) {
 			cmask[a]= 0;
@@ -234,13 +189,14 @@ void RE_init_filt_mask(void)
 		gamtab= MEM_mallocN(65536*sizeof(short), "initGaus2");
 		igamtab1= MEM_mallocN(256*sizeof(short), "initGaus2");
 		igamtab2= MEM_mallocN(65536*sizeof(short), "initGaus2");
-
+		
+		return;	// this case is called on startup
 	}
 
 	if(R.r.alphamode==R_ALPHAKEY) gamma= 1.0;	/* gamma correction of alpha is nasty */
-
-	if(R.r.mode & R_GAMMA) gamma= 2.0;
+	else if(R.r.mode & R_GAMMA) gamma= 2.0;
 	else gamma= 1.0;
+	
 	igamma= 1.0/gamma;
 
 	if(gamma!= lastgamma) {
@@ -277,159 +233,160 @@ void RE_init_filt_mask(void)
 		}
 	}
 
-	if(firsttime) {
-		firsttime= 0;
-		return;
-	}
-
-	val= 1.0/((float)R.osa);
-	for(a=0; a<256; a++) {
-		fmask[a]= ((float)cmask[a])*val;
-	}
-
-	for(a=0; a<9;a++) {
-		memset(mask1[a], 0, 256*2);
-		memset(mask2[a], 0, 256*2);
-	}
-
-	/* calculate totw */
-	totw= 0.0;
-	for(j= -1; j<2; j++) {
-		for(i= -1; i<2; i++) {
-			totw+= calc_weight(weight, i, j);
+	if(R.osa && (lastosa!=R.osa || lastgauss != (R.r.mode & R_GAUSS)) ) {
+		lastosa= R.osa;
+		lastgauss= R.r.mode & R_GAUSS;
+			
+		val= 1.0/((float)R.osa);
+		for(a=0; a<256; a++) {
+			fmask[a]= ((float)cmask[a])*val;
 		}
-	}
 
-	for(j= -1; j<2; j++) {
-		for(i= -1; i<2; i++) {
-			/* calculate using jit, with offset the weights */
+		for(a=0; a<9;a++) {
+			memset(fmask1[a], 0, 256*sizeof(float));
+			memset(fmask2[a], 0, 256*sizeof(float));
+		}
 
-			memset(weight, 0, 32*2);
-			calc_weight(weight, i, j);
+		/* calculate totw */
+		totw= 0.0;
+		for(j= -1; j<2; j++) {
+			for(i= -1; i<2; i++) {
+				totw+= calc_weight(weight, i, j);
+			}
+		}
 
-			for(a=0; a<16; a++) shweight[a]= weight[a]*(65535.0/totw);
+		for(j= -1; j<2; j++) {
+			for(i= -1; i<2; i++) {
+				/* calculate using jit, with offset the weights */
 
-			m1= mask1[ 3*(j+1)+i+1 ];
-			m2= mask2[ 3*(j+1)+i+1 ];
+				memset(weight, 0, sizeof(weight));
+				calc_weight(weight, i, j);
 
-			for(a=0; a<256; a++) {
-				if(a &   1) {
-					m1[a]+= shweight[0];
-					m2[a]+= shweight[8];
-				}
-				if(a &   2) {
-					m1[a]+= shweight[1];
-					m2[a]+= shweight[9];
-				}
-				if(a &   4) {
-					m1[a]+= shweight[2];
-					m2[a]+= shweight[10];
-				}
-				if(a &   8) {
-					m1[a]+= shweight[3];
-					m2[a]+= shweight[11];
-				}
-				if(a &  16) {
-					m1[a]+= shweight[4];
-					m2[a]+= shweight[12];
-				}
-				if(a &  32) {
-					m1[a]+= shweight[5];
-					m2[a]+= shweight[13];
-				}
-				if(a &  64) {
-					m1[a]+= shweight[6];
-					m2[a]+= shweight[14];
-				}
-				if(a & 128) {
-					m1[a]+= shweight[7];
-					m2[a]+= shweight[15];
+				for(a=0; a<16; a++) flweight[a]= weight[a]*(1.0/totw);
+
+				m3= fmask1[ 3*(j+1)+i+1 ];
+				m4= fmask2[ 3*(j+1)+i+1 ];
+
+				for(a=0; a<256; a++) {
+					if(a &   1) {
+						m3[a]+= flweight[0];
+						m4[a]+= flweight[8];
+					}
+					if(a &   2) {
+						m3[a]+= flweight[1];
+						m4[a]+= flweight[9];
+					}
+					if(a &   4) {
+						m3[a]+= flweight[2];
+						m4[a]+= flweight[10];
+					}
+					if(a &   8) {
+						m3[a]+= flweight[3];
+						m4[a]+= flweight[11];
+					}
+					if(a &  16) {
+						m3[a]+= flweight[4];
+						m4[a]+= flweight[12];
+					}
+					if(a &  32) {
+						m3[a]+= flweight[5];
+						m4[a]+= flweight[13];
+					}
+					if(a &  64) {
+						m3[a]+= flweight[6];
+						m4[a]+= flweight[14];
+					}
+					if(a & 128) {
+						m3[a]+= flweight[7];
+						m4[a]+= flweight[15];
+					}
 				}
 			}
 		}
+
+		/* centmask: the correct subpixel offset per mask */
+
+		fpx1= MEM_mallocN(256*sizeof(float), "initgauss4");
+		fpx2= MEM_mallocN(256*sizeof(float), "initgauss4");
+		fpy1= MEM_mallocN(256*sizeof(float), "initgauss4");
+		fpy2= MEM_mallocN(256*sizeof(float), "initgauss4");
+		for(a=0; a<256; a++) {
+			fpx1[a]= fpx2[a]= 0.0;
+			fpy1[a]= fpy2[a]= 0.0;
+			if(a & 1) {
+				fpx1[a]+= jit[0][0];
+				fpy1[a]+= jit[0][1];
+				fpx2[a]+= jit[8][0];
+				fpy2[a]+= jit[8][1];
+			}
+			if(a & 2) {
+				fpx1[a]+= jit[1][0];
+				fpy1[a]+= jit[1][1];
+				fpx2[a]+= jit[9][0];
+				fpy2[a]+= jit[9][1];
+			}
+			if(a & 4) {
+				fpx1[a]+= jit[2][0];
+				fpy1[a]+= jit[2][1];
+				fpx2[a]+= jit[10][0];
+				fpy2[a]+= jit[10][1];
+			}
+			if(a & 8) {
+				fpx1[a]+= jit[3][0];
+				fpy1[a]+= jit[3][1];
+				fpx2[a]+= jit[11][0];
+				fpy2[a]+= jit[11][1];
+			}
+			if(a & 16) {
+				fpx1[a]+= jit[4][0];
+				fpy1[a]+= jit[4][1];
+				fpx2[a]+= jit[12][0];
+				fpy2[a]+= jit[12][1];
+			}
+			if(a & 32) {
+				fpx1[a]+= jit[5][0];
+				fpy1[a]+= jit[5][1];
+				fpx2[a]+= jit[13][0];
+				fpy2[a]+= jit[13][1];
+			}
+			if(a & 64) {
+				fpx1[a]+= jit[6][0];
+				fpy1[a]+= jit[6][1];
+				fpx2[a]+= jit[14][0];
+				fpy2[a]+= jit[14][1];
+			}
+			if(a & 128) {
+				fpx1[a]+= jit[7][0];
+				fpy1[a]+= jit[7][1];
+				fpx2[a]+= jit[15][0];
+				fpy2[a]+= jit[15][1];
+			}
+		}
+
+		for(a= (1<<R.osa)-1; a>0; a--) {
+			val= count_mask(a);
+			i= 8+(15.9*(fpy1[a & 255]+fpy2[a>>8])/val);
+			CLAMP(i, 0, 15);
+			j= 8+(15.9*(fpx1[a & 255]+fpx2[a>>8])/val);
+			CLAMP(j, 0, 15);
+			i= j + (i<<4);
+			centmask[a]= i;
+		}
+
+		MEM_freeN(fpx1);
+		MEM_freeN(fpx2);
+		MEM_freeN(fpy1);
+		MEM_freeN(fpy2);
 	}
-
-	/* centmask: the correct subpixel offset per mask */
-
-	fpx1= MEM_mallocN(256*sizeof(float), "initgauss4");
-	fpx2= MEM_mallocN(256*sizeof(float), "initgauss4");
-	fpy1= MEM_mallocN(256*sizeof(float), "initgauss4");
-	fpy2= MEM_mallocN(256*sizeof(float), "initgauss4");
-	for(a=0; a<256; a++) {
-		fpx1[a]= fpx2[a]= 0.0;
-		fpy1[a]= fpy2[a]= 0.0;
-		if(a & 1) {
-			fpx1[a]+= jit[0][0];
-			fpy1[a]+= jit[0][1];
-			fpx2[a]+= jit[8][0];
-			fpy2[a]+= jit[8][1];
-		}
-		if(a & 2) {
-			fpx1[a]+= jit[1][0];
-			fpy1[a]+= jit[1][1];
-			fpx2[a]+= jit[9][0];
-			fpy2[a]+= jit[9][1];
-		}
-		if(a & 4) {
-			fpx1[a]+= jit[2][0];
-			fpy1[a]+= jit[2][1];
-			fpx2[a]+= jit[10][0];
-			fpy2[a]+= jit[10][1];
-		}
-		if(a & 8) {
-			fpx1[a]+= jit[3][0];
-			fpy1[a]+= jit[3][1];
-			fpx2[a]+= jit[11][0];
-			fpy2[a]+= jit[11][1];
-		}
-		if(a & 16) {
-			fpx1[a]+= jit[4][0];
-			fpy1[a]+= jit[4][1];
-			fpx2[a]+= jit[12][0];
-			fpy2[a]+= jit[12][1];
-		}
-		if(a & 32) {
-			fpx1[a]+= jit[5][0];
-			fpy1[a]+= jit[5][1];
-			fpx2[a]+= jit[13][0];
-			fpy2[a]+= jit[13][1];
-		}
-		if(a & 64) {
-			fpx1[a]+= jit[6][0];
-			fpy1[a]+= jit[6][1];
-			fpx2[a]+= jit[14][0];
-			fpy2[a]+= jit[14][1];
-		}
-		if(a & 128) {
-			fpx1[a]+= jit[7][0];
-			fpy1[a]+= jit[7][1];
-			fpx2[a]+= jit[15][0];
-			fpy2[a]+= jit[15][1];
-		}
-	}
-
-	for(a= (1<<R.osa)-1; a>0; a--) {
-		val= count_mask(a);
-		i= (15.9*(fpy1[a & 255]+fpy2[a>>8])/val);
-		i<<=4;
-		i+= (15.9*(fpx1[a & 255]+fpx2[a>>8])/val);
-		centmask[a]= i;
-	}
-
-	MEM_freeN(fpx1);
-	MEM_freeN(fpx2);
-	MEM_freeN(fpy1);
-	MEM_freeN(fpy2);
-
 }
 
-void RE_free_filt_mask()
+static void free_filt_mask()
 {
 	int a;
 
 	for(a=0; a<9; a++) {
-		MEM_freeN(mask1[a]);
-		MEM_freeN(mask2[a]);
+		MEM_freeN(fmask1[a]);
+		MEM_freeN(fmask2[a]);
 	}
 	MEM_freeN(gamtab);
 	MEM_freeN(igamtab1);
@@ -438,9 +395,9 @@ void RE_free_filt_mask()
 	MEM_freeN(centmask);
 }
 
-/* add stuff */
-
+/* unused */
 
+#if 0
 void defaultlamp()
 {
 	LampRen *lar;
@@ -458,8 +415,60 @@ void defaultlamp()
 	lar->b= 1.0;
 	lar->lay= 65535;
 }
+#endif
 
 
+
+/* ********************* init calls *********************** */
+
+
+static void init_def_material(void)
+{
+	Material *ma;
+	
+	ma= &defmaterial;
+	
+	init_material(&defmaterial);
+	
+	init_render_material(ma);
+}
+
+void RE_init_render_data(void)
+{
+	memset(&R, 0, sizeof(RE_Render));
+	
+	R.blove= (VertRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Blove");
+	R.blovl= (VlakRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Blovl");
+	R.bloha= (HaloRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Bloha");
+	R.la= (LampRen **)MEM_mallocN(LAMPINITSIZE*sizeof(void *),"renderlamparray");
+	
+	init_def_material();
+	init_filt_mask();
+}
+
+void RE_free_render_data()
+{
+	MEM_freeN(R.blove);
+	R.blove= NULL;
+	MEM_freeN(R.blovl);
+	R.blovl= NULL;
+	MEM_freeN(R.bloha);
+	R.bloha= NULL;
+	MEM_freeN(R.la);
+	R.la= NULL;
+	if(R.rectot) MEM_freeN(R.rectot);
+	if(R.rectftot) MEM_freeN(R.rectftot);
+	if(R.rectz) MEM_freeN(R.rectz);
+	if(R.rectspare) MEM_freeN(R.rectspare);
+	R.rectot= NULL;
+	R.rectftot= NULL;
+	R.rectz= NULL;
+	R.rectspare= NULL;
+	
+	end_render_material(&defmaterial);
+	free_filt_mask();
+}
+
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
 
 void RE_make_existing_file(char *name)
@@ -481,11 +490,11 @@ void RE_make_existing_file(char *name)
 void RE_setwindowclip(int mode, int jmode)
 {
 	extern float bluroffsx, bluroffsy;	// rendercore.c... hackish (ton)
-	Camera *cam=0;
+	Camera *cam=NULL;
 	float lens, minx, miny, maxx, maxy;
 	float xd, yd, afmx, afmy;
 
-	if(G.scene->camera==0) return;
+	if(G.scene->camera==NULL) return;
 
 	afmx= R.afmx;
 	afmy= R.afmy;
@@ -512,37 +521,39 @@ void RE_setwindowclip(int mode, int jmode)
 			lens= 16.0;
 		}
 
-		if( (R.r.xasp*afmx) >= (R.r.yasp*afmy) ) {
-			R.viewfac= (afmx*lens)/16.0;
+		if(R.r.mode & R_ORTHO) {
+			if( (R.r.xasp*afmx) >= (R.r.yasp*afmy) ) {
+				R.viewfac= 2.0*afmx;
+			}
+			else {
+				R.viewfac= 2.0*R.ycor*afmy;
+			}
+			/* ortho_scale == 1.0 means exact 1 to 1 mapping */
+			R.pixsize= cam->ortho_scale/R.viewfac;
 		}
 		else {
-			R.viewfac= R.ycor*(afmy*lens)/16.0;
+			if( (R.r.xasp*afmx) >= (R.r.yasp*afmy) ) {
+				R.viewfac= (afmx*lens)/16.0;
+			}
+			else {
+				R.viewfac= R.ycor*(afmy*lens)/16.0;
+			}
+			
+			R.pixsize= R.near/R.viewfac;
 		}
-		if(R.r.mode & R_ORTHO) {
-			R.near*= 100.0; 
-			R.viewfac*= 100.0;
-		}
-
-		R.pixsize= R.near/R.viewfac;
-
+		
+		/* pixsize is not a real global... get rid of it! (ton) */
 	}
 
-	/* I think these offsets are wrong. They do not coincide with shadow     */
-	/* calculations, btw.                                                    */	
-  	minx= R.xstart+.5; 
-  	miny= R.ycor*(R.ystart+.5); 
-  	maxx= R.xend+.4999; 
-  	maxy= R.ycor*(R.yend+.4999); 
-	/* My guess: (or rather, what should be) */
-	/*    	minx= R.xstart - 0.5;  */
-	/*    	miny= R.ycor * (R.ystart - 0.5);  */
-	/* Since the SCS-s map directly to the pixel center coordinates, we need */
-	/* to stretch the clip area a bit, not just shift it. However, this gives*/
-	/* nasty problems for parts...                                           */
-
-	/* Dunno who wrote previous comment, but I found an error with uncorrected
-	   blur offset in shadepixel(). Now solved with 2 globals, seems to work.
-	   The whole method how to retrieve the correct coordinate needs revision. (ton) */
+	/* revision / simplification of subpixel offsets:
+	   - the matrix will go without offset from start (e.g. -100) to end (e.g. +99).
+	   - filling in with zbuffer will set offset of 0.5. to make sure clipped faces fill in too
+	   - in shadepixel() again that 0.5 offset is corrected
+	*/
+  	minx= R.xstart; 
+  	miny= R.ycor*(R.ystart); 
+  	maxx= R.xend; 
+  	maxy= R.ycor*(R.yend); 
 	   
 	if(R.flag & R_SEC_FIELD) {
 		if(R.r.mode & R_ODDFIELD) {
@@ -567,20 +578,52 @@ void RE_setwindowclip(int mode, int jmode)
 	miny= R.pixsize*(miny+yd);
 	maxy= R.pixsize*(maxy+yd);
 
-	if(R.r.mode & R_ORTHO) {
-		i_window(minx, maxx, miny, maxy, R.near, 100.0*R.far, R.winmat);
-	}
-	else i_window(minx, maxx, miny, maxy, R.near, R.far, R.winmat);
+	if(R.r.mode & R_ORTHO)
+		i_ortho(minx, maxx, miny, maxy, R.near, R.far, R.winmat);
+	else 
+		i_window(minx, maxx, miny, maxy, R.near, R.far, R.winmat);
 
+//	printmatrix4("win", R.winmat);
 }
 
 
-/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
-void initparts()
+/* ~~~~~~~~~~~~~~~~ PARTS ~~~~~~~~~~~~~~~~~~~~~~ */
+
+/** 
+* Part as in part-rendering. An image rendered in parts is rendered
+* to a list of parts, with x,y size, and a pointer to the render
+* output stored per part. Internal!
+*/
+typedef struct Part
 {
+	struct Part *next, *prev;
+	unsigned int *rect;		// color 4x8 bits
+	float *rectf;			// color 4x32 bits
+	unsigned int *rectz;	// zbuffer
+
+	short minx, miny, maxx, maxy, x, y;
+} Part;
+
+static void freeparts(void)
+{
+	Part *part= R.parts.first;
+	while(part) {
+		if(part->rect) MEM_freeN(part->rect);
+		if(part->rectz) MEM_freeN(part->rectz);
+		if(part->rectf) MEM_freeN(part->rectf);
+		part= part->next;
+	}
+	BLI_freelistN(&R.parts);
+}
+
+static void initparts(void)
+{
+	Part *pa;
 	short nr, xd, yd, xpart, ypart, xparts, yparts;
 	short a, xminb, xmaxb, yminb, ymaxb;
 
+	freeparts();
+	
 	if(R.r.mode & R_BORDER) {
 		xminb= R.r.border.xmin*R.rectx;
 		xmaxb= R.r.border.xmax*R.rectx;
@@ -602,9 +645,6 @@ void initparts()
 	xparts= R.r.xparts;	/* for border */
 	yparts= R.r.yparts;
 
-	for(nr=0;nr<xparts*yparts;nr++)
-		allparts[nr][0]= -1;	/* clear array */
-
 	xpart= R.rectx/xparts;
 	ypart= R.recty/yparts;
 
@@ -618,65 +658,110 @@ void initparts()
 		xpart= (xmaxb-xminb)/xparts;
 		ypart= (ymaxb-yminb)/yparts;
 	}
-
+	
 	for(nr=0; nr<xparts*yparts; nr++) {
-
+		pa= MEM_callocN(sizeof(Part), "new part");
+		
 		if(R.r.mode & R_PANORAMA) {
-			allparts[nr][0]= 0;
-			allparts[nr][1]= 0;
-			allparts[nr][2]= R.rectx;
-			allparts[nr][3]= R.recty;
+			pa->minx= pa->miny= 0;
+			pa->maxx= pa->x= R.rectx;
+			pa->maxy= pa->y= R.recty;
 		}
 		else {
 			xd= (nr % xparts);
 			yd= (nr-xd)/xparts;
 
-			allparts[nr][0]= xminb+ xd*xpart;
-			allparts[nr][1]= yminb+ yd*ypart;
-			if(xd<R.r.xparts-1) allparts[nr][2]= allparts[nr][0]+xpart;
-			else allparts[nr][2]= xmaxb;
-			if(yd<R.r.yparts-1) allparts[nr][3]= allparts[nr][1]+ypart;
-			else allparts[nr][3]= ymaxb;
+			pa->minx= xminb+ xd*xpart;
+			pa->miny= yminb+ yd*ypart;
+			if(xd<R.r.xparts-1) pa->maxx= pa->minx+xpart;
+			else pa->maxx= xmaxb;
+			if(yd<R.r.yparts-1) pa->maxy= pa->miny+ypart;
+			else pa->maxy= ymaxb;
 
-			if(allparts[nr][2]-allparts[nr][0]<=0) allparts[nr][0]= -1;
-			if(allparts[nr][3]-allparts[nr][1]<=0) allparts[nr][0]= -1;
+			pa->x= pa->maxx-pa->minx;
+			pa->y= pa->maxy-pa->miny;
 		}
+		
+		if(pa->x>0 && pa->y>0) {
+			/* Gauss needs 1 pixel extra to work */
+			if(xparts*yparts>1 && (R.r.mode & R_GAUSS)) {
+				pa->minx-= 1;
+				pa->miny-= 1;
+				pa->maxx+= 1;
+				pa->maxy+= 1;
+				pa->x+= 2;
+				pa->y+= 2;
+			}
+			BLI_addtail(&R.parts, pa);
+		}
+		else MEM_freeN(pa);
 	}
+	
 }
 
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
-short setpart(short nr)	/* return 0 if incorrect part */
+static void setpart(Part *pa)
 {
 
-	if(allparts[nr][0]== -1) return 0;
-
-	R.xstart= allparts[nr][0]-R.afmx;
-	R.ystart= allparts[nr][1]-R.afmy;
-	R.xend= allparts[nr][2]-R.afmx;
-	R.yend= allparts[nr][3]-R.afmy;
-	R.rectx= R.xend-R.xstart;
-	R.recty= R.yend-R.ystart;
-
-	return 1;
+	R.xstart= pa->minx-R.afmx;
+	R.ystart= pa->miny-R.afmy;
+	R.xend= pa->maxx-R.afmx;
+	R.yend= pa->maxy-R.afmy;
+	R.rectx= pa->x;
+	R.recty= pa->y;
 }
 
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
-void addparttorect(short nr, Part *part)
+static void addparttorect(Part *pa)
 {
-	unsigned int *rt, *rp;
-	short y, heigth, len;
+	float *rf, *rfp;
+	unsigned int *rt, *rtp, *rz, *rzp;
+	int y, heigth, len, copylen;
 
-	/* the right offset in rectot */
+	/* calc the right offset in rects, zbuffer cannot exist... */
 
-	rt= R.rectot+ (allparts[nr][1]*R.rectx+ allparts[nr][0]);
-	rp= part->rect;
-	len= (allparts[nr][2]-allparts[nr][0]);
-	heigth= (allparts[nr][3]-allparts[nr][1]);
+	if(pa->rect==NULL) return;
+	
+	rtp= pa->rect;
+	rzp= pa->rectz;
+	rfp= pa->rectf;
+	
+	copylen=len= pa->x;
+	heigth= pa->y;
+	
+	if(R.r.mode & R_GAUSS) {
+		
+		rtp+= 1+len;
+		if(rzp) rzp+= 1+len;
+		if(rfp) rfp+= 4*(1+len);
+		
+		copylen= len-2;
+		heigth -= 2;
+		rt= R.rectot+ (pa->miny + 1)*R.rectx+ (pa->minx+1);
+		rz= R.rectz+ (pa->miny + 1)*R.rectx+ (pa->minx+1);
+		rf= R.rectftot+ 4*( (pa->miny + 1)*R.rectx + (pa->minx+1) );
+	}
+	else {
+		rt= R.rectot+ pa->miny*R.rectx+ pa->minx;
+		rz= R.rectz+ pa->miny*R.rectx+ pa->minx;
+		rf= R.rectftot+ 4*(pa->miny*R.rectx+ pa->minx);
+	}
 
-	for(y=0;y<heigth;y++) {
-		memcpy(rt, rp, 4*len);
-		rt+=R.rectx;
-		rp+= len;
+	for(y=0; y<heigth; y++) {
+		memcpy(rt, rtp, 4*copylen);
+		rt+= R.rectx;
+		rtp+= len;
+		
+		if(rzp) {
+			memcpy(rz, rzp, 4*copylen);
+			rz+= R.rectx;
+			rzp+= len;
+		}
+		if(rfp) {
+			memcpy(rf, rfp, 16*copylen);
+			rf+= 4*R.rectx;
+			rfp+= 4*len;
+		}
 	}
 }
 
@@ -684,7 +769,7 @@ void addparttorect(short nr, Part *part)
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
 
  
-void add_to_blurbuf(int blur)
+static void add_to_blurbuf(int blur)
 {
 	static unsigned int *blurrect= 0;
 	int tot, gamval;
@@ -748,57 +833,113 @@ void add_to_blurbuf(int blur)
 
 
 /* yafray: main yafray render/export call */
-void yafrayRender()
+static void yafrayRender(void)
 {
 	R.flag |= R_RENDERING;	/* !!! */
+
+	/* all allocs moved here, out of export code */
+	/* display rgba buf */
+	if (R.rectot) MEM_freeN(R.rectot);
+	R.rectot = MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
+	/* zbuf */
+	if (R.rectz) MEM_freeN(R.rectz);
+	R.rectz = (unsigned int *)MEM_mallocN(sizeof(int)*R.rectx*R.recty, "rectz");
+	/* float rgba buf */
+	if (R.rectftot) MEM_freeN(R.rectftot);
+	if (R.r.mode & R_FBUF) R.rectftot= MEM_callocN(4*sizeof(float)*R.rectx*R.recty, "rectftot");
+
+	// switch must be done before prepareScene()
+	if (!R.r.YFexportxml)
+		YAF_switchFile();
+	else
+		YAF_switchPlugin();
+
+	RE_local_init_render_display();
+	RE_local_clear_render_display(R.win);
+	RE_local_timecursor((G.scene->r.cfra));
+
 	printf("Starting scene conversion.\n");
 	prepareScene();
 	printf("Scene conversion done.\n");
+
 	YAF_exportScene();
 	finalizeScene();
-}
 
-
-/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
-void render() {
-	/* yafray: render, see above */
-	if (R.r.renderer==R_YAFRAY)
-		yafrayRender();
-	else {
-		/* not too neat... should improve... */
-		if(R.r.mode & R_UNIFIED) {
-			unifiedRenderingLoop();
-		} else {
-			oldRenderLoop();
-		}
+	// show postpro effects if floatbuffer used (plugin only)
+	if (R.r.YFexportxml) {
+		if ((R.r.mode & R_FBUF) && R.rectftot)
+			RE_floatbuffer_to_output();
 	}
 }
 
 
-void oldRenderLoop(void)  /* here the PART and FIELD loops */
+/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
+
+// exported to other files, belongs in include... later
+SDL_mutex *render_abuf_lock=NULL, *load_ibuf_lock=NULL, *make_table_lock= NULL;
+
+
+static void renderloop_setblending(void)
 {
-	Part *part;
-	unsigned int *rt, *rt1, *rt2;
-	int len, y;
-	short blur, a,fields,fi,parts;  /* pa is a global because of print */
+	
+	/* this value should only be set here. do_gamma is for gammablended adding of subpixels */
+	do_gamma= 0;
+	if(R.r.mode & R_GAMMA) {
+		if(R.r.alphamode==R_ALPHAKEY);	// alpha corrected gamma doesnt work for key alpha
+		else if((R.r.mode & R_OSA)) do_gamma= 1;
+	}
+	
+	/* always call, it does gamma tables used by alphaunder, but call after R.osa and jit was set */
+	init_filt_mask();
+	
+	switch (R.r.alphamode) {
+		case R_ALPHAKEY:	
+			setSkyBlendingMode(RE_ALPHA_KEY);
+			break;
+		case R_ALPHAPREMUL:	
+			setSkyBlendingMode(RE_ALPHA_PREMUL);
+			break;
+		default:
+			setSkyBlendingMode(RE_ALPHA_SKY);		
+	}
+	
+	/* SHould use slider when the gamma button is pressed. */
+	if (do_gamma) {		
+		makeGammaTables(2.0);
+	} else {
+		makeGammaTables(1.0);
+	}
+	
+}
 
-	if (R.rectz) MEM_freeN(R.rectz);
-	R.rectz = 0;
+static void mainRenderLoop(void)  /* here the PART and FIELD loops */
+{
+	Part *pa;
+	int blur, fields, fi, totparts, nr;
 
+	/* create mutexes for threaded render */
+	render_abuf_lock = SDL_CreateMutex();
+	load_ibuf_lock = SDL_CreateMutex();
+	make_table_lock = SDL_CreateMutex();
+	
+	if(R.rectz) MEM_freeN(R.rectz);
+	R.rectz = NULL;
+	if(R.rectftot) MEM_freeN(R.rectftot);
+	R.rectftot = NULL;
+	
 	/* FIELD LOOP */
+	totparts= R.r.xparts*R.r.yparts;
 	fields= 1;
-	parts= R.r.xparts*R.r.yparts;
 
 	if(R.r.mode & R_FIELDS) {
 		fields= 2;
-		R.rectf1= R.rectf2= 0;	/* field rects */
+		R.rectf1= R.rectf2= NULL;	/* field rects */
 		R.r.ysch/= 2;
 		R.afmy/= 2;
 		R.r.yasp*= 2;
 		R.ycor= ( (float)R.r.yasp)/( (float)R.r.xasp);
 
 	}
-
 	
 	for(fi=0; fi<fields; fi++) {
 
@@ -825,7 +966,7 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 			if(R.r.mode & R_MBLUR) set_mblur_offs(R.osa-blur);
 
 			initparts(); /* always do, because of border */
-			setpart(0);
+			setpart(R.parts.first);
 
 			RE_local_init_render_display();
 			RE_local_clear_render_display(R.win);
@@ -833,20 +974,18 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 
 			prepareScene();
 
-			/* PARTS */
-			R.parts.first= R.parts.last= 0;
-			for(pa=0; pa<parts; pa++) {
+			/* PARTS LOOP */
+			nr= 0;
+			for(pa= R.parts.first; pa; pa= pa->next, nr++) {
 
 				if(RE_local_test_break()) break;
 
-				if(pa) {	/* because case pa==0 has been done */
-					if(setpart(pa)==0) break;
-				}
+				setpart(pa);
 
 				if(R.r.mode & R_MBLUR) RE_setwindowclip(0, blur);
 				else RE_setwindowclip(0,-1);
 
-				if(R.r.mode & R_PANORAMA) setPanoRot(pa);
+				if(R.r.mode & R_PANORAMA) setPanoRot(nr);
 
 				/* HOMOGENIC COORDINATES AND ZBUF AND CLIP OPTIMISATION (per part) */
 				/* There may be some interference with z-coordinate    */
@@ -854,42 +993,47 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 
 				doClipping(RE_projectverto);
 				if(RE_local_test_break()) break;
-
-
-				/* ZBUFFER & SHADE: zbuffer stores integer distances, and integer face indices */
-				R.rectot= (unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
-				R.rectz =  (unsigned int *)MEM_mallocN(sizeof(int)*R.rectx*R.recty, "rectz");
-
+				
+				/* rectot is for result and integer face indices */
+				if(R.rectot) MEM_freeN(R.rectot);
+				R.rectot= MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
+				
+				if(R.rectftot) MEM_freeN(R.rectftot);
+				if(R.r.mode & R_FBUF) R.rectftot= MEM_callocN(4*sizeof(float)*R.rectx*R.recty, "rectftot");
+				
 				if(R.r.mode & R_MBLUR) {
 					RE_local_printrenderinfo(0.0, R.osa - blur);
 					if(G.background && blur<R.osa) printf("\n"); // newline for percentage print
 				}
 				else RE_local_printrenderinfo(0.0, -1);
 
-				/* choose render pipeline type, and whether or not to use the */
-				/* delta accumulation buffer. 3 choices.                      */
-				if(R.r.mode & R_OSA) zbufshadeDA();
-				else                 zbufshade();
-
-				if(RE_local_test_break()) break;
+				if(R.r.mode & R_UNIFIED) {
+					zBufShadeAdvanced();
+				}
+				else {
+					if(R.rectz) MEM_freeN(R.rectz);
+					R.rectz =  (unsigned int *)MEM_mallocN(sizeof(int)*R.rectx*R.recty, "rectz");
 
+					if(R.r.mode & R_OSA) zbufshadeDA();
+					else zbufshade();
+				}
+				
 				/* exception */
 				if( (R.r.mode & R_BORDER) && (R.r.mode & R_MOVIECROP));
 				else {
 					/* HANDLE PART OR BORDER */
-					if(parts>1 || (R.r.mode & R_BORDER)) {
+					if(totparts>1 || (R.r.mode & R_BORDER)) {
 
-						part= MEM_callocN(sizeof(Part), "part");
-						BLI_addtail(&R.parts, part);
-						part->rect= R.rectot;
+						pa->rect= R.rectot;
 						R.rectot= NULL;
-
-						if (R.rectz) {
-							MEM_freeN(R.rectz);
-							R.rectz= NULL;
-						}
+						pa->rectf= R.rectftot;
+						R.rectftot= NULL;
+						pa->rectz= R.rectz;
+						R.rectz= NULL;
 					}
 				}
+
+				if(RE_local_test_break()) break;
 			}
 
 			/* JOIN PARTS OR INSERT BORDER */
@@ -902,40 +1046,39 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 
 				if(R.r.mode & R_PANORAMA) R.rectx*= R.r.xparts;
 
-				if(parts>1 || (R.r.mode & R_BORDER)) {
+				if(totparts>1 || (R.r.mode & R_BORDER)) {
+					int a;
+					
 					if(R.rectot) MEM_freeN(R.rectot);
+					if(R.rectftot) MEM_freeN(R.rectftot);
+					if(R.rectz) MEM_freeN(R.rectz);
 					
-					R.rectot=(unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
+					R.rectot= MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
+					R.rectz= MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectz");
+					if(R.r.mode & R_FBUF) R.rectftot= MEM_callocN(4*sizeof(float)*R.rectx*R.recty, "rectftot");
+					else R.rectftot= NULL;
 					
-					part= R.parts.first;
-					for(pa=0; pa<parts; pa++) {
-						if(allparts[pa][0]== -1) break;
-						if(part==0) break;
+					for(a=0, pa= R.parts.first; pa; pa= pa->next, a++) {
 						
-						if(R.r.mode & R_PANORAMA) {
-							if(pa) {
-								allparts[pa][0] += pa*R.r.xsch;
-								allparts[pa][2] += pa*R.r.xsch;
-							}
+						if(R.r.mode & R_PANORAMA) {		// pano is fake parts...
+							pa->minx += a*R.r.xsch;
+							pa->maxx += a*R.r.xsch;
 						}
-						addparttorect(pa, part);
-						
-						part= part->next;
+						addparttorect(pa);
 					}
-					
-					part= R.parts.first;
-					while(part) {
-						MEM_freeN(part->rect);
-						part= part->next;
-					}
-					BLI_freelistN(&R.parts);
 				}
 			}
-
+			
+			freeparts();
+			
 			if( (R.flag & R_HALO)) {
 				if(RE_local_test_break()==0) add_halo_flare();
 			}
 
+			if( (R.r.mode & R_ZBLUR)) {
+				if(RE_local_test_break()==0) add_zblur();
+			}
+
 			if(R.r.mode & R_MBLUR) {
 				add_to_blurbuf(blur);
 			}
@@ -953,7 +1096,7 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 		if(R.r.mode & R_FIELDS) {
 			if(R.flag & R_SEC_FIELD) R.rectf2= R.rectot;
 			else R.rectf1= R.rectot;
-			R.rectot= 0;
+			R.rectot= NULL;
 		}
 
 		if(RE_local_test_break()) break;
@@ -968,8 +1111,11 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 
 		if(R.rectot) MEM_freeN(R.rectot);	/* happens when a render has been stopped */
 		R.rectot=(unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
-
+		
 		if(RE_local_test_break()==0) {
+			unsigned int *rt, *rt1, *rt2;
+			int len, a;
+			
 			rt= R.rectot;
 
 			if(R.r.mode & R_ODDFIELD) {
@@ -992,48 +1138,60 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 				rt2+= R.rectx;
 			}
 		}
+		
+		if(R.rectf1) MEM_freeN(R.rectf1);
+		R.rectf1= NULL;
+		if(R.rectf2) MEM_freeN(R.rectf2);
+		R.rectf2= NULL;
+		/* fbuf and zbuf free, image size differs now */
+		if(R.rectftot) MEM_freeN(R.rectftot);
+		R.rectftot= NULL;
+		if(R.rectz) MEM_freeN(R.rectz);
+		R.rectz= NULL;
+		
 	}
 
-	/* R.rectx= R.r.xsch; */
-	/* if(R.r.mode & R_PANORAMA) R.rectx*= R.r.xparts; */
-	/* R.recty= R.r.ysch; */
-
 	/* if border: still do skybuf */
 	if(R.r.mode & R_BORDER) {
 		if( (R.r.mode & R_MOVIECROP)==0) {
 			if(R.r.bufflag & 1) {
+				unsigned int *rt;
+				int x, y;
+				
 				R.xstart= -R.afmx;
 				R.ystart= -R.afmy;
 				rt= R.rectot;
-				for(y=0; y<R.recty; y++, rt+= R.rectx) scanlinesky((char *)rt, y);
+				for(y=0; y<R.recty; y++) {
+					for(x=0; x<R.rectx; x++, rt++) {
+						if(*rt==0) fillBackgroundImageChar((char *)rt, x, y);
+					}
+				}
 			}
 		}
 	}
 
 	set_mblur_offs(0);
 
-	/* FREE */
+	/* mutexes free */
+	SDL_DestroyMutex(load_ibuf_lock);
+	SDL_DestroyMutex(render_abuf_lock);
+	SDL_DestroyMutex(make_table_lock);
+	load_ibuf_lock= NULL;
+	render_abuf_lock= NULL;
+	make_table_lock= NULL;
+}
 
-	/* zbuf test */
-
-	/* don't free R.rectz, only when its size is not the same as R.rectot */
-
-	if (R.rectz && parts == 1 && (R.r.mode & R_FIELDS) == 0);
-	else {
-		if(R.rectz) MEM_freeN(R.rectz);
-		R.rectz= 0;
-	}
-
-	if(R.rectf1) MEM_freeN(R.rectf1);
-	R.rectf1= 0;
-	if(R.rectf2) MEM_freeN(R.rectf2);
-	R.rectf2= 0;
+void render() {
+	/* yafray: render, see above */
+	if (R.r.renderer==R_YAFRAY)
+		yafrayRender();
+	else
+		mainRenderLoop();
 }
 
 
-
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
-extern unsigned short usegamtab;
+
 void RE_initrender(struct View3D *ogl_render_view3d)
 {
 	double start_time;
@@ -1047,37 +1205,60 @@ void RE_initrender(struct View3D *ogl_render_view3d)
 	/* WINDOW size (sch='scherm' dutch for screen...) */
 	R.r.xsch= (R.r.size*R.r.xsch)/100;
 	R.r.ysch= (R.r.size*R.r.ysch)/100;
-
+	
 	R.afmx= R.r.xsch/2;
 	R.afmy= R.r.ysch/2;
-
+	
 	/* to be sure: when a premature return (rectx can differ from xsch) */
 	R.rectx= R.r.xsch;
 	R.recty= R.r.ysch;
-
+	
 	/* IS RENDERING ALLOWED? */
-
+	
 	/* forbidden combination */
-	if((R.r.mode & R_BORDER) && (R.r.mode & R_PANORAMA)) {
-		error("No border allowed for Panorama");
+	if(R.r.mode & R_PANORAMA) {
+		if(R.r.mode & R_BORDER) {
+			error("No border supported for Panorama");
+			G.afbreek= 1;
+		}
+		if(R.r.yparts>1) {
+			error("No Y-Parts supported for Panorama");
+			G.afbreek= 1;
+		}
+		if(R.r.mode & R_ORTHO) {
+			error("No Ortho render possible for Panorama");
+			G.afbreek= 1;
+		}
+	}
+
+	if(R.r.mode & R_BORDER) {
+		if(R.r.border.xmax <= R.r.border.xmin || 
+			R.r.border.ymax <= R.r.border.ymin) {
+			error("No border area selected.");
+			G.afbreek= 1;
+		}
+	}
+	
+	if(R.r.xparts*R.r.yparts>=2 && (R.r.mode & R_MOVIECROP) && (R.r.mode & R_BORDER)) {
+		error("Combination of border, crop and parts not allowed");
 		G.afbreek= 1;
 		return;
 	}
-
 	
 	if(R.r.xparts*R.r.yparts>64) {
-		error("No more than 64 parts");
+		error("No more than 64 parts supported");
 		G.afbreek= 1;
 		return;
 	}
-
+	
 	if(R.r.yparts>1 && (R.r.mode & R_PANORAMA)) {
-		error("No Y-Parts allowed for Panorama");
+		error("No Y-Parts supported for Panorama");
 		G.afbreek= 1;
 		return;
 	}
-
-
+	
+	if(G.afbreek==1) return;
+	
 	/* TEST BACKBUF */
 	/* If an image is specified for use as backdrop, that image is loaded    */
 	/* here.                                                                 */
@@ -1085,122 +1266,133 @@ void RE_initrender(struct View3D *ogl_render_view3d)
 		if(R.r.alphamode == R_ADDSKY) {
 			strcpy(name, R.r.backbuf);
 			BLI_convertstringcode(name, G.sce, G.scene->r.cfra);
-
+			
 			if(R.backbuf) {
 				R.backbuf->id.us--;
 				bima= R.backbuf;
 			}
-			else bima= 0;
-
+			else bima= NULL;
+			
 			R.backbuf= add_image(name);
-
+			
 			if(bima && bima->id.us<1) {
 				free_image_buffers(bima);
 			}
-			if(R.backbuf==0) {
+
+			if(R.backbuf && R.backbuf->ibuf==NULL) {
+				R.backbuf->ibuf= IMB_loadiffname(R.backbuf->name, IB_rect);
+				if(R.backbuf->ibuf==NULL) R.backbuf->ok= 0;
+				else R.backbuf->ok= 1;
+			}
+			if(R.backbuf==NULL || R.backbuf->ok==0) {
 				// error() doesnt work with render window open
 				//error("No backbuf there!");
 				printf("Error: No backbuf %s\n", name);
-				G.afbreek= 1;
-				return;
 			}
 		}
 	}
-
-
-	usegamtab= 0; /* see also further */
-
+	
 	if(R.r.mode & (R_OSA|R_MBLUR)) {
 		R.osa= R.r.osa;
 		if(R.osa>16) R.osa= 16;
-
+		
 		init_render_jit(R.osa);
-		RE_init_filt_mask();
-
-		/* this value sometimes is reset temporally, for example in transp zbuf */
-		if(R.r.mode & R_GAMMA) {
-			if((R.r.mode & R_OSA)) usegamtab= 1;
-		}
+		
 	}
 	else R.osa= 0;
-
+	
+	renderloop_setblending();	// alpha, sky, gamma
+	
 	/* when rendered without camera object */
 	/* it has to done here because of envmaps */
 	R.near= 0.1;
 	R.far= 1000.0;
-
-
+	
+	
 	if(R.afmx<1 || R.afmy<1) {
 		error("Image too small");
 		return;
 	}
 	R.ycor= ( (float)R.r.yasp)/( (float)R.r.xasp);
-
+	
 	start_time= PIL_check_seconds_timer();
-
-	if(R.r.scemode & R_DOSEQ) {
+	
+	if(R.r.scemode & R_OGL) {
 		R.rectx= R.r.xsch;
 		R.recty= R.r.ysch;
-
+		
 		if(R.rectot) MEM_freeN(R.rectot);
 		R.rectot= (unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
-
-		RE_local_timecursor((G.scene->r.cfra));
-
-		if(RE_local_test_break()==0) do_render_seq();
-
-		/* display */
-		if(R.rectot) RE_local_render_display(0, R.recty-1,
-											 R.rectx, R.recty,
-											 R.rectot);
-	}
-	else if(R.r.scemode & R_OGL) {
-		R.rectx= R.r.xsch;
-		R.recty= R.r.ysch;
-
-		if(R.rectot) MEM_freeN(R.rectot);
-		R.rectot= (unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
-
+		
+		if(R.rectftot) MEM_freeN(R.rectftot);
+		R.rectftot= NULL;
+		
 		RE_local_init_render_display();
 		drawview3d_render(ogl_render_view3d);
 	}
+	else if(R.r.scemode & R_DOSEQ) {
+		R.rectx= R.r.xsch;
+		R.recty= R.r.ysch;
+		if(R.r.mode & R_PANORAMA) {
+			R.rectx*= R.r.xparts;
+		}
+		
+		if(R.rectot) MEM_freeN(R.rectot);
+		R.rectot= (unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
+		
+		if(R.rectftot) MEM_freeN(R.rectftot);
+		R.rectftot= NULL;
+		
+		RE_local_timecursor((G.scene->r.cfra));
+		
+		if(RE_local_test_break()==0) do_render_seq();
+		
+		/* display */
+		if(R.rectot) RE_local_render_display(0, R.recty-1, R.rectx, R.recty,R.rectot);
+	}
 	else {
 		if(G.scene->camera==0) {
 			G.scene->camera= scene_find_camera(G.scene);
 		}
-
+		
 		if(G.scene->camera==0) {
 			error("No camera");
+			/* needed because R.rectx and R.recty can be unmatching R.rectot */
+			
+			if(R.rectot) MEM_freeN(R.rectot);
+			R.rectot= NULL;
+			if(R.rectftot) MEM_freeN(R.rectftot);
+			R.rectftot= NULL;
+			
 			G.afbreek=1;
 			return;
 		}
 		else {
-
+			
 			if(G.scene->camera->type==OB_CAMERA) {
 				Camera *cam= G.scene->camera->data;
 				if(cam->type==CAM_ORTHO) R.r.mode |= R_ORTHO;
 			}
-
+			
 			render(); /* returns with complete rect xsch-ysch */
 		}
 	}
-
+	
 	/* display again: fields/seq/parts/pano etc */
 	if(R.rectot) {
 		RE_local_init_render_display();
-		RE_local_render_display(0, R.recty-1,
-								R.rectx, R.recty,
-								R.rectot);
+		RE_local_render_display(0, R.recty-1, R.rectx, R.recty, R.rectot);
 	}
 	else RE_local_clear_render_display(R.win);
-
-	RE_local_printrenderinfo((PIL_check_seconds_timer() - start_time), -1);
 	
-	/* restore variables */
-	//R.osatex= 0;
-	//R.vlr= 0;	
-	///* at cubemap */
-	R.flag= 0;
+	if ((G.scene->r.scemode & R_OGL)==0) /* header gets scrabled if renderwindow holds OGL context */	
+		RE_local_printrenderinfo((PIL_check_seconds_timer() - start_time), -1);
+	
+	/* grms... this is a nasty global */
+	do_gamma= 0;
+	
+	/* these flags remain on, until reset in caller to render (renderwin.c) */
+	R.flag &= (R_RENDERING|R_ANIMRENDER|R_REDRAW_PRV);
 }
 
 void RE_animrender(struct View3D *ogl_render_view3d)
@@ -1208,24 +1400,32 @@ void RE_animrender(struct View3D *ogl_render_view3d)
 	int cfrao;
 	char name[256];
 
-	if(G.scene==0) return;
-
+	if(G.scene==NULL) return;
+	if(G.scene->r.sfra > G.scene->r.efra) {
+		error("Startframe larger than Endframe");
+		return;
+	}
+	
 	/* scenedata to R: (for backbuf, R.rectx etc) */
 	R.r= G.scene->r;
 
 	/* START ANIMLOOP, everywhere NOT the cfra from R.r is gebruikt: because of rest blender */
 	cfrao= (G.scene->r.cfra);
 
-	if(G.scene->r.scemode & R_OGL) R.r.mode &= ~R_PANORAMA;
+	/* disable options for ogl render */
+	if(G.scene->r.scemode & R_OGL) R.r.mode &= ~(R_PANORAMA|R_MOVIECROP);
 	
-	// these calculations apply for
-	// all movie formats
+	// these calculations apply for all movie formats
 	R.rectx= (R.r.size*R.r.xsch)/100;
 	R.recty= (R.r.size*R.r.ysch)/100;
 	if(R.r.mode & R_PANORAMA) {
 		R.rectx*= R.r.xparts;
 		R.recty*= R.r.yparts;
 	}
+	if(R.r.mode & R_MOVIECROP) {
+		initparts();
+		setpart(R.parts.first);		// this will adjust r.rectx
+	}
 
 	if (0) {
 #ifdef __sgi
@@ -1246,11 +1446,12 @@ void RE_animrender(struct View3D *ogl_render_view3d)
 		}
 		start_avi();
 	}
-
+// set initial conditions for softbodies here
+// ******************************************
 	for((G.scene->r.cfra)=(G.scene->r.sfra); (G.scene->r.cfra)<=(G.scene->r.efra); (G.scene->r.cfra)++) {
 		double starttime= PIL_check_seconds_timer();
 
-		R.flag= R_ANIMRENDER;
+		R.flag |= R_ANIMRENDER; // unused now (ton)
 
 		RE_initrender(ogl_render_view3d);
 		
@@ -1287,13 +1488,13 @@ void RE_animrender(struct View3D *ogl_render_view3d)
 
 	}
 
-	(G.scene->r.cfra)= cfrao;
+	G.scene->r.cfra= cfrao;
 
 	/* restore time */
 	if(R.r.mode & (R_FIELDS|R_MBLUR)) {
 		do_all_ipos();
 		do_all_keys();
-		do_all_actions();
+		do_all_actions(NULL);
 		do_all_ikas();
 	}
 
