@@ -1,5 +1,5 @@
 /**
- * $Id: convertBlenderScene.c,v 1.102 2004/12/19 18:04:48 ton Exp $
+ * $Id: convertBlenderScene.c,v 1.132 2005/06/06 13:10:02 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -51,22 +51,24 @@
 #include "BLI_arithb.h"
 #include "BLI_blenlib.h"
 #include "BLI_rand.h"
+#include "BLI_memarena.h"
 
-#include "DNA_scene_types.h"
-#include "DNA_lamp_types.h"
+#include "DNA_armature_types.h"
 #include "DNA_camera_types.h"
 #include "DNA_material_types.h"
 #include "DNA_curve_types.h"
-#include "DNA_texture_types.h"
-#include "DNA_lattice_types.h"
 #include "DNA_effect_types.h"
 #include "DNA_ika_types.h"
-#include "DNA_armature_types.h"
-#include "DNA_object_types.h"
-#include "DNA_view3d_types.h"
+#include "DNA_lamp_types.h"
+#include "DNA_lattice_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_meta_types.h"
+#include "DNA_object_types.h"
+#include "DNA_object_force.h"
+#include "DNA_scene_types.h"
+#include "DNA_texture_types.h"
+#include "DNA_view3d_types.h"
 
 #include "BKE_anim.h"
 #include "BKE_armature.h"
@@ -75,6 +77,7 @@
 #include "BKE_constraint.h"
 #include "BKE_displist.h"
 #include "BKE_deform.h"
+#include "BKE_DerivedMesh.h"
 #include "BKE_effect.h"
 #include "BKE_global.h"
 #include "BKE_key.h"
@@ -107,9 +110,10 @@
 
 #include "butspace.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "radio.h"
+#include "YafRay_Api.h"
+
+extern void error (char *fmt, ...);  /* defined in BIF_toolbox.h, but we dont need to include the rest */ 
 
 /* yafray: Identity transform 'hack' removed, exporter now transforms vertices back to world.
  * Same is true for lamp coords & vec.
@@ -126,7 +130,7 @@ static Material *give_render_material(Object *ob, int nr);
 
 
 /* blenderWorldManipulation.c */
-static void split_u_renderfaces(int startvlak, int startvert, int usize, int plek, int cyclu);
+/*static void split_u_renderfaces(int startvlak, int startvert, int usize, int plek, int cyclu);*/
 static void split_v_renderfaces(int startvlak, int startvert, int usize, int vsize, int plek, int cyclu, int cyclv);
 static int contrpuntnormr(float *n, float *puno);
 static void as_addvert(VertRen *v1, VlakRen *vlr);
@@ -193,16 +197,16 @@ void RE_make_stars(void (*initfunc)(void),
 	double dblrand, hlfrand;
 	float vec[4], fx, fy, fz;
 	float fac, starmindist, clipend;
-	float mat[4][4], stargrid, maxrand, force, alpha;
+	float mat[4][4], stargrid, maxrand, maxjit, force, alpha;
 /* 	float loc_far_var, loc_near_var; */
-	int x, y, z, sx, sy, sz, ex, ey, ez, maxjit, done = 0;
+	int x, y, z, sx, sy, sz, ex, ey, ez, done = 0;
 	Camera * camera;
 
 	if(initfunc) R.wrld= *(G.scene->world);
 
 	stargrid = R.wrld.stardist;		/* distance between stars */
 	maxrand = 2.0;						/* amount a star can be shifted (in grid units) */
-	maxjit = (256.0* R.wrld.starcolnoise);			/* amount a color is being shifted */
+	maxjit = (R.wrld.starcolnoise);			/* amount a color is being shifted */
 
 /* 	loc_far_var = R.far; */
 /* 	loc_near_var = R.near; */
@@ -299,7 +303,7 @@ void RE_make_stars(void (*initfunc)(void),
 						if (har) {
 							har->alfa = sqrt(sqrt(alpha));
 							har->add= 255;
-							har->r = har->g = har->b = 255;
+							har->r = har->g = har->b = 1.0;
 							if (maxjit) {
 								har->r += ((maxjit * BLI_drand()) ) - maxjit;
 								har->g += ((maxjit * BLI_drand()) ) - maxjit;
@@ -313,21 +317,15 @@ void RE_make_stars(void (*initfunc)(void),
 					}
 				}
 			}
-//			if(done > MAXVERT) {
-//				printf("Too many stars\n");
-//				break;
-//			}
-			if(blender_test_break()) break;
+			/* do not call blender_test_break() here, since it is used in UI as well, confusing the callback system */
+			/* main cause is G.afbreek of course, a global again... (ton) */
 		}
-//		if(done > MAXVERT) break;
-
-		if(blender_test_break()) break;
 	}
 	if (termfunc) termfunc();
 }
 
 /* ------------------------------------------------------------------------ */
-/* more star stuff */
+/* more star stuff, here used to be a cliptest, removed for envmap render or panorama... */
 static HaloRen *initstar(float *vec, float hasize)
 {
 	HaloRen *har;
@@ -335,24 +333,22 @@ static HaloRen *initstar(float *vec, float hasize)
 
 	RE_projectverto(vec, hoco);
 
-	if(	(R.r.mode & R_PANORAMA) || (R.r.xparts*R.r.yparts>1) || RE_testclip(hoco)==0 ) {
-		har= RE_findOrAddHalo(R.tothalo++);
-	
-		/* projectvert is done in function zbufvlaggen again, because of parts */
-		VECCOPY(har->co, vec);
-		har->hasize= hasize;
-	
-		har->zd= 0.0;
-	
-		return har;
-	}
-	return NULL;
+	har= RE_findOrAddHalo(R.tothalo++);
+
+	/* projectvert is done in function zbufvlaggen again, because of parts */
+	VECCOPY(har->co, vec);
+	har->hasize= hasize;
+
+	har->zd= 0.0;
+
+	return har;
 }
 
 
 
 /* ------------------------------------------------------------------------- */
 
+#if 0
 static void split_u_renderfaces(int startvlak, int startvert, int usize, int plek, int cyclu)
 {
 	VlakRen *vlr;
@@ -379,6 +375,7 @@ static void split_u_renderfaces(int startvlak, int startvert, int usize, int ple
 	}
 
 }
+#endif 
 
 /* ------------------------------------------------------------------------- */
 
@@ -798,7 +795,7 @@ static void make_render_halos(Object *ob, Mesh *me, int totvert, MVert *mvert, M
 	HaloRen *har;
 	float xn, yn, zn, nor[3], view[3];
 	float *orco, vec[3], hasize, mat[4][4], imat[3][3];
-	int start, end, a, ok;
+	int start, end, a, ok, seed= ma->seed1;
 
 	MTC_Mat4MulMat4(mat, ob->obmat, R.viewmat);
 	MTC_Mat3CpyMat4(imat, ob->imat);
@@ -813,8 +810,6 @@ static void make_render_halos(Object *ob, Mesh *me, int totvert, MVert *mvert, M
 	mvert+= start;
 	if(extverts) extverts+= 3*start;
 
-	ma->ren->seed1= ma->seed1;
-
 	for(a=start; a<end; a++, mvert++) {
 		ok= 1;
 
@@ -849,12 +844,12 @@ static void make_render_halos(Object *ob, Mesh *me, int totvert, MVert *mvert, M
 				else hasize*= zn*zn*zn*zn;
 			}
 
-			if(orco) har= RE_inithalo(ma, vec, 0, orco, hasize, 0);
-			else har= RE_inithalo(ma, vec, 0, mvert->co, hasize, 0);
+			if(orco) har= RE_inithalo(ma, vec, NULL, orco, hasize, 0.0, seed);
+			else har= RE_inithalo(ma, vec, NULL, mvert->co, hasize, 0.0, seed);
 			if(har) har->lay= ob->lay;
 		}
 		if(orco) orco+= 3;
-		ma->ren->seed1++;
+		seed++;
 	}
 }
 
@@ -867,7 +862,7 @@ static void render_particle_system(Object *ob, PartEff *paf)
 	HaloRen *har=0;
 	Material *ma=0;
 	float xn, yn, zn, imat[3][3], mat[4][4], hasize, ptime, ctime, vec[3], vec1[3], view[3], nor[3];
-	int a, mat_nr=1;
+	int a, mat_nr=1, seed;
 
 	pa= paf->keys;
 	if(pa==NULL) {
@@ -877,7 +872,6 @@ static void render_particle_system(Object *ob, PartEff *paf)
 	}
 
 	ma= give_render_material(ob, 1);
-	if(ma==NULL) ma= &defmaterial;
 
 	MTC_Mat4MulMat4(mat, ob->obmat, R.viewmat);
 	MTC_Mat4Invert(ob->imat, mat);	/* this is correct, for imat texture */
@@ -890,7 +884,7 @@ static void render_particle_system(Object *ob, PartEff *paf)
 	if(ob->ipoflag & OB_OFFS_PARTICLE) ptime= ob->sf;
 	else ptime= 0.0;
 	ctime= bsystem_time(ob, 0, (float)G.scene->r.cfra, ptime);
-	ma->ren->seed1= ma->seed1;
+	seed= ma->seed1;
 
 	for(a=0; a<paf->totpart; a++, pa+=paf->totkey) {
 
@@ -912,7 +906,6 @@ static void render_particle_system(Object *ob, PartEff *paf)
 				if(pa->mat_nr != mat_nr) {
 					mat_nr= pa->mat_nr;
 					ma= give_render_material(ob, mat_nr);
-					if(ma==0) ma= &defmaterial;
 				}
 
 				if(ma->ipo) {
@@ -943,9 +936,9 @@ static void render_particle_system(Object *ob, PartEff *paf)
 					else hasize*= zn*zn*zn*zn;
 				}
 
-				if(paf->stype==PAF_VECT) har= RE_inithalo(ma, vec, vec1, pa->co, hasize, paf->vectsize);
+				if(paf->stype==PAF_VECT) har= RE_inithalo(ma, vec, vec1, pa->co, hasize, paf->vectsize, seed);
 				else {
-					har= RE_inithalo(ma, vec, 0, pa->co, hasize, 0);
+					har= RE_inithalo(ma, vec, NULL, pa->co, hasize, 0.0, seed);
 					if(har && ma->mode & MA_HALO_SHADE) {
 						VecSubf(har->no, vec, vec1);
 						Normalise(har->no);
@@ -954,7 +947,7 @@ static void render_particle_system(Object *ob, PartEff *paf)
 				if(har) har->lay= ob->lay;
 			}
 		}
-		ma->ren->seed1++;
+		seed++;
 	}
 
 	/* restore material */
@@ -968,6 +961,14 @@ static void render_particle_system(Object *ob, PartEff *paf)
 
 /* ------------------------------------------------------------------------- */
 
+/* when objects are duplicated, they are freed immediate, but still might be
+in use for render... */
+static Object *vlr_set_ob(Object *ob)
+{
+	if(ob->flag & OB_FROMDUPLI) return (Object *)ob->id.newid;
+	return ob;
+}
+
 static void render_static_particle_system(Object *ob, PartEff *paf)
 {
 	Particle *pa=0;
@@ -977,7 +978,7 @@ static void render_static_particle_system(Object *ob, PartEff *paf)
 	VlakRen *vlr;
 	float xn, yn, zn, imat[3][3], mat[4][4], hasize;
 	float mtime, ptime, ctime, vec[3], vec1[3], view[3], nor[3];
-	int a, mat_nr=1;
+	int a, mat_nr=1, seed;
 
 	pa= paf->keys;
 	if(pa==NULL || (paf->flag & PAF_ANIMATED)) {
@@ -987,7 +988,6 @@ static void render_static_particle_system(Object *ob, PartEff *paf)
 	}
 
 	ma= give_render_material(ob, 1);
-	if(ma==NULL) ma= &defmaterial;
 
 	MTC_Mat4MulMat4(mat, ob->obmat, R.viewmat);
 	MTC_Mat4Invert(ob->imat, mat);	/* need to be that way, for imat texture */
@@ -999,7 +999,7 @@ static void render_static_particle_system(Object *ob, PartEff *paf)
 	if(ob->ipoflag & OB_OFFS_PARTICLE) ptime= ob->sf;
 	else ptime= 0.0;
 	ctime= bsystem_time(ob, 0, (float)G.scene->r.cfra, ptime);
-	ma->ren->seed1= ma->seed1;
+	seed= ma->seed1;
 
 	for(a=0; a<paf->totpart; a++, pa+=paf->totkey) {
 
@@ -1026,7 +1026,6 @@ static void render_static_particle_system(Object *ob, PartEff *paf)
 			if(pa->mat_nr != mat_nr) {
 				mat_nr= pa->mat_nr;
 				ma= give_render_material(ob, mat_nr);
-				if(ma==0) ma= &defmaterial;
 			}
 
 			if(ma->mode & MA_WIRE) {
@@ -1035,10 +1034,10 @@ static void render_static_particle_system(Object *ob, PartEff *paf)
 					VECCOPY(v1->co, vec);
 				}
 				else {
-					float cvec[3]={-1.0, 0.0, 0.0};
+//					float cvec[3]={-1.0, 0.0, 0.0};
 					
 					vlr= RE_findOrAddVlak(R.totvlak++);
-					vlr->ob= ob;
+					vlr->ob= vlr_set_ob(ob);
 					vlr->v1= v1;
 					vlr->v2= RE_findOrAddVert(R.totvert++);
 					vlr->v3= vlr->v2;
@@ -1085,9 +1084,9 @@ static void render_static_particle_system(Object *ob, PartEff *paf)
 					else hasize*= zn*zn*zn*zn;
 				}
 
-				if(paf->stype==PAF_VECT) har= RE_inithalo(ma, vec, vec1, pa->co, hasize, paf->vectsize);
+				if(paf->stype==PAF_VECT) har= RE_inithalo(ma, vec, vec1, pa->co, hasize, paf->vectsize, seed);
 				else {
-					har= RE_inithalo(ma, vec, 0, pa->co, hasize, 0);
+					har= RE_inithalo(ma, vec, NULL, pa->co, hasize, 0.0, seed);
 					if(har && (ma->mode & MA_HALO_SHADE)) {
 						VecSubf(har->no, vec, vec1);
 						Normalise(har->no);
@@ -1099,7 +1098,7 @@ static void render_static_particle_system(Object *ob, PartEff *paf)
 			
 			VECCOPY(vec1, vec);
 		}
-		ma->ren->seed1++;
+		seed++;
 	}
 
 }
@@ -1167,7 +1166,9 @@ static void sort_halos(void)
 
 static Material *give_render_material(Object *ob, int nr)
 {
+	extern Material defmaterial;	// initrender.c
 	Object *temp;
+	Material *ma;
 
 	if(ob->flag & OB_FROMDUPLI) {
 		temp= (Object *)ob->id.newid;
@@ -1176,7 +1177,10 @@ static Material *give_render_material(Object *ob, int nr)
 		}
 	}
 
-	return give_current_material(ob, nr);
+	ma= give_current_material(ob, nr);
+	if(ma==NULL) ma= &defmaterial;
+	
+	return ma;
 }
 
 /* ------------------------------------------------------------------------- */
@@ -1197,10 +1201,9 @@ static void init_render_mball(Object *ob)
 	MTC_Mat3CpyMat4(imat, ob->imat);
 
 	ma= give_render_material(ob, 1);
-	if(ma==0) ma= &defmaterial;
 
 	need_orco= 0;
-	if(ma->ren->texco & TEXCO_ORCO) {
+	if(ma->texco & TEXCO_ORCO) {
 		need_orco= 1;
 	}
 
@@ -1239,7 +1242,7 @@ static void init_render_mball(Object *ob)
 	for(a=0; a<dl->parts; a++, index+=4) {
 
 		vlr= RE_findOrAddVlak(R.totvlak++);
-		vlr->ob= ob;
+		vlr->ob= vlr_set_ob(ob);
 		vlr->v1= RE_findOrAddVert(startvert+index[0]);
 		vlr->v2= RE_findOrAddVert(startvert+index[1]);
 		vlr->v3= RE_findOrAddVert(startvert+index[2]);
@@ -1283,22 +1286,20 @@ static void init_render_mball(Object *ob)
 /* convert */
 static void init_render_mesh(Object *ob)
 {
-	MFace *mface;
-	MVert *mvert;
 	Mesh *me;
-	VlakRen *vlr, *vlr1;
+	MVert *mvert = NULL;
+	MFace *mface;
+	VlakRen *vlr; //, *vlr1;
 	VertRen *ver;
 	Material *ma;
-	MSticky *ms;
+	MSticky *ms = NULL;
 	PartEff *paf;
-	DispList *dl;
-	TFace *tface;
 	unsigned int *vertcol;
-	float xn, yn, zn, nor[3], imat[3][3], mat[4][4];
+	float xn, yn, zn,  imat[3][3], mat[4][4];  //nor[3],
 	float *extverts=0, *orco;
 	int a, a1, ok, do_puno, need_orco=0, totvlako, totverto, vertofs;
-	int start, end, do_autosmooth=0, totvert;
-	DispListMesh *dlm;
+	int start, end, do_autosmooth=0, totvert = 0;
+	DispListMesh *dlm = NULL;
 
 	me= ob->data;
 
@@ -1328,7 +1329,7 @@ static void init_render_mesh(Object *ob)
 		for(a=1; a<=ob->totcol; a++) {
 			ma= give_render_material(ob, a);
 			if(ma) {
-				if(ma->ren->texco & TEXCO_ORCO) {
+				if(ma->texco & TEXCO_ORCO) {
 					need_orco= 1;
 					break;
 				}
@@ -1338,7 +1339,7 @@ static void init_render_mesh(Object *ob)
 	
 	/* we do this before deform */
 	if(need_orco) {
-		if (mesh_uses_displist(me)) 
+		if ((me->flag&ME_SUBSURF) && me->subdivr) 
 			make_orco_displist_mesh(ob, me->subdivr);
 		else
 			make_orco_mesh(me);
@@ -1347,44 +1348,21 @@ static void init_render_mesh(Object *ob)
 	/* after orco, because this changes mesh vertices (too) */
 	do_puno= mesh_modifier(ob, 's');
 	
-	if (mesh_uses_displist(me)) {
-		dl= me->disp.first;
-
-		/* Force a displist rebuild if this is a subsurf and we have a different subdiv level */
-		/* also when object is in editmode, displist ordering for editmode is different, giving orco probs */
-		
-		if((dl==NULL) || ((me->subdiv != me->subdivr)) || (ob==G.obedit)) {
-			/* prevent subsurf called again for duplicate use of mesh, tface pointers change */
-			if(dl==NULL || (me->subdivdone-1)!=me->subdivr) {
-				DispList *dlVerts;
-
-				dlVerts= find_displist(&ob->disp, DL_VERTS);
-				dlm= subsurf_make_dispListMesh_from_mesh(me, dlVerts?dlVerts->verts:NULL, me->subdivr, me->flag);
-				dl= MEM_callocN(sizeof(*dl), "dl");
-				dl->type= DL_MESH;
-				dl->mesh= dlm;
-
-				free_displist_by_type(&me->disp, DL_MESH);
-				BLI_addtail(&me->disp, dl);
-
-				me->subdivdone= me->subdivr+1;	/* stupid hack, add one because otherwise old files will get
-												* subdivdone==0, so me->subdivr==0 won't work. proper caching
-												* will remove this hack.
-												*/
-			}
+	if ((me->flag&ME_SUBSURF) && me->subdivr) {
+		int needsFree;
+		DerivedMesh *dm = mesh_get_derived_render(ob, &needsFree);
+		dlm = dm->convertToDispListMesh(dm);
+		if (needsFree) {
+			dm->release(dm);
 		}
 
-		if(dl==NULL || dl->type!=DL_MESH);  // here used to be a return, but why?
-		else {
-			dlm= dl->mesh;
-	
-			mvert= dlm->mvert;
-			totvert= dlm->totvert;
-	
-			ms= NULL; // no stick in displistmesh
-		}
-		
+		mvert= dlm->mvert;
+		totvert= dlm->totvert;
+
+		ms= NULL; // no stick in displistmesh
 	} else {
+		DispList *dl;
+
 		dlm= NULL;
 		mvert= me->mvert;
 		totvert= me->totvert;
@@ -1398,8 +1376,6 @@ static void init_render_mesh(Object *ob)
 	orco= me->orco;
 
 	ma= give_render_material(ob, 1);
-	if(ma==0) ma= &defmaterial;
-
 
 	if(ma->mode & MA_HALO) {
 		make_render_halos(ob, me, totvert, mvert, ma, extverts);
@@ -1444,7 +1420,6 @@ static void init_render_mesh(Object *ob)
 		for(a1=0; (a1<ob->totcol || (a1==0 && ob->totcol==0)); a1++) {
 
 			ma= give_render_material(ob, a1+1);
-			if(ma==0) ma= &defmaterial;
 			
 			/* test for 100% transparant */
 			ok= 1;
@@ -1457,6 +1432,12 @@ static void init_render_mesh(Object *ob)
 					}
 				}
 			}
+			
+			/* if wire material, and we got edges, don't do the faces */
+			if(ma->mode & MA_WIRE) {
+				end= dlm?dlm->totedge:me->totedge;
+				if(end) ok= 0;
+			}
 
 			if(ok) {
 				TFace *tface= NULL;
@@ -1473,7 +1454,7 @@ static void init_render_mesh(Object *ob)
 					mface= dlm->mface + start;
 					if (dlm->tface) {
 						tface= dlm->tface + start;
-						vertcol= dlm->tface->col;
+						vertcol= NULL;
 					} else if (dlm->mcol) {
 						vertcol= (unsigned int *)dlm->mcol;
 					} else {
@@ -1483,14 +1464,14 @@ static void init_render_mesh(Object *ob)
 					mface= ((MFace*) me->mface) + start;
 					if (me->tface) {
 						tface= ((TFace*) me->tface) + start;
-						vertcol= ((TFace*) me->tface)->col;
+						vertcol= NULL;
 					} else if (me->mcol) {
 						vertcol= (unsigned int *)me->mcol;
 					} else {
 						vertcol= NULL;
 					}
 				}
-				
+
 				for(a=start; a<end; a++) {
 					int v1, v2, v3, v4, edcode, flag;
 					
@@ -1502,14 +1483,11 @@ static void init_render_mesh(Object *ob)
 						flag= mface->flag;
 						edcode= mface->edcode;
 						
-						/* cannot use edges data for render, this has no vcol or tface... */
-						if(dlm && (dlm->flag & ME_OPT_EDGES)==0) edcode= ME_V1V2|ME_V2V3|ME_V3V4|ME_V4V1;
-						
 						if(v3) {
 							float len;
 							
 							vlr= RE_findOrAddVlak(R.totvlak++);
-							vlr->ob= ob;
+							vlr->ob= vlr_set_ob(ob);
 							vlr->v1= RE_findOrAddVert(vertofs+v1);
 							vlr->v2= RE_findOrAddVert(vertofs+v2);
 							vlr->v3= RE_findOrAddVert(vertofs+v3);
@@ -1532,20 +1510,30 @@ static void init_render_mesh(Object *ob)
 
 							if(len==0) R.totvlak--;
 							else {
-
-								if(vertcol) {
-									if(tface) vlr->vcol= vertcol+sizeof(TFace)*a/4;	/* vertcol is int */
-									else vlr->vcol= vertcol+sizeof(int)*a;
+								if(dlm) {
+									if(tface) {
+										vlr->tface= BLI_memarena_alloc(R.memArena, sizeof(*vlr->tface));
+										vlr->vcol= vlr->tface->col;
+										memcpy(vlr->tface, tface, sizeof(*tface));
+									} 
+									else if (vertcol) {
+										vlr->vcol= BLI_memarena_alloc(R.memArena, sizeof(int)*16);
+										memcpy(vlr->vcol, vertcol+4*a, sizeof(int)*16);
+									}
+								} else {
+									if(tface) {
+										vlr->vcol= tface->col;
+										vlr->tface= tface;
+									} 
+									else if (vertcol) {
+										vlr->vcol= vertcol+4*a;
+									}
 								}
-								else vlr->vcol= 0;
-								
-								vlr->tface= tface;
-								
 							}
 						}
 						else if(v2 && (ma->mode & MA_WIRE)) {
 							vlr= RE_findOrAddVlak(R.totvlak++);
-							vlr->ob= ob;
+							vlr->ob= vlr_set_ob(ob);
 							vlr->v1= RE_findOrAddVert(vertofs+v1);
 							vlr->v2= RE_findOrAddVert(vertofs+v2);
 							vlr->v3= vlr->v2;
@@ -1565,6 +1553,46 @@ static void init_render_mesh(Object *ob)
 				}
 			}
 		}
+		
+		/* exception... we do edges for wire mode. potential conflict when faces exist... */
+		end= dlm?dlm->totedge:me->totedge;
+		mvert= dlm?dlm->mvert:me->mvert;
+		ma= give_render_material(ob, 1);
+		if(end && (ma->mode & MA_WIRE)) {
+			MEdge *medge;
+			medge= dlm?dlm->medge:me->medge;
+			
+			for(a1=0; a1<end; a1++, medge++) {
+				
+				if( (me->flag & ME_OPT_EDGES)==0 || (medge->flag & ME_EDGEDRAW)) {
+					MVert *v0 = &mvert[medge->v1];
+					MVert *v1 = &mvert[medge->v2];
+
+					vlr= RE_findOrAddVlak(R.totvlak++);
+					vlr->ob= vlr_set_ob(ob);
+					vlr->v1= RE_findOrAddVert(vertofs+medge->v1);
+					vlr->v2= RE_findOrAddVert(vertofs+medge->v2);
+					vlr->v3= vlr->v2;
+					vlr->v4= NULL;
+					
+					xn= (v0->no[0]+v1->no[0]);
+					yn= (v0->no[1]+v1->no[1]);
+					zn= (v0->no[2]+v1->no[2]);
+					if(do_puno==0) {
+						/* transpose ! */
+						vlr->n[0]= imat[0][0]*xn+imat[0][1]*yn+imat[0][2]*zn;
+						vlr->n[1]= imat[1][0]*xn+imat[1][1]*yn+imat[1][2]*zn;
+						vlr->n[2]= imat[2][0]*xn+imat[2][1]*yn+imat[2][2]*zn;
+						Normalise(vlr->n);
+					}
+					
+					vlr->mat= ma;
+					vlr->flag= 0;
+					vlr->ec= ME_V1V2;
+					vlr->lay= ob->lay;
+				}
+			}
+		}
 	}
 	
 	if(do_puno) {
@@ -1583,6 +1611,8 @@ static void init_render_mesh(Object *ob)
 	if(do_puno) calc_vertexnormals(totverto, totvlako);
 	
 	mesh_modifier(ob, 'e');  // end
+
+	if(dlm) displistmesh_free(dlm);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -1721,6 +1751,9 @@ void RE_add_render_lamp(Object *ob, int doshadbuf)
 		lar->YF_causticblur = la->YF_causticblur;
 		lar->YF_ltradius = la->YF_ltradius;
 		lar->YF_bufsize = la->YF_bufsize;
+		lar->YF_glowint = la->YF_glowint;
+		lar->YF_glowofs = la->YF_glowofs;
+		lar->YF_glowtype = la->YF_glowtype;
 	}
 
 	lar->spotsi= la->spotsize;
@@ -1804,14 +1837,12 @@ void RE_add_render_lamp(Object *ob, int doshadbuf)
 				lar->mode &= ~LA_SHAD_RAY;
 		}
 	}
-	
-	lar->org= MEM_dupallocN(lar);
-
 }
 
 /* ------------------------------------------------------------------------- */
 static void init_render_surf(Object *ob)
 {
+	extern Material defmaterial;	// initrender.c
 	Nurb *nu=0;
 	Curve *cu;
 	ListBase displist;
@@ -1825,7 +1856,7 @@ static void init_render_surf(Object *ob)
 	int u, v;
 	int sizeu, sizev;
 	VlakRen *vlr1, *vlr2, *vlr3;
-	float n2[3], vn[3];
+	float  vn[3]; // n2[3],
 #endif
 
 	cu= ob->data;
@@ -1840,8 +1871,7 @@ static void init_render_surf(Object *ob)
 	matar[0]= &defmaterial;
 	for(a=0; a<ob->totcol; a++) {
 		matar[a]= give_render_material(ob, a+1);
-		if(matar[a]==0) matar[a]= &defmaterial;
-		if(matar[a] && matar[a]->ren->texco & TEXCO_ORCO) {
+		if(matar[a] && matar[a]->texco & TEXCO_ORCO) {
 			need_orco= 1;
 		}
 	}
@@ -2019,7 +2049,7 @@ static void init_render_surf(Object *ob)
 
 
 					vlr= RE_findOrAddVlak(R.totvlak++);
-					vlr->ob= ob;
+					vlr->ob= vlr_set_ob(ob);
 					vlr->v1= v1; vlr->v2= v2; vlr->v3= v3; vlr->v4= v4;
 					
 					flen= CalcNormFloat4(vlr->v4->co, vlr->v3->co, vlr->v2->co, vlr->v1->co, n1);
@@ -2143,7 +2173,7 @@ static void init_render_surf(Object *ob)
 					flen= CalcNormFloat4(v1->co, v3->co, v4->co, v2->co, n1);
 					if(flen!=0.0) {
 						vlr= RE_findOrAddVlak(R.totvlak++);
-						vlr->ob= ob;
+						vlr->ob= vlr_set_ob(ob);
 						vlr->v1= v1;
 						vlr->v2= v3;
 						vlr->v3= v4;
@@ -2185,8 +2215,8 @@ static void init_render_surf(Object *ob)
 
 static void init_render_curve(Object *ob)
 {
+	extern Material defmaterial;	// initrender.c
 	Ika *ika=0;
-
 	Lattice *lt=0;
 	Curve *cu;
 	VertRen *ver;
@@ -2228,8 +2258,7 @@ static void init_render_curve(Object *ob)
 	matar[0]= &defmaterial;
 	for(a=0; a<ob->totcol; a++) {
 		matar[a]= give_render_material(ob, a+1);
-		if(matar[a]==0) matar[a]= &defmaterial;
-		if(matar[a]->ren->texco & TEXCO_ORCO) {
+		if(matar[a]->texco & TEXCO_ORCO) {
 			need_orco= 1;
 		}
 	}
@@ -2383,7 +2412,7 @@ static void init_render_curve(Object *ob)
 					for(; b<bl->nr; b++) {
 
 						vlr= RE_findOrAddVlak(R.totvlak++);
-						vlr->ob= ob;
+						vlr->ob= vlr_set_ob(ob);
 						vlr->v1= RE_findOrAddVert(p2);
 						vlr->v2= RE_findOrAddVert(p1);
 						vlr->v3= RE_findOrAddVert(p3);
@@ -2497,7 +2526,7 @@ static void init_render_curve(Object *ob)
 			for(a=0; a<dl->parts; a++, index+=3) {
 
 				vlr= RE_findOrAddVlak(R.totvlak++);
-				vlr->ob = ob;	/* yafray: correction for curve rendering, obptr was not set */
+				vlr->ob = vlr_set_ob(ob);	/* yafray: correction for curve rendering, obptr was not set */
 				vlr->v1= RE_findOrAddVert(startvert+index[0]);
 				vlr->v2= RE_findOrAddVert(startvert+index[1]);
 				vlr->v3= RE_findOrAddVert(startvert+index[2]);
@@ -2568,7 +2597,7 @@ static void init_render_curve(Object *ob)
 /* prevent phong interpolation for giving ray shadow errors (terminator problem) */
 static void set_phong_threshold(Object *ob, int startface, int numface, int startvert, int numvert )
 {
-	VertRen *ver;
+//	VertRen *ver;
 	VlakRen *vlr;
 	float thresh= 0.0, dot;
 	int tot=0, i;
@@ -2666,6 +2695,9 @@ void RE_freeRotateBlenderScene(void)
 	char *ctile;
 
 	/* FREE */
+	
+	BLI_memarena_free(R.memArena);
+	R.memArena = NULL;
 
 	for(a=0; a<R.totlamp; a++) {
 		if(R.la[a]->shb) {
@@ -2681,26 +2713,28 @@ void RE_freeRotateBlenderScene(void)
 			MEM_freeN(shb->cbuf);
 			MEM_freeN(R.la[a]->shb);
 		}
-		if(R.la[a]->org) MEM_freeN(R.la[a]->org);
 		if(R.la[a]->jitter) MEM_freeN(R.la[a]->jitter);
 		MEM_freeN(R.la[a]);
 	}
+
+	/* note; these pointer arrays were allocated, with last element NULL to stop loop */
 	a=0;
 	while(R.blove[a]) {
 		MEM_freeN(R.blove[a]);
-		R.blove[a]=0;
+		R.blove[a]= NULL;
 		a++;
 	}
+
 	a=0;
 	while(R.blovl[a]) {
 		MEM_freeN(R.blovl[a]);
-		R.blovl[a]=0;
+		R.blovl[a]= NULL;
 		a++;
 	}
 	a=0;
 	while(R.bloha[a]) {
 		MEM_freeN(R.bloha[a]);
-		R.bloha[a]=0;
+		R.bloha[a]= NULL;
 		a++;
 	}
 
@@ -2721,10 +2755,9 @@ void RE_freeRotateBlenderScene(void)
 				MEM_freeN(me->orco);
 				me->orco= 0;
 			}
-			if (mesh_uses_displist(me) && ((me->subdiv!=me->subdivr) || (ob->effect.first != NULL) || ob==G.obedit) ) { 
+			if ((me->flag&ME_SUBSURF) && ((me->subdiv!=me->subdivr) || (ob->effect.first != NULL) || ob==G.obedit) ) { 
 			    /* Need to recalc for effects since they are time dependant */
 				makeDispList(ob);  /* XXX this should be replaced with proper caching */
-				me->subdivdone= 0;	/* needed to prevent multiple used meshes being recalculated */
 			}
 		}
 		else if(ob->type==OB_MBALL) {
@@ -2898,21 +2931,20 @@ void RE_rotateBlenderScene(void)
 
 	if(G.scene->camera==0) return;
 
-	O.dxwin[0]= 0.5/(float)R.r.xsch;
-	O.dywin[1]= 0.5/(float)R.r.ysch;
+	R.memArena = BLI_memarena_new(BLI_MEMARENA_STD_BUFSIZE);
 
 	slurph_opt= 0;
 
 	R.totvlak=R.totvert=R.totlamp=R.tothalo= 0;
 
 	do_all_ipos();
-	BPY_do_all_scripts(SCRIPT_FRAMECHANGED);
+	if (G.f & G_DOSCRIPTLINKS) BPY_do_all_scripts(SCRIPT_FRAMECHANGED);
 	do_all_keys();
 #ifdef __NLA
-	do_all_actions();
+	do_all_actions(NULL);
 	rebuild_all_armature_displists();
 	/* so nice, better do it twice */
-	do_all_actions();
+	do_all_actions(NULL);
 	rebuild_all_armature_displists();
 #endif
 	do_all_ikas();
@@ -2929,14 +2961,35 @@ void RE_rotateBlenderScene(void)
 
 	base= G.scene->base.first;
 	while(base) {
-		clear_object_constraint_status(base->object);
-		if (base->object->type==OB_ARMATURE) {
-			where_is_armature (base->object);
-		}
+		ob= base->object;
+		
+		clear_object_constraint_status(ob);
+		
+		if (ob->type==OB_ARMATURE) 
+			where_is_armature (ob);
 		else
+			where_is_object(ob);
 
-		where_is_object(base->object);
-
+		if(ob->disp.first==NULL) {
+			/* check for need for displist (it's zero when parent, key, or hook changed) */
+			/* this part was added to mimic drawing code better, will be solved with dep graph! (ton) */
+			/* added note; this is also needed for curves, for example, deformed with hooks */
+			if(ob->parent && ob->partype==PARSKEL) makeDispList(ob);
+			else if(ob->parent && ob->parent->type==OB_LATTICE) makeDispList(ob);
+			else if(ob->hooks.first) makeDispList(ob);
+			else if(ob->softflag & OB_SB_ENABLE) makeDispList(ob);
+		
+			if(ob->type==OB_MESH) {
+				Mesh *me= ob->data;
+				if(me->disp.first==NULL && (me->flag&ME_SUBSURF)) makeDispList(ob);
+				else if(ob->effect.first) {	// as last check
+					Effect *eff= ob->effect.first;
+					if(eff->type==EFF_WAVE) makeDispList(ob);
+				}
+			}
+		}
+		
+		
 		if(base->next==0 && G.scene->set && base==G.scene->base.last) base= G.scene->set->base.first;
 		else base= base->next;
 	}
@@ -2945,9 +2998,6 @@ void RE_rotateBlenderScene(void)
 	MTC_Mat4Ortho(R.viewinv);
 	MTC_Mat4Invert(R.viewmat, R.viewinv);
 
-	/* not so neat: now the viewinv is not equal to viewmat. used for Texcos and such. Improve! */
-	if(R.r.mode & R_ORTHO) R.viewmat[3][2]*= 100.0;
-
 	RE_setwindowclip(1,-1); /*  no jit:(-1) */
 
 	/* clear imat flags */
@@ -2957,7 +3007,7 @@ void RE_rotateBlenderScene(void)
 		ob= ob->id.next;
 	}
 
-	init_render_world();	/* do first, because of ambient */
+	init_render_world();	/* do first, because of ambient. also requires R.osa set correct */
 	init_render_textures();
 	init_render_materials();
 
@@ -3076,11 +3126,14 @@ void RE_rotateBlenderScene(void)
 				else {
 					/* yafray: if there are linked data objects (except lamps, empties or armatures),
 					   yafray only needs to know about one, the rest can be instanciated.
-					   The dupliMtx list is used for this purpose */
-					if (R.r.renderer==R_YAFRAY) {
-						/* Special case, parent object dupli's: ignore lattices & empty's */
+					   The dupliMtx list is used for this purpose.
+					   Exception: objects which have object linked materials, these cannot be instanciated. */
+					if ((R.r.renderer==R_YAFRAY) && (ob->colbits==0))
+					{
+						/* Special case, parent object dupli's: ignore if object itself is lamp or parent is lattice or empty */
 						if (ob->parent) {
-							if ((ob->parent->type!=OB_EMPTY) && (ob->parent->type!=OB_LATTICE) && YAF_objectKnownData(ob))
+							if ((ob->type!=OB_LAMP) && (ob->parent->type!=OB_EMPTY) &&
+									(ob->parent->type!=OB_LATTICE) && YAF_objectKnownData(ob))
 								printf("From parent: Added dupli matrix for linked data object %s\n", ob->id.name);
 							else
 								init_render_object(ob);
@@ -3123,6 +3176,78 @@ void RE_rotateBlenderScene(void)
 	set_normalflags();
 }
 
+/* **************************************************************** */
+/*                sticky texture coords                             */
+/* **************************************************************** */
+
+void RE_make_sticky(void)
+{
+	Object *ob;
+	Base *base;
+	MVert *mvert;
+	Mesh *me;
+	MSticky *ms;
+	float ho[4], mat[4][4];
+	int a;
+	
+	if(G.scene->camera==0) return;
+	
+	if(G.obedit) {
+		error("Unable to make sticky in Edit Mode");
+		return;
+	}
+	base= FIRSTBASE;
+	while(base) {
+		if TESTBASELIB(base) {
+			if(base->object->type==OB_MESH) {
+				ob= base->object;
+				
+				me= ob->data;
+				mvert= me->mvert;
+				if(me->msticky) MEM_freeN(me->msticky);
+				me->msticky= MEM_mallocN(me->totvert*sizeof(MSticky), "sticky");
+				
+				/* like convert to render data */		
+				R.r= G.scene->r;
+				R.r.xsch= (R.r.size*R.r.xsch)/100;
+				R.r.ysch= (R.r.size*R.r.ysch)/100;
+				
+				R.afmx= R.r.xsch/2;
+				R.afmy= R.r.ysch/2;
+				
+				R.ycor= ( (float)R.r.yasp)/( (float)R.r.xasp);
+				
+				R.rectx= R.r.xsch; 
+				R.recty= R.r.ysch;
+				R.xstart= -R.afmx; 
+				R.ystart= -R.afmy;
+				R.xend= R.xstart+R.rectx-1;
+				R.yend= R.ystart+R.recty-1;
+				
+				where_is_object(G.scene->camera);
+				Mat4CpyMat4(R.viewinv, G.scene->camera->obmat);
+				Mat4Ortho(R.viewinv);
+				Mat4Invert(R.viewmat, R.viewinv);
+				
+				RE_setwindowclip(1, -1);
+				
+				where_is_object(ob);
+				Mat4MulMat4(mat, ob->obmat, R.viewmat);
+				
+				ms= me->msticky;
+				for(a=0; a<me->totvert; a++, ms++, mvert++) {
+					VECCOPY(ho, mvert->co);
+					Mat4MulVecfl(mat, ho);
+					RE_projectverto(ho, ho);
+					ms->co[0]= ho[0]/ho[3];
+					ms->co[1]= ho[1]/ho[3];
+				}
+			}
+		}
+		base= base->next;
+	}
+}
+
 
 /* **************************************************************** */
 /*                Displacement mapping                              */
@@ -3136,7 +3261,7 @@ static short test_for_displace(Object *ob)
 	for (i=1; i<=ob->totcol; i++) {
 		ma=give_render_material(ob, i);
 		/* ma->mapto is ORed total of all mapto channels */
-		if(ma && (ma->ren->mapto & MAP_DISPLACE)) return 1;
+		if(ma && (ma->mapto & MAP_DISPLACE)) return 1;
 	}
 	return 0;
 }
@@ -3146,9 +3271,9 @@ static void do_displacement(Object *ob, int startface, int numface, int startver
 {
 	VertRen *vr;
 	VlakRen *vlr;
-	float min[3]={1e30, 1e30, 1e30}, max[3]={-1e30, -1e30, -1e30};
-	float scale[3]={1.0f, 1.0f, 1.0f}, temp[3], xn;
-	int i, texflag=0;
+//	float min[3]={1e30, 1e30, 1e30}, max[3]={-1e30, -1e30, -1e30};
+	float scale[3]={1.0f, 1.0f, 1.0f}, temp[3];//, xn
+	int i; //, texflag=0;
 	Object *obt;
 		
 	/* Object Size with parenting */
@@ -3177,18 +3302,18 @@ static void do_displacement(Object *ob, int startface, int numface, int startver
 static void displace_render_face(VlakRen *vlr, float *scale)
 {
 	ShadeInput shi;
-	VertRen vr;
-	float samp1,samp2, samp3, samp4, xn;
+//	VertRen vr;
+//	float samp1,samp2, samp3, samp4, xn;
 	short hasuv=0;
 	/* set up shadeinput struct for multitex() */
 	
 	shi.osatex= 0;		/* signal not to use dx[] and dy[] texture AA vectors */
 	shi.vlr= vlr;		/* current render face */
 	shi.mat= vlr->mat;		/* current input material */
-	shi.matren= shi.mat->ren;	/* material temp block where output is written into */
+
 	
 	/* UV coords must come from face */
-	hasuv = vlr->tface && (shi.matren->texco & TEXCO_UV);
+	hasuv = vlr->tface && (shi.mat->texco & TEXCO_UV);
 	if (hasuv) shi.uv[2]=0.0f; 
 	/* I don't think this is used, but seting it just in case */
 	
@@ -3244,7 +3369,7 @@ static void displace_render_face(VlakRen *vlr, float *scale)
 
 static void displace_render_vert(ShadeInput *shi, VertRen *vr, float *scale)
 {
-	short texco= shi->matren->texco;
+	short texco= shi->mat->texco;
 	float sample=0;
 	/* shi->co is current render coord, just make sure at least some vector is here */
 	VECCOPY(shi->co, vr->co);
