@@ -42,7 +42,6 @@
 #include <unistd.h>
 #else
 #include <io.h>
-#include "BLI_winstuff.h"
 #endif   
 
 #include "MEM_guardedalloc.h"
@@ -82,6 +81,9 @@
 #include "BIF_keyval.h"
 #include "BIF_editsound.h"
 
+#include "BIF_editsca.h"
+
+
 #include "BDR_editcurve.h"
 #include "BSE_buttons.h"
 #include "BSE_headerbuttons.h"
@@ -93,6 +95,8 @@
 #include "butspace.h" // own module
 
 /* internals */
+void buttons_enji(uiBlock *, Object *);
+void buttons_ketsji(uiBlock *, Object *);
 
 /****/
 
@@ -640,6 +644,8 @@ static char *sensor_name(int type)
 		return "Ray";
 	case SENS_MESSAGE:
 		return "Message";
+	case SENS_JOYSTICK:
+		return "Joystick";
 	}
 	return "unknown";
 }
@@ -649,7 +655,7 @@ static char *sensor_pup(void)
 	/* the number needs to match defines in game.h */
 	return "Sensors %t|Always %x0|Keyboard %x3|Mouse %x5|"
 		"Touch %x1|Collision %x6|Near %x2|Radar %x7|"
-		"Property %x4|Random %x8|Ray %x9|Message %x10";
+		"Property %x4|Random %x8|Ray %x9|Message %x10|Joystick %x11";
 }
 
 static char *controller_name(int type)
@@ -928,6 +934,7 @@ static int get_col_sensor(int type)
 	case SENS_RANDOM:		return TH_BUT_NEUTRAL;
 	case SENS_RAY:			return TH_BUT_SETTING1;
 	case SENS_MESSAGE:		return TH_BUT_SETTING2;
+	case SENS_JOYSTICK:		return TH_BUT_NEUTRAL;
 	default:				return TH_BUT_NEUTRAL;
 	}
 }
@@ -980,6 +987,8 @@ static short draw_sensorbuttons(bSensor *sens, uiBlock *block, short xco, short
 	bRandomSensor    *randomSensor = NULL;
 	bRaySensor       *raySens      = NULL;
 	bMessageSensor   *mes          = NULL;
+	bJoystickSensor	 *joy		   = NULL;
+
 	short ysize;
 	char *str;
 	
@@ -1299,6 +1308,64 @@ static short draw_sensorbuttons(bSensor *sens, uiBlock *block, short xco, short
 			yco -= ysize;
 			break;
 		}
+		case SENS_JOYSTICK:
+		{
+
+			ysize =  72;
+			
+			glRects(xco, yco-ysize, xco+width, yco);
+			uiEmboss((float)xco, (float)yco-ysize, (float)xco+width, (float)yco, 1);
+			
+			/* line 1: header */
+			draw_default_sensor_header(sens, block, xco, yco, width);
+
+			joy= sens->data;
+			
+
+			str= "Type %t|Button %x0|Axis %x1|Hat%x2"; 
+			uiDefButS(block, MENU, B_REDR, str, xco+10, yco-44, 0.6 * (width-20), 19,
+				&joy->type, 0, 31, 0, 0,
+				"The type of event this joystick sensor is triggered on.");
+			
+			if(joy->type == SENS_JOY_BUTTON)
+			{
+				uiDefButI(block, NUM, 1, "Number:", xco+10, yco-68, 0.6 * (width-20), 19,
+				&joy->button, 0, 18, 100, 0,
+				"Specify which button to use");
+				
+				str = "Type %t|Pressed %x0|Released %x1"; 
+				uiDefButI(block, MENU, B_REDR, str, xco+10 + 0.6 * (width-20), yco-68, 0.4 * (width-20), 19,
+				&joy->buttonf, 2.0, 31, 0, 0,
+				"Button pressed or released.");
+			}
+			else if(joy->type == SENS_JOY_AXIS)
+			{
+				uiDefButI(block, NUM, 1, "Number:", xco+10, yco-68, 0.6 * (width-20), 19,
+				&joy->axis, 1, 2.0, 100, 0,
+				"Specify which axis to use");
+
+				uiDefButI(block, NUM, 1, "Threshold:", xco+10 + 0.6 * (width-20),yco-44, 0.4 * (width-20), 19,
+				&joy->precision, 0, 32768.0, 100, 0,
+				"Specify the precision of the axis");
+
+				str = "Type %t|Up Axis %x1 |Down Axis %x3|Left Axis %x2|Right Axis %x0"; 
+				uiDefButI(block, MENU, B_REDR, str, xco+10 + 0.6 * (width-20), yco-68, 0.4 * (width-20), 19,
+				&joy->axisf, 2.0, 31, 0, 0,
+				"The direction of the axis");
+			}
+			else
+			{
+				uiDefButI(block, NUM, 1, "Number:", xco+10, yco-68, 0.6 * (width-20), 19,
+				&joy->hat, 1, 2.0, 100, 0,
+				"Specify which hat to use");
+				
+				uiDefButI(block, NUM, 1, "Direction:", xco+10 + 0.6 * (width-20), yco-68, 0.4 * (width-20), 19,
+				&joy->hatf, 0, 12, 100, 0,
+				"Specify hat direction");
+			}
+			yco-= ysize;
+			break;
+		}
 	}
 	
 	uiBlockSetEmboss(block, UI_EMBOSSM);
@@ -1504,13 +1571,13 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 			}
 			else
 			{
-				uiDefButS(block, NUM, 0, "Sta: ",xco+30, yco-64, (width-60)/2, 19, &aa->sta, 0.0, 18000.0, 0, 0, "Start frame");
-				uiDefButS(block, NUM, 0, "End: ",xco+30+(width-60)/2, yco-64, (width-60)/2, 19, &aa->end, 0.0, 18000.0, 0, 0, "End frame");
+				uiDefButS(block, NUM, 0, "Sta: ",xco+30, yco-64, (width-60)/2, 19, &aa->sta, 0.0, MAXFRAMEF, 0, 0, "Start frame");
+				uiDefButS(block, NUM, 0, "End: ",xco+30+(width-60)/2, yco-64, (width-60)/2, 19, &aa->end, 0.0, MAXFRAMEF, 0, 0, "End frame");
 			}
 			
 			
 			
-			uiDefButS(block, NUM, 0, "Blendin: ", xco+30, yco-84, (width-60)/2, 19, &aa->blendin, 0.0, 18000.0, 0.0, 0.0, "Number of frames of motion blending");
+			uiDefButS(block, NUM, 0, "Blendin: ", xco+30, yco-84, (width-60)/2, 19, &aa->blendin, 0.0, MAXFRAMEF, 0.0, 0.0, "Number of frames of motion blending");
 			uiDefButS(block, NUM, 0, "Priority: ", xco+30+(width-60)/2, yco-84, (width-60)/2, 19, &aa->priority, 0.0, 100.0, 0.0, 0.0, "Execution priority - lower numbers will override actions with higher numbers");
 			
 #ifdef __NLA_ACTION_BY_MOTION_ACTUATOR
@@ -1564,11 +1631,11 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 			else {
 				uiDefButS(block, NUM, 0, 
 					"Sta",		xco+20, yco-44, (width-100)/2, 19, 
-					&ia->sta, 0.0, 18000.0, 0, 0, 
+					&ia->sta, 0.0, MAXFRAMEF, 0, 0, 
 					"Start frame");
 				uiDefButS(block, NUM, 0, 
 					"End",		xco+18+(width-90)/2, yco-44, (width-100)/2, 19, 
-					&ia->end, 0.0, 18000.0, 0, 0, 
+					&ia->end, 0.0, MAXFRAMEF, 0, 0, 
 					"End frame");
 				
 				uiDefButS(block, TOG|BIT|ACT_IPOFORCE_BIT, B_REDR, 
@@ -2431,8 +2498,9 @@ void logic_buts(void)
 	uiBlockSetCol(block, TH_BUT_SETTING2);
 
 	if(wrld) {
-		if (wrld->physicsEngine == 1) buttons_enji(block, ob);
-		if ( (wrld->physicsEngine == 4) || (wrld->physicsEngine == 2) )
+		if 
+			(wrld->physicsEngine == 1) buttons_enji(block, ob);
+		else
 			buttons_ketsji(block, ob);
 	}
 	else buttons_ketsji(block, ob);
