@@ -39,11 +39,8 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include "MEM_guardedalloc.h"
+
 #include "DNA_screen_types.h"
 #include "DNA_space_types.h"
 #include "DNA_scene_types.h"
@@ -51,11 +48,13 @@
 #include "BKE_global.h"
 #include "BKE_main.h"
 #include "BKE_library.h"
+#include "BKE_softbody.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 
 #include "BSE_filesel.h"
+#include "BSE_headerbuttons.h"
 
 #include "BIF_gl.h"
 #include "BIF_graphics.h"
@@ -71,6 +70,8 @@
 #include "BDR_editcurve.h"
 #include "BDR_drawobject.h"
 
+//#include "BIF_editsca.h"
+
 #include "BIF_butspace.h"
 
 #include "mydevice.h"
@@ -95,6 +96,7 @@
 #include "DNA_meta_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_object_types.h"
+#include "DNA_object_force.h"
 #include "DNA_radio_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_sound_types.h"
@@ -133,14 +135,21 @@ static float prspeed=0.0;
 float prlen=0.0;
 
 
+/* ********************* function prototypes ******************** */
+void object_panel_draw(Object *);
+void object_panel_hooks(Object *);
+void object_panel_effects(Object *);
 
 /* ********************* CONSTRAINT ***************************** */
 
+#if 0
 static void add_influence_key_to_constraint_func (void *arg1v, void *unused)
 {
 	bConstraint *con = arg1v;
 	add_influence_key_to_constraint(con);
 }
+#endif
+
 static void activate_constraint_ipo_func (void *arg1v, void *unused)
 {
 
@@ -440,8 +449,8 @@ static void draw_constraint (uiBlock *block, ListBase *list, bConstraint *con, s
 				uiDefButI(block, MENU, B_CONSTRAINT_REDRAW, "Key on%t|X Rot%x0|Y Rot%x1|Z Rot%x2", *xco+((width/2)-117), *yco-84, 78, 18, &data->type, 0, 24, 0, 0, "Specify which transformation channel from the target is used to key the action");
 
 				uiBlockBeginAlign(block);
-				uiDefButS(block, NUM, B_CONSTRAINT_CHANGETARGET, "Start:", *xco+((width/2)-36), *yco-64, 78, 18, &data->start, 1, 18000, 0.0, 0.0, "Starting frame of the keyed motion"); 
-				uiDefButS(block, NUM, B_CONSTRAINT_CHANGETARGET, "End:", *xco+((width/2)-36), *yco-84, 78, 18, &data->end, 1, 18000, 0.0, 0.0, "Ending frame of the keyed motion"); 
+				uiDefButS(block, NUM, B_CONSTRAINT_CHANGETARGET, "Start:", *xco+((width/2)-36), *yco-64, 78, 18, &data->start, 1, MAXFRAME, 0.0, 0.0, "Starting frame of the keyed motion"); 
+				uiDefButS(block, NUM, B_CONSTRAINT_CHANGETARGET, "End:", *xco+((width/2)-36), *yco-84, 78, 18, &data->end, 1, MAXFRAME, 0.0, 0.0, "Ending frame of the keyed motion"); 
 				uiBlockEndAlign(block);
 				
 				uiBlockBeginAlign(block);
@@ -1095,6 +1104,54 @@ void object_panel_hooks(Object *ob)
 	uiDefBut(block, BUT, B_CLR_HOOK, "Clear offset", 		160,80,150,19, NULL, 0.0, 0.0, 0, 0, "Recalculate and clear offset (transform) of hook");
 }
 
+static void softbody_bake(Object *ob)
+{
+	SoftBody *sb= ob->soft;
+	ScrArea *sa;
+	float frameleno= G.scene->r.framelen;
+	int cfrao= CFRA;
+	unsigned short event=0;
+	short val;
+	
+	G.scene->r.framelen= 1.0;	// baking has to be in uncorrected time
+	CFRA= sb->sfra;
+	sbObjectToSoftbody(ob);
+	ob->softflag |= OB_SB_BAKEDO;
+	
+	curarea->win_swap= 0;		// clean swapbuffers
+	
+	for(; CFRA <= sb->efra; CFRA++) {
+		set_timecursor(CFRA);
+		
+		update_for_newframe_muted();
+		
+		for(sa= G.curscreen->areabase.first; sa; sa= sa->next) {
+			if(sa->spacetype == SPACE_VIEW3D) {
+				scrarea_do_windraw(sa);
+			}
+		}
+		screen_swapbuffers();
+		
+		while(qtest()) {
+			
+			event= extern_qread(&val);
+			if(event==ESCKEY) break;
+		}
+		if(event==ESCKEY) break;
+	}
+	
+	if(event==ESCKEY) sbObjectToSoftbody(ob);	// clears all
+	
+	/* restore */
+	waitcursor(0);
+	ob->softflag &= ~OB_SB_BAKEDO;
+	CFRA= cfrao;
+	G.scene->r.framelen= frameleno;
+	update_for_newframe_muted();
+	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWBUTSOBJECT, 0);
+}
+
 
 void do_object_panels(unsigned short event)
 {
@@ -1184,6 +1241,34 @@ void do_object_panels(unsigned short event)
 		allqueue(REDRAWVIEW3D, 0);
 		break;
 	
+	case B_SOFTBODY_CHANGE:
+		ob= OBACT;
+		if(ob) {
+			ob->softflag |= OB_SB_REDO;
+			allqueue(REDRAWBUTSOBJECT, 0);
+			allqueue(REDRAWVIEW3D, 0);
+		}
+		break;
+	case B_SOFTBODY_DEL_VG:
+		ob= OBACT;
+		if(ob && ob->soft) {
+			ob->soft->vertgroup= 0;
+			ob->softflag |= OB_SB_REDO;
+			allqueue(REDRAWBUTSOBJECT, 0);
+			allqueue(REDRAWVIEW3D, 0);
+		}
+		break;
+	case B_SOFTBODY_BAKE:
+		ob= OBACT;
+		if(ob && ob->soft) softbody_bake(ob);
+		break;
+	case B_SOFTBODY_BAKE_FREE:
+		ob= OBACT;
+		if(ob && ob->soft) sbObjectToSoftbody(ob);
+		allqueue(REDRAWBUTSOBJECT, 0);
+		allqueue(REDRAWVIEW3D, 0);
+		break;
+		
 	default:
 		if(event>=B_SELEFFECT && event<B_SELEFFECT+MAX_EFFECT) {
 			ob= OBACT;
@@ -1235,9 +1320,9 @@ static void object_panel_anim(Object *ob)
 	uiDefButC(block, TOG|BIT|5, REDRAWVIEW3D, "Rot",		200,128,31,19, &ob->transflag, 0, 0, 0, 0, "Rotate dupli according to facenormal");
 	uiDefButC(block, TOG|BIT|6, REDRAWVIEW3D, "No Speed",	234,128,82,19, &ob->transflag, 0, 0, 0, 0, "Set dupliframes to still, regardless of frame");
 	uiBlockBeginAlign(block);
-	uiDefButS(block, NUM, REDRAWVIEW3D, "DupSta:",		24,105,141,19, &ob->dupsta, 1.0, 17999.0, 0, 0, "Specify startframe for Dupliframes");
+	uiDefButS(block, NUM, REDRAWVIEW3D, "DupSta:",		24,105,141,19, &ob->dupsta, 1.0, (MAXFRAMEF - 1.0f), 0, 0, "Specify startframe for Dupliframes");
 	uiDefButS(block, NUM, REDRAWVIEW3D, "DupOn:",		170,105,146,19, &ob->dupon, 1.0, 1500.0, 0, 0, "");
-	uiDefButS(block, NUM, REDRAWVIEW3D, "DupEnd",		24,82,140,19, &ob->dupend, 1.0, 18000.0, 0, 0, "Specify endframe for Dupliframes");
+	uiDefButS(block, NUM, REDRAWVIEW3D, "DupEnd",		24,82,140,19, &ob->dupend, 1.0, MAXFRAMEF, 0, 0, "Specify endframe for Dupliframes");
 	uiDefButS(block, NUM, REDRAWVIEW3D, "DupOff",		171,82,145,19, &ob->dupoff, 0.0, 1500.0, 0, 0, "");
 	uiBlockBeginAlign(block);
 	uiDefButC(block, TOG|BIT|2, REDRAWALL, "Offs Ob",			24,51,56,20, &ob->ipoflag, 0, 0, 0, 0, "Let the timeoffset work on its own objectipo");
@@ -1272,6 +1357,7 @@ void do_effects_panels(unsigned short event)
 	case B_FRAMEMAP:
 		G.scene->r.framelen= G.scene->r.framapto;
 		G.scene->r.framelen/= G.scene->r.images;
+		allqueue(REDRAWALL, 0);
 		break;
 	case B_NEWEFFECT:
 		if(ob) {
@@ -1413,35 +1499,178 @@ static void object_panel_deflectors(Object *ob)
 	if(ob->pd==NULL) {
 		ob->pd= MEM_callocN(sizeof(PartDeflect), "PartDeflect");
 		/* and if needed, init here */
+		ob->pd->pdef_sbdamp = 0.1;
+		ob->pd->pdef_sbift  = 0.2;
+		ob->pd->pdef_sboft  = 0.02;
 	}
 	
 	if(ob->pd) {
+		PartDeflect *pd= ob->pd;
+		
+		uiDefBut(block, LABEL, 0, "Fields",		10,180,140,20, NULL, 0.0, 0, 0, 0, "");
+		
 		uiBlockBeginAlign(block);
+		uiDefButS(block, ROW, REDRAWVIEW3D, "None",			10,160,50,20, &pd->forcefield, 1.0, 0, 0, 0, "No force");
+		uiDefButS(block, ROW, REDRAWVIEW3D, "Force field",	60,160,90,20, &pd->forcefield, 1.0, PFIELD_FORCE, 0, 0, "Object center attracts or repels particles");
+		uiDefButS(block, ROW, REDRAWVIEW3D, "Wind",			10,140,50,20, &pd->forcefield, 1.0, PFIELD_WIND, 0, 0, "Constant force applied in direction of Object Z axis");
+		uiDefButS(block, ROW, REDRAWVIEW3D, "Vortex field",	60,140,90,20, &pd->forcefield, 1.0, PFIELD_VORTEX, 0, 0, "Particles swirl around Z-axis of the object");
 
-		uiDefButS(block, ROW, REDRAWVIEW3D, "None",	10,160,200,20, &ob->pd->forcefield, 1.0, 0, 0, 0, "No force");
-		uiDefButS(block, ROW, REDRAWVIEW3D, "Force field",	10,140,200,20, &ob->pd->forcefield, 1.0, PFIELD_FORCE, 0, 0, "Object center attracts or repels particles");
-		uiDefButS(block, ROW, REDRAWVIEW3D, "Vortex field",	10,120,200,20, &ob->pd->forcefield, 1.0, PFIELD_VORTEX, 0, 0, "Particles swirl around Z-axis of the object");
-
-		uiDefButF(block, NUM, REDRAWVIEW3D, "Strength: ",	10,100,200,20, &ob->pd->f_strength, -1000, 1000, 1000, 0, "Strength of force field");
-		uiDefButF(block, NUM, REDRAWVIEW3D, "Fall-off: ",	10,80,200,20, &ob->pd->f_power, 0, 10, 100, 0, "Falloff power (real gravitational fallof = 2)");
-		/* only meshes collide now */
-		if(ob->type==OB_MESH) {
-			uiBlockBeginAlign(block);
-			uiDefButS(block, TOG|BIT|0, B_DIFF, "Deflection",10,50,200,20, &ob->pd->deflect, 0, 0, 0, 0, "Deflects particles based on collision");
-			uiDefButF(block, NUM, B_DIFF, "Damping: ",	10,30,200,20, &ob->pd->pdef_damp, 0.0, 1.0, 10, 0, "Amount of damping during particle collision");
-			uiDefButF(block, NUM, B_DIFF, "Rnd Damping: ",	10,10,200,20, &ob->pd->pdef_rdamp, 0.0, 1.0, 10, 0, "Random variation of damping");
-			uiDefButF(block, NUM, B_DIFF, "Permeability: ",		10,-10,200,20, &ob->pd->pdef_perm, 0.0, 1.0, 10, 0, "Chance that the particle will pass through the mesh");
-		}
+		uiBlockBeginAlign(block);
+		uiDefButF(block, NUM, REDRAWVIEW3D, "Strength: ",	10,110,140,20, &pd->f_strength, -1000, 1000, 1000, 0, "Strength of force field");
+		uiDefButF(block, NUM, REDRAWVIEW3D, "Fall-off: ",	10,90,140,20, &pd->f_power, 0, 10, 100, 0, "Falloff power (real gravitational fallof = 2)");
+		
+		uiBlockBeginAlign(block);
+		uiDefButBitS(block, TOG, PFIELD_USEMAX, REDRAWVIEW3D, "Use MaxDist",	10,60,140,20, &pd->flag, 0.0, 0, 0, 0, "Use a maximum distance for the field to work");
+		uiDefButF(block, NUM, REDRAWVIEW3D, "MaxDist: ",	10,40,140,20, &pd->maxdist, 0, 1000.0, 100, 0, "Maximum distance for the field to work");
 		uiBlockEndAlign(block);
-	}
 
-	if(strncmp(ob->id.name+2, "soft", 4)==0) {
-		uiDefButS(block, TOG|BIT|0, B_DIFF, "Soft Body",	220,160,200,20, &ob->softflag, 0, 0, 0, 0, "Sets object to become soft body");
-		uiDefButF(block, NUM, B_DIFF, "Spring: ",			220,140,200,20, &ob->springf, 0.0, 1.0, 10, 0, "Spring constant");
-		uiDefButF(block, NUM, B_DIFF, "Damp: ",				220,120,200,20, &ob->damping, 0.0, 1.0, 10, 0, "General damping in softbody on point movements");
+		if(ob->softflag & OB_SB_ENABLE) {
+			uiDefBut(block, LABEL, 0, "Object is Softbody,",		160,160,150,20, NULL, 0.0, 0, 0, 0, "");
+			uiDefBut(block, LABEL, 0, "no Deflection possible",		160,140,150,20, NULL, 0.0, 0, 0, 0, "");
+			pd->deflect= 0;
+		}
+		else {
+			
+			uiDefBut(block, LABEL, 0, "Deflection",	160,180,140,20, NULL, 0.0, 0, 0, 0, "");
+			
+			/* only meshes collide now */
+			if(ob->type==OB_MESH) {
+				uiDefButS(block, TOG|BIT|0, B_REDR, "Deflection",160,160,150,20, &pd->deflect, 0, 0, 0, 0, "Deflects particles based on collision");
+				uiDefBut(block, LABEL, 0, "Particles",			160,140,150,20, NULL, 0.0, 0, 0, 0, "");
+				
+				uiBlockBeginAlign(block);
+				uiDefButF(block, NUM, B_DIFF, "Damping: ",		160,120,150,20, &pd->pdef_damp, 0.0, 1.0, 10, 0, "Amount of damping during particle collision");
+				uiDefButF(block, NUM, B_DIFF, "Rnd Damping: ",	160,100,150,20, &pd->pdef_rdamp, 0.0, 1.0, 10, 0, "Random variation of damping");
+				uiDefButF(block, NUM, B_DIFF, "Permeability: ",	160,80,150,20, &pd->pdef_perm, 0.0, 1.0, 10, 0, "Chance that the particle will pass through the mesh");
+				uiBlockEndAlign(block);
+				
+				uiDefBut(block, LABEL, 0, "Soft Body",			160,60,150,20, NULL, 0.0, 0, 0, 0, "");
+
+				uiBlockBeginAlign(block);
+				uiDefButF(block, NUM, B_DIFF, "Damping:",	160,40,150,20, &pd->pdef_sbdamp, 0.0, 1.0, 10, 0, "Amount of damping during softbody collision");
+				uiDefButF(block, NUM, B_DIFF, "Inner:",	160,20,150,20, &pd->pdef_sbift, 0.001, 1.0, 10, 0, "Inner face thickness");
+				uiDefButF(block, NUM, B_DIFF, "Outer:",	160, 0,150,20, &pd->pdef_sboft, 0.001, 1.0, 10, 0, "Outer face thickness");
+			}		
+		}	
 	}
 }
 
+
+/* Panel for softbodies */
+
+static void object_softbodies(Object *ob)
+{
+	uiBlock *block;
+	
+	block= uiNewBlock(&curarea->uiblocks, "object_softbodies", UI_EMBOSS, UI_HELV, curarea->win);
+	uiNewPanelTabbed("Constraints", "Object");
+	if(uiNewPanel(curarea, block, "Softbody", "Object", 640, 0, 318, 204)==0) return;
+
+	/* do not allow to combine with force fields */
+	if(ob->pd) {
+		PartDeflect *pd= ob->pd;
+		
+		if(pd->deflect) {
+			uiDefBut(block, LABEL, 0, "Object has Deflection,",		10,160,300,20, NULL, 0.0, 0, 0, 0, "");
+			uiDefBut(block, LABEL, 0, "no Softbody possible",		10,140,300,20, NULL, 0.0, 0, 0, 0, "");
+			ob->softflag &= ~OB_SB_ENABLE;
+			
+			return;
+		}
+	}
+	
+	uiDefButBitS(block, TOG, OB_SB_ENABLE, REDRAWBUTSOBJECT, "Enable Soft Body",	10,200,150,20, &ob->softflag, 0, 0, 0, 0, "Sets object to become soft body");
+	uiDefBut(block, LABEL, 0, "",	160, 200,150,20, NULL, 0.0, 0.0, 0, 0, "");	// alignment reason
+	
+	if(ob->softflag & OB_SB_ENABLE) {
+		SoftBody *sb= ob->soft;
+		int defCount;
+		char *menustr;
+		
+		if(sb==NULL) {
+			sb= ob->soft= sbNew();
+			ob->softflag |= OB_SB_GOAL|OB_SB_EDGES;
+			// default add edges for softbody
+			if(ob->type==OB_MESH) {
+				Mesh *me= ob->data;
+				if(me->medge==NULL) make_edges(me);
+			}
+		}
+		
+		uiDefButBitS(block, TOG, OB_SB_BAKESET, REDRAWBUTSOBJECT, "Bake settings",	180,200,130,20, &ob->softflag, 0, 0, 0, 0, "To convert simulation into baked (cached) result");
+		
+		if(sb->keys) uiSetButLock(1, "SoftBody is baked, free it first");
+		
+		if(ob->softflag & OB_SB_BAKESET) {
+			uiBlockBeginAlign(block);
+			uiDefButS(block, NUM, B_DIFF, "Start:",			10, 170,100,20, &sb->sfra, 1.0, 10000.0, 10, 0, "Start frame for baking");
+			uiDefButS(block, NUM, B_DIFF, "End:",			110, 170,100,20, &sb->efra, 1.0, 10000.0, 10, 0, "End frame for baking");
+			uiDefButS(block, NUM, B_DIFF, "Interval:",		210, 170,100,20, &sb->interval, 1.0, 10.0, 10, 0, "Interval in frames between baked keys");
+			
+			uiClearButLock();
+			
+			uiBlockBeginAlign(block);
+			if(sb->keys) {
+				char str[128];
+				uiDefIconTextBut(block, BUT, B_SOFTBODY_BAKE_FREE, ICON_X, "FREE BAKE", 10, 120,300,20, NULL, 0.0, 0.0, 0, 0, "Free baked result");
+				sprintf(str, "Stored %d vertices %d keys %.3f MB", sb->totpoint, sb->totkey, ((float)16*sb->totpoint*sb->totkey)/(1024.0*1024.0));
+				uiDefBut(block, LABEL, 0, str, 10, 100,300,20, NULL, 0.0, 0.0, 00, 0, "");
+			}
+			else				
+				uiDefBut(block, BUT, B_SOFTBODY_BAKE, "BAKE",	10, 120,300,20, NULL, 0.0, 0.0, 10, 0, "Start baking. Press ESC to exit without baking");
+		}
+		else {
+			/* GENERAL STUFF */
+			uiBlockBeginAlign(block);
+			uiDefButF(block, NUM, B_DIFF, "Friction:",		10, 170,150,20, &sb->mediafrict, 0.0, 10.0, 10, 0, "General media friction for point movements");
+			uiDefButF(block, NUM, B_DIFF, "Mass:",			160, 170,150,20, &sb->nodemass , 0.001, 50.0, 10, 0, "Point Mass, the heavier the slower");
+			uiDefButF(block, NUM, B_DIFF, "Grav:",			10,150,150,20, &sb->grav , 0.0, 10.0, 10, 0, "Apply gravitation to point movement");
+			uiDefButF(block, NUM, B_DIFF, "Speed:",			160,150,150,20, &sb->physics_speed , 0.01, 100.0, 10, 0, "Tweak timing for physics to control frequency and speed");
+			uiDefButF(block, NUM, B_DIFF, "Error Limit:",	10,130,150,20, &sb->rklimit , 0.01, 1.0, 10, 0, "The Runge-Kutta ODE solver error limit, low value gives more precision");
+			uiDefButBitS(block, TOG, OB_SB_POSTDEF, B_DIFF, "Apply Deform First",	160,130,150,20, &ob->softflag, 0, 0, 0, 0, "Softbody is calculated AFTER Deformation");
+			uiBlockEndAlign(block);
+			
+			/* GOAL STUFF */
+			uiBlockBeginAlign(block);
+			uiDefButBitS(block, TOG, OB_SB_GOAL, B_SOFTBODY_CHANGE, "Use Goal",	10,100,130,20, &ob->softflag, 0, 0, 0, 0, "Define forces for vertices to stick to animated position");
+			
+			menustr= get_vertexgroup_menustr(ob);
+			defCount=BLI_countlist(&ob->defbase);
+			if(defCount==0) sb->vertgroup= 0;
+			uiDefButS(block, MENU, B_SOFTBODY_CHANGE, menustr,		140,100,20,20, &sb->vertgroup, 0, defCount, 0, 0, "Browses available vertex groups");
+			
+			if(sb->vertgroup) {
+				bDeformGroup *defGroup = BLI_findlink(&ob->defbase, sb->vertgroup-1);
+				if(defGroup)
+					uiDefBut(block, BUT, B_DIFF, defGroup->name,	160,100,130,20, NULL, 0.0, 0.0, 0, 0, "Name of current vertex group");
+				else
+					uiDefBut(block, BUT, B_DIFF, "(no group)",	160,100,130,20, NULL, 0.0, 0.0, 0, 0, "Vertex Group doesn't exist anymore");
+				uiDefIconBut(block, BUT, B_SOFTBODY_DEL_VG, ICON_X, 290,100,20,20, 0, 0, 0, 0, 0, "Disable use of vertex group");
+			}
+			else {
+				uiDefButF(block, NUM, B_SOFTBODY_CHANGE, "Goal:",	160,100,150,20, &sb->defgoal, 0.0, 1.0, 10, 0, "Default Goal (vertex target position) value, when no Vertex Group used");
+			}
+			MEM_freeN (menustr);
+
+			uiDefButF(block, NUM, B_DIFF, "G Stiff:",	10,80,150,20, &sb->goalspring, 0.0, 0.999, 10, 0, "Goal (vertex target position) spring stiffness");
+			uiDefButF(block, NUM, B_DIFF, "G Damp:",	160,80,150,20, &sb->goalfrict  , 0.0, 10.0, 10, 0, "Goal (vertex target position) friction");
+			uiDefButF(block, NUM, B_SOFTBODY_CHANGE, "G Min:",		10,60,150,20, &sb->mingoal, 0.0, 1.0, 10, 0, "Goal minimum, vertex group weights are scaled to match this range");
+			uiDefButF(block, NUM, B_SOFTBODY_CHANGE, "G Max:",		160,60,150,20, &sb->maxgoal, 0.0, 1.0, 10, 0, "Goal maximum, vertex group weights are scaled to match this range");
+			uiBlockEndAlign(block);
+			
+			/* EDGE SPRING STUFF */
+			uiBlockBeginAlign(block);
+			uiDefButBitS(block, TOG, OB_SB_EDGES, B_SOFTBODY_CHANGE, "Use Edges",		10,30,150,20, &ob->softflag, 0, 0, 0, 0, "Use Edges as springs");
+			uiDefButBitS(block, TOG, OB_SB_QUADS, B_SOFTBODY_CHANGE, "Stiff Quads",		160,30,150,20, &ob->softflag, 0, 0, 0, 0, "Adds diagonal springs on 4-gons");
+			uiDefButF(block, NUM, B_DIFF, "E Stiff:",	10,10,150,20, &sb->inspring, 0.0,  0.999, 10, 0, "Edge spring stiffness");
+			uiDefButF(block, NUM, B_DIFF, "E Damp:",	160,10,150,20, &sb->infrict, 0.0,  10.0, 10, 0, "Edge spring friction");
+			uiBlockEndAlign(block);
+		}
+	}
+	uiBlockEndAlign(block);
+
+}
+
 void object_panel_effects(Object *ob)
 {
 	Effect *eff;
@@ -1451,7 +1680,7 @@ void object_panel_effects(Object *ob)
 	
 	block= uiNewBlock(&curarea->uiblocks, "object_panel_effects", UI_EMBOSS, UI_HELV, curarea->win);
 	uiNewPanelTabbed("Constraints", "Object");
-	if(uiNewPanel(curarea, block, "Effects", "Object", 640, 0, 418, 204)==0) return;
+	if(uiNewPanel(curarea, block, "Effects", "Object", 640, 0, 318, 204)==0) return;
 
 	/* EFFECTS */
 	
@@ -1579,8 +1808,8 @@ void object_panel_effects(Object *ob)
 			uiDefButS(block, ROW, B_CALCEFFECT, "Int",		875,9,32,43, &paf->texmap, 14.0, 0.0, 0, 0, "Use texture intensity as a factor for texture force");
 
 			uiBlockBeginAlign(block);
-			uiDefButS(block, ROW, B_CALCEFFECT, "RGB",		911,31,45,20, &paf->texmap, 14.0, 1.0, 0, 0, "Use RGB values as a factor for particle speed");
-			uiDefButS(block, ROW, B_CALCEFFECT, "Grad",		958,31,44,20, &paf->texmap, 14.0, 2.0, 0, 0, "Use texture gradient as a factor for particle speed");
+			uiDefButS(block, ROW, B_CALCEFFECT, "RGB",		911,31,45,20, &paf->texmap, 14.0, 1.0, 0, 0, "Use RGB values as a factor for particle speed vector");
+			uiDefButS(block, ROW, B_CALCEFFECT, "Grad",		958,31,44,20, &paf->texmap, 14.0, 2.0, 0, 0, "Use texture gradient as a factor for particle speed vector");
 			uiDefButF(block, NUM, B_CALCEFFECT, "Nabla:",		911,9,91,20, &paf->nabla, 0.0001f, 1.0, 1, 0, "Specify the dimension of the area for gradient calculation");
 
 		}
@@ -1604,6 +1833,7 @@ void object_panels()
 			object_panel_effects(ob);
 		}
 		object_panel_deflectors(ob);
+		object_softbodies(ob);
 
 		uiClearButLock();
 	}
