@@ -1,5 +1,5 @@
 /**
-* $Id: cursors.c,v 1.4 2004/05/04 19:10:55 zuster Exp $
+* $Id: cursors.c,v 1.6 2005/03/19 21:08:11 zuster Exp $
 *
 * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
 *
@@ -36,10 +36,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif   
-
 #include "DNA_listBase.h"
 #include "DNA_userdef_types.h" 
 
@@ -115,11 +111,11 @@ void SetBlenderCursor(short curs){
 	}
 }
 
-
+/* unused no prototypes 
 short GetCurrentCursor(void){
 	return(CurrentCursor);
 }
-
+*/
 
 void InitCursorData(void){
 
