@@ -1,5 +1,5 @@
 /**
- * $Id: edit.c,v 1.27 2004/04/24 21:51:38 ton Exp $
+ * $Id: edit.c,v 1.66 2005/05/07 15:50:08 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,6 +31,7 @@
  */
 
 #include <math.h>
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -40,14 +41,18 @@
 #include <unistd.h>
 #else
 #include <io.h>
-#include "BLI_winstuff.h"
 #endif   
+
 #include "MEM_guardedalloc.h"
 
 #include "BMF_Api.h"
 
+#include "PIL_time.h"
+
+#include "DNA_action_types.h"
 #include "DNA_armature_types.h"
 #include "DNA_curve_types.h"
+#include "DNA_ipo_types.h"
 #include "DNA_lattice_types.h"
 #include "DNA_meta_types.h"
 #include "DNA_mesh_types.h"
@@ -61,29 +66,35 @@
 #include "BLI_arithb.h"
 #include "BLI_editVert.h"
 
-#include "BKE_utildefines.h"
+#include "BKE_action.h"
+#include "BKE_armature.h"
 #include "BKE_anim.h"
-#include "BKE_object.h"
+#include "BKE_curve.h"
 #include "BKE_displist.h"
 #include "BKE_global.h"
+#include "BKE_ipo.h"
 #include "BKE_lattice.h"
 #include "BKE_mesh.h"
+#include "BKE_object.h"
+#include "BKE_utildefines.h"
 
-#include "BIF_gl.h"
-#include "BIF_mywindow.h"
-#include "BIF_screen.h"
-#include "BIF_interface.h"
-#include "BIF_space.h"
-#include "BIF_editview.h"
-#include "BIF_glutil.h"
-#include "BIF_toolbox.h"
 #include "BIF_editmesh.h"
+#include "BIF_editview.h"
+#include "BIF_editarmature.h"
+#include "BIF_gl.h"
+#include "BIF_glutil.h"
+#include "BIF_interface.h"
+#include "BIF_mywindow.h"
+#include "BIF_resources.h"
+#include "BIF_space.h"
+#include "BIF_screen.h"
+#include "BIF_toolbox.h"
 
-#include "BSE_view.h"
 #include "BSE_edit.h"
-#include "BSE_trans_types.h"
 #include "BSE_drawipo.h"
 #include "BSE_drawview.h"
+#include "BSE_trans_types.h"
+#include "BSE_view.h"
 
 #include "BDR_editobject.h"
 #include "BDR_editmball.h"
@@ -101,12 +112,6 @@
 #include "BIF_editarmature.h"
 #endif
 
-/* editmball.c */
-extern ListBase editelems;  /* go away ! */
-
-
-/* from editobject */
-extern void make_trans_verts(float *min, float *max, int mode);     
 
 /* circle selection callback */
 typedef void (*select_CBfunc)(short selecting, Object *editobj, short *mval, float rad);
@@ -118,6 +123,10 @@ extern void uvedit_selectionCB(short selecting, Object *editobj,
 
 void circle_selectCB(select_CBfunc func);
 
+/* local protos ---------------*/
+void snap_curs_to_firstsel(void);
+
+
 int get_border(rcti *rect, short col)
 {
 	float dvec[4], fac1, fac2;
@@ -146,14 +155,10 @@ int get_border(rcti *rect, short col)
 	/* draws the selection initial cross */
 	sdrawXORline4(0, 0,  mvalo[1],  curarea->winx,  mvalo[1]);
 	sdrawXORline4(1, mvalo[0],  0,  mvalo[0],  curarea->winy); 
-
+	glFlush();
+	
 	while(TRUE) {
 	
-		/* for when a renderwindow is open, and a mouse cursor activates it */
-		persp(PERSP_VIEW);
-		mywinset(curarea->win);
-		persp(PERSP_WIN);
-		
 		/* selection loop while mouse pressed */
 		getmouseco_areawin(mval);
 
@@ -170,6 +175,12 @@ int get_border(rcti *rect, short col)
 		event= extern_qread(&val);
 
 		if(event && val) {
+
+			/* for when a renderwindow is open, and a mouse cursor activates it */
+			persp(PERSP_VIEW);
+			mywinset(curarea->win);
+			persp(PERSP_WIN);
+			
 			if(event==ESCKEY) {
 				retval= 0;
 				break;
@@ -183,6 +194,8 @@ int get_border(rcti *rect, short col)
 			else if(event==MIDDLEMOUSE) break;
 			else if(event==RIGHTMOUSE) break;
 		}
+		else PIL_sleep_ms(10);
+		
 	} /* end while (TRUE) */
 
 	/* erase XORed lines */
@@ -209,6 +222,63 @@ int get_border(rcti *rect, short col)
 				sdrawXORline4(1, x1, mval[1], mval[0], mval[1]); 
 				sdrawXORline4(2, mval[0], mval[1], mval[0], y1); 
 				sdrawXORline4(3,  mval[0], y1, x1, y1); 
+				
+				/* draw size information in corner */
+				if(curarea->spacetype==SPACE_VIEW3D) {
+					BIF_ThemeColor(TH_BACK);
+					glRecti(10, 25, 250, 40);
+	
+					if(G.vd->persp==0) {
+						window_to_3d(dvec, mvalo[0]-x1, mvalo[1]-y1);
+	
+						sprintf(str, "X %.4f  Y %.4f  Z %.4f  Dia %.4f", dvec[0], dvec[1], dvec[2], sqrt(dvec[0]*dvec[0]+dvec[1]*dvec[1]+dvec[2]*dvec[2]));
+						glColor3f(0.0, 0.0, 0.0); 
+						glRasterPos2i(15,  27);
+						BMF_DrawString(G.fonts, str);
+						glColor3f(0.7, 0.7, 0.7); 
+						glRasterPos2i(16,  28);
+						BMF_DrawString(G.fonts, str);
+					}
+					else if(G.vd->persp==2) {
+						rcti vb;
+	
+						calc_viewborder(G.vd, &vb);
+	
+						fac1= (mvalo[0]-x1)/( (float) (vb.xmax-vb.xmin) );
+						fac1*= 0.01*G.scene->r.size*G.scene->r.xsch;
+						
+						fac2= (mvalo[1]-y1)/( (float) (vb.ymax-vb.ymin) );
+						fac2*= 0.01*G.scene->r.size*G.scene->r.ysch;
+						
+						sprintf(str, "X %.1f  Y %.1f  Dia %.1f", fabs(fac1), fabs(fac2), sqrt(fac1*fac1 + fac2*fac2) );
+						glColor3f(0.0, 0.0, 0.0); 
+						glRasterPos2i(15,  27);
+						BMF_DrawString(G.fonts, str);
+						glColor3f(0.7, 0.7, 0.7); 
+						glRasterPos2i(16,  28);
+						BMF_DrawString(G.fonts, str);
+					}
+				}
+				else if(curarea->spacetype==SPACE_IPO) {
+					SpaceIpo *sipo= curarea->spacedata.first;
+	
+					BIF_ThemeColor(TH_BACK);
+					glRecti(20, 30, 170, 40);
+								
+					mvalo[2]= x1;
+					mvalo[3]= y1;
+					areamouseco_to_ipoco(&sipo->v2d, mval, dvec, dvec+1);
+					areamouseco_to_ipoco(&sipo->v2d, mvalo+2, dvec+2, dvec+3);
+
+					sprintf(str, "Time: %.4f  Y %.4f", dvec[0]-dvec[2], dvec[1]-dvec[3]);
+					glRasterPos2i(30,  30);
+					glColor3f(0.0, 0.0, 0.0); 
+					BMF_DrawString(G.fonts, str);
+					glRasterPos2i(31,  31);
+					glColor3f(0.9, 0.9, 0.9); 
+					BMF_DrawString(G.fonts, str);
+				}
+
 				glFlush();
 
 				mvalo[0]= mval[0];
@@ -217,12 +287,12 @@ int get_border(rcti *rect, short col)
 			
 			event= extern_qread(&val);
 			
-			/* still because of the renderwindow... */
-			persp(PERSP_VIEW);
-			mywinset(curarea->win);
-			persp(PERSP_WIN);
+			if(event && val==0) {
+				/* still because of the renderwindow... */
+				persp(PERSP_VIEW);
+				mywinset(curarea->win);
+				persp(PERSP_WIN);
 				
-			if(val==0) {
 				if(event==ESCKEY) {
 					retval= 0;
 					break;
@@ -232,50 +302,6 @@ int get_border(rcti *rect, short col)
 				else if(event==RIGHTMOUSE) break;
 			}
 			
-			if(curarea->spacetype==SPACE_VIEW3D) {
-				glColor3f(0.4375, 0.4375, 0.4375); 
-				glRecti(0, 10, 250, 20);
-				glColor3f(0.0, 0.0, 0.0); 
-
-				if(G.vd->persp==0) {
-					window_to_3d(dvec, mvalo[0]-x1, mvalo[1]-y1);
-
-					glRasterPos2i(10,  10);
-					sprintf(str, "X %.4f  Y %.4f  Z %.4f  Dia %.4f", dvec[0], dvec[1], dvec[2], sqrt(dvec[0]*dvec[0]+dvec[1]*dvec[1]+dvec[2]*dvec[2]));
-					BMF_DrawString(G.fonts, str);
-				}
-				else if(G.vd->persp==2) {
-					rcti vb;
-
-					calc_viewborder(G.vd, &vb);
-
-					fac1= (mvalo[0]-x1)/( (float) (vb.xmax-vb.xmin) );
-					fac1*= 0.01*G.scene->r.size*G.scene->r.xsch;
-					
-					fac2= (mvalo[1]-y1)/( (float) (vb.ymax-vb.ymin) );
-					fac2*= 0.01*G.scene->r.size*G.scene->r.ysch;
-					
-					glRasterPos2i(10,  10);
-					sprintf(str, "X %.1f  Y %.1f  Dia %.1f", fabs(fac1), fabs(fac2), sqrt(fac1*fac1 + fac2*fac2) );
-					BMF_DrawString(G.fonts, str);
-				}
-			}
-			else if(curarea->spacetype==SPACE_IPO) {
-				SpaceIpo *sipo= curarea->spacedata.first;
-
-				glColor3f(.40625, .40625, .40625);
-				glRecti(20, 30, 170, 40);
-				glColor3f(0.0, 0.0, 0.0); 
-							
-				mvalo[2]= x1;
-				mvalo[3]= y1;
-				areamouseco_to_ipoco(&sipo->v2d, mval, dvec, dvec+1);
-				areamouseco_to_ipoco(&sipo->v2d, mvalo+2, dvec+2, dvec+3);
-
-				glRasterPos2i(30,  30);
-				sprintf(str, "Time: %.4f  Y %.4f", dvec[0]-dvec[2], dvec[1]-dvec[3]);
-				BMF_DrawString(G.fonts, str);
-			}
 		} /* end while (TRUE) */
 		sdrawXORline4(-1, 0, 0, 0, 0);
 		
@@ -311,11 +337,17 @@ int get_border(rcti *rect, short col)
 	
 	/* pressed B again ? -> brush select */
 	if(event==BKEY) {
+		setlinestyle(0);
 		switch (curarea->spacetype) {
 		case SPACE_VIEW3D:
-			if (G.obedit)
-				circle_selectCB(&obedit_selectionCB);
+			if (G.obedit) {
+				if ELEM4(G.obedit->type, OB_MESH, OB_CURVE, OB_SURF, OB_LATTICE) {
+					circle_selectCB(&obedit_selectionCB);
+					return 0;
+				}
+			}
 			return 0;
+			
 		case SPACE_IMAGE: // brush select in UV editor
 			circle_selectCB(&uvedit_selectionCB);
 			// this is a hack; we return 0 that the caller from get_border
@@ -330,28 +362,31 @@ void draw_sel_circle(short *mval, short *mvalo, float rad, float rado, int selec
 {
 	static short no_mvalo=0;
 
-	if(mval==0 && mvalo==0) {	/* signal */
+	if(mval==NULL && mvalo==NULL) {	/* signal */
 		no_mvalo= 1;
 		return;
 	}
 
 	persp(PERSP_WIN);
+	glReadBuffer(GL_FRONT);
 	glDrawBuffer(GL_FRONT);
+	//setlinestyle(2);
 
 	/* draw circle */
-
 	if(mvalo && no_mvalo==0) {
-		sdrawXORcirc(mvalo[0], mvalo[1], rado);
+		fdrawXORcirc(mvalo[0], mvalo[1], rado);
 	}
 	
 	if(mval) {
-		sdrawXORcirc(mval[0], mval[1], rad);
+		fdrawXORcirc(mval[0], mval[1], rad);
 	}
+	//setlinestyle(0);
 
 	glFlush();
 	persp(PERSP_VIEW);
 	glDrawBuffer(GL_BACK);
-	
+	glReadBuffer(GL_BACK);
+
 	no_mvalo= 0;
 }
 
@@ -365,6 +400,7 @@ void circle_selectCB(select_CBfunc callback)
 	static float rad= 40.0;
 	float rado;
 	int firsttime=1;
+	int escape= 0;
 	unsigned short event;
 	short mvalo[2], mval[2], val;
 	short selecting=0;
@@ -373,68 +409,79 @@ void circle_selectCB(select_CBfunc callback)
 	if(G.obedit) obj = G.obedit;
 	else obj = OBACT;
 
+	mywinset(curarea->win);
 	
 	getmouseco_areawin(mvalo);
-	draw_sel_circle(mvalo, 0, rad, 0.0, selecting);
+	mval[0]= mvalo[0]; mval[1]= mvalo[1];
+
+	draw_sel_circle(mval, NULL, rad, 0.0, selecting); // draws frontbuffer, but sets backbuf again
 	
 	rado= rad;
 	
 	while(TRUE) {
 		
-		/* for when another window is open and a mouse cursor activates it */
-
-		mywinset(curarea->win);
-
-		getmouseco_areawin(mval);
-		
 		if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1] || rado!=rad || firsttime) {
 			firsttime= 0;
 			
+			if(selecting) {
+				callback(selecting, obj, mval, rad);
+			}
+
 			draw_sel_circle(mval, mvalo, rad, rado, selecting);
 		
 			mvalo[0]= mval[0];
 			mvalo[1]= mval[1];
 			rado= rad;
 
-			if(selecting) {
-				callback(selecting, obj, mval, rad);
-			}
 		}
-		event= extern_qread(&val);
-		if (event) {
-			int afbreek= 0;
-			
-			switch(event) {
-			case LEFTMOUSE:
-			case MIDDLEMOUSE:
-				if(val) selecting= event;
-				else selecting= 0;
-				firsttime= 1;
-				
-				break;
-			case WHEELDOWNMOUSE:
-			case PADPLUSKEY:
-				if(val) if(rad<200.0) rad*= 1.2;
-				break;
-			case WHEELUPMOUSE:
-			case PADMINUS:
-				if(val) if(rad>5.0) rad/= 1.2;
-				break;
-			
-			case ESCKEY: case SPACEKEY: case RIGHTMOUSE:
-			case GKEY: case SKEY: case RKEY: case XKEY: case EKEY: case TABKEY:
-				afbreek= 1;
-				break;
+		
+		while(qtest()) {
+			event= extern_qread(&val);
+			if (event) {
 
-			}
+				/* for when another window is open and a mouse cursor activates it */
+				if(event!=MOUSEY && event!=MOUSEX) mywinset(curarea->win);
+				
+				getmouseco_areawin(mval);	// important to do here, trust events!
+				
+				switch(event) {
 			
-			if(afbreek) break;
+				case LEFTMOUSE:
+				case MIDDLEMOUSE:
+					if(val) selecting= event;
+					else selecting= 0;
+					firsttime= 1;
+					
+					break;
+				case WHEELDOWNMOUSE:
+				case PADPLUSKEY:
+				case EQUALKEY:
+					if(val) if(rad<200.0) rad*= 1.2;
+					break;
+				case WHEELUPMOUSE:
+				case PADMINUS:
+				case MINUSKEY:
+					if(val) if(rad>5.0) rad/= 1.2;
+					break;
+				
+				case ESCKEY: case SPACEKEY: case RIGHTMOUSE: case INPUTCHANGE: 
+				case GKEY: case SKEY: case RKEY: case XKEY: case EKEY: case TABKEY:
+					escape= 1;
+					break;
+
+				}
+				
+				if(escape) break;
+			}
 		}
+		PIL_sleep_ms(10);
+		
+		if(escape) break;
 	}
 	
 	/* clear circle */
-	draw_sel_circle(0, mvalo, 0, rad, 1);
-
+	draw_sel_circle(NULL, mvalo, 0, rad, 1);
+	BIF_undo_push("Circle Select");
 	countall();
 	allqueue(REDRAWINFO, 0);
 }
@@ -451,11 +498,11 @@ void count_object(Object *ob, int sel)
 		me= get_mesh(ob);
 		if(me) {
 			int totvert, totface;
-				/* hack, should be getting displistmesh from a central function */
-			if (mesh_uses_displist(me) && me->disp.first && ((DispList*)me->disp.first)->type==DL_MESH) {
-				DispListMesh *dlm= ((DispList*)me->disp.first)->mesh;
-				totvert= dlm->totvert;
-				totface= dlm->totface;
+			
+			/* note; do not make disp or get derived mesh here. spoils dependency order */
+			if (me->flag & ME_SUBSURF) {
+				totvert= me->totvert<<me->subdiv;
+				totface= me->totface<<me->subdiv;
 			} else {
 				totvert= me->totvert;
 				totface= me->totface;
@@ -502,11 +549,11 @@ void count_object(Object *ob, int sel)
 	
 }
 
+/* countall does statistics */
+/* is called on most actions, like select/add/delete/layermove */
 void countall()
 {
-/*  	extern Lattice *editLatt; in BKE_lattice.h*/
 	extern ListBase editNurb;
-	/* extern ListBase bpbase; */
 	Base *base;
 	Object *ob;
 	Mesh *me;
@@ -514,46 +561,33 @@ void countall()
 	BezTriple *bezt;
 	BPoint *bp;
 	MetaElem *ml;
-	/* struct BodyPoint *bop; */
-	struct EditVert *eve;
-	struct EditVlak *evl;
-#ifdef __NLA
 	struct EditBone *ebo;
-#endif
 	int a;
 
-	G.totvert= G.totvertsel= G.totfacesel= G.totface= G.totobj= 
+	G.totvert= G.totvertsel= G.totedge= G.totedgesel= G.totfacesel= G.totface= G.totobj= 
 	    G.totmesh= G.totlamp= G.totcurve= G.totobjsel=  0;
 
 	if(G.obedit) {
 		
 		if(G.obedit->type==OB_MESH) {
 			EditMesh *em = G.editMesh;
-			eve= em->verts.first;
-			while(eve) {
+			EditVert *eve;
+			EditEdge *eed;
+			EditFace *efa;
+			
+			for(eve= em->verts.first; eve; eve= eve->next) {
 				G.totvert++;
-				if(eve->f & 1) G.totvertsel++;
-				eve= eve->next;
+				if(eve->f & SELECT) G.totvertsel++;
 			}
-			evl= em->faces.first;
-			while(evl) {
+			for(eed= em->edges.first; eed; eed= eed->next) {
+				G.totedge++;
+				if(eed->f & SELECT) G.totedgesel++;
+			}
+			for(efa= em->faces.first; efa; efa= efa->next) {
 				G.totface++;
-				if(evl->v1->f & 1) {
-					if(evl->v2->f & 1) {
-						if(evl->v3->f & 1) {
-							if(evl->v4)  {
-								if(evl->v4->f & 1) G.totfacesel++;
-							}
-							else {
-								G.totfacesel++;
-							}
-						}
-					}
-				}
-				evl= evl->next;
+				if(efa->f & SELECT) G.totfacesel++;
 			}
 		}
-#ifdef __NLA
 		else if (G.obedit->type==OB_ARMATURE){
 			for (ebo=G.edbo.first;ebo;ebo=ebo->next){
 				
@@ -587,7 +621,6 @@ void countall()
 
 			}
 		}
-#endif
 		else if ELEM3(G.obedit->type, OB_CURVE, OB_SURF, OB_FONT) {
 			nu= editNurb.first;
 			while(nu) {
@@ -615,6 +648,9 @@ void countall()
 			}
 		}
 		else if(G.obedit->type==OB_MBALL) {
+			/* editmball.c */
+			extern ListBase editelems;  /* go away ! */
+			
 			ml= editelems.first;
 			while(ml) {
 				G.totvert++;
@@ -646,12 +682,11 @@ void countall()
 		return;
 	}
 
-	if(G.vd==0) return;
-	if(G.scene==0) return;
+	if(G.scene==NULL) return;
 
 	base= (G.scene->base.first);
 	while(base) {
-		if(G.vd->lay & base->lay) {
+		if(G.scene->lay & base->lay) {
 			
 			G.totobj++;
 			if(base->flag & SELECT) G.totobjsel++;
@@ -676,11 +711,432 @@ void countall()
 	allqueue(REDRAWINFO, 1); 	/* 1, because header->win==0! */
 }
 
+/* ************************************************** */
+/* ********************* old transform stuff ******** */
+/* ************************************************** */
+
+static TransVert *transvmain=NULL;
+static int tottrans= 0;
+
+/* selected things moved, and might need update in displists */
+static void update_select_dependency(void)
+{
+	Base *base;
+	Object *ob;
+	for(base= (G.scene->base.first); base; base= base->next) {
+		ob= base->object;
+		if(ob->hooks.first) {
+			ObHook *hook= ob->hooks.first;
+			while(hook) {
+				if(hook->parent && (hook->parent->flag & SELECT)) freedisplist(&ob->disp);
+				hook= hook->next;
+			}
+		}
+	}
+	
+}
+
+static void calc_trans_verts(void)
+{
+	extern ListBase editNurb;
+	
+	if (ELEM(G.obedit->type, OB_MESH, OB_MBALL))
+		makeDispList(G.obedit);
+	else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) {
+		Nurb *nu= editNurb.first;
+		while(nu) {
+			test2DNurb(nu);
+			testhandlesNurb(nu); /* test for bezier too */
+			nu= nu->next;
+		}
+		makeDispList(G.obedit);
+	}
+}
+
+static int pose_flags_reset_done(Object *ob) {
+	/* Clear the constraint done status for every pose channe;
+	* that has been flagged as needing constant updating
+	*/
+	bPoseChannel *chan;
+	int numreset = 0;
+	
+	if (ob->pose) {
+		for (chan = ob->pose->chanbase.first; chan; chan=chan->next){
+			if (chan->flag & PCHAN_TRANS_UPDATE) {
+				chan->flag &= ~PCHAN_DONE;
+				numreset++;
+			}
+			
+		}
+	}
+	return numreset;
+}
+
+
+/* copied from editobject.c, should be replaced with proper depgraph */
+static void special_trans_update(void)
+{
+	Base *base;
+	Curve *cu;
+	IpoCurve *icu;
+	
+	if(G.obedit) {
+		if(G.obedit->type==OB_MESH) {
+			recalc_editnormals();	// does face centers too
+		}
+		if(G.obedit->type==OB_CURVE) {
+			cu= G.obedit->data;
+			
+			makeBevelList(G.obedit); // might be needed for deform
+			calc_curvepath(G.obedit);
+			
+			base= FIRSTBASE;
+			while(base) {
+				if(base->lay & G.vd->lay) {
+					if(base->object->parent==G.obedit && base->object->partype==PARSKEL)
+						makeDispList(base->object);
+					else if(base->object->type==OB_CURVE) {
+						Curve *cu= base->object->data;
+						if(G.obedit==cu->bevobj || G.obedit==cu->taperobj)
+							makeDispList(base->object);
+					}
+				}
+				base= base->next;
+			}
+		}
+		else if(G.obedit->type==OB_ARMATURE){
+			EditBone *ebo;
+			
+			/* Ensure all bones are correctly adjusted */
+			for (ebo=G.edbo.first; ebo; ebo=ebo->next){
+				
+				if ((ebo->flag & BONE_IK_TOPARENT) && ebo->parent){
+					/* If this bone has a parent tip that has been moved */
+					if (ebo->parent->flag & BONE_TIPSEL){
+						VECCOPY (ebo->head, ebo->parent->tail);
+					}
+					/* If this bone has a parent tip that has NOT been moved */
+					else{
+						VECCOPY (ebo->parent->tail, ebo->head);
+					}
+				}
+			}
+		}
+		else if(G.obedit->type==OB_LATTICE) {
+			
+			if(editLatt->flag & LT_OUTSIDE) outside_lattice(editLatt);
+			
+			base= FIRSTBASE;
+			while(base) {
+				if(base->lay & G.vd->lay) {
+					if(base->object->parent==G.obedit) {
+						makeDispList(base->object);
+					}
+				}
+				base= base->next;
+			}
+		}
+	}
+	else {
+		base= FIRSTBASE;
+		while(base) {
+			if(base->flag & BA_DO_IPO) {
+				
+				base->object->ctime= -1234567.0;
+				
+				icu= base->object->ipo->curve.first;
+				while(icu) {
+					calchandles_ipocurve(icu);
+					icu= icu->next;
+				}
+				
+			}
+			if(base->object->partype & PARSLOW) {
+				base->object->partype -= PARSLOW;
+				where_is_object(base->object);
+				base->object->partype |= PARSLOW;
+			}
+			else if(base->flag & BA_WHERE_UPDATE) {
+				where_is_object(base->object);
+			}
+			
+			base= base->next;
+		} 
+		
+		base= FIRSTBASE;
+		while(base) {
+			
+			if(base->flag & BA_DISP_UPDATE) makeDispList(base->object);
+			
+			base= base->next;
+		}
+		
+	}
+	
+	base= FIRSTBASE;
+	while(base) {
+		if (pose_flags_reset_done(base->object)) {
+			if (!is_delay_deform())
+				make_displists_by_armature(base->object);
+		}
+		
+		base= base->next;
+	}
+	
+#if 1
+	if (G.obpose && G.obpose->type == OB_ARMATURE)
+		clear_pose_constraint_status(G.obpose);
+	
+	if (!is_delay_deform()) make_displists_by_armature(G.obpose);
+#endif
+	
+	if(G.vd->drawtype == OB_SHADED) reshadeall_displist();
+	
+}
+
+/* copied from editobject.c, needs to be replaced with new transform code still */
+/* mode: 1 = proportional */
+static void make_trans_verts(float *min, float *max, int mode)	
+{
+	extern ListBase editNurb;
+	EditMesh *em = G.editMesh;
+	Nurb *nu;
+	BezTriple *bezt;
+	BPoint *bp;
+	TransVert *tv=NULL;
+	MetaElem *ml;
+	EditVert *eve;
+	EditBone	*ebo;
+	float total, centre[3], centroid[3];
+	int a;
+
+	tottrans= 0; // global!
+	
+	INIT_MINMAX(min, max);
+	centroid[0]=centroid[1]=centroid[2]= 0.0;
+	
+	/* note for transform refactor: dont rely on countall anymore... its ancient */
+	/* I skip it for editmesh now (ton) */
+	if(G.obedit->type!=OB_MESH) {
+		countall();
+		if(mode) tottrans= G.totvert;
+		else tottrans= G.totvertsel;
+
+		if(G.totvertsel==0) {
+			tottrans= 0;
+			return;
+		}
+		tv=transvmain= MEM_callocN(tottrans*sizeof(TransVert), "maketransverts");
+	}
+	
+	/* we count again because of hide (old, not for mesh!) */
+	tottrans= 0;
+	
+	if(G.obedit->type==OB_MESH) {
+		int proptrans= 0;
+		
+		// transform now requires awareness for select mode, so we tag the f1 flags in verts
+		tottrans= 0;
+		if(G.scene->selectmode & SCE_SELECT_VERTEX) {
+			for(eve= em->verts.first; eve; eve= eve->next) {
+				if(eve->h==0 && (eve->f & SELECT)) {
+					eve->f1= SELECT;
+					tottrans++;
+				}
+				else eve->f1= 0;
+			}
+		}
+		else if(G.scene->selectmode & SCE_SELECT_EDGE) {
+			EditEdge *eed;
+			for(eve= em->verts.first; eve; eve= eve->next) eve->f1= 0;
+			for(eed= em->edges.first; eed; eed= eed->next) {
+				if(eed->h==0 && (eed->f & SELECT)) eed->v1->f1= eed->v2->f1= SELECT;
+			}
+			for(eve= em->verts.first; eve; eve= eve->next) if(eve->f1) tottrans++;
+		}
+		else {
+			EditFace *efa;
+			for(eve= em->verts.first; eve; eve= eve->next) eve->f1= 0;
+			for(efa= em->faces.first; efa; efa= efa->next) {
+				if(efa->h==0 && (efa->f & SELECT)) {
+					efa->v1->f1= efa->v2->f1= efa->v3->f1= SELECT;
+					if(efa->v4) efa->v4->f1= SELECT;
+				}
+			}
+			for(eve= em->verts.first; eve; eve= eve->next) if(eve->f1) tottrans++;
+		}
+		
+		/* proportional edit exception... */
+		if(mode==1 && tottrans) {
+			for(eve= em->verts.first; eve; eve= eve->next) {
+				if(eve->h==0) {
+					eve->f1 |= 2;
+					proptrans++;
+				}
+			}
+			if(proptrans>tottrans) tottrans= proptrans;
+		}
+		
+		/* and now make transverts */
+		if(tottrans) {
+			tv=transvmain= MEM_callocN(tottrans*sizeof(TransVert), "maketransverts");
+
+			for(eve= em->verts.first; eve; eve= eve->next) {
+				if(eve->f1) {
+					VECCOPY(tv->oldloc, eve->co);
+					tv->loc= eve->co;
+					if(eve->no[0]!=0.0 || eve->no[1]!=0.0 ||eve->no[2]!=0.0)
+						tv->nor= eve->no; // note this is a hackish signal (ton)
+					tv->flag= eve->f1 & SELECT;
+					tv++;
+				}
+			}
+		}
+	}
+	else if (G.obedit->type==OB_ARMATURE){
+		for (ebo=G.edbo.first;ebo;ebo=ebo->next){
+			if (ebo->flag & BONE_TIPSEL){
+				VECCOPY (tv->oldloc, ebo->tail);
+				tv->loc= ebo->tail;
+				tv->nor= NULL;
+				tv->flag= 1;
+				tv++;
+				tottrans++;
+			}
+
+			/*  Only add the root if there is no selected IK parent */
+			if (ebo->flag & BONE_ROOTSEL){
+				if (!(ebo->parent && (ebo->flag & BONE_IK_TOPARENT) && ebo->parent->flag & BONE_TIPSEL)){
+					VECCOPY (tv->oldloc, ebo->head);
+					tv->loc= ebo->head;
+					tv->nor= NULL;
+					tv->flag= 1;
+					tv++;
+					tottrans++;
+				}		
+			}
+			
+		}
+	}
+	else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) {
+		nu= editNurb.first;
+		while(nu) {
+			if((nu->type & 7)==CU_BEZIER) {
+				a= nu->pntsu;
+				bezt= nu->bezt;
+				while(a--) {
+					if(bezt->hide==0) {
+						if(mode==1 || (bezt->f1 & 1)) {
+							VECCOPY(tv->oldloc, bezt->vec[0]);
+							tv->loc= bezt->vec[0];
+							tv->flag= bezt->f1 & 1;
+							tv++;
+							tottrans++;
+						}
+						if(mode==1 || (bezt->f2 & 1)) {
+							VECCOPY(tv->oldloc, bezt->vec[1]);
+							tv->loc= bezt->vec[1];
+							tv->val= &(bezt->alfa);
+							tv->oldval= bezt->alfa;
+							tv->flag= bezt->f2 & 1;
+							tv++;
+							tottrans++;
+						}
+						if(mode==1 || (bezt->f3 & 1)) {
+							VECCOPY(tv->oldloc, bezt->vec[2]);
+							tv->loc= bezt->vec[2];
+							tv->flag= bezt->f3 & 1;
+							tv++;
+							tottrans++;
+						}
+					}
+					bezt++;
+				}
+			}
+			else {
+				a= nu->pntsu*nu->pntsv;
+				bp= nu->bp;
+				while(a--) {
+					if(bp->hide==0) {
+						if(mode==1 || (bp->f1 & 1)) {
+							VECCOPY(tv->oldloc, bp->vec);
+							tv->loc= bp->vec;
+							tv->val= &(bp->alfa);
+							tv->oldval= bp->alfa;
+							tv->flag= bp->f1 & 1;
+							tv++;
+							tottrans++;
+						}
+					}
+					bp++;
+				}
+			}
+			nu= nu->next;
+		}
+	}
+	else if(G.obedit->type==OB_MBALL) {
+		extern ListBase editelems;  /* go away ! */
+		ml= editelems.first;
+		while(ml) {
+			if(ml->flag & SELECT) {
+				tv->loc= &ml->x;
+				VECCOPY(tv->oldloc, tv->loc);
+				tv->val= &(ml->rad);
+				tv->oldval= ml->rad;
+				tv->flag= 1;
+				tv++;
+				tottrans++;
+			}
+			ml= ml->next;
+		}
+	}
+	else if(G.obedit->type==OB_LATTICE) {
+		bp= editLatt->def;
+		
+		a= editLatt->pntsu*editLatt->pntsv*editLatt->pntsw;
+		
+		while(a--) {
+			if(mode==1 || (bp->f1 & 1)) {
+				if(bp->hide==0) {
+					VECCOPY(tv->oldloc, bp->vec);
+					tv->loc= bp->vec;
+					tv->flag= bp->f1 & 1;
+					tv++;
+					tottrans++;
+				}
+			}
+			bp++;
+		}
+	}
+	
+	/* cent etc */
+	tv= transvmain;
+	total= 0.0;
+	for(a=0; a<tottrans; a++, tv++) {
+		if(tv->flag & SELECT) {
+			centroid[0]+= tv->oldloc[0];
+			centroid[1]+= tv->oldloc[1];
+			centroid[2]+= tv->oldloc[2];
+			total+= 1.0;
+			DO_MINMAX(tv->oldloc, min, max);
+		}
+	}
+	if(total!=0.0) {
+		centroid[0]/= total;
+		centroid[1]/= total;
+		centroid[2]/= total;
+	}
+
+	centre[0]= (min[0]+max[0])/2.0;
+	centre[1]= (min[1]+max[1])/2.0;
+	centre[2]= (min[2]+max[2])/2.0;
+	
+}
+
+
 
 void snap_sel_to_grid()
 {
-	extern TransVert *transvmain;
-	extern int tottrans;
 	extern float originmat[3][3];	/* object.c */
 	TransVert *tv;
 	Base *base;
@@ -688,14 +1144,16 @@ void snap_sel_to_grid()
 	float gridf, imat[3][3], bmat[3][3], vec[3];
 	int a;
 
-	gridf= G.vd->grid;
+	gridf= G.vd->gridview;
 
 
 		if(G.obedit) {
+			tottrans= 0;
+			
 #ifdef __NLA
-			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+			if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
 #else
-			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+			if ELEM5(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
 #endif
 			if(tottrans==0) return;
 
@@ -705,26 +1163,25 @@ void snap_sel_to_grid()
 			tv= transvmain;
 			for(a=0; a<tottrans; a++, tv++) {
 
-			VECCOPY(vec, tv->loc);
-			Mat3MulVecfl(bmat, vec);
-			VecAddf(vec, vec, G.obedit->obmat[3]);
-			vec[0]= G.vd->grid*floor(.5+ vec[0]/gridf);
-			vec[1]= G.vd->grid*floor(.5+ vec[1]/gridf);
-			vec[2]= G.vd->grid*floor(.5+ vec[2]/gridf);
-			VecSubf(vec, vec, G.obedit->obmat[3]);
+				VECCOPY(vec, tv->loc);
+				Mat3MulVecfl(bmat, vec);
+				VecAddf(vec, vec, G.obedit->obmat[3]);
+				vec[0]= G.vd->gridview*floor(.5+ vec[0]/gridf);
+				vec[1]= G.vd->gridview*floor(.5+ vec[1]/gridf);
+				vec[2]= G.vd->gridview*floor(.5+ vec[2]/gridf);
+				VecSubf(vec, vec, G.obedit->obmat[3]);
 
-			Mat3MulVecfl(imat, vec);
-			VECCOPY(tv->loc, vec);
+				Mat3MulVecfl(imat, vec);
+				VECCOPY(tv->loc, vec);
 
 			}
 
 			MEM_freeN(transvmain);
 			transvmain= 0;
+			
+			calc_trans_verts(); // does test2d, makedisplist too */
 
-			if ELEM(G.obedit->type, OB_SURF, OB_CURVE) makeDispList(G.obedit);
-
-			if (G.obedit->type == OB_ARMATURE)
-				special_trans_update(0);
+			special_trans_update();
 
 			allqueue(REDRAWVIEW3D, 0);
 			return;
@@ -740,9 +1197,9 @@ void snap_sel_to_grid()
 			if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
 				ob= base->object;
 
-				vec[0]= -ob->obmat[3][0]+G.vd->grid*floor(.5+ ob->obmat[3][0]/gridf);
-				vec[1]= -ob->obmat[3][1]+G.vd->grid*floor(.5+ ob->obmat[3][1]/gridf);
-				vec[2]= -ob->obmat[3][2]+G.vd->grid*floor(.5+ ob->obmat[3][2]/gridf);
+				vec[0]= -ob->obmat[3][0]+G.vd->gridview*floor(.5+ ob->obmat[3][0]/gridf);
+				vec[1]= -ob->obmat[3][1]+G.vd->gridview*floor(.5+ ob->obmat[3][1]/gridf);
+				vec[2]= -ob->obmat[3][2]+G.vd->gridview*floor(.5+ ob->obmat[3][2]/gridf);
 
 				if(ob->parent) {
 					where_is_object(ob);
@@ -762,13 +1219,12 @@ void snap_sel_to_grid()
 
 			base= base->next;
 		}
+		update_select_dependency();
 		allqueue(REDRAWVIEW3D, 0);
 }
 
 void snap_sel_to_curs()
 {
-	extern TransVert *transvmain;
-	extern int tottrans;
 	extern float originmat[3][3];	/* object.c */
 	TransVert *tv;
 	Base *base;
@@ -779,10 +1235,11 @@ void snap_sel_to_curs()
 	curs= give_cursor();
 
 		if(G.obedit) {
+			tottrans= 0;
 #ifdef __NLA
-			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+			if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
 #else
-			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+			if ELEM5(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
 #endif
 			if(tottrans==0) return;
 
@@ -805,10 +1262,9 @@ void snap_sel_to_curs()
 			MEM_freeN(transvmain);
 			transvmain= 0;
 
-			if ELEM(G.obedit->type, OB_SURF, OB_CURVE) makeDispList(G.obedit);
+			calc_trans_verts(); // does test2d, makedisplist too */
 
-			if (G.obedit->type == OB_ARMATURE)
-				special_trans_update(0);
+			special_trans_update();
 
 			allqueue(REDRAWVIEW3D, 0);
 			return;
@@ -847,6 +1303,7 @@ void snap_sel_to_curs()
 
 			base= base->next;
 		}
+		update_select_dependency();
 		allqueue(REDRAWVIEW3D, 0);
 }
 
@@ -854,12 +1311,12 @@ void snap_curs_to_grid()
 {
 	float gridf, *curs;
 
-	gridf= G.vd->grid;
+	gridf= G.vd->gridview;
 	curs= give_cursor();
 
-	curs[0]= G.vd->grid*floor(.5+curs[0]/gridf);
-	curs[1]= G.vd->grid*floor(.5+curs[1]/gridf);
-	curs[2]= G.vd->grid*floor(.5+curs[2]/gridf);
+	curs[0]= G.vd->gridview*floor(.5+curs[0]/gridf);
+	curs[1]= G.vd->gridview*floor(.5+curs[1]/gridf);
+	curs[2]= G.vd->gridview*floor(.5+curs[2]/gridf);
 
 	allqueue(REDRAWVIEW3D, 0);
 
@@ -867,8 +1324,6 @@ void snap_curs_to_grid()
 
 void snap_curs_to_sel()
 {
-	extern TransVert *transvmain;
-	extern int tottrans;
 	TransVert *tv;
 	Base *base;
 	float *curs, bmat[3][3], vec[3], min[3], max[3], centroid[3];
@@ -883,9 +1338,9 @@ void snap_curs_to_sel()
 		if(G.obedit) {
 			tottrans=0;
 #ifdef __NLA
-			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+			if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
 #else
-			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+			if ELEM5(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
 #endif
 			if(tottrans==0) return;
 
@@ -940,8 +1395,6 @@ void snap_curs_to_sel()
 
 void snap_curs_to_firstsel()
 {
-	extern TransVert *transvmain;
-	extern int tottrans;
 	TransVert *tv;
 	Base *base;
 	float *curs, bmat[3][3], vec[3], min[3], max[3], centroid[3];
@@ -956,9 +1409,9 @@ void snap_curs_to_firstsel()
 		if(G.obedit) {
 			tottrans=0;
 #ifdef __NLA
-			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+			if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
 #else
-			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+			if ELEM5(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
 #endif
 			if(tottrans==0) return;
 
@@ -1011,8 +1464,6 @@ void snap_curs_to_firstsel()
 
 void snap_to_center()
 {
-	extern TransVert *transvmain;
-	extern int tottrans;
 	extern float originmat[3][3];
 	TransVert *tv;
 	Base *base;
@@ -1027,11 +1478,11 @@ void snap_to_center()
 	centroid[0]= centroid[1]= centroid[2]= 0.0;
 
 		if(G.obedit) {
-			/*tottrans=0;*/
+			tottrans= 0;
 #ifdef __NLA
-			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+			if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
 #else
-			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+			if ELEM5(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
 #endif
 			if(tottrans==0) return;
 
@@ -1089,10 +1540,11 @@ void snap_to_center()
 /* Snap the selection to the snaplocation (duh!) */
 
 		if(G.obedit) {
+			tottrans= 0;
 #ifdef __NLA
-			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+			if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
 #else
-			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+			if ELEM5(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
 #endif
 			if(tottrans==0) return;
 
@@ -1115,10 +1567,9 @@ void snap_to_center()
 			MEM_freeN(transvmain);
 			transvmain= 0;
 
-			if ELEM(G.obedit->type, OB_SURF, OB_CURVE) makeDispList(G.obedit);
+			if ELEM4(G.obedit->type, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) makeDispList(G.obedit);
 
-			if (G.obedit->type == OB_ARMATURE)
-				special_trans_update(0);
+			special_trans_update();
 
 			allqueue(REDRAWVIEW3D, 0);
 			return;
@@ -1157,7 +1608,7 @@ void snap_to_center()
 
 			base= base->next;
 		}
-
+		update_select_dependency();
 		allqueue(REDRAWVIEW3D, 0);
 }
 
@@ -1171,9 +1622,11 @@ void snapmenu()
 	switch (event) {
 		case 1: /*Selection to grid*/
 		    snap_sel_to_grid();
+			BIF_undo_push("Snap selection to grid");
 		    break;
 		case 2: /*Selection to cursor*/
 		    snap_sel_to_curs();
+			BIF_undo_push("Snap selection to cursor");
 		    break;	    
 		case 3: /*Cursor to grid*/
 		    snap_curs_to_grid();
@@ -1183,6 +1636,7 @@ void snapmenu()
 		    break;
 		case 5: /*Selection to center of selection*/
 		    snap_to_center();
+			BIF_undo_push("Snap selection to center");
 		    break;
 	}
 }
@@ -1193,20 +1647,20 @@ void mergemenu(void)
 	extern float doublimit;
 	short event;
 
-	event = pupmenu("MERGE %t|At Center%x1|At Cursor%x2");
+	event = pupmenu("Merge %t|At Center%x1|At Cursor%x2");
 
 	if (event==-1) return; /* Return if the menu is closed without any choices */
 
-	undo_push_mesh("Merge"); /* The action has been confirmed, push the mesh down the undo pipe */
-
 	if (event==1) 
 		snap_to_center(); /*Merge at Center*/
 	else
 		snap_sel_to_curs(); /*Merge at Cursor*/
 
-	notice("Removed: %d", removedoublesflag(1, doublimit));
+	notice("Removed %d Vertices", removedoublesflag(1, doublimit));
 	allqueue(REDRAWVIEW3D, 0);
 	countall();
+	BIF_undo_push("Merge"); /* push the mesh down the undo pipe */
+
 }
 
 void delete_context_selected(void) {
@@ -1226,20 +1680,22 @@ void duplicate_context_selected(void) {
 		else if(G.obedit->type==OB_MBALL) adduplicate_mball();
 		else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) adduplicate_nurb();
 	}
-	else if(G.obpose){
-		error ("Duplicate not possible in posemode.");
+	else if(!(G.obpose)){
+		adduplicate(0);
 	}
-	else adduplicate(0);
 }
 
-void toggle_shading(void) {
-	if(G.qual & LR_CTRLKEY) {
-		reshadeall_displist();
-		G.vd->drawtype= OB_SHADED;
-	}
-	else if(G.qual & LR_SHIFTKEY) {
-		if(G.vd->drawtype== OB_SHADED) G.vd->drawtype= OB_WIRE;
-		else G.vd->drawtype= OB_SHADED;
+void toggle_shading(void) 
+{
+	if(G.qual & LR_SHIFTKEY) {
+		if(G.qual & LR_ALTKEY) {
+			reshadeall_displist();
+			G.vd->drawtype= OB_SHADED;
+		}
+		else {
+			if(G.vd->drawtype== OB_SHADED) G.vd->drawtype= OB_WIRE;
+			else G.vd->drawtype= OB_SHADED;
+		}
 	}
 	else if(G.qual & LR_ALTKEY) {
 		if(G.vd->drawtype== OB_TEXTURE) G.vd->drawtype= OB_SOLID;
@@ -1253,8 +1709,6 @@ void toggle_shading(void) {
 
 void minmax_verts(float *min, float *max)
 {
-	extern TransVert *transvmain;
-	extern int tottrans;
 	TransVert *tv;
 	float centroid[3], vec[3], bmat[3][3];
 	int a;
@@ -1280,4 +1734,5 @@ void minmax_verts(float *min, float *max)
 	
 	MEM_freeN(transvmain);
 	transvmain= 0;
-}
\ No newline at end of file
+}
+
