@@ -1,5 +1,5 @@
 /**
- * $Id: editarmature.c,v 1.19 2003/11/21 04:07:45 hos Exp $
+ * $Id: editarmature.c,v 1.50 2005/05/10 04:21:11 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -39,17 +39,15 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include "MEM_guardedalloc.h"
 
 #include "BMF_Api.h"
 
 #include "DNA_action_types.h"
 #include "DNA_armature_types.h"
+#include "DNA_constraint_types.h"
 #include "DNA_mesh_types.h"
+#include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
@@ -67,6 +65,7 @@
 #include "BKE_global.h"
 #include "BKE_object.h"
 #include "BKE_subsurf.h"
+#include "BKE_deform.h"
 
 #include "BIF_gl.h"
 #include "BIF_graphics.h"
@@ -76,9 +75,11 @@
 #include "BIF_space.h"
 #include "BIF_toolbox.h"
 #include "BIF_editarmature.h"
+#include "BIF_editconstraint.h"
 #include "BIF_poseobject.h"
 #include "BIF_mywindow.h"
 #include "BIF_editdeform.h"
+#include "BIF_transform.h"
 
 #include "BDR_editobject.h"
 #include "BDR_drawobject.h"
@@ -88,6 +89,8 @@
 #include "BSE_trans_types.h"
 #include "BSE_editaction.h"
 
+#include "PIL_time.h"
+
 #include "mydevice.h"
 #include "blendef.h"
 #include "nla.h"
@@ -100,8 +103,6 @@
 #define B_CYAN		0xFFFF00
 #define	B_AQUA		0xFFBB55 /* 0xFF8833*/
 
-extern	int tottrans;					/* Originally defined in editobject.c */
-extern	struct TransOb *transmain;				/* Originally defined in editobject.c */
 extern	float centre[3], centroid[3];	/* Originally defined in editobject.c */
 
 /*	Macros	*/
@@ -123,10 +124,9 @@ static void delete_bone(struct EditBone* exBone);
 static void clear_armature_children (struct Bone *bone, struct bPose *pose, char mode);
 
 static int	count_bones (struct bArmature *arm, int flagmask, int allbones);
-
 static int	count_bonechildren (struct Bone *bone, int incount, int flagmask, int allbones);
-static int	add_trans_bonechildren (struct Object* ob, struct Bone* bone, struct TransOb* buffer, int index, char mode);
-static void deselect_bonechildren (struct Bone *bone, int mode);
+
+static void deselect_bonechildren (Object *ob, struct Bone *bone, int mode);
 static void selectconnected_posebonechildren (struct Bone *bone);
 
 static int	editbone_name_exists (char* name);
@@ -136,6 +136,15 @@ static EditBone * get_nearest_editbonepoint (int findunsel, int *selmask);
 
 static Bone *get_first_selected_bonechildren (Bone *bone);
 
+static EditBone *get_named_editbone(char *name);
+static void update_dup_subtarget(EditBone *dupBone);
+static int bone_skinnable(Object *ob, Bone *bone, void *data);
+static int add_defgroup_unique_bone(Object *ob, Bone *bone, void *data);
+static int dgroup_skinnable(Object *ob, Bone *bone, void *data);
+static void add_verts_to_closest_dgroup(Object *ob, Object *par);
+static int hide_selected_pose_bone(Object *ob, Bone *bone, void *ptr);
+static int hide_unselected_pose_bone(Object *ob, Bone *bone, void *ptr);
+static int show_pose_bone(Object *ob, Bone *bone, void *ptr);
 
 /* Functions */
 
@@ -420,6 +429,9 @@ void selectconnected_armature(void)
 
 	countall();
 	allqueue (REDRAWVIEW3D, 0);
+	allqueue (REDRAWBUTSEDIT, 0);
+	allqueue(REDRAWBUTSOBJECT, 0);
+	allqueue(REDRAWOOPS, 0);
 
 }
 
@@ -456,7 +468,10 @@ void selectconnected_posearmature(void)
 	
 	countall();
 	allqueue (REDRAWVIEW3D, 0);
+	allqueue(REDRAWBUTSEDIT, 0);
+	allqueue(REDRAWBUTSOBJECT, 0);
 	allqueue (REDRAWACTION, 0);
+	allqueue(REDRAWOOPS, 0);
 }
 
 static void selectconnected_posebonechildren (Bone *bone)
@@ -680,7 +695,7 @@ void delete_armature(void)
 	EditBone	*curBone, *next;
 	
 	TEST_EDITARMATURE;
-	if(okee("Erase selected")==0) return;
+	if(okee("Erase selected bone")==0) return;
 	
 	for (curBone=G.edbo.first;curBone;curBone=next){
 		next=curBone->next;
@@ -692,6 +707,7 @@ void delete_armature(void)
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
 	allqueue(REDRAWBUTSOBJECT, 0);
+	allqueue(REDRAWOOPS, 0);
 	countall();
 }
 
@@ -723,19 +739,22 @@ static void delete_bone(EditBone* exBone)
 
 	allqueue(REDRAWBUTSOBJECT, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
+	allqueue(REDRAWBUTSOBJECT, 0);
+	allqueue(REDRAWOOPS, 0);
 
 	BLI_freelinkN (&G.edbo,exBone);
 }
 
 void remake_editArmature(void)
 {
-	if(okee("Reload Original data")==0) return;
+	if(okee("Reload original data")==0) return;
 	
 	make_editArmature();
 	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWOOPS, 0);
 	allqueue(REDRAWBUTSHEAD, 0);
-	allqueue(REDRAWBUTSOBJECT, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
+	allqueue(REDRAWBUTSOBJECT, 0);
 }
 
 void mouse_armature(void)
@@ -758,6 +777,7 @@ void mouse_armature(void)
 		allqueue(REDRAWVIEW3D, 0);
 		allqueue(REDRAWBUTSEDIT, 0);
 		allqueue(REDRAWBUTSOBJECT, 0);
+		allqueue(REDRAWOOPS, 0);
 	};
 	countall();
 	rightmouse_transform();
@@ -1112,12 +1132,13 @@ static void make_boneList(ListBase* list, ListBase *bones, EditBone *parent)
 #if 0
 static EditVert*	 add_armatureVert (float *loc)
 {
+	EditMesh *em = G.editMesh;
 	EditVert*	vert=NULL;
 
 	vert = MEM_callocN (sizeof (EditVert), "armaturevert");
 	if (vert){
 		VECCOPY (vert->co, loc);
-		BLI_addtail (&G.edve,vert);
+		BLI_addtail (&em->verts,vert);
 	}
 
 	return vert;
@@ -1126,9 +1147,10 @@ static EditVert*	 add_armatureVert (float *loc)
 
 static EditVert* get_armatureVert (float *loc)
 {
+	EditMesh *em = G.editMesh;
 	EditVert*	vert;
 
-	for (vert=G.edve.first;vert;vert=vert->next){
+	for (vert=em->verts.first;vert;vert=vert->next){
 		if ((vert->co[0]==loc[0])&&(vert->co[1]==loc[1])&&(vert->co[2]==loc[2])){
 			return (vert);
 		}
@@ -1483,10 +1505,13 @@ static void draw_bone (int armflag, int boneflag, unsigned int id, char *name, f
 	glPopMatrix();
 
 	/*	Draw the bone name */
-	if (armflag & ARM_DRAWNAMES){
-		glRasterPos3f(0,  length/2.0,  0);
-		BMF_DrawString(G.font, " ");
-		BMF_DrawString(G.font, name);
+	if (armflag & ARM_DRAWNAMES) {
+		// patch for several 3d cards (IBM mostly) that crash on glSelect with text drawing
+		if((G.f & G_PICKSEL) == 0) {
+			glRasterPos3f(0,  length/2.0,  0);
+			BMF_DrawString(G.font, " ");
+			BMF_DrawString(G.font, name);
+		}
 	}
 }
 
@@ -1542,7 +1567,7 @@ static void add_bone_input (Object *ob)
 {
 	float		*cursLoc, cent[3], dx, dy;
 	float		mat[3][3], curs[3],	cmat[3][3], imat[3][3], rmat[4][4], itmat[4][4];
-	short		xo, yo, mval[2], afbreek=0;
+	short		xo, yo, mval[2], mvalo[2], afbreek=0, drawall;
 	short		val;
 	float		restmat[4][4], tempVec[4];
 	EditBone	*bone;
@@ -1557,6 +1582,7 @@ static void add_bone_input (Object *ob)
 	cursLoc= give_cursor();
 	
 	VECCOPY (curs,cursLoc);
+	G.moving= G_TRANSFORM_EDIT;
 	
 	while (addbones>0){
 		afbreek=0;
@@ -1599,9 +1625,10 @@ static void add_bone_input (Object *ob)
 
 		/*	Project cursor center to screenspace. */
 		getmouseco_areawin(mval);
-		xo= mval[0];
-		yo= mval[1];
+		xo= mvalo[0]= mval[0];
+		yo= mvalo[1]= mval[1];
 		window_to_3d(dvecp, xo, yo);
+		drawall= 2;
 
 		while (1) {
 
@@ -1668,7 +1695,18 @@ static void add_bone_input (Object *ob)
 			Mat4One (bone->obmat);
 			bone->size[0]=bone->size[1]=bone->size[2]=1.0F;
 
-			force_draw();
+			/* only draw if... */
+			if(drawall) {
+				drawall--;	// draw twice to have 2 identical buffers
+				force_draw_all(1);
+			}
+			else if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1]) {
+				mvalo[0]= mval[0];
+				mvalo[1]= mval[1];
+				force_draw(1);
+			}
+			else PIL_sleep_ms(10);
+			
 			while(qtest()) {
 				event= extern_qread(&val);
 				if(val) {
@@ -1702,9 +1740,38 @@ static void add_bone_input (Object *ob)
 	}	/*	End of bone adding loop*/
 	
 	countall();
+	G.moving= 0;
 
 }
 
+void attach_bone_to_parent_cb(void *bonev, void *arg2_unused)
+{
+	EditBone *curBone= bonev;
+	attach_bone_to_parent(curBone);
+}
+
+void attach_bone_to_parent(EditBone *bone)
+{
+	EditBone *curbone;
+
+	if (bone->flag & BONE_IK_TOPARENT) {
+
+		/* See if there are any other bones that refer to the same 
+		 * parent and disconnect them 
+		 */
+		for (curbone = G.edbo.first; curbone; curbone=curbone->next){
+			if (curbone!=bone){
+				if (curbone->parent && 
+					(curbone->parent == bone->parent) && 
+					(curbone->flag & BONE_IK_TOPARENT))
+					curbone->flag &= ~BONE_IK_TOPARENT;
+			}
+		}
+
+        /* Attach this bone to its parent */
+		VECCOPY(bone->head, bone->parent->tail);
+	}
+}
 
 void deselectall_armature(void)
 /*	Actually, it toggles selection, deselecting
@@ -1732,8 +1799,8 @@ void deselectall_armature(void)
 	};
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
-	allqueue(REDRAWBUTSHEAD, 0);
 	allqueue(REDRAWBUTSOBJECT, 0);
+	allqueue(REDRAWOOPS, 0);
 	countall();
 }
 
@@ -1753,7 +1820,7 @@ void join_armature(void)
 	if(ob->type!=OB_ARMATURE) return;
 	
 	/*	Make sure the user wants to continue*/
-	if(okee("Join selected Armatures")==0) return;
+	if(okee("Join selected armatures")==0) return;
 	
 	/*	Put the active armature into editmode and join the bones from the other one*/
 #if 1
@@ -1839,6 +1906,7 @@ void join_armature(void)
 	}
 #endif
 	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWOOPS, 0);
 
 }
 
@@ -1884,7 +1952,7 @@ void extrude_armature(void)
 	TEST_EDITARMATURE;
 	
 	
-	if(okee("Extrude Bone Segments")==0) return;
+	if(okee("Extrude bone segments")==0) return;
 	
 	/* Duplicate the necessary bones */
 	for (curbone = G.edbo.first; ((curbone) && (curbone!=first)); curbone=curbone->next){
@@ -1928,9 +1996,12 @@ void extrude_armature(void)
 	
 	/* Transform the endpoints */
 	countall();
-	transform('g');		
+	BIF_TransformSetUndo("Extrude");
+	initTransform(TFM_TRANSLATION, CTX_NO_PET);
+	Transform();
 	allqueue(REDRAWBUTSEDIT, 0);
 	allqueue(REDRAWBUTSOBJECT, 0);
+	allqueue(REDRAWOOPS, 0);
 }
 
 void addvert_armature(void)
@@ -1942,8 +2013,52 @@ void addvert_armature(void)
 }
 
 
+static EditBone *get_named_editbone(char *name)
+{
+	EditBone  *eBone;
+
+	if (name)
+		for (eBone=G.edbo.first; eBone; eBone=eBone->next){
+			if (!strcmp (name, eBone->name))
+				return eBone;
+		}
+
+	return NULL;
+}
+
+static void update_dup_subtarget(EditBone *dupBone)
+{
+	/* If an edit bone has been duplicated, lets
+	 * update it's constraints if the subtarget
+	 * they point to has also been duplicated
+	 */
+	EditBone     *oldtarget, *newtarget;
+	bPoseChannel *chan;
+	bConstraint  *curcon;
+	ListBase     *conlist;
+	char         *subname;
 
 
+	if ( (chan = verify_pose_channel(OBACT->pose, dupBone->name)) )
+		if ( (conlist = &chan->constraints) )
+			for (curcon = conlist->first; curcon; curcon=curcon->next) {
+				/* does this constraint have a subtarget in
+				 * this armature?
+				 */
+				subname = get_con_subtarget_name(curcon, G.obedit);
+				oldtarget = get_named_editbone(subname);
+				if (oldtarget)
+					/* was the subtarget bone duplicated too? If
+					 * so, update the constraint to point at the 
+					 * duplicate of the old subtarget.
+					 */
+					if (oldtarget->flag & BONE_SELECTED){
+						newtarget = (EditBone*) oldtarget->temp;
+						strcpy(subname, newtarget->name);
+					}
+			}
+	
+}
 
 void adduplicate_armature(void)
 {
@@ -1975,6 +2090,27 @@ void adduplicate_armature(void)
 			BLI_addtail (&G.edbo, eBone);
 			if (!firstDup)
 				firstDup=eBone;
+
+			/* Lets duplicate the list of constraits that the
+			 * current bone has.
+			 */
+			if (OBACT->pose) {
+				bPoseChannel *chanold, *channew;
+				ListBase     *listold, *listnew;
+
+				chanold = verify_pose_channel (OBACT->pose, curBone->name);
+				if (chanold) {
+					listold = &chanold->constraints;
+					if (listold){
+						channew = 
+							verify_pose_channel(OBACT->pose, eBone->name);
+						if (channew) {
+							listnew = &channew->constraints;
+							copy_constraints (listnew, listold);
+						}
+					}
+				}
+			}
 		}
 	}
 
@@ -2011,6 +2147,10 @@ void adduplicate_armature(void)
 				eBone->parent=(EditBone*) curBone->parent; 
 				eBone->flag &= ~BONE_IK_TOPARENT;
 			}
+
+			/* Lets try to fix any constraint subtargets that might
+			   have been duplicated */
+			update_dup_subtarget(eBone);
 			 
 		}
 	} 
@@ -2021,10 +2161,12 @@ void adduplicate_armature(void)
 		curBone->flag &= ~(BONE_SELECTED | BONE_TIPSEL | BONE_ROOTSEL);
 	}
 
-
-	transform('g');
+	BIF_TransformSetUndo("Add Duplicate");
+	initTransform(TFM_TRANSLATION, CTX_NO_PET);
+	Transform();
 	allqueue(REDRAWBUTSEDIT, 0);
 	allqueue(REDRAWBUTSOBJECT, 0);
+	allqueue(REDRAWOOPS, 0);
 }
 
 /*
@@ -2058,8 +2200,7 @@ static void clear_armature_children (Bone *bone, bPose *pose, char mode){
 	if (!bone)
 		return;
 	
-	verify_pose_channel (pose, bone->name);
-	chan=get_pose_channel (pose, bone->name);
+	chan = verify_pose_channel (pose, bone->name);
 
 	if (!chan)
 		return;
@@ -2119,68 +2260,14 @@ void mousepose_armature(void)
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWACTION, 0);
 	allqueue(REDRAWIPO, 0);		/* To force action ipo update */
+	allqueue(REDRAWBUTSEDIT, 0);
 	allqueue(REDRAWBUTSOBJECT, 0);
+	allqueue(REDRAWOOPS, 0);
 
-//	countall();
 	rightmouse_transform();
 	
 }
 
-void make_trans_bones (char mode)
-/*	Used in pose mode	*/
-{
-	bArmature		*arm;
-	Bone			*curBone;
-	int				count=0;
-
-	transmain=NULL;
-
-	arm=get_armature (G.obpose);
-	if (!arm)
-		return;
-
-	if (arm->flag & ARM_RESTPOS){
-		notice ("Transformation not possible while Rest Position is enabled");
-		return;
-	}
-
-
-	if (!(G.obpose->lay & G.vd->lay))
-		return;
-
-
-	centroid[0]=centroid[1]=centroid[2]=0;
-
-	apply_pose_armature(arm, G.obpose->pose, 0);
-	where_is_armature (G.obpose);
-
-	/*	Allocate memory for the transformation record */
-	tottrans= count_bones (arm, BONE_SELECTED, 0);
-
-	if (!tottrans)
-		return;
-
-	transmain= MEM_callocN(tottrans*sizeof(TransOb), "bonetransmain");
-
-	for (curBone=arm->bonebase.first; curBone; curBone=curBone->next){
-		count = add_trans_bonechildren (G.obpose, curBone, transmain, count, mode);
-	}
-	
-	tottrans=count;
-	
-	if (tottrans){
-		centroid[0]/=tottrans;
-		centroid[1]/=tottrans;
-		centroid[2]/=tottrans;
-		Mat4MulVecfl (G.obpose->obmat, centroid);
-	}
-	else{
-		MEM_freeN (transmain);
-	}
-	return;
-
-}
-
 static int	count_bones (bArmature *arm, int flagmask, int allbones)
 {
 	int	count=0;
@@ -2218,94 +2305,7 @@ static int count_bonechildren (Bone *bone, int incount, int flagmask, int allbon
 }
 
 
-static int add_trans_bonechildren (Object* ob, Bone* bone, TransOb* buffer, int index, char mode)
-{
-	Bone	*curBone;
-	TransOb	*curOb;
-	float	parmat[4][4], tempmat[4][4];
-	float tempobmat[4][4];
-	float vec[3];
-	if (!bone)
-		return index;
-
-	
-	
-	/* We don't let IK children get "grabbed" */
-	if (bone->flag & BONE_SELECTED){
-		if (!((mode=='g' || mode=='G') && (bone->flag & BONE_IK_TOPARENT))){
-			
-			get_bone_root_pos (bone, vec, 1);
-		
-			VecAddf (centroid, centroid, vec);
-			
-			curOb=&buffer[index];
-			
-			curOb->ob = ob;
-			curOb->rot=NULL;
-
-			curOb->quat= bone->quat;
-			curOb->size= bone->size;
-			curOb->loc = bone->loc;
-
-			curOb->data = bone; //	FIXME: Dangerous
-			
-			memcpy (curOb->oldquat, bone->quat, sizeof (bone->quat));
-			memcpy (curOb->oldsize, bone->size, sizeof (bone->size));
-			memcpy (curOb->oldloc, bone->loc, sizeof (bone->loc));
-
-#if 0
-			if (bone->parent)
-				get_objectspace_bone_matrix(bone->parent, tempmat, 1, 1);
-			else
-				Mat4One (tempmat);
-#else
-			/* Get the matrix of this bone minus the usertransform */
-			Mat4CpyMat4 (tempobmat, bone->obmat);
-			Mat4One (bone->obmat);
-			get_objectspace_bone_matrix(bone, tempmat, 1, 1);
-			Mat4CpyMat4 (bone->obmat, tempobmat);
-
-			
-#endif
-
-#if 1
-			Mat4MulMat4 (parmat, tempmat, ob->obmat);	/* Original */
-
-			/* Get world transform */
-			get_objectspace_bone_matrix(bone, tempmat, 1, 1);
-			if (ob->parent){
-				where_is_object(ob->parent);
-				Mat4MulSerie (tempobmat, ob->parent->obmat, ob->obmat, tempmat, NULL, NULL, NULL, NULL, NULL);
-			}
-			else
-				Mat4MulSerie (tempobmat, ob->obmat, tempmat, NULL, NULL, NULL, NULL, NULL, NULL);
-			Mat3CpyMat4 (curOb->axismat, tempobmat);
-			Mat3Ortho(curOb->axismat);
-
-#else
-			Mat4MulMat4 (parmat, ob->obmat, tempmat);
-#endif
-			Mat3CpyMat4 (curOb->parmat, parmat);
-			Mat3Inv (curOb->parinv, curOb->parmat);
-
-			Mat3CpyMat4 (curOb->obmat, bone->obmat);
-			Mat3Inv (curOb->obinv, curOb->obmat);
-			
-			index++;
-			return index;
-		}
-
-	}
-	
-	/*	Recursively search  */
-	for (curBone = bone->childbase.first; curBone; curBone=curBone->next){
-		index=add_trans_bonechildren (ob, curBone, buffer, index, mode);
-	}
-
-	return index;
-}
-
-static void deselect_bonechildren (Bone *bone, int mode)
+static void deselect_bonechildren (Object *ob, Bone *bone, int mode)
 {
 	Bone	*curBone;
 
@@ -2317,29 +2317,36 @@ static void deselect_bonechildren (Bone *bone, int mode)
 	else if (!(bone->flag & BONE_HIDDEN))
 		bone->flag |= BONE_SELECTED;
 
-	select_actionchannel_by_name(G.obpose->action, bone->name, mode);
+	select_actionchannel_by_name(ob->action, bone->name, mode);
 
 	for (curBone=bone->childbase.first; curBone; curBone=curBone->next){
-		deselect_bonechildren(curBone, mode);
+		deselect_bonechildren(ob, curBone, mode);
 	}
 }
 
 
 void deselectall_posearmature (int test){
+	Object *ob= OBACT;
 	int	selectmode	=	0;
 	Bone*	curBone;
 	
+	/* we call this from outliner, also without obpose, but with OBACT set OK */
+	if(G.obpose) ob= G.obpose;
+	
 	/*	Determine if we're selecting or deselecting	*/
 	if (test){
-		if (!count_bones (get_armature(G.obpose), BONE_SELECTED, 0))
+		if (!count_bones (get_armature(ob), BONE_SELECTED, 0))
 			selectmode = 1;
 	}
 	
 	/*	Set the flags accordingly	*/
-	for (curBone=get_armature(G.obpose)->bonebase.first; curBone; curBone=curBone->next)
-		deselect_bonechildren (curBone, selectmode);
+	for (curBone=get_armature(ob)->bonebase.first; curBone; curBone=curBone->next)
+		deselect_bonechildren (ob, curBone, selectmode);
 	
+	allqueue(REDRAWBUTSEDIT, 0);
+	allqueue(REDRAWBUTSOBJECT, 0);
 	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWOOPS, 0);
 	allqueue(REDRAWACTION, 0);
 
 }
@@ -2410,7 +2417,44 @@ int bone_looper(Object *ob, Bone *bone, void *data,
     return count;
 }
 
-int bone_skinnable(Object *ob, Bone *bone, void *data)
+int ik_chain_looper(Object *ob, Bone *bone, void *data,
+				   int (*bone_func)(Object *, Bone *, void *)) {
+
+    /* We want to apply the function bone_func to every bone 
+     * in an ik chain -- feed ikchain_looper a bone in the chain and 
+     * a pointer to the bone_func and watch it go!. The int count 
+     * can be useful for counting bones with a certain property
+     * (e.g. skinnable)
+     */
+	Bone *curBone;
+    int   count = 0;
+
+    if (bone) {
+
+        /* This bone */
+        count += bone_func(ob, bone, data);
+
+        /* The parents */
+		for (curBone = bone; curBone; curBone=curBone->parent) {
+			if (!curBone->parent)
+				break;
+			else if (!(curBone->flag & BONE_IK_TOPARENT))
+				break;
+			count += bone_func(ob, curBone->parent, data);
+		}
+
+		/* The children */
+		for (curBone = bone->childbase.first; curBone; curBone=curBone->next){
+			if (curBone->flag & BONE_IK_TOPARENT) {
+				count += bone_func(ob, curBone, data);
+			}
+		}
+    }
+
+    return count;
+}
+
+static int bone_skinnable(Object *ob, Bone *bone, void *data)
 {
     /* Bones that are not of boneclass BONE_UNSKINNABLE
      * are regarded to be "skinnable" and are eligible for
@@ -2447,7 +2491,7 @@ int bone_skinnable(Object *ob, Bone *bone, void *data)
     return 0;
 }
 
-int add_defgroup_unique_bone(Object *ob, Bone *bone, void *data) {
+static int add_defgroup_unique_bone(Object *ob, Bone *bone, void *data) {
     /* This group creates a vertex group to ob that has the
      * same name as bone (provided the bone is skinnable). 
 	 * If such a vertex group aleady exist the routine exits.
@@ -2461,7 +2505,7 @@ int add_defgroup_unique_bone(Object *ob, Bone *bone, void *data) {
     return 0;
 }
 
-int dgroup_skinnable(Object *ob, Bone *bone, void *data) {
+static int dgroup_skinnable(Object *ob, Bone *bone, void *data) {
     /* Bones that are not of boneclass BONE_UNSKINNABLE
      * are regarded to be "skinnable" and are eligible for
      * auto-skinning.
@@ -2502,7 +2546,7 @@ int dgroup_skinnable(Object *ob, Bone *bone, void *data) {
     return 0;
 }
 
-void add_verts_to_closest_dgroup(Object *ob, Object *par)
+static void add_verts_to_closest_dgroup(Object *ob, Object *par)
 {
     /* This function implements a crude form of 
      * auto-skinning: vertices are assigned to the
@@ -2570,7 +2614,7 @@ void add_verts_to_closest_dgroup(Object *ob, Object *par)
     mesh = (Mesh*)ob->data;
 
 	/* Is subsurf on? Lets use the verts on the limit surface then */
-	if ( (mesh->flag&ME_SUBSURF) && (mesh->subdiv > 0) ) {
+	if (mesh->flag&ME_SUBSURF) {
 		subverts = MEM_mallocN(3*mesh->totvert*sizeof(float), "subverts");
 		subsurf_calculate_limit_positions(mesh, (void *)subverts);	/* (ton) made void*, dunno how to cast */
 	}
@@ -2707,3 +2751,81 @@ void create_vgroups_from_armature(Object *ob, Object *par)
 
 	}
 } 
+
+static int hide_selected_pose_bone(Object *ob, Bone *bone, void *ptr) {
+	if (bone->flag & BONE_SELECTED) {
+		bone->flag |= BONE_HIDDEN;
+		bone->flag &= ~BONE_SELECTED;
+	}
+	return 0;
+}
+
+void hide_selected_pose_bones(void) {
+	bArmature		*arm;
+
+	arm=get_armature (G.obpose);
+
+	if (!arm)
+		return;
+
+	bone_looper(G.obpose, arm->bonebase.first, NULL, 
+				hide_selected_pose_bone);
+
+	force_draw(1);
+}
+
+static int hide_unselected_pose_bone(Object *ob, Bone *bone, void *ptr) {
+	if (~bone->flag & BONE_SELECTED) {
+		bone->flag |= BONE_HIDDEN;
+	}
+	return 0;
+}
+
+void hide_unselected_pose_bones(void) {
+	bArmature		*arm;
+
+	arm=get_armature (G.obpose);
+
+	if (!arm)
+		return;
+
+	bone_looper(G.obpose, arm->bonebase.first, NULL, 
+				hide_unselected_pose_bone);
+
+	force_draw(1);
+}
+
+static int show_pose_bone(Object *ob, Bone *bone, void *ptr) {
+	if (bone->flag & BONE_HIDDEN) {
+		bone->flag &= ~BONE_HIDDEN;
+		bone->flag |= BONE_SELECTED;
+	}
+
+	return 0;
+}
+
+void show_all_pose_bones(void) {
+	bArmature		*arm;
+
+	arm=get_armature (G.obpose);
+
+	if (!arm)
+		return;
+
+	bone_looper(G.obpose, arm->bonebase.first, NULL, 
+				show_pose_bone);
+
+	force_draw(1);
+}
+
+int is_delay_deform(void)
+{
+	bArmature               *arm;
+
+	arm=get_armature (G.obpose);
+
+	if (!arm)
+		return 0;
+
+	return (arm->flag & ARM_DELAYDEFORM);
+}
