@@ -1,5 +1,5 @@
 /**
- * $Id: editcurve.c,v 1.7 2003/05/13 14:22:28 phase Exp $
+ * $Id: editcurve.c,v 1.30 2005/06/13 19:15:02 stiv Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,6 +31,7 @@
  */
 
 #include <math.h>
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -40,7 +41,6 @@
 #include <unistd.h>
 #else
 #include <io.h>
-#include "BLI_winstuff.h"
 #endif
 #include <stdlib.h>
 #include "MEM_guardedalloc.h"
@@ -49,7 +49,7 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
-#include "BLI_editVert.h"
+#include "BLI_dynstr.h"
 
 #include "DNA_curve_types.h"
 #include "DNA_ipo_types.h"
@@ -59,6 +59,7 @@
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_view3d_types.h"
+#include "DNA_userdef_types.h"
 
 #include "BKE_utildefines.h"
 #include "BKE_library.h"
@@ -69,6 +70,7 @@
 #include "BKE_object.h"
 #include "BKE_main.h"
 
+#include "BIF_editmode_undo.h"
 #include "BIF_gl.h"
 #include "BIF_graphics.h"
 #include "BIF_screen.h"
@@ -77,6 +79,7 @@
 #include "BIF_editkey.h"
 #include "BIF_mywindow.h"
 #include "BIF_interface.h"
+#include "BIF_transform.h"
 
 #include "BSE_view.h"	/* For persp... */
 #include "BSE_edit.h"
@@ -85,9 +88,6 @@
 #include "BDR_editcurve.h"
 #include "BDR_editobject.h"
 
-/*  #include "graphics.h"  */
-#include "interface.h" /* MAART: for NUM and FLO types, + pupmenu */
-/*  #include "edit.h" */
 #include "mydevice.h"
 #include "blendef.h"
 
@@ -193,7 +193,7 @@ static void printweightsNurb(void)
 
 	if(G.obedit==0) return;
 
-	persp(0);
+	persp(PERSP_WIN);
 
 	glDrawBuffer(GL_FRONT); 
 
@@ -226,7 +226,7 @@ static void printweightsNurb(void)
 	}
 
 	glDrawBuffer(GL_BACK); 
-	persp(1);
+	persp(PERSP_VIEW);
 }
 #endif
 
@@ -283,7 +283,6 @@ void load_editNurb()
 			nu= editNurb.first;
 			while(nu) {
 				newnu= duplicateNurb(nu);
-				newnu->hide= 0;
 				BLI_addtail(&(cu->nurb), newnu);
 				
 				if((nu->type & 7)==CU_NURBS) {
@@ -296,7 +295,7 @@ void load_editNurb()
 		
 	}
 	
-	lastnu= 0;	/* for selected */
+	lastnu= NULL;	/* for selected */
 	
 }
 
@@ -305,14 +304,11 @@ void make_editNurb()
 	/* make copy of baseNurb in editNurb */
 	Curve *cu=0;
 	Nurb *nu, *newnu;
-	BezTriple *bezt;
-	BPoint *bp;
 	KeyBlock *actkey=0;
-	int a, tot=0;
 
-	if(G.obedit==0) return;
+	if(G.obedit==NULL) return;
 
-	lastselbp= 0;   /* global for select row */
+	lastselbp= NULL;   /* global for select row */
 
 	if ELEM(G.obedit->type, OB_CURVE, OB_SURF) {
 		freeNurblist(&editNurb);
@@ -322,27 +318,8 @@ void make_editNurb()
 		
 		while(nu) {
 			newnu= duplicateNurb(nu);
+			test2DNurb(newnu);	// after join, or any other creation of curve
 			BLI_addtail(&editNurb, newnu);
-			/* flags zero */
-			newnu->hide= 0;
-			if((nu->type & 7)==CU_BEZIER) {
-				a= nu->pntsu;
-				bezt= newnu->bezt;
-				while(a--) {
-					bezt->f1= bezt->f2= bezt->f3= bezt->hide= 0;
-					bezt++;
-					tot+= 3;
-				}
-			}
-			else {
-				a= nu->pntsu*nu->pntsv;
-				bp= newnu->bp;
-				while(a--) {
-					bp->f1= bp->hide= 0;
-					bp++;
-					tot++;
-				}
-			}
 			nu= nu->next;
 		}
 		
@@ -359,17 +336,17 @@ void make_editNurb()
 		}
 		makeDispList(G.obedit);
 	}
-	else G.obedit= 0;
+	else G.obedit= NULL;
 	
 	countall();
 	
-	lastnu= 0;	/* for selected */
+	lastnu= NULL;	/* for selected */
 }
 
 void remake_editNurb()
 {
 
-	if(okee("Reload Original data")==0) return;
+	if(okee("Reload original data")==0) return;
 	
 	make_editNurb();
 	allqueue(REDRAWVIEW3D, 0);
@@ -394,7 +371,7 @@ void separate_nurb()
 	
 	cu= G.obedit->data;
 	if(cu->key) {
-		error("Can't separate with vertex keys");
+		error("Can't separate a curve with vertex keys");
 		return;
 	}
 	
@@ -452,6 +429,8 @@ void separate_nurb()
 	
 	G.obedit= 0;	/* displists behave different in edit mode */
 	makeDispList(OBACT);	/* this is the separated one */
+
+	curve_changes_other_objects(oldob);
 	
 	G.obedit= oldob;
 	BASACT= oldbase;
@@ -461,8 +440,9 @@ void separate_nurb()
 
 	countall();
 	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWBUTSEDIT, 0);
 
-	lastnu= 0;	/* for selected */
+	lastnu= NULL;	/* for selected */
 }
 
 /* ******************* FLAGS ********************* */
@@ -636,7 +616,7 @@ void deleteflagNurb(short flag)
 	if(G.obedit && G.obedit->type==OB_SURF);
 	else return;
 
-	lastselbp= 0;
+	lastselbp= NULL;
 
 	nu= editNurb.first;
 	while(nu) {
@@ -1052,7 +1032,10 @@ void switchdirectionNurb2(void)
 	}
 	
 	makeDispList(G.obedit);
+	curve_changes_other_objects(G.obedit);
+
 	allqueue(REDRAWVIEW3D, 0);
+	BIF_undo_push("Switch direction");
 }
 
 void switchdirection_knots(float *base, int tot)
@@ -1107,6 +1090,8 @@ void deselectall_nurb()
 
 	if(G.obedit->lay & G.vd->lay);
 	else return;
+
+	BIF_undo_push("Deselect all");
 	
 	a= 0;
 	nu= editNurb.first;
@@ -1196,6 +1181,8 @@ void hideNurb(int swap)
 
 	if(G.obedit==0) return;
 
+	BIF_undo_push("Hide");
+
 	nu= editNurb.first;
 	while(nu) {
 		if((nu->type & 7)==CU_BEZIER) {
@@ -1239,6 +1226,7 @@ void hideNurb(int swap)
 	makeDispList(G.obedit);
 	countall();
 	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWBUTSEDIT, 0);
 }
 
 void revealNurb()
@@ -1283,6 +1271,8 @@ void revealNurb()
 	makeDispList(G.obedit);
 	countall();
 	allqueue(REDRAWVIEW3D, 0);
+	BIF_undo_push("Reveal");
+
 }
 
 void selectswapNurb()
@@ -1327,6 +1317,8 @@ void selectswapNurb()
 
 	countall();
 	allqueue(REDRAWVIEW3D, 0);
+	BIF_undo_push("Select swap");
+
 }
 
 /** Divide the line segments associated with the currently selected
@@ -1342,14 +1334,14 @@ void subdivideNurb()
 	Nurb *nu;
 	BezTriple *prevbezt, *bezt, *beztnew, *beztn;
 	BPoint *bp, *prevbp, *bpnew, *bpn;
-	float vec[12];
-	int a, b, sel, aantal, *usel, *vsel;
+	float vec[15];
+	int a, b, sel, amount, *usel, *vsel;
 
    // printf("*** subdivideNurb: entering subdivide\n");
 
 	nu= editNurb.first;
 	while(nu) {
-		aantal= 0;
+		amount= 0;
 		if((nu->type & 7)==CU_BEZIER) {
         /* 
            Insert a point into a 2D Bezier curve. 
@@ -1368,15 +1360,15 @@ void subdivideNurb()
 				bezt= prevbezt+1;
 			}
 			while(a--) {
-				if( BEZSELECTED(prevbezt) && BEZSELECTED(bezt) ) aantal++;
+				if( BEZSELECTED(prevbezt) && BEZSELECTED(bezt) ) amount++;
 				prevbezt= bezt;
 				bezt++;
 			}
 
-			if(aantal) {
+			if(amount) {
 				/* insert */
 				beztnew =
-					(BezTriple*)MEM_mallocN((aantal + nu->pntsu) * sizeof(BezTriple), "subdivNurb");
+					(BezTriple*)MEM_mallocN((amount + nu->pntsu) * sizeof(BezTriple), "subdivNurb");
 				beztn= beztnew;
 				if(nu->flagu & 1) {
 					a= nu->pntsu;
@@ -1391,18 +1383,28 @@ void subdivideNurb()
 				while(a--) {
 					memcpy(beztn, prevbezt, sizeof(BezTriple));
 					beztn++;
-               // printf("*** subdivideNurb: insert Bezier point\n");
 
 					if( BEZSELECTED(prevbezt) && BEZSELECTED(bezt) ) {
 						memcpy(beztn, bezt, sizeof(BezTriple));
-						maakbez(prevbezt->vec[1][0],prevbezt->vec[2][0],
-						    bezt->vec[0][0],bezt->vec[1][0],vec,2);
-						maakbez(prevbezt->vec[1][1],prevbezt->vec[2][1],
-						    bezt->vec[0][1],bezt->vec[1][1],vec+1,2);
-						maakbez(prevbezt->vec[1][2],prevbezt->vec[2][2],
-						    bezt->vec[0][2],bezt->vec[1][2],vec+2,2);
-						VECCOPY(beztn->vec[1], vec+3);
-						beztn->h1= beztn->h2= HD_AUTO;
+						
+						/* midpoint subdividing */
+						VecMidf(vec, prevbezt->vec[1], prevbezt->vec[2]);
+						VecMidf(vec+3, prevbezt->vec[2], bezt->vec[0]);
+						VecMidf(vec+6, bezt->vec[0], bezt->vec[1]);
+						
+						VecMidf(vec+9, vec, vec+3);
+						VecMidf(vec+12, vec+3, vec+6);
+						
+						/* change handle of prev beztn */
+						VECCOPY((beztn-1)->vec[2], vec);
+						/* new point */
+						VECCOPY(beztn->vec[0], vec+9);
+						VecMidf(beztn->vec[1], vec+9, vec+12);
+						VECCOPY(beztn->vec[2], vec+12);
+						/* handle of next bezt */
+						if(a==0 && (nu->flagu & 1)) {VECCOPY(beztnew->vec[0], vec+6);}
+						else {VECCOPY(bezt->vec[0], vec+6);}
+						
 						beztn++;
 					}
 
@@ -1414,7 +1416,7 @@ void subdivideNurb()
 
 				MEM_freeN(nu->bezt);
 				nu->bezt= beztnew;
-				nu->pntsu+= aantal;
+				nu->pntsu+= amount;
 
 				calchandlesNurb(nu);
 			}
@@ -1438,15 +1440,15 @@ void subdivideNurb()
 				bp= prevbp+1;
 			}
 			while(a--) {
-				if( (bp->f1 & 1) && (prevbp->f1 & 1) ) aantal++;
+				if( (bp->f1 & 1) && (prevbp->f1 & 1) ) amount++;
 				prevbp= bp;
 				bp++;
 			}
 
-			if(aantal) {
+			if(amount) {
 				/* insert */
 				bpnew =
-					(BPoint*)MEM_mallocN((aantal + nu->pntsu) * sizeof(BPoint), "subdivNurb2");
+					(BPoint*)MEM_mallocN((amount + nu->pntsu) * sizeof(BPoint), "subdivNurb2");
 				bpn= bpnew;
 
 				if(nu->flagu & 1) {
@@ -1480,7 +1482,7 @@ void subdivideNurb()
 
 				MEM_freeN(nu->bp);
 				nu->bp= bpnew;
-				nu->pntsu+= aantal;
+				nu->pntsu+= amount;
 
 				if(nu->type & 4) {
 					makeknots(nu, 1, nu->flagu>>1);
@@ -1692,8 +1694,13 @@ void subdivideNurb()
 
 
 	makeDispList(G.obedit);
+	curve_changes_other_objects(G.obedit);
+
 	countall();
 	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWBUTSEDIT, 0);
+	BIF_undo_push("Subdivide");
+
 }
 
 
@@ -1846,144 +1853,156 @@ void findselectedNurbvert(Nurb **nu, BezTriple **bezt, BPoint **bp)
 	}
 }
 
-void setsplinetype(short type)
+int convertspline(short type, Nurb *nu)
 {
-	Nurb *nu;
 	BezTriple *bezt;
 	BPoint *bp;
 	int a, c, nr;
 
+	if((nu->type & 7)==0) {		/* Poly */
+		if(type==CU_BEZIER) {			    /* to Bezier with vecthandles  */
+			nr= nu->pntsu;
+			bezt =
+				(BezTriple*)MEM_callocN(nr * sizeof(BezTriple), "setsplinetype2");
+			nu->bezt= bezt;
+			a= nr;
+			bp= nu->bp;
+			while(a--) {
+				VECCOPY(bezt->vec[1], bp->vec);
+				bezt->f1=bezt->f2=bezt->f3= bp->f1;
+				bezt->h1= bezt->h2= HD_VECT;
+				bp++;
+				bezt++;
+			}
+			MEM_freeN(nu->bp);
+			nu->bp= 0;
+			nu->pntsu= nr;
+			nu->type &= ~7;
+			nu->type |= 1;
+			calchandlesNurb(nu);
+		}
+		else if(type==4) {		    /* to Nurb */
+			nu->type &= ~7;
+			nu->type+= 4;
+			nu->orderu= 4;
+			nu->flagu &= 1;
+			nu->flagu += 4;
+			makeknots(nu, 1, nu->flagu>>1);
+			a= nu->pntsu*nu->pntsv;
+			bp= nu->bp;
+			while(a--) {
+				bp->vec[3]= 1.0;
+				bp++;
+			}
+		}
+	}
+	else if((nu->type & 7)==CU_BEZIER) {	/* Bezier */
+		if(type==0 || type==4) {	    /* to Poly or Nurb */
+			nr= 3*nu->pntsu;
+			nu->bp = MEM_callocN(nr * sizeof(BPoint), "setsplinetype");
+			a= nu->pntsu;
+			bezt= nu->bezt;
+			bp= nu->bp;
+			while(a--) {
+				if(type==0 && bezt->h1==HD_VECT && bezt->h2==HD_VECT) {
+					/* vector handle becomes 1 poly vertice */
+					VECCOPY(bp->vec, bezt->vec[1]);
+					bp->vec[3]= 1.0;
+					bp->f1= bezt->f2;
+					nr-= 2;
+					bp++;
+				}
+				else {
+					for(c=0;c<3;c++) {
+						VECCOPY(bp->vec, bezt->vec[c]);
+						bp->vec[3]= 1.0;
+						if(c==0) bp->f1= bezt->f1;
+						else if(c==1) bp->f1= bezt->f2;
+						else bp->f1= bezt->f3;
+						bp++;
+					}
+				}
+				bezt++;
+			}
+			MEM_freeN(nu->bezt); 
+			nu->bezt= 0;
+			nu->pntsu= nr;
+			nu->pntsv= 1;
+			nu->orderu= 4;
+			nu->orderv= 1;
+			nu->type &= ~7;
+			nu->type+= type;
+			if(nu->flagu & 1) c= nu->orderu-1; 
+			else c= 0;
+			if(type== 4) {
+				nu->flagu &= 1;
+				nu->flagu += 4;
+				makeknots(nu, 1, nu->flagu>>1);
+			}
+		}
+	}
+	else if( (nu->type & 7)==CU_NURBS) {
+		if(type==0) {			/* to Poly */
+			nu->type &= ~7;
+			MEM_freeN(nu->knotsu);
+			nu->knotsu= 0;
+			if(nu->knotsv) MEM_freeN(nu->knotsv);
+			nu->knotsv= 0;
+		}
+		else if(type==CU_BEZIER) {		/* to Bezier */
+			nr= nu->pntsu/3;
+
+			if(nr<2) 
+				return 1;	/* conversion impossible */
+			else {
+				bezt = MEM_callocN(nr * sizeof(BezTriple), "setsplinetype2");
+				nu->bezt= bezt;
+				a= nr;
+				bp= nu->bp;
+				while(a--) {
+					VECCOPY(bezt->vec[0], bp->vec);
+					bezt->f1= bp->f1;
+					bp++;
+					VECCOPY(bezt->vec[1], bp->vec);
+					bezt->f2= bp->f1;
+					bp++;
+					VECCOPY(bezt->vec[2], bp->vec);
+					bezt->f3= bp->f1;
+					bp++;
+					bezt++;
+				}
+				MEM_freeN(nu->bp);
+				nu->bp= 0;
+				MEM_freeN(nu->knotsu);
+				nu->knotsu= 0;
+				nu->pntsu= nr;
+				nu->type &= ~7;
+				nu->type+= 1;
+			}
+		}
+	}
+	return 0;
+}
+
+void setsplinetype(short type)
+{
+	Nurb *nu;
+
 	if(type==CU_CARDINAL || type==CU_BSPLINE) {
 		error("Not implemented yet");
 		return;
 	}
-
+	
 	nu= editNurb.first;
 	while(nu) {
 		if(isNurbsel(nu)) {
-
-			if((nu->type & 7)==0) {		/* Poly */
-				if(type==CU_BEZIER) {			    /* to Bezier with vecthandles  */
-					nr= nu->pntsu;
-					bezt =
-						(BezTriple*)MEM_callocN(nr * sizeof(BezTriple), "setsplinetype2");
-					nu->bezt= bezt;
-					a= nr;
-					bp= nu->bp;
-					while(a--) {
-						VECCOPY(bezt->vec[1], bp->vec);
-						bezt->f1=bezt->f2=bezt->f3= bp->f1;
-						bezt->h1= bezt->h2= HD_VECT;
-						bp++;
-						bezt++;
-					}
-					MEM_freeN(nu->bp);
-					nu->bp= 0;
-					nu->pntsu= nr;
-					nu->type &= ~7;
-					nu->type |= 1;
-					calchandlesNurb(nu);
-				}
-				else if(type==4) {		    /* to Nurb */
-					nu->type &= ~7;
-					nu->type+= 4;
-					nu->orderu= 4;
-					nu->flagu &= 1;
-					nu->flagu += 4;
-					makeknots(nu, 1, nu->flagu>>1);
-					a= nu->pntsu*nu->pntsv;
-					bp= nu->bp;
-					while(a--) {
-						bp->vec[3]= 1.0;
-						bp++;
-					}
-				}
-			}
-			else if((nu->type & 7)==CU_BEZIER) {	/* Bezier */
-				if(type==0 || type==4) {	    /* to Poly or Nurb */
-					nr= 3*nu->pntsu;
-					nu->bp =
-						(BPoint*)MEM_callocN(nr * sizeof(BPoint), "setsplinetype");
-					a= nu->pntsu;
-					bezt= nu->bezt;
-					bp= nu->bp;
-					while(a--) {
-						if(type==0 && bezt->h1==HD_VECT && bezt->h2==HD_VECT) {
-							/* vector handle becomes 1 poly vertice */
-							VECCOPY(bp->vec, bezt->vec[1]);
-							bp->vec[3]= 1.0;
-							bp->f1= bezt->f2;
-							nr-= 2;
-							bp++;
-						}
-						else {
-							for(c=0;c<3;c++) {
-								VECCOPY(bp->vec, bezt->vec[c]);
-								bp->vec[3]= 1.0;
-								if(c==0) bp->f1= bezt->f1;
-								else if(c==1) bp->f1= bezt->f2;
-								else bp->f1= bezt->f3;
-								bp++;
-							}
-						}
-						bezt++;
-					}
-					MEM_freeN(nu->bezt); 
-					nu->bezt= 0;
-					nu->pntsu= nr;
-					nu->pntsv= 1;
-					nu->orderu= 4;
-					nu->orderv= 1;
-					nu->type &= ~7;
-					nu->type+= type;
-					if(nu->flagu & 1) c= nu->orderu-1; 
-					else c= 0;
-					if(type== 4) {
-						nu->flagu &= 1;
-						nu->flagu += 4;
-						makeknots(nu, 1, nu->flagu>>1);
-					}
-				}
-			}
-			else if( (nu->type & 7)==CU_NURBS && G.obedit->type==OB_CURVE) {
-				if(type==0) {			/* to Poly */
-					nu->type &= ~7;
-					MEM_freeN(nu->knotsu);
-					nu->knotsu= 0;
-					if(nu->knotsv) MEM_freeN(nu->knotsv);
-					nu->knotsv= 0;
-				}
-				else if(type==CU_BEZIER) {		/* to Bezier */
-					nr= nu->pntsu/3;
-					bezt =
-						(BezTriple*)MEM_callocN(nr * sizeof(BezTriple), "setsplinetype2");
-					nu->bezt= bezt;
-					a= nr;
-					bp= nu->bp;
-					while(a--) {
-						VECCOPY(bezt->vec[0], bp->vec);
-						bezt->f1= bp->f1;
-						bp++;
-						VECCOPY(bezt->vec[1], bp->vec);
-						bezt->f2= bp->f1;
-						bp++;
-						VECCOPY(bezt->vec[2], bp->vec);
-						bezt->f3= bp->f1;
-						bp++;
-						bezt++;
-					}
-					MEM_freeN(nu->bp);
-					nu->bp= 0;
-					MEM_freeN(nu->knotsu);
-					nu->knotsu= 0;
-					nu->pntsu= nr;
-					nu->type &= ~7;
-					nu->type+= 1;
-				}
-			}
+			if (convertspline(type, nu))
+				error("no conversion possible");
 		}
 		nu= nu->next;
 	}
+	BIF_undo_push("Set spline type");
+	
 }
 
 /* ******************** SKINNING LOFTING!!! ******************** */
@@ -2159,7 +2178,7 @@ void merge_2_nurb(Nurb *nu1, Nurb *nu2)
 	}
 	
 	if( nu1->pntsv != nu2->pntsv ) {
-		error("resolution doesn't match");
+		error("Resolution doesn't match");
 		return;
 	}
 	
@@ -2212,12 +2231,14 @@ void merge_2_nurb(Nurb *nu1, Nurb *nu2)
 		}
 	}
 
-	/* merge knots */
-	makeknots(nu1, 1, nu1->flagu>>1);
-
-	/* make knots, for merged curved for example */
-	makeknots(nu1, 2, nu1->flagv>>1);
-
+	if((nu1->type & 7)==4) {
+		/* merge knots */
+		makeknots(nu1, 1, nu1->flagu>>1);
+	
+		/* make knots, for merged curved for example */
+		makeknots(nu1, 2, nu1->flagv>>1);
+	}
+	
 	MEM_freeN(temp);
 	BLI_remlink(&editNurb, nu2);
 	freeNurb(nu2);
@@ -2232,7 +2253,7 @@ void merge_nurb()
 	
 	if(nsortbase.first == nsortbase.last) {
 		BLI_freelistN(&nsortbase);
-		error("Too few selections");
+		error("Too few selections to merge");
 		return;
 	}
 	
@@ -2255,7 +2276,7 @@ void merge_nurb()
 	}
 	
 	if(ok==0) {
-		error("resolution doesn't match");
+		error("Resolution doesn't match");
 		BLI_freelistN(&nsortbase);
 		return;
 	}
@@ -2267,8 +2288,15 @@ void merge_nurb()
 	
 	BLI_freelistN(&nsortbase);
 	
+	countall();
+	lastnu= NULL;
+
 	makeDispList(G.obedit);
+	curve_changes_other_objects(G.obedit);
+
 	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWBUTSEDIT, 0);
+	BIF_undo_push("Merge");
 	
 }
 
@@ -2284,7 +2312,7 @@ void addsegment_nurb()
 
 	/* first decide if this is a surface merge! */
 	if(G.obedit->type==OB_SURF) nu= editNurb.first;
-	else nu= 0;
+	else nu= NULL;
 	
 	while(nu) {
 		if( isNurbsel(nu) ) {
@@ -2395,27 +2423,40 @@ void addsegment_nurb()
 
 				/* now join the knots */
 				if((nu1->type & 7)==4) {
-					fp= MEM_mallocN(sizeof(float)*KNOTSU(nu1), "addsegment3");
-					memcpy(fp, nu1->knotsu, sizeof(float)*a);
-					MEM_freeN(nu1->knotsu);
-					nu1->knotsu= fp;
-					
-					
-					offset= nu1->knotsu[a-1] +1.0;
-					fp= nu1->knotsu+a;
-					for(a=0; a<nu2->pntsu; a++, fp++) {
-						if(nu2->knotsu) 
-							*fp= offset+nu2->knotsu[a+1];
-						else 
-							*fp = offset;
+					if(nu1->knotsu==NULL) {
+						makeknots(nu1, 1, nu1->flagu>>1);
+					}
+					else {
+						fp= MEM_mallocN(sizeof(float)*KNOTSU(nu1), "addsegment3");
+						memcpy(fp, nu1->knotsu, sizeof(float)*a);
+						MEM_freeN(nu1->knotsu);
+						nu1->knotsu= fp;
+						
+						
+						offset= nu1->knotsu[a-1] +1.0;
+						fp= nu1->knotsu+a;
+						for(a=0; a<nu2->pntsu; a++, fp++) {
+							if(nu2->knotsu) 
+								*fp= offset+nu2->knotsu[a+1];
+							else 
+								*fp = offset;
+						}
 					}
 				}
 				freeNurb(nu2);
 			}
 		}
+		
+		lastnu= NULL;	/* for selected */
+
 		makeDispList(G.obedit);
+		curve_changes_other_objects(G.obedit);
+
 		countall();
 		allqueue(REDRAWVIEW3D, 0);
+		allqueue(REDRAWBUTSEDIT, 0);
+		BIF_undo_push("Add segment");
+	
 	}
 	else error("Can't make segment");
 }
@@ -2506,7 +2547,7 @@ void mouse_nurb()
 	
 }
 
-void spinNurb(float *dvec, short mode)
+static void spin_nurb(float *dvec, short mode)
 {
 	Nurb *nu;
 	float *curs, si,phi,n[3],q[4],cmat[3][3],tmat[3][3],imat[3][3];
@@ -2605,6 +2646,32 @@ void spinNurb(float *dvec, short mode)
 			nu= nu->next;
 		}
 	}
+	
+}
+
+/* external one, for undo */
+void spinNurb(float *dvec, short mode)
+{
+	spin_nurb(dvec, mode);
+	BIF_undo_push("Spin");
+}
+
+void curve_changes_other_objects(Object *ob)
+{
+	Base *base= FIRSTBASE;
+	while(base) {
+		if(base->lay & G.vd->lay) {
+			if(base->object->parent==ob && base->object->partype==PARSKEL)
+				freedisplist(&base->object->disp);
+			
+			if(base->object->type==OB_CURVE) {
+				Curve *cu= base->object->data;
+				if(ob==cu->bevobj || ob==cu->taperobj)
+					makeDispList(base->object);
+			}
+		}
+		base= base->next;
+	}
 }
 
 void addvert_Nurb(int mode)
@@ -2732,9 +2799,21 @@ void addvert_Nurb(int mode)
 	makeDispList(G.obedit);
 	countall();
 	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWBUTSEDIT, 0);
 
-	if(mode=='e') transform('d');
+	if(mode=='e') {
+		BIF_TransformSetUndo("Extrude");
+		initTransform(TFM_TRANSLATION, CTX_NO_PET);
+		Transform();
+	}
 	else while(get_mbut()&R_MOUSE) BIF_wait_for_statechange();
+
+	if(mode!='e') {
+		/* dependencies with other objects, should become event */
+		curve_changes_other_objects(G.obedit);
+		BIF_undo_push("Add vertex");
+	
+	}
 }
 
 void extrude_nurb()
@@ -2756,12 +2835,15 @@ void extrude_nurb()
 		else {
 
 			if(okee("Extrude")==0) return;
+	
 			ok= extrudeflagNurb(1); /* '1'= flag */
 		
 			if(ok) {
 				makeDispList(G.obedit);
 				countall();
-				transform('d');
+				BIF_TransformSetUndo("Extrude");
+				initTransform(TFM_TRANSLATION, CTX_NO_PET);
+				Transform();
 			}
 		}
 	}
@@ -2873,6 +2955,8 @@ void makecyclicNurb()
 		nu= nu->next;
 	}
 	makeDispList(G.obedit);
+	curve_changes_other_objects(G.obedit);
+	BIF_undo_push("Cyclic");
 }
 
 void selectconnected_nurb()
@@ -2920,6 +3004,8 @@ void selectconnected_nurb()
 
 	countall();
 	allqueue(REDRAWVIEW3D, 0);
+	BIF_undo_push("Select connected");
+	
 }
 
 void selectrow_nurb()
@@ -2931,8 +3017,8 @@ void selectrow_nurb()
 	int u = 0, v = 0, a, b, ok=0;
 
 	if(editNurb.first==0) return;
-	if(G.obedit==0 || G.obedit->type!=OB_SURF) return;
-	if(lastselbp==0) return;
+	if(G.obedit==NULL || G.obedit->type!=OB_SURF) return;
+	if(lastselbp==NULL) return;
 
 	/* find the correct nurb and toggle with u of v */
 	nu= editNurb.first;
@@ -2973,6 +3059,8 @@ void selectrow_nurb()
 		}
 		nu= nu->next;
 	}
+	BIF_undo_push("Select Row");
+	
 }
 
 void adduplicate_nurb()
@@ -2983,7 +3071,9 @@ void adduplicate_nurb()
 	adduplicateflagNurb(1);
 
 	countall();
-	transform('d');
+	BIF_TransformSetUndo("Add Duplicate");
+	initTransform(TFM_TRANSLATION, CTX_NO_PET);
+	Transform();
 }
 
 void delNurb()
@@ -2997,8 +3087,8 @@ void delNurb()
 	if(G.obedit==0 ) return;
 	if( (G.vd->lay & G.obedit->lay)==0 ) return;
 
-	if(G.obedit->type==OB_SURF) event= pupmenu("ERASE %t|Selected%x0|All%x2");
-	else event= pupmenu("ERASE %t|Selected%x0|Segment%x1|All%x2");
+	if(G.obedit->type==OB_SURF) event= pupmenu("Erase %t|Selected%x0|All%x2");
+	else event= pupmenu("Erase %t|Selected%x0|Segment%x1|All%x2");
 
 	if(event== -1) return;
 
@@ -3009,6 +3099,9 @@ void delNurb()
 		countall();
 		makeDispList(G.obedit);
 		allqueue(REDRAWVIEW3D, 0);
+		allqueue(REDRAWBUTSEDIT, 0);
+		BIF_undo_push("Delete");
+	
 		return;
 	}
 
@@ -3123,6 +3216,8 @@ void delNurb()
 									nu->flagu--;
 									makeDispList(G.obedit);
 									allqueue(REDRAWVIEW3D, 0);
+									allqueue(REDRAWBUTSEDIT, 0);
+									BIF_undo_push("Delete");
 								}
 							}
 							return;
@@ -3148,6 +3243,8 @@ void delNurb()
 									nu->flagu--;
 									makeDispList(G.obedit);
 									allqueue(REDRAWVIEW3D, 0);
+									allqueue(REDRAWBUTSEDIT, 0);
+									BIF_undo_push("Delete");
 								}
 							}
 							return;
@@ -3246,7 +3343,12 @@ void delNurb()
 
 	countall();
 	makeDispList(G.obedit);
+	curve_changes_other_objects(G.obedit);
+
 	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWBUTSEDIT, 0);
+	BIF_undo_push("Delete");
+	
 }
 
 
@@ -3270,9 +3372,9 @@ void join_curve(int type)
 	tempbase.first= tempbase.last= 0;
 	
 	if(type==OB_SURF) {
-		if(okee("Join selected Nurbs")==0) return;
+		if(okee("Join selected NURBS")==0) return;
 	}
-	else if(okee("Join selected Curves")==0) return;
+	else if(okee("Join selected curves")==0) return;
 	
 	/* trasnform all selected curves inverse in obact */
 	Mat4Invert(imat, ob->obmat);
@@ -3280,7 +3382,7 @@ void join_curve(int type)
 	base= FIRSTBASE;
 	while(base) {
 		nextb= base->next;
-		if TESTBASE(base) {
+		if TESTBASELIB(base) {
 			if(base->object->type==type) {
 				if(base->object != ob) {
 				
@@ -3329,6 +3431,9 @@ void join_curve(int type)
 	exit_editmode(1);
 	
 	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWBUTSEDIT, 0);
+	BIF_undo_push("Join");
+	
 }
 
 
@@ -3376,7 +3481,6 @@ Nurb *addNurbprim(int type, int stype, int newname)
 		nu->type= type;
 		nu->resolu= 12;
 		nu->resolv= 12;
-		/* if(G.obedit && (G.mainb==5 || G.mainb==9)) nu->col= 0; */
 	}
 
 	switch(stype) {
@@ -3570,7 +3674,7 @@ Nurb *addNurbprim(int type, int stype, int newname)
 			nu->pntsv= 4;
 			nu->orderu= 4;
 			nu->orderv= 4;
-			nu->flag= ME_SMOOTH;
+			nu->flag= CU_SMOOTH;
 			nu->bp= callocstructN(BPoint, 4*4, "addNurbprim6");
 			nu->flagu= 0;
 			nu->flagv= 0;
@@ -3606,7 +3710,7 @@ Nurb *addNurbprim(int type, int stype, int newname)
 
 			nu= addNurbprim(4, 1, 0);  /* circle */
 			nu->resolu= 32;
-			nu->flag= ME_SMOOTH;
+			nu->flag= CU_SMOOTH;
 			BLI_addtail(&editNurb, nu); /* temporal for extrude and translate */
 			vec[0]=vec[1]= 0.0;
 			vec[2]= -G.vd->grid;
@@ -3640,7 +3744,7 @@ Nurb *addNurbprim(int type, int stype, int newname)
 			nu->orderu= 3;
 			nu->resolu= 24;
 			nu->resolv= 32;
-			nu->flag= ME_SMOOTH;
+			nu->flag= CU_SMOOTH;
 			nu->bp= callocstructN(BPoint, 5, "addNurbprim6");
 			nu->flagu= 0;
 			bp= nu->bp;
@@ -3659,7 +3763,7 @@ Nurb *addNurbprim(int type, int stype, int newname)
 			makeknots(nu, 1, nu->flagu>>1);
 
 			BLI_addtail(&editNurb, nu); /* temporal for spin */
-			spinNurb(0, 0);
+			spin_nurb(0, 0);
 
 			makeknots(nu, 2, nu->flagv>>1);
 
@@ -3684,9 +3788,9 @@ Nurb *addNurbprim(int type, int stype, int newname)
 			xzproj= 0;
 			nu->resolu= 24;
 			nu->resolv= 32;
-			nu->flag= ME_SMOOTH;
+			nu->flag= CU_SMOOTH;
 			BLI_addtail(&editNurb, nu); /* temporal for extrude and translate */
-			spinNurb(0, 0);
+			spin_nurb(0, 0);
 
 			BLI_remlink(&editNurb, nu);
 
@@ -3702,7 +3806,7 @@ Nurb *addNurbprim(int type, int stype, int newname)
 	}
 	
 	/* always do: */
-	nu->flag= ME_SMOOTH;
+	nu->flag= CU_SMOOTH;
 	
 	test2DNurb(nu);
 	
@@ -3767,8 +3871,8 @@ void add_primitiveCurve(int stype)
 	check_editmode(OB_CURVE);
 	
 	/* if no obedit: new object and enter editmode */
-	if(G.obedit==0) {
-		add_object(OB_CURVE);
+	if(G.obedit==NULL) {
+		add_object_draw(OB_CURVE);
 		base_init_from_view3d(BASACT, G.vd);
 		G.obedit= BASACT->object;
 		
@@ -3785,7 +3889,9 @@ void add_primitiveCurve(int stype)
 			default_curve_ipo(cu);
 		}
 	}
-	else cu= G.obedit->data;
+	else {
+		cu= G.obedit->data;
+	}
 
 	if(cu->flag & CU_3D) type &= ~CU_2D;
 
@@ -3795,9 +3901,14 @@ void add_primitiveCurve(int stype)
 	
 	BLI_addtail(&editNurb, nu);
 	makeDispList(G.obedit);
-	
+	curve_changes_other_objects(G.obedit);
+
 	countall();
 	allqueue(REDRAWALL, 0);
+	
+	/* if a new object was created, it stores it in Curve, for reload original data and undo */
+	if(newname) load_editNurb();
+	BIF_undo_push("Add primitive");
 }
 
 void add_primitiveNurb(int type)
@@ -3812,7 +3923,7 @@ void add_primitiveNurb(int type)
 
 	/* if no obedit: new object and enter editmode */
 	if(G.obedit==0) {
-		add_object(OB_SURF);
+		add_object_draw(OB_SURF);
 		base_init_from_view3d(BASACT, G.vd);
 		G.obedit= BASACT->object;
 		
@@ -3829,6 +3940,10 @@ void add_primitiveNurb(int type)
 
 	countall();
 	allqueue(REDRAWALL, 0);
+
+	/* if a new object was created, it stores it in Curve, for reload original data and undo */
+	if(newname) load_editNurb();
+	else BIF_undo_push("Add primitive");
 }
 
 
@@ -3865,70 +3980,8 @@ void clear_tilt()
 
 	makeBevelList(G.obedit);
 	allqueue(REDRAWVIEW3D, 0);
-}
-
-void clever_numbuts_curve()
-{
-	BPoint *bp;
-	BezTriple *bezt;
-	float old[3], delta[3];
-	int a;
+	BIF_undo_push("Clear tilt");
 	
-	if(lastnu==0) return;
-	if(lastnu->bp) {
-		bp= lastnu->bp;
-		a= lastnu->pntsu*lastnu->pntsv;
-		while(a--) {
-			if(bp->f1 & 1) break;
-			bp++;
-		}
-		if(bp==0) return;
-		
-		add_numbut(0, NUM|FLO, "LocX:", -G.vd->far, G.vd->far, bp->vec, 0);
-		add_numbut(1, NUM|FLO, "LocY:", -G.vd->far, G.vd->far, bp->vec+1, 0);
-		add_numbut(2, NUM|FLO, "LocZ:", -G.vd->far, G.vd->far, bp->vec+2, 0);
-		add_numbut(3, NUM|FLO, " W:", 0.0, 100.0, bp->vec+3, 0);
-		
-		do_clever_numbuts("Active BPoint", 4, REDRAW);
-		makeDispList(G.obedit);
-	}
-	else if(lastnu->bezt) {
-		bezt= lastnu->bezt;
-		a= lastnu->pntsu;
-		while(a--) {
-			if(BEZSELECTED(bezt)) break;
-			bezt++;
-		}
-		if(bezt==0) return;
-		
-		if(bezt->f2 & 1) {
-			add_numbut(0, NUM|FLO, "LocX:", -G.vd->far, G.vd->far, bezt->vec[1], 0);
-			add_numbut(1, NUM|FLO, "LocY:", -G.vd->far, G.vd->far, bezt->vec[1]+1, 0);
-			add_numbut(2, NUM|FLO, "LocZ:", -G.vd->far, G.vd->far, bezt->vec[1]+2, 0);
-			VECCOPY(old, bezt->vec[1]);
-			do_clever_numbuts("Active BezierPoint", 3, REDRAW);
-			
-			VecSubf(delta, bezt->vec[1], old);
-			VecAddf(bezt->vec[0], bezt->vec[0], delta);
-			VecAddf(bezt->vec[2], bezt->vec[2], delta);
-			makeDispList(G.obedit);
-		}
-		else if(bezt->f1 & 1) {
-			add_numbut(0, NUM|FLO, "LocX:", -G.vd->far, G.vd->far, bezt->vec[0], 0);
-			add_numbut(1, NUM|FLO, "LocY:", -G.vd->far, G.vd->far, bezt->vec[0]+1, 0);
-			add_numbut(2, NUM|FLO, "LocZ:", -G.vd->far, G.vd->far, bezt->vec[0]+2, 0);
-		
-			do_clever_numbuts("Active HandlePoint", 3, REDRAW);
-		}
-		else if(bezt->f3 & 1) {
-			add_numbut(0, NUM|FLO, "LocX:", -G.vd->far, G.vd->far, bezt->vec[2], 0);
-			add_numbut(1, NUM|FLO, "LocY:", -G.vd->far, G.vd->far, bezt->vec[2]+1, 0);
-			add_numbut(2, NUM|FLO, "LocZ:", -G.vd->far, G.vd->far, bezt->vec[2]+2, 0);
-		
-			do_clever_numbuts("Active HandlePoint", 3, REDRAW);
-		}
-	}
-
 }
 
 int bezt_compare (const void *e1, const void *e2)
@@ -3954,3 +4007,64 @@ int bezt_compare (const void *e1, const void *e2)
 
 	return 0;
 }
+
+
+/* **************** undo for curves ************** */
+
+static void undoCurve_to_editCurve(void *lbv)
+{
+	ListBase *lb= lbv;
+	Nurb *nu, *newnu;
+	int nr, lastnunr= 0;
+
+	/* we try to restore lastnu too, for buttons */
+	for(nu= editNurb.first; nu; nu = nu->next, lastnunr++) if(nu==lastnu) break;
+	
+	freeNurblist(&editNurb);
+
+	/* copy  */
+	nu= lb->first;
+	while(nu) {
+		newnu= duplicateNurb(nu);
+		BLI_addtail(&editNurb, newnu);
+		nu= nu->next;
+	}
+	/* restore */
+	for(nr=0, lastnu= editNurb.first; lastnu; lastnu = lastnu->next, nr++) if(nr==lastnunr) break;
+	
+}
+
+static void *editCurve_to_undoCurve(void)
+{
+	ListBase *lb;
+	Nurb *nu, *newnu;
+
+	lb= MEM_callocN(sizeof(ListBase), "listbase undo");
+	
+	/* copy  */
+	nu= editNurb.first;
+	while(nu) {
+		newnu= duplicateNurb(nu);
+		BLI_addtail(lb, newnu);
+		nu= nu->next;
+	}
+	return lb;
+}
+
+static void free_undoCurve(void *lbv)
+{
+	ListBase *lb= lbv;
+	
+	freeNurblist(lb);
+	MEM_freeN(lb);
+}
+
+/* and this is all the undo system needs to know */
+void undo_push_curve(char *name)
+{
+	undo_editmode_push(name, free_undoCurve, undoCurve_to_editCurve, editCurve_to_undoCurve);
+}
+
+
+
+/***/
