@@ -1,5 +1,5 @@
 /**
- * $Id: editdeform.c,v 1.9 2004/03/28 08:46:35 zuster Exp $
+ * $Id: editdeform.c,v 1.16 2005/03/28 19:55:04 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -29,12 +29,14 @@
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  * Creator-specific support for vertex deformation groups
+ * Added: apply deform function (ton)
  */
 
 #include <string.h>
 
 #include "MEM_guardedalloc.h"
 
+#include "DNA_curve_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
@@ -44,9 +46,11 @@
 
 #include "BKE_global.h"
 #include "BKE_deform.h"
+#include "BKE_displist.h"
 #include "BKE_mesh.h"
 
 #include "BIF_editdeform.h"
+#include "BIF_editmesh.h"
 #include "BIF_toolbox.h"
 
 #include "BSE_edit.h"
@@ -55,6 +59,11 @@
 #include <config.h>
 #endif
 
+/* ----------------- function prototypes ---------------- */
+void remove_vert_def_nr (Object *, int , int );
+void add_vert_defnr (Object *, int , int vertnum, float , int );
+
+
 void sel_verts_defgroup (int select)
 {
 	EditMesh *em = G.editMesh;
@@ -73,8 +82,9 @@ void sel_verts_defgroup (int select)
 			if (eve->totweight){
 				for (i=0; i<eve->totweight; i++){
 					if (eve->dw[i].def_nr == (ob->actdef-1)){
-						if (select) eve->f |= 1;
-						else eve->f &= ~1;
+						if (select) eve->f |= SELECT;
+						else eve->f &= ~SELECT;
+						
 						break;
 					}
 				}
@@ -85,7 +95,9 @@ void sel_verts_defgroup (int select)
 		break;
 	}
 	countall();
-
+	/* this has to be called, because this function operates on vertices only */
+	if(select) EM_select_flush();	// vertices to edges/faces
+	else EM_deselect_flush();
 }
 
 MDeformWeight *verify_defweight (MDeformVert *dv, int defgroup)
@@ -197,48 +209,6 @@ void create_dverts(Mesh *me)
 	}
 }
 
-int  get_defgroup_num (Object *ob, bDeformGroup	*dg)
-{
-	/* Fetch the location of this deform group
-	 * within the linked list of deform groups.
-	 * (this number is stored in the deform
-	 * weights of the deform verts to link them
-	 * to this deform group) deform deform
-	 * deform blah blah deform
-	 */
-
-	bDeformGroup	*eg;
-	int def_nr;
-
-	eg = ob->defbase.first;
-	def_nr = 0;
-
-	/* loop through all deform groups
-	 */
-	while (eg != NULL){
-
-		/* if the current deform group is
-		 * the one we are after, return
-		 * def_nr
-		 */
-		if (eg == dg){
-			break;
-		}
-		++def_nr;
-		eg = eg->next;
-	}
-
-	/* if there was no deform group found then
-	 * return -1 (should set up a nice symbolic
-	 * constant for this)
-	 */
-	if (eg == NULL) return -1;
-	
-	return def_nr;
-    
-}
-
-
 void remove_vert_def_nr (Object *ob, int def_nr, int vertnum)
 {
 	/* This routine removes the vertex from the deform
@@ -429,7 +399,7 @@ void assign_verts_defgroup (void)
 
 	dg=BLI_findlink(&ob->defbase, ob->actdef-1);
 	if (!dg){
-		error ("No vertex group active");
+		error ("No vertex group is active");
 		return;
 	}
 
@@ -522,7 +492,7 @@ void remove_verts_defgroup (int allverts)
 
 	dg=BLI_findlink(&ob->defbase, ob->actdef-1);
 	if (!dg){
-		error ("No vertex group active");
+		error ("No vertex group is active");
 		return;
 	}
 
@@ -583,22 +553,6 @@ void verify_defgroups (Object *ob)
 	}
 }
 
-bDeformGroup *get_named_vertexgroup(Object *ob, char *name)
-{
-	/* return a pointer to the deform group with this name
-	 * or return NULL otherwise.
-	 */
-	bDeformGroup *curdef;
-
-	for (curdef = ob->defbase.first; curdef; curdef=curdef->next){
-		if (!strcmp(curdef->name, name)){
-			return curdef;
-		}
-	}
-	return NULL;
-}
-
-
 void unique_vertexgroup_name (bDeformGroup *dg, Object *ob)
 {
 	char		tempname[64];
@@ -645,3 +599,35 @@ void unique_vertexgroup_name (bDeformGroup *dg, Object *ob)
 		}
 	}	
 }
+
+/* ******************* other deform edit stuff ********** */
+
+void object_apply_deform(Object *ob)
+{
+	char *err= NULL;
+	
+	if(ob->type==OB_MESH) {
+		Mesh *me= ob->data;
+		if(me->id.us>1) {
+			err= "Can't apply deformation to Mesh with other users";
+		} else {
+			mesh_modifier(ob, 's'); // start
+			mesh_modifier(ob, 'a'); // apply and end
+			freedisplist(&ob->disp);
+		}
+	}
+	else if (ob->type==OB_CURVE || ob->type==OB_SURF) {
+		Curve *cu= ob->data;
+		if(cu->id.us>1) {
+			err= "Can't apply deformation to Curve with other users";
+		} else {
+			curve_modifier(ob, 's'); // start
+			curve_modifier(ob, 'a'); // apply and end
+			freedisplist(&ob->disp);
+		}
+	}
+
+	if(err) error(err);
+
+}
+
