@@ -1,5 +1,5 @@
 /**
- * $Id: editface.c,v 1.33 2004/11/07 13:15:04 blendix Exp $
+ * $Id: editface.c,v 1.40 2005/05/12 19:17:12 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -38,9 +38,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
 #include "MEM_guardedalloc.h"
 
 #include "BLI_blenlib.h"
@@ -57,6 +54,7 @@
 #include "DNA_object_types.h"
 #include "DNA_space_types.h"
 #include "DNA_screen_types.h"
+#include "DNA_scene_types.h"
 #include "DNA_view3d_types.h"
 #include "DNA_userdef_types.h"
 
@@ -87,7 +85,6 @@
 
 #include "mydevice.h"
 #include "blendef.h"
-#include "render.h"
 #include "butspace.h"
 
 #include "TPT_DependKludge.h"
@@ -652,69 +649,115 @@ void hide_tface()
 	
 }
 
-void select_linked_tfaces()
+void select_linked_tfaces(int mode)
 {
+	Object *ob;
 	Mesh *me;
-	TFace *tface;
-	MFace *mface;
+	TFace *tf;
+	MFace *mf;
 	int a, doit=1, mark=0;
-	char *cpmain;
-	
-	me= get_mesh(OBACT);
+	char *cpmain, *linkflag;
+	short mval[2];
+	unsigned int index=0;
+
+	ob = OBACT;
+	me = get_mesh(ob);
 	if(me==0 || me->tface==0 || me->totface==0) return;
 
+	if (mode==0 || mode==1) {
+		if (!(ob->lay & G.vd->lay))
+			error("The active object is not in this layer");
+			
+		getmouseco_areawin(mval);
+		if (!face_pick(me, mval[0], mval[1], &index)) return;
+	}
+
 	if(me->medge) {
-		select_linked_tfaces_with_seams();
+		select_linked_tfaces_with_seams(mode, me, index);
 		return;
 	}
-	
+
 	cpmain= MEM_callocN(me->totvert, "cpmain");
+	linkflag= MEM_callocN(sizeof(char)*me->totface, "linkflaguv");
+
+	if (mode==0 || mode==1) {
+		/* only put face under cursor in array */
+		mf= ((MFace*)me->mface) + index;
+		cpmain[mf->v1]= cpmain[mf->v2]= cpmain[mf->v3]= 1;
+		if (mf->v4) cpmain[mf->v4]= 1;
+		linkflag[index]= 1;
+	}
+	else {
+		/* fill array by selection */
+		tf= me->tface;
+		mf= me->mface;
+		for(a=0; a<me->totface; a++, tf++, mf++) {
+			if(tf->flag & TF_HIDE);
+			else if(tf->flag & TF_SELECT) {
+				if(mf->v3) {
+					cpmain[mf->v1]= 1;
+					cpmain[mf->v2]= 1;
+					cpmain[mf->v3]= 1;
+					if(mf->v4) cpmain[mf->v4]= 1;
+					linkflag[a]= 1;
+				}
+			}
+		}
+	}
 	
 	while(doit) {
 		doit= 0;
 		
-		/* select connected: fill array */
-		tface= me->tface;
-		mface= me->mface;
-		a= me->totface;
-		while(a--) {
-			if(tface->flag & TF_HIDE);
-			else if(tface->flag & TF_SELECT) {
-				if(mface->v3) {
-					cpmain[mface->v1]= 1;
-					cpmain[mface->v2]= 1;
-					cpmain[mface->v3]= 1;
-					if(mface->v4) cpmain[mface->v4]= 1;
-				}
-			}
-			tface++; mface++;
-		}
-		
-		/* reverse: using array select the faces */
-
-		tface= me->tface;
-		mface= me->mface;
-		a= me->totface;
-		while(a--) {
-			if(tface->flag & TF_HIDE);
-			else if(mface->v3 && ((tface->flag & TF_SELECT)==0)) {
+		/* expand selection */
+		tf= me->tface;
+		mf= me->mface;
+		for(a=0; a<me->totface; a++, tf++, mf++) {
+			if(tf->flag & TF_HIDE);
+			else if(mf->v3 && !linkflag[a]) {
 				mark= 0;
 
-				if(cpmain[mface->v1] || cpmain[mface->v2] || cpmain[mface->v3])
+				if(cpmain[mf->v1] || cpmain[mf->v2] || cpmain[mf->v3])
 					mark= 1;
-				else if(mface->v4 && cpmain[mface->v4])
+				else if(mf->v4 && cpmain[mf->v4])
 					mark= 1;
 
 				if(mark) {
-					tface->flag |= TF_SELECT;
+					linkflag[a]= 1;
+					cpmain[mf->v1]= cpmain[mf->v2]= cpmain[mf->v3]= 1;
+					if(mf->v4) cpmain[mf->v4]= 1;
 					doit= 1;
 				}
 			}
-			tface++; mface++;
 		}
 		
 	}
+
+	if(mode==0 || mode==2) {
+		for(a=0, tf=me->tface; a<me->totface; a++, tf++)
+			if(linkflag[a])
+				tf->flag |= TF_SELECT;
+			else
+				tf->flag &= ~TF_SELECT;
+	}
+	else if(mode==1) {
+		for(a=0, tf=me->tface; a<me->totface; a++, tf++)
+			if(linkflag[a] && (tf->flag & TF_SELECT))
+				break;
+
+		if (a<me->totface) {
+			for(a=0, tf=me->tface; a<me->totface; a++, tf++)
+				if(linkflag[a])
+					tf->flag &= ~TF_SELECT;
+		}
+		else {
+			for(a=0, tf=me->tface; a<me->totface; a++, tf++)
+				if(linkflag[a])
+					tf->flag |= TF_SELECT;
+		}
+	}
+	
 	MEM_freeN(cpmain);
+	MEM_freeN(linkflag);
 	
 	BIF_undo_push("Select linked UV face");
 	allqueue(REDRAWVIEW3D, 0);
@@ -847,25 +890,72 @@ void rotate_uv_tface()
 	allqueue(REDRAWIMAGE, 0);
 }
 
+void minmax_tface(float *min, float *max)
+{
+	Object *ob;
+	Mesh *me;
+	MFace *mf;
+	TFace *tf;
+	MVert *mv;
+	int a;
+	float vec[3], bmat[3][3];
+	
+	ob = OBACT;
+	if (ob==0) return;
+	me= get_mesh(ob);
+	if(me==0 || me->tface==0) return;
+	
+	Mat3CpyMat4(bmat, ob->obmat);
+
+	mv= me->mvert;
+	mf= me->mface;
+	tf= me->tface;
+	for (a=me->totface; a>0; a--, mf++, tf++) {
+		if (!mf->v3 || tf->flag & TF_HIDE || !(tf->flag & TF_SELECT))
+			continue;
+
+		VECCOPY(vec, (mv+mf->v1)->co);
+		Mat3MulVecfl(bmat, vec);
+		VecAddf(vec, vec, ob->obmat[3]);
+		DO_MINMAX(vec, min, max);		
+
+		VECCOPY(vec, (mv+mf->v2)->co);
+		Mat3MulVecfl(bmat, vec);
+		VecAddf(vec, vec, ob->obmat[3]);
+		DO_MINMAX(vec, min, max);		
+
+		VECCOPY(vec, (mv+mf->v3)->co);
+		Mat3MulVecfl(bmat, vec);
+		VecAddf(vec, vec, ob->obmat[3]);
+		DO_MINMAX(vec, min, max);		
+
+		if (mf->v4) {
+			VECCOPY(vec, (mv+mf->v4)->co);
+			Mat3MulVecfl(bmat, vec);
+			VecAddf(vec, vec, ob->obmat[3]);
+			DO_MINMAX(vec, min, max);
+		}
+	}
+}
+
 /**
  * Returns the face under the give position in screen coordinates.
  * Code extracted from face_select routine.
  * Question: why is all of the backbuffer drawn?
  * We're only interested in one pixel!
  * @author	Maarten Gribnau
- * @param	me	the mesh with the faces to be picked
- * @param	x	the x-coordinate to pick at
- * @param	y	the y-coordinate to pick at
- * @return the face under the cursor (-1 if there was no face found)
+ * @param	me		the mesh with the faces to be picked
+ * @param	x		the x-coordinate to pick at
+ * @param	y		the y-coordinate to pick at
+ * @param	index	the index of the face
+ * @return 1 if found, 0 if none found
  */
-int face_pick(Mesh *me, short x, short y)
+int face_pick(Mesh *me, short x, short y, unsigned int *index)
 {
 	unsigned int col;
-	int index;
 
-	if (me==0 || me->tface==0) {
-		return -1;
-	}
+	if (me==0 || me->tface==0)
+		return 0;
 
 	/* Have OpenGL draw in the back buffer with color coded face indices */
 	if (curarea->win_swap==WIN_EQUAL) {
@@ -888,11 +978,13 @@ int face_pick(Mesh *me, short x, short y)
 		SWITCH_INT(col);
 	}
 	/* Convert the color back to a face index */
-	index = framebuffer_to_index(col);
-	if (col==0 || index<=0 || index>me->totface) {
-		return -1;
-	}
-	return (index-1);
+	*index = framebuffer_to_index(col);
+	if (col==0 || (*index)<=0 || (*index)>me->totface)
+		return 0;
+
+	(*index)--;
+	
+	return 1;
 }
 
 void face_select()
@@ -902,7 +994,7 @@ void face_select()
 	TFace *tface, *tsel;
 	MFace *msel;
 	short mval[2];
-	int a, index;
+	unsigned int a, index;
 
 	/* Get the face under the cursor */
 	ob = OBACT;
@@ -911,8 +1003,7 @@ void face_select()
 	}
 	me = get_mesh(ob);
 	getmouseco_areawin(mval);
-	index = face_pick(me, mval[0], mval[1]);
-	if (index==-1) return;
+	if (!face_pick(me, mval[0], mval[1], &index)) return;
 	
 	tsel= (((TFace*)me->tface)+index);
 	msel= (((MFace*)me->mface)+index);
@@ -1111,7 +1202,8 @@ void set_faceselect()	/* toggle */
 	Mesh *me = 0;
 	
 	scrarea_queue_headredraw(curarea);
-
+	if(ob->id.lib) return;
+	
 	if(G.f & G_FACESELECT) G.f &= ~G_FACESELECT;
 	else {
 		if (ob && ob->type == OB_MESH) G.f |= G_FACESELECT;
@@ -1154,7 +1246,7 @@ void set_faceselect()	/* toggle */
  * @param	org		origin of the view ray.
  * @param	dir		direction of the view ray.
  */
-void get_pick_ray(short x, short y, float org[3], float dir[3])
+static void get_pick_ray(short x, short y, float org[3], float dir[3])
 {
 	double mvmatrix[16];
 	double projmatrix[16];
@@ -1190,7 +1282,7 @@ void get_pick_ray(short x, short y, float org[3], float dir[3])
 }
 
 
-int triangle_ray_intersect(float tv0[3], float tv1[3], float tv2[3], float org[3], float dir[3], float uv[2])
+static int triangle_ray_intersect(float tv0[3], float tv1[3], float tv2[3], float org[3], float dir[3], float uv[2])
 {
 	float v1v0[3];
 	float v2v0[3];
@@ -1283,7 +1375,7 @@ int triangle_ray_intersect(float tv0[3], float tv1[3], float tv2[3], float org[3
  * @param	v4		vertex 4 coordinates.
  * @return	number of vertices of this face
  */
-int face_get_vertex_coordinates(Mesh* mesh, TFace* face, float v1[3], float v2[3], float v3[3], float v4[3])
+static int face_get_vertex_coordinates(Mesh* mesh, TFace* face, float v1[3], float v2[3], float v3[3], float v4[3])
 {
 	int num_vertices;
 	MVert *mv;
@@ -1316,7 +1408,7 @@ int face_get_vertex_coordinates(Mesh* mesh, TFace* face, float v1[3], float v2[3
  * @param	u		(u,v) coordinate.
  * @param	v		(u,v) coordinate.
  */
-void face_get_uv(TFace* face, int v1, int v2, int v3, float a, float b, float* u, float* v)
+static void face_get_uv(TFace* face, int v1, int v2, int v3, float a, float b, float* u, float* v)
 {
 	float uv01[2], uv21[2];
 
@@ -1359,7 +1451,7 @@ void face_get_uv(TFace* face, int v1, int v2, int v3, float a, float b, float* u
  *			0 == no intersection, (u,v) invalid
  *			1 == intersection, (u,v) valid
  */
-int face_pick_uv(Object* object, Mesh* mesh, TFace* face, short x, short y, float* u, float* v)
+static int face_pick_uv(Object* object, Mesh* mesh, TFace* face, short x, short y, float* u, float* v)
 {
 	float org[3], dir[3];
 	float ab[2];
@@ -1412,7 +1504,7 @@ void face_draw()
 	Image *img=NULL, *img_old = NULL;
 	IMG_BrushPtr brush;
 	IMG_CanvasPtr canvas = 0;
-	int rowBytes, face_index;
+	unsigned int rowBytes, face_index;
 	char *warn_packed_file = 0;
 	float uv[2], uv_old[2];
 	extern VPaint Gvp;
@@ -1447,8 +1539,7 @@ void face_draw()
 		if ((xy[0] != xy_old[0]) || (xy[1] != xy_old[1])) {
 
 			/* Get face to draw on */
-			face_index = face_pick(me, xy[0], xy[1]);
-			if (face_index == -1) face = NULL;
+			if (!face_pick(me, xy[0], xy[1], &face_index)) face = NULL;
 			else face = (((TFace*)me->tface)+face_index);
 
 			/* Check if this is another face. */
