@@ -1,5 +1,5 @@
 /**
- * $Id: editkey.c,v 1.19 2004/12/11 12:41:51 ton Exp $
+ * $Id: editkey.c,v 1.21 2005/03/19 21:08:12 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,7 +40,6 @@
 #include <unistd.h>
 #else
 #include <io.h>
-#include "BLI_winstuff.h"
 #endif   
 #include "MEM_guardedalloc.h"
 
@@ -89,6 +88,10 @@
 
 extern ListBase editNurb; /* in editcurve.c */
 
+
+/* local prototypes ------------------ */
+void make_rvk_slider(uiBlock *, Key *, int , int , int , int , int );  /* used in drawaction.c too */
+
 /* temporary storage for slider values */
 float meshslidervals[64] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
