@@ -44,9 +44,6 @@ editmesh_lib: generic (no UI, no menus) operations/evaluators for editmesh data
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
 #include "MEM_guardedalloc.h"
 
 
@@ -178,7 +175,8 @@ int EM_nfaces_selected(void)
 	return count;
 }
 
-int EM_nedges(void)
+#if 0
+static int EM_nedges(void)
 {
 	EditMesh *em = G.editMesh;
 	EditEdge *eed;
@@ -187,6 +185,7 @@ int EM_nedges(void)
 	for (eed= em->edges.first; eed; eed= eed->next) count++;
 	return count;
 }
+#endif
 
 int EM_nvertices_selected(void)
 {
@@ -412,6 +411,9 @@ void EM_selectmode_set(void)
 
 	if(G.scene->selectmode & SCE_SELECT_VERTEX) {
 		/* vertices -> edges -> faces */
+		for (eed= em->edges.first; eed; eed= eed->next) eed->f &= ~SELECT;
+		for (efa= em->faces.first; efa; efa= efa->next) efa->f &= ~SELECT;
+		
 		EM_select_flush();
 	}
 	else if(G.scene->selectmode & SCE_SELECT_EDGE) {
@@ -455,37 +457,65 @@ void EM_hide_reset(void)
 
 /* ********  EXTRUDE ********* */
 
-static void set_edge_directions(void)
+static void add_normal_aligned(float *nor, float *add)
+{
+	if( INPR(nor, add) < 0.0 ) 
+		VecSubf(nor, nor, add);
+	else
+		VecAddf(nor, nor, add);
+}
+
+static void set_edge_directions_f2(int val)
 {
 	EditMesh *em= G.editMesh;
-	EditFace *efa= em->faces.first;
+	EditFace *efa;
+	int do_all= 1;
 	
-	while(efa) {
-		// non selected face
-		if(efa->f== 0) {
-			if(efa->e1->f) {
+	/* edge directions are used for extrude, to detect direction of edges that make new faces */
+	/* we have set 'f2' flags in edges that need to get a direction set (e.g. get new face) */
+	/* the val argument differs... so we need it as arg */
+	
+	for(efa= em->faces.first; efa; efa= efa->next) {
+		if(efa->f & SELECT) {
+			do_all= 0;
+			if(efa->e1->f2<val) {
 				if(efa->e1->v1 == efa->v1) efa->e1->dir= 0;
 				else efa->e1->dir= 1;
 			}
-			if(efa->e2->f) {
+			if(efa->e2->f2<val) {
 				if(efa->e2->v1 == efa->v2) efa->e2->dir= 0;
 				else efa->e2->dir= 1;
 			}
-			if(efa->e3->f) {
+			if(efa->e3->f2<val) {
 				if(efa->e3->v1 == efa->v3) efa->e3->dir= 0;
 				else efa->e3->dir= 1;
 			}
-			if(efa->e4 && efa->e4->f) {
+			if(efa->e4 && efa->e4->f2<val) {
 				if(efa->e4->v1 == efa->v4) efa->e4->dir= 0;
 				else efa->e4->dir= 1;
 			}
 		}
-		efa= efa->next;
 	}	
+	/* ok, no faces done... then we at least set it for exterior edges */
+	if(do_all) {
+		for(efa= em->faces.first; efa; efa= efa->next) {
+			if(efa->e1->v1 == efa->v1) efa->e1->dir= 0;
+			else efa->e1->dir= 1;
+			if(efa->e2->v1 == efa->v2) efa->e2->dir= 0;
+			else efa->e2->dir= 1;
+			if(efa->e3->v1 == efa->v3) efa->e3->dir= 0;
+			else efa->e3->dir= 1;
+			if(efa->e4) {
+				if(efa->e4->v1 == efa->v4) efa->e4->dir= 0;
+				else efa->e4->dir= 1;
+			}
+		}	
+	}
 }
 
 /* individual face extrude */
-short extrudeflag_face_indiv(short flag)
+/* will use vertex normals for extrusion directions, so *nor is unaffected */
+short extrudeflag_face_indiv(short flag, float *nor)
 {
 	EditMesh *em = G.editMesh;
 	EditVert *eve, *v1, *v2, *v3, *v4;
@@ -574,17 +604,21 @@ short extrudeflag_face_indiv(short flag)
 
 
 /* extrudes individual edges */
-short extrudeflag_edges_indiv(short flag) 
+/* nor is filled with constraint vector */
+short extrudeflag_edges_indiv(short flag, float *nor) 
 {
 	EditMesh *em = G.editMesh;
 	EditVert *eve;
 	EditEdge *eed;
 	EditFace *efa;
-	float nor[3]={0.0, 0.0, 0.0};
 	
 	for(eve= em->verts.first; eve; eve= eve->next) eve->vn= NULL;
-
-	set_edge_directions();
+	for(eed= em->edges.first; eed; eed= eed->next) {
+		eed->vn= NULL;
+		eed->f2= ((eed->f & flag)!=0);
+	}
+	
+	set_edge_directions_f2(2);
 
 	/* sample for next loop */
 	for(efa= em->faces.first; efa; efa= efa->next) {
@@ -605,7 +639,7 @@ short extrudeflag_edges_indiv(short flag)
 			/* for transform */
 			if(eed->vn) {
 				efa= (EditFace *)eed->vn;
-				if(efa->f & SELECT) VecAddf(nor, nor, efa->n);
+				if(efa->f & SELECT) add_normal_aligned(nor, efa->n);
 			}
 		}
 	}
@@ -616,7 +650,6 @@ short extrudeflag_edges_indiv(short flag)
 	for(eve= em->verts.last; eve; eve= eve->prev) {
 		if(eve->vn) {
 			eve->vn->f |= flag;
-			VECCOPY(eve->vn->no, nor); // transform() uses it
 		}
 	}
 
@@ -624,12 +657,12 @@ short extrudeflag_edges_indiv(short flag)
 		if(eed->v1->f & eed->v2->f & flag) eed->f |= flag;
 	}
 	
-	if(nor[0]==0.0 && nor[1]==0.0 && nor[2]==0.0) return 'h'; // h is grab, for correct undo print
-	return 'n';
+	if(nor[0]==0.0 && nor[1]==0.0 && nor[2]==0.0) return 'g'; // g is grab
+	return 'n';  // n is for normal constraint
 }
 
 /* extrudes individual vertices */
-short extrudeflag_verts_indiv(short flag) 
+short extrudeflag_verts_indiv(short flag, float *nor) 
 {
 	EditMesh *em = G.editMesh;
 	EditVert *eve;
@@ -648,13 +681,13 @@ short extrudeflag_verts_indiv(short flag)
 
 	for(eve= em->verts.last; eve; eve= eve->prev) if(eve->vn) eve->vn->f |= flag;
 
-	return 'h';	// h is grab, for correct undo print
+	return 'g';	// g is grab
 }
 
 
 /* this is actually a recode of extrudeflag(), using proper edge/face select */
 /* hurms, doesnt use 'flag' yet, but its not called by primitive making stuff anyway */
-static short extrudeflag_edge(short flag)
+static short extrudeflag_edge(short flag, float *nor)
 {
 	/* all select edges/faces: extrude */
 	/* old select is cleared, in new ones it is set */
@@ -662,7 +695,6 @@ static short extrudeflag_edge(short flag)
 	EditVert *eve, *nextve;
 	EditEdge *eed, *nexted;
 	EditFace *efa, *nextfa;
-	float nor[3]={0.0, 0.0, 0.0};
 	short del_old= 0;
 	
 	if(G.obedit==0 || get_mesh(G.obedit)==0) return 0;
@@ -687,8 +719,8 @@ static short extrudeflag_edge(short flag)
 	}
 
 	for(eed= em->edges.first; eed; eed= eed->next) {
-		eed->f1= 0; // amount of selected faces
-		eed->f2= 0; // amount of unselected faces
+		eed->f1= 0; // amount of unselected faces
+		eed->f2= 0; // amount of selected faces
 		if(eed->f & SELECT) {
 			eed->v1->f1= 1; // we call this 'selected vertex' now
 			eed->v2->f1= 1;
@@ -697,17 +729,17 @@ static short extrudeflag_edge(short flag)
 	}
 	for(efa= em->faces.first; efa; efa= efa->next) {
 		if(efa->f & SELECT) {
-			efa->e1->f1++;
-			efa->e2->f1++;
-			efa->e3->f1++;
-			if(efa->e4) efa->e4->f1++;
-		}
-		else {
 			efa->e1->f2++;
 			efa->e2->f2++;
 			efa->e3->f2++;
 			if(efa->e4) efa->e4->f2++;
 		}
+		else {
+			efa->e1->f1++;
+			efa->e2->f1++;
+			efa->e3->f1++;
+			if(efa->e4) efa->e4->f1++;
+		}
 		// sample for next loop
 		efa->e1->vn= (EditVert *)efa;
 		efa->e2->vn= (EditVert *)efa;
@@ -715,18 +747,29 @@ static short extrudeflag_edge(short flag)
 		if(efa->e4) efa->e4->vn= (EditVert *)efa;
 	}
 	
-	set_edge_directions();
+	set_edge_directions_f2(2);
 	
+	/* step 1.5: if *one* selected face has edge with unselected face; remove old selected faces */
+	for(efa= em->faces.last; efa; efa= efa->prev) {
+		if(efa->f & SELECT) {
+			if(efa->e1->f1 || efa->e2->f1 || efa->e3->f1 || (efa->e4 && efa->e4->f1)) {
+				del_old= 1;
+				break;
+			}
+		}
+	}
+				
 	/* step 2: make new faces from edges */
 	for(eed= em->edges.last; eed; eed= eed->prev) {
 		if(eed->f & SELECT) {
-			if(eed->f1<2) {
+			if(eed->f2<2) {
 				if(eed->v1->vn==NULL)
 					eed->v1->vn= addvertlist(eed->v1->co);
 				if(eed->v2->vn==NULL)
 					eed->v2->vn= addvertlist(eed->v2->co);
-					
-				if(eed->dir==1) addfacelist(eed->v1, eed->v2, eed->v2->vn, eed->v1->vn, (EditFace *)eed->vn, NULL);
+				
+				/* if del_old, the preferred normal direction is exact opposite as for keep old faces */
+				if(eed->dir!=del_old) addfacelist(eed->v1, eed->v2, eed->v2->vn, eed->v1->vn, (EditFace *)eed->vn, NULL);
 				else addfacelist(eed->v2, eed->v1, eed->v1->vn, eed->v2->vn, (EditFace *)eed->vn, NULL);
 			}
 		}
@@ -740,17 +783,21 @@ static short extrudeflag_edge(short flag)
 			if(efa->v3->vn==NULL) efa->v3->vn= addvertlist(efa->v3->co);
 			if(efa->v4 && efa->v4->vn==NULL) efa->v4->vn= addvertlist(efa->v4->co);
 			
-			if(efa->v4)
-				addfacelist(efa->v1->vn, efa->v2->vn, efa->v3->vn, efa->v4->vn, efa, efa);
-			else
-				addfacelist(efa->v1->vn, efa->v2->vn, efa->v3->vn, NULL, efa, efa);
+			if(del_old==0) {	// keep old faces means flipping normal
+				if(efa->v4)
+					addfacelist(efa->v4->vn, efa->v3->vn, efa->v2->vn, efa->v1->vn, efa, efa);
+				else
+					addfacelist(efa->v3->vn, efa->v2->vn, efa->v1->vn, NULL, efa, efa);
+			}
+			else {
+				if(efa->v4)
+					addfacelist(efa->v1->vn, efa->v2->vn, efa->v3->vn, efa->v4->vn, efa, efa);
+				else
+					addfacelist(efa->v1->vn, efa->v2->vn, efa->v3->vn, NULL, efa, efa);
+			}
 	
-			/* if *one* selected face has edge with unselected face; remove old selected faces */
-			if(efa->e1->f2 || efa->e2->f2 || efa->e3->f2 || (efa->e4 && efa->e4->f2))
-				del_old= 1;
-				
 			/* for transform */
-			VecAddf(nor, nor, efa->n);
+			add_normal_aligned(nor, efa->n);
 		}
 	}
 	
@@ -814,17 +861,16 @@ static short extrudeflag_edge(short flag)
 	for(eve= em->verts.first; eve; eve= eve->next) {
 		if(eve->vn) {
 			eve->vn->f |= SELECT;
-			VECCOPY(eve->vn->no, nor);
 		}
 	}
 
 	EM_select_flush();
 
-	if(nor[0]==0.0 && nor[1]==0.0 && nor[2]==0.0) return 'h';
-	return 'n';
+	if(nor[0]==0.0 && nor[1]==0.0 && nor[2]==0.0) return 'g'; // grab
+	return 'n'; // normal constraint 
 }
 
-short extrudeflag_vert(short flag)
+short extrudeflag_vert(short flag, float *nor)
 {
 	/* all verts/edges/faces with (f & 'flag'): extrude */
 	/* from old verts, 'flag' is cleared, in new ones it is set */
@@ -832,7 +878,6 @@ short extrudeflag_vert(short flag)
 	EditVert *eve, *v1, *v2, *v3, *v4, *nextve;
 	EditEdge *eed, *e1, *e2, *e3, *e4, *nexted;
 	EditFace *efa, *efa2, *nextvl;
-	float nor[3]={0.0, 0.0, 0.0};
 	short sel=0, del_old= 0;
 
 	if(G.obedit==0 || get_mesh(G.obedit)==0) return 0;
@@ -900,7 +945,7 @@ short extrudeflag_vert(short flag)
 		efa= efa->next;
 	}
 
-	set_edge_directions();
+	set_edge_directions_f2(3);
 
 	/* the current state now is:
 		eve->f1==1: loose selected vertex 
@@ -946,6 +991,16 @@ short extrudeflag_vert(short flag)
 	   if del_old==0 the extrude creates a volume.
 	*/
 	
+	 /* find if we delete old faces */
+	for(eed= em->edges.first; eed; eed= eed->next) {
+		if( (eed->f2==1 || eed->f2==2) ) {
+			if(eed->f1==2) {
+				del_old= 1;
+				break;
+			}
+		}
+	}
+	
 	eed= em->edges.last;
 	while(eed) {
 		nexted= eed->prev;
@@ -954,9 +1009,9 @@ short extrudeflag_vert(short flag)
 			eed->v2->f |= 128;
 		}
 		if( (eed->f2==1 || eed->f2==2) ) {
-			if(eed->f1==2) del_old= 1;
 			
-			if(eed->dir==1) efa2= addfacelist(eed->v1, eed->v2, eed->v2->vn, eed->v1->vn, NULL, NULL);
+			/* if del_old, the preferred normal direction is exact opposite as for keep old faces */
+			if(eed->dir!=del_old) efa2= addfacelist(eed->v1, eed->v2, eed->v2->vn, eed->v1->vn, NULL, NULL);
 			else efa2= addfacelist(eed->v2, eed->v1, eed->v1->vn, eed->v2->vn, NULL, NULL);
 			
 			if(eed->vn) {
@@ -1000,10 +1055,13 @@ short extrudeflag_vert(short flag)
 			v3= efa->v3->vn;
 			if(efa->v4) v4= efa->v4->vn; else v4= 0;
 			
-			efa2= addfacelist(v1, v2, v3, v4, efa, efa); /* hmm .. not sure about edges here */
+			if(del_old==0)	// if we keep old, we flip normal
+				efa2= addfacelist(v3, v2, v1, v4, efa, efa); /* hmm .. not sure about edges here */
+			else
+				efa2= addfacelist(v1, v2, v3, v4, efa, efa); /* hmm .. not sure about edges here */
 			
 			/* for transform */
-			VecAddf(nor, nor, efa->n);
+			add_normal_aligned(nor, efa->n);
 
 			if(del_old) {
 				BLI_remlink(&em->faces, efa);
@@ -1033,9 +1091,6 @@ short extrudeflag_vert(short flag)
 			}
 		}
 		if(eve) {
-			if(eve->f & flag) {
-				VECCOPY(eve->no, nor);
-			}
 			eve->f &= ~128;
 		}
 		eve= nextve;
@@ -1043,17 +1098,17 @@ short extrudeflag_vert(short flag)
 	// since its vertex select mode now, it also deselects higher order
 	EM_selectmode_flush();
 
-	if(nor[0]==0.0 && nor[1]==0.0 && nor[2]==0.0) return 'h'; // h is grab, for correct undo print
+	if(nor[0]==0.0 && nor[1]==0.0 && nor[2]==0.0) return 'g'; // g is grab, for correct undo print
 	return 'n';
 }
 
 /* generic extrude */
-short extrudeflag(short flag)
+short extrudeflag(short flag, float *nor)
 {
 	if(G.scene->selectmode & SCE_SELECT_VERTEX)
-		return extrudeflag_vert(flag);
+		return extrudeflag_vert(flag, nor);
 	else 
-		return extrudeflag_edge(flag);
+		return extrudeflag_edge(flag, nor);
 		
 }
 
@@ -1164,14 +1219,19 @@ void adduplicateflag(int flag)
 void delfaceflag(int flag)
 {
 	EditMesh *em = G.editMesh;
-	/* delete all faces with 'flag', including edges and loose vertices */
+	/* delete all faces with 'flag', including loose edges and loose vertices */
+	/* this is maybe a bit weird, but this function is used for 'split' and 'separate' */
 	/* in remaining vertices/edges 'flag' is cleared */
 	EditVert *eve,*nextve;
 	EditEdge *eed, *nexted;
 	EditFace *efa,*nextvl;
 
-	for(eed= em->edges.first; eed; eed= eed->next) eed->f2= 0;
-
+	/* to detect loose edges, we put f2 flag on 1 */
+	for(eed= em->edges.first; eed; eed= eed->next) {
+		if(eed->f & flag) eed->f2= 1;
+		else eed->f2= 0;
+	}
+	
 	/* delete faces */
 	efa= em->faces.first;
 	while(efa) {
@@ -1676,3 +1736,5 @@ void EM_fgon_flags(void)
 }
 
 
+
+
