@@ -1,5 +1,5 @@
 /**
- * $Id: editseq.c,v 1.21 2004/11/13 12:55:59 ton Exp $
+ * $Id: editseq.c,v 1.27 2005/05/20 12:18:11 jesterking Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,7 +42,6 @@
 #include <unistd.h>
 #else
 #include <io.h>
-#include "BLI_winstuff.h"
 #endif
 #include <sys/types.h>
 
@@ -70,6 +69,7 @@
 #include "BKE_image.h"
 #include "BKE_library.h"
 #include "BKE_main.h"
+#include "BKE_scene.h"
 
 #include "BIF_space.h"
 #include "BIF_interface.h"
@@ -95,10 +95,8 @@
 #include "mydevice.h"
 
 Sequence *last_seq=0;
-char last_imagename[80]= "/";
-char last_sounddir[80]= "";
-
-/*  void transform_seq(int mode); already in BIF_editseq.h */
+char last_imagename[FILE_MAXDIR+FILE_MAXFILE]= "/";
+char last_sounddir[FILE_MAXDIR+FILE_MAXFILE]= "";
 
 #define SEQ_DESEL	~(SELECT+SEQ_LEFTSEL+SEQ_RIGHTSEL)
 
@@ -382,12 +380,12 @@ void mouse_select_seq(void)
 
 		if ((seq->type == SEQ_IMAGE) || (seq->type == SEQ_MOVIE)) {
 			if(seq->strip) {
-				strcpy(last_imagename, seq->strip->dir);
+				strncpy(last_imagename, seq->strip->dir, FILE_MAXDIR-1);
 			}
 		} else
 		if (seq->type == SEQ_SOUND) {
 			if(seq->strip) {
-				strcpy(last_sounddir, seq->strip->dir);
+				strncpy(last_sounddir, seq->strip->dir, FILE_MAXDIR-1);
 			}
 		}
 
@@ -443,7 +441,8 @@ static Sequence *alloc_sequence(int cfra, int machine)
 	seq->flag= SELECT;
 	seq->start= cfra;
 	seq->machine= machine;
-
+	seq->mul= 1.0;
+	
 	return seq;
 }
 
@@ -453,6 +452,7 @@ static Sequence *sfile_to_sequence(SpaceFile *sfile, int cfra, int machine, int
 	Strip *strip;
 	StripElem *se;
 	int totsel, a;
+	char name[160], rel[160];
 
 	/* are there selected files? */
 	totsel= 0;
@@ -481,18 +481,26 @@ static Sequence *sfile_to_sequence(SpaceFile *sfile, int cfra, int machine, int
 	}
 
 	calc_sequence(seq);
+	
+	if(sfile->flag & FILE_STRINGCODE) {
+		strcpy(name, sfile->dir);
+		strcpy(rel, G.sce);
+		BLI_makestringcode(rel, name);
+	} else {
+		strcpy(name, sfile->dir);
+	}
 
 	/* strip and stripdata */
 	seq->strip= strip= MEM_callocN(sizeof(Strip), "strip");
 	strip->len= totsel;
 	strip->us= 1;
-	strcpy(strip->dir, sfile->dir);
+	strncpy(strip->dir, name, FILE_MAXDIR-1);
 	strip->stripdata= se= MEM_callocN(totsel*sizeof(StripElem), "stripelem");
 
 	for(a=0; a<sfile->totfile; a++) {
 		if(sfile->filelist[a].flags & ACTIVE) {
 			if( (sfile->filelist[a].type & S_IFDIR)==0 ) {
-				strcpy(se->name, sfile->filelist[a].relname);
+				strncpy(se->name, sfile->filelist[a].relname, FILE_MAXFILE-1);
 				se->ok= 1;
 				se++;
 			}
@@ -500,12 +508,12 @@ static Sequence *sfile_to_sequence(SpaceFile *sfile, int cfra, int machine, int
 	}
 	/* no selected file: */
 	if(totsel==1 && se==strip->stripdata) {
-		strcpy(se->name, sfile->file);
+		strncpy(se->name, sfile->file, FILE_MAXFILE-1);
 		se->ok= 1;
 	}
 
 	/* last active name */
-	strcpy(last_imagename, seq->strip->dir);
+	strncpy(last_imagename, seq->strip->dir, FILE_MAXDIR-1);
 
 	return seq;
 }
@@ -517,12 +525,13 @@ static void sfile_to_mv_sequence(SpaceFile *sfile, int cfra, int machine)
 	Strip *strip;
 	StripElem *se;
 	int totframe, a;
-	char str[256];
+	char name[160], rel[160];
+	char str[FILE_MAXDIR+FILE_MAXFILE];
 
 	totframe= 0;
 
-	strcpy(str, sfile->dir);
-	strcat(str, sfile->file);
+	strncpy(str, sfile->dir, FILE_MAXDIR-1);
+	strncat(str, sfile->file, FILE_MAXDIR-1);
 
 	/* is it a movie? */
 	anim = openanim(str, IB_rect);
@@ -540,16 +549,24 @@ static void sfile_to_mv_sequence(SpaceFile *sfile, int cfra, int machine)
 	seq->anim= anim;
 
 	calc_sequence(seq);
+	
+	if(sfile->flag & FILE_STRINGCODE) {
+		strcpy(name, sfile->dir);
+		strcpy(rel, G.sce);
+		BLI_makestringcode(rel, name);
+	} else {
+		strcpy(name, sfile->dir);
+	}
 
 	/* strip and stripdata */
 	seq->strip= strip= MEM_callocN(sizeof(Strip), "strip");
 	strip->len= totframe;
 	strip->us= 1;
-	strcpy(strip->dir, sfile->dir);
+	strncpy(strip->dir, name, FILE_MAXDIR-1);
 	strip->stripdata= se= MEM_callocN(totframe*sizeof(StripElem), "stripelem");
 
 	/* name movie in first strip */
-	strcpy(se->name, sfile->file);
+	strncpy(se->name, sfile->file, FILE_MAXFILE-1);
 
 	for(a=1; a<=totframe; a++, se++) {
 		se->ok= 1;
@@ -557,7 +574,7 @@ static void sfile_to_mv_sequence(SpaceFile *sfile, int cfra, int machine)
 	}
 
 	/* last active name */
-	strcpy(last_imagename, seq->strip->dir);
+	strncpy(last_imagename, seq->strip->dir, FILE_MAXDIR-1);
 }
 
 static Sequence *sfile_to_snd_sequence(SpaceFile *sfile, int cfra, int machine)
@@ -568,12 +585,13 @@ static Sequence *sfile_to_snd_sequence(SpaceFile *sfile, int cfra, int machine)
 	StripElem *se;
 	double totframe;
 	int a;
+	char name[160], rel[160];
 	char str[256];
 
 	totframe= 0.0;
 
-	strcpy(str, sfile->dir);
-	strcat(str, sfile->file);
+	strncpy(str, sfile->dir, FILE_MAXDIR-1);
+	strncat(str, sfile->file, FILE_MAXFILE-1);
 
 	sound= sound_new_sound(str);
 	if (!sound || sound->sample->type == SAMPLE_INVALID) {
@@ -597,16 +615,24 @@ static Sequence *sfile_to_snd_sequence(SpaceFile *sfile, int cfra, int machine)
 	seq->sound = sound;
 
 	calc_sequence(seq);
+	
+	if(sfile->flag & FILE_STRINGCODE) {
+		strcpy(name, sfile->dir);
+		strcpy(rel, G.sce);
+		BLI_makestringcode(rel, name);
+	} else {
+		strcpy(name, sfile->dir);
+	}
 
 	/* strip and stripdata */
 	seq->strip= strip= MEM_callocN(sizeof(Strip), "strip");
 	strip->len= totframe;
 	strip->us= 1;
-	strcpy(strip->dir, sfile->dir);
+	strncpy(strip->dir, name, FILE_MAXDIR-1);
 	strip->stripdata= se= MEM_callocN(totframe*sizeof(StripElem), "stripelem");
 
 	/* name sound in first strip */
-	strcpy(se->name, sfile->file);
+	strncpy(se->name, sfile->file, FILE_MAXFILE-1);
 
 	for(a=1; a<=totframe; a++, se++) {
 		se->ok= 2; /* why? */
@@ -615,7 +641,7 @@ static Sequence *sfile_to_snd_sequence(SpaceFile *sfile, int cfra, int machine)
 	}
 
 	/* last active name */
-	strcpy(last_sounddir, seq->strip->dir);
+	strncpy(last_sounddir, seq->strip->dir, FILE_MAXDIR-1);
 
 	return seq;
 }
@@ -655,7 +681,7 @@ static void add_image_strips(char *name)
 	for(a=0; a<totfile; a++) {
 		if(files[a].flags & ACTIVE) {
 			if( (files[a].type & S_IFDIR) ) {
-				strcat(sfile->dir, files[a].relname);
+				strncat(sfile->dir, files[a].relname, FILE_MAXFILE-1);
 				strcat(sfile->dir,"/");
 				read_dir(sfile);
 
@@ -678,7 +704,7 @@ static void add_image_strips(char *name)
 	waitcursor(0);
 
 	BIF_undo_push("Add image strip Sequencer");
-	transform_seq('g');
+	transform_seq('g', 0);
 
 }
 
@@ -713,7 +739,7 @@ static void add_movie_strip(char *name)
 	waitcursor(0);
 
 	BIF_undo_push("Add movie strip Sequencer");
-	transform_seq('g');
+	transform_seq('g', 0);
 
 }
 
@@ -742,7 +768,7 @@ static void add_sound_strip(char *name)
 	waitcursor(0);
 
 	BIF_undo_push("Add sound strip Sequencer");
-	transform_seq('g');
+	transform_seq('g', 0);
 }
 
 #if 0
@@ -960,7 +986,7 @@ static void load_plugin_seq(char *str)		/* called from fileselect */
 		if( test_overlap_seq(last_seq) ) shuffle_seq(last_seq);
 
 		BIF_undo_push("Add plugin strip Sequencer");
-		transform_seq('g');
+		transform_seq('g', 0);
 	}
 }
 
@@ -1085,7 +1111,7 @@ void add_sequence(int type)
 				if(seq->len>0) strip->stripdata= MEM_callocN(seq->len*sizeof(StripElem), "stripelem");
 
 				BIF_undo_push("Add scene strip Sequencer");
-				transform_seq('g');
+				transform_seq('g', 0);
 			}
 		}
 		MEM_freeN(str);
@@ -1108,12 +1134,12 @@ void add_sequence(int type)
 			activate_fileselect(FILE_SPECIAL, "Select Plugin", U.plugseqdir, load_plugin_seq);
 		}
 		else {
-			if( add_seq_effect(event) ) transform_seq('g');
+			if( add_seq_effect(event) ) transform_seq('g', 0);
 		}
 
 		break;
 	case 103:
-		if (!last_sounddir[0]) strcpy(last_sounddir, U.sounddir);
+		if (!last_sounddir[0]) strncpy(last_sounddir, U.sounddir, FILE_MAXDIR-1);
 		activate_fileselect(FILE_SPECIAL, "Select Wav", last_sounddir, add_sound_strip);
 		break;
 	}
@@ -1419,7 +1445,7 @@ void add_duplicate_seq(void)
 	addlisttolist(ed->seqbasep, &new);
 
 	BIF_undo_push("Add duplicate Sequencer");
-	transform_seq('g');
+	transform_seq('g', 0);
 }
 
 int insert_gap(int gap, int cfra)
@@ -1731,7 +1757,7 @@ typedef struct TransSeq {
 	int startofs, endofs;
 } TransSeq;
 
-void transform_seq(int mode)
+void transform_seq(int mode, int context)
 {
 	Sequence *seq;
 	Editing *ed;
