@@ -1,5 +1,5 @@
 /**
- * $Id: editsound.c,v 1.17 2004/05/03 21:21:29 ton Exp $
+ * $Id: editsound.c,v 1.25 2005/03/09 19:45:56 lukep Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,6 +31,7 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <math.h>
 #include <string.h>
 #include <fcntl.h>
@@ -40,8 +41,11 @@
 #endif
 
 #ifndef WIN32 
+#define __USE_XOPEN /* Needed for swab on linux */
 #include <unistd.h>
+#undef __USE_XOPEN
 #else
+
 #include <io.h>
 #endif
 #include "MEM_guardedalloc.h"
@@ -54,6 +58,7 @@
 #include "DNA_scene_types.h"
 #include "DNA_sound_types.h"
 #include "DNA_packedFile_types.h"
+#include "DNA_userdef_types.h"
 
 #include "BKE_utildefines.h"
 #include "BKE_global.h"
@@ -80,6 +85,9 @@
 
 #include "SYS_System.h"
 
+#include "PIL_time.h"
+
+
 /* this might move to the external header */
 void* sound_get_libraryinterface(void);
 
@@ -104,6 +112,7 @@ void winqreadsoundspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 	float dx, dy;
 	int doredraw= 0, cfra, first = 0;
 	short mval[2], nr;
+	short mousebut = L_MOUSE;
 	
 	if(curarea->win==0) return;
 
@@ -111,6 +120,17 @@ void winqreadsoundspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 		
 		if( uiDoBlocks(&curarea->uiblocks, event)!=UI_NOTHING ) event= 0;
 
+		/* swap mouse buttons based on user preference */
+		if (U.flag & USER_LMOUSESELECT) {
+			if (event == LEFTMOUSE) {
+				event = RIGHTMOUSE;
+				mousebut = L_MOUSE;
+			} else if (event == RIGHTMOUSE) {
+				event = LEFTMOUSE;
+				mousebut = R_MOUSE;
+			}
+		}
+
 		switch(event) {
 		case LEFTMOUSE:
 			ssound->flag |= SND_CFRA_NUM;
@@ -126,10 +146,11 @@ void winqreadsoundspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 					first= 0;
 					CFRA= cfra;
 					update_for_newframe();
-					force_draw_plus(SPACE_VIEW3D);
+					force_draw_plus(SPACE_VIEW3D, 1);
 				}
+				else PIL_sleep_ms(30);
 			
-			} while(get_mbut()&L_MOUSE);
+			} while(get_mbut() & mousebut);
 			ssound->flag &= ~SND_CFRA_NUM;
 			
 			doredraw= 1;
@@ -186,14 +207,14 @@ void sound_initialize_sounds(void)
 		/* clear the soundscene */
 		SND_RemoveAllSounds(ghSoundScene);
 		SND_RemoveAllSamples(ghSoundScene);
+	}
 	
-		/* initialize sounds */
-		sound = G.main->sound.first;
-		while (sound)
-		{
-			sound_sample_is_null(sound);
-			sound = (bSound *) sound->id.next;
-		}
+	/* initialize sample blocks (doesnt call audio system, needs to be done once after load */
+	sound = G.main->sound.first;
+	while (sound)
+	{
+		sound_sample_is_null(sound);
+		sound = (bSound *) sound->id.next;
 	}
 }
 
@@ -431,7 +452,7 @@ void sound_read_wav_data(bSound* sound, PackedFile* pf)
 
 
 /* ugly, but it works (for now) */
-int sound_get_filetype_from_header(bSound* sound, PackedFile* pf)
+static int sound_get_filetype_from_header(bSound* sound, PackedFile* pf)
 {
 	int filetype = SAMPLE_INVALID;
 	int i;
@@ -513,7 +534,7 @@ int sound_get_filetype_from_header(bSound* sound, PackedFile* pf)
 
 
 
-int check_filetype(bSound* sound, PackedFile* pf)
+static int check_filetype(bSound* sound, PackedFile* pf)
 {
 //	char* pdest;
 	sound->sample->type = SAMPLE_INVALID;
@@ -998,9 +1019,6 @@ void sound_init_audio(void)
 	SYS_SystemHandle hSystem = NULL;
 	
 	if(ghSoundScene==NULL) {
-		
-		printf("sound init audio\n");
-	
 		hSystem = SYS_GetSystem();
 		noaudio = SYS_GetCommandLineInt(hSystem,"noaudio",0);
 		
@@ -1026,5 +1044,6 @@ void sound_exit_audio(void)
 	if(ghSoundScene) {
 		SND_DeleteScene(ghSoundScene);
 		SND_ReleaseDevice();
+		ghSoundScene = NULL;
 	}
 }
