@@ -1,5 +1,5 @@
 /**
- * $Id: ghostwinlay.c,v 1.35 2004/11/28 12:14:27 lukep Exp $
+ * $Id: ghostwinlay.c,v 1.39 2005/05/04 08:46:14 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -36,10 +36,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include "MEM_guardedalloc.h"
 
 #include "DNA_listBase.h"	
@@ -69,7 +65,9 @@
 #include <Carbon/Carbon.h>
 
 /*declarations*/
-int checkAppleVideoCard();
+int checkAppleVideoCard(void);
+void getMacAvailableBounds(short *, short *, short *, short *);
+
 #endif
 ///
 
@@ -88,7 +86,6 @@ struct _Window {
 	int		lmouse[2];
 	int		lqual;		/* (LR_SHFTKEY, LR_CTRLKEY, LR_ALTKEY, LR_COMMANDKEY) */
 	int		lmbut;		/* (L_MOUSE, M_MOUSE, R_MOUSE) */
-	int		commandqual;
 
 		/* Tracks the faked mouse button, if non-zero it is
 		 * the event number of the last faked button.
@@ -114,7 +111,7 @@ struct _Window {
 		
 static int macPrefState = 0;
 		
-int checkAppleVideoCard() {
+int checkAppleVideoCard(void) {
 	long theErr;
 	unsigned long display_mask;
 	CGLRendererInfoObj rend;
@@ -332,9 +329,9 @@ Window *window_open(char *title, int posx, int posy, int sizex, int sizey, int s
 		GHOST_kWindowStateFullScreen:GHOST_kWindowStateNormal;
 #else
 #ifdef _WIN32	// FULLSCREEN
-	if (start_maximized == G_WINDOWSTATE_FULLSCREEN)
-		inital_state= GHOST_kWindowStateFullScreen;
-	else
+//	if (start_maximized == G_WINDOWSTATE_FULLSCREEN)
+//		inital_state= GHOST_kWindowStateFullScreen;
+//	else
 		inital_state= start_maximized?GHOST_kWindowStateMaximized:GHOST_kWindowStateNormal;
 #else			// APPLE
 	inital_state= start_maximized?GHOST_kWindowStateMaximized:GHOST_kWindowStateNormal;
@@ -524,7 +521,7 @@ static int event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private)
 		
 			if (bbut==LEFTMOUSE) {
 				if (val) {
-					if (win->commandqual) {
+					if (win->lqual & LR_COMMANDKEY) {
 						bbut= win->faked_mbut= RIGHTMOUSE;
 					} else if ((win->lqual & LR_ALTKEY) && (U.flag & USER_TWOBUTTONMOUSE)) {
 						/* finally, it actually USES the userpref! :) -intrr */
@@ -556,10 +553,6 @@ static int event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private)
 			int val= (type==GHOST_kEventKeyDown);
 			int bkey= convert_key(kd->key);
 
-			if (kd->key == GHOST_kKeyCommand) {
-				win->commandqual= val;
-			}
-
 			if (bkey) {
 				if (bkey==LEFTSHIFTKEY || bkey==RIGHTSHIFTKEY) {
 					win->lqual= change_bit(win->lqual, LR_SHIFTKEY, val);
@@ -609,8 +602,6 @@ static int event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private)
 					win->lqual= change_bit(win->lqual, LR_COMMANDKEY, 0);
 					window_handle(win, LR_COMMANDKEY, 0);
 				}
-				/* probably redundant now (ton) */
-				win->commandqual= query_qual('C');
 
 				/* 
 				 * XXX quick hack so OSX version works better
@@ -728,7 +719,8 @@ void window_raise(Window *win) {
 #endif
 }
 
-#ifdef _WIN32	//FULLSCREEN
+#if 0
+//#ifdef _WIN32	//FULLSCREEN
 void window_toggle_fullscreen(Window *win, int fullscreen) {
 	/* these two lines make sure front and backbuffer are equal. for swapbuffers */
 	markdirty_all();
