@@ -4,7 +4,7 @@
  * Functions to draw the "Action Editor" window header
  * and handle user events sent to it.
  * 
- * $Id: header_action.c,v 1.13 2004/05/06 19:42:38 ton Exp $
+ * $Id: header_action.c,v 1.23 2005/03/19 21:08:12 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,16 +43,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
-#include "BMF_Api.h"
-#include "BIF_language.h"
-#ifdef INTERNATIONAL
-#include "FTF_Api.h"
-#endif
-
 #include "DNA_ID.h"
 #include "DNA_action_types.h"
 #include "DNA_ipo_types.h"
@@ -74,6 +64,7 @@
 #include "BKE_main.h"
 #include "BKE_constraint.h"
 #include "BKE_armature.h"
+#include "BKE_utildefines.h"
 
 #include "BSE_drawipo.h"
 #include "BSE_editaction.h"
@@ -99,6 +90,9 @@
 #define ACTMENU_KEY_DELETE        1
 #define ACTMENU_KEY_BAKE          2
 
+#define ACTMENU_KEY_TRANSFORM_MOVE	0
+#define ACTMENU_KEY_TRANSFORM_SCALE	1
+
 #define ACTMENU_KEY_HANDLE_AUTO   0
 #define ACTMENU_KEY_HANDLE_ALIGN  1
 #define ACTMENU_KEY_HANDLE_FREE   2
@@ -135,8 +129,8 @@ void do_action_buttons(unsigned short event)
 			G.v2d->cur.ymin=-SCROLLB;
 
 			if (!G.saction->action){	// here the mesh rvk?
-				G.v2d->cur.xmax=0;
-				G.v2d->cur.xmax=100;
+				G.v2d->cur.xmax= -5;
+				G.v2d->cur.xmax= 100;
 			}
 			else {
 				float extra;
@@ -145,6 +139,11 @@ void do_action_buttons(unsigned short event)
 				extra= 0.05*(G.v2d->cur.xmax - G.v2d->cur.xmin);
 				G.v2d->cur.xmin-= extra;
 				G.v2d->cur.xmax+= extra;
+				
+				if(G.v2d->cur.xmin==G.v2d->cur.xmax) {
+					G.v2d->cur.xmax= -5;
+					G.v2d->cur.xmax= 100;
+				}
 			}
 
 			G.v2d->tot= G.v2d->cur;
@@ -209,7 +208,7 @@ static void do_action_viewmenu(void *arg, int event)
 			play_anim(1);
 			break;	
 		case ACTMENU_VIEW_ALL: /* View All */
-			do_action_buttons(B_SIMAGEHOME);
+			do_action_buttons(B_ACTHOME);
 			break;
 		case ACTMENU_VIEW_MAXIMIZE: /* Maximize Window */
 			/* using event B_FULL */
@@ -378,6 +377,64 @@ static uiBlock *action_selectmenu(void *arg_unused)
 	return block;
 }
 
+static void do_action_keymenu_transformmenu(void *arg, int event)
+{
+	SpaceAction *saction;
+	bAction	*act;
+	Key *key;
+	
+	key = get_action_mesh_key();
+	saction= curarea->spacedata.first;
+	
+	act=saction->action;
+	
+	switch(event)
+	{
+		case ACTMENU_KEY_TRANSFORM_MOVE:
+			if (key) {
+				transform_meshchannel_keys('g', key);
+			} 
+			else if (act) {	
+				transform_actionchannel_keys ('g');
+			}
+			break;
+		case ACTMENU_KEY_TRANSFORM_SCALE:
+			if (key) {
+				transform_meshchannel_keys('s', key);
+			} 
+			else if (act) {
+				transform_actionchannel_keys ('s');
+			}
+			break;
+	}
+
+	scrarea_queue_winredraw(curarea);
+}
+
+static uiBlock *action_keymenu_transformmenu(void *arg_unused)
+{
+	uiBlock *block;
+	short yco= 0, menuwidth=120;
+
+	block= uiNewBlock(&curarea->uiblocks, "action_keymenu_transformmenu", 
+					  UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_action_keymenu_transformmenu, NULL);
+
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
+					 "Grab/Move|G", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					 ACTMENU_KEY_TRANSFORM_MOVE, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
+					 "Scale|S", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					 ACTMENU_KEY_TRANSFORM_SCALE, "");
+
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 60);
+
+	return block;
+}
+
 static void do_action_keymenu_handlemenu(void *arg, int event)
 {
 	Key *key;
@@ -521,7 +578,6 @@ static void do_action_keymenu(void *arg, int event)
 	if (!saction)
 		return;
 
-
 	act = saction->action;
 	key = get_action_mesh_key();
 
@@ -531,7 +587,7 @@ static void do_action_keymenu(void *arg, int event)
 			if (key) {
 				duplicate_meshchannel_keys(key);
 			}
-			else {
+			else if (act) {
 				duplicate_actionchannel_keys();
 				remake_action_ipos(act);
 			}
@@ -541,15 +597,13 @@ static void do_action_keymenu(void *arg, int event)
 			if (key) {
 				delete_meshchannel_keys(key);
 			}
-			else {
+			else if (act) {
 				delete_actionchannel_keys ();
 			}
 			break;
 		case ACTMENU_KEY_BAKE:
 			bake_action_with_client (G.saction->action, OBACT, 0.01);
 			break;
-
-			break;
 	}
 }
 
@@ -561,15 +615,22 @@ static uiBlock *action_keymenu(void *arg_unused)
 	block= uiNewBlock(&curarea->uiblocks, "action_keymenu", 
 					  UI_EMBOSSP, UI_HELV, curarea->headwin);
 	uiBlockSetButmFunc(block, do_action_keymenu, NULL);
+	
+	uiDefIconTextBlockBut(block, action_keymenu_transformmenu, 
+					NULL, ICON_RIGHTARROW_THIN, 
+					"Transform", 0, yco-=20, 120, 20, "");
+						  
+	uiDefBut(block, SEPR, 0, "", 0, yco-=6, 
+					menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
-					 "Duplicate|Shift D", 0, yco-=20, 
-					 menuwidth, 19, NULL, 0.0, 0.0, 0, 
-					 ACTMENU_KEY_DUPLICATE, "");
+					"Duplicate|Shift D", 0, yco-=20, 
+					menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					ACTMENU_KEY_DUPLICATE, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
-					 "Delete|X", 0, yco-=20, 
-					 menuwidth, 19, NULL, 0.0, 0.0, 0, 
-					 ACTMENU_KEY_DELETE, "");
+					"Delete|X", 0, yco-=20, 
+					menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					ACTMENU_KEY_DELETE, "");
 
 	uiDefBut(block, SEPR, 0, "", 0, yco-=6, 
 			 menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
@@ -665,17 +726,17 @@ void action_buttons(void)
 		uiBlockSetEmboss(block, UI_EMBOSSP);
 	
 		xmax= GetButStringLength("View");
-		uiDefBlockBut(block, action_viewmenu, NULL, 
+		uiDefPulldownBut(block, action_viewmenu, NULL, 
 					  "View", xco, -2, xmax-3, 24, "");
 		xco+= xmax;
 		
 		xmax= GetButStringLength("Select");
-		uiDefBlockBut(block, action_selectmenu, NULL, 
+		uiDefPulldownBut(block, action_selectmenu, NULL, 
 					  "Select", xco, -2, xmax-3, 24, "");
 		xco+= xmax;
 	
 		xmax= GetButStringLength("Key");
-		uiDefBlockBut(block, action_keymenu, NULL, 
+		uiDefPulldownBut(block, action_keymenu, NULL, 
 					  "Key", xco, -2, xmax-3, 24, "");
 		xco+= xmax;
 	}
@@ -683,7 +744,9 @@ void action_buttons(void)
 	uiBlockSetEmboss(block, UI_EMBOSSX);
 	
 	// object action is allowed to be zero!
-	if (!get_action_mesh_key()) {
+	/* (ton) commented out below line, since people can apparently link Action to any object (mesh) and
+		not unlink anymore when theres a mesh key. Needs to be rethought this stuff! */
+	//if (!get_action_mesh_key()) {
 	
 		/* NAME ETC */
 		ob=OBACT;
@@ -703,7 +766,7 @@ void action_buttons(void)
 				 "converted into Ipo keys");
 		xco+=64;
 
-	}
+	//}
 	uiClearButLock();
 
 	/* draw LOCK */
