@@ -4,7 +4,7 @@
  * Functions to draw the "Buttons Window" window header
  * and handle user events sent to it.
  * 
- * $Id: header_buttonswin.c,v 1.30 2004/12/04 21:49:02 ton Exp $
+ * $Id: header_buttonswin.c,v 1.33 2005/03/31 18:49:52 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,16 +42,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
-#include "BMF_Api.h"
-#include "BIF_language.h"
-#ifdef INTERNATIONAL
-#include "FTF_Api.h"
-#endif
-
 #include "DNA_ID.h"
 #include "DNA_armature_types.h"
 #include "DNA_lamp_types.h"
@@ -753,7 +743,7 @@ void buts_buttons(void)
 	uiBlockEndAlign(block);
 	
 	xco+=XIC;
-	uiDefButS(block, NUM, B_NEWFRAME, "",	(short)(xco+20),0,60,YIC, &(G.scene->r.cfra), 1.0, 18000.0, 0, 0, "Displays Current Frame of animation. Click to change.");
+	uiDefButS(block, NUM, B_NEWFRAME, "",	(short)(xco+20),0,60,YIC, &(G.scene->r.cfra), 1.0, MAXFRAMEF, 0, 0, "Displays Current Frame of animation. Click to change.");
 	xco+= 80;
 
 	buttons_active_id(&id, &idfrom);
