@@ -4,7 +4,7 @@
  * Functions to draw the "UV/Image Editor" window header
  * and handle user events sent to it.
  * 
- * $Id: header_image.c,v 1.16 2004/05/12 16:53:11 broken Exp $
+ * $Id: header_image.c,v 1.36 2005/05/03 10:54:42 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,16 +42,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
-#include "BMF_Api.h"
-#include "BIF_language.h"
-#ifdef INTERNATIONAL
-#include "FTF_Api.h"
-#endif
-
 #include "DNA_ID.h"
 #include "DNA_image_types.h"
 #include "DNA_mesh_types.h"
@@ -62,10 +52,12 @@
 #include "DNA_userdef_types.h"
 
 #include "BDR_drawmesh.h"
+#include "BDR_unwrapper.h"
 #include "BKE_global.h"
 #include "BKE_image.h"
 #include "BKE_main.h"
 #include "BKE_packedFile.h"
+#include "BKE_utildefines.h"
 
 #include "BLI_blenlib.h"
 #include "BIF_drawimage.h"
@@ -81,6 +73,9 @@
 #include "BSE_filesel.h"
 #include "BSE_headerbuttons.h"
 
+#include "BPY_extern.h"
+#include "BPY_menus.h"
+
 #include "IMB_imbuf.h"
 #include "IMB_imbuf_types.h"
 
@@ -89,7 +84,7 @@
 
 #include "TPT_DependKludge.h"
 
-void load_space_image(char *str)	/* called from fileselect */
+static void load_space_image(char *str)	/* called from fileselect */
 {
 	Image *ima=0;
  
@@ -108,11 +103,11 @@ void load_space_image(char *str)	/* called from fileselect */
 		image_changed(G.sima, 0);
 
 	}
-
+	BIF_undo_push("Load image UV");
 	allqueue(REDRAWIMAGE, 0);
 }
 
-void image_replace(Image *old, Image *new)
+static void image_replace(Image *old, Image *new)
 {
 	TFace *tface;
 	Mesh *me;
@@ -147,7 +142,7 @@ void image_replace(Image *old, Image *new)
 	else error("Nothing replaced");
 }
 
-void replace_space_image(char *str)		/* called from fileselect */
+static void replace_space_image(char *str)		/* called from fileselect */
 {
 	Image *ima=0;
 
@@ -171,10 +166,11 @@ void replace_space_image(char *str)		/* called from fileselect */
 		image_changed(G.sima, 0);
 
 	}
+	BIF_undo_push("Replace image UV");
 	allqueue(REDRAWIMAGE, 0);
 }
 
-void save_paint(char *name)
+static void save_paint(char *name)
 {
 	char str[FILE_MAXDIR+FILE_MAXFILE];
 	Image *ima = G.sima->image;
@@ -248,6 +244,7 @@ void do_image_buttons(unsigned short event)
 		}
 		/* also when image is the same: assign! 0==no tileflag: */
 		image_changed(G.sima, 0);
+		BIF_undo_push("Assign image UV");
 
 		break;
 	case B_SIMAGELOAD:
@@ -350,8 +347,66 @@ void do_image_buttons(unsigned short event)
 	}
 }
 
-/* This should not be a stack var! */
-// static int headerbuttons_packdummy;
+static void do_image_view_viewnavmenu(void *arg, int event)
+{
+	switch(event) {
+	case 1: /* Zoom In */
+		image_viewzoom(PADPLUSKEY, 0);
+		break;
+	case 2: /* Zoom Out */
+		image_viewzoom(PADMINUS, 0);
+		break;
+	case 3: /* Zoom 8:1 */
+		image_viewzoom(PAD8, 0);
+		break;
+	case 4: /* Zoom 4:1 */
+		image_viewzoom(PAD4, 0);
+		break;
+	case 5: /* Zoom 2:1 */
+		image_viewzoom(PAD2, 0);
+		break;
+	case 6: /* Zoom 1:1 */
+		image_viewzoom(PAD1, 0);
+		break;
+	case 7: /* Zoom 1:2 */
+		image_viewzoom(PAD2, 1);
+		break;
+	case 8: /* Zoom 1:4 */
+		image_viewzoom(PAD4, 1);
+		break;
+	case 9: /* Zoom 1:8 */
+		image_viewzoom(PAD8, 1);
+		break;
+	}
+	allqueue(REDRAWIMAGE, 0);
+}
+
+static uiBlock *image_view_viewnavmenu(void *arg_unused)
+{
+/*		static short tog=0; */
+	uiBlock *block;
+	short yco= 0, menuwidth=120;
+	
+	block= uiNewBlock(&curarea->uiblocks, "image_view_viewnavmenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_image_view_viewnavmenu, NULL);
+	
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom In|NumPad +", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom Out|NumPad -",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
+
+	uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 1:8", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 1:4", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 4, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 1:2", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 1:1", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 2:1", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 4:1", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 8:1", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 9, "");
+
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 50);
+	return block;
+}
 
 static void do_image_viewmenu(void *arg, int event)
 {
@@ -381,6 +436,14 @@ static void do_image_viewmenu(void *arg, int event)
 			G.f |= G_DRAWFACES;
 		allqueue(REDRAWIMAGE, 0);
 		break;
+	case 7: /* Properties  Panel */
+		add_blockhandler(curarea, IMAGE_HANDLER_PROPERTIES, UI_PNL_UNSTOW);
+		break;
+	case 8: /* Paint Panel... */
+		add_blockhandler(curarea, IMAGE_HANDLER_PAINT, UI_PNL_UNSTOW);
+		break;
+	case 9:
+		image_viewcentre();
 	}
 	allqueue(REDRAWVIEW3D, 0);
 }
@@ -393,21 +456,28 @@ static uiBlock *image_viewmenu(void *arg_unused)
 	
 	block= uiNewBlock(&curarea->uiblocks, "image_viewmenu", UI_EMBOSSP, UI_HELV, curarea->headwin);
 	uiBlockSetButmFunc(block, do_image_viewmenu, NULL);
+
+	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "View Properties...",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "View Paint Tool...",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
+
 	
 	if(G.f & G_DRAWFACES) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Draw Faces", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
 	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Draw Faces|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
 	if(G.sima->flag & SI_DRAWSHADOW) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Draw Shadow Mesh", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
 	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Draw Shadow Mesh|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
+	
 	uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+	uiDefIconTextBlockBut(block, image_view_viewnavmenu, NULL, ICON_RIGHTARROW_THIN, "View Navigation", 0, yco-=20, 120, 19, "");
 
 	if(G.sima->lock) {
 		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Update Automatically|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 0, "");
 	} else {
 		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Update Automatically|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 0, "");
-	}
+	} 
 
 	uiDefBut(block, SEPR, 0, "",					0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 	
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "View Selected|NumPad .",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 9, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "View All|Home", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
 		
 	if(!curarea->full) uiDefIconTextBut(block, BUTM, B_FULL, ICON_BLANK1, "Maximize Window|Ctrl UpArrow", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
@@ -440,7 +510,7 @@ static void do_image_selectmenu(void *arg, int event)
 		unlink_selection();
 		break;
 	case 3: /* Select Linked UVs */
-		select_linked_tface_uv();
+		select_linked_tface_uv(2);
 		break;
 	case 4: /* Toggle Local UVs Stick to Vertex in Mesh */
 		if(G.sima->flag & SI_LOCALSTICKY)
@@ -497,7 +567,7 @@ static uiBlock *image_selectmenu(void *arg_unused)
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Unlink Selection|Alt L", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
 
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Select Linked UVs|L", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Select Linked UVs|Ctrl L", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
 
 	if(curarea->headertype==HEADERTOP) {
 		uiBlockSetDirection(block, UI_DOWN);
@@ -558,9 +628,9 @@ static void do_image_imagemenu(void *arg, int event)
 		if(G.sima->image) strcpy(name, G.sima->image->name);
 		else strcpy(name, U.textudir);
 		if(G.qual==LR_CTRLKEY)
-			activate_imageselect(FILE_SPECIAL, "SELECT IMAGE", name, load_space_image);
+			activate_imageselect(FILE_SPECIAL, "Open Image", name, load_space_image);
 		else
-			activate_fileselect(FILE_SPECIAL, "SELECT IMAGE", name, load_space_image);
+			activate_fileselect(FILE_SPECIAL, "Open Image", name, load_space_image);
 		break;
 	case 1: /* Replace */
 		if(G.sima->image) strcpy(name, G.sima->image->name);
@@ -584,6 +654,7 @@ static void do_image_imagemenu(void *arg, int event)
 				}
 			}
 		}
+		BIF_undo_push("Pack image");
 		allqueue(REDRAWBUTSSHADING, 0);
 		allqueue(REDRAWHEADERS, 0);
 		break;
@@ -604,6 +675,7 @@ static void do_image_imagemenu(void *arg, int event)
 				error("There are no packed images to unpack");
 			}
 		}
+		BIF_undo_push("Unpack image");
 		allqueue(REDRAWBUTSSHADING, 0);
 		allqueue(REDRAWHEADERS, 0);
 		break;
@@ -640,6 +712,7 @@ static void do_image_imagemenu(void *arg, int event)
 			image_changed(G.sima, 0);
 		}
 		allqueue(REDRAWIMAGE, 0);
+		allqueue(REDRAWVIEW3D, 0);
 		break;
 	}
 }
@@ -729,14 +802,13 @@ static uiBlock *image_uvs_showhidemenu(void *arg_unused)
 
 static void do_image_uvs_propfalloffmenu(void *arg, int event)
 {
-	extern int prop_mode;
 	
 	switch(event) {
 	case 0: /* proportional edit - sharp*/
-		prop_mode = 0;
+		G.scene->prop_mode = 0;
 		break;
 	case 1: /* proportional edit - smooth*/
-		prop_mode = 1;
+		G.scene->prop_mode = 1;
 		break;
 	}
 }
@@ -745,14 +817,13 @@ static uiBlock *image_uvs_propfalloffmenu(void *arg_unused)
 {
 	uiBlock *block;
 	short yco = 20, menuwidth = 120;
-	extern int prop_mode;
 
 	block= uiNewBlock(&curarea->uiblocks, "image_uvs_propfalloffmenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
 	uiBlockSetButmFunc(block, do_image_uvs_propfalloffmenu, NULL);
 	
-	if (prop_mode==0) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Sharp|Shift O",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 0, "");
+	if (G.scene->prop_mode==0) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Sharp|Shift O",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 0, "");
 	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Sharp|Shift O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 0, "");
-	if (prop_mode==1) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Smooth|Shift O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
+	if (G.scene->prop_mode==1) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Smooth|Shift O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
 	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Smooth|Shift O",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
 		
 	uiBlockSetDirection(block, UI_RIGHT);
@@ -764,19 +835,13 @@ static void do_image_uvs_transformmenu(void *arg, int event)
 {
 	switch(event) {
 	case 0: /* Grab */
-		transform_tface_uv('g');
+		transform_tface_uv('g', 0);
 		break;
 	case 1: /* Rotate */
-		transform_tface_uv('r');
+		transform_tface_uv('r', 0);
 		break;
 	case 2: /* Scale */
-		transform_tface_uv('s');
-		break;
-	case 3: /* Weld / Align */
-		transform_tface_uv('w');
-		break;
-	case 4: /* Mirror */
-		mirrormenu_tface_uv();
+		transform_tface_uv('s', 0);
 		break;
 	}
 }
@@ -792,18 +857,83 @@ static uiBlock *image_uvs_transformmenu(void *arg_unused)
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Grab/Move|G", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Rotate|R", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Scale|S", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Weld/Align|W", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Mirror...|M", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 4, "");
 
 	uiBlockSetDirection(block, UI_RIGHT);
 	uiTextBoundsBlock(block, 60);
 	return block;
 }
 
+static void do_image_uvs_mirrormenu(void *arg, int event)
+{
+	switch(event) {
+	case 0: /* X axis */
+		mirror_tface_uv('x');
+		break;
+	case 1: /* Y axis */
+		mirror_tface_uv('y');
+		break;
+	}
+	
+	BIF_undo_push("Mirror UV");
+}
+
+static uiBlock *image_uvs_mirrormenu(void *arg_unused)
+{
+	uiBlock *block;
+	short yco = 20, menuwidth = 120;
+
+	block= uiNewBlock(&curarea->uiblocks, "image_uvs_mirrormenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_image_uvs_mirrormenu, NULL);
+	
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "X Axis|M, 1", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Y Axis|M, 2", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
+
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 60);
+	return block;
+}
+
+static void do_image_uvs_weldalignmenu(void *arg, int event)
+{
+	switch(event) {
+	case 0: /* Weld */
+		weld_align_tface_uv('w');
+		break;
+	case 1: /* Align X */
+		weld_align_tface_uv('x');
+		break;
+	case 2: /* Align Y */
+		weld_align_tface_uv('y');
+		break;
+	}
+	
+	if(event==0) BIF_undo_push("Weld UV");
+	else if(event==1 || event==2) BIF_undo_push("Align UV");
+}
+
+static uiBlock *image_uvs_weldalignmenu(void *arg_unused)
+{
+	uiBlock *block;
+	short yco = 20, menuwidth = 120;
+
+	block= uiNewBlock(&curarea->uiblocks, "image_uvs_weldalignmenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_image_uvs_weldalignmenu, NULL);
+	
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Weld|W, 1", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Align X|W, 2", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Align Y|W, 3", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
+
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 60);
+	return block;
+}
 
 static void do_image_uvsmenu(void *arg, int event)
 {
-	switch(event)
+	/* events >=20 are registered bpython scripts */
+	if (event >= 20) BPY_menu_do_python(PYMENU_UV, event - 20);
+
+	else switch(event)
 	{
 	case 1: /* UVs Constrained Rectangular */
 		if(G.sima->flag & SI_BE_SQUARE) G.sima->flag &= ~SI_BE_SQUARE;
@@ -820,15 +950,24 @@ static void do_image_uvsmenu(void *arg, int event)
 		stitch_uv_tface(0);
 		break;
 	case 5: /* Proportional Edit (toggle) */
-		if(G.f & G_PROPORTIONAL)
-			G.f &= ~G_PROPORTIONAL;
+		if(G.scene->proportional)
+			G.scene->proportional= 0;
 		else
-			G.f |= G_PROPORTIONAL;
+			G.scene->proportional= 1;
 		break;
 	case 7: /* UVs Snap to Pixel */
 		if(G.sima->flag & SI_NOPIXELSNAP) G.sima->flag &= ~SI_NOPIXELSNAP;
 		else G.sima->flag |= SI_NOPIXELSNAP;
 		break;
+    case 8:
+		pin_tface_uv(1);
+		break;
+    case 9:
+		pin_tface_uv(0);
+		break;
+    case 10:
+		unwrap_lscm();
+		break;
 	}
 }
 
@@ -836,6 +975,8 @@ static uiBlock *image_uvsmenu(void *arg_unused)
 {
 	uiBlock *block;
 	short yco= 0, menuwidth=120;
+	BPyMenu *pym;
+	int i = 0;
 
 	block= uiNewBlock(&curarea->uiblocks, "image_uvsmenu", UI_EMBOSSP, UI_HELV, curarea->headwin);
 	uiBlockSetButmFunc(block, do_image_uvsmenu, NULL);
@@ -853,13 +994,24 @@ static uiBlock *image_uvsmenu(void *arg_unused)
 
 	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
 
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Unpin|Alt P", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 9, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Pin|P", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "LSCM Unwrap|E", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 10, "");
+
+	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
+
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Limit Stitch...|Shift V", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Stitch|V", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 4, "");
 	uiDefIconTextBlockBut(block, image_uvs_transformmenu, NULL, ICON_RIGHTARROW_THIN, "Transform", 0, yco-=20, 120, 19, "");
 
+	uiDefBut(block, SEPR, 0, "",				0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+	
+	uiDefIconTextBlockBut(block, image_uvs_mirrormenu, NULL, ICON_RIGHTARROW_THIN, "Mirror", 0, yco-=20, 120, 19, "");
+	uiDefIconTextBlockBut(block, image_uvs_weldalignmenu, NULL, ICON_RIGHTARROW_THIN, "Weld/Align", 0, yco-=20, 120, 19, "");
+	
 	uiDefBut(block, SEPR, 0, "",				0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
-	if(G.f & G_PROPORTIONAL)
+	if(G.scene->proportional)
 		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Proportional Editing|O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 5, "");
 	else
 		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Proportional Editing|O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 5, "");
@@ -868,6 +1020,16 @@ static uiBlock *image_uvsmenu(void *arg_unused)
 
 	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
 	
+	/* note that we acount for the 3 previous entries with i+3: */
+	for (pym = BPyMenuTable[PYMENU_UV]; pym; pym = pym->next, i++) {
+
+		uiDefIconTextBut(block, BUTM, 1, ICON_PYTHON, pym->name, 0, yco-=20, menuwidth, 19, 
+				 NULL, 0.0, 0.0, 1, i+20, 
+				 pym->tooltip?pym->tooltip:pym->filename);
+	}
+
+	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
+
 	uiDefIconTextBlockBut(block, image_uvs_showhidemenu, NULL, ICON_RIGHTARROW_THIN, "Show/Hide Faces", 0, yco-=20, menuwidth, 19, "");
 
 	if(curarea->headertype==HEADERTOP) {
@@ -888,7 +1050,10 @@ void image_buttons(void)
 	uiBlock *block;
 	short xco, xmax;
 	char naam[256];
-	int headerbuttons_packdummy = 0;
+	/* This should not be a static var */
+	static int headerbuttons_packdummy;
+
+	headerbuttons_packdummy = 0;
 		
 	sprintf(naam, "header %d", curarea->headwin);
 	block= uiNewBlock(&curarea->uiblocks, naam, UI_EMBOSS, UI_HELV, curarea->headwin);
@@ -922,20 +1087,20 @@ void image_buttons(void)
 		uiBlockSetEmboss(block, UI_EMBOSSP);
 	
 		xmax= GetButStringLength("View");
-		uiDefBlockBut(block, image_viewmenu, NULL, "View", xco, -2, xmax-3, 24, "");
+		uiDefPulldownBut(block, image_viewmenu, NULL, "View", xco, -2, xmax-3, 24, "");
 		xco+= xmax;
 		
 		xmax= GetButStringLength("Select");
-		uiDefBlockBut(block, image_selectmenu, NULL, "Select", xco, -2, xmax-3, 24, "");
+		uiDefPulldownBut(block, image_selectmenu, NULL, "Select", xco, -2, xmax-3, 24, "");
         
 		xco+= xmax;
 		
 		xmax= GetButStringLength("Image");
-		uiDefBlockBut(block, image_imagemenu, NULL, "Image", xco, -2, xmax-3, 24, "");
+		uiDefPulldownBut(block, image_imagemenu, NULL, "Image", xco, -2, xmax-3, 24, "");
 		xco+= xmax;
 		
 		xmax= GetButStringLength("UVs");
-		uiDefBlockBut(block, image_uvsmenu, NULL, "UVs", xco, -2, xmax-3, 24, "");
+		uiDefPulldownBut(block, image_uvsmenu, NULL, "UVs", xco, -2, xmax-3, 24, "");
 		xco+= xmax;
 	}
 	
