@@ -4,7 +4,7 @@
  * Functions to draw the "Ipo Curve Editor" window header
  * and handle user events sent to it.
  * 
- * $Id: header_ipo.c,v 1.26 2004/12/04 21:49:02 ton Exp $
+ * $Id: header_ipo.c,v 1.28 2005/03/19 21:08:12 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,16 +43,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
-#include "BMF_Api.h"
-#include "BIF_language.h"
-#ifdef INTERNATIONAL
-#include "FTF_Api.h"
-#endif
-
 #include "DNA_ID.h"
 #include "DNA_curve_types.h"
 #include "DNA_key_types.h"
