@@ -4,7 +4,7 @@
  * Functions to draw the "Text Editor" window header
  * and handle user events sent to it.
  * 
- * $Id: header_text.c,v 1.21 2004/11/14 13:37:29 broken Exp $
+ * $Id: header_text.c,v 1.26 2005/05/13 16:11:28 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,15 +43,8 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include "BMF_Api.h"
 #include "BIF_language.h"
-#ifdef INTERNATIONAL
-#include "FTF_Api.h"
-#endif
 
 #include "BSE_headerbuttons.h"
 
@@ -133,7 +126,7 @@ void do_text_buttons(unsigned short event)
 				st->top= 0;
 				
 				pop_space_text(st);
-			
+				if (st->showsyntax) get_format_string();
 				allqueue(REDRAWTEXT, 0);
 				allqueue(REDRAWHEADERS, 0);
 			}
@@ -161,11 +154,6 @@ void do_text_buttons(unsigned short event)
 		break;
 */		 
 	case B_TEXTLINENUM:
-		if(st->showlinenrs)
-			st->showlinenrs = 0;
-		else
-			st->showlinenrs = 1;
-
 		allqueue(REDRAWTEXT, 0);
 		allqueue(REDRAWHEADERS, 0);
 		break;
@@ -184,6 +172,14 @@ void do_text_buttons(unsigned short event)
 
 		break;
 	case B_TAB_NUMBERS:
+		if (st->showsyntax) get_format_string();
+		allqueue(REDRAWTEXT, 0);
+		allqueue(REDRAWHEADERS, 0);
+		break;
+	case B_SYNTAX:
+		if (st->showsyntax) {
+			get_format_string();
+		}
 		allqueue(REDRAWTEXT, 0);
 		allqueue(REDRAWHEADERS, 0);
 		break;
@@ -215,6 +211,7 @@ static void do_text_filemenu(void *arg, int event)
 				if (!reopen_text(text)) {
 					error("Could not reopen file");
 				}
+			if (st->showsyntax) get_format_string();
 			}
 		break;
 	case 5:
@@ -259,6 +256,7 @@ static void do_text_editmenu(void *arg, int event)
 		break;
 	case 5:
 		txt_paste(text);
+		if (st->showsyntax) get_format_string();
 		break;
 	case 6:
 		txt_print_cutbuffer();
@@ -272,9 +270,6 @@ static void do_text_editmenu(void *arg, int event)
 	case 9:
 		txt_find_panel(st,0);
 		break;
-	case 10:
-		txt_export_to_object(text);
-		break;
 	default:
 		break;
 	}
@@ -351,8 +346,8 @@ static void do_text_formatmenu(void *arg, int event)
 	switch(event) {
 	case 3:
 		if (txt_has_sel(text)) {
-			txt_cut_sel(text);
-			indent_paste(text);
+			txt_order_cursors(text);
+			indent(text);
 			break;
 		}
 		else {
@@ -361,21 +356,21 @@ static void do_text_formatmenu(void *arg, int event)
 		}
 	case 4:
 		if ( txt_has_sel(text)) {
-			txt_cut_sel(text);
+			txt_order_cursors(text);
 			unindent(text);
 			break;
 		}
 		break;
 	case 5:
 		if ( txt_has_sel(text)) {
-			txt_cut_sel(text);
+			txt_order_cursors(text);
 			comment(text);
 			break;
 		}
 		break;
 	case 6:
 		if ( txt_has_sel(text)) {
-			txt_cut_sel(text);
+			txt_order_cursors(text);
 			uncomment(text);
 			break;
 		}
@@ -456,6 +451,38 @@ static uiBlock *text_formatmenu(void *arg_unused)
 	return block;
 }
 
+
+/* action executed after clicking in Object to 3d Sub Menu */
+void do_text_editmenu_to3dmenu(void *arg, int event)
+{
+	SpaceText *st= curarea->spacedata.first;
+	Text *text= st->text;
+	
+	switch(event) {
+	case 1: txt_export_to_object(text); break;
+	case 2: txt_export_to_objects(text); break;
+	}
+	allqueue(REDRAWVIEW3D, 0);
+}
+
+/* Object to 3d Sub Menu */
+static uiBlock *text_editmenu_to3dmenu(void *arg_unused)
+{
+	uiBlock *block;
+	short yco = 20, menuwidth = 120;
+
+	block= uiNewBlock(&curarea->uiblocks, "do_text_editmenu_to3dmenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_text_editmenu_to3dmenu, NULL);
+
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "One Object | Alt-M",		0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "One Object Per Line",		0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 2, "");
+	
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 60);
+	return block;
+}
+
+
 /* Edit menu */
 static uiBlock *text_editmenu(void *arg_unused)
 {
@@ -480,7 +507,7 @@ static uiBlock *text_editmenu(void *arg_unused)
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Find...|Alt Ctrl F", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Find Again|Alt F", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 9, "");
 	uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Convert to 3D Text|Alt M", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 10, "");
+	uiDefIconTextBlockBut(block, text_editmenu_to3dmenu, NULL, ICON_RIGHTARROW_THIN, "Text to 3d Object", 0, yco-=20, 120, 19, "");
 	
 	if(curarea->headertype==HEADERTOP) {
 		uiBlockSetDirection(block, UI_DOWN);
@@ -589,11 +616,9 @@ void text_buttons(void)
 	if(curarea->full) uiDefIconBut(block, BUT,B_FULL, ICON_SPLITSCREEN,	xco,0,XIC,YIC, 0, 0, 0, 0, 0, "Returns to multiple views window (CTRL+Up arrow)");
 	else uiDefIconBut(block, BUT,B_FULL, ICON_FULLSCREEN,	xco,0,XIC,YIC, 0, 0, 0, 0, 0, "Makes current window full screen (CTRL+Down arrow)");
 		
-	if(st->showlinenrs)
-		uiDefIconBut(block, BUT, B_TEXTLINENUM, ICON_SHORTDISPLAY, xco+=XIC,0,XIC,YIC, 0, 0, 0, 0, 0, "Hides line numbers");
-	else
-		uiDefIconBut(block, BUT, B_TEXTLINENUM, ICON_LONGDISPLAY, xco+=XIC,0,XIC,YIC, 0, 0, 0, 0, 0, "Displays line numbers");
+	uiDefIconButI(block, ICONTOG, B_TEXTLINENUM, ICON_LONGDISPLAY, xco+=XIC,0,XIC,YIC, &st->showlinenrs, 0, 0, 0, 0, "Displays line numbers");
 
+	uiDefIconButI(block, ICONTOG, B_SYNTAX, ICON_SYNTAX, xco+=XIC,0,XIC,YIC, &st->showsyntax, 0, 0, 0, 0, "Enables Syntax Highlighting");
 
 	/* STD TEXT BUTTONS */
 	xco+= 2*XIC;
