@@ -1,5 +1,5 @@
 /**
- * $Id: headerbuttons.c,v 1.90 2004/11/19 22:44:06 ton Exp $
+ * $Id: headerbuttons.c,v 1.105 2005/05/28 13:22:09 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,10 +40,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include "MEM_guardedalloc.h"
 
 #include "BMF_Api.h"
@@ -173,7 +169,7 @@
 
 #include "mydevice.h"
 #include "blendef.h"
-#include "render.h"
+#include "interface.h"
 #include "nla.h"	/* __NLA : To be removed later */
 #include "butspace.h"  // test_idbutton
 
@@ -206,7 +202,8 @@ char *windowtype_pup(void)
 	strcat(string, "|UV/Image Editor %x6"); //117
 
 	strcat(string, "|Video Sequence Editor %x8"); //143
-	strcat(string, "|Audio Timeline %x11"); //163
+	strcat(string, "|Timeline %x15"); //163
+	strcat(string, "|Audio Window %x11"); //163
 	strcat(string, "|Text Editor %x9"); //179
 
 	strcat(string, "|%l"); //192
@@ -491,50 +488,73 @@ int std_libbuttons(uiBlock *block, short xco, short yco,
 	return xco;
 }
 
-void do_update_for_newframe(int mute)
+
+
+static void do_update_for_newframe(int mute, int events)
 {
 	extern void audiostream_scrub(unsigned int frame);	/* seqaudio.c */
+	ScrArea *sa;
+	
+	if(events) {
+		allqueue(REDRAWVIEW3D, 0);
+		allqueue(REDRAWACTION,0);
+		allqueue(REDRAWNLA,0);
+		allqueue(REDRAWIPO, 0);
+		allqueue(REDRAWINFO, 1);
+		allqueue(REDRAWSEQ, 1);
+		allqueue(REDRAWSOUND, 1);
+		allqueue(REDRAWTIME, 1);
+		allqueue(REDRAWBUTSHEAD, 0);
+		allqueue(REDRAWBUTSSHADING, 0);
+		allqueue(REDRAWBUTSOBJECT, 0);
+	}
 	
-	allqueue(REDRAWVIEW3D, 0);
-	allqueue(REDRAWACTION,0);
-	allqueue(REDRAWNLA,0);
-	allqueue(REDRAWIPO, 0);
-	allqueue(REDRAWINFO, 1);
-	allqueue(REDRAWSEQ, 1);
-	allqueue(REDRAWSOUND, 1);
-	allqueue(REDRAWBUTSHEAD, 0);
-	allqueue(REDRAWBUTSSHADING, 0);
-	allqueue(REDRAWBUTSOBJECT, 0);
-
 	/* layers/materials, object ipos are calculted in where_is_object (too) */
 	do_all_ipos();
-	BPY_do_all_scripts(SCRIPT_FRAMECHANGED);
+	if (G.f & G_DOSCRIPTLINKS) BPY_do_all_scripts(SCRIPT_FRAMECHANGED);
 	clear_all_constraints();
 	do_all_keys();
 
-	do_all_actions();
+	do_all_actions(NULL);
 	rebuild_all_armature_displists();
 	/* so nice, better do it twice */
-	do_all_actions();
+	do_all_actions(NULL);
 	rebuild_all_armature_displists();
 
 	do_all_ikas();
 
 	test_all_displists();
 	
+	/* manipulators like updates too */
+	for(sa=G.curscreen->areabase.first; sa; sa=sa->next) {
+		if(sa->spacetype==SPACE_VIEW3D) {
+			View3D *v3d= sa->spacedata.first;
+			if(v3d->twflag & V3D_USE_MANIPULATOR) break;
+			else break;	// for now
+		}
+	}
+	if(sa) countall();	// does manipulator centers
+	
 	if ( (CFRA>1) && (!mute) && (G.scene->audio.flag & AUDIO_SCRUB)) audiostream_scrub( CFRA );
 }
 
 void update_for_newframe(void)
 {
-	do_update_for_newframe(0);
+	do_update_for_newframe(0, 1);
 }
 
 void update_for_newframe_muted(void)
 {
-	do_update_for_newframe(1);
+	do_update_for_newframe(1, 1);
 }
 
+/* used by new animated UI playback */
+void update_for_newframe_nodraw(int nosound)
+{
+	do_update_for_newframe(nosound, 0);
+}
+
+
 static void show_splash(void)
 {
 	extern char datatoc_splash_jpg[];
@@ -559,7 +579,7 @@ static void show_splash(void)
 /* Functions for user preferences fileselect windows */
 
 /* yafray: export dir select */
-void filesel_u_yfexportdir(char *name)
+static void filesel_u_yfexportdir(char *name)
 {
 	char dir[FILE_MAXDIR], file[FILE_MAXFILE];
 	BLI_split_dirfile(name, dir, file);
@@ -568,7 +588,7 @@ void filesel_u_yfexportdir(char *name)
 	allqueue(REDRAWALL, 0);
 }
 
-void filesel_u_fontdir(char *name)
+static void filesel_u_fontdir(char *name)
 {
 	char dir[FILE_MAXDIR], file[FILE_MAXFILE];
 	BLI_split_dirfile(name, dir, file);
@@ -577,7 +597,7 @@ void filesel_u_fontdir(char *name)
 	allqueue(REDRAWALL, 0);
 }
 
-void filesel_u_textudir(char *name)
+static void filesel_u_textudir(char *name)
 {
 	char dir[FILE_MAXDIR], file[FILE_MAXFILE];
 	BLI_split_dirfile(name, dir, file);
@@ -586,7 +606,7 @@ void filesel_u_textudir(char *name)
 	allqueue(REDRAWALL, 0);
 }
 
-void filesel_u_plugtexdir(char *name)
+static void filesel_u_plugtexdir(char *name)
 {
 	char dir[FILE_MAXDIR], file[FILE_MAXFILE];
 	BLI_split_dirfile(name, dir, file);
@@ -595,7 +615,7 @@ void filesel_u_plugtexdir(char *name)
 	allqueue(REDRAWALL, 0);
 }
 
-void filesel_u_plugseqdir(char *name)
+static void filesel_u_plugseqdir(char *name)
 {
 	char dir[FILE_MAXDIR], file[FILE_MAXFILE];
 	BLI_split_dirfile(name, dir, file);
@@ -604,7 +624,7 @@ void filesel_u_plugseqdir(char *name)
 	allqueue(REDRAWALL, 0);
 }
 
-void filesel_u_renderdir(char *name)
+static void filesel_u_renderdir(char *name)
 {
 	char dir[FILE_MAXDIR], file[FILE_MAXFILE];
 	BLI_split_dirfile(name, dir, file);
@@ -613,7 +633,7 @@ void filesel_u_renderdir(char *name)
 	allqueue(REDRAWALL, 0);
 }
 
-void filesel_u_pythondir(char *name)
+static void filesel_u_pythondir(char *name)
 {
 	char dir[FILE_MAXDIR], file[FILE_MAXFILE];
 	BLI_split_dirfile(name, dir, file);
@@ -622,7 +642,7 @@ void filesel_u_pythondir(char *name)
 	allqueue(REDRAWALL, 0);
 }
 
-void filesel_u_sounddir(char *name)
+static void filesel_u_sounddir(char *name)
 {
 	char dir[FILE_MAXDIR], file[FILE_MAXFILE];
 	BLI_split_dirfile(name, dir, file);
@@ -631,7 +651,7 @@ void filesel_u_sounddir(char *name)
 	allqueue(REDRAWALL, 0);
 }
 
-void filesel_u_tempdir(char *name)
+static void filesel_u_tempdir(char *name)
 {
 	char dir[FILE_MAXDIR], file[FILE_MAXFILE];
 	BLI_split_dirfile(name, dir, file);
@@ -1437,7 +1457,7 @@ void do_global_buttons(unsigned short event)
 				show_splash();
 		break;
 	case B_MIPMAPCHANGED:
-		set_mipmap(!(U.gameflags & USER_DISABLE_SOUND));
+		set_mipmap(!(U.gameflags & USER_DISABLE_MIPMAP));
 		allqueue(REDRAWVIEW3D, 0);
 		break;
 	case B_NEWSPACE:
@@ -1459,7 +1479,8 @@ void do_global_buttons(unsigned short event)
 		scrarea_queue_headredraw(curarea);
 		break;
 
-#ifdef _WIN32 // FULLSCREEN
+#if 0
+//#ifdef _WIN32 // FULLSCREEN
 	case B_FLIPFULLSCREEN:
 		if(U.uiflag & USER_FLIPFULLSCREEN)
 			U.uiflag &= ~USER_FLIPFULLSCREEN;
@@ -1576,6 +1597,7 @@ void do_global_buttons(unsigned short event)
 		break;
 
 	case B_SETFONTSIZE:		/* is button from space.c  *info* */
+		refresh_interface_font();
 		FTF_SetSize(U.fontsize); 
 		allqueue(REDRAWALL, 0);
 		break;
@@ -1590,6 +1612,11 @@ void do_global_buttons(unsigned short event)
 		allqueue(REDRAWALL, 0);
 		break;
 		
+	case B_USETEXTUREFONT:		/* is button from space.c  *info* */
+		refresh_interface_font();
+		allqueue(REDRAWALL, 0);
+		break;
+
 	case B_DOLANGUIFONT:	/* is button from space.c  *info* */
 		if(U.transopts & USER_DOTRANSLATE)
 			start_interface_font();
@@ -2081,7 +2108,9 @@ void do_headerbuttons(short event)
 	else if(event<600) do_file_buttons(event);
 	else if(event<650) do_seq_buttons(event);
 	else if(event<700) do_sound_buttons(event);
-	else if(event<800) do_action_buttons(event);
+	else if(event<750) do_action_buttons(event);
+	else if(event<800) do_time_buttons(curarea, event);
 	else if(event<900) do_nla_buttons(event);
+	else if(event>=REDRAWVIEW3D) allqueue(event, 0);
 }
 
