@@ -1,5 +1,5 @@
 /**
- * $Id: interface_panel.c,v 1.28 2004/12/14 18:18:31 ton Exp $
+ * $Id: interface_panel.c,v 1.30 2005/03/19 21:08:12 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -49,7 +49,6 @@
 #include <unistd.h>
 #else
 #include <io.h>
-#include "BLI_winstuff.h"
 #endif   
 
 #include "MEM_guardedalloc.h"
@@ -1517,7 +1516,7 @@ static void test_add_new_tabs(ScrArea *sa)
 /* ------------ panel drag ---------------- */
 
 
-void ui_drag_panel(uiBlock *block)
+static void ui_drag_panel(uiBlock *block)
 {
 	Panel *panel= block->panel;
 	short align=0, first=1, ofsx, ofsy, dx=0, dy=0, dxo=0, dyo=0, mval[2], mvalo[2];
