@@ -1,5 +1,5 @@
 /**
- * $Id: language.c,v 1.24 2004/05/13 14:21:08 larstiq Exp $
+ * $Id: language.c,v 1.34 2005/05/28 13:22:09 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,9 +40,11 @@
 #include "BLI_blenlib.h"
 #include "BLI_linklist.h"	/* linknode */
 
+#include "BIF_gl.h"
 #include "BIF_language.h"
 #include "BIF_space.h"		/* allqueue() */
 #include "BIF_toolbox.h"	/* error() */
+#include "datatoc.h"		/* std font */
 
 #include "MEM_guardedalloc.h"	/* vprintf, etc ??? */
 
@@ -65,20 +67,57 @@ struct _LANGMenuEntry {
 
 static LANGMenuEntry *langmenu= 0;
 static int tot_lang = 0;
+
 #endif // INTERNATIONAL
 
+void BIF_RasterPos(float x, float y)
+{
+#ifdef INTERNATIONAL
+	FTF_SetPosition(x, y);
+#endif // INTERNATIONAL
+}
+
+void BIF_SetScale(float aspect)
+{
+#ifdef INTERNATIONAL
+	FTF_SetScale(aspect);
+#endif // INTERNATIONAL
+}
+
+void refresh_interface_font(void)
+{
+#ifdef INTERNATIONAL
+	if(U.transopts & USER_DOTRANSLATE)
+		start_interface_font();
+	else
+		G.ui_international = FALSE;
+#else // INTERNATIONAL
+	G.ui_international = FALSE;
+#endif
+}
 
 int BIF_DrawString(BMF_Font* font, char *str, int translate)
 {
 
 #ifdef INTERNATIONAL
-	if(G.ui_international == TRUE)
+	if(G.ui_international == TRUE) {
 		if(translate)
 			return FTF_DrawString(str, FTF_USE_GETTEXT | FTF_INPUT_UTF8);
 		else
 			return FTF_DrawString(str, FTF_NO_TRANSCONV | FTF_INPUT_UTF8);
-	else
+	} else {
 		return BMF_DrawString(font, str);
+/*
+		glEnable(GL_TEXTURE_2D);
+		glEnable(GL_BLEND);
+		BMF_GetFontTexture(font);??
+		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+		BMF_DrawStringTexture(font, str, pen_x, pen_y, 0.0);
+		glDisable(GL_TEXTURE_2D);
+		glDisable(GL_BLEND);
+		return 0;
+*/
+	}
 #else
 	return BMF_DrawString(font, str);
 #endif
@@ -152,7 +191,7 @@ char *language_pup(void)
 }
 
 
-LANGMenuEntry *find_language(short langid)
+static LANGMenuEntry *find_language(short langid)
 {
 	LANGMenuEntry *lme = langmenu;
 
@@ -178,20 +217,18 @@ void lang_setlanguage(void)
 /* called from fileselector */
 void set_interface_font(char *str) 
 {
-	char di[FILE_MAXDIR];
 
 	/* this test needed because fileselect callback can happen after disable AA fonts */
 	if(U.transopts & USER_DOTRANSLATE) {
-		if(FTF_SetFont(str, U.fontsize)) {
+		if(FTF_SetFont(str, 0, U.fontsize)) {
 			lang_setlanguage();
-			BLI_split_dirfile(str, di, U.fontname);
-	
-			if(strlen(di) < FILE_MAXDIR) strcpy(U.fontdir, di);
-	
+			
+			if(strlen(str) < FILE_MAXDIR) strcpy(U.fontname, str);
 			G.ui_international = TRUE;
-		} else {
-			sprintf(U.fontname, "Invalid font.");
-			G.ui_international = FALSE;
+		} 
+		else {
+			U.fontname[0]= 0;
+			G.ui_international = TRUE;	// this case will switch to standard font
 		}
 		allqueue(REDRAWALL, 0);
 	}
@@ -200,69 +237,37 @@ void set_interface_font(char *str)
 
 void start_interface_font(void) 
 {
-	char tstr[FILE_MAXDIR+FILE_MAXFILE];
 	int result = 0;
-#ifdef __APPLE__
-	char *bundlepath;
-#endif
+
+	if(U.transopts & USER_USETEXTUREFONT)
+		FTF_SetMode(FTF_TEXTUREFONT);
+	else
+		FTF_SetMode(FTF_PIXMAPFONT);
 	
-	/* hack to find out if we have saved language/font settings.
-	   if not, set defaults and try .bfont.tff --phase */
+	if(U.fontsize && U.fontname[0] ) { // we have saved user settings + fontpath
+		
+		// try loading font from U.fontname = full path to font in usersettings
+		result = FTF_SetFont(U.fontname, 0, U.fontsize);
+	}
+	else if(U.fontsize) {	// user settings, default
+		result = FTF_SetFont(datatoc_bfont_ttf, datatoc_bfont_ttf_size, U.fontsize);
+	}
 	
-	if(U.fontsize != 0) { // we have saved user settings
-		// try load the font from the font dir
-		BLI_make_file_string("/", tstr, U.fontdir, U.fontname);
-		result = FTF_SetFont(tstr, U.fontsize);
-
-		if(!result) {	// else try loading font from current dir
-			result = FTF_SetFont(U.fontname, U.fontsize);
-		}
-
-		// try home dir (special case for .bfont.ttf) (aphex)
-
-		if(!result) {
-			strcpy(tstr, BLI_gethome());
-			if (strstr(tstr,".blender") == 0) {
-				strcat(tstr,".blender/");
-			}
-			strcat(tstr, U.fontname);
-			result = FTF_SetFont(tstr, U.fontsize);
-		}
-	} else {
+	if(result==0) {		// use default
 		U.language= 0;
 		U.fontsize= 11;
 		U.encoding= 0;
-
-#if defined (__APPLE__)
-		bundlepath = BLI_getbundle();
-		strcpy(tstr, bundlepath);
-		strcat(tstr, "/Contents/Resources/");
-		strcat(tstr, ".bfont.ttf");
-		result = FTF_SetFont(tstr, U.fontsize);
-
-		//sprintf(U.fontname, ".blender/.bfont.ttf");
-		strncpy(U.fontname, tstr, 255);		
-		
-#elif defined (WIN32)
-		strcpy(tstr, BLI_gethome());
-		strcat(tstr, "/.bfont.ttf\0");
-		result = FTF_SetFont(tstr, U.fontsize);
-
-		sprintf(U.fontname, "/.bfont.ttf\0");
-#else
-		strcpy(tstr, BLI_gethome());
-		strcat(tstr, "/.blender/.bfont.ttf");
-		result = FTF_SetFont(tstr, U.fontsize);
-
-		strncpy(U.fontname, tstr, 255);		
-#endif
+		U.fontname[0]= 0;
+		result = FTF_SetFont(datatoc_bfont_ttf, datatoc_bfont_ttf_size, U.fontsize);
 	}
 
 	if(result) {
+		
 		lang_setlanguage();
 
 		G.ui_international = TRUE;
-	} else {
+	} 
+	else {
 		printf("no font found for international support\n");
 		G.ui_international = FALSE;
 		U.transopts &= ~USER_DOTRANSLATE;
@@ -273,7 +278,7 @@ void start_interface_font(void)
 }
 
 
-char *first_dpointchar(char *string) 
+static char *first_dpointchar(char *string) 
 {
 	char *dpointchar;
 	
@@ -283,7 +288,7 @@ char *first_dpointchar(char *string)
 }
 
 
-void splitlangline(char *line, LANGMenuEntry *lme)
+static void splitlangline(char *line, LANGMenuEntry *lme)
 {
 	char *dpointchar= first_dpointchar(line);
 
@@ -297,7 +302,7 @@ void splitlangline(char *line, LANGMenuEntry *lme)
 }
 
 
-void puplang_insert_entry(char *line)
+static void puplang_insert_entry(char *line)
 {
 	LANGMenuEntry *lme, *prev;
 	int sorted = 0;
@@ -335,12 +340,13 @@ int read_languagefile(void)
 {
 	char name[FILE_MAXDIR+FILE_MAXFILE];
 	LinkNode *l, *lines;
-#ifdef WIN32
-	int result;
-#endif
 	
-	/* .Blanguages */
+	/* .Blanguages, http://www.blender3d.org/cms/Installation_Policy.352.0.html*/
+#if defined (__APPLE__) || (WIN32)
 	BLI_make_file_string("/", name, BLI_gethome(), ".Blanguages");
+#else
+	BLI_make_file_string("/", name, BLI_gethome(), ".blender/.Blanguages");
+#endif
 
 	lines= BLI_read_file_as_lines(name);
 
@@ -351,11 +357,9 @@ int read_languagefile(void)
 		char *bundlePath = BLI_getbundle();
 		strcpy(name, bundlePath);
 		strcat(name, "/Contents/Resources/.Blanguages");
-#elif defined (WIN32)
-		/* Check the installation dir in Windows */
-		strcpy(name, BLI_gethome());
-		strcat(name,"/.Blanguages");
 #else
+		/* Check the CWD. Takes care of the case where users
+		 * unpack blender tarball; cd blender-dir; ./blender */
 		strcpy(name, ".blender/.Blanguages");
 #endif
 		lines= BLI_read_file_as_lines(name);
@@ -365,8 +369,8 @@ int read_languagefile(void)
 			strcpy(name, ".Blanguages");
 			lines= BLI_read_file_as_lines(name);
 			if(lines == NULL) {
-					error("File \".Blanguages\" not found");
-					return 0;
+				if(G.f & G_DEBUG) printf("File .Blanguages not found\n");
+				return 0;
 			}
 		}
 	}
