@@ -2,7 +2,7 @@
  * 
  * maart 95
  * 
- * $Id: previewrender.c,v 1.44 2004/12/06 18:05:42 ton Exp $
+ * $Id: previewrender.c,v 1.58 2005/04/17 17:43:07 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -61,6 +61,7 @@
 #include "DNA_world_types.h"
 #include "DNA_camera_types.h"
 #include "DNA_image_types.h"
+#include "DNA_material_types.h"
 #include "DNA_object_types.h"
 #include "DNA_lamp_types.h"
 #include "DNA_space_types.h"
@@ -80,16 +81,18 @@
 #include "BIF_screen.h"
 #include "BIF_space.h"		/* allqueue */
 #include "BIF_butspace.h"	
-#include "BIF_drawimage.h"	/* rectwrite_part */
 #include "BIF_mywindow.h"
 #include "BIF_interface.h"
 #include "BIF_glutil.h"
 
+#include "BIF_previewrender.h"  /* include ourself for prototypes */
+
 #include "PIL_time.h"
 
 #include "RE_renderconverter.h"
 
 #include "blendef.h"	/* CLAMP */
+#include "interface.h"	/* ui_graphics_to_window() SOLVE! (ton) */
 
 #define PR_RECTX	141
 #define PR_RECTY	141
@@ -100,8 +103,7 @@
 
 #define PR_FACY		(PR_YMAX-PR_YMIN-4)/(PR_RECTY)
 
-static rcti prerect;
-static int pr_sizex, pr_sizey;
+static rctf prerect;
 static float pr_facx, pr_facy;
 
 
@@ -205,9 +207,9 @@ static int ray_previewrender(int x,  int y,  float *vec, float *vn)
 		
 		CalcNormFloat(rcubev[rcubi[hitface][0]], rcubev[rcubi[hitface][1]], rcubev[rcubi[hitface][2]], vn);
 		
-		vec[0]= (minlabda*(ray1[0]-ray2[0])+ray2[0])/3.7;
-		vec[1]= (minlabda*(ray1[1]-ray2[1])+ray2[1])/3.7;
-		vec[2]= (minlabda*(ray1[2]-ray2[2])+ray2[2])/3.7;
+		vec[0]= (minlabda*(ray1[0]-ray2[0])+ray2[0])/4.1;
+		vec[1]= (minlabda*(ray1[1]-ray2[1])+ray2[1])/4.1;
+		vec[2]= (minlabda*(ray1[2]-ray2[2])+ray2[2])/4.1;
 		
 		return 1;
 	}
@@ -235,61 +237,67 @@ static unsigned int previewback(int type, int x, int y)
 	}
 }
 
-static void view2d_to_window(int win, int *x_r, int *y_r)
-{
-	int x= *x_r, y= *y_r;
-	int size[2], origin[2];
-	float winmat[4][4];
-
-	bwin_getsinglematrix(win, winmat);
-	bwin_getsize(win, &size[0], &size[1]);
-	bwin_getsuborigin(win, &origin[0], &origin[1]);
-	
-	*x_r= origin[0] + (size[0]*(0.5 + 0.5*(x*winmat[0][0] + y*winmat[1][0] + winmat[3][0])));
-	*y_r= origin[1] + (size[1]*(0.5 + 0.5*(x*winmat[0][1] + y*winmat[1][1] + winmat[3][1])));
-}
-
 static void set_previewrect(int win, int xmin, int ymin, int xmax, int ymax)
 {
+	float pr_sizex, pr_sizey;
+	
 	prerect.xmin= xmin;
 	prerect.ymin= ymin;
 	prerect.xmax= xmax;
 	prerect.ymax= ymax;
 
-	view2d_to_window(win, &prerect.xmin, &prerect.ymin);
-	view2d_to_window(win, &prerect.xmax, &prerect.ymax);
+	ui_graphics_to_window(win, &prerect.xmin, &prerect.ymin);
+	ui_graphics_to_window(win, &prerect.xmax, &prerect.ymax);
 	
 	pr_sizex= (prerect.xmax-prerect.xmin);
 	pr_sizey= (prerect.ymax-prerect.ymin);
 
-	pr_facx= ( (float)pr_sizex-1)/PR_RECTX;
-	pr_facy= ( (float)pr_sizey-1)/PR_RECTY;
+	pr_facx= ( pr_sizex-1.0)/PR_RECTX;
+	pr_facy= ( pr_sizey-1.0)/PR_RECTY;
+
+	/* correction for gla draw */
+	prerect.xmin-= curarea->winrct.xmin;
+	prerect.ymin-= curarea->winrct.ymin;
+	
+	glMatrixMode(GL_PROJECTION);
+	glPushMatrix();
+	glMatrixMode(GL_MODELVIEW);
+	glPushMatrix();
+	
+	glaDefine2DArea(&curarea->winrct);
+
+	glPixelZoom(pr_facx, pr_facy);
+	
+}
+
+static void end_previewrect(void)
+{
+	glMatrixMode(GL_PROJECTION);
+	glPopMatrix();
+	glMatrixMode(GL_MODELVIEW);
+	glPopMatrix();
+	
+	glPixelZoom(1.0, 1.0);
+	
+	// restore viewport / scissor which was set by glaDefine2DArea
+	glViewport(curarea->winrct.xmin, curarea->winrct.ymin, curarea->winx, curarea->winy);
+	glScissor(curarea->winrct.xmin, curarea->winrct.ymin, curarea->winx, curarea->winy);
+
 }
 
 static void display_pr_scanline(unsigned int *rect, int recty)
 {
-	static double lasttime= 0;
-	/* we display 3 new scanlines, one old, the overlap is for wacky 3d cards that cant handle zoom proper */
-
-	if(recty % 2) return;
-	if(recty<2) return;
 	
-	rect+= (recty-2)*PR_RECTX;
-
-	/* enlarge a bit in the y direction, to avoid GL/mesa bug */
-	glPixelZoom(pr_facx, pr_facy);
-
-	glRasterPos2f( (float)PR_XMIN+0.5, 1.0+(float)PR_YMIN + (recty*PR_FACY) );
-	glDrawPixels(PR_RECTX, 3, GL_RGBA, GL_UNSIGNED_BYTE,  rect);
-
-	//glaDrawPixelsTex((float)PR_XMIN, (float)PR_YMIN + (recty*PR_FACY), PR_RECTX, 3, rect);
-
-	glPixelZoom(1.0, 1.0);
-	
-	/* flush opengl for cards with frontbuffer slowness */
-	if(recty==PR_RECTY-1 || (PIL_check_seconds_timer() - lasttime > 0.05)) {
-		lasttime= PIL_check_seconds_timer();
-		glFlush();
+	/* we do steps of 4 scanlines. but draw 5, because of errors in some gfx cards (nvidia geforce, ati...) */
+	if( (recty & 3)==3) {
+		
+		if(recty == 3) {
+			glaDrawPixelsSafe(prerect.xmin, prerect.ymin, PR_RECTX, 4, rect);
+		}
+		else {
+			rect+= (recty-4)*PR_RECTX;
+			glaDrawPixelsSafe(prerect.xmin, prerect.ymin + (((float)recty-4.0)*pr_facy), PR_RECTX, 5, rect);
+		}
 	}
 }
 
@@ -370,21 +378,24 @@ void BIF_previewdraw(void)
 {
 	SpaceButs *sbuts= curarea->spacedata.first;
 	
-	set_previewrect(sbuts->area->win, PR_XMIN, PR_YMIN, PR_XMAX, PR_YMAX);
-
 	if (sbuts->rect==0) BIF_preview_changed(sbuts);
 	else {
 		int y;
 
+		set_previewrect(sbuts->area->win, PR_XMIN, PR_YMIN, PR_XMAX, PR_YMAX);
+		
 		for (y=0; y<PR_RECTY; y++) {
 			display_pr_scanline(sbuts->rect, y);
 		}
 
+		end_previewrect();
+		
 		if (sbuts->mainb==CONTEXT_SHADING && sbuts->tab[CONTEXT_SHADING]==TAB_SHADING_TEX) {
 			draw_tex_crop(sbuts->lockpoin);
 		}
 	}
 	if(sbuts->cury==0) BIF_preview_changed(sbuts);
+	
 }
 
 static void sky_preview_pixel(float lens, int x, int y, char *rect)
@@ -407,7 +418,7 @@ static void sky_preview_pixel(float lens, int x, int y, char *rect)
 
 static void lamp_preview_pixel(ShadeInput *shi, LampRen *la, int x, int y, char *rect)
 {
-	float inpr, i, t, dist, distkw, vec[3];
+	float inpr, i, t, dist, distkw, vec[3], lacol[3];
 	int col;
 	
 	shi->co[0]= (float)x/(PR_RECTX/4);
@@ -420,7 +431,11 @@ static void lamp_preview_pixel(ShadeInput *shi, LampRen *la, int x, int y, char
 	VECCOPY(shi->view, vec);
 	dist= Normalise(shi->view);
 
-	if(la->mode & LA_TEXTURE) do_lamp_tex(la, vec, shi);
+	lacol[0]= la->r;
+	lacol[1]= la->g;
+	lacol[2]= la->b;
+	
+	if(la->mode & LA_TEXTURE) do_lamp_tex(la, vec, shi, lacol);
 
 	if(la->type==LA_SUN || la->type==LA_HEMI) {
 		dist= 1.0;
@@ -471,13 +486,13 @@ static void lamp_preview_pixel(ShadeInput *shi, LampRen *la, int x, int y, char
 	}
 	else if ELEM(la->type, LA_LOCAL, LA_AREA) dist*= shi->view[2];
 	
-	col= 255.0*dist*la->r;
+	col= 255.0*dist*lacol[0];
 	if(col<=0) rect[0]= 0; else if(col>=255) rect[0]= 255; else rect[0]= col;
 
-	col= 255.0*dist*la->g;
+	col= 255.0*dist*lacol[1];
 	if(col<=0) rect[1]= 0; else if(col>=255) rect[1]= 255; else rect[1]= col;
 
-	col= 255.0*dist*la->b;
+	col= 255.0*dist*lacol[2];
 	if(col<=0) rect[2]= 0; else if(col>=255) rect[2]= 255; else rect[2]= col;
 }
 
@@ -492,9 +507,9 @@ static void init_previewhalo(HaloRen *har, Material *mat)
 	har->radsq= PR_RECTX*PR_RECTX/4.0;
 	har->alfa= mat->alpha;
 	har->add= 255.0*mat->add;
-	har->r= 255.0*mat->r;
-	har->g= 255.0*mat->g; 
-	har->b= 255.0*mat->b;
+	har->r= mat->r;
+	har->g= mat->g; 
+	har->b= mat->b;
 	har->xs= PR_RECTX/2.0;
 	har->ys= PR_RECTX/2.0;
 	har->zs= har->zd= 0;
@@ -520,7 +535,7 @@ static void init_previewhalo(HaloRen *har, Material *mat)
 
 static void halo_preview_pixel(HaloRen *har, int startx, int endx, int y, char *rect)
 {
-	float dist, xn, yn, xsq, ysq;
+	float dist, xn, yn, xsq, ysq, colf[4];
 	int x;
 	char front[4];
 	
@@ -536,10 +551,8 @@ static void halo_preview_pixel(HaloRen *har, int startx, int endx, int y, char *
 		xsq= xn*xn;
 		dist= xsq+ysq;
 
-		
-		
 		if(dist<har->radsq) {
-			RE_shadehalo(har, front, 0, dist, xn, yn, har->flarec);
+			RE_shadehalo(har, front, colf, 0, dist, xn, yn, har->flarec);
 			RE_addalphaAddfac(rect, front, har->add);
 		}
 		rect+= 4;
@@ -551,7 +564,7 @@ static void previewflare(SpaceButs *sbuts, HaloRen *har, unsigned int *rect)
 	uiBlock *block;
 	float ycor;
 	unsigned int *rectot;
-	int afmx, afmy, rectx, recty;
+	int afmx, afmy, rectx, recty, y;
 	
 	block= uiFindOpenPanelBlockName(&curarea->uiblocks, "Preview");
 	if(block==NULL) return;
@@ -564,6 +577,8 @@ static void previewflare(SpaceButs *sbuts, HaloRen *har, unsigned int *rect)
 	afmy= R.afmy;
 	rectot= R.rectot;
 
+	R.r.postmul= R.r.postgamma= R.r.postsat= 1.0;
+	R.r.posthue= R.r.postadd= 0.0;
 	R.ycor= 1.0;
 	R.rectx= PR_RECTX;	
 	R.recty= PR_RECTY;
@@ -574,13 +589,13 @@ static void previewflare(SpaceButs *sbuts, HaloRen *har, unsigned int *rect)
 	waitcursor(1);
 	RE_renderflare(har);
 	waitcursor(0);
-	// not sure why, either waitcursor or renderflare screws up
-	areawinset(curarea->win);
-	
-	uiPanelPush(block);
-	BIF_previewdraw();
-	uiPanelPop(block);
+	// not sure why, either waitcursor or renderflare screws up (disabled then)
+	//areawinset(curarea->win);
 	
+	/* draw can just be this way, all settings are OK */
+	for (y=0; y<PR_RECTY; y++) {
+		display_pr_scanline(sbuts->rect, y);
+	}
 	
 	/* temps */
 	R.ycor= ycor;
@@ -591,12 +606,12 @@ static void previewflare(SpaceButs *sbuts, HaloRen *har, unsigned int *rect)
 	R.rectot= rectot;
 }
 
-extern float Tin, Tr, Tg, Tb, Ta; /* texture.c */
 static void texture_preview_pixel(Tex *tex, int x, int y, char *rect)
 {
-	float i, v1, xsq, ysq, texvec[3], dummy[3];
+	float i, v1, xsq, ysq, texvec[3];
+	float tin=1.0, tr, tg, tb, ta;
 	int rgbnor, tracol, skip=0;
-		
+	
 	if(tex->type==TEX_IMAGE) {
 		v1= 1.0/PR_RECTX;
 		
@@ -637,12 +652,12 @@ static void texture_preview_pixel(Tex *tex, int x, int y, char *rect)
 			}
 			else {
 				skip= 1;
-				Ta= 0.0;
+				ta= 0.0;
 			}
 		}
 		else {
 			skip= 1;
-			Ta= 0.0;
+			ta= 0.0;
 		}
 	}
 	else {
@@ -653,44 +668,32 @@ static void texture_preview_pixel(Tex *tex, int x, int y, char *rect)
 		texvec[2]= 0.0;
 	}
 	
-	/* does not return Tin */
-	if(tex->type==TEX_STUCCI) {
-		tex->nor= dummy;
-		dummy[0]= 1.0;
-		dummy[1]= dummy[2]= 0.0;
-	}
-	
-	if(skip==0) rgbnor= multitex(tex, texvec, NULL, NULL, 0);
+	if(skip==0) rgbnor= multitex_ext(tex, texvec, &tin, &tr, &tg, &tb, &ta);
 	else rgbnor= 1;
 	
 	if(rgbnor & 1) {
 		
-		v1= 255.0*Tr;
+		v1= 255.0*tr;
 		rect[0]= CLAMPIS(v1, 0, 255);
-		v1= 255.0*Tg;
+		v1= 255.0*tg;
 		rect[1]= CLAMPIS(v1, 0, 255);
-		v1= 255.0*Tb;
+		v1= 255.0*tb;
 		rect[2]= CLAMPIS(v1, 0, 255);
 		
-		if(Ta!=1.0) {
+		if(ta!=1.0) {
 			tracol=  64+100*(abs(x)>abs(y));
-			tracol= (1.0-Ta)*tracol;
+			tracol= (1.0-ta)*tracol;
 			
-			rect[0]= tracol+ (rect[0]*Ta) ;
-			rect[1]= tracol+ (rect[1]*Ta) ;
-			rect[2]= tracol+ (rect[2]*Ta) ;
+			rect[0]= tracol+ (rect[0]*ta) ;
+			rect[1]= tracol+ (rect[1]*ta) ;
+			rect[2]= tracol+ (rect[2]*ta) ;
 					
 		}
 	}
 	else {
-	
-		if(tex->type==TEX_STUCCI) {
-			Tin= 0.5 + 0.7*tex->nor[0];
-			CLAMP(Tin, 0.0, 1.0);
-		}
-		rect[0]= 255.0*Tin;
-		rect[1]= 255.0*Tin;
-		rect[2]= 255.0*Tin;
+		rect[0]= 255.0*tin;
+		rect[1]= 255.0*tin;
+		rect[2]= 255.0*tin;
 	}
 }
 
@@ -740,14 +743,23 @@ static void shade_preview_pixel(ShadeInput *shi, float *vec, int x, int y,char *
 	int temp, a;
 	char tracol;
 		
-	mat= shi->matren;
+	mat= shi->mat;
 
+	// copy all relevant material vars, note, keep this synced with render_types.h
+	memcpy(&shi->r, &mat->r, 23*sizeof(float));
+	// set special cases:
+	shi->har= mat->har;
+	if((mat->mode & MA_RAYMIRROR)==0) shi->ray_mirror= 0.0;
+	
 	v1= 1.0/PR_RECTX;
 	shi->view[0]= v1*x;
 	shi->view[1]= v1*y;
 	shi->view[2]= 1.0;
 	Normalise(shi->view);
 	
+	shi->xs= (float)x;
+	shi->ys= (float)y;
+	
 	shi->refcol[0]= shi->refcol[1]= shi->refcol[2]= shi->refcol[3]= 0.0;
 
 	/* texture handling */
@@ -824,7 +836,7 @@ static void shade_preview_pixel(ShadeInput *shi, float *vec, int x, int y,char *
 		
 	}
 	/* set it here, because ray_mirror will affect it */
-	alpha= mat->alpha;
+	alpha= shi->alpha;
 
 	if(mat->mapto & MAP_DISPLACE) { /* Quick hack of fake displacement preview */
 		shi->vn[0]-=2.0*shi->displace[2];
@@ -838,13 +850,13 @@ static void shade_preview_pixel(ShadeInput *shi, float *vec, int x, int y,char *
 			alpha*= fresnel_fac(shi->view, shi->vn, mat->fresnel_tra_i, mat->fresnel_tra);
 
 	if(mat->mode & MA_SHLESS) {
-		temp= 255.0*(mat->r);
+		temp= 255.0*(shi->r);
 		if(temp>255) rect[0]= 255; else if(temp<0) rect[0]= 0; else rect[0]= temp;
 
-		temp= 255.0*(mat->g);
+		temp= 255.0*(shi->g);
 		if(temp>255) rect[1]= 255; else if(temp<0) rect[1]= 0; else rect[1]= temp;
 
-		temp= 255.0*(mat->b);
+		temp= 255.0*(shi->b);
 		if(temp>255) rect[2]= 255; else if(temp<0) rect[2]= 0; else rect[2]= temp;
 	}
 	else {
@@ -864,22 +876,24 @@ static void shade_preview_pixel(ShadeInput *shi, float *vec, int x, int y,char *
 			is= shi->vn[0]*lv[0]+shi->vn[1]*lv[1]+shi->vn[2]*lv[2];
 			if(is<0.0) is= 0.0;
 			
-			if(mat->spec)  {
+			if(shi->spec>0.0)  {
 				
 				if(is>0.0) {
 					/* specular shaders */
 					float specfac;
 					
 					if(mat->spec_shader==MA_SPEC_PHONG) 
-						specfac= Phong_Spec(shi->vn, lv, shi->view, mat->har);
+						specfac= Phong_Spec(shi->vn, lv, shi->view, shi->har);
 					else if(mat->spec_shader==MA_SPEC_COOKTORR) 
-						specfac= CookTorr_Spec(shi->vn, lv, shi->view, mat->har);
+						specfac= CookTorr_Spec(shi->vn, lv, shi->view, shi->har);
 					else if(mat->spec_shader==MA_SPEC_BLINN) 
-						specfac= Blinn_Spec(shi->vn, lv, shi->view, mat->refrac, (float)mat->har);
+						specfac= Blinn_Spec(shi->vn, lv, shi->view, mat->refrac, (float)shi->har);
+					else if(mat->spec_shader==MA_SPEC_WARDISO)
+						specfac= WardIso_Spec(shi->vn, lv, shi->view, mat->rms);
 					else 
 						specfac= Toon_Spec(shi->vn, lv, shi->view, mat->param[2], mat->param[3]);
 				
-					inprspec= specfac*mat->spec;
+					inprspec= specfac*shi->spec;
 					
 					if(mat->mode & MA_RAMP_SPEC) {
 						float spec[3];
@@ -889,18 +903,19 @@ static void shade_preview_pixel(ShadeInput *shi, float *vec, int x, int y,char *
 						isb+= inprspec*spec[2];
 					}
 					else {	
-						isr+= inprspec*mat->specr;
-						isg+= inprspec*mat->specg;
-						isb+= inprspec*mat->specb;
+						isr+= inprspec*shi->specr;
+						isg+= inprspec*shi->specg;
+						isb+= inprspec*shi->specb;
 					}
 				}
 			}
 			/* diffuse shaders */
 			if(mat->diff_shader==MA_DIFF_ORENNAYAR) is= OrenNayar_Diff(shi->vn, lv, shi->view, mat->roughness);
 			else if(mat->diff_shader==MA_DIFF_TOON) is= Toon_Diff(shi->vn, lv, shi->view, mat->param[0], mat->param[1]);
+			else if(mat->diff_shader==MA_DIFF_MINNAERT) is= Minnaert_Diff(is, shi->vn, shi->view, mat->darkness);
 			// else Lambert
 			
-			inp= (mat->ref*is + mat->emit);
+			inp= (shi->refl*is + shi->emit);
 			
 			if(a==0) la= pr1_col;
 			else la= pr2_col;
@@ -924,7 +939,7 @@ static void shade_preview_pixel(ShadeInput *shi, float *vec, int x, int y,char *
 			/* scale */
 			div= (0.85*shi->ref[1]);
 			
-			shi->refcol[0]= mat->ray_mirror*fresnel_fac(shi->view, shi->vn, mat->fresnel_mir_i, mat->fresnel_mir);
+			shi->refcol[0]= shi->ray_mirror*fresnel_fac(shi->view, shi->vn, mat->fresnel_mir_i, mat->fresnel_mir);
 			/* not real 'alpha', but mirror overriding transparency */
 			if(mat->mode & MA_RAYTRANSP) {
 				float fac= sqrt(shi->refcol[0]);
@@ -953,55 +968,66 @@ static void shade_preview_pixel(ShadeInput *shi, float *vec, int x, int y,char *
 		if(mat->mode & MA_RAMP_SPEC) ramp_spec_result(&isr, &isg, &isb, shi);
 		
 		if(shi->refcol[0]==0.0) {
-			a= 255.0*(diff[0] +mat->ambr +isr);
+			a= 255.0*(diff[0] +shi->ambr +isr);
 			if(a>255) a=255; else if(a<0) a= 0;
 			rect[0]= a;
-			a= 255.0*(diff[1] +mat->ambg +isg);
+			a= 255.0*(diff[1] +shi->ambg +isg);
 			if(a>255) a=255; else if(a<0) a= 0;
 			rect[1]= a;
-			a= 255*(diff[2] +mat->ambb +isb);
+			a= 255*(diff[2] +shi->ambb +isb);
 			if(a>255) a=255; else if(a<0) a= 0;
 			rect[2]= a;
 		}
 		else {
-			a= 255.0*( mat->mirr*shi->refcol[1] + (1.0 - mat->mirr*shi->refcol[0])*(diff[0] +mat->ambr) +isr);
+			a= 255.0*( shi->mirr*shi->refcol[1] + (1.0 - shi->mirr*shi->refcol[0])*(diff[0] +shi->ambr) +isr);
 			if(a>255) a=255; else if(a<0) a= 0;
 			rect[0]= a;
-			a= 255.0*( mat->mirg*shi->refcol[2] + (1.0 - mat->mirg*shi->refcol[0])*(diff[1] +mat->ambg) +isg);
+			a= 255.0*( shi->mirg*shi->refcol[2] + (1.0 - shi->mirg*shi->refcol[0])*(diff[1] +shi->ambg) +isg);
 			if(a>255) a=255; else if(a<0) a= 0;
 			rect[1]= a;
-			a= 255.0*( mat->mirb*shi->refcol[3] + (1.0 - mat->mirb*shi->refcol[0])*(diff[2] +mat->ambb) +isb);
+			a= 255.0*( shi->mirb*shi->refcol[3] + (1.0 - shi->mirb*shi->refcol[0])*(diff[2] +shi->ambb) +isb);
 			if(a>255) a=255; else if(a<0) a= 0;
 			rect[2]= a;
 		}
 	}
 
 		/* ztra shade */
-	if(mat->spectra!=0.0) {
+	if(shi->spectra!=0.0) {
 		inp = MAX3(isr, isg, isb);
-		inp *= mat->spectra;
+		inp *= shi->spectra;
 		if(inp>1.0) inp= 1.0;
 		alpha= (1.0-inp)*alpha+inp;
 	}
 
 	if(alpha!=1.0) {
 		if(mat->mode & MA_RAYTRANSP) {
-			refraction_prv(&x, &y, shi->vn, mat->ang);
+			refraction_prv(&x, &y, shi->vn, shi->ang);
 		}
 		
 		tracol=  previewback(mat->pr_back, x, y) & 255;
 		
 		tracol= (1.0-alpha)*tracol;
 		
-		rect[0]= tracol+ (rect[0]*alpha) ;
-		rect[1]= tracol+ (rect[1]*alpha) ;
-		rect[2]= tracol+ (rect[2]*alpha) ;
+		if((mat->mode & MA_RAYTRANSP) && mat->filter!=0.0) {
+			float fr= 1.0+ mat->filter*(shi->r-1.0);
+			rect[0]= fr*tracol+ (rect[0]*alpha) ;
+			fr= 1.0+ mat->filter*(shi->g-1.0);
+			rect[1]= fr*tracol+ (rect[1]*alpha) ;
+			fr= 1.0+ mat->filter*(shi->b-1.0);
+			rect[2]= fr*tracol+ (rect[2]*alpha) ;
+		}
+		else {
+			rect[0]= tracol+ (rect[0]*alpha) ;
+			rect[1]= tracol+ (rect[1]*alpha) ;
+			rect[2]= tracol+ (rect[2]*alpha) ;
+		}
 	}
 }
 
 
 void BIF_previewrender(SpaceButs *sbuts)
 {
+	static double lasttime= 0;
 	ID *id, *idfrom;
 	Material *mat= NULL;
 	Tex *tex= NULL;
@@ -1091,8 +1117,8 @@ void BIF_previewrender(SpaceButs *sbuts)
 			}
 		}
 		shi.vlr= 0;
+		
 		shi.mat= mat;
-		shi.matren= mat->ren;
 		
 		if(mat->mode & MA_HALO) init_previewhalo(&har, mat);
 	}
@@ -1139,7 +1165,9 @@ void BIF_previewrender(SpaceButs *sbuts)
 		init_render_textures();	/* dont do it twice!! (brightness) */
 	}
 
-	set_previewrect(sbuts->area->win, PR_XMIN, PR_YMIN, PR_XMAX, PR_YMAX);
+	uiPanelPush(block);	// sets UImat
+	
+	set_previewrect(sbuts->area->win, PR_XMIN, PR_YMIN, PR_XMAX, PR_YMAX); // uses UImat
 
 	if(sbuts->rect==NULL) {
 		sbuts->rect= MEM_callocN(sizeof(int)*PR_RECTX*PR_RECTY, "butsrect");
@@ -1175,7 +1203,6 @@ void BIF_previewrender(SpaceButs *sbuts)
 
 	/* here it starts! */
 	glDrawBuffer(GL_FRONT);
-	uiPanelPush(block);
 
 	for(y=starty; y<endy; y++) {
 		
@@ -1266,9 +1293,17 @@ void BIF_previewrender(SpaceButs *sbuts)
 
 		display_pr_scanline(sbuts->rect, sbuts->cury);
 		
+		/* flush opengl for cards with frontbuffer slowness */
+		if(sbuts->cury==PR_RECTY-1 || (PIL_check_seconds_timer() - lasttime > 0.05)) {
+			lasttime= PIL_check_seconds_timer();
+			glFlush();
+		}
+		
 		sbuts->cury++;
 	}
 
+	end_previewrect();
+	
 	if(sbuts->cury>=PR_RECTY && tex) 
 		if (sbuts->tab[CONTEXT_SHADING]==TAB_SHADING_TEX) 
 			draw_tex_crop(sbuts->lockpoin);
@@ -1293,7 +1328,6 @@ void BIF_previewrender(SpaceButs *sbuts)
 	}
 	else if(la) {
 		if(R.totlamp) {
-			if(R.la[0]->org) MEM_freeN(R.la[0]->org);
 			MEM_freeN(R.la[0]);
 		}
 		R.totlamp= 0;
