@@ -1,5 +1,5 @@
 /**
- * $Id: renderwin.c,v 1.34 2004/04/27 12:17:16 ton Exp $
+ * $Id: renderwin.c,v 1.53 2005/06/06 13:10:02 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -53,6 +53,8 @@
 
 #endif
 
+#include <limits.h>
+
 #include "BLI_blenlib.h"
 
 #include "MEM_guardedalloc.h"
@@ -72,10 +74,13 @@
 #include "BIF_screen.h"
 #include "BIF_space.h"
 #include "BIF_mywindow.h"
+#include "BIF_previewrender.h"
 #include "BIF_renderwin.h"
+#include "BIF_resources.h"
 #include "BIF_toets.h"
 
 #include "BDR_editobject.h"
+#include "BPY_extern.h" /* for BPY_do_all_scripts */
 
 #include "BSE_view.h"
 #include "BSE_drawview.h"
@@ -108,6 +113,11 @@
 */
 #define RW_FLAGS_PIXEL_EXAMINING	(1<<3)
 
+/* forces draw of alpha */
+#define RW_FLAGS_ALPHA		(1<<4)
+
+/* space for info text */
+#define RW_HEADERY		18
 
 typedef struct {
 	Window *win;
@@ -123,6 +133,8 @@ typedef struct {
 	float pan_mouse_start[2], pan_ofs_start[2];
 
 	char *info_text;
+	char *render_text, *render_text_spare;
+	
 } RenderWin;
 
 static RenderWin *render_win= NULL;
@@ -140,6 +152,7 @@ static RenderWin *renderwin_alloc(Window *win)
 	rw->flags= 0;
 	rw->zoomofs[0]= rw->zoomofs[1]= 0;
 	rw->info_text= NULL;
+	rw->render_text= rw->render_text_spare= NULL;
 
 	rw->lmouse[0]= rw->lmouse[1]= 0;
 	rw->mbut[0]= rw->mbut[1]= rw->mbut[2]= 0;
@@ -165,6 +178,7 @@ static void renderwin_get_disprect(RenderWin *rw, float disprect_r[2][2])
 	int w, h;
 
 	window_get_size(rw->win, &w, &h);
+	h-= RW_HEADERY;
 
 	display_w= R.rectx*rw->zoom;
 	display_h= R.recty*rw->zoom;
@@ -202,6 +216,7 @@ static int renderwin_win_to_ndc(RenderWin *rw, int win_co[2], float ndc_r[2])
 	int w, h;
 
 	window_get_size(rw->win, &w, &h);
+	h-= RW_HEADERY;
 
 	ndc_r[0]=  ((float)(win_co[0]*2)/(w-1) - 1.0);
 	ndc_r[1]=  ((float)(win_co[1]*2)/(h-1) - 1.0);
@@ -223,10 +238,20 @@ static void renderwin_reset_view(RenderWin *rw)
 
 	/* now calculate a zoom for when image is larger than window */
 	window_get_size(rw->win, &w, &h);
-		/* at this point the r.rectx/y values are not correct yet */
+	h-= RW_HEADERY;
+	
+	/* at this point the r.rectx/y values are not correct yet */
 	rectx= (G.scene->r.size*G.scene->r.xsch)/100;
 	recty= (G.scene->r.size*G.scene->r.ysch)/100;
-
+	
+	/* crop option makes image smaller */
+	if ((G.scene->r.mode & R_BORDER) && (G.scene->r.mode & R_MOVIECROP)) { 
+		if(!(G.scene->r.scemode & R_OGL)) {
+			rectx*= (G.scene->r.border.xmax-G.scene->r.border.xmin);
+			recty*= (G.scene->r.border.ymax-G.scene->r.border.ymin);
+		}
+	}
+	
 	if(rectx>w || recty>h) {
 		if(rectx-w > recty-h) rw->zoom= ((float)w)/((float)rectx);
 		else rw->zoom= ((float)h)/((float)recty);
@@ -237,16 +262,54 @@ static void renderwin_reset_view(RenderWin *rw)
 	renderwin_queue_redraw(rw);
 }
 
+static void renderwin_draw_render_info(RenderWin *rw)
+{
+	/* render text is added to top */
+	if(RW_HEADERY) {
+		float colf[3];
+		rcti rect;
+		
+		window_get_size(rw->win, &rect.xmax, &rect.ymax);
+		rect.xmin= 0;
+		rect.ymin= rect.ymax-RW_HEADERY;
+		glEnable(GL_SCISSOR_TEST);
+		glaDefine2DArea(&rect);
+		
+		/* clear header rect */
+		BIF_SetTheme(NULL);	// sets view3d theme by default
+		BIF_GetThemeColor3fv(TH_HEADER, colf);
+		glClearColor(colf[0], colf[1], colf[2], 1.0); 
+		glClear(GL_COLOR_BUFFER_BIT);
+		
+		if(rw->render_text) {
+			BIF_ThemeColor(TH_TEXT);
+			glRasterPos2i(12, 5);
+			BMF_DrawString(G.fonts, rw->render_text);
+		}
+	}	
+	
+}
+
 static void renderwin_draw(RenderWin *rw, int just_clear)
 {
 	float disprect[2][2];
+	int set_back_mainwindow;
 	rcti rect;
-
+	
+	/* since renderwin uses callbacks (controlled by ghost) it can
+	   mess up active window output with redraw events after a render. 
+	   this is patchy, still WIP */
+	set_back_mainwindow = (winlay_get_active_window() != rw->win);
+	window_make_active(rw->win);
+	
 	rect.xmin= rect.ymin= 0;
 	window_get_size(rw->win, &rect.xmax, &rect.ymax);
+	rect.ymax-= RW_HEADERY;
+	
 	renderwin_get_disprect(rw, disprect);
 	
-	window_make_active(rw->win);
+	/* do this first, so window ends with correct scissor */
+	renderwin_draw_render_info(rw);
 	
 	glEnable(GL_SCISSOR_TEST);
 	glaDefine2DArea(&rect);
@@ -259,10 +322,25 @@ static void renderwin_draw(RenderWin *rw, int just_clear)
 		glRectfv(disprect[0], disprect[1]);
 	} else {
 		glPixelZoom(rw->zoom, rw->zoom);
-		glaDrawPixelsSafe(disprect[0][0], disprect[0][1], R.rectx, R.recty, R.rectot);
+		if(rw->flags & RW_FLAGS_ALPHA) {
+			char *rect= (char *)R.rectot;
+			
+			glColorMask(1, 0, 0, 0);
+			glaDrawPixelsSafe(disprect[0][0], disprect[0][1], R.rectx, R.recty, rect+3);
+			glColorMask(0, 1, 0, 0);
+			glaDrawPixelsSafe(disprect[0][0], disprect[0][1], R.rectx, R.recty, rect+2);
+			glColorMask(0, 0, 1, 0);
+			glaDrawPixelsSafe(disprect[0][0], disprect[0][1], R.rectx, R.recty, rect+1);
+			glColorMask(1, 1, 1, 1);
+			
+		}
+		else {
+			glaDrawPixelsSafe(disprect[0][0], disprect[0][1], R.rectx, R.recty, R.rectot);
+		}
 		glPixelZoom(1.0, 1.0);
 	}
 	
+	/* info text is overlayed on bottom */
 	if (rw->info_text) {
 		float w;
 		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
@@ -275,8 +353,10 @@ static void renderwin_draw(RenderWin *rw, int just_clear)
 		glRasterPos2i(10, 10);
 		BMF_DrawString(G.font, rw->info_text);
 	}
-
+	
 	window_swap_buffers(rw->win);
+	
+	if (set_back_mainwindow) mainwindow_make_active();	
 }
 
 /* ------ interactivity calls for RenderWin ------------- */
@@ -286,11 +366,20 @@ static void renderwin_mouse_moved(RenderWin *rw)
 	if (rw->flags & RW_FLAGS_PIXEL_EXAMINING) {
 		int imgco[2];
 		char buf[64];
+		int *pxlz;	// zbuffer is signed 
+		char *pxl;
 
 		if (R.rectot && renderwin_win_to_image_co(rw, rw->lmouse, imgco)) {
-			unsigned char *pxl= (char*) &R.rectot[R.rectx*imgco[1] + imgco[0]];
+			pxl= (char*) &R.rectot[R.rectx*imgco[1] + imgco[0]];
+			
+			if (R.rectz) {
+				pxlz= &R.rectz[R.rectx*imgco[1] + imgco[0]];			
+				sprintf(buf, "R: %d, G: %d, B: %d, A: %d, Z: %f", pxl[0], pxl[1], pxl[2], pxl[3], 0.5+0.5*( ((float)*pxlz)/(float)INT_MAX) );
+			}
+			else {
+				sprintf(buf, "R: %d, G: %d, B: %d, A: %d", pxl[0], pxl[1], pxl[2], pxl[3]);			
+			}
 
-			sprintf(buf, "R: %d, G: %d, B: %d, A: %d", pxl[0], pxl[1], pxl[2], pxl[3]);
 			renderwin_set_infotext(rw, buf);
 			renderwin_queue_redraw(rw);
 		} else {
@@ -314,6 +403,7 @@ static void renderwin_mouse_moved(RenderWin *rw)
 		int w, h;
 
 		window_get_size(rw->win, &w, &h);
+		h-= RW_HEADERY;
 		renderwin_win_to_ndc(rw, rw->lmouse, ndc);
 
 		rw->zoomofs[0]= -0.5*ndc[0]*(w-R.rectx*rw->zoom)/rw->zoom;
@@ -354,6 +444,13 @@ static void renderwin_handler(Window *win, void *user_data, short evt, short val
 {
 	RenderWin *rw= user_data;
 
+	// added this for safety, while render it's just creating bezerk results
+	if(R.flag & R_RENDERING) {
+		if(evt==ESCKEY && val) 
+			rw->flags|= RW_FLAGS_ESCAPE;
+		return;
+	}
+	
 	if (evt==RESHAPE) {
 		renderwin_reshape(rw);
 	} 
@@ -393,6 +490,10 @@ static void renderwin_handler(Window *win, void *user_data, short evt, short val
 				rw->active= 0;
 			}
 		} 
+		else if( evt==AKEY) {
+			rw->flags ^= RW_FLAGS_ALPHA;
+			renderwin_queue_redraw(render_win);
+		}
 		else if (evt==JKEY) {
 			if(R.flag==0) BIF_swap_render_rects();
 		} 
@@ -445,19 +546,34 @@ static void renderwin_handler(Window *win, void *user_data, short evt, short val
 	}
 }
 
+static char *renderwin_get_title(int doswap)
+{
+	static int swap= 0;
+	char *title="";
+	
+	swap+= doswap;
+	
+	if(swap & 1) {
+		if (G.scene->r.renderer==R_YAFRAY) title = "YafRay:Render (spare)";
+		else title = "Blender:Render (spare)";
+	}
+	else {
+		if (G.scene->r.renderer==R_YAFRAY) title = "YafRay:Render";
+		else title = "Blender:Render";
+	}
+
+	return title;
+}
 
 /* opens window and allocs struct */
 static void open_renderwin(int winpos[2], int winsize[2])
 {
+	extern void mywindow_build_and_set_renderwin( int orx, int ory, int sizex, int sizey); // mywindow.c
 	Window *win;
-	/* yafray: Window title change for yafray, totally unnecessary of course, but... */
-	char* title;
-	if (G.scene->r.renderer==R_YAFRAY)
-		title = "YafRay:Render";
-	else
-		title = "Blender:Render";
-
-	win= window_open(title, winpos[0], winpos[1], winsize[0], winsize[1], 0);
+	char *title;
+	
+	title= renderwin_get_title(0);	/* 0 = no swap */
+	win= window_open(title, winpos[0], winpos[1], winsize[0], winsize[1]+RW_HEADERY, 0);
 
 	render_win= renderwin_alloc(win);
 
@@ -467,7 +583,12 @@ static void open_renderwin(int winpos[2], int winsize[2])
 	winlay_process_events(0);
 	window_make_active(render_win->win);
 	winlay_process_events(0);
-
+	
+	/* mywindow has to know about it too */
+	mywindow_build_and_set_renderwin(winpos[0], winpos[1], winsize[0], winsize[1]+RW_HEADERY);
+	/* and we should be able to draw 3d in it */
+	init_gl_stuff();
+	
 	renderwin_draw(render_win, 1);
 	renderwin_draw(render_win, 1);
 }
@@ -488,6 +609,13 @@ void calc_renderwin_rectangle(int posmask, int renderpos_r[2], int rendersize_r[
 	
 	rendersize_r[0]= (G.scene->r.size*G.scene->r.xsch)/100;
 	rendersize_r[1]= (G.scene->r.size*G.scene->r.ysch)/100;
+	
+	/* crop option makes image smaller */
+	if ((G.scene->r.mode & R_BORDER) && (G.scene->r.mode & R_MOVIECROP)) { 
+		rendersize_r[0]*= (G.scene->r.border.xmax-G.scene->r.border.xmin);
+		rendersize_r[1]*= (G.scene->r.border.ymax-G.scene->r.border.ymin);
+	}
+	
 	if(G.scene->r.mode & R_PANORAMA) {
 		rendersize_r[0]*= G.scene->r.xparts;
 		rendersize_r[1]*= G.scene->r.yparts;
@@ -521,7 +649,7 @@ static void renderwin_init_display_cb(void)
 	if (G.afbreek == 0) {
 		int rendersize[2], renderpos[2];
 
-		calc_renderwin_rectangle(R.winpos, renderpos, rendersize);
+		calc_renderwin_rectangle(G.winpos, renderpos, rendersize);
 
 		if (!render_win) {
 			open_renderwin(renderpos, rendersize);
@@ -532,19 +660,35 @@ static void renderwin_init_display_cb(void)
 
 			window_get_position(render_win->win, &win_x, &win_y);
 			window_get_size(render_win->win, &win_w, &win_h);
+			win_h-= RW_HEADERY;
 
 				/* XXX, this is nasty and I guess bound to cause problems,
 				 * but to ensure the window is at the user specified position
 				 * and size we reopen the window all the time... we need
 				 * a ghost _set_position to fix this -zr
 				 */
-			BIF_close_render_display();
-			open_renderwin(renderpos, rendersize);
+				 
+				 /* XXX, well... it is nasty yes, and reopens windows each time on
+				    subsequent renders. Better rule is to make it reopen only only
+				    size change, and use the preferred position only on open_renderwin
+					cases (ton)
+				 */
+			if(rendersize[0]!= win_w || rendersize[1]!= win_h) {
+				BIF_close_render_display();
+				open_renderwin(renderpos, rendersize);
+			}
+			else {
+				window_raise(render_win->win);
+				window_make_active(render_win->win);
+				mywinset(2);	// to assign scissor/viewport again in mywindow.c. is hackish yes
+			}
 
 			renderwin_reset_view(render_win);
 			render_win->flags&= ~RW_FLAGS_ESCAPE;
 			render_win->active= 1;
 		}
+		/* make sure we are in normal draw again */
+		render_win->flags &= ~RW_FLAGS_ALPHA;
 	}
 }
 
@@ -561,7 +705,9 @@ static void renderwin_clear_display_cb(short ignore)
 * ... better is to make this an optimization of a more clear
 * implementation. the bug shows up when you do something like
 * open the window, then draw part of the progress, then get
-* a redraw event. whatever can go wrong will.
+* a redraw event. whatever can go wrong will. -zr
+*
+* Note: blocked queue handling while rendering to prevent that (ton)
 */
 
 /* in render window; display a couple of scanlines of rendered image (see callback below) */
@@ -572,9 +718,12 @@ static void renderwin_progress(RenderWin *rw, int start_y, int nlines, int rect_
 
 	win_rct.xmin= win_rct.ymin= 0;
 	window_get_size(rw->win, &win_rct.xmax, &win_rct.ymax);
+	win_rct.ymax-= RW_HEADERY;
+	
 	renderwin_get_disprect(rw, disprect);
-
-	window_make_active(rw->win);
+	
+	/* for efficiency & speed; not drawing in Blender UI while rendering */
+	//window_make_active(rw->win);
 
 	glEnable(GL_SCISSOR_TEST);
 	glaDefine2DArea(&win_rct);
@@ -637,7 +786,24 @@ static void renderview_progress_display_cb(int y1, int y2, int w, int h, unsigne
 		rcti win_rct, vb;
 
 		calc_viewborder(v3d, &vb);
-
+		
+		/* if border render  */
+		if(G.scene->r.mode & R_BORDER) { 
+			
+			/* but, if image is full (at end of border render, without crop) we don't */
+			if(R.rectx != (G.scene->r.size*G.scene->r.xsch)/100 ||
+			   R.recty != (G.scene->r.size*G.scene->r.ysch)/100 ) {
+			
+				facx= (float) (vb.xmax-vb.xmin);
+				facy= (float) (vb.ymax-vb.ymin);
+				
+				vb.xmax= vb.xmin + facx*G.scene->r.border.xmax;
+				vb.ymax= vb.ymin + facy*G.scene->r.border.ymax;
+				vb.xmin+= facx*G.scene->r.border.xmin;
+				vb.ymin+= facy*G.scene->r.border.ymin;
+			}
+		}
+			
 		facx= (float) (vb.xmax-vb.xmin)/R.rectx;
 		facy= (float) (vb.ymax-vb.ymin)/R.recty;
 
@@ -662,28 +828,87 @@ static void renderview_progress_display_cb(int y1, int y2, int w, int h, unsigne
 	}
 }
 
+/* in 3d view; display stats of rendered image */
+static void renderview_draw_render_info(char *str)
+{
+	if (render_view3d) {
+		View3D *v3d= render_view3d;
+		rcti vb, win_rct;
+		
+		calc_viewborder(v3d, &vb);
+		
+		bwin_get_rect(v3d->area->win, &win_rct);
+		glaDefine2DArea(&win_rct);
+		
+		glDrawBuffer(GL_FRONT);
+		
+		/* clear header rect */
+		BIF_ThemeColor(TH_HEADER);
+		glRecti(vb.xmin, vb.ymax, vb.xmax, vb.ymax+RW_HEADERY);
+		
+		if(str) {
+			BIF_ThemeColor(TH_TEXT);
+			glRasterPos2i(vb.xmin+12, vb.ymax+5);
+			BMF_DrawString(G.fonts, str);
+		}
+		
+		glFlush();
+		glDrawBuffer(GL_BACK);
+
+		v3d->area->win_swap= WIN_FRONT_OK;
+		
+	}
+}
+
+
 /* -------------- callbacks for render loop: interactivity ----------------------- */
 
 
-/* callback for print info in top header in interface */
+/* callback for print info in top header of renderwin */
+/* time is only not zero on last call, we then don't update the other stats */ 
 static void printrenderinfo_cb(double time, int sample)
 {
 	extern int mem_in_use;
-	float megs_used_memory= mem_in_use/(1024.0*1024.0);
-	char str[300], tstr[32], *spos= str;
+	static int totvert=0, totvlak=0, tothalo=0, totlamp=0;
+	static float megs_used_memory=0.0;
+	char str[300], *spos= str;
 		
-	timestr(time, tstr);
-	spos+= sprintf(spos, "RENDER  Fra:%d  Ve:%d Fa:%d La:%d", (G.scene->r.cfra), R.totvert, R.totvlak, R.totlamp);
-	spos+= sprintf(spos, "Mem:%.2fM Time:%s ", megs_used_memory, tstr);
-
-	if (R.r.mode & R_FIELDS) {
-		spos+= sprintf(spos, "Field %c ", (R.flag&R_SEC_FIELD)?'B':'A');
-	}
-	if (sample!=-1) {
-		spos+= sprintf(spos, "Sample: %d    ", sample);
+	if(time==0.0) {
+		megs_used_memory= mem_in_use/(1024.0*1024.0);
+		totvert= R.totvert;
+		totvlak= R.totvlak;
+		totlamp= R.totlamp;
+		tothalo= R.tothalo;
 	}
 	
-	screen_draw_info_text(G.curscreen, str);
+	if(tothalo)
+		spos+= sprintf(spos, "Fra:%d  Ve:%d Fa:%d Ha:%d La:%d Mem:%.2fM", (G.scene->r.cfra), totvert, totvlak, tothalo, totlamp, megs_used_memory);
+	else 
+		spos+= sprintf(spos, "Fra:%d  Ve:%d Fa:%d La:%d Mem:%.2fM", (G.scene->r.cfra), totvert, totvlak, totlamp, megs_used_memory);
+
+	if(time==0.0) {
+		if (R.r.mode & R_FIELDS) {
+			spos+= sprintf(spos, "Field %c ", (R.flag&R_SEC_FIELD)?'B':'A');
+		}
+		if (sample!=-1) {
+			spos+= sprintf(spos, "Sample: %d    ", sample);
+		}
+	}
+	else {
+		extern char info_time_str[32];	// header_info.c
+		timestr(time, info_time_str);
+		spos+= sprintf(spos, " Time:%s ", info_time_str);
+	}
+	
+	if(render_win) {
+		if(render_win->render_text) MEM_freeN(render_win->render_text);
+		render_win->render_text= BLI_strdup(str);
+		glDrawBuffer(GL_FRONT);
+		renderwin_draw_render_info(render_win);
+		glFlush();
+		glDrawBuffer(GL_BACK);
+	}
+	else renderview_draw_render_info(str);
 }
 
 /* -------------- callback system to allow ESC from rendering ----------------------- */
@@ -797,8 +1022,9 @@ static void do_render(View3D *ogl_render_view3d, int anim, int force_dispwin)
 	
 	/* we set this flag to prevent renderwindow queue to execute another render */
 	R.flag= R_RENDERING;
+	G.afbreek= 0;
 
-	if (R.displaymode == R_DISPLAYWIN || force_dispwin) {
+	if (G.displaymode == R_DISPLAYWIN || force_dispwin) {
 		RE_set_initrenderdisplay_callback(NULL);
 		RE_set_clearrenderdisplay_callback(renderwin_clear_display_cb);
 		RE_set_renderdisplay_callback(renderwin_progress_display_cb);
@@ -822,11 +1048,10 @@ static void do_render(View3D *ogl_render_view3d, int anim, int force_dispwin)
 		window_set_cursor(render_win->win, CURSOR_WAIT);
 		// when opening new window... not cross platform identical behaviour, so
 		// for now call it each time
-		if(ogl_render_view3d) init_gl_stuff();
+		// if(ogl_render_view3d) init_gl_stuff();
 	}
 	waitcursor(1);
 
-	G.afbreek= 0;
 	if(G.obedit && !(G.scene->r.scemode & R_OGL)) {
 		exit_editmode(0);	/* 0 = no free data */
 	}
@@ -839,20 +1064,28 @@ static void do_render(View3D *ogl_render_view3d, int anim, int force_dispwin)
 	}
 
 	if(anim) update_for_newframe_muted();  // only when anim, causes redraw event which frustrates dispview
-	R.flag= 0;
 	
 	if (render_win) window_set_cursor(render_win->win, CURSOR_STD);
-	waitcursor(0);
 
 	free_filesel_spec(G.scene->r.pic);
 
 	G.afbreek= 0;
 	end_test_break_callback();
-
+	
 	/* in dispiew it will destroy the image otherwise
 	   window_make_active() raises window at osx and sends redraws */
-	if(R.displaymode==R_DISPLAYWIN) mainwindow_make_active();
-
+	if(G.displaymode==R_DISPLAYWIN) {
+		mainwindow_make_active();
+	
+		/* after an envmap creation...  */
+		if(R.flag & R_REDRAW_PRV) {
+			BIF_all_preview_changed();
+		}
+		allqueue(REDRAWBUTSSCENE, 0);	// visualize fbuf for example
+	}
+	
+	R.flag= 0;
+	waitcursor(0);	// waitcursor checks rendering R.flag...
 }
 
 /* finds area with a 'dispview' set */
@@ -897,10 +1130,45 @@ static void scalefastrect(unsigned int *recto, unsigned int *rectn, int oldx, in
 
 /* -------------- API: externally called --------------- */
 
+/* not used anywhere ??? */
+#if 0
+void BIF_renderwin_make_active(void)
+{
+	if(render_win) {
+		window_make_active(render_win->win);
+		mywinset(2);
+	}
+}
+#endif
+
 /* set up display, render an image or scene */
 void BIF_do_render(int anim)
 {
-	do_render(NULL, anim, 0);
+	int slink_flag = 0;
+
+	if (G.f & G_DOSCRIPTLINKS) {
+		BPY_do_all_scripts(SCRIPT_RENDER);
+		if (!anim) { /* avoid FRAMECHANGED slink in render callback */
+			G.f &= ~G_DOSCRIPTLINKS;
+			slink_flag = 1;
+		}
+	}
+
+	/* if start render in 3d win, use layer from window (e.g also local view) */
+	if(curarea && curarea->spacetype==SPACE_VIEW3D) {
+		int lay= G.scene->lay;
+		if(G.vd->lay & 0xFF000000)	// localview
+			G.scene->lay |= G.vd->lay;
+		else G.scene->lay= G.vd->lay;
+		
+		do_render(NULL, anim, 0);
+		
+		G.scene->lay= lay;
+	}
+	else do_render(NULL, anim, 0);
+
+	if (slink_flag) G.f |= G_DOSCRIPTLINKS;
+	if (G.f & G_DOSCRIPTLINKS) BPY_do_all_scripts(SCRIPT_POSTRENDER);
 }
 
 /* set up display, render the current area view in an image */
@@ -911,6 +1179,22 @@ void BIF_do_ogl_render(View3D *ogl_render_view3d, int anim)
 	G.scene->r.scemode &= ~R_OGL;
 }
 
+void BIF_redraw_render_rect(void)
+{
+	
+	/* redraw */
+	if (G.displaymode == R_DISPLAYWIN) {
+		// don't open render_win if rendering has been
+	    // canceled or the render_win has been actively closed
+		if (render_win) {
+			renderwin_queue_redraw(render_win);
+		}
+	} else {
+		renderview_init_display_cb();
+		renderview_progress_display_cb(0, R.recty-1, R.rectx, R.recty, R.rectot);
+	}
+}	
+
 void BIF_swap_render_rects(void)
 {
 	unsigned int *temp;
@@ -934,18 +1218,20 @@ void BIF_swap_render_rects(void)
 	temp= R.rectot;
 	R.rectot= R.rectspare;
 	R.rectspare= temp;
+	
+	if (G.displaymode == R_DISPLAYWIN) {
+		if (render_win) {
+			char *tmp= render_win->render_text_spare;
+			render_win->render_text_spare= render_win->render_text;
+			render_win->render_text= tmp;
+			
+			window_set_title(render_win->win, renderwin_get_title(1));
+			
+		}
+	}
 
 	/* redraw */
-	if (R.displaymode == R_DISPLAYWIN) {
-		// don't open render_win if rendering has been
-	    // canceled or the render_win has been actively closed
-		if (render_win) {
-			renderwin_queue_redraw(render_win);
-		}
-	} else {
-		renderview_init_display_cb();
-		renderview_progress_display_cb(0, R.recty-1, R.rectx, R.recty, R.rectot);
-	}
+	BIF_redraw_render_rect();
 }				
 
 /* called from usiblender.c too, to free and close renderwin */
@@ -954,6 +1240,8 @@ void BIF_close_render_display(void)
 	if (render_win) {
 
 		if (render_win->info_text) MEM_freeN(render_win->info_text);
+		if (render_win->render_text) MEM_freeN(render_win->render_text);
+		if (render_win->render_text_spare) MEM_freeN(render_win->render_text_spare);
 
 		window_destroy(render_win->win); /* ghost close window */
 		MEM_freeN(render_win);
@@ -969,7 +1257,7 @@ void BIF_toggle_render_display(void)
 	ScrArea *sa= find_dispimage_v3d();
 	
 	if(R.rectot==NULL);		// do nothing
-	else if (render_win && R.displaymode==R_DISPLAYWIN) {
+	else if (render_win && G.displaymode==R_DISPLAYWIN) {
 		if(render_win->active) {
 			mainwindow_raise();
 			mainwindow_make_active();
@@ -981,13 +1269,13 @@ void BIF_toggle_render_display(void)
 			render_win->active= 1;
 		}
 	} 
-	else if (sa && R.displaymode==R_DISPLAYVIEW) {
+	else if (sa && G.displaymode==R_DISPLAYVIEW) {
 		View3D *vd= sa->spacedata.first;
 		vd->flag &= ~V3D_DISPIMAGE;
 		scrarea_queue_winredraw(sa);
 	} 
 	else {
-		if (R.displaymode == R_DISPLAYWIN) {
+		if (G.displaymode == R_DISPLAYWIN) {
 			renderwin_init_display_cb();
 		} else {
 			if (render_win) {
