@@ -1,5 +1,5 @@
 /**
- * $Id: resources.c,v 1.32 2004/10/06 18:55:00 ton Exp $
+ * $Id: resources.c,v 1.43 2005/05/13 16:11:28 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -38,10 +38,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include "MEM_guardedalloc.h"
 
 #include "DNA_listBase.h"
@@ -52,6 +48,8 @@
 #include "IMB_imbuf.h"
 #include "IMB_imbuf_types.h"
 
+#include "BKE_utildefines.h"
+
 #include "BIF_gl.h"
 #include "BIF_resources.h"
 
@@ -96,6 +94,7 @@ static Icon *icon_from_data(unsigned char *rect, GLuint texid, int xofs, int yof
 	return icon;
 }
 
+#if 0
 static float icon_x=0.0, icon_y=0.0;
 void BIF_icon_pos(float xs, float ys)
 {
@@ -140,6 +139,7 @@ static void icon_draw_tex(Icon *icon)
 	glEnd();
 	glDisable(GL_TEXTURE_2D);
 }
+#endif
 
 
 static void icon_draw(Icon *icon)
@@ -217,7 +217,7 @@ static void def_icon(ImBuf *bbuf, GLuint texid, BIFIconID icon, int xidx, int yi
 	
 	if (iconidx>=0 && iconidx<BIFNICONIDS) {
 		int rowstride= bbuf->x*4;
-		unsigned char *start= ((char*) bbuf->rect) + (yidx*21 + 3 + offsy)*rowstride + (xidx*20 + 3 + offsx)*4;
+		unsigned char *start= ((unsigned char*) bbuf->rect) + (yidx*21 + 3 + offsy)*rowstride + (xidx*20 + 3 + offsx)*4;
 
 		common_icons_arr[iconidx]= 
 			icon_from_data(start, texid, (xidx*20 + 3 + offsx), (yidx*21 + 3 + offsy), w, h, rowstride);
@@ -227,7 +227,42 @@ static void def_icon(ImBuf *bbuf, GLuint texid, BIFIconID icon, int xidx, int yi
 	}
 }
 
-/***/
+/* this only works for the hardcoded buttons image, turning the grey AA pixels to alpha, and slight off-grey to half alpha */
+
+static void clear_transp_rect_soft(unsigned char *transp, unsigned char *rect, int w, int h, int rowstride)
+{
+	int x, y, val;
+	
+	for (y=1; y<h-1; y++) {
+		unsigned char *row0= &rect[(y-1)*rowstride];
+		unsigned char *row= &rect[y*rowstride];
+		unsigned char *row1= &rect[(y+1)*rowstride];
+		for (x=1; x<w-1; x++) {
+			unsigned char *pxl0= &row0[x*4];
+			unsigned char *pxl= &row[x*4];
+			unsigned char *pxl1= &row1[x*4];
+			
+			if(pxl[3]!=0) {
+				val= (abs(pxl[0]-transp[0]) + abs(pxl[1]-transp[1]) + abs(pxl[2]-transp[2]))/3;
+				if(val<20) {
+					pxl[3]= 128;
+				}
+				else if(val<50) {
+					// one of pixels surrounding has alpha null?
+					if(pxl[3-4]==0 || pxl[3+4]==0 || pxl0[3]==0 || pxl1[3]==0) {
+				
+						if(pxl[0]>val) pxl[0]-= val; else pxl[0]= 0;
+						if(pxl[1]>val) pxl[1]-= val; else pxl[1]= 0;
+						if(pxl[2]>val) pxl[2]-= val; else pxl[2]= 0;
+						
+						pxl[3]= 128;
+					}
+				}
+			}
+		}
+	}
+}
+
 
 static void clear_transp_rect(unsigned char *transp, unsigned char *rect, int w, int h, int rowstride)
 {
@@ -251,10 +286,11 @@ void BIF_resources_init(void)
 
 	common_icons_arr= MEM_mallocN(sizeof(*common_icons_arr)*BIFNICONIDS, "common_icons");
 
-	for (y=0; y<10; y++) {
+	/* hack! */
+	for (y=0; y<12; y++) {
 		for (x=0; x<21; x++) {
 			int rowstride= bbuf->x*4;
-			unsigned char *start= ((char*) bbuf->rect) + (y*21 + 3)*rowstride + (x*20 + 3)*4;
+			unsigned char *start= ((unsigned char*) bbuf->rect) + (y*21 + 3)*rowstride + (x*20 + 3)*4;
 			unsigned char transp[4];
 			/* this sets backdrop of icon to zero alpha */
 			transp[0]= start[0];
@@ -262,11 +298,14 @@ void BIF_resources_init(void)
 			transp[2]= start[2];
 			transp[3]= start[3];
 			clear_transp_rect(transp, start, 20, 21, rowstride);
-			
+			clear_transp_rect_soft(transp, start, 20, 21, rowstride);
+				
 			/* this sets outside of icon to zero alpha */
-			start= ((char*) bbuf->rect) + (y*21)*rowstride + (x*20)*4;
+			start= ((unsigned char*) bbuf->rect) + (y*21)*rowstride + (x*20)*4;
 			QUATCOPY(transp, start);
 			clear_transp_rect(transp, start, 20, 21, rowstride);
+			
+			
 		}
 	} 
 
@@ -274,7 +313,7 @@ void BIF_resources_init(void)
 	// texid= init_icon_texture(bbuf);
 
 		/* hack! */
-	for (y=0; y<10; y++) {
+	for (y=0; y<12; y++) {
 		for (x=0; x<21; x++) {
 			if (x==11 && y==6) {
 				def_icon(bbuf, texid, ICON_BEVELBUT_HLT,			x, y, 7, 13, 4, 2);
@@ -402,6 +441,9 @@ char *BIF_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colorid)
 			case SPACE_INFO:
 				ts= &btheme->tinfo;
 				break;
+			case SPACE_TIME:
+				ts= &btheme->ttime;
+				break;
 			default:
 				ts= &btheme->tv3d;
 				break;
@@ -468,6 +510,17 @@ char *BIF_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colorid)
 			case TH_NORMAL:
 				cp= ts->normal; break;
 
+			case TH_SYNTAX_B:
+				cp= ts->syntaxb; break;
+			case TH_SYNTAX_V:
+				cp= ts->syntaxv; break;
+			case TH_SYNTAX_C:
+				cp= ts->syntaxc; break;
+			case TH_SYNTAX_L:
+				cp= ts->syntaxl; break;
+			case TH_SYNTAX_N:
+				cp= ts->syntaxn; break;
+
 			}
 
 		}
@@ -627,7 +680,14 @@ void BIF_InitTheme(void)
 	SETCOL(btheme->text.shade1, 	143, 143, 143, 255);
 	SETCOL(btheme->text.shade2, 	0xc6, 0x77, 0x77, 255);
 	SETCOL(btheme->text.hilite, 	255, 0, 0, 255);
-
+	
+	/* syntax highlighting */
+	SETCOL(btheme->text.syntaxn,	0, 0, 200, 255);	/* Numbers  Blue*/
+	SETCOL(btheme->text.syntaxl,	100, 0, 0, 255);	/* Strings  red */
+	SETCOL(btheme->text.syntaxc,	0, 100, 50, 255);	/* Comments greenish */
+	SETCOL(btheme->text.syntaxv,	95, 95, 0, 255);	/* Special */
+	SETCOL(btheme->text.syntaxb,	128, 0, 80, 255);	/* Builtin, red-purple */
+	
 	/* space oops */
 	btheme->toops= btheme->tv3d;
 	SETCOL(btheme->toops.back, 	153, 153, 153, 255);
@@ -642,7 +702,8 @@ void BIF_InitTheme(void)
 	SETCOL(btheme->tsnd.shade1,  173, 173, 173, 255);		// sliders
 	SETCOL(btheme->tsnd.grid, 140, 140, 140, 255);
 	
-
+	/* space time */
+	btheme->ttime= btheme->tsnd;	// same as sound space
 }
 
 char *BIF_ThemeColorsPup(int spacetype)
@@ -702,6 +763,7 @@ char *BIF_ThemeColorsPup(int spacetype)
 		else if(spacetype==SPACE_IPO) {
 			sprintf(str, "Panel %%x%d|", TH_PANEL); strcat(cp, str);
 			strcat(cp,"%l|");
+			sprintf(str, "Grid %%x%d|", TH_GRID); strcat(cp, str);
 			sprintf(str, "Window Sliders %%x%d|", TH_SHADE1); strcat(cp, str);
 			sprintf(str, "Ipo Channels %%x%d|", TH_SHADE2); strcat(cp, str);
 			sprintf(str, "Vertex %%x%d|", TH_VERTEX); strcat(cp, str);
@@ -713,6 +775,7 @@ char *BIF_ThemeColorsPup(int spacetype)
 		else if(spacetype==SPACE_NLA) {
 			//sprintf(str, "Panel %%x%d|", TH_PANEL); strcat(cp, str);
 			strcat(cp,"%l|");
+			sprintf(str, "Grid %%x%d|", TH_GRID); strcat(cp, str);
 			sprintf(str, "View Sliders %%x%d|", TH_SHADE1); strcat(cp, str);
 			sprintf(str, "Bars %%x%d|", TH_SHADE2); strcat(cp, str);
 			sprintf(str, "Bars selected %%x%d|", TH_HILITE); strcat(cp, str);
@@ -720,6 +783,7 @@ char *BIF_ThemeColorsPup(int spacetype)
 		else if(spacetype==SPACE_ACTION) {
 			//sprintf(str, "Panel %%x%d|", TH_PANEL); strcat(cp, str);
 			strcat(cp,"%l|");
+			sprintf(str, "Grid %%x%d|", TH_GRID); strcat(cp, str);
 			sprintf(str, "RVK Sliders %%x%d|", TH_FACE); strcat(cp, str);
 			sprintf(str, "View Sliders %%x%d|", TH_SHADE1); strcat(cp, str);
 			sprintf(str, "Channels %%x%d|", TH_SHADE2); strcat(cp, str);
@@ -734,9 +798,11 @@ char *BIF_ThemeColorsPup(int spacetype)
 			sprintf(str, "Face Selected %%x%d", TH_FACE_SELECT); strcat(cp, str);
 		}
 		else if(spacetype==SPACE_SEQ) {
+			sprintf(str, "Grid %%x%d|", TH_GRID); strcat(cp, str);
 			sprintf(str, "Window Sliders %%x%d|", TH_SHADE1); strcat(cp, str);
 		}
 		else if(spacetype==SPACE_SOUND) {
+			sprintf(str, "Grid %%x%d|", TH_GRID); strcat(cp, str);
 			sprintf(str, "Window Slider %%x%d|", TH_SHADE1); strcat(cp, str);
 		}
 		else if(spacetype==SPACE_BUTS) {
@@ -749,6 +815,15 @@ char *BIF_ThemeColorsPup(int spacetype)
 			sprintf(str, "Scroll Bar %%x%d|", TH_SHADE1); strcat(cp, str);
 			sprintf(str, "Selected Text %%x%d|", TH_SHADE2); strcat(cp, str);
 			sprintf(str, "Cursor %%x%d|", TH_HILITE); strcat(cp, str);
+			strcat(cp,"%l|");
+			sprintf(str, "Syntax Builtin %%x%d|", TH_SYNTAX_B); strcat(cp, str);
+			sprintf(str, "Syntax Special %%x%d|", TH_SYNTAX_V); strcat(cp, str);
+			sprintf(str, "Syntax Comment %%x%d|", TH_SYNTAX_C); strcat(cp, str);
+			sprintf(str, "Syntax Strings %%x%d|", TH_SYNTAX_L); strcat(cp, str);
+			sprintf(str, "Syntax Numbers %%x%d|", TH_SYNTAX_N); strcat(cp, str);
+		}
+		else if(spacetype==SPACE_TIME) {
+			sprintf(str, "Grid %%x%d|", TH_GRID); strcat(cp, str);
 		}
 	}
 	return cp;
