@@ -1,5 +1,5 @@
 /**
- * $Id: toolbox.c,v 1.85 2004/11/26 20:20:42 ton Exp $
+ * $Id: toolbox.c,v 1.97 2005/06/02 10:12:13 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -46,7 +46,6 @@
 #include <unistd.h>
 #else
 #include <io.h>
-#include "BLI_winstuff.h"
 #endif   
 
 #include <fcntl.h>
@@ -75,12 +74,7 @@
 #include "BKE_global.h"
 #include "BKE_main.h"
 
-#include "BIF_gl.h"
-#include "BIF_graphics.h"
-#include "BIF_mainqueue.h"
-#include "BIF_interface.h"
-#include "BIF_toolbox.h"
-#include "BIF_mywindow.h"
+#include "BIF_editnla.h"
 #include "BIF_editarmature.h"
 #include "BIF_editdeform.h"
 #include "BIF_editfont.h"
@@ -89,11 +83,17 @@
 #include "BIF_editlattice.h"
 #include "BIF_editsima.h"
 #include "BIF_editoops.h"
+#include "BIF_gl.h"
+#include "BIF_graphics.h"
 #include "BIF_imasel.h"
+#include "BIF_mainqueue.h"
+#include "BIF_interface.h"
+#include "BIF_toolbox.h"
+#include "BIF_mywindow.h"
 #include "BIF_screen.h"
 #include "BIF_space.h"
 #include "BIF_tbcallback.h"
-#include "BIF_editnla.h"
+#include "BIF_transform.h"
 
 #include "BDR_editobject.h"
 #include "BDR_editcurve.h"
@@ -109,7 +109,7 @@
 
 #include "mydevice.h"
 #include "blendef.h"
-#include "render.h"
+#include "render.h"		// R.flag
 
 static int tbx1, tbx2, tby1, tby2, tbfontyofs, tbmain=0;
 static int tbmemx=TBOXX/2, tbmemy=(TBOXEL-0.5)*TBOXH, tboldwin, addmode= 0;
@@ -120,6 +120,10 @@ static char *tbstr, *tbstr1, *keystr;
 static void (*tbfunc)(int);
 static int tbval;
 
+/* prototypes ----------------------- */
+TBcallback *callback_dummy(int level, int entry);
+
+
 /* *********** PC PATCH ************* */
 
 void ColorFunc(int i)
@@ -797,7 +801,7 @@ void tbox_embossbox(short x1, short y1, short x2, short y2, short type)
 }
 
 
-void tbox_drawelem_body(x, y, type)
+void tbox_drawelem_body( int x, int y, int type)
 {
 	int x1 = 0, y1, x2 = 0, y2;
 	
@@ -816,7 +820,7 @@ void tbox_drawelem_body(x, y, type)
 	
 }
 
-void tbox_drawelem_text(x, y, type)
+void tbox_drawelem_text( int x, int y, int type)
 {
 	int x1 = 0, y1, x2 = 0, y2, len1, len2;
 	
@@ -1154,7 +1158,7 @@ short sbutton(char *var, float min, float max, char *str)
 	if(mval[1]>G.curscreen->sizey) mval[1]= G.curscreen->sizey-10;
 
 	block= uiNewBlock(&listb, "button", UI_EMBOSS, UI_HELV, G.curscreen->mainwin);
-	uiBlockSetFlag(block, UI_BLOCK_LOOP|UI_BLOCK_REDRAW|UI_BLOCK_RET_1);
+	uiBlockSetFlag(block, UI_BLOCK_LOOP|UI_BLOCK_REDRAW|UI_BLOCK_RET_1|UI_BLOCK_ENTER_OK);
 
 	x1=mval[0]-150; 
 	y1=mval[1]-20; 
@@ -1678,10 +1682,11 @@ static TBitem tb_curve_edit_seg[]= {
 static TBitem tb_curve_edit_cv[]= {
 {	0, "Tilt|T", 	't', NULL},
 {	0, "Clear Tilt|Alt T", 			TB_ALT|'t', NULL},
+{	0, "Separate|P", 	'p', NULL},
 {	0, "SEPR", 								0, NULL},
 {	0, "Automatic|Shift H", 		'H', NULL},
 {	0, "Toggle Free/Aligned|H", 	'h', NULL},
-{	0, "Vector|V", 					TB_ALT|'t', NULL},
+{	0, "Vector|V", 					'v', NULL},
 {	0, "SEPR", 								0, NULL},
 {	0, "Make Vertex Parent|Ctrl P", TB_CTRL|'p', NULL},
 {	0, "Add Hook|Ctrl H",			TB_CTRL|'h', NULL},
@@ -1828,7 +1833,7 @@ static TBitem tb_curve_hide[]= {
 static TBitem tb_curve[]= {
 {	0, "Insert Keyframe|I", 		'i', 		NULL},
 {	0, "SEPR", 						0, NULL},
-{	0, "Show/Hide Control Points", 	0, 		tb_curve_hide},
+{	0, "Show/Hide Points", 	0, 		tb_curve_hide},
 {  -1, "", 			0, tb_do_hotkey}};
 
 static TBitem tb_obdata[]= {
@@ -1986,13 +1991,16 @@ static void tb_do_transform(void *arg, int event)
 	switch(event)
 	{
 		case 0: /* Grab/move */
-			transform('g');
+			initTransform(TFM_TRANSLATION, CTX_NONE);
+			Transform();
 			break;
 		case 1: /* Rotate */
-			transform('r');
+			initTransform(TFM_ROTATION, CTX_NONE);
+			Transform();
 			break;
 		case 2: /* Scale */
-			transform('s');
+			initTransform(TFM_RESIZE,CTX_NONE);
+			Transform();
 			break;
 		case 3: /* transform properties */
 			add_blockhandler(curarea, VIEW3D_HANDLER_OBJECT, UI_PNL_UNSTOW);
@@ -2001,17 +2009,20 @@ static void tb_do_transform(void *arg, int event)
 			snapmenu();
 			break;
 		case 5: /* Shrink/Fatten Along Normals */
-			transform('N');
+			initTransform(TFM_SHRINKFATTEN, CTX_NONE);
+			Transform();
 			break;
 		case 6: /* Shear */
-			transform('S');
+			initTransform(TFM_SHEAR, CTX_NONE);
+			Transform();
 			break;
 		case 7: /* Warp */
-			transform('w');
+			initTransform(TFM_WARP, CTX_NONE);
+			Transform();
 			break;
 		case 8: /* proportional edit (toggle) */
-			if(G.f & G_PROPORTIONAL) G.f &= ~G_PROPORTIONAL;
-			else G.f |= G_PROPORTIONAL;
+			if(G.scene->proportional) G.scene->proportional= 0;
+			else G.scene->proportional= 1;
 			break;
 	}
 	allqueue(REDRAWVIEW3D, 0);
@@ -2368,4 +2379,4 @@ void toolbox_n_add(void)
 	tb_mainx= 0;
 	tb_mainy= -5;
 	toolbox_n();
-}
\ No newline at end of file
+}
