@@ -1,5 +1,5 @@
 /**
- * $Id: view.c,v 1.25 2004/04/07 16:09:22 ton Exp $
+ * $Id: view.c,v 1.46 2005/06/05 13:50:21 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -41,7 +41,6 @@
 
 #ifdef WIN32
 #include <io.h>
-#include "BLI_winstuff.h"
 #else
 #include <unistd.h>
 #endif   
@@ -76,16 +75,21 @@
 #include "BSE_drawview.h"	/* For inner_play_anim_loop */
 
 #include "BDR_drawobject.h"	/* For draw_object */
+#include "BDR_editface.h"	/* For minmax_tface */
 
 #include "mydevice.h"
 #include "blendef.h"
 
 /* Modules used */
-#include "render.h"
+#include "render.h"		/* R. stuff for ogl view render */
 
 #define TRACKBALLSIZE  (1.1)
 #define BL_NEAR_CLIP 0.001
 
+
+/* local prototypes ----------*/
+void setcameratoview3d(void); /* windows.c & toets.c */
+
 void persp_general(int a)
 {
 	/* for all window types, not 3D */
@@ -174,15 +178,39 @@ void project_short(float *vec, short *adr)	/* clips */
 	Mat4MulVec4fl(G.vd->persmat, vec4);
 
 	if( vec4[3]>BL_NEAR_CLIP ) {	/* 0.001 is the NEAR clipping cutoff for picking */
-		fx= (curarea->winx/2)+(curarea->winx/2)*vec4[0]/vec4[3];
+		fx= (curarea->winx/2)*(1 + vec4[0]/vec4[3]);
 		
 		if( fx>0 && fx<curarea->winx) {
 		
-			fy= (curarea->winy/2)+(curarea->winy/2)*vec4[1]/vec4[3];
+			fy= (curarea->winy/2)*(1 + vec4[1]/vec4[3]);
 			
 			if(fy>0.0 && fy< (float)curarea->winy) {
-				adr[0]= floor(fx+0.5); 
-				adr[1]= floor(fy+0.5);
+				adr[0]= floor(fx); 
+				adr[1]= floor(fy);
+			}
+		}
+	}
+}
+
+void project_int(float *vec, int *adr)
+{
+	float fx, fy, vec4[4];
+
+	adr[0]= 2140000000.0f;
+	VECCOPY(vec4, vec);
+	vec4[3]= 1.0;
+	
+	Mat4MulVec4fl(G.vd->persmat, vec4);
+
+	if( vec4[3]>BL_NEAR_CLIP ) {	/* 0.001 is the NEAR clipping cutoff for picking */
+		fx= (curarea->winx/2)*(1 + vec4[0]/vec4[3]);
+		
+		if( fx>-2140000000.0f && fx<2140000000.0f) {
+			fy= (curarea->winy/2)*(1 + vec4[1]/vec4[3]);
+			
+			if(fy>-2140000000.0f && fy<2140000000.0f) {
+				adr[0]= floor(fx); 
+				adr[1]= floor(fy);
 			}
 		}
 	}
@@ -199,15 +227,15 @@ void project_short_noclip(float *vec, short *adr)
 	Mat4MulVec4fl(G.vd->persmat, vec4);
 
 	if( vec4[3]>BL_NEAR_CLIP ) {	/* 0.001 is the NEAR clipping cutoff for picking */
-		fx= (curarea->winx/2)+(curarea->winx/2)*vec4[0]/vec4[3];
+		fx= (curarea->winx/2)*(1 + vec4[0]/vec4[3]);
 		
 		if( fx>-32700 && fx<32700) {
 		
-			fy= (curarea->winy/2)+(curarea->winy/2)*vec4[1]/vec4[3];
+			fy= (curarea->winy/2)*(1 + vec4[1]/vec4[3]);
 			
 			if(fy>-32700.0 && fy<32700.0) {
-				adr[0]= floor(fx+0.5); 
-				adr[1]= floor(fy+0.5);
+				adr[0]= floor(fx); 
+				adr[1]= floor(fy);
 			}
 		}
 	}
@@ -405,13 +433,12 @@ void calctrackballvec(rcti *area, short *mval, float *vec)
 void viewmove(int mode)
 {
 	float firstvec[3], newvec[3], dvec[3];
-	float oldquat[4], q1[4], si, phi;
+	float oldquat[4], q1[4], si, phi, dist0;
 	int firsttime=1;
 	short mvalball[2], mval[2], mvalo[2];
 	
 	/* sometimes this routine is called from headerbuttons */
 	areawinset(curarea->win);
-	curarea->head_swap= 0;
 	
 	initgrabz(-G.vd->ofs[0], -G.vd->ofs[1], -G.vd->ofs[2]);
 	
@@ -420,6 +447,7 @@ void viewmove(int mode)
 	getmouseco_sc(mvalo);		/* work with screen coordinates because of trackball function */
 	mvalball[0]= mvalo[0];			/* needed for turntable to work */
 	mvalball[1]= mvalo[1];
+	dist0= G.vd->dist;
 	
 	calctrackballvec(&curarea->winrct, mvalo, firstvec);
 
@@ -428,9 +456,11 @@ void viewmove(int mode)
 	while(TRUE) {
 		getmouseco_sc(mval);
 		
-		if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1] || (G.f & G_PLAYANIM)) {
+		// if playanim = alt+A, screenhandlers are for animated UI, python, etc
+		if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1] || (G.f & G_PLAYANIM) || do_screenhandlers(G.curscreen)) {
 			
 			if(firsttime) {
+				
 				firsttime= 0;
 				/* are we translating, rotating or zooming? */
 				if(mode==0) {
@@ -447,10 +477,9 @@ void viewmove(int mode)
 
 
 			if(mode==0) {	/* view rotate */
-			
-				/* if turntable method, we don't change mvalball[0] */
-			
-				if(U.flag & USER_TRACKBALL) mvalball[0]= mval[0];
+				if (U.uiflag & USER_AUTOPERSP) G.vd->persp= 1;
+
+				if (U.flag & USER_TRACKBALL) mvalball[0]= mval[0];
 				mvalball[1]= mval[1];
 				
 				calctrackballvec(&curarea->winrct, mvalball, newvec);
@@ -459,36 +488,65 @@ void viewmove(int mode)
 				
 				si= sqrt(dvec[0]*dvec[0]+ dvec[1]*dvec[1]+ dvec[2]*dvec[2]);
 				si/= (2.0*TRACKBALLSIZE);
-				
-				/* is there an acceptable solution? (180 degrees limitor) */
-				if(si<1.0) {
+			
+				if (U.flag & USER_TRACKBALL) {
 					Crossf(q1+1, firstvec, newvec);
-
-					Normalise(q1+1);
-		
-					phi= asin(si);
 	
+					Normalise(q1+1);
+	
+					/* Allow for rotation beyond the interval
+					 * [-pi, pi] */
+					while (si > 1.0)
+						si -= 2.0;
+		
+					/* This relation is used instead of
+					 * phi = asin(si) so that the angle
+					 * of rotation is linearly proportional
+					 * to the distance that the mouse is
+					 * dragged. */
+					phi = si * M_PI / 2.0;
+		
 					si= sin(phi);
 					q1[0]= cos(phi);
 					q1[1]*= si;
 					q1[2]*= si;
-					q1[3]*= si;
-					
+					q1[3]*= si;						
 					QuatMul(G.vd->viewquat, q1, oldquat);
+				} else {
+					/* New turntable view code by John Aughey */
 
-					if( (U.flag & USER_TRACKBALL)==0 ) {
-					
-						/* rotate around z-axis (mouse x moves)  */
-						
-						phi= 2*(mval[0]-mvalball[0]);
-						phi/= (float)curarea->winx;
-						si= sin(phi);
-						q1[0]= cos(phi);
-						q1[1]= q1[2]= 0.0;
-						q1[3]= si;
-						
-						QuatMul(G.vd->viewquat, G.vd->viewquat, q1);
-					}
+					float m[3][3];
+					float m_inv[3][3];
+					float xvec[3] = {1,0,0};
+					/* Sensitivity will control how fast the viewport rotates.  0.0035 was
+					   obtained experimentally by looking at viewport rotation sensitivities
+					   on other modeling programs. */
+					/* Perhaps this should be a configurable user parameter. */
+					const float sensitivity = 0.0035;
+
+					/* Get the 3x3 matrix and its inverse from the quaternion */
+					QuatToMat3(G.vd->viewquat, m);
+					Mat3Inv(m_inv,m);
+
+					/* Determine the direction of the x vector (for rotating up and down) */
+					/* This can likely be compuated directly from the quaternion. */
+					Mat3MulVecfl(m_inv,xvec);
+
+					/* Perform the up/down rotation */
+					phi = sensitivity * -(mval[1] - mvalo[1]);
+					si = sin(phi);
+					q1[0] = cos(phi);
+					q1[1] = si * xvec[0];
+					q1[2] = si * xvec[1];
+					q1[3] = si * xvec[2];
+					QuatMul(G.vd->viewquat, G.vd->viewquat, q1);
+
+					/* Perform the orbital rotation */
+					phi = sensitivity * (mval[0] - mvalo[0]);
+					q1[0] = cos(phi);
+					q1[1] = q1[2] = 0.0;
+					q1[3] = sin(phi);
+					QuatMul(G.vd->viewquat, G.vd->viewquat, q1);
 				}
 			}
 			else if(mode==1) {	/* translate */
@@ -509,13 +567,34 @@ void viewmove(int mode)
 				}
 			}
 			else if(mode==2) {
-				G.vd->dist*= 1.0+(float)(mvalo[0]-mval[0]+mvalo[1]-mval[1])/1000.0;
+				if(U.viewzoom==USER_ZOOM_CONT) {
+					// oldstyle zoom
+					G.vd->dist*= 1.0+(float)(mvalo[0]-mval[0]+mvalo[1]-mval[1])/1000.0;
+				}
+				else if(U.viewzoom==USER_ZOOM_SCALE) {
+					int ctr[2], len1, len2;
+					// method which zooms based on how far you move the mouse
+					
+					ctr[0] = (curarea->winrct.xmax + curarea->winrct.xmin)/2;
+					ctr[1] = (curarea->winrct.ymax + curarea->winrct.ymin)/2;
+					
+					len1 = (int)sqrt((ctr[0] - mval[0])*(ctr[0] - mval[0]) + (ctr[1] - mval[1])*(ctr[1] - mval[1])) + 5;
+					len2 = (int)sqrt((ctr[0] - mvalo[0])*(ctr[0] - mvalo[0]) + (ctr[1] - mvalo[1])*(ctr[1] - mvalo[1])) + 5;
+					
+					G.vd->dist= dist0 * ((float)len2/len1);
+				}
+				else {	/* USER_ZOOM_DOLLY */
+					float len1 = (curarea->winrct.ymax - mval[1]) + 5;
+					float len2 = (curarea->winrct.ymax - mvalo[1]) + 5;
+					
+					G.vd->dist= dist0 * (2.0*((len2/len1)-1.0) + 1.0);
+				}
 				
 				/* these limits are in toets.c too */
 				if(G.vd->dist<0.001*G.vd->grid) G.vd->dist= 0.001*G.vd->grid;
 				if(G.vd->dist>10.0*G.vd->far) G.vd->dist=10.0*G.vd->far;
 				
-				mval[1]= mvalo[1]; /* keeps zooming that way */
+				mval[1]= mvalo[1]; /* preserve first value */
 				mval[0]= mvalo[0];
 			}
 			
@@ -529,23 +608,27 @@ void viewmove(int mode)
 			screen_swapbuffers();
 		}
 		else {
+			short val;
+			unsigned short event;
+			/* we need to empty the queue... when you do this very long it overflows */
+			while(qtest()) event= extern_qread(&val);
+			
 			BIF_wait_for_statechange();
 		}
 		
 		/* this in the end, otherwise get_mbut does not work on a PC... */
 		if( !(get_mbut() & (L_MOUSE|M_MOUSE))) break;
 	}
-
-	curarea->head_swap= WIN_FRONT_OK;
 }
 
 short v3d_windowmode=0;
 
+/* important to not set windows active in here, can be renderwin for example */
 void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
 {
 	Camera *cam=0;
-	float d, near, far, winx = 0.0, winy = 0.0;
-	float lens, dfac, fac, x1, y1, x2, y2;
+	float near, far, winx = 0.0, winy = 0.0;
+	float lens, fac, x1, y1, x2, y2;
 	short orth;
 	
 	lens= G.vd->lens;	
@@ -574,17 +657,11 @@ void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
 				lens= cam->lens;
 				near= cam->clipsta;
 				far= cam->clipend;
-				
-				if(cam->type==CAM_ORTHO) {
-					lens*= 100.0;
-					near= (near+1.0)*100.0;	/* otherwise zbuffer troubles. a Patch! */
-					far*= 100.0;
-				}
 			}
 		}
 	}
 	
-	if(v3d_windowmode) {
+	if(v3d_windowmode) { // hackish
 		winx= R.rectx;
 		winy= R.recty;
 	}
@@ -593,21 +670,16 @@ void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
 		winy= curarea->winy;
 	}
 	
-	if(winx>winy) d= 0.015625*winx*lens;
-	else d= 0.015625*winy*lens;
-	
-	dfac= near/d;
-
 	if(G.vd->persp==0) {
 		if(winx>winy) x1= -G.vd->dist;
 		else x1= -winx*G.vd->dist/winy;
-		
 		x2= -x1;
 
 		if(winx>winy) y1= -winy*G.vd->dist/winx;
 		else y1= -G.vd->dist;
-		
 		y2= -y1;
+		
+		near= -far;
 		orth= 1;
 	}
 	else {
@@ -617,12 +689,32 @@ void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
 		}
 		else fac= 2.0;
 		
-		x1= -dfac*(winx/fac);
-		x2= -x1;
-		y1= -dfac*(winy/fac);
-		y2= -y1;
-
-		orth= 0;
+		/* viewplane size depends... */
+		if(cam && cam->type==CAM_ORTHO) {
+			/* ortho_scale == 1 means exact 1 to 1 mapping */
+			float dfac= 2.0*cam->ortho_scale/fac;
+			
+			if(winx>winy) x1= -dfac;
+			else x1= -winx*dfac/winy;
+			x2= -x1;
+			
+			if(winx>winy) y1= -winy*dfac/winx;
+			else y1= -dfac;
+			y2= -y1;
+			orth= 1;
+		}
+		else {
+			float dfac;
+			
+			if(winx>winy) dfac= 64.0/(fac*winx*lens);
+			else dfac= 64.0/(fac*winy*lens);
+			
+			x1= - near*winx*dfac;
+			x2= -x1;
+			y1= - near*winy*dfac;
+			y2= -y1;
+			orth= 0;
+		}
 	}
 
 	if(rect) {		/* picking */
@@ -641,18 +733,12 @@ void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
 	}
 	else {
 		if(v3d_windowmode) {
-			if(orth) i_ortho(x1, x2, y1, y2, -far, far, R.winmat);
-			else {
-				if(cam && cam->type==CAM_ORTHO) i_window(x1, x2, y1, y2, near, far, R.winmat);
-				else i_window(x1, x2, y1, y2, near, far, R.winmat);
-			}
+			if(orth) i_ortho(x1, x2, y1, y2, near, far, R.winmat);
+			else i_window(x1, x2, y1, y2, near, far, R.winmat);
 		}
 		else {
-			if(orth) myortho(x1, x2, y1, y2, -far, far);
-			else {
-				if(cam && cam->type==CAM_ORTHO) mywindow(x1, x2, y1, y2, near, far);
-				else mywindow(x1, x2, y1, y2, near, far);
-			}
+			if(orth) myortho(x1, x2, y1, y2, near, far);
+			else mywindow(x1, x2, y1, y2, near, far);
 		}
 	}
 
@@ -678,7 +764,7 @@ void obmat_to_viewmat(Object *ob)
 	Mat3ToQuat(tmat, G.vd->viewquat);
 }
 
-
+/* dont set windows active in in here, is used by renderwin too */
 void setviewmatrixview3d()
 {
 	Camera *cam;
@@ -691,7 +777,7 @@ void setviewmatrixview3d()
 			
 			if(G.vd->camera->type==OB_CAMERA) {
 				cam= G.vd->camera->data;
-				if(cam->type==CAM_ORTHO) G.vd->viewmat[3][2]*= 100.0;
+				//if(cam->type==CAM_ORTHO) G.vd->viewmat[3][2]*= 100.0;
 			}
 		}
 		else {
@@ -707,7 +793,7 @@ void setviewmatrixview3d()
 	}
 }
 
-void setcameratoview3d()
+void setcameratoview3d(void)
 {
 	Object *ob;
 	float dvec[3];
@@ -738,10 +824,10 @@ short selectprojektie(unsigned int *buffer, short x1, short y1, short x2, short
 	
 	if(x1==0 && x2==0 && y1==0 && y2==0) {
 		getmouseco_areawin(mval);
-		rect.xmin= mval[0]-7;
-		rect.xmax= mval[0]+7;
-		rect.ymin= mval[1]-7;
-		rect.ymax= mval[1]+7;
+		rect.xmin= mval[0]-12;	// seems to be default value for bones only now
+		rect.xmax= mval[0]+12;
+		rect.ymin= mval[1]-12;
+		rect.ymax= mval[1]+12;
 	}
 	else {
 		rect.xmin= x1;
@@ -785,7 +871,7 @@ short selectprojektie(unsigned int *buffer, short x1, short y1, short x2, short
 	}
 	glPopName();	/* see above (pushname) */
 	hits= glRenderMode(GL_RENDER);
-	if(hits<0) error("Too many objects in selectbuf");
+	if(hits<0) error("Too many objects in select buffer");
 
 	G.f &= ~G_PICKSEL;
 	setwinmatrixview3d(0);
@@ -889,7 +975,7 @@ void initlocalview()
 		afm[0]= (max[0]-min[0]);
 		afm[1]= (max[1]-min[1]);
 		afm[2]= (max[2]-min[2]);
-		size= MAX3(afm[0], afm[1], afm[2]);
+		size= 0.7*MAX3(afm[0], afm[1], afm[2]);
 		if(size<=0.01) size= 0.01;
 	}
 	
@@ -903,6 +989,13 @@ void initlocalview()
 
 		G.vd->dist= size;
 
+		// correction for window aspect ratio
+		if(curarea->winy>2 && curarea->winx>2) {
+			size= (float)curarea->winx/(float)curarea->winy;
+			if(size<1.0) size= 1.0/size;
+			G.vd->dist*= size;
+		}
+		
 		if(G.vd->persp>1) {
 			G.vd->persp= 1;
 			
@@ -950,6 +1043,10 @@ void centreview()	/* like a localview without local! */
 		
 		ok= 1;
 	}
+	else if (G.f & G_FACESELECT) {
+		minmax_tface(min, max);
+		ok= 1;
+	}
 	else {
 		base= FIRSTBASE;
 		while(base) {
@@ -966,21 +1063,25 @@ void centreview()	/* like a localview without local! */
 	afm[0]= (max[0]-min[0]);
 	afm[1]= (max[1]-min[1]);
 	afm[2]= (max[2]-min[2]);
-	size= MAX3(afm[0], afm[1], afm[2]);
+	size= 0.7*MAX3(afm[0], afm[1], afm[2]);
 	
 	if(size<=0.01) size= 0.01;
 	
-	
-
 	G.vd->ofs[0]= -(min[0]+max[0])/2.0;
 	G.vd->ofs[1]= -(min[1]+max[1])/2.0;
 	G.vd->ofs[2]= -(min[2]+max[2])/2.0;
 
 	G.vd->dist= size;
 
+	// correction for window aspect ratio
+	if(curarea->winy>2 && curarea->winx>2) {
+		size= (float)curarea->winx/(float)curarea->winy;
+		if(size<1.0) size= 1.0/size;
+		G.vd->dist*= size;
+	}
+	
 	if(G.vd->persp>1) {
 		G.vd->persp= 1;
-		
 	}
 
 	G.vd->cursor[0]= -G.vd->ofs[0];
@@ -1078,7 +1179,7 @@ void view3d_home(int centre)
 	afm[0]= (max[0]-min[0]);
 	afm[1]= (max[1]-min[1]);
 	afm[2]= (max[2]-min[2]);
-	size= MAX3(afm[0], afm[1], afm[2]);
+	size= 0.7*MAX3(afm[0], afm[1], afm[2]);
 	if(size==0.0) ok= 0;
 		
 	if(ok) {
@@ -1120,4 +1221,5 @@ void view3d_align_axis_to_vector(View3D *v3d, int axisidx, float vec[3])
 
 	v3d->view= 0;
 	if (v3d->persp>=2) v3d->persp= 0; /* switch out of camera mode */
-}
\ No newline at end of file
+}
+
