@@ -1,5 +1,5 @@
 /**
- * $Id: vpaint.c,v 1.22 2004/11/07 13:15:04 blendix Exp $
+ * $Id: vpaint.c,v 1.28 2005/05/18 20:32:32 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -39,7 +39,6 @@
 
 #ifdef WIN32
 #include <io.h>
-#include "BLI_winstuff.h"
 #else
 #include <unistd.h>
 #endif   
@@ -55,15 +54,17 @@
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
+#include "DNA_object_force.h"
 #include "DNA_screen_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_view3d_types.h"
 #include "DNA_userdef_types.h"
 
-#include "BKE_utildefines.h"
-#include "BKE_mesh.h"
 #include "BKE_displist.h"
 #include "BKE_global.h"
+#include "BKE_mesh.h"
+#include "BKE_object.h"
+#include "BKE_utildefines.h"
 
 #include "BIF_graphics.h"
 #include "BIF_interface.h"
@@ -243,27 +244,34 @@ void make_vertexcol()	/* single ob */
 		me->flag &= ~ME_TWOSIDED;
 	}
 	
-	dl= ob->disp.first;
-	
-	if(dl==0 || dl->col1==NULL) {
-		shadeDispList(ob);
-		dl= ob->disp.first;
-	}
-	if(dl && dl->col1) {
-		int i;
-		
+	/* but subsurf vertex colors are wrong */
+	if (me->flag & ME_SUBSURF) {
 		if(me->mcol) MEM_freeN(me->mcol);
+		me->mcol= MEM_callocN(4*me->totface*sizeof(MCol), "mcol");
+	}
+	else {
+		dl= ob->disp.first;
 		
-		me->mcol= MEM_dupallocN(dl->col1);
-		if (me->mcol) {
-			for (i=0; i<me->totface*4; i++) {
-				MCol *mcol= &me->mcol[i];
-				mcol->a= 255;
-			}
+		if(dl==NULL || dl->col1==NULL) {
+			shadeDispList(ob);
+			dl= find_displist(&ob->disp, DL_VERTCOL);
+		}
+		if(dl && dl->col1) {
+			if(me->mcol) MEM_freeN(me->mcol);
+			me->mcol= MEM_dupallocN(dl->col1);
+		}
+	}
+	
+	if(me->mcol) {
+		int i;
+		for (i=0; i<me->totface*4; i++) {
+			MCol *mcol= &me->mcol[i];
+			mcol->a= 255;
 		}
 		
 		if(me->tface) mcol_to_tface(me, 1);
 	}
+	
 	freedisplist(&(ob->disp));
 	
 	
@@ -328,6 +336,8 @@ void clear_vpaint()
 
 	ob= OBACT;
 	me= get_mesh(ob);
+	if(ob->id.lib) return;
+
 	if(me==0 || me->totface==0) return;
 
 	if(me->tface) tface_to_mcol(me);
@@ -792,6 +802,8 @@ void weight_paint(void)
 	if(indexar==NULL) init_vertexpaint();
 	
 	ob= OBACT;
+	if(ob->id.lib) return;
+
 	me= get_mesh(ob);
 	if (!me->dvert){
 		return;
@@ -931,6 +943,8 @@ void weight_paint(void)
 	}
 	
 	makeDispList(ob);
+	// this flag is event for softbody to refresh weightpaint values
+	if(ob->soft) ob->softflag |= OB_SB_REDO;
 	
 	allqueue(REDRAWVIEW3D, 0);
 
@@ -954,8 +968,10 @@ void vertex_paint()
 	if(indexar==NULL) init_vertexpaint();
 	
 	ob= OBACT;
+	if(ob->id.lib) return;
+
 	me= get_mesh(ob);
-	if(me==0 || me->totface==0) return;
+	if(me==NULL || me->totface==0) return;
 	if(ob->lay & G.vd->lay); else error("Active object is not in this layer");
 	
 	if(me->tface==NULL && me->mcol==NULL) make_vertexcol();
@@ -1116,6 +1132,7 @@ void set_wpaint(void)		/* toggle */
 	
 	scrarea_queue_headredraw(curarea);
 	ob= OBACT;
+	if(ob->id.lib) return;
 	me= get_mesh(ob);
 		
 	if(me && me->totface>=MAXINDEX) {
@@ -1152,6 +1169,11 @@ void set_vpaint(void)		/* toggle */
 	
 	scrarea_queue_headredraw(curarea);
 	ob= OBACT;
+	if(ob->id.lib) {
+		G.f &= ~G_VERTEXPAINT;
+		return;
+	}
+	
 	me= get_mesh(ob);
 	
 	if(me && me->totface>=MAXINDEX) {
