@@ -1,5 +1,5 @@
 /**
- * $Id: BL_KetsjiEmbedStart.cpp,v 1.5 2004/03/31 12:20:01 kester Exp $
+ * $Id: BL_KetsjiEmbedStart.cpp,v 1.7 2005/03/25 10:33:35 kester Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -35,6 +35,8 @@
 #include <config.h>
 #endif
 
+#include <signal.h>
+
 #ifdef WIN32
 // don't show stl-warnings
 #pragma warning (disable:4786)
@@ -100,7 +102,7 @@ extern "C" void StartKetsjiShell(struct ScrArea *area,
 	BlendFileData *bfd= NULL;
 	
 	bgl::InitExtensions(1);
-
+	
 	do
 	{
 		View3D *v3d= (View3D*) area->spacedata.first;
@@ -221,6 +223,26 @@ extern "C" void StartKetsjiShell(struct ScrArea *area,
 			}
 		}
 		
+		Scene *blscene = NULL;
+		if (!bfd)
+		{
+			blscene = (Scene*) maggie->scene.first;
+			for (Scene *sce= (Scene*) maggie->scene.first; sce; sce= (Scene*) sce->id.next)
+			{
+				if (startscenename == (sce->id.name+2))
+				{
+					blscene = sce;
+					break;
+				}
+			}
+		} else {
+			blscene = bfd->curscene;
+		}
+
+		// Quad buffered needs a special window.
+		if (blscene->r.stereomode != RAS_IRasterizer::RAS_STEREO_QUADBUFFERED)
+			rasterizer->SetStereoMode((RAS_IRasterizer::StereoMode) blscene->r.stereomode);
+		
 		if (exitrequested != KX_EXIT_REQUEST_QUIT_GAME)
 		{
 			if (v3d->persp != 2)
@@ -324,7 +346,6 @@ extern "C" void StartKetsjiShell(struct ScrArea *area,
 		
 		// clean up some stuff
 		audiodevice->StopCD();
-		SND_DeviceManager::Unsubscribe();
 		
 		if (ketsjiengine)
 		{
@@ -366,6 +387,9 @@ extern "C" void StartKetsjiShell(struct ScrArea *area,
 			delete canvas;
 			canvas = NULL;
 		}
+		
+		SND_DeviceManager::Unsubscribe();
+	
 	} while (exitrequested == KX_EXIT_REQUEST_RESTART_GAME || exitrequested == KX_EXIT_REQUEST_START_OTHER_GAME);
 
 	if (bfd) BLO_blendfiledata_free(bfd);
