@@ -1,5 +1,5 @@
 /**
-* $Id: BL_ActionActuator.cpp,v 1.5 2004/03/22 22:01:25 jesterking Exp $
+* $Id: BL_ActionActuator.cpp,v 1.12 2004/12/05 02:50:57 kester Exp $
 *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -30,6 +30,12 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
 
+#if defined (__sgi)
+#include <math.h>
+#else
+#include <cmath>
+#endif
+
 #include "SCA_LogicManager.h"
 #include "BL_ActionActuator.h"
 #include "BL_ArmatureObject.h"
@@ -77,7 +83,7 @@ void BL_ActionActuator::ProcessReplica(){
 	
 	m_pose = NULL;
 	m_blendpose = NULL;
-	m_localtime=m_starttime;
+	m_localtime=m_startframe;
 	m_lastUpdate=-1;
 	
 }
@@ -93,9 +99,62 @@ CValue* BL_ActionActuator::GetReplica() {
 	// this will copy properties and so on...
 	CValue::AddDataToReplica(replica);
 	return replica;
-};
+}
 
-bool BL_ActionActuator::Update(double curtime,double deltatime)
+bool BL_ActionActuator::ClampLocalTime()
+{
+	if (m_startframe < m_endframe)
+	{
+		if (m_localtime < m_startframe)
+		{
+			m_localtime = m_startframe;
+			return true;
+		} 
+		else if (m_localtime > m_endframe)
+		{
+			m_localtime = m_endframe;
+			return true;
+		}
+	} else {
+		if (m_localtime > m_startframe)
+		{
+			m_localtime = m_startframe;
+			return true;
+		}
+		else if (m_localtime < m_endframe)
+		{
+			m_localtime = m_endframe;
+			return true;
+		}
+	}
+	return false;
+}
+
+void BL_ActionActuator::SetStartTime(float curtime)
+{
+	float direction = m_startframe < m_endframe ? 1.0 : -1.0;
+	
+	if (!(m_flag & ACT_FLAG_REVERSE))
+		m_starttime = curtime - direction*(m_localtime - m_startframe)/KX_FIXED_FRAME_PER_SEC;
+	else
+		m_starttime = curtime - direction*(m_endframe - m_localtime)/KX_FIXED_FRAME_PER_SEC;
+}
+
+void BL_ActionActuator::SetLocalTime(float curtime)
+{
+	float delta_time = (curtime - m_starttime)*KX_FIXED_FRAME_PER_SEC;
+	
+	if (m_endframe < m_startframe)
+		delta_time = -delta_time;
+
+	if (!(m_flag & ACT_FLAG_REVERSE))
+		m_localtime = m_startframe + delta_time;
+	else
+		m_localtime = m_endframe - delta_time;
+}
+
+
+bool BL_ActionActuator::Update(double curtime, bool frame)
 {
 	bool bNegativeEvent = false;
 	bool bPositiveEvent = false;
@@ -107,24 +166,34 @@ bool BL_ActionActuator::Update(double curtime,double deltatime)
 	
 	// result = true if animation has to be continued, false if animation stops
 	// maybe there are events for us in the queue !
-	
-	for (vector<CValue*>::iterator i=m_events.end(); !(i==m_events.begin());)
+	if (frame)
 	{
-		i--;
-		if ((*i)->GetNumber() == 0.0f)
+		for (vector<CValue*>::iterator i=m_events.end(); !(i==m_events.begin());)
 		{
-			bNegativeEvent = true;
+			i--;
+			if ((*i)->GetNumber() == 0.0f)
+				bNegativeEvent = true;
+			else
+				bPositiveEvent= true;
+			(*i)->Release();
+			m_events.pop_back();
+		}
+		
+		if (bPositiveEvent)
+			m_flag |= ACT_FLAG_ACTIVE;
+		
+		if (bNegativeEvent)
+		{
+			if (!(m_flag & ACT_FLAG_ACTIVE))
+				return false;
+			m_flag &= ~ACT_FLAG_ACTIVE;
 		}
-		else
-			bPositiveEvent= true;
-		(*i)->Release();
-		m_events.pop_back();
 	}
 	
 	/*	We know that action actuators have been discarded from all non armature objects:
 	if we're being called, we're attached to a BL_ArmatureObject */
 	BL_ArmatureObject *obj = (BL_ArmatureObject*)GetParent();
-	float length = m_endtime - m_starttime;
+	float length = m_endframe - m_startframe;
 	
 	priority = m_priority;
 	
@@ -148,7 +217,8 @@ bool BL_ActionActuator::Update(double curtime,double deltatime)
 				m_flag &= ~ACT_FLAG_KEYUP;
 				m_flag &= ~ACT_FLAG_REVERSE;
 				m_flag |= ACT_FLAG_LOCKINPUT;
-				m_localtime = m_starttime;
+				m_localtime = m_startframe;
+				m_starttime = curtime;
 			}
 		}
 		if (bNegativeEvent){
@@ -159,9 +229,14 @@ bool BL_ActionActuator::Update(double curtime,double deltatime)
 		if (bPositiveEvent){
 			if (!(m_flag & ACT_FLAG_LOCKINPUT)){
 				m_flag &= ~ACT_FLAG_REVERSE;
+				m_flag &= ~ACT_FLAG_KEYUP;
+				m_flag |= ACT_FLAG_LOCKINPUT;
+				SetStartTime(curtime);
 			}
 		}
 		if (bNegativeEvent){
+			m_flag |= ACT_FLAG_KEYUP;
+			m_flag &= ~ACT_FLAG_LOCKINPUT;
 			keepgoing=false;
 			apply=false;
 		}
@@ -170,10 +245,14 @@ bool BL_ActionActuator::Update(double curtime,double deltatime)
 		if (bPositiveEvent){
 			if (!(m_flag & ACT_FLAG_LOCKINPUT)){
 				m_flag &= ~ACT_FLAG_REVERSE;
+				m_flag |= ACT_FLAG_LOCKINPUT;
+				SetStartTime(curtime);
 			}
 		}
 		else if (bNegativeEvent){
 			m_flag |= ACT_FLAG_REVERSE;
+			m_flag &= ~ACT_FLAG_LOCKINPUT;
+			SetStartTime(curtime);
 		}
 		break;
 	case ACT_ACTION_PLAY:
@@ -181,6 +260,7 @@ bool BL_ActionActuator::Update(double curtime,double deltatime)
 			if (!(m_flag & ACT_FLAG_LOCKINPUT)){
 				m_flag &= ~ACT_FLAG_REVERSE;
 				m_localtime = m_starttime;
+				m_starttime = curtime;
 				m_flag |= ACT_FLAG_LOCKINPUT;
 			}
 		}
@@ -203,44 +283,28 @@ bool BL_ActionActuator::Update(double curtime,double deltatime)
 			m_lastpos = newpos;
 		}
 		else{
-			if (m_flag & ACT_FLAG_REVERSE)
-				m_localtime -= deltatime* KX_FIXED_FRAME_PER_SEC;
-			else
-				m_localtime += deltatime* KX_FIXED_FRAME_PER_SEC;
+			SetLocalTime(curtime);
 		}
 	}
 	
 	/* Check if a wrapping response is needed */
 	if (length){
-		if (m_flag & ACT_FLAG_REVERSE){
-			if (m_localtime < m_starttime){
-				m_localtime = m_endtime+(
-					(int)((m_localtime - m_starttime)/length)
-					*(int)length);
-				wrap = true;
-			}			
+		if (m_localtime < m_startframe || m_localtime > m_endframe)
+		{
+			m_localtime = m_startframe + fmod(m_localtime, length);
+			wrap = true;
 		}
-		else{
-			if (m_localtime > m_endtime){
-				m_localtime = m_starttime+(
-					(int)((m_localtime - m_endtime)/length)
-					*(int)length);
-				wrap = true;
-			}
-		}
-		
 	}
 	else
-		m_localtime = m_starttime;
+		m_localtime = m_startframe;
 	
 	/* Perform post-increment tasks */
 	switch (m_playtype){
 	case ACT_ACTION_FROM_PROP:
 		{
 			CValue* propval = GetParent()->GetProperty(m_propname);
-			if (propval)		 {
+			if (propval)
 				m_localtime = propval->GetNumber();
-			};
 			
 			if (bNegativeEvent){
 				keepgoing=false;
@@ -254,11 +318,11 @@ bool BL_ActionActuator::Update(double curtime,double deltatime)
 	case ACT_ACTION_FLIPPER:
 		if (wrap){
 			if (!(m_flag & ACT_FLAG_REVERSE)){
-				m_localtime=m_endtime;
-				keepgoing = false;
+				m_localtime=m_endframe;
+				//keepgoing = false;
 			}
 			else {
-				m_localtime=m_starttime;
+				m_localtime=m_startframe;
 				keepgoing = false;
 			}
 		}
@@ -267,14 +331,15 @@ bool BL_ActionActuator::Update(double curtime,double deltatime)
 		if (wrap){
 			if (m_flag & ACT_FLAG_KEYUP){
 				keepgoing = false;
-				m_localtime = m_endtime;
+				m_localtime = m_endframe;
 				m_flag &= ~ACT_FLAG_LOCKINPUT;
 			}
+			SetStartTime(curtime);
 		}
 		break;
 	case ACT_ACTION_PLAY:
 		if (wrap){
-			m_localtime = m_endtime;
+			m_localtime = m_endframe;
 			keepgoing = false;
 			m_flag &= ~ACT_FLAG_LOCKINPUT;
 		}
@@ -285,11 +350,8 @@ bool BL_ActionActuator::Update(double curtime,double deltatime)
 	}
 	
 	
-	if (bNegativeEvent){
+	if (bNegativeEvent)
 		m_blendframe=0.0;
-		
-	}
-	
 	
 	/* Apply the pose if necessary*/
 	if (apply){
@@ -316,6 +378,7 @@ bool BL_ActionActuator::Update(double curtime,double deltatime)
 				/* If this is the start of a blending sequence... */
 				if ((m_blendframe==0.0) || (!m_blendpose)){
 					obj->GetMRDPose(&m_blendpose);
+					m_blendstart = curtime;
 				}
 				
 				/* Find percentages */
@@ -323,7 +386,7 @@ bool BL_ActionActuator::Update(double curtime,double deltatime)
 				blend_poses(m_pose, m_blendpose, 1.0 - newweight, POSE_BLEND);
 
 				/* Increment current blending percentage */
-				m_blendframe+=(deltatime*KX_FIXED_FRAME_PER_SEC);
+				m_blendframe = (curtime - m_blendstart)*KX_FIXED_FRAME_PER_SEC;
 				if (m_blendframe>m_blendin)
 					m_blendframe = m_blendin;
 				
@@ -395,11 +458,12 @@ PyMethodDef BL_ActionActuator::Methods[] = {
 	{"getProperty", (PyCFunction) BL_ActionActuator::sPyGetProperty, METH_VARARGS, GetProperty_doc},
 	{"setChannel", (PyCFunction) BL_ActionActuator::sPySetChannel, METH_VARARGS, SetChannel_doc},
 //	{"getChannel", (PyCFunction) BL_ActionActuator::sPyGetChannel, METH_VARARGS},
-	
+	{"getType", (PyCFunction) BL_ActionActuator::sPyGetType, METH_VARARGS, GetType_doc},	
+	{"setType", (PyCFunction) BL_ActionActuator::sPySetType, METH_VARARGS, SetType_doc},
 	{NULL,NULL} //Sentinel
 };
 
-PyObject* BL_ActionActuator::_getattr(char* attr) {
+PyObject* BL_ActionActuator::_getattr(const STR_String& attr) {
 	_getattr_up(SCA_IActuator);
 }
 
@@ -464,7 +528,7 @@ PyObject* BL_ActionActuator::PyGetEnd(PyObject* self,
 									  PyObject* kwds) {
 	PyObject *result;
 	
-	result = Py_BuildValue("f", m_endtime);
+	result = Py_BuildValue("f", m_endframe);
 	
 	return result;
 }
@@ -479,7 +543,7 @@ PyObject* BL_ActionActuator::PyGetStart(PyObject* self,
 										PyObject* kwds) {
 	PyObject *result;
 	
-	result = Py_BuildValue("f", m_starttime);
+	result = Py_BuildValue("f", m_startframe);
 	
 	return result;
 }
@@ -565,7 +629,7 @@ PyObject* BL_ActionActuator::PySetStart(PyObject* self,
 	
 	if (PyArg_ParseTuple(args,"f",&start))
 	{
-		m_starttime = start;
+		m_startframe = start;
 	}
 	
 	Py_INCREF(Py_None);
@@ -584,7 +648,7 @@ PyObject* BL_ActionActuator::PySetEnd(PyObject* self,
 	
 	if (PyArg_ParseTuple(args,"f",&end))
 	{
-		m_endtime = end;
+		m_endframe = end;
 	}
 	
 	Py_INCREF(Py_None);
@@ -670,10 +734,10 @@ PyObject* BL_ActionActuator::PySetFrame(PyObject* self,
 	if (PyArg_ParseTuple(args,"f",&frame))
 	{
 		m_localtime = frame;
-		if (m_localtime<m_starttime)
-			m_localtime=m_starttime;
-		else if (m_localtime>m_endtime)
-			m_localtime=m_endtime;
+		if (m_localtime<m_startframe)
+			m_localtime=m_startframe;
+		else if (m_localtime>m_endframe)
+			m_localtime=m_endframe;
 	}
 	
 	Py_INCREF(Py_None);
@@ -805,3 +869,42 @@ PyObject* BL_ActionActuator::PySetChannel(PyObject* self,
 	return Py_None;
 }
 
+/* getType */
+char BL_ActionActuator::GetType_doc[] =
+"getType()\n"
+"\tReturns the operation mode of the actuator.\n";
+PyObject* BL_ActionActuator::PyGetType(PyObject* self,
+                                       PyObject* args, 
+                                       PyObject* kwds) {
+    return Py_BuildValue("h", m_playtype);
+}
+
+/* setType */
+char BL_ActionActuator::SetType_doc[] =
+"setType(mode)\n"
+"\t - mode: Play (0), Flipper (2), LoopStop (3), LoopEnd (4) or Property (6)\n"
+"\tSet the operation mode of the actuator.\n";
+PyObject* BL_ActionActuator::PySetType(PyObject* self,
+                                       PyObject* args,
+                                       PyObject* kwds) {
+	short typeArg;
+                                                                                                             
+    if (!PyArg_ParseTuple(args, "h", &typeArg)) {
+        return NULL;
+    }
+
+	switch (typeArg) {
+	case KX_ACT_ACTION_PLAY:
+	case KX_ACT_ACTION_FLIPPER:
+	case KX_ACT_ACTION_LOOPSTOP:
+	case KX_ACT_ACTION_LOOPEND:
+	case KX_ACT_ACTION_PROPERTY:
+		m_playtype = typeArg;
+		break;
+	default:
+		printf("Invalid type for action actuator: %d\n", typeArg); /* error */
+    }
+	
+    Py_Return;
+}
+
