@@ -1,5 +1,5 @@
 /**
- * $Id: KX_ConvertSensors.cpp,v 1.5 2004/05/02 23:45:02 kester Exp $
+ * $Id: KX_ConvertSensors.cpp,v 1.7 2005/03/25 10:33:35 kester Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -65,7 +65,7 @@ probably misplaced */
 #include "KX_NearSensor.h"
 #include "KX_RadarSensor.h"
 #include "KX_MouseFocusSensor.h"
-
+#include "SCA_JoystickSensor.h"
 #include "KX_NetworkMessageSensor.h"
 
 #include "SCA_PropertySensor.h"
@@ -79,6 +79,7 @@ probably misplaced */
 #include "KX_BlenderKeyboardDevice.h"
 #include "KX_BlenderGL.h"
 #include "RAS_ICanvas.h"
+#include "PHY_IPhysicsEnvironment.h"
 
 #include "KX_KetsjiEngine.h"
 #include "KX_BlenderSceneConverter.h"
@@ -374,11 +375,23 @@ void BL_ConvertSensors(struct Object* blenderobject,
 					
 					// this sumoObject is not deleted by a gameobj, so delete it ourself
 					// later (memleaks)!
-					//SM_Object* sumoObj 		=	new SM_Object(shape,NULL,NULL,NULL);
-					//sumoObj->setMargin(blendernearsensor->dist);
-					//sumoObj->setPosition(gameobj->NodeGetWorldPosition());
+					float radius = blendernearsensor->dist;
+					PHY__Vector3 pos;
+					const MT_Vector3& wpos = gameobj->NodeGetWorldPosition();
+					pos[0] = wpos[0];
+					pos[1] = wpos[1];
+					pos[2] = wpos[2];
+					pos[3] = 0.f;
 					bool bFindMaterial = false;
-					gamesensor = new KX_NearSensor(eventmgr,gameobj,blendernearsensor->dist,blendernearsensor->resetdist,bFindMaterial,nearpropertyname,kxscene);
+					PHY_IPhysicsController* physCtrl = kxscene->GetPhysicsEnvironment()->CreateSphereController(radius,pos);
+
+					gamesensor = new KX_NearSensor(eventmgr,gameobj,
+						blendernearsensor->dist,
+						blendernearsensor->resetdist,
+						bFindMaterial,
+						nearpropertyname,kxscene,
+						physCtrl
+						);
 					
 				}
 				break;
@@ -548,12 +561,15 @@ void BL_ConvertSensors(struct Object* blenderobject,
 					// this sumoObject is not deleted by a gameobj, so delete it ourself
 					// later (memleaks)!
 					MT_Scalar smallmargin = 0.0;
-					MT_Scalar largemargin = 0.1;
+					MT_Scalar largemargin = 0.0;
 					
 					bool bFindMaterial = false;
+					PHY_IPhysicsController* ctrl = kxscene->GetPhysicsEnvironment()->CreateConeController(coneradius,coneheight);
+
 					gamesensor = new KX_RadarSensor(
 						eventmgr,
 						gameobj,
+						ctrl,
 						coneradius,
 						coneheight,
 						radaraxis,
@@ -609,6 +625,62 @@ void BL_ConvertSensors(struct Object* blenderobject,
 						gamesensor = new SCA_RandomSensor(eventmgr, gameobj, randomSeed);
 					}
 				}
+				break;
+			}
+		case SENS_JOYSTICK:
+			{
+				int joysticktype = SCA_JoystickSensor::KX_JOYSENSORMODE_NODEF;
+				
+				bJoystickSensor* bjoy = (bJoystickSensor*) sens->data;
+				
+				SCA_JoystickManager *eventmgr 
+					= (SCA_JoystickManager*) logicmgr->FindEventManager(SCA_EventManager::JOY_EVENTMGR);
+				if (eventmgr) 
+				{
+					int axis	=0;
+					int axisf	=0;
+					int button	=0;
+					int buttonf =0; 
+					int hat		=0; 
+					int hatf	=0;
+					int prec	=0;
+					
+					switch(bjoy->type)
+					{
+					case SENS_JOY_AXIS:
+						axis	= bjoy->axis;
+						axisf	= bjoy->axisf;
+						prec	= bjoy->precision;
+						joysticktype  = SCA_JoystickSensor::KX_JOYSENSORMODE_AXIS;
+						break;
+					case SENS_JOY_BUTTON:
+						button	= bjoy->button;
+						buttonf	= bjoy->buttonf;
+						joysticktype  = SCA_JoystickSensor::KX_JOYSENSORMODE_BUTTON;
+						break;
+					case SENS_JOY_HAT:
+						hat		= bjoy->hat;
+						hatf	= bjoy->hatf;
+						joysticktype  = SCA_JoystickSensor::KX_JOYSENSORMODE_HAT;
+						break;
+					default:
+						printf("Error: bad case statement\n");
+						break;
+					}
+					gamesensor = new SCA_JoystickSensor(
+						eventmgr,
+						gameobj,
+						joysticktype,
+						axis,axisf,
+						prec,
+						button,buttonf,
+						hat,hatf);
+				} 
+				else
+				{
+					printf("Error there was a problem finding the event manager\n");
+				}
+
 				break;
 			}
 		default:
