@@ -28,8 +28,6 @@ int listvalue_bufferlen(PyObject* list)
 	return ( ((CListValue*)list)->GetCount());
 }
 
-
-
 PyObject* listvalue_buffer_item(PyObject* list,int index)
 {
 	if (index >= 0 && index < ((CListValue*) list)->GetCount())
@@ -45,6 +43,28 @@ PyObject* listvalue_buffer_item(PyObject* list,int index)
 	return NULL;
 }
 
+PyObject* listvalue_mapping_subscript(PyObject* list,PyObject* pyindex)
+{
+	if (PyString_Check(pyindex))
+	{
+		STR_String  index(PyString_AsString(pyindex));
+		CValue *item = ((CListValue*) list)->FindValue(index);
+		if (item)
+			return (PyObject*) item;
+			
+	}
+	if (PyInt_Check(pyindex))
+	{
+		int index = PyInt_AsLong(pyindex);
+		return listvalue_buffer_item(list, index);
+	}
+	
+	PyObject *pyindex_str = PyObject_Repr(pyindex); /* new ref */
+	STR_String index_str(PyString_AsString(pyindex_str));
+	PyErr_Format(PyExc_KeyError, "'%s' not in list", index_str.Ptr());
+	Py_DECREF(pyindex_str);
+	return NULL;
+}
 
 
 /* just slice it into a python list... */
@@ -157,7 +177,7 @@ static PySequenceMethods listvalue_as_sequence = {
 /* Is this one used ? */
 static PyMappingMethods instance_as_mapping = {
 	(inquiry)listvalue_bufferlen, /*mp_length*/
-	0,//(binaryfunc)instance_subscript, /*mp_subscript*/
+	(binaryfunc)listvalue_mapping_subscript, /*mp_subscript*/
 	0,//(objobjargproc)instance_ass_subscript, /*mp_ass_subscript*/
 };
 
@@ -178,7 +198,7 @@ PyTypeObject CListValue::Type = {
 	__repr,			        /*tp_repr*/
 	0,			        /*tp_as_number*/
 	&listvalue_as_sequence, /*tp_as_sequence*/
-	0,			        /*tp_as_mapping*/
+	&instance_as_mapping,	        /*tp_as_mapping*/
 	0,			        /*tp_hash*/
 	0,				/*tp_call */
 };
@@ -205,7 +225,7 @@ PyMethodDef CListValue::Methods[] = {
 
 
 
-PyObject* CListValue::_getattr(char* attr) {
+PyObject* CListValue::_getattr(const STR_String& attr) {
 	_getattr_up(CValue);
 }
 
