@@ -1,5 +1,5 @@
 /**
- * $Id: SCA_ILogicBrick.cpp,v 1.4 2004/03/22 22:01:33 jesterking Exp $
+ * $Id: SCA_ILogicBrick.cpp,v 1.6 2004/06/26 09:15:39 kester Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -254,7 +254,7 @@ PyMethodDef SCA_ILogicBrick::Methods[] = {
 
 
 PyObject*
-SCA_ILogicBrick::_getattr(char* attr)
+SCA_ILogicBrick::_getattr(const STR_String& attr)
 {
   _getattr_up(CValue);
 }
@@ -309,12 +309,9 @@ PyObject* SCA_ILogicBrick::PyGetExecutePriority(PyObject* self,
 /* Conversions for making life better. */
 bool SCA_ILogicBrick::PyArgToBool(int boolArg)
 {
-	if (boolArg == KX_TRUE) {
+	if (boolArg) {
 		return true;
-	} else if (boolArg == KX_FALSE){
-		return false;
 	} else {
-		; /* error: bad conversion: how to catch this? */
 		return false;
 	}
 }
