@@ -1,5 +1,5 @@
 /**
- * $Id: KX_NetworkEventManager.h,v 1.6 2004/03/22 22:01:50 jesterking Exp $
+ * $Id: KX_NetworkEventManager.h,v 1.7 2004/10/16 11:41:49 kester Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -48,7 +48,7 @@ public:
 	virtual void RegisterSensor(class SCA_ISensor* sensor);
 	virtual void RemoveSensor(class SCA_ISensor* sensor);
 
-	virtual void NextFrame(double curtime, double deltatime);
+	virtual void NextFrame();
 	virtual void EndFrame();
 
 	SCA_LogicManager* GetLogicManager() { return m_logicmgr; }
