@@ -1,5 +1,5 @@
 /**
- * $Id: KX_GameObject.cpp,v 1.6 2004/04/11 02:50:00 kester Exp $
+ * $Id: KX_GameObject.cpp,v 1.15 2005/04/18 11:44:21 kester Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -55,6 +55,8 @@
 #include "KX_ClientObjectInfo.h"
 #include "RAS_BucketManager.h"
 
+#include "KX_PyMath.h"
+
 // This file defines relationships between parents and children
 // in the game engine.
 
@@ -70,18 +72,19 @@ KX_GameObject::KX_GameObject(
 	m_bSuspendDynamics(false),
 	m_bUseObjectColor(false),
 	m_bVisible(true),
-	m_pPhysicsController1(NULL)
+	m_pPhysicsController1(NULL),
+	m_isDeformable(false)
 {
 	m_ignore_activity_culling = false;
 	m_pClient_info = new KX_ClientObjectInfo(this, KX_ClientObjectInfo::ACTOR);
 	m_pSGNode = new SG_Node(this,sgReplicationInfo,callbacks);
 	
 	// define the relationship between this node and it's parent.
-
+	
 	KX_NormalParentRelation * parent_relation = 
 		KX_NormalParentRelation::New();
 	m_pSGNode->SetParentRelation(parent_relation);
-
+	
 
 };
 
@@ -186,7 +189,7 @@ void KX_GameObject::ProcessReplica(KX_GameObject* replica)
 	replica->m_pPhysicsController1 = NULL;
 	replica->m_pSGNode = NULL;
 	replica->m_pClient_info = new KX_ClientObjectInfo(*m_pClient_info);
-	replica->m_pClient_info->m_clientobject = replica;
+	replica->m_pClient_info->m_gameobject = replica;
 }
 
 
@@ -303,11 +306,13 @@ void KX_GameObject::UpdateNonDynas()
 void KX_GameObject::UpdateTransform()
 {
 	if (m_pPhysicsController1)
-	{
 		m_pPhysicsController1->SetSumoTransform(false);
-	}
 }
 
+void KX_GameObject::UpdateTransformFunc(SG_IObject* node, void* gameobj, void* scene)
+{
+	((KX_GameObject*)gameobj)->UpdateTransform();
+}
 
 
 void KX_GameObject::SetDebugColor(unsigned int bgra)
@@ -356,31 +361,6 @@ void KX_GameObject::UpdateIPO(float curframetime,
 	UpdateTransform();
 }
 
-
-/*
-void KX_GameObject::RegisterSumoObject(class SM_Scene* sumoScene,
-									   DT_SceneHandle solidscene,
-									   class SM_Object* sumoObj,
-									   const char* matname,
-									   bool isDynamic,
-									   bool isActor)
-{
-	m_bDyna = isDynamic;
-
-	// need easy access, not via 'node' etc.
-	m_pPhysicsController = new KX_PhysicsController(sumoScene,solidscene,sumoObj,isDynamic);
-				
-	GetSGNode()->AddSGController(m_pPhysicsController);
-	
-	m_pClient_info->m_type = (isActor ? 1 : 0);
-	m_pClient_info->m_clientobject = this;
-
-	// store materialname in auxinfo, needed for touchsensors
-	m_pClient_info->m_auxilary_info = (matname? (void*)(matname+2) : NULL);
-	m_pPhysicsController->SetObject(this->GetSGNode());
-}
-*/
-
 bool
 KX_GameObject::GetVisible(
 	void
@@ -408,7 +388,7 @@ KX_GameObject::MarkVisible(
 	 * determined on this level. Maybe change this to mesh level
 	 * later on? */
 	
-	double* fl = GetOpenGLMatrix();
+	double* fl = GetOpenGLMatrixPtr()->getPointer();
 	for (size_t i=0;i<m_meshes.size();i++)
 	{
 		m_meshes[i]->MarkVisible(fl,this,visible,m_bUseObjectColor,m_objectColor);
@@ -422,7 +402,7 @@ KX_GameObject::MarkVisible(
 	void
 	)
 {
-	double* fl = GetOpenGLMatrix();
+	double* fl = GetOpenGLMatrixPtr()->getPointer();
 	for (size_t i=0;i<m_meshes.size();i++)
 	{
 		m_meshes[i]->MarkVisible(fl,
@@ -464,10 +444,11 @@ void KX_GameObject::ResolveCombinedVelocities(
 ){
 	if (m_pPhysicsController1)
 	{
+
+		MT_Vector3 lv = lin_vel_local ? NodeGetWorldOrientation() * lin_vel : lin_vel;
+		MT_Vector3 av = ang_vel_local ? NodeGetWorldOrientation() * ang_vel : ang_vel;
 		m_pPhysicsController1->resolveCombinedVelocities(
-			lin_vel_local ? NodeGetWorldOrientation() * lin_vel : lin_vel,
-			ang_vel_local ? NodeGetWorldOrientation() * ang_vel : ang_vel			
-		);		
+			lv.x(),lv.y(),lv.z(),av.x(),av.y(),av.z());
 	}
 }
 
@@ -603,10 +584,10 @@ PyMethodDef KX_GameObject::Methods[] = {
 	{"setVisible",(PyCFunction) KX_GameObject::sPySetVisible, METH_VARARGS},  
 	{"setPosition", (PyCFunction) KX_GameObject::sPySetPosition, METH_VARARGS},
 	{"getPosition", (PyCFunction) KX_GameObject::sPyGetPosition, METH_VARARGS},
-	{"getLinearVelocity", (PyCFunction) KX_GameObject::sPyGetLinearVelocity, METH_VARARGS},
-	{"getVelocity", (PyCFunction) KX_GameObject::sPyGetVelocity, METH_VARARGS},
 	{"getOrientation", (PyCFunction) KX_GameObject::sPyGetOrientation, METH_VARARGS},
 	{"setOrientation", (PyCFunction) KX_GameObject::sPySetOrientation, METH_VARARGS},
+	{"getLinearVelocity", (PyCFunction) KX_GameObject::sPyGetLinearVelocity, METH_VARARGS},
+	{"getVelocity", (PyCFunction) KX_GameObject::sPyGetVelocity, METH_VARARGS},
 	{"getMass", (PyCFunction) KX_GameObject::sPyGetMass, METH_VARARGS},
 	{"getReactionForce", (PyCFunction) KX_GameObject::sPyGetReactionForce, METH_VARARGS},
 	{"applyImpulse", (PyCFunction) KX_GameObject::sPyApplyImpulse, METH_VARARGS},
@@ -617,14 +598,14 @@ PyMethodDef KX_GameObject::Methods[] = {
 	{"getParent", (PyCFunction)KX_GameObject::sPyGetParent,METH_VARARGS},
 	{"getMesh", (PyCFunction)KX_GameObject::sPyGetMesh,METH_VARARGS},
 	{"getPhysicsId", (PyCFunction)KX_GameObject::sPyGetPhysicsId,METH_VARARGS},
-	
+	KX_PYMETHODTABLE(KX_GameObject, getDistanceTo),
 	
 	{NULL,NULL} //Sentinel
 };
 
 
 
-
+/*
 bool KX_GameObject::ConvertPythonVectorArgs(PyObject* args,
 											MT_Vector3& pos,
 											MT_Vector3& pos2)
@@ -638,7 +619,7 @@ bool KX_GameObject::ConvertPythonVectorArgs(PyObject* args,
 		
 	return error;
 }
-
+*/
 
 
 PyObject* KX_GameObject::sPySetPosition(PyObject* self,
@@ -654,17 +635,7 @@ PyObject* KX_GameObject::PyGetPosition(PyObject* self,
 									   PyObject* args, 
 									   PyObject* kwds)
 {
-	MT_Point3 pos = NodeGetWorldPosition();
-	
-	PyObject* resultlist = PyList_New(3);
-	int index;
-	for (index=0;index<3;index++)
-	{
-		PyList_SetItem(resultlist,index,PyFloat_FromDouble(pos[index]));
-	}
-	
-	return resultlist;
-	
+	return PyObjectFrom(NodeGetWorldPosition());
 }
 
 
@@ -700,11 +671,137 @@ PyParentObject KX_GameObject::Parents[] = {
 
 
 
-PyObject* KX_GameObject::_getattr(char* attr)
+PyObject* KX_GameObject::_getattr(const STR_String& attr)
 {
+	if (m_pPhysicsController1)
+	{
+		if (attr == "mass")
+			return PyFloat_FromDouble(GetPhysicsController()->GetMass());
+	}
+
+	if (attr == "parent")
+	{	
+		KX_GameObject* parent = GetParent();
+		if (parent)
+		{
+			parent->AddRef();
+			return parent;
+		}
+		Py_Return;
+	}
+
+	if (attr == "visible")
+		return PyInt_FromLong(m_bVisible);
+	
+	if (attr == "position")
+		return PyObjectFrom(NodeGetWorldPosition());
+	
+	if (attr == "orientation")
+		return PyObjectFrom(NodeGetWorldOrientation());
+	
+	if (attr == "scaling")
+		return PyObjectFrom(NodeGetWorldScaling());
+		
+	if (attr == "name")
+		return PyString_FromString(m_name.ReadPtr());
+	
 	_getattr_up(SCA_IObject);
 }
 
+int KX_GameObject::_setattr(const STR_String& attr, PyObject *value)	// _setattr method
+{
+	if (attr == "mass")
+		return 1;
+	
+	if (attr == "parent")
+		return 1;
+		
+	if (PyInt_Check(value))
+	{
+		int val = PyInt_AsLong(value);
+		if (attr == "visible")
+		{
+			SetVisible(val != 0);
+			return 0;
+		}
+	}
+	
+	if (PySequence_Check(value))
+	{
+		if (attr == "orientation")
+		{
+			MT_Matrix3x3 rot;
+			if (PyObject_IsMT_Matrix(value, 3))
+			{
+				if (PyMatTo(value, rot))
+				{
+					NodeSetLocalOrientation(rot);
+					return 0;
+				}
+				return 1;
+			}
+			
+			if (PySequence_Size(value) == 4)
+			{
+				MT_Quaternion qrot;
+				if (PyVecTo(value, qrot))
+				{
+					rot.setRotation(qrot);
+					NodeSetLocalOrientation(rot);
+					return 0;
+				}
+				return 1;
+			}
+			
+			if (PySequence_Size(value) == 3)
+			{
+				MT_Vector3 erot;
+				if (PyVecTo(value, erot))
+				{
+					rot.setEuler(erot);
+					NodeSetLocalOrientation(rot);
+					return 0;
+				}
+				return 1;
+			}
+			
+			return 1;
+		}
+		
+		if (attr == "position")
+		{
+			MT_Point3 pos;
+			if (PyVecTo(value, pos))
+			{
+				NodeSetLocalPosition(pos);
+				return 0;
+			}
+			return 1;
+		}
+		
+		if (attr == "scaling")
+		{
+			MT_Vector3 scale;
+			if (PyVecTo(value, scale))
+			{
+				NodeSetLocalScale(scale);
+				return 0;
+			}
+			return 1;
+		}
+	}
+	
+	if (PyString_Check(value))
+	{
+		if (attr == "name")
+		{
+			m_name = PyString_AsString(value);
+			return 0;
+		}
+	}
+	
+	return SCA_IObject::_setattr(attr, value);
+}
 
 
 PyObject* KX_GameObject::PyGetLinearVelocity(PyObject* self, 
@@ -712,16 +809,7 @@ PyObject* KX_GameObject::PyGetLinearVelocity(PyObject* self,
 											 PyObject* kwds)
 {
 	// only can get the velocity if we have a physics object connected to us...
-	MT_Vector3 velocity = GetLinearVelocity();
-	
-	PyObject* resultlist = PyList_New(3);
-	int index;
-	for (index=0;index<3;index++)
-	{
-		PyList_SetItem(resultlist,index,PyFloat_FromDouble(velocity[index]));
-	}
-
-	return resultlist;
+	return PyObjectFrom(GetLinearVelocity());
 }
 
 
@@ -756,66 +844,19 @@ PyObject* KX_GameObject::PyGetVelocity(PyObject* self,
 	MT_Point3 point(0.0,0.0,0.0);
 	
 	
-	MT_Point3 pos;
-	PyObject* pylist;
-	bool	error = false;
-	
-	int len = PyTuple_Size(args);
-	
-	if ((len > 0) && PyArg_ParseTuple(args,"O",&pylist))
+	PyObject* pypos = NULL;
+	if (PyArg_ParseTuple(args, "|O", &pypos))
 	{
-		if (pylist->ob_type == &CListValue::Type)
-		{
-			CListValue* listval = (CListValue*) pylist;
-			if (listval->GetCount() == 3)
-			{
-				int index;
-				for (index=0;index<3;index++)
-				{
-					pos[index] = listval->GetValue(index)->GetNumber();
-				}
-			}	else
-			{
-				error = true;
-			}
-			
-		} else
-		{
-			
-			// assert the list is long enough...
-			int numitems = PyList_Size(pylist);
-			if (numitems == 3)
-			{
-				int index;
-				for (index=0;index<3;index++)
-				{
-					pos[index] = PyFloat_AsDouble(PyList_GetItem(pylist,index));
-				}
-			}
-			else
-			{
-				error = true;
-			}
-		}
-		
-		if (!error)
-			point = pos;	
+		if (pypos)
+			PyVecTo(pypos, point);
 	}
 	
-	
 	if (m_pPhysicsController1)
 	{
 		velocity = m_pPhysicsController1->GetVelocity(point);
 	}
 	
-	PyObject* resultlist = PyList_New(3);
-	int index;
-	for (index=0;index<3;index++)
-	{
-		PyList_SetItem(resultlist,index,PyFloat_FromDouble(velocity[index]));
-	}
-
-	return resultlist;
+	return PyObjectFrom(velocity);
 }
 
 
@@ -842,18 +883,7 @@ PyObject* KX_GameObject::PyGetReactionForce(PyObject* self,
 											PyObject* kwds)
 {
 	// only can get the velocity if we have a physics object connected to us...
-	
-	MT_Vector3 reaction_force = GetPhysicsController()->getReactionForce();
-	
-	PyObject* resultlist = PyList_New(3);
-	int index;
-	for (index=0;index<3;index++)
-	{
-		PyList_SetItem(resultlist,index,
-			PyFloat_FromDouble(reaction_force[index]));
-	}
-
-	return resultlist;
+	return PyObjectFrom(GetPhysicsController()->getReactionForce());
 }
 
 
@@ -887,7 +917,10 @@ PyObject* KX_GameObject::PyGetParent(PyObject* self,
 {
 	KX_GameObject* parent = this->GetParent();
 	if (parent)
+	{
+		parent->AddRef();
 		return parent;
+	}
 	Py_Return;
 }
 
@@ -897,12 +930,16 @@ PyObject* KX_GameObject::PyGetMesh(PyObject* self,
 								   PyObject* args, 
 								   PyObject* kwds)
 {
-	if (m_meshes.size() > 0)
-	{
-		KX_MeshProxy* meshproxy = new KX_MeshProxy(m_meshes[0]);
-		return meshproxy;
-	}
+	int mesh = 0;
 	
+	if (PyArg_ParseTuple(args, "|i", &mesh))
+	{
+		if (((unsigned int)mesh < m_meshes.size()) && mesh >= 0)
+		{
+			KX_MeshProxy* meshproxy = new KX_MeshProxy(m_meshes[mesh]);
+			return meshproxy;
+		}
+	}
 	Py_Return;
 }
 
@@ -913,19 +950,24 @@ PyObject* KX_GameObject::PyApplyImpulse(PyObject* self,
 										PyObject* kwds)
 {
 	
-	MT_Point3  attach(0, 1, 0);
-	MT_Vector3 impulse(1, 0, 0);
 	
-	if (ConvertPythonVectorArgs(args,attach,impulse))
+	PyObject* pyattach;
+	PyObject* pyimpulse;
+	
+	if (PyArg_ParseTuple(args, "OO", &pyattach, &pyimpulse))
 	{
-		if (m_pPhysicsController1)
+		MT_Point3  attach;
+		MT_Vector3 impulse;
+	
+		if (PyVecTo(pyattach, attach) && PyVecTo(pyimpulse, impulse) && m_pPhysicsController1)
 		{
 			m_pPhysicsController1->applyImpulse(attach, impulse);
+			Py_Return;
 		}
-		
+
 	}
 	
-	Py_Return;
+	return NULL;
 }
 
 
@@ -975,27 +1017,7 @@ PyObject* KX_GameObject::PyGetOrientation(PyObject* self,
 										  PyObject* args,
 										  PyObject* kwds) //keywords
 {
-	// do the conversion of a C++ matrix to a python list
-	
-	PyObject* resultlist = PyList_New(3);
-	
-	
-	int row,col;
-	const MT_Matrix3x3& orient = NodeGetWorldOrientation();
-	
-	for (row=0;row<3;row++)
-	{
-		PyObject* veclist = PyList_New(3);
-		
-		for (col=0;col<3;col++)
-		{
-			const MT_Scalar fl = orient[row][col];
-			PyList_SetItem(veclist,col,PyFloat_FromDouble(fl));
-		}
-		PyList_SetItem(resultlist,row,veclist);
-		
-	}
-	return resultlist;
+	return PyObjectFrom(NodeGetWorldOrientation());
 }
 
 
@@ -1004,68 +1026,26 @@ PyObject* KX_GameObject::PySetOrientation(PyObject* self,
 										  PyObject* args, 
 										  PyObject* kwds)
 {
-	MT_Matrix3x3 matrix;
-	
 	PyObject* pylist;
-	bool	error = false;
-	int row,col;
 	
-	PyArg_ParseTuple(args,"O",&pylist);
-	
-	if (pylist->ob_type == &CListValue::Type)
+	if (PyArg_ParseTuple(args,"O",&pylist))
 	{
-		CListValue* listval = (CListValue*) pylist;
-		if (listval->GetCount() == 3)
+		MT_Matrix3x3 matrix;
+		if (PyObject_IsMT_Matrix(pylist, 3) && PyMatTo(pylist, matrix))
 		{
-			for (row=0;row<3;row++) // each row has a 3-vector [x,y,z]
-			{
-				CListValue* vecval = (CListValue*)listval->GetValue(row);
-				for (col=0;col<3;col++)
-				{
-					matrix[row][col] = vecval->GetValue(col)->GetNumber();
-					
-				}
-			}
+			NodeSetLocalOrientation(matrix);
+			Py_Return;
 		}
-		else
+	
+		MT_Quaternion quat;
+		if (PyVecTo(pylist, quat))
 		{
-			error = true;
+			matrix.setRotation(quat);
+			NodeSetLocalOrientation(matrix);
+			Py_Return;
 		}
 	}
-	else
-	{
-		// assert the list is long enough...
-		int numitems = PyList_Size(pylist);
-		if (numitems == 3)
-		{
-			for (row=0;row<3;row++) // each row has a 3-vector [x,y,z]
-			{
-				
-				PyObject* veclist = PyList_GetItem(pylist,row); // here we have a vector3 list
-				for (col=0;col<3;col++)
-				{
-					matrix[row][col] =  PyFloat_AsDouble(PyList_GetItem(veclist,col));
-					
-				}
-			}
-		}
-		else
-		{
-			error = true;
-		}
-	}
-	
-	if (!error)
-	{
-		if (m_pPhysicsController1)
-		{
-			m_pPhysicsController1->setOrientation(matrix.getRotation());
-		}
-		NodeSetLocalOrientation(matrix);
-	}
-	
-	Py_INCREF(Py_None);
-	return Py_None;
+	return NULL;
 }
 
 
@@ -1074,17 +1054,14 @@ PyObject* KX_GameObject::PySetPosition(PyObject* self,
 									   PyObject* args, 
 									   PyObject* kwds)
 {
-	// make a general function for this, it's needed many times
-	
-	MT_Point3 pos = ConvertPythonVectorArg(args);
-	if (this->m_pPhysicsController1)
+	MT_Point3 pos;
+	if (PyVecArgTo(args, pos))
 	{
-		this->m_pPhysicsController1->setPosition(pos);
+		NodeSetLocalPosition(pos);
+		Py_Return;
 	}
-	NodeSetLocalPosition(pos);
 	
-	Py_INCREF(Py_None);
-	return Py_None;
+	return NULL;
 }
 
 PyObject* KX_GameObject::PyGetPhysicsId(PyObject* self,
@@ -1100,6 +1077,27 @@ PyObject* KX_GameObject::PyGetPhysicsId(PyObject* self,
 	return PyInt_FromLong(physid);
 }
 
+KX_PYMETHODDEF_DOC(KX_GameObject, getDistanceTo,
+"getDistanceTo(other): get distance to another point/KX_GameObject")
+{
+	MT_Point3 b;
+	if (PyVecArgTo(args, b))
+	{
+		return PyFloat_FromDouble(NodeGetWorldPosition().distance(b));
+	}
+	PyErr_Clear();
+	
+	PyObject *pyother;
+	if (PyArg_ParseTuple(args, "O!", &KX_GameObject::Type, &pyother))
+	{
+		KX_GameObject *other = static_cast<KX_GameObject*>(pyother);
+		return PyFloat_FromDouble(NodeGetWorldPosition().distance(other->NodeGetWorldPosition()));
+	}
+	
+	return NULL;
+}
+
+
 /* --------------------------------------------------------------------- 
  * Some stuff taken from the header
  * --------------------------------------------------------------------- */
