@@ -1,7 +1,7 @@
 /**
  * Do Ipo stuff
  *
- * $Id: KX_IpoActuator.cpp,v 1.4 2004/03/22 22:01:52 jesterking Exp $
+ * $Id: KX_IpoActuator.cpp,v 1.14 2004/12/06 10:58:38 kester Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -32,6 +32,12 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
+#if defined (__sgi)
+#include <math.h>
+#else
+#include <cmath>
+#endif
+ 
 #include "KX_IpoActuator.h"
 #include "KX_GameObject.h"
 
@@ -39,6 +45,8 @@
 #include <config.h>
 #endif
 
+#include "KX_KetsjiEngine.h"
+
 /* ------------------------------------------------------------------------- */
 /* Type strings                                                              */
 /* ------------------------------------------------------------------------- */
@@ -100,8 +108,8 @@ KX_IpoActuator::KX_IpoActuator(SCA_IObject* gameobj,
 							   PyTypeObject* T) 
 	: SCA_IActuator(gameobj,T),
 	m_bNegativeEvent(false),
-	m_starttime (starttime),
-	m_endtime(endtime),
+	m_startframe (starttime),
+	m_endframe(endtime),
 	m_recurse(recurse),
 	m_localtime(starttime),
 	m_direction(1),
@@ -110,143 +118,257 @@ KX_IpoActuator::KX_IpoActuator(SCA_IObject* gameobj,
 	m_force_ipo_local(force_ipo_local),
 	m_type((IpoActType)acttype)
 {
-	// intentionally empty
+	m_starttime = -2.0*fabs(m_endframe - m_startframe) - 1.0;
 }
 
 void KX_IpoActuator::SetStart(float starttime) 
 { 
-	m_starttime=starttime;
+	m_startframe=starttime;
 }
 
 void KX_IpoActuator::SetEnd(float endtime) 
 { 
-	m_endtime=endtime;
+	m_endframe=endtime;
 }
 
+bool KX_IpoActuator::ClampLocalTime()
+{
+	if (m_startframe < m_endframe)
+	{
+		if (m_localtime < m_startframe)
+		{
+			m_localtime = m_startframe;
+			return true;
+		} 
+		else if (m_localtime > m_endframe)
+		{
+			m_localtime = m_endframe;
+			return true;
+		}
+	} else {
+		if (m_localtime > m_startframe)
+		{
+			m_localtime = m_startframe;
+			return true;
+		}
+		else if (m_localtime < m_endframe)
+		{
+			m_localtime = m_endframe;
+			return true;
+		}
+	}
+	return false;
+}
 
-bool KX_IpoActuator::Update(double curtime,double delta_time)
+void KX_IpoActuator::SetStartTime(float curtime)
+{
+	float direction = m_startframe < m_endframe ? 1.0 : -1.0;
+	
+	if (m_direction > 0)
+		m_starttime = curtime - direction*(m_localtime - m_startframe)/KX_FIXED_FRAME_PER_SEC;
+	else
+		m_starttime = curtime - direction*(m_endframe - m_localtime)/KX_FIXED_FRAME_PER_SEC;
+}
+
+void KX_IpoActuator::SetLocalTime(float curtime)
+{
+	float delta_time = (curtime - m_starttime)*KX_FIXED_FRAME_PER_SEC;
+	
+	if (m_endframe < m_startframe)
+		delta_time = -delta_time;
+
+	if (m_direction > 0)
+		m_localtime = m_startframe + delta_time;
+	else
+		m_localtime = m_endframe - delta_time;
+}
+
+bool KX_IpoActuator::Update(double curtime, bool frame)
 {
-	SCA_IActuator::Update(curtime,delta_time);
 	// result = true if animation has to be continued, false if animation stops
 	// maybe there are events for us in the queue !
-	
-
 	bool bNegativeEvent = false;
-	int numevents = m_events.size();
+	int numevents = 0;
 
-	for (vector<CValue*>::iterator i=m_events.end(); !(i==m_events.begin());)
+	if (frame)
 	{
-		i--;
-		if ((*i)->GetNumber() == 0.0f)
+		numevents = m_events.size();
+		for (vector<CValue*>::iterator i=m_events.end(); !(i==m_events.begin());)
 		{
-			int ka=0;
-			bNegativeEvent = true;
+			--i;
+			if ((*i)->GetNumber() == 0.0f)
+				bNegativeEvent = true;
+			
+			(*i)->Release();
 		}
-		(*i)->Release();
-		m_events.pop_back();
-	}
-
-	if (bNegativeEvent)
-	{
-		RemoveAllEvents();
+		m_events.clear();
+	
+		if (bNegativeEvent)
+			RemoveAllEvents();
 	}
 	
-
-	double  start_smaller_then_end = ( m_starttime < m_endtime ? 1.0 : -1.0);
-
-	double deltaframetime = start_smaller_then_end  * delta_time * KX_FIXED_FRAME_PER_SEC;
+	double  start_smaller_then_end = ( m_startframe < m_endframe ? 1.0 : -1.0);
 
 	bool result=true;
+	if (m_starttime < -2.0*start_smaller_then_end*(m_endframe - m_startframe))
+		m_starttime = curtime;
 	
 	switch (m_type)
 	{
 		
 	case KX_ACT_IPO_PLAY:
+	{
+		// Check if playing forwards.  result = ! finished
+		if (start_smaller_then_end > 0.0)
+			result = (m_localtime < m_endframe && !(m_localtime == m_startframe && bNegativeEvent));
+		else
+			result = (m_localtime > m_endframe && !(m_localtime == m_startframe && bNegativeEvent));
+		
+		if (result)
 		{
-			
-			if (start_smaller_then_end > 0.0)
-				result = (m_localtime < m_endtime && !(m_localtime == m_starttime && bNegativeEvent));
-			else
-				result = (m_localtime > m_endtime && !(m_localtime == m_starttime && bNegativeEvent));
-			if (result)
-			{
-				m_localtime += m_direction * deltaframetime;
-				
-				/* Perform clamping */
-				if ((m_localtime*start_smaller_then_end)>(m_endtime*start_smaller_then_end))
-					m_localtime=m_endtime;
-
-				CIpoAction ipoaction(
-					(KX_GameObject*)GetParent(), 
-					m_localtime, 
-					m_recurse, 
-					m_ipo_as_force,
-					m_force_ipo_local);
-				GetParent()->Execute(ipoaction);
-			} else
-			{
-				m_localtime=m_starttime;
-				m_direction=1;
-			}
-			break;
-		}
-	case KX_ACT_IPO_PINGPONG:
-		{
-			result = true;
-			if (bNegativeEvent && ((m_localtime == m_starttime )|| (m_localtime == m_endtime)))
-			{
-				result = false;
-			} else
-			{
-				m_localtime += m_direction * deltaframetime;
-			}
-			
-			if (m_localtime*start_smaller_then_end < m_starttime*start_smaller_then_end)
-			{
-				m_localtime = m_starttime;
-				result = false;
-				m_direction = 1;
-			}else
-			if (m_localtime*start_smaller_then_end > m_endtime*start_smaller_then_end)
-			{
-				m_localtime = m_endtime;
-				result = false;
-				m_direction = -1;
-			} 
+			SetLocalTime(curtime);
+		
+			/* Perform clamping */
+			ClampLocalTime();
 			
 			CIpoAction ipoaction(
-				(KX_GameObject*) GetParent(),
-				m_localtime,
+				(KX_GameObject*)GetParent(), 
+				m_localtime, 
 				m_recurse, 
 				m_ipo_as_force,
 				m_force_ipo_local);
 			GetParent()->Execute(ipoaction);
-			break;
-		}
-
-	case KX_ACT_IPO_FLIPPER:
+		} else
 		{
-			result = true;
-			if (numevents)
-			{
-				if (bNegativeEvent)
-					m_direction = -1;
-				else
-					m_direction = 1;
-			}
-
-			m_localtime += m_direction * deltaframetime;
-
-			if (m_localtime*start_smaller_then_end > m_endtime*start_smaller_then_end)
-			{
-				m_localtime = m_endtime;
-			} else
-			 if (m_localtime*start_smaller_then_end < m_starttime*start_smaller_then_end)
-			 {
-				m_localtime = m_starttime;
-				result = false;
-			 }
+			m_localtime=m_startframe;
+			m_starttime=curtime;
+			m_direction=1;
+		}
+		break;
+	}
+	case KX_ACT_IPO_PINGPONG:
+	{
+		result = true;
+		if (bNegativeEvent && ((m_localtime == m_startframe )|| (m_localtime == m_endframe)))
+			result = false;
+		else
+			SetLocalTime(curtime);
 			
+		if (ClampLocalTime())
+		{
+			result = false;
+			m_direction = -m_direction;
+		}
+		
+		CIpoAction ipoaction(
+			(KX_GameObject*) GetParent(),
+			m_localtime,
+			m_recurse, 
+			m_ipo_as_force,
+			m_force_ipo_local);
+		GetParent()->Execute(ipoaction);
+		break;
+	}
+	case KX_ACT_IPO_FLIPPER:
+	{
+		result = !(bNegativeEvent && (m_localtime == m_startframe));
+		if (numevents)
+		{
+			if (bNegativeEvent)
+				m_direction = -1;
+			else
+				m_direction = 1;
+			SetStartTime(curtime);
+		}
+		
+		SetLocalTime(curtime);
+		
+		if (ClampLocalTime() && m_localtime == m_startframe)
+			result = false;
+			
+		CIpoAction ipoaction(
+			(KX_GameObject*) GetParent(),
+			m_localtime,
+			m_recurse,
+			m_ipo_as_force,
+			m_force_ipo_local);
+		GetParent()->Execute(ipoaction);
+		break;
+	}
+
+	case KX_ACT_IPO_LOOPSTOP:
+	{
+		if (numevents)
+		{
+			if (bNegativeEvent)
+			{
+				result = false;
+				m_bNegativeEvent = false;
+				numevents = 0;
+			}
+			SetStartTime(curtime);
+		} // fall through to loopend, and quit the ipo animation immediatly 
+	}
+	case KX_ACT_IPO_LOOPEND:
+	{
+		if (numevents){
+			if (bNegativeEvent){
+				m_bNegativeEvent = true;
+			}
+		}
+		
+		if (bNegativeEvent && m_localtime == m_startframe){
+			result = false;
+		} 
+		else
+		{
+			if (m_localtime*start_smaller_then_end < m_endframe*start_smaller_then_end)
+			{
+				SetLocalTime(curtime);
+			}
+			else{
+				if (!m_bNegativeEvent){
+					/* Perform wraparound */
+					SetLocalTime(curtime);
+					m_localtime = m_startframe + fmod(m_localtime, m_startframe - m_endframe);
+					SetStartTime(curtime);
+				}
+				else
+				{	
+					/* Perform clamping */
+					m_localtime=m_endframe;
+					result = false;
+					m_bNegativeEvent = false;
+				}
+			}
+		}
+		
+		CIpoAction ipoaction(
+			(KX_GameObject*) GetParent(),
+			m_localtime,
+			m_recurse,
+			m_ipo_as_force,
+			m_force_ipo_local);
+		GetParent()->Execute(ipoaction);
+		break;
+	}
+	
+	case KX_ACT_IPO_KEY2KEY:
+	{
+		// not implemented yet
+		result = false;
+		break;
+	}
+	
+	case KX_ACT_IPO_FROM_PROP:
+	{
+		result = !bNegativeEvent;
+
+		CValue* propval = GetParent()->GetProperty(m_propname);
+		if (propval)
+		{
+			m_localtime = propval->GetNumber(); 
+	
 			CIpoAction ipoaction(
 				(KX_GameObject*) GetParent(),
 				m_localtime,
@@ -254,99 +376,20 @@ bool KX_IpoActuator::Update(double curtime,double delta_time)
 				m_ipo_as_force,
 				m_force_ipo_local);
 			GetParent()->Execute(ipoaction);
-			break;
-		}
 
-		case KX_ACT_IPO_LOOPSTOP:
+		} else
 		{
-			if (numevents)
-			{
-				if (bNegativeEvent)
-				{
-					result = false;
-					m_bNegativeEvent = false;
-					numevents = 0;
-				}
-			} // fall through to loopend, and quit the ipo animation immediatly 
-		}
-	
-		case KX_ACT_IPO_LOOPEND:
-			{
-				if (numevents){
-					if (bNegativeEvent){
-						m_bNegativeEvent = true;
-					}
-				}
-				
-				if (bNegativeEvent && m_localtime == m_starttime){
-					result = false;
-				} 
-				else{
-					if (m_localtime*start_smaller_then_end < m_endtime*start_smaller_then_end){
-						m_localtime += m_direction * deltaframetime;
-					}
-					else{
-						if (!m_bNegativeEvent){
-							/* Perform wraparound */
-							float slop = m_localtime-m_endtime;
-							float length = fabs(m_starttime-m_endtime);
-							m_localtime = m_starttime + (slop - (int(slop/length)*(int(length))));
-							
-						}
-						else
-						{	
-							/* Perform clamping */
-							if ((m_localtime*start_smaller_then_end)>(m_endtime*start_smaller_then_end))
-								m_localtime=m_endtime;
-							
-							result = false;
-							m_bNegativeEvent = false;
-						}
-					}
-				}	
-				CIpoAction ipoaction(
-					(KX_GameObject*) GetParent(),
-					m_localtime,
-					m_recurse,
-					m_ipo_as_force,
-					m_force_ipo_local);
-				GetParent()->Execute(ipoaction);
-				break;
-		}
-	case KX_ACT_IPO_KEY2KEY:
-		{
-			// not implemented yet
 			result = false;
-			break;
-		}
-	case KX_ACT_IPO_FROM_PROP:
-		{
-			result = !bNegativeEvent;
-
-			 CValue* propval = GetParent()->GetProperty(m_propname);
-			 if (propval)
-			 {
-				m_localtime = propval->GetNumber();
-				CIpoAction ipoaction(
-					(KX_GameObject*) GetParent(),
-					m_localtime,
-					m_recurse,
-					m_ipo_as_force,
-					m_force_ipo_local);
-				GetParent()->Execute(ipoaction);
-
-			 } else
-			 {
-				 result = false;
-			 }
-			break;
 		}
+		break;
+	}
 		
 	default:
-		{
-			result = false;
-		}
+		result = false;
 	}
+	
+	if (!result && m_type != KX_ACT_IPO_LOOPSTOP)
+		m_starttime = -2.0*start_smaller_then_end*(m_endframe - m_startframe) - 1.0;
 
 	return result;
 }
@@ -428,10 +471,14 @@ PyMethodDef KX_IpoActuator::Methods[] = {
 		METH_VARARGS, SetType_doc},
 	{"getType", (PyCFunction) KX_IpoActuator::sPyGetType, 
 		METH_VARARGS, GetType_doc},	
+	{"setForceIpoActsLocal", (PyCFunction) KX_IpoActuator::sPySetForceIpoActsLocal,
+		METH_VARARGS, SetForceIpoActsLocal_doc},
+	{"getForceIpoActsLocal", (PyCFunction) KX_IpoActuator::sPyGetForceIpoActsLocal,
+		METH_VARARGS, GetForceIpoActsLocal_doc},
 	{NULL,NULL} //Sentinel
 };
 
-PyObject* KX_IpoActuator::_getattr(char* attr) {
+PyObject* KX_IpoActuator::_getattr(const STR_String& attr) {
 	_getattr_up(SCA_IActuator);
 }
 
@@ -468,8 +515,8 @@ PyObject* KX_IpoActuator::PySet(PyObject* self,
 	case KX_ACT_IPO_LOOPSTOP:
 	case KX_ACT_IPO_LOOPEND:
 		m_type         = modenum;
-		m_starttime    = startFrame;
-		m_endtime      = stopFrame;
+		m_startframe    = startFrame;
+		m_endframe      = stopFrame;
 		m_ipo_as_force = PyArgToBool(forceToggle);
 		break;
 	default:
@@ -494,6 +541,8 @@ PyObject* KX_IpoActuator::PySetProperty(PyObject* self,
 		return NULL;
 	}
 
+	m_propname = propertyName;
+	
 	Py_Return;
 }
 
@@ -510,7 +559,7 @@ PyObject* KX_IpoActuator::PySetStart(PyObject* self,
 		return NULL;		
 	}
 	
-	m_starttime = startArg;
+	m_startframe = startArg;
 
 	Py_Return;
 }
@@ -521,7 +570,7 @@ char KX_IpoActuator::GetStart_doc[] =
 PyObject* KX_IpoActuator::PyGetStart(PyObject* self, 
 									 PyObject* args, 
 									 PyObject* kwds) {
-	return PyFloat_FromDouble(m_starttime);
+	return PyFloat_FromDouble(m_startframe);
 }
 
 /* 6. setEnd:                                                                */
@@ -537,7 +586,7 @@ PyObject* KX_IpoActuator::PySetEnd(PyObject* self,
 		return NULL;		
 	}
 	
-	m_endtime = endArg;
+	m_endframe = endArg;
 
 	Py_Return;
 }
@@ -548,7 +597,7 @@ char KX_IpoActuator::GetEnd_doc[] =
 PyObject* KX_IpoActuator::PyGetEnd(PyObject* self, 
 								   PyObject* args, 
 								   PyObject* kwds) {
-	return PyFloat_FromDouble(m_endtime);
+	return PyFloat_FromDouble(m_endframe);
 }
 
 /* 6. setIpoAsForce:                                                           */
