@@ -1,5 +1,5 @@
 /*
- * $Id: KX_KetsjiEngine.cpp,v 1.5 2004/04/08 12:01:48 kester Exp $
+ * $Id: KX_KetsjiEngine.cpp,v 1.16 2005/03/25 10:33:37 kester Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -65,6 +65,7 @@
 #include "KX_PythonInit.h"
 #include "KX_PyConstraintBinding.h"
 #include "PHY_IPhysicsEnvironment.h"
+#include "SumoPhysicsEnvironment.h"
 
 #include "SND_Scene.h"
 #include "SND_IAudioDevice.h"
@@ -81,7 +82,8 @@
 // If define: little test for Nzc: guarded drawing. If the canvas is
 // not valid, skip rendering this frame.
 //#define NZC_GUARDED_OUTPUT
-
+#define DEFAULT_LOGIC_TIC_RATE 30.0
+#define DEFAULT_PHYSICS_TIC_RATE 90.0
 
 const char KX_KetsjiEngine::m_profileLabels[tc_numCategories][15] = {
 	"Physics:",		// tc_physics
@@ -95,47 +97,70 @@ const char KX_KetsjiEngine::m_profileLabels[tc_numCategories][15] = {
 	"Outside:"		// tc_outside
 };
 
-
+double KX_KetsjiEngine::m_ticrate = DEFAULT_LOGIC_TIC_RATE;
 
 
 /**
  *	Constructor of the Ketsji Engine
  */
 KX_KetsjiEngine::KX_KetsjiEngine(KX_ISystem* system)
-:
+     :	m_canvas(NULL),
 	m_rasterizer(NULL),
-	m_bInitialized(false),
-	m_activecam(0)
-{
-	m_kxsystem = system;
-	m_bFixedTime = false;
+	m_kxsystem(system),
+	m_rendertools(NULL),
+	m_sceneconverter(NULL),
+	m_networkdevice(NULL),
+	m_audiodevice(NULL),
+	m_pythondictionary(NULL),
+	m_keyboarddevice(NULL),
+	m_mousedevice(NULL),
 
+	m_propertiesPresent(false),
+
+	m_bInitialized(false),
+	m_activecam(0),
+	m_bFixedTime(false),
+	
+	m_firstframe(true),
+	
+	m_previoustime(0.0),
+	m_deltatime(0.0),
+
+	m_exitcode(KX_EXIT_REQUEST_NO_REQUEST),
+	m_exitstring(""),
+	
+	m_drawingmode(5),
+	m_cameraZoom(1.0),
+	
+	m_overrideCam(false),
+	m_overrideCamUseOrtho(false),
+
+	m_stereo(false),
+	m_curreye(0),
+
+	m_logger(NULL),
+	
+	// Set up timing info display variables
+	m_show_framerate(false),
+	m_show_profile(false),
+	m_showProperties(false),
+	m_showBackground(false),
+	m_show_debug_properties(false),
+
+	// Default behavior is to hide the cursor every frame.
+	m_hideCursor(false),
+
+	m_overrideFrameColor(false),
+	m_overrideFrameColorR(0.0),
+	m_overrideFrameColorG(0.0),
+	m_overrideFrameColorB(0.0)
+{
 	// Initialize the time logger
 	m_logger = new KX_TimeCategoryLogger (25);
 
 	for (int i = tc_first; i < tc_numCategories; i++)
 		m_logger->AddCategory((KX_TimeCategory)i);
-
-	// Set up timing info display variables
-	m_show_framerate = false;
-	m_show_profile   = false;
-	m_show_debug_properties = false;
-	m_propertiesPresent = false;
-
-	// Default behavior is to hide the cursor every frame.
-	m_hideCursor = false;
-
-	m_overrideFrameColor = false;
-	m_overrideFrameColorR = (float)0;
-	m_overrideFrameColorG = (float)0;
-	m_overrideFrameColorB = (float)0;
-	
-	m_cameraZoom = 1.0;
-	m_drawingmode = 5; /* textured drawing mode */
-	m_overrideCam = false;
-
-	m_exitcode = KX_EXIT_REQUEST_NO_REQUEST;
-	m_exitstring = "";
+		
 }
 
 
@@ -145,15 +170,14 @@ KX_KetsjiEngine::KX_KetsjiEngine(KX_ISystem* system)
  */
 KX_KetsjiEngine::~KX_KetsjiEngine()
 {
-	if (m_logger)
-		delete m_logger;
+	delete m_logger;
 }
 
 
 
 void KX_KetsjiEngine::SetKeyboardDevice(SCA_IInputDevice* keyboarddevice)
 {
-	assert(keyboarddevice);
+	MT_assert(keyboarddevice);
 	m_keyboarddevice = keyboarddevice;
 }
 
@@ -161,7 +185,7 @@ void KX_KetsjiEngine::SetKeyboardDevice(SCA_IInputDevice* keyboarddevice)
 
 void KX_KetsjiEngine::SetMouseDevice(SCA_IInputDevice* mousedevice)
 {
-	assert(mousedevice);
+	MT_assert(mousedevice);
 	m_mousedevice = mousedevice;
 }
 
@@ -169,7 +193,7 @@ void KX_KetsjiEngine::SetMouseDevice(SCA_IInputDevice* mousedevice)
 
 void KX_KetsjiEngine::SetNetworkDevice(NG_NetworkDeviceInterface* networkdevice)
 {
-	assert(networkdevice);
+	MT_assert(networkdevice);
 	m_networkdevice = networkdevice;
 }
 
@@ -177,7 +201,7 @@ void KX_KetsjiEngine::SetNetworkDevice(NG_NetworkDeviceInterface* networkdevice)
 
 void KX_KetsjiEngine::SetAudioDevice(SND_IAudioDevice* audiodevice)
 {
-	assert(audiodevice);
+	MT_assert(audiodevice);
 	m_audiodevice = audiodevice;
 }
 
@@ -185,7 +209,7 @@ void KX_KetsjiEngine::SetAudioDevice(SND_IAudioDevice* audiodevice)
 
 void KX_KetsjiEngine::SetCanvas(RAS_ICanvas* canvas)
 {
-	assert(canvas);
+	MT_assert(canvas);
 	m_canvas = canvas;
 }
 
@@ -193,7 +217,7 @@ void KX_KetsjiEngine::SetCanvas(RAS_ICanvas* canvas)
 
 void KX_KetsjiEngine::SetRenderTools(RAS_IRenderTools* rendertools)
 {
-	assert(rendertools);
+	MT_assert(rendertools);
 	m_rendertools = rendertools;
 }
 
@@ -201,7 +225,7 @@ void KX_KetsjiEngine::SetRenderTools(RAS_IRenderTools* rendertools)
 
 void KX_KetsjiEngine::SetRasterizer(RAS_IRasterizer* rasterizer)
 {
-	assert(rasterizer);
+	MT_assert(rasterizer);
 	m_rasterizer = rasterizer;
 }
 
@@ -209,7 +233,7 @@ void KX_KetsjiEngine::SetRasterizer(RAS_IRasterizer* rasterizer)
 
 void KX_KetsjiEngine::SetPythonDictionary(PyObject* pythondictionary)
 {
-	assert(pythondictionary);
+	MT_assert(pythondictionary);
 	m_pythondictionary = pythondictionary;
 }
 
@@ -217,7 +241,7 @@ void KX_KetsjiEngine::SetPythonDictionary(PyObject* pythondictionary)
 
 void KX_KetsjiEngine::SetSceneConverter(KX_ISceneConverter* sceneconverter)
 {
-	assert(sceneconverter);
+	MT_assert(sceneconverter);
 	m_sceneconverter = sceneconverter;
 }
 
@@ -230,61 +254,12 @@ void KX_KetsjiEngine::SetSceneConverter(KX_ISceneConverter* sceneconverter)
  */
 void KX_KetsjiEngine::StartEngine()
 {
-	m_previoustime = 0.0;
-	m_missedtime = 0.0;
+	m_previoustime = m_kxsystem->GetTimeInSeconds();
 	m_firstframe = true;
-	
-	// for all scenes, initialize the scenegraph for the first time
-	m_lasttime = m_kxsystem->GetTimeInSeconds()*100.0;
-
 	m_bInitialized = true;
+	m_ticrate = DEFAULT_LOGIC_TIC_RATE;
 }
 
-
-
-#define DELTALENGTH 25 
-
-double	KX_KetsjiEngine::CalculateAverage(double newdelta)
-{
-	if (m_deltatimes.size() < DELTALENGTH)
-	{
-		m_deltatimes.push_back(newdelta);
-	} else
-	{
-		//
-		double totaltime = 0.0;
-		double newlasttime,lasttime = newdelta;
-		double peakmin = 10000;
-		double peakmax = -10000;
-
-		for (int i=m_deltatimes.size()-1;i>=0;i--)
-		{	newlasttime = m_deltatimes[i];
-			totaltime += newlasttime;
-			if (peakmin > newlasttime)
-				peakmin = newlasttime;
-			if (peakmax < newlasttime)
-				peakmax = newlasttime;
-
-			m_deltatimes[i] = lasttime;
-			lasttime = newlasttime;
-		};
-		double averagetime;
-		
-		if (peakmin < peakmax)
-		{
-		 	averagetime = ((totaltime - peakmin) - peakmax) / (double) (m_deltatimes.size()-2); 
-		} else
-		{	
-			averagetime = totaltime / (double) m_deltatimes.size();
-		}
-		return averagetime;
-	}	
-
-	return newdelta;
-}
-
-
-
 bool KX_KetsjiEngine::BeginFrame()
 {
 	bool result = false;
@@ -338,106 +313,144 @@ void KX_KetsjiEngine::NextFrame()
 {
 	m_logger->StartLog(tc_services, m_kxsystem->GetTimeInSeconds(), true);
 
-	double deltatime = 0.02; 
 	double curtime;
-
 	if (m_bFixedTime)
-	{
-		curtime = m_previoustime + deltatime;
-	}
+		curtime = m_previoustime + 1.0/m_ticrate;
 	else
-	{
 		curtime = m_kxsystem->GetTimeInSeconds();
-		if (m_previoustime)
-			deltatime = curtime - m_previoustime;
-
-		if (deltatime > 0.1)
-			deltatime = 0.1;
-
-		deltatime = CalculateAverage(deltatime);
-	}
+	m_deltatime += curtime - m_previoustime;
+	float	realDeltaTime = curtime - m_previoustime;
 
 	m_previoustime = curtime;
+	double localtime = curtime - m_deltatime;
 
+	// Compute the number of logic frames to do each update (fixed tic bricks)
+	int frames = (int) (m_deltatime*m_ticrate);
+	m_deltatime -= double(frames)/m_ticrate;
+	
 	KX_SceneList::iterator sceneit;
-	for (sceneit = m_scenes.begin();sceneit != m_scenes.end(); sceneit++)
+	
+	while (frames)
+	{
+		localtime += 1.0/m_ticrate;
+		for (sceneit = m_scenes.begin();sceneit != m_scenes.end(); ++sceneit)
+		// for each scene, call the proceed functions
+		{
+			KX_Scene* scene = *sceneit;
+	
+			/* Suspension holds the physics and logic processing for an
+			* entire scene. Objects can be suspended individually, and
+			* the settings for that preceed the logic and physics
+			* update. */
+			m_logger->StartLog(tc_logic, m_kxsystem->GetTimeInSeconds(), true);
+			scene->UpdateObjectActivity();
+	
+			if (!scene->IsSuspended())
+			{
+				m_logger->StartLog(tc_physics, m_kxsystem->GetTimeInSeconds(), true);
+				m_logger->StartLog(tc_network, m_kxsystem->GetTimeInSeconds(), true);
+				scene->GetNetworkScene()->proceed(localtime);
+	
+				// set Python hooks for each scene
+				PHY_SetActiveEnvironment(scene->GetPhysicsEnvironment());
+				PHY_SetActiveScene(scene);
+	
+				scene->GetPhysicsEnvironment()->endFrame();
+				
+				// Update scenegraph after physics step. This maps physics calculations
+				// into node positions.		
+				m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
+				scene->UpdateParents(localtime);
+				
+				// Process sensors, and controllers
+				m_logger->StartLog(tc_logic, m_kxsystem->GetTimeInSeconds(), true);
+				scene->LogicBeginFrame(localtime);
+	
+				// Scenegraph needs to be updated again, because Logic Controllers 
+				// can affect the local matrices.
+				m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
+				scene->UpdateParents(localtime);
+	
+				// Process actuators
+	
+				// Do some cleanup work for this logic frame
+				m_logger->StartLog(tc_logic, m_kxsystem->GetTimeInSeconds(), true);
+				scene->LogicUpdateFrame(localtime, true);
+				scene->LogicEndFrame();
+	
+				// Actuators can affect the scenegraph
+				m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
+				scene->UpdateParents(localtime);
+				
+				scene->GetPhysicsEnvironment()->beginFrame();
+		
+				// Perform physics calculations on the scene. This can involve 
+				// many iterations of the physics solver.
+				m_logger->StartLog(tc_physics, m_kxsystem->GetTimeInSeconds(), true);
+				scene->GetPhysicsEnvironment()->proceedDeltaTime(localtime,realDeltaTime);
+			} // suspended
+	
+			DoSound(scene);
+	
+			m_logger->StartLog(tc_services, m_kxsystem->GetTimeInSeconds(), true);
+		}
+
+		// update system devices
+		m_logger->StartLog(tc_logic, m_kxsystem->GetTimeInSeconds(), true);
+		if (m_keyboarddevice)
+			m_keyboarddevice->NextFrame();
+	
+		if (m_mousedevice)
+			m_mousedevice->NextFrame();
+		
+		if (m_networkdevice)
+			m_networkdevice->NextFrame();
+	
+		if (m_audiodevice)
+			m_audiodevice->NextFrame();
+	
+		// scene management
+		ProcessScheduledScenes();
+		
+		frames--;
+	}
+
+	// Logic update sub frame: this will let some logic bricks run at the
+	// full frame rate.
+	for (sceneit = m_scenes.begin();sceneit != m_scenes.end(); ++sceneit)
 	// for each scene, call the proceed functions
 	{
 		KX_Scene* scene = *sceneit;
 
-
-
-		/* Suspension holds the physics and logic processing for an
-		 * entire scene. Objects can be suspended individually, and
-		 * the settings for that preceed the logic and physics
-		 * update. */
-		m_logger->StartLog(tc_logic, m_kxsystem->GetTimeInSeconds(), true);
-		scene->UpdateObjectActivity();
-
 		if (!scene->IsSuspended())
 		{
-			m_logger->StartLog(tc_network, m_kxsystem->GetTimeInSeconds(), true);
-			scene->GetNetworkScene()->proceed(curtime, deltatime);
-
 			// set Python hooks for each scene
 			PHY_SetActiveEnvironment(scene->GetPhysicsEnvironment());
 			PHY_SetActiveScene(scene);
-
-			// Process sensors, and controllers
-			m_logger->StartLog(tc_logic, m_kxsystem->GetTimeInSeconds(), true);
-			scene->LogicBeginFrame(curtime,deltatime);
-
-			// Scenegraph needs to be updated again, because Logic Controllers 
-			// can affect the local matrices.
-			m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
-			scene->UpdateParents(curtime);
-
-			// Process actuators
-
-			// Do some cleanup work for this logic frame
-			m_logger->StartLog(tc_logic, m_kxsystem->GetTimeInSeconds(), true);
-			scene->LogicUpdateFrame(curtime,deltatime);		
-			scene->LogicEndFrame();
-
-			// Actuators can affect the scenegraph
-			m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
-			scene->UpdateParents(curtime);
-
+			
 			// Perform physics calculations on the scene. This can involve 
 			// many iterations of the physics solver.
 			m_logger->StartLog(tc_physics, m_kxsystem->GetTimeInSeconds(), true);
-			scene->GetPhysicsEnvironment()->proceed(deltatime);
-
+			scene->GetPhysicsEnvironment()->proceedDeltaTime(curtime,0.f);
 			// Update scenegraph after physics step. This maps physics calculations
 			// into node positions.		
 			m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
 			scene->UpdateParents(curtime);
+			
+			// Do some cleanup work for this logic frame
+			m_logger->StartLog(tc_logic, m_kxsystem->GetTimeInSeconds(), true);
+			scene->LogicUpdateFrame(curtime, false);
 
+			// Actuators can affect the scenegraph
+			m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
+			scene->UpdateParents(curtime);
 		} // suspended
 
 		DoSound(scene);
 
 		m_logger->StartLog(tc_services, m_kxsystem->GetTimeInSeconds(), true);
 	}
-
-	// update system devices
-	m_logger->StartLog(tc_logic, m_kxsystem->GetTimeInSeconds(), true);
-
-	if (m_keyboarddevice)
-		m_keyboarddevice->NextFrame();
-
-	if (m_mousedevice)
-		m_mousedevice->NextFrame();
-
-	if (m_networkdevice)
-		m_networkdevice->NextFrame();
-
-	if (m_audiodevice)
-		m_audiodevice->NextFrame();
-
-	// scene management
-	ProcessScheduledScenes();
-
+	
 	// Start logging time spend outside main loop
 	m_logger->StartLog(tc_outside, m_kxsystem->GetTimeInSeconds(), true);
 }
@@ -736,6 +749,7 @@ void KX_KetsjiEngine::SetupRenderFrame(KX_Scene *scene)
 void KX_KetsjiEngine::RenderFrame(KX_Scene* scene)
 {
 	float left, right, bottom, top, nearfrust, farfrust;
+	const float ortho = 100.0;
 	KX_Camera* cam = scene->GetActiveCamera();
 	
 	if (!cam)
@@ -746,16 +760,30 @@ void KX_KetsjiEngine::RenderFrame(KX_Scene* scene)
 	if (m_overrideCam && (scene->GetName() == m_overrideSceneName) && m_overrideCamUseOrtho) {
 		MT_CmMatrix4x4 projmat = m_overrideCamProjMat;
 		m_rasterizer->SetProjectionMatrix(projmat);
-	} else {
+	} else if (cam->hasValidProjectionMatrix())
+	{
+		m_rasterizer->SetProjectionMatrix(cam->GetProjectionMatrix());
+	} else
+	{
 		RAS_FrameFrustum frustum;
+		float lens = cam->GetLens();
+		nearfrust = cam->GetCameraNear();
+		farfrust = cam->GetCameraFar();
 
+		if (!cam->GetCameraData()->m_perspective)
+		{
+			lens *= ortho;
+			nearfrust = (nearfrust + 1.0)*ortho;
+			farfrust *= ortho;
+		}
+		
 		RAS_FramingManager::ComputeFrustum(
 			scene->GetFramingType(),
 			m_canvas->GetDisplayArea(),
 			scene->GetSceneViewport(),
-			cam->GetLens(),
-			cam->GetCameraNear(),
-			cam->GetCameraFar(),
+			lens,
+			nearfrust,
+			farfrust,
 			frustum
 		);
 
@@ -769,17 +797,18 @@ void KX_KetsjiEngine::RenderFrame(KX_Scene* scene)
 		MT_Matrix4x4 projmat = m_rasterizer->GetFrustumMatrix(
 			left, right, bottom, top, nearfrust, farfrust);
 	
-		m_rasterizer->SetProjectionMatrix(projmat);
-		cam->SetProjectionMatrix(projmat);	
+		cam->SetProjectionMatrix(projmat);
+		
+		// Otherwise the projection matrix for each eye will be the same...
+		if (m_rasterizer->Stereo())
+			cam->InvalidateProjectionMatrix();
 	}
 
-	MT_Scalar cammat[16];
-	cam->GetWorldToCamera().getValue(cammat);
-	MT_Matrix4x4 viewmat;
-	viewmat.setValue(cammat); // this _should transpose ... 
-	                          // if finally transposed take care of correct usage
-	                          // in RAS_OpenGLRasterizer ! (row major vs column major)
-
+	MT_Transform camtrans(cam->GetWorldToCamera());
+	if (!cam->GetCameraData()->m_perspective)
+		camtrans.getOrigin()[2] *= ortho;
+	MT_Matrix4x4 viewmat(camtrans);
+	
 	m_rasterizer->SetViewMatrix(viewmat, cam->NodeGetWorldPosition(),
 		cam->GetCameraLocation(), cam->GetCameraOrientation());
 	cam->SetModelviewMatrix(viewmat);
@@ -792,7 +821,7 @@ void KX_KetsjiEngine::RenderFrame(KX_Scene* scene)
 	// runs through the individual objects.
 	scene->CalculateVisibleMeshes(m_rasterizer);
 
-	scene->RenderBuckets(cam->GetWorldToCamera(), m_rasterizer, m_rendertools);
+	scene->RenderBuckets(camtrans, m_rasterizer, m_rendertools);
 }
 
 
@@ -836,7 +865,7 @@ void KX_KetsjiEngine::PostProcessScene(KX_Scene* scene)
 		KX_Camera* activecam = NULL;
 
 		RAS_CameraData camdata = RAS_CameraData();
-		activecam = new KX_Camera(scene,KX_Scene::m_callbacks,camdata);
+		activecam = new KX_Camera(scene,KX_Scene::m_callbacks,camdata, false);
 		activecam->SetName("__default__cam__");
 	
 			// set transformation
@@ -1163,7 +1192,15 @@ bool KX_KetsjiEngine::GetUseFixedTime(void) const
 	return m_bFixedTime;
 }
 
+double KX_KetsjiEngine::GetTicRate()
+{
+	return m_ticrate;
+}
 
+void KX_KetsjiEngine::SetTicRate(double ticrate)
+{
+	m_ticrate = ticrate;
+}
 
 void KX_KetsjiEngine::SetTimingDisplay(bool frameRate, bool profile, bool properties)
 {
