@@ -1,5 +1,5 @@
 /**
- * $Id: KX_MouseFocusSensor.cpp,v 1.6 2004/04/26 07:19:18 kester Exp $
+ * $Id: KX_MouseFocusSensor.cpp,v 1.13 2005/03/25 16:31:05 sirdude Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,10 +31,6 @@
  * KX_MouseFocusSensor determines mouse in/out/over events.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef WIN32
 // This warning tells us about truncation of __long__ stl-generated names.
 // It can occasionally cause DevStudio to have internal compiler warnings.
@@ -42,7 +38,6 @@
 #endif
 
 #include "MT_Point3.h"
-#include "KX_ClientObjectInfo.h"
 #include "RAS_FramingManager.h"
 #include "RAS_ICanvas.h"
 #include "RAS_IRasterizer.h"
@@ -51,11 +46,13 @@
 #include "KX_Camera.h"
 #include "KX_MouseFocusSensor.h"
 
+#include "KX_RayCast.h"
+#include "KX_IPhysicsController.h"
+#include "PHY_IPhysicsController.h"
+#include "PHY_IPhysicsEnvironment.h"
+
+
 #include "KX_ClientObjectInfo.h"
-#include "SM_Object.h"
-#include "SM_Scene.h"
-#include "SumoPhysicsEnvironment.h"
-#include "KX_SumoPhysicsController.h"
 
 /* ------------------------------------------------------------------------- */
 /* Native functions                                                          */
@@ -118,10 +115,39 @@ bool KX_MouseFocusSensor::Evaluate(CValue* event)
 	return result;
 }
 
+bool KX_MouseFocusSensor::RayHit(KX_ClientObjectInfo* client_info, MT_Point3& hit_point, MT_Vector3& hit_normal, void * const data)
+{
+	KX_GameObject* hitKXObj = client_info->m_gameobject;
+	
+	if (client_info->m_type > KX_ClientObjectInfo::ACTOR)
+	{
+		// false hit
+		return false;
+	}
+	
+	/* Is this me? In the ray test, there are a lot of extra checks
+	* for aliasing artefacts from self-hits. That doesn't happen
+	* here, so a simple test suffices. Or does the camera also get
+	* self-hits? (No, and the raysensor shouldn't do it either, since
+	* self-hits are excluded by setting the correct ignore-object.)
+	* Hitspots now become valid. */
+	KX_GameObject* thisObj = (KX_GameObject*) GetParent();
+	if (hitKXObj == thisObj)
+	{
+		m_hitPosition = hit_point;
+		m_hitNormal = hit_normal;
+		return true;
+	}
+	
+	return true;     // object must be visible to trigger
+	//return false;  // occluded objects can trigger
+}
+
+
+
 bool KX_MouseFocusSensor::ParentObjectHasFocus(void)
 {
 	
-  	bool res = false;
 	m_hitPosition = MT_Vector3(0,0,0);
 	m_hitNormal =	MT_Vector3(1,0,0);
 	MT_Point3 resultpoint;
@@ -208,7 +234,13 @@ bool KX_MouseFocusSensor::ParentObjectHasFocus(void)
 	);
 
 	/* camera to world  */
-	MT_Matrix4x4 camcs_wcs_matrix = MT_Matrix4x4(cam->GetCameraToWorld());
+	MT_Transform wcs_camcs_tranform = cam->GetWorldToCamera();
+	if (!cam->GetCameraData()->m_perspective)
+		wcs_camcs_tranform.getOrigin()[2] *= 100.0;
+	MT_Transform cams_wcs_transform;
+	cams_wcs_transform.invert(wcs_camcs_tranform);
+	
+	MT_Matrix4x4 camcs_wcs_matrix = MT_Matrix4x4(cams_wcs_transform);
 
 	/* badly defined, the first time round.... I wonder why... I might
 	 * want to guard against floating point errors here.*/
@@ -229,89 +261,101 @@ bool KX_MouseFocusSensor::ParentObjectHasFocus(void)
 								   topoint[1]/topoint[3], 
 								   topoint[2]/topoint[3]); 
 	m_prevTargetPoint = topoint3;
+	m_prevSourcePoint = frompoint3;
 	
-	/* 2. Get the object from SuMO*/
+	/* 2. Get the object from PhysicsEnvironment */
 	/* Shoot! Beware that the first argument here is an
 	 * ignore-object. We don't ignore anything... */
-	KX_GameObject* thisObj = (KX_GameObject*) GetParent();
 	
-	SumoPhysicsEnvironment *spe = dynamic_cast<SumoPhysicsEnvironment* > (m_kxscene->GetPhysicsEnvironment());
-	SM_Scene *sumoScene = spe->GetSumoScene();
-	KX_SumoPhysicsController *spc = dynamic_cast<KX_SumoPhysicsController*> (cam->GetPhysicsController());
-	SM_Object *sumoCam = spc?spc->GetSumoObject():NULL;
-	MT_Vector3 todir = topoint3 - frompoint3;
-	if (todir.dot(todir) < MT_EPSILON)
-		return false;
-	todir.normalize();
+	KX_IPhysicsController* physics_controller = (cam->GetPhysicsController());
+	PHY_IPhysicsEnvironment* physics_environment = m_kxscene->GetPhysicsEnvironment();
 
-	while (true)
-	{	
-		SM_Object* hitSMObj = sumoScene->rayTest(sumoCam, 
-							frompoint3,
-							topoint3,
-							resultpoint, 
-							resultnormal);
-		
-		if (!hitSMObj)
-			return false;
-			
-		/* all this casting makes me nervous... */
-		KX_ClientObjectInfo* client_info 
-			= ( hitSMObj ?
-				(KX_ClientObjectInfo*) hitSMObj->getClientObject() :
-				NULL);
-		
-		if (!client_info)
-		{
-			std::cout<< "WARNING:  MouseOver sensor " << GetName() << " cannot sense SM_Object " << hitSMObj << " - no client info.\n" << std::endl;
-			return false;
-		} 
+// 	MT_Vector3 todir = topoint3 - frompoint3;
+// 	if (todir.dot(todir) < MT_EPSILON)
+// 		return false;
+// 	todir.normalize();
 	
-		KX_GameObject* hitKXObj = (KX_GameObject*)client_info->m_clientobject;
-		
-		if (client_info->m_type > KX_ClientObjectInfo::ACTOR)
-		{
-			// false hit
-			KX_SumoPhysicsController *hitspc = dynamic_cast<KX_SumoPhysicsController *> (static_cast<KX_GameObject*> (hitKXObj) ->GetPhysicsController());
-			if (hitspc)
-			{
-				/* We add 0.01 of fudge, so that if the margin && radius == 0., we don't endless loop. */
-				MT_Scalar marg = 0.01 + hitspc->GetSumoObject()->getMargin();
-				if (hitspc->GetSumoObject()->getShapeProps())
-				{
-					marg += 2*hitspc->GetSumoObject()->getShapeProps()->m_radius;
-				}
-				
-				/* Calculate the other side of this object */
-				MT_Point3 hitObjPos;
-				hitspc->GetWorldPosition(hitObjPos);
-				MT_Vector3 hitvector = hitObjPos - resultpoint;
-				if (hitvector.dot(hitvector) > MT_EPSILON)
-				{
-					hitvector.normalize();
-					marg *= 2.*todir.dot(hitvector);
-				}
-				frompoint3 = resultpoint + marg * todir;
-			} else {
-				return false;
-			}
-			continue;
-		}
-		/* Is this me? In the ray test, there are a lot of extra checks
-		* for aliasing artefacts from self-hits. That doesn't happen
-		* here, so a simple test suffices. Or does the camera also get
-		* self-hits? (No, and the raysensor shouldn't do it either, since
-		* self-hits are excluded by setting the correct ignore-object.)
-		* Hitspots now become valid. */
-		if (hitKXObj == thisObj)
-		{
-			m_hitPosition = resultpoint;
-			m_hitNormal = resultnormal;
-			return true;
-		}
-		
-		return false;
-	}
+	KX_RayCast::RayTest(physics_controller, physics_environment, frompoint3, topoint3, resultpoint, resultnormal, KX_RayCast::Callback<KX_MouseFocusSensor>(this));
+	
+
+// 	while (true)
+// 	{	
+// 		PHY__Vector3 resultpt;
+// 		PHY__Vector3 resultnr;
+// 
+// 		PHY_IPhysicsController* hitCtrl= physEnv->rayTest(camCtrl, 
+// 							frompoint3.x(),frompoint3.y(),frompoint3.z(),
+// 							topoint3.x(),topoint3.y(),topoint3.z(),
+// 							resultpt[0], resultpt[1],resultpt[2],
+// 							resultnr[0],resultnr[1],resultnr[2]);
+// 		
+// 		if (!hitCtrl)
+// 			return false;
+// 			
+// 		resultpoint = MT_Vector3(resultpt);
+// 		resultnormal = MT_Vector3(resultnr);
+// 
+// 		/* all this casting makes me nervous... */
+// 		KX_ClientObjectInfo* client_info 
+// 			= 	static_cast<KX_ClientObjectInfo*>( hitCtrl->getNewClientInfo());
+// 		
+// 		if (!client_info)
+// 		{
+// 			std::cout<< "WARNING:  MouseOver sensor " << GetName() << " cannot sense - no client info.\n" << std::endl;
+// 
+// 			return false;
+// 		} 
+// 	
+// 		KX_GameObject* hitKXObj = client_info->m_gameobject;
+// 		
+// 		if (client_info->m_type > KX_ClientObjectInfo::ACTOR)
+// 		{
+// 			// false hit
+// 			// FIXME: add raytest interface to KX_IPhysicsController, remove casting
+// 			PHY_IPhysicsController* hitspc = (PHY_IPhysicsController*) (static_cast<KX_GameObject*> (hitKXObj)->GetPhysicsController());
+// 			if (hitspc)
+// 			{
+// 				/* We add 0.01 of fudge, so that if the margin && radius == 0., we don't endless loop. */
+// 				MT_Scalar marg = 0.01 + hitspc->GetMargin();
+// 				marg += hitspc->GetRadius(); //this is changed, check !
+// 
+// 				//if (hitspc->GetSumoObject()->getShapeProps())
+// 				//{
+// 				//	marg += 2*hitspc->GetSumoObject()->getShapeProps()->m_radius;
+// 				//}
+// 				
+// 				/* Calculate the other side of this object */
+// 				MT_Point3 hitObjPos;
+// 				PHY__Vector3 hitpos;
+// 				hitspc->getPosition(hitpos);
+// 				hitObjPos = MT_Vector3(hitpos);
+// 				MT_Vector3 hitvector = hitObjPos - resultpoint;
+// 				if (hitvector.dot(hitvector) > MT_EPSILON)
+// 				{
+// 					hitvector.normalize();
+// 					marg *= 2.*todir.dot(hitvector);
+// 				}
+// 				frompoint3 = resultpoint + marg * todir;
+// 			} else {
+// 				return false;
+// 			}
+// 			continue;
+// 		}
+// 		/* Is this me? In the ray test, there are a lot of extra checks
+// 		* for aliasing artefacts from self-hits. That doesn't happen
+// 		* here, so a simple test suffices. Or does the camera also get
+// 		* self-hits? (No, and the raysensor shouldn't do it either, since
+// 		* self-hits are excluded by setting the correct ignore-object.)
+// 		* Hitspots now become valid. */
+// 		if (hitKXObj == thisObj)
+// 		{
+// 			m_hitPosition = resultpoint;
+// 			m_hitNormal = resultnormal;
+// 			return true;
+// 		}
+// 		
+// 		return false;
+// 	}
 
 	return false;
 }
@@ -357,7 +401,7 @@ PyMethodDef KX_MouseFocusSensor::Methods[] = {
 	{NULL,NULL} //Sentinel
 };
 
-PyObject* KX_MouseFocusSensor::_getattr(char* attr) {
+PyObject* KX_MouseFocusSensor::_getattr(const STR_String& attr) {
 	_getattr_up(SCA_MouseSensor);
 }
 
