@@ -1,5 +1,5 @@
 /**
- * $Id: KX_RadarSensor.h,v 1.6 2004/03/22 22:01:52 jesterking Exp $
+ * $Id: KX_RadarSensor.h,v 1.9 2005/03/25 10:33:37 kester Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -65,6 +65,7 @@ public:
 
 	KX_RadarSensor(SCA_EventManager* eventmgr,
 		KX_GameObject* gameobj,
+			PHY_IPhysicsController* physCtrl,
 			double coneradius,
 			double coneheight,
 			int	axis,
@@ -77,12 +78,13 @@ public:
 	KX_RadarSensor();
 	virtual ~KX_RadarSensor();
 	virtual void SynchronizeTransform();
+	virtual CValue* GetReplica();
 
 	/* --------------------------------------------------------------------- */
 	/* Python interface ---------------------------------------------------- */
 	/* --------------------------------------------------------------------- */
 	
-	virtual PyObject*  _getattr(char *attr);
+	virtual PyObject* _getattr(const STR_String& attr);
 
 	KX_PYMETHOD_DOC(KX_RadarSensor,GetConeOrigin);
 	KX_PYMETHOD_DOC(KX_RadarSensor,GetConeTarget);
