@@ -1,5 +1,5 @@
 /*
- * $Id: KX_Scene.cpp,v 1.3 2002/11/25 15:29:49 mein Exp $
+ * $Id: KX_Scene.cpp,v 1.19 2005/06/04 16:22:50 lukep Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,17 +31,15 @@
  * Ketsji scene. Holds references to all scene data.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef WIN32
 #pragma warning (disable : 4786)
 #endif //WIN32
 
+#include "KX_Scene.h"
+#include "MT_assert.h"
+
 #include "KX_KetsjiEngine.h"
 #include "RAS_IPolygonMaterial.h"
-#include "KX_Scene.h"
 #include "ListValue.h"
 #include "SCA_LogicManager.h"
 #include "SCA_TimeEventManager.h"
@@ -53,6 +51,7 @@
 #include "SCA_MouseManager.h"
 #include "SCA_PropertyEventManager.h"
 #include "KX_Camera.h"
+#include "SCA_JoystickManager.h"
 
 #include "RAS_MeshObject.h"
 #include "RAS_IRasterizer.h"
@@ -65,6 +64,7 @@
 #include "SYS_System.h"
 #include "SG_Controller.h"
 #include "SG_IObject.h"
+#include "SG_Tree.h"
 
 #include "KX_SG_NodeRelationships.h"
 
@@ -73,6 +73,9 @@
 #include "PHY_IPhysicsEnvironment.h"
 #include "KX_IPhysicsController.h"
 
+#include "BL_SkinDeformer.h"
+#include "BL_DeformableGameObject.h"
+
 
 void* KX_SceneReplicationFunc(SG_IObject* node,void* gameobj,void* scene)
 {
@@ -88,30 +91,26 @@ void* KX_SceneDestructionFunc(SG_IObject* node,void* gameobj,void* scene)
 	return NULL;
 };
 
-
-SG_Callbacks KX_Scene::m_callbacks = SG_Callbacks(KX_SceneReplicationFunc,KX_SceneDestructionFunc);
+SG_Callbacks KX_Scene::m_callbacks = SG_Callbacks(KX_SceneReplicationFunc,KX_SceneDestructionFunc,KX_GameObject::UpdateTransformFunc);
 
 // temporarily var until there is a button in the userinterface
 // (defined in KX_PythonInit.cpp)
 extern bool gUseVisibilityTemp;
 
-
-
-
 KX_Scene::KX_Scene(class SCA_IInputDevice* keyboarddevice,
 				   class SCA_IInputDevice* mousedevice,
 				   class NG_NetworkDeviceInterface *ndi,
 				   class SND_IAudioDevice* adi,
 				   const STR_String& sceneName): 
-
-	m_mousemgr(NULL),
+	PyObjectPlus(&KX_Scene::Type),
 	m_keyboardmgr(NULL),
-	m_active_camera(NULL),
-	m_ueberExecutionPriority(0),
-	m_adi(adi),
+	m_mousemgr(NULL),
+	m_physicsEnvironment(0),
 	m_sceneName(sceneName),
+	m_adi(adi),
 	m_networkDeviceInterface(ndi),
-	m_physicsEnvironment(0)
+	m_active_camera(NULL),
+	m_ueberExecutionPriority(0)
 {
 		
 
@@ -130,31 +129,27 @@ KX_Scene::KX_Scene(class SCA_IInputDevice* keyboarddevice,
 	m_keyboardmgr = new SCA_KeyboardManager(m_logicmgr,keyboarddevice);
 	m_mousemgr = new SCA_MouseManager(m_logicmgr,mousedevice);
 	
-//	m_solidScene = DT_CreateScene();
-//	m_respTable = DT_CreateRespTable();
-
 	SCA_AlwaysEventManager* alwaysmgr = new SCA_AlwaysEventManager(m_logicmgr);
-	//KX_TouchEventManager* touchmgr = new KX_TouchEventManager(m_logicmgr, m_respTable, m_solidScene);
 	SCA_PropertyEventManager* propmgr = new SCA_PropertyEventManager(m_logicmgr);
 	SCA_RandomEventManager* rndmgr = new SCA_RandomEventManager(m_logicmgr);
 	KX_RayEventManager* raymgr = new KX_RayEventManager(m_logicmgr);
 
 	KX_NetworkEventManager* netmgr = new KX_NetworkEventManager(m_logicmgr, ndi);
+	
+	SCA_JoystickManager *joymgr	= new SCA_JoystickManager(m_logicmgr);
 
 	m_logicmgr->RegisterEventManager(alwaysmgr);
 	m_logicmgr->RegisterEventManager(propmgr);
 	m_logicmgr->RegisterEventManager(m_keyboardmgr);
 	m_logicmgr->RegisterEventManager(m_mousemgr);
-	//m_logicmgr->RegisterEventManager(touchmgr);
 	m_logicmgr->RegisterEventManager(m_timemgr);
 	m_logicmgr->RegisterEventManager(rndmgr);
 	m_logicmgr->RegisterEventManager(raymgr);
 	m_logicmgr->RegisterEventManager(netmgr);
+	m_logicmgr->RegisterEventManager(joymgr);
 
-	//m_sumoScene = new SM_Scene();
-	//m_sumoScene->setSecondaryRespTable(m_respTable);
 	m_soundScene = new SND_Scene(adi);
-	assert (m_networkDeviceInterface != NULL);
+	MT_assert (m_networkDeviceInterface != NULL);
 	m_networkScene = new NG_NetworkScene(m_networkDeviceInterface);
 	
 	m_rootnode = NULL;
@@ -163,12 +158,15 @@ KX_Scene::KX_Scene(class SCA_IInputDevice* keyboarddevice,
 
 	m_canvasDesignWidth = 0;
 	m_canvasDesignHeight = 0;
+	
+	m_attrlist = PyDict_New(); /* new ref */
 }
 
 
 
 KX_Scene::~KX_Scene()
 {
+	
 //	int numobj = m_objectlist->GetCount();
 
 	//int numrootobjects = GetRootParentList()->GetCount();
@@ -192,7 +190,7 @@ KX_Scene::~KX_Scene()
 
 	if (m_euthanasyobjects)
 		m_euthanasyobjects->Release();
-	
+
 	if (m_logicmgr)
 		delete m_logicmgr;
 
@@ -209,6 +207,7 @@ KX_Scene::~KX_Scene()
 	{
 		delete m_bucketmanager;
 	}
+	Py_DECREF(m_attrlist);
 }
 
 
@@ -400,7 +399,7 @@ KX_GameObject* KX_Scene::AddNodeReplicaObject(class SG_IObject* node, class CVal
 	}
 	
 	SG_IObject* replicanode = newobj->GetSGNode();
-	SG_Node* rootnode = (replicanode == m_rootnode ? NULL : m_rootnode);
+//	SG_Node* rootnode = (replicanode == m_rootnode ? NULL : m_rootnode);
 
 	replicanode->SetSGClientObject(newobj);
 
@@ -603,23 +602,27 @@ SCA_IObject* KX_Scene::AddReplicaObject(class CValue* originalobject,
 
 	MT_Matrix3x3 newori = ((KX_GameObject*) parentobject)->NodeGetWorldOrientation();
 	replica->NodeSetLocalOrientation(newori);
-
-	if (replica->GetPhysicsController())
-	{
-		replica->GetPhysicsController()->setPosition(newpos);
-		replica->GetPhysicsController()->setOrientation(newori.getRotation());
-	}
-
-	// here we want to set the relative scale: the rootnode's scale will override all other
-	// scalings, so lets better prepare for it
-
+	
 	// get the rootnode's scale
 	MT_Vector3 newscale = parentobj->GetSGNode()->GetRootSGParent()->GetLocalScale();
 
 	// set the replica's relative scale with the rootnode's scale
 	replica->NodeSetRelativeScale(newscale);
 
+	if (replica->GetPhysicsController())
+	{
+		replica->GetPhysicsController()->setPosition(newpos);
+		replica->GetPhysicsController()->setOrientation(newori.getRotation());
+		replica->GetPhysicsController()->setScaling(newscale);
+	}
+
+	// here we want to set the relative scale: the rootnode's scale will override all other
+	// scalings, so lets better prepare for it
+
+
 	replica->GetSGNode()->UpdateWorldData(0);
+	replica->GetSGNode()->SetBBox(originalobj->GetSGNode()->BBox());
+	replica->GetSGNode()->SetRadius(originalobj->GetSGNode()->Radius());
 	
 	return replica;
 }
@@ -658,11 +661,10 @@ void KX_Scene::DelayedRemoveObject(class CValue* gameobj)
 void KX_Scene::NewRemoveObject(class CValue* gameobj)
 {
 	KX_GameObject* newobj = (KX_GameObject*) gameobj;
-	//SM_Object* sumoObj = newobj->GetSumoObject();
-	//if (sumoObj)
-	//{
-	//	this->GetSumoScene()->remove(*sumoObj);
-	//}
+
+	//todo: look at this
+	//GetPhysicsEnvironment()->RemovePhysicsController(gameobj->getPhysicsController());
+
 	// remove all sensors/controllers/actuators from logicsystem...
 	
 	SCA_SensorList& sensors = newobj->GetSensors();
@@ -708,6 +710,11 @@ void KX_Scene::NewRemoveObject(class CValue* gameobj)
 		newobj->Release();
 	if (m_euthanasyobjects->RemoveValue(newobj))
 		newobj->Release();
+		
+	if (newobj == m_active_camera)
+	{
+		m_active_camera = NULL;
+	}
 }
 
 
@@ -715,8 +722,33 @@ void KX_Scene::NewRemoveObject(class CValue* gameobj)
 void KX_Scene::ReplaceMesh(class CValue* gameobj,void* meshobj)
 {
 	KX_GameObject* newobj = (KX_GameObject*) gameobj;
+	RAS_MeshObject* mesh = (RAS_MeshObject*) meshobj;
+
 	newobj->RemoveMeshes();
-	newobj->AddMesh((RAS_MeshObject*)meshobj);
+	newobj->AddMesh(mesh);
+
+	if (newobj->m_isDeformable && mesh->m_class == 1) {
+		Object* blendobj = (struct Object*)m_logicmgr->FindBlendObjByGameObj(newobj);
+		Object* oldblendobj = (struct Object*)m_logicmgr->FindBlendObjByGameMeshName(mesh->GetName());
+		if (blendobj->parent && blendobj->parent->type == OB_ARMATURE && blendobj->partype==PARSKEL && ((Mesh*)blendobj->data)->dvert) {
+			BL_SkinDeformer* skindeformer = new BL_SkinDeformer(oldblendobj, blendobj, (BL_SkinMeshObject*)mesh);
+			skindeformer->SetArmature((BL_ArmatureObject*) newobj->GetParent());
+			
+			// FIXME: should the old m_pDeformer be deleted?
+			// delete ((BL_DeformableGameObject*)newobj)->m_pDeformer
+			
+			((BL_DeformableGameObject*)newobj)->m_pDeformer = skindeformer;
+		}
+		else if (((Mesh*)blendobj->data)->dvert) {
+			BL_MeshDeformer* meshdeformer = new BL_MeshDeformer(oldblendobj, (BL_SkinMeshObject*)mesh);
+			
+			// FIXME: should the old m_pDeformer be deleted?
+			// delete ((BL_DeformableGameObject*)newobj)->m_pDeformer
+			
+			((BL_DeformableGameObject*)newobj)->m_pDeformer = meshdeformer;
+		}
+	}
+
 	newobj->Bucketize();
 }
 
@@ -768,7 +800,6 @@ void KX_Scene::AddCamera(KX_Camera* cam)
 	m_cameras.insert(cam);
 }
 
-
 KX_Camera* KX_Scene::GetActiveCamera()
 {	
 	// NULL if not defined
@@ -796,33 +827,152 @@ void KX_Scene::UpdateMeshTransformations()
 	{
 		KX_GameObject* gameobj = (KX_GameObject*)m_objectlist->GetValue(i);
 		gameobj->GetOpenGLMatrix();
-		gameobj->UpdateNonDynas();
+//		gameobj->UpdateNonDynas();
+	}
+}
+
+void KX_Scene::MarkVisible(SG_Tree *node, RAS_IRasterizer* rasty)
+{
+	int intersect = KX_Camera::INTERSECT;
+	KX_GameObject *gameobj = node->Client()?(KX_GameObject*) node->Client()->GetSGClientObject():NULL;
+	bool dotest = (gameobj && gameobj->GetVisible()) || node->Left() || node->Right();
+	
+	/* If the camera is inside the box, assume intersect. */
+	if (dotest && !node->inside(GetActiveCamera()->NodeGetWorldPosition()))
+	{
+		MT_Scalar radius = node->Radius();
+		MT_Point3 centre = node->Centre();
+		
+		intersect = GetActiveCamera()->SphereInsideFrustum(centre, radius); 
+		
+		if (intersect == KX_Camera::INTERSECT)
+		{
+			MT_Point3 box[8];
+			node->get(box);
+			intersect = GetActiveCamera()->BoxInsideFrustum(box);
+		}
+	}
+
+	switch (intersect)
+	{
+		case KX_Camera::OUTSIDE:
+			MarkSubTreeVisible(node, rasty, false);
+			break;
+		case KX_Camera::INTERSECT:
+			if (gameobj)
+				MarkVisible(rasty, gameobj);
+			if (node->Left())
+				MarkVisible(node->Left(), rasty);
+			if (node->Right())
+				MarkVisible(node->Right(), rasty);
+			break;
+		case KX_Camera::INSIDE:
+			MarkSubTreeVisible(node, rasty, true);
+			break;
+	}
+}
+
+void KX_Scene::MarkSubTreeVisible(SG_Tree *node, RAS_IRasterizer* rasty, bool visible)
+{
+	if (node->Client())
+	{
+		KX_GameObject *gameobj = (KX_GameObject*) node->Client()->GetSGClientObject();
+		if (gameobj->GetVisible())
+		{
+			if (visible)
+			{
+				int nummeshes = gameobj->GetMeshCount();
+				MT_Transform t( GetActiveCamera()->GetWorldToCamera() * gameobj->GetSGNode()->GetWorldTransform());
+	
+				
+				for (int m=0;m<nummeshes;m++)
+				{
+					// this adds the vertices to the display list
+					(gameobj->GetMesh(m))->SchedulePolygons(t, rasty->GetDrawingMode());
+				}
+			}
+			gameobj->MarkVisible(visible);
+		}
+	}
+	if (node->Left())
+		MarkSubTreeVisible(node->Left(), rasty, visible);
+	if (node->Right())
+		MarkSubTreeVisible(node->Right(), rasty, visible);
+}
+
+void KX_Scene::MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj)
+{
+	// User (Python/Actuator) has forced object invisible...
+	if (!gameobj->GetVisible())
+		return;
+	// If Frustum culling is off, the object is always visible.
+	bool vis = !GetActiveCamera()->GetFrustumCulling();
+	
+	// If the camera is inside this node, then the object is visible.
+	if (!vis)
+	{
+		vis = gameobj->GetSGNode()->inside( GetActiveCamera()->GetCameraLocation() );
+	}
+		
+	// Test the object's bound sphere against the view frustum.
+	if (!vis)
+	{
+		MT_Vector3 scale = gameobj->GetSGNode()->GetWorldScaling();
+		MT_Scalar radius = fabs(scale[scale.closestAxis()] * gameobj->GetSGNode()->Radius());
+		switch (GetActiveCamera()->SphereInsideFrustum(gameobj->NodeGetWorldPosition(), radius))
+		{
+			case KX_Camera::INSIDE:
+				vis = true;
+				break;
+			case KX_Camera::OUTSIDE:
+				vis = false;
+				break;
+			case KX_Camera::INTERSECT:
+				// Test the object's bound box against the view frustum.
+				MT_Point3 box[8];
+				gameobj->GetSGNode()->getBBox(box); 
+				vis = GetActiveCamera()->BoxInsideFrustum(box) != KX_Camera::OUTSIDE;
+				break;
+		}
+	}
+	
+	if (vis)
+	{
+		int nummeshes = gameobj->GetMeshCount();
+		MT_Transform t(GetActiveCamera()->GetWorldToCamera() * gameobj->GetSGNode()->GetWorldTransform());
+		
+		for (int m=0;m<nummeshes;m++)
+		{
+			// this adds the vertices to the display list
+			(gameobj->GetMesh(m))->SchedulePolygons(t, rasty->GetDrawingMode());
+		}
+		// Visibility/ non-visibility are marked
+		// elsewhere now.
+		gameobj->MarkVisible();
+	} else {
+		gameobj->MarkVisible(false);
 	}
 }
 
 void KX_Scene::CalculateVisibleMeshes(RAS_IRasterizer* rasty)
 {
-	
+// FIXME: When tree is operational
+#if 1
 	// do this incrementally in the future
 	for (int i = 0; i < m_objectlist->GetCount(); i++)
 	{
-		KX_GameObject* gameobj = (KX_GameObject*)m_objectlist->GetValue(i);
-		
-		int nummeshes = gameobj->GetMeshCount();
-		
-		for (int m=0;m<nummeshes;m++)
-		{
-			// this adds the vertices to the display list
-			(gameobj->GetMesh(m))->SchedulePolygons(rasty->GetDrawingMode(),rasty);
-			// Visibility/ non-visibility are marked
-			// elsewhere now.
-			gameobj->MarkVisible();
-		}
+		MarkVisible(rasty, static_cast<KX_GameObject*>(m_objectlist->GetValue(i)));
 	}
+#else
+	if (GetActiveCamera()->GetFrustumCulling())
+		MarkVisible(m_objecttree, rasty);
+	else
+		MarkSubTreeVisible(m_objecttree, rasty, true);
+#endif
 }
 
 // logic stuff
-void KX_Scene::LogicBeginFrame(double curtime,double deltatime)
+void KX_Scene::LogicBeginFrame(double curtime)
 {
 	// have a look at temp objects ...
 	int lastobj = m_tempObjectList->GetCount() - 1;
@@ -834,7 +984,7 @@ void KX_Scene::LogicBeginFrame(double curtime,double deltatime)
 		
 		if (propval)
 		{
-			float timeleft = propval->GetNumber() - deltatime;
+			float timeleft = propval->GetNumber() - 1.0/KX_KetsjiEngine::GetTicRate();
 			
 			if (timeleft > 0)
 			{
@@ -851,14 +1001,14 @@ void KX_Scene::LogicBeginFrame(double curtime,double deltatime)
 			// all object is the tempObjectList should have a clock
 		}
 	}
-	m_logicmgr->BeginFrame(curtime,deltatime);
+	m_logicmgr->BeginFrame(curtime, 1.0/KX_KetsjiEngine::GetTicRate());
 }
 
 
 
-void KX_Scene::LogicUpdateFrame(double curtime,double deltatime)
+void KX_Scene::LogicUpdateFrame(double curtime, bool frame)
 {
-	m_logicmgr->UpdateFrame(curtime,deltatime);
+	m_logicmgr->UpdateFrame(curtime, frame);
 }
 
 
@@ -873,14 +1023,6 @@ void KX_Scene::LogicEndFrame()
 		KX_GameObject* gameobj = (KX_GameObject*)m_euthanasyobjects->GetValue(i);
 		this->RemoveObject(gameobj);
 	}
-	
-	numobj = m_euthanasyobjects->GetCount();
-	if (numobj != 0)
-	{
-		// huh?
-		int ii=0;
-	}
-	// numobj is 0 we hope
 }
 
 
@@ -979,3 +1121,125 @@ void	KX_Scene::SetGravity(const MT_Vector3& gravity)
 {
 	GetPhysicsEnvironment()->setGravity(gravity[0],gravity[1],gravity[2]);
 }
+
+void KX_Scene::SetNodeTree(SG_Tree* root)
+{
+	m_objecttree = root;
+}
+
+void KX_Scene::SetPhysicsEnvironment(class PHY_IPhysicsEnvironment* physEnv)
+{
+	m_physicsEnvironment = physEnv;
+
+	KX_TouchEventManager* touchmgr = new KX_TouchEventManager(m_logicmgr, physEnv);
+	m_logicmgr->RegisterEventManager(touchmgr);
+	return;
+}
+
+//----------------------------------------------------------------------------
+//Python
+
+PyMethodDef KX_Scene::Methods[] = {
+	KX_PYMETHODTABLE(KX_Scene, getLightList),
+	KX_PYMETHODTABLE(KX_Scene, getObjectList),
+	KX_PYMETHODTABLE(KX_Scene, getName),
+	
+	{NULL,NULL} //Sentinel
+};
+
+PyTypeObject KX_Scene::Type = {
+	PyObject_HEAD_INIT(&PyType_Type)
+		0,
+		"KX_Scene",
+		sizeof(KX_Scene),
+		0,
+		PyDestructor,
+		0,
+		__getattr,
+		__setattr,
+		0, //&MyPyCompare,
+		__repr,
+		0, //&cvalue_as_number,
+		0,
+		0,
+		0,
+		0, 0, 0, 0, 0, 0
+};
+
+PyParentObject KX_Scene::Parents[] = {
+	&KX_Scene::Type,
+		&CValue::Type,
+		NULL
+};
+
+PyObject* KX_Scene::_getattr(const STR_String& attr)
+{
+	if (attr == "name")
+		return PyString_FromString(GetName());
+	
+	if (attr == "active_camera")
+	{
+		KX_Camera *camera = GetActiveCamera();
+		camera->AddRef();
+		return (PyObject*) camera;
+	}
+	
+	if (attr == "suspended")
+		return PyInt_FromLong(m_suspend);
+	
+	if (attr == "activity_culling")
+		return PyInt_FromLong(m_activity_culling);
+	
+	if (attr == "activity_culling_radius")
+		return PyFloat_FromDouble(m_activity_box_radius);
+	
+	PyObject* value = PyDict_GetItemString(m_attrlist, const_cast<char *>(attr.ReadPtr()));
+	if (value)
+	{
+		Py_INCREF(value);
+		return value;
+	}
+	
+	_getattr_up(PyObjectPlus);
+}
+
+int KX_Scene::_delattr(const STR_String &attr)
+{
+	PyDict_DelItemString(m_attrlist, const_cast<char *>(attr.ReadPtr()));
+	return 0;
+}
+
+int KX_Scene::_setattr(const STR_String &attr, PyObject *pyvalue)
+{
+
+	if (!PyDict_SetItemString(m_attrlist, const_cast<char *>(attr.ReadPtr()), pyvalue))
+		return 0;
+
+	return PyObjectPlus::_setattr(attr, pyvalue);
+}
+
+KX_PYMETHODDEF_DOC(KX_Scene, getLightList,
+"getLightList() -> list [KX_Light]\n"
+"Returns a list of all lights in the scene.\n"
+)
+{
+	m_lightlist->AddRef();
+	return (PyObject*) m_lightlist;
+}
+
+KX_PYMETHODDEF_DOC(KX_Scene, getObjectList,
+"getObjectList() -> list [KX_GameObject]\n"
+"Returns a list of all game objects in the scene.\n"
+)
+{
+	m_objectlist->AddRef();
+	return (PyObject*) m_objectlist;
+}
+
+KX_PYMETHODDEF_DOC(KX_Scene, getName,
+"getName() -> string\n"
+"Returns the name of the scene.\n"
+)
+{
+	return PyString_FromString(GetName());
+}
