@@ -1,5 +1,5 @@
 /**
- * $Id: PHY_IPhysicsEnvironment.h,v 1.6 2004/03/22 22:02:11 jesterking Exp $
+ * $Id: PHY_IPhysicsEnvironment.h,v 1.8 2005/03/25 10:33:39 kester Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,8 +43,15 @@ class PHY_IPhysicsEnvironment
 {
 	public:
 		virtual		~PHY_IPhysicsEnvironment();
+		virtual	void		beginFrame() = 0;
+		virtual void		endFrame() = 0;
 		/// Perform an integration step of duration 'timeStep'.
-		virtual	void		proceed(double	timeStep)=0;
+		virtual	bool		proceedDeltaTime(double curTime,float timeStep)=0;
+		virtual	void		setFixedTimeStep(bool useFixedTimeStep,float fixedTimeStep)=0;
+		//returns 0.f if no fixed timestep is used
+		virtual	float		getFixedTimeStep()=0;
+
+
 		virtual	void		setGravity(float x,float y,float z)=0;
 
 		virtual int			createConstraint(class PHY_IPhysicsController* ctrl,class PHY_IPhysicsController* ctrl2,PHY_ConstraintType type,
@@ -52,8 +59,19 @@ class PHY_IPhysicsEnvironment
 			float axisX,float axisY,float axisZ)=0;
 		virtual void		removeConstraint(int constraintid)=0;
 
-		virtual PHY_IPhysicsController* rayTest(void* ignoreClient, float fromX,float fromY,float fromZ, float toX,float toY,float toZ, 
+		virtual PHY_IPhysicsController* rayTest(PHY_IPhysicsController* ignoreClient, float fromX,float fromY,float fromZ, float toX,float toY,float toZ, 
 										float& hitX,float& hitY,float& hitZ,float& normalX,float& normalY,float& normalZ)=0;
+
+
+		//Methods for gamelogic collision/physics callbacks
+		//todo:
+		virtual void addSensor(PHY_IPhysicsController* ctrl)=0;
+		virtual void removeSensor(PHY_IPhysicsController* ctrl)=0;
+		virtual void addTouchCallback(int response_class, PHY_ResponseCallback callback, void *user)=0;
+		virtual void requestCollisionCallback(PHY_IPhysicsController* ctrl)=0;
+		virtual PHY_IPhysicsController*	CreateSphereController(float radius,const PHY__Vector3& position) =0;
+		virtual PHY_IPhysicsController* CreateConeController(float coneradius,float coneheight)=0;
+		
 };
 
 #endif //_IPHYSICSENVIRONMENT
