@@ -1,5 +1,5 @@
 /**
- * $Id: RAS_MeshObject.h,v 1.9 2004/05/04 13:17:46 kester Exp $
+ * $Id: RAS_MeshObject.h,v 1.13 2004/11/22 10:19:19 kester Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -120,9 +120,6 @@ inline  bool operator <( const RAS_MatArrayIndex& rhs,const RAS_MatArrayIndex& l
 class RAS_MeshObject
 {
 	
-	enum { BUCKET_MAX_INDICES = 2048 };//2048};//8192};
-	enum { BUCKET_MAX_TRIANGLES = 1024 };
-	
 	//	GEN_Map<class RAS_IPolyMaterial,KX_ArrayOptimizer*> m_matVertexArrayS;
 	//vector<class RAS_IPolyMaterial*,KX_ArrayOptimizer> m_vertexArrays;
 	virtual KX_ArrayOptimizer* GetArrayOptimizer(RAS_IPolyMaterial* polymat);
@@ -135,8 +132,17 @@ class RAS_MeshObject
 	vector<class RAS_Polygon*> 	m_Polygons;
 	STR_String					m_name;
 	static STR_String			s_emptyname;
+	bool					m_zsort;
+	
+	struct polygonSlot;
+	struct backtofront;
+	struct fronttoback;
+
 	
 protected:
+	enum { BUCKET_MAX_INDICES = 65535 };//2048};//8192};
+	enum { BUCKET_MAX_TRIANGLES = 65535 };
+	
 	GEN_Map<class RAS_IPolyMaterial,KX_ArrayOptimizer*> m_matVertexArrayS;
 	
 	RAS_MaterialBucket::Set			m_materials;
@@ -184,9 +190,14 @@ public:
 
 	void				DebugColor(unsigned int abgr);
 	
+	/**
+	 *  Sorts the polygons by their transformed z values.
+	 */
+	void				SortPolygons(const MT_Transform &transform);
+
 	void				SchedulePolygons(
-							int drawingmode,
-							class RAS_IRasterizer* rasty
+							const MT_Transform &transform,
+							int drawingmode
 						);
 
 	void				ClearArrayData();
@@ -196,7 +207,7 @@ public:
 	
 	virtual RAS_TexVert*	GetVertex(
 								short array,
-								short index,
+								unsigned int index,
 								RAS_IPolyMaterial* polymat
 							);
 	
