@@ -1,5 +1,5 @@
 /**
- * $Id: RAS_Polygon.cpp,v 1.4 2004/03/22 22:02:13 jesterking Exp $
+ * $Id: RAS_Polygon.cpp,v 1.6 2005/01/03 18:05:24 sirdude Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -67,7 +67,6 @@ RAS_Polygon::RAS_Polygon(RAS_MaterialBucket* bucket,
 		m_numverts(numverts),
 		m_edgecode(0)
 {
-	int size = sizeof(RAS_Polygon);
 	m_vertexindexbase.m_vtxarray = vtxarrayindex ;//m_bucket->FindVertexArray(numverts);
 	m_polyFlags.Visible = visible;
 }
@@ -82,7 +81,7 @@ int RAS_Polygon::VertexCount()
 
 
 void RAS_Polygon::SetVertex(int i,
-							int vertexindex ) //const MT_Point3& xyz,const MT_Point2& uv,const unsigned int rgbacolor,const MT_Vector3& normal)
+						unsigned int vertexindex ) //const MT_Point3& xyz,const MT_Point2& uv,const unsigned int rgbacolor,const MT_Vector3& normal)
 {
 	m_vertexindexbase.SetIndex(i,vertexindex); //m_bucket->FindOrAddVertex(m_vertexindexbase.m_vtxarray,
 	//xyz,uv,rgbacolor,normal));
